module MARC
  module Marc8
    # Data to map from marc8 bytes to unicode codepoints.
    #
    # Ported from pymarc python to ruby using some regexps and such. Original pymarc:
    #
    # https://github.com/edsu/pymarc/blob/master/pymarc/marc8_mapping.py
    module MapToUnicode
      CHARSET_34 = { # Extended Arabic
        0xa1 => [0x6fd, false], # DOUBLE ALEF WITH HAMZA ABOVE / ARABIC SIGN SINDHI AMPERSAND
        0xa2 => [0x672, false], # ARABIC LETTER ALEF WITH WAVY HAMZA ABOVE
        0xa3 => [0x673, false], # ARABIC LETTER ALEF WITH WAVY HAMZA BELOW
        0xa4 => [0x679, false], # ARABIC LETTER TTEH
        0xa5 => [0x67a, false], # ARABIC LETTER TTEHEH
        0xa6 => [0x67b, false], # ARABIC LETTER BBEH
        0xa7 => [0x67c, false], # ARABIC LETTER TEH WITH RING
        0xa8 => [0x67d, false], # ARABIC LETTER TEH WITH THREE DOTS ABOVE DOWNWARDS
        0xa9 => [0x67e, false], # ARABIC LETTER PEH
        0xaa => [0x67f, false], # ARABIC LETTER TEHEH
        0xab => [0x680, false], # ARABIC LETTER BEHEH
        0xac => [0x681, false], # ARABIC LETTER HAH WITH HAMZA ABOVE
        0xad => [0x682, false], # ARABIC LETTER HAH WITH TWO ABOVE DOTS VERTICAL ABOVE
        0xae => [0x683, false], # ARABIC LETTER NYEH
        0xaf => [0x684, false], # ARABIC LETTER DYEH
        0xb0 => [0x685, false], # ARABIC LETTER HAH WITH THREE DOTS ABOVE
        0xb1 => [0x686, false], # ARABIC LETTER TCHEH
        0xb2 => [0x6bf, false], # ARABIC LETTER TCHEH WITH DOT ABOVE
        0xb3 => [0x687, false], # ARABIC LETTER TCHEHEH
        0xb4 => [0x688, false], # ARABIC LETTER DDAL
        0xb5 => [0x689, false], # ARABIC LETTER DAL WITH RING
        0xb6 => [0x68a, false], # ARABIC LETTER DAL WITH DOT BELOW
        0xb7 => [0x68b, false], # ARABIC LETTER DAL WITH DOT BELOW AND SMALL TAH
        0xb8 => [0x68c, false], # ARABIC LETTER DAHAL
        0xb9 => [0x68d, false], # ARABIC LETTER DDAHAL
        0xba => [0x68e, false], # ARABIC LETTER DUL
        0xbb => [0x68f, false], # ARABIC LETTER DAL WITH THREE DOTS ABOVE DOWNWARDS
        0xbc => [0x690, false], # ARABIC LETTER DAL WITH FOUR DOTS ABOVE
        0xbd => [0x691, false], # ARABIC LETTER RREH
        0xbe => [0x692, false], # ARABIC LETTER REH WITH SMALL V
        0xbf => [0x693, false], # ARABIC LETTER REH WITH RING
        0xc0 => [0x694, false], # ARABIC LETTER REH WITH DOT BELOW
        0xc1 => [0x695, false], # ARABIC LETTER REH WITH SMALL V BELOW
        0xc2 => [0x696, false], # ARABIC LETTER REH WITH DOT BELOW AND DOT ABOVE
        0xc3 => [0x697, false], # ARABIC LETTER REH WITH TWO DOTS ABOVE
        0xc4 => [0x698, false], # ARABIC LETTER JEH
        0xc5 => [0x699, false], # ARABIC LETTER REH WITH FOUR DOTS ABOVE
        0xc6 => [0x69a, false], # ARABIC LETTER SEEN WITH DOT BELOW AND DOT ABOVE
        0xc7 => [0x69b, false], # ARABIC LETTER SEEN WITH THREE DOTS BELOW
        0xc8 => [0x69c, false], # ARABIC LETTER SEEN WITH THREE DOTS BELOW AND THREE DOTS ABOVE
        0xc9 => [0x6fa, false], # ARABIC LETTER SHEEN WITH DOT BELOW
        0xca => [0x69d, false], # ARABIC LETTER SAD WITH TWO DOTS BELOW
        0xcb => [0x69e, false], # ARABIC LETTER SAD WITH THREE DOTS ABOVE
        0xcc => [0x6fb, false], # ARABIC LETTER DAD WITH DOT BELOW
        0xcd => [0x69f, false], # ARABIC LETTER TAH WITH THREE DOTS ABOVE
        0xce => [0x6a0, false], # ARABIC LETTER AIN WITH THREE DOTS ABOVE
        0xcf => [0x6fc, false], # ARABIC LETTER GHAIN WITH DOT BELOW
        0xd0 => [0x6a1, false], # ARABIC LETTER DOTLESS FEH
        0xd1 => [0x6a2, false], # ARABIC LETTER FEH WITH DOT MOVED BELOW
        0xd2 => [0x6a3, false], # ARABIC LETTER FEH WITH DOT BELOW
        0xd3 => [0x6a4, false], # ARABIC LETTER VEH
        0xd4 => [0x6a5, false], # ARABIC LETTER FEH WITH THREE DOTS BELOW
        0xd5 => [0x6a6, false], # ARABIC LETTER PEHEH
        0xd6 => [0x6a7, false], # ARABIC LETTER QAF WITH DOT ABOVE
        0xd7 => [0x6a8, false], # ARABIC LETTER QAF WITH THREE DOTS ABOVE
        0xd8 => [0x6a9, false], # ARABIC LETTER KEHEH
        0xd9 => [0x6aa, false], # ARABIC LETTER SWASH KAF
        0xda => [0x6ab, false], # ARABIC LETTER KAF WITH RING
        0xdb => [0x6ac, false], # ARABIC LETTER KAF WITH DOT ABOVE
        0xdc => [0x6ad, false], # ARABIC LETTER NG
        0xdd => [0x6ae, false], # ARABIC LETTER KAF WITH THREE DOTS BELOW
        0xde => [0x6af, false], # ARABIC LETTER GAF
        0xdf => [0x6b0, false], # ARABIC LETTER GAF WITH RING
        0xe0 => [0x6b1, false], # ARABIC LETTER NGOEH
        0xe1 => [0x6b2, false], # ARABIC LETTER GAF WITH TWO DOTS BELOW
        0xe2 => [0x6b3, false], # ARABIC LETTER GUEH
        0xe3 => [0x6b4, false], # ARABIC LETTER GAF WITH THREE DOTS ABOVE
        0xe4 => [0x6b5, false], # ARABIC LETTER LAM WITH SMALL V
        0xe5 => [0x6b6, false], # ARABIC LETTER LAM WITH DOT ABOVE
        0xe6 => [0x6b7, false], # ARABIC LETTER LAM WITH THREE DOTS ABOVE
        0xe7 => [0x6b8, false], # ARABIC LETTER LAM WITH THREE DOTS BELOW
        0xe8 => [0x6ba, false], # ARABIC LETTER NOON GHUNNA
        0xe9 => [0x6bb, false], # ARABIC LETTER RNOON
        0xea => [0x6bc, false], # ARABIC LETTER NOON WITH RING
        0xeb => [0x6bd, false], # ARABIC LETTER NOON WITH THREE DOTS ABOVE
        0xec => [0x6b9, false], # ARABIC LETTER NOON WITH DOT BELOW
        0xed => [0x6be, false], # ARABIC LETTER HEH DOACHASHMEE
        0xee => [0x6c0, false], # HEH WITH HAMZA ABOVE / ARABIC LETTER HEH WITH YEH ABOVE
        0xef => [0x6c4, false], # ARABIC LETTER WAW WITH RING
        0xf0 => [0x6c5, false], # KYRGHYZ OE / ARABIC LETTER KIRGHIZ OE
        0xf1 => [0x6c6, false], # ARABIC LETTER OE
        0xf2 => [0x6ca, false], # ARABIC LETTER WAW WITH TWO DOTS ABOVE
        0xf3 => [0x6cb, false], # ARABIC LETTER VE
        0xf4 => [0x6cd, false], # ARABIC LETTER YEH WITH TAIL
        0xf5 => [0x6ce, false], # ARABIC LETTER YEH WITH SMALL V
        0xf6 => [0x6d0, false], # ARABIC LETTER E
        0xf7 => [0x6d2, false], # ARABIC LETTER YEH BARREE
        0xf8 => [0x6d3, false], # ARABIC LETTER YEH BARREE WITH HAMZA ABOVE
        0xfd => [0x306, true], # SHORT E / COMBINING BREVE
        0xfe => [0x30c, true] # SHORT U / COMBINING CARON
      }
      CHARSET_45 = { # Extended Latin (ANSEL)
        0x88 => [0x98, false], # NON-SORT BEGIN / START OF STRING
        0x89 => [0x9c, false], # NON-SORT END / STRING TERMINATOR
        0x8d => [0x200d, false], # JOINER / ZERO WIDTH JOINER
        0x8e => [0x200c, false], # NON-JOINER / ZERO WIDTH NON-JOINER
        0xa1 => [0x141, false], # UPPERCASE POLISH L / LATIN CAPITAL LETTER L WITH STROKE
        0xa2 => [0xd8, false], # UPPERCASE SCANDINAVIAN O / LATIN CAPITAL LETTER O WITH STROKE
        0xa3 => [0x110, false], # UPPERCASE D WITH CROSSBAR / LATIN CAPITAL LETTER D WITH STROKE
        0xa4 => [0xde, false], # UPPERCASE ICELANDIC THORN / LATIN CAPITAL LETTER THORN (Icelandic)
        0xa5 => [0xc6, false], # UPPERCASE DIGRAPH AE / LATIN CAPITAL LIGATURE AE
        0xa6 => [0x152, false], # UPPERCASE DIGRAPH OE / LATIN CAPITAL LIGATURE OE
        0xa7 => [0x2b9, false], # SOFT SIGN, PRIME / MODIFIER LETTER PRIME
        0xa8 => [0xb7, false], # MIDDLE DOT
        0xa9 => [0x266d, false], # MUSIC FLAT SIGN
        0xaa => [0xae, false], # PATENT MARK / REGISTERED SIGN
        0xab => [0xb1, false], # PLUS OR MINUS / PLUS-MINUS SIGN
        0xac => [0x1a0, false], # UPPERCASE O-HOOK / LATIN CAPITAL LETTER O WITH HORN
        0xad => [0x1af, false], # UPPERCASE U-HOOK / LATIN CAPITAL LETTER U WITH HORN
        0xae => [0x2be, false], # ALIF / MODIFIER LETTER RIGHT HALF RING
        0xb0 => [0x2bb, false], # AYN / MODIFIER LETTER TURNED COMMA
        0xb1 => [0x142, false], # LOWERCASE POLISH L / LATIN SMALL LETTER L WITH STROKE
        0xb2 => [0xf8, false], # LOWERCASE SCANDINAVIAN O / LATIN SMALL LETTER O WITH STROKE
        0xb3 => [0x111, false], # LOWERCASE D WITH CROSSBAR / LATIN SMALL LETTER D WITH STROKE
        0xb4 => [0xfe, false], # LOWERCASE ICELANDIC THORN / LATIN SMALL LETTER THORN (Icelandic)
        0xb5 => [0xe6, false], # LOWERCASE DIGRAPH AE / LATIN SMALL LIGATURE AE
        0xb6 => [0x153, false], # LOWERCASE DIGRAPH OE / LATIN SMALL LIGATURE OE
        0xb7 => [0x2ba, false], # HARD SIGN, DOUBLE PRIME / MODIFIER LETTER DOUBLE PRIME
        0xb8 => [0x131, false], # LOWERCASE TURKISH I / LATIN SMALL LETTER DOTLESS I
        0xb9 => [0xa3, false], # BRITISH POUND / POUND SIGN
        0xba => [0xf0, false], # LOWERCASE ETH / LATIN SMALL LETTER ETH (Icelandic)
        0xbc => [0x1a1, false], # LOWERCASE O-HOOK / LATIN SMALL LETTER O WITH HORN
        0xbd => [0x1b0, false], # LOWERCASE U-HOOK / LATIN SMALL LETTER U WITH HORN
        0xc0 => [0xb0, false], # DEGREE SIGN
        0xc1 => [0x2113, false], # SCRIPT SMALL L
        0xc2 => [0x2117, false], # SOUND RECORDING COPYRIGHT
        0xc3 => [0xa9, false], # COPYRIGHT SIGN
        0xc4 => [0x266f, false], # MUSIC SHARP SIGN
        0xc5 => [0xbf, false], # INVERTED QUESTION MARK
        0xc6 => [0xa1, false], # INVERTED EXCLAMATION MARK
        0xe0 => [0x309, true], # PSEUDO QUESTION MARK / COMBINING HOOK ABOVE
        0xe1 => [0x300, true], # GRAVE / COMBINING GRAVE ACCENT (Varia)
        0xe2 => [0x301, true], # ACUTE / COMBINING ACUTE ACCENT (Oxia)
        0xe3 => [0x302, true], # CIRCUMFLEX / COMBINING CIRCUMFLEX ACCENT
        0xe4 => [0x303, true], # TILDE / COMBINING TILDE
        0xe5 => [0x304, true], # MACRON / COMBINING MACRON
        0xe6 => [0x306, true], # BREVE / COMBINING BREVE (Vrachy)
        0xe7 => [0x307, true], # SUPERIOR DOT / COMBINING DOT ABOVE
        0xe8 => [0x308, true], # UMLAUT, DIAERESIS / COMBINING DIAERESIS (Dialytika)
        0xe9 => [0x30c, true], # HACEK / COMBINING CARON
        0xea => [0x30a, true], # CIRCLE ABOVE, ANGSTROM / COMBINING RING ABOVE
        0xeb => [0xfe20, true], # LIGATURE, FIRST HALF / COMBINING LIGATURE LEFT HALF
        0xec => [0xfe21, true], # LIGATURE, SECOND HALF / COMBINING LIGATURE RIGHT HALF
        0xed => [0x315, true], # HIGH COMMA, OFF CENTER / COMBINING COMMA ABOVE RIGHT
        0xee => [0x30b, true], # DOUBLE ACUTE / COMBINING DOUBLE ACUTE ACCENT
        0xef => [0x310, true], # CANDRABINDU / COMBINING CANDRABINDU
        0xf0 => [0x327, true], # CEDILLA / COMBINING CEDILLA
        0xf1 => [0x328, true], # RIGHT HOOK, OGONEK / COMBINING OGONEK
        0xf2 => [0x323, true], # DOT BELOW / COMBINING DOT BELOW
        0xf3 => [0x324, true], # DOUBLE DOT BELOW / COMBINING DIAERESIS BELOW
        0xf4 => [0x325, true], # CIRCLE BELOW / COMBINING RING BELOW
        0xf5 => [0x333, true], # DOUBLE UNDERSCORE / COMBINING DOUBLE LOW LINE
        0xf6 => [0x332, true], # UNDERSCORE / COMBINING LOW LINE
        0xf7 => [0x326, true], # LEFT HOOK (COMMA BELOW) / COMBINING COMMA BELOW
        0xf8 => [0x31c, true], # RIGHT CEDILLA / COMBINING LEFT HALF RING BELOW
        0xf9 => [0x32e, true], # UPADHMANIYA / COMBINING BREVE BELOW
        0xfa => [0xfe22, true], # DOUBLE TILDE, FIRST HALF / COMBINING DOUBLE TILDE LEFT HALF
        0xfb => [0xfe23, true], # DOUBLE TILDE, SECOND HALF / COMBINING DOUBLE TILDE RIGHT HALF
        0xfe => [0x313, true] # HIGH COMMA, CENTERED / COMBINING COMMA ABOVE (Psili)
      }
      CHARSET_33 = { # Basic Arabic
        0x21 => [0x21, false], # EXCLAMATION MARK
        0x22 => [0x22, false], # QUOTATION MARK
        0x23 => [0x23, false], # NUMBER SIGN
        0x24 => [0x24, false], # DOLLAR SIGN
        0x25 => [0x66a, false], # PERCENT SIGN / ARABIC PERCENT SIGN
        0x26 => [0x26, false], # AMPERSAND
        0x27 => [0x27, false], # APOSTROPHE
        0x28 => [0x28, false], # OPENING PARENTHESIS / LEFT PARENTHESIS
        0x29 => [0x29, false], # CLOSING PARENTHESIS / RIGHT PARENTHESIS
        0x2a => [0x66d, false], # ASTERISK / ARABIC FIVE POINTED STAR
        0x2b => [0x2b, false], # PLUS SIGN
        0x2c => [0x60c, false], # ARABIC COMMA
        0x2d => [0x2d, false], # HYPHEN-MINUS
        0x2e => [0x2e, false], # PERIOD, DECIMAL POINT / FULL STOP
        0x2f => [0x2f, false], # SLASH / SOLIDUS
        0x30 => [0x660, false], # ARABIC-INDIC DIGIT ZERO
        0x31 => [0x661, false], # ARABIC-INDIC DIGIT ONE
        0x32 => [0x662, false], # ARABIC-INDIC DIGIT TWO
        0x33 => [0x663, false], # ARABIC-INDIC DIGIT THREE
        0x34 => [0x664, false], # ARABIC-INDIC DIGIT FOUR
        0x35 => [0x665, false], # ARABIC-INDIC DIGIT FIVE
        0x36 => [0x666, false], # ARABIC-INDIC DIGIT SIX
        0x37 => [0x667, false], # ARABIC-INDIC DIGIT SEVEN
        0x38 => [0x668, false], # ARABIC-INDIC DIGIT EIGHT
        0x39 => [0x669, false], # ARABIC-INDIC DIGIT NINE
        0x3a => [0x3a, false], # COLON
        0x3b => [0x61b, false], # ARABIC SEMICOLON
        0x3c => [0x3c, false], # LESS-THAN SIGN
        0x3d => [0x3d, false], # EQUALS SIGN
        0x3e => [0x3e, false], # GREATER-THAN SIGN
        0x3f => [0x61f, false], # ARABIC QUESTION MARK
        0x41 => [0x621, false], # HAMZAH / ARABIC LETTER HAMZA
        0x42 => [0x622, false], # ARABIC LETTER ALEF WITH MADDA ABOVE
        0x43 => [0x623, false], # ARABIC LETTER ALEF WITH HAMZA ABOVE
        0x44 => [0x624, false], # ARABIC LETTER WAW WITH HAMZA ABOVE
        0x45 => [0x625, false], # ARABIC LETTER ALEF WITH HAMZA BELOW
        0x46 => [0x626, false], # ARABIC LETTER YEH WITH HAMZA ABOVE
        0x47 => [0x627, false], # ARABIC LETTER ALEF
        0x48 => [0x628, false], # ARABIC LETTER BEH
        0x49 => [0x629, false], # ARABIC LETTER TEH MARBUTA
        0x4a => [0x62a, false], # ARABIC LETTER TEH
        0x4b => [0x62b, false], # ARABIC LETTER THEH
        0x4c => [0x62c, false], # ARABIC LETTER JEEM
        0x4d => [0x62d, false], # ARABIC LETTER HAH
        0x4e => [0x62e, false], # ARABIC LETTER KHAH
        0x4f => [0x62f, false], # ARABIC LETTER DAL
        0x50 => [0x630, false], # ARABIC LETTER THAL
        0x51 => [0x631, false], # ARABIC LETTER REH
        0x52 => [0x632, false], # ARABIC LETTER ZAIN
        0x53 => [0x633, false], # ARABIC LETTER SEEN
        0x54 => [0x634, false], # ARABIC LETTER SHEEN
        0x55 => [0x635, false], # ARABIC LETTER SAD
        0x56 => [0x636, false], # ARABIC LETTER DAD
        0x57 => [0x637, false], # ARABIC LETTER TAH
        0x58 => [0x638, false], # ARABIC LETTER ZAH
        0x59 => [0x639, false], # ARABIC LETTER AIN
        0x5a => [0x63a, false], # ARABIC LETTER GHAIN
        0x5b => [0x5b, false], # OPENING SQUARE BRACKET / LEFT SQUARE BRACKET
        0x5d => [0x5d, false], # CLOSING SQUARE BRACKET / RIGHT SQUARE BRACKET
        0x60 => [0x640, false], # ARABIC TATWEEL
        0x61 => [0x641, false], # ARABIC LETTER FEH
        0x62 => [0x642, false], # ARABIC LETTER QAF
        0x63 => [0x643, false], # ARABIC LETTER KAF
        0x64 => [0x644, false], # ARABIC LETTER LAM
        0x65 => [0x645, false], # ARABIC LETTER MEEM
        0x66 => [0x646, false], # ARABIC LETTER NOON
        0x67 => [0x647, false], # ARABIC LETTER HEH
        0x68 => [0x648, false], # ARABIC LETTER WAW
        0x69 => [0x649, false], # ARABIC LETTER ALEF MAKSURA
        0x6a => [0x64a, false], # ARABIC LETTER YEH
        0x6b => [0x64b, true], # ARABIC FATHATAN
        0x6c => [0x64c, true], # ARABIC DAMMATAN
        0x6d => [0x64d, true], # ARABIC KASRATAN
        0x6e => [0x64e, true], # ARABIC FATHA
        0x6f => [0x64f, true], # ARABIC DAMMA
        0x70 => [0x650, true], # ARABIC KASRA
        0x71 => [0x651, true], # ARABIC SHADDA
        0x72 => [0x652, true], # ARABIC SUKUN
        0x73 => [0x671, false], # ARABIC LETTER ALEF WASLA
        0x74 => [0x670, false], # ARABIC LETTER SUPERSCRIPT ALEF
        0x78 => [0x66c, false], # ARABIC THOUSANDS SEPARATOR
        0x79 => [0x201d, false], # RIGHT DOUBLE QUOTATION MARK
        0x7a => [0x201c, false] # LEFT DOUBLE QUOTATION MARK
      }
      CHARSET_32 = { # Basic Hebrew
        0x21 => [0x21, false], # EXCLAMATION MARK
        0x22 => [0x5f4, false], # QUOTATION MARK, GERSHAYIM / HEBREW PUNCTUATION GERSHAYIM
        0x23 => [0x23, false], # NUMBER SIGN
        0x24 => [0x24, false], # DOLLAR SIGN
        0x25 => [0x25, false], # PERCENT SIGN
        0x26 => [0x26, false], # AMPERSAND
        0x27 => [0x5f3, false], # APOSTROPHE, GERESH / HEBREW PUNCTUATION GERESH
        0x28 => [0x28, false], # OPENING PARENTHESIS / LEFT PARENTHESIS
        0x29 => [0x29, false], # CLOSING PARENTHESIS / RIGHT PARENTHESIS
        0x2a => [0x2a, false], # ASTERISK
        0x2b => [0x2b, false], # PLUS SIGN
        0x2c => [0x2c, false], # COMMA
        0x2d => [0x5be, false], # HYPHEN-MINUS, MAKEF / HEBREW PUNCTUATION MAQAF
        0x2e => [0x2e, false], # PERIOD, DECIMAL POINT / FULL STOP
        0x2f => [0x2f, false], # SLASH / SOLIDUS
        0x30 => [0x30, false], # DIGIT ZERO
        0x31 => [0x31, false], # DIGIT ONE
        0x32 => [0x32, false], # DIGIT TWO
        0x33 => [0x33, false], # DIGIT THREE
        0x34 => [0x34, false], # DIGIT FOUR
        0x35 => [0x35, false], # DIGIT FIVE
        0x36 => [0x36, false], # DIGIT SIX
        0x37 => [0x37, false], # DIGIT SEVEN
        0x38 => [0x38, false], # DIGIT EIGHT
        0x39 => [0x39, false], # DIGIT NINE
        0x3a => [0x3a, false], # COLON
        0x3b => [0x3b, false], # SEMICOLON
        0x3c => [0x3c, false], # LESS-THAN SIGN
        0x3d => [0x3d, false], # EQUALS SIGN
        0x3e => [0x3e, false], # GREATER-THAN SIGN
        0x3f => [0x3f, false], # QUESTION MARK
        0x40 => [0x5b7, true], # HEBREW POINT PATAH
        0x41 => [0x5b8, true], # KAMATS / HEBREW POINT QAMATS
        0x42 => [0x5b6, true], # HEBREW POINT SEGOL
        0x43 => [0x5b5, true], # TSEREH / HEBREW POINT TSERE
        0x44 => [0x5b4, true], # HIRIK / HEBREW POINT HIRIQ
        0x45 => [0x5b9, true], # HOLAM, LEFT SIN DOT / HEBREW POINT HOLAM
        0x46 => [0x5bb, true], # KUBUTS / HEBREW POINT QUBUTS
        0x47 => [0x5b0, true], # HEBREW POINT SHEVA
        0x48 => [0x5b2, true], # HEBREW POINT HATAF PATAH
        0x49 => [0x5b3, true], # HATAF KAMATS / HEBREW POINT HATAF QAMATS
        0x4a => [0x5b1, true], # HEBREW POINT HATAF SEGOL
        0x4b => [0x5bc, true], # HEBREW POINT DAGESH OR MAPIQ
        0x4c => [0x5bf, true], # RAFEH / HEBREW POINT RAFE
        0x4d => [0x5c1, true], # RIGHT SHIN DOT / HEBREW POINT  SHIN DOT
        0x4e => [0xfb1e, true], # VARIKA / HEBREW POINT JUDEO-SPANISH VARIKA
        0x5b => [0x5b, false], # OPENING SQUARE BRACKET / LEFT SQUARE BRACKET
        0x5d => [0x5d, false], # CLOSING SQUARE BRACKET / RIGHT SQUARE BRACKET
        0x60 => [0x5d0, false], # HEBREW LETTER ALEF
        0x61 => [0x5d1, false], # HEBREW LETTER BET
        0x62 => [0x5d2, false], # HEBREW LETTER GIMEL
        0x63 => [0x5d3, false], # HEBREW LETTER DALET
        0x64 => [0x5d4, false], # HEBREW LETTER HE
        0x65 => [0x5d5, false], # HEBREW LETTER VAV
        0x66 => [0x5d6, false], # HEBREW LETTER ZAYIN
        0x67 => [0x5d7, false], # HEBREW LETTER HET
        0x68 => [0x5d8, false], # HEBREW LETTER TET
        0x69 => [0x5d9, false], # HEBREW LETTER YOD
        0x6a => [0x5da, false], # HEBREW LETTER FINAL KAF
        0x6b => [0x5db, false], # HEBREW LETTER KAF
        0x6c => [0x5dc, false], # HEBREW LETTER LAMED
        0x6d => [0x5dd, false], # HEBREW LETTER FINAL MEM
        0x6e => [0x5de, false], # HEBREW LETTER MEM
        0x6f => [0x5df, false], # HEBREW LETTER FINAL NUN
        0x70 => [0x5e0, false], # HEBREW LETTER NUN
        0x71 => [0x5e1, false], # HEBREW LETTER SAMEKH
        0x72 => [0x5e2, false], # HEBREW LETTER AYIN
        0x73 => [0x5e3, false], # HEBREW LETTER FINAL PE
        0x74 => [0x5e4, false], # HEBREW LETTER PE
        0x75 => [0x5e5, false], # HEBREW LETTER FINAL TSADI
        0x76 => [0x5e6, false], # HEBREW LETTER TSADI
        0x77 => [0x5e7, false], # HEBREW LETTER QOF / KOF
        0x78 => [0x5e8, false], # HEBREW LETTER RESH
        0x79 => [0x5e9, false], # HEBREW LETTER SHIN
        0x7a => [0x5ea, false], # HEBREW LETTER TAV
        0x7b => [0x5f0, false], # HEBREW LIGATURE YIDDISH DOUBLE VAV / TSVEY VOVN
        0x7c => [0x5f1, false], # HEBREW LIGATURE YIDDISH VAV YOD / VOV YUD
        0x7d => [0x5f2, false] # HEBREW LIGATURE YIDDISH DOUBLE YOD / TSVEY YUDN
      }
      CHARSET_31 = { # Chinese, Japanese, Korean (EACC)
        0x215556 => [0x8461, false], # East Asian ideograph
        0x6f5557 => [0xc5d1, false], # Korean hangul
        0x456324 => [0x9f61, false], # East Asian ideograph
        0x6f5140 => [0xbccf, false], # Korean hangul
        0x6f5558 => [0xc5d4, false], # Korean hangul
        0x213536 => [0x53ec, false], # East Asian ideograph
        0x6f5d3c => [0xd64b, false], # Korean hangul
        0x215559 => [0x8438, false], # East Asian ideograph
        0x2d555a => [0x8386, false], # East Asian ideograph
        0x6f5c7c => [0xd5d0, false], # Korean hangul
        0x295b60 => [0x9e55, false], # East Asian ideograph
        0x2d555b => [0x8385, false], # East Asian ideograph
        0x6f555c => [0xc5e3, false], # Korean hangul
        0x6f5141 => [0xbcd0, false], # Korean hangul
        0x27555d => [0x5e2d, false], # East Asian ideograph
        0x23555e => [0x9b1f, false], # East Asian ideograph
        0x333f24 => [0x7718, false], # East Asian ideograph
        0x6f555f => [0xc5ed, false], # Korean hangul
        0x6f4f5c => [0xb9ac, false], # Korean hangul
        0x6f5560 => [0xc5ee, false], # Korean hangul
        0x6f4e21 => [0xb540, false], # Korean hangul
        0x4b4146 => [0x6362, false], # East Asian ideograph
        0x235031 => [0x9874, false], # East Asian ideograph
        0x225561 => [0x7273, false], # East Asian ideograph
        0x274257 => [0x6bd9, false], # East Asian ideograph
        0x295c28 => [0x9e58, false], # East Asian ideograph
        0x6f5142 => [0xbcd1, false], # Korean hangul
        0x6f5562 => [0xc5f4, false], # Korean hangul
        0x213727 => [0x5616, false], # East Asian ideograph
        0x215563 => [0x84c0, false], # East Asian ideograph
        0x215564 => [0x8499, false], # East Asian ideograph
        0x6f562e => [0xc679, false], # Korean hangul
        0x2d4674 => [0x51b2, false], # East Asian ideograph
        0x6f5565 => [0xc5fc, false], # Korean hangul
        0x4b4147 => [0x633f, false], # East Asian ideograph
        0x215566 => [0x8490, false], # East Asian ideograph
        0x6f5143 => [0xbcd2, false], # Korean hangul
        0x275567 => [0x82cd, false], # East Asian ideograph
        0x215568 => [0x853d, false], # East Asian ideograph
        0x6f5569 => [0xc600, false], # Korean hangul
        0x27314c => [0x6765, false], # East Asian ideograph
        0x276071 => [0x517b, false], # East Asian ideograph
        0x6f556a => [0xc601, false], # Korean hangul
        0x33325d => [0x4fa1, false], # East Asian ideograph
        0x6f5839 => [0xc9dd, false], # Korean hangul
        0x2d6b5f => [0x5273, false], # East Asian ideograph
        0x21556b => [0x851a, false], # East Asian ideograph
        0x6f5144 => [0xbcd5, false], # Korean hangul
        0x27556c => [0x83b2, false], # East Asian ideograph
        0x22556d => [0x727c, false], # East Asian ideograph
        0x21556e => [0x852d, false], # East Asian ideograph
        0x6f556f => [0xc610, false], # Korean hangul
        0x295721 => [0x9c86, false], # East Asian ideograph
        0x466074 => [0x76b2, false], # East Asian ideograph
        0x333529 => [0x53dc, false], # East Asian ideograph
        0x6f5145 => [0xbcf4, false], # Korean hangul
        0x225571 => [0x727f, false], # East Asian ideograph
        0x225d42 => [0x7521, false], # East Asian ideograph
        0x275949 => [0x8a89, false], # East Asian ideograph
        0x6f5037 => [0xba84, false], # Korean hangul
        0x215573 => [0x8514, false], # East Asian ideograph
        0x215574 => [0x84ec, false], # East Asian ideograph
        0x4c2330 => [0x5c53, false], # East Asian ideograph
        0x69656e => [0x7dd5, false], # East Asian ideograph
        0x6f5146 => [0xbcf5, false], # Korean hangul
        0x215576 => [0x8569, false], # East Asian ideograph
        0x282441 => [0x5c98, false], # East Asian ideograph
        0x234021 => [0x9132, false], # East Asian ideograph
        0x4d4176 => [0x91db, false], # East Asian ideograph
        0x335577 => [0x8602, false], # East Asian ideograph
        0x394022 => [0x6443, false], # East Asian ideograph
        0x6f5578 => [0xc634, false], # Korean hangul
        0x6f4f5d => [0xb9ad, false], # Korean hangul
        0x2d3749 => [0x5650, false], # East Asian ideograph
        0x287139 => [0x7ee8, false], # East Asian ideograph
        0x234024 => [0x9126, false], # East Asian ideograph
        0x6f557a => [0xc637, false], # Korean hangul
        0x213c35 => [0x5dde, false], # East Asian ideograph
        0x6f5147 => [0xbcf6, false], # Korean hangul
        0x6f557b => [0xc639, false], # Korean hangul
        0x215945 => [0x8b66, false], # East Asian ideograph
        0x21372c => [0x5606, false], # East Asian ideograph (variant of 4B372C which maps to 5606)
        0x27557c => [0x829c, false], # East Asian ideograph
        0x224027 => [0x69bf, false], # East Asian ideograph
        0x23557d => [0x9b34, false], # East Asian ideograph
        0x6f557e => [0xc640, false], # Korean hangul
        0x2d4029 => [0x5214, false], # East Asian ideograph
        0x6f5148 => [0xbcf8, false], # Korean hangul
        0x23402b => [0x9134, false], # East Asian ideograph
        0x21372d => [0x5609, false], # East Asian ideograph
        0x23402c => [0x9136, false], # East Asian ideograph
        0x6f5876 => [0xcc14, false], # Korean hangul
        0x22402d => [0x69a3, false], # East Asian ideograph
        0x22507c => [0x70dc, false], # East Asian ideograph
        0x22402e => [0x69a4, false], # East Asian ideograph
        0x6f5149 => [0xbcfc, false], # Korean hangul
        0x6f575f => [0xc8b0, false], # Korean hangul
        0x295a75 => [0x9e41, false], # East Asian ideograph
        0x4b525a => [0x7ffa, false], # East Asian ideograph
        0x234031 => [0x913a, false], # East Asian ideograph
        0x2d383f => [0x575a, false], # East Asian ideograph
        0x294371 => [0x94fd, false], # East Asian ideograph
        0x234032 => [0x913b, false], # East Asian ideograph
        0x6f5c27 => [0xd38c, false], # Korean hangul
        0x224034 => [0x69d4, false], # East Asian ideograph
        0x6f514a => [0xbd04, false], # Korean hangul
        0x335f73 => [0x9759, false], # East Asian ideograph
        0x6f4c33 => [0xb17c, false], # Korean hangul
        0x4b525b => [0x66dc, false], # East Asian ideograph (variant of 39525B which maps to 66DC)
        0x2e7c2e => [0x831c, false], # East Asian ideograph
        0x224038 => [0x69c3, false], # East Asian ideograph
        0x6f5b4d => [0xd280, false], # Korean hangul
        0x2d4039 => [0x67c6, false], # East Asian ideograph
        0x6f514b => [0xbd05, false], # Korean hangul
        0x276036 => [0x54cd, false], # East Asian ideograph
        0x395477 => [0x85a6, false], # East Asian ideograph
        0x213730 => [0x5617, false], # East Asian ideograph
        0x4b525c => [0x8002, false], # East Asian ideograph
        0x23403b => [0x9143, false], # East Asian ideograph
        0x295b6b => [0x9e57, false], # East Asian ideograph
        0x2d5963 => [0x8c98, false], # East Asian ideograph
        0x224c3c => [0x6f3b, false], # East Asian ideograph
        0x22403e => [0x6a11, false], # East Asian ideograph
        0x6f5a73 => [0xd0c8, false], # Korean hangul
        0x6f514c => [0xbd07, false], # Korean hangul
        0x213b74 => [0x5cfd, false], # East Asian ideograph
        0x23403f => [0x9145, false], # East Asian ideograph
        0x21594a => [0x8b80, false], # East Asian ideograph
        0x213731 => [0x560d, false], # East Asian ideograph
        0x225d49 => [0x752f, false], # East Asian ideograph
        0x234040 => [0x9148, false], # East Asian ideograph
        0x224041 => [0x6a00, false], # East Asian ideograph
        0x295b6c => [0x9e4b, false], # East Asian ideograph
        0x234042 => [0x9150, false], # East Asian ideograph
        0x234043 => [0x914e, false], # East Asian ideograph
        0x6f514d => [0xbd09, false], # Korean hangul
        0x213b75 => [0x5ced, false], # East Asian ideograph
        0x394a60 => [0x9ae6, false], # East Asian ideograph
        0x213732 => [0x562e, false], # East Asian ideograph
        0x334045 => [0x629b, false], # East Asian ideograph
        0x292a34 => [0x86ac, false], # East Asian ideograph
        0x224046 => [0x69e6, false], # East Asian ideograph
        0x2f2d79 => [0x88b5, false], # East Asian ideograph
        0x234048 => [0x9159, false], # East Asian ideograph
        0x6f514e => [0xbd10, false], # Korean hangul
        0x276039 => [0x9877, false], # East Asian ideograph
        0x234049 => [0x915c, false], # East Asian ideograph
        0x33494a => [0x70d6, false], # East Asian ideograph
        0x294372 => [0x9513, false], # East Asian ideograph
        0x22404b => [0x6a0b, false], # East Asian ideograph
        0x22404c => [0x69e5, false], # East Asian ideograph
        0x2e2f7a => [0x6738, false], # East Asian ideograph
        0x22404d => [0x69e9, false], # East Asian ideograph
        0x6f514f => [0xbd14, false], # Korean hangul
        0x27603a => [0x9879, false], # East Asian ideograph
        0x2d4f29 => [0x9f9d, false], # East Asian ideograph
        0x213734 => [0x564e, false], # East Asian ideograph
        0x2d404f => [0x6294, false], # East Asian ideograph
        0x6f5039 => [0xba87, false], # Korean hangul
        0x224050 => [0x69fc, false], # East Asian ideograph
        0x6f5b4e => [0xd284, false], # Korean hangul
        0x234052 => [0x915a, false], # East Asian ideograph
        0x6f5150 => [0xbd24, false], # Korean hangul
        0x213b78 => [0x5cf0, false], # East Asian ideograph
        0x234053 => [0x9161, false], # East Asian ideograph
        0x6f596b => [0xce6b, false], # Korean hangul
        0x225d4d => [0x753a, false], # East Asian ideograph
        0x224054 => [0x6a17, false], # East Asian ideograph
        0x4b4c3c => [0x7573, false], # East Asian ideograph
        0x224056 => [0x69e7, false], # East Asian ideograph
        0x224057 => [0x69eb, false], # East Asian ideograph
        0x6f5151 => [0xbd48, false], # Korean hangul
        0x213b79 => [0x5cf6, false], # East Asian ideograph
        0x294621 => [0x9553, false], # East Asian ideograph
        0x4b6266 => [0x9ed2, false], # East Asian ideograph
        0x6f5631 => [0xc688, false], # Korean hangul
        0x22405b => [0x69f1, false], # East Asian ideograph
        0x6f5152 => [0xbd49, false], # Korean hangul
        0x27603d => [0x9884, false], # East Asian ideograph
        0x22405e => [0x6a2b, false], # East Asian ideograph
        0x29444d => [0x952b, false], # East Asian ideograph
        0x22405f => [0x69ff, false], # East Asian ideograph
        0x224060 => [0x6a20, false], # East Asian ideograph
        0x234061 => [0x916f, false], # East Asian ideograph
        0x6f5153 => [0xbd4c, false], # Korean hangul
        0x27603e => [0x987c, false], # East Asian ideograph
        0x234062 => [0x916e, false], # East Asian ideograph
        0x275d60 => [0x94e8, false], # East Asian ideograph
        0x6f5a71 => [0xd0c1, false], # Korean hangul
        0x4b4e21 => [0x7b36, false], # East Asian ideograph
        0x224064 => [0x69ed, false], # East Asian ideograph
        0x28355b => [0x6484, false], # East Asian ideograph
        0x6f547d => [0xc545, false], # Korean hangul
        0x234066 => [0x917a, false], # East Asian ideograph
        0x6f582e => [0xc9ca, false], # Korean hangul
        0x6f5154 => [0xbd50, false], # Korean hangul
        0x27603f => [0x987d, false], # East Asian ideograph
        0x224067 => [0x6a1b, false], # East Asian ideograph
        0x213739 => [0x5657, false], # East Asian ideograph
        0x2d7143 => [0x55e2, false], # East Asian ideograph
        0x234068 => [0x9172, false], # East Asian ideograph
        0x2d5a63 => [0x8de5, false], # East Asian ideograph
        0x2d384a => [0x5872, false], # East Asian ideograph
        0x234069 => [0x9179, false], # East Asian ideograph
        0x27632c => [0x9f9a, false], # East Asian ideograph
        0x23406a => [0x9176, false], # East Asian ideograph
        0x4c233f => [0x5c76, false], # East Asian ideograph
        0x23406b => [0x9174, false], # East Asian ideograph
        0x6f5155 => [0xbd58, false], # Korean hangul
        0x213b7d => [0x5d1b, false], # East Asian ideograph
        0x276040 => [0x987f, false], # East Asian ideograph
        0x23406c => [0x9173, false], # East Asian ideograph
        0x23406d => [0x9185, false], # East Asian ideograph
        0x22406e => [0x6a18, false], # East Asian ideograph
        0x23406f => [0x9182, false], # East Asian ideograph
        0x4b5f30 => [0x9686, false], # East Asian ideograph (variant of 215F30 which maps to 9686)
        0x234070 => [0x918a, false], # East Asian ideograph
        0x6f5a75 => [0xd0d0, false], # Korean hangul
        0x213c38 => [0x5de8, false], # East Asian ideograph
        0x6f5156 => [0xbd59, false], # Korean hangul
        0x276041 => [0x9881, false], # East Asian ideograph
        0x234071 => [0x9186, false], # East Asian ideograph
        0x21373b => [0x5653, false], # East Asian ideograph
        0x234072 => [0x918c, false], # East Asian ideograph
        0x234073 => [0x9181, false], # East Asian ideograph
        0x224075 => [0x6a0c, false], # East Asian ideograph
        0x6f5157 => [0xbd64, false], # Korean hangul
        0x224076 => [0x6a0f, false], # East Asian ideograph
        0x21373c => [0x563f, false], # East Asian ideograph
        0x4b3f74 => [0x623b, false], # East Asian ideograph
        0x282f43 => [0x6206, false], # East Asian ideograph
        0x454738 => [0x6cfa, false], # East Asian ideograph
        0x275e6a => [0x9610, false], # East Asian ideograph
        0x274f36 => [0x5e0c, false], # East Asian ideograph
        0x6f5e21 => [0xd79d, false], # Korean hangul
        0x232b24 => [0x876a, false], # East Asian ideograph
        0x212b25 => [0x300c, false], # Ideographic left corner bracket
        0x2f5158 => [0x7cc7, false], # East Asian ideograph
        0x23407b => [0x9191, false], # East Asian ideograph
        0x225d55 => [0x754a, false], # East Asian ideograph
        0x294628 => [0x9552, false], # East Asian ideograph
        0x4b3050 => [0x4e8a, false], # East Asian ideograph
        0x22407c => [0x69ee, false], # East Asian ideograph
        0x232b27 => [0x874e, false], # East Asian ideograph
        0x695b37 => [0x6737, false], # East Asian ideograph
        0x23407d => [0x9190, false], # East Asian ideograph
        0x23407e => [0x918e, false], # East Asian ideograph
        0x4c6775 => [0x7962, false], # Unrelated variant of EACC 293032 which maps to 7962
        0x6f5159 => [0xbd81, false], # Korean hangul
        0x21373e => [0x5637, false], # East Asian ideograph
        0x294629 => [0x84e5, false], # East Asian ideograph
        0x4b3051 => [0x5f10, false], # East Asian ideograph
        0x6f503b => [0xbaa9, false], # Korean hangul
        0x222b2d => [0x602b, false], # East Asian ideograph
        0x6f5b50 => [0xd290, false], # Korean hangul
        0x455847 => [0x8a25, false], # East Asian ideograph (variant of 215847 which maps to 8A25)
        0x396b2f => [0x521f, false], # East Asian ideograph
        0x6f515a => [0xbd84, false], # Korean hangul
        0x6f5861 => [0xcad9, false], # Korean hangul
        0x222b30 => [0x6019, false], # East Asian ideograph
        0x225d57 => [0x754e, false], # East Asian ideograph
        0x4b3052 => [0x6275, false], # East Asian ideograph
        0x212b31 => [0xff3b, false], # Ideographic left square bracket
        0x4b3749 => [0x5668, false], # East Asian ideograph (variant of 213749 which maps to 5668)
        0x3a3b7d => [0x67b1, false], # East Asian ideograph
        0x216b33 => [0x5231, false], # East Asian ideograph
        0x23504a => [0x98bf, false], # East Asian ideograph
        0x4b5f35 => [0x6b92, false], # East Asian ideograph
        0x474270 => [0x94bc, false], # East Asian ideograph
        0x6f515b => [0xbd87, false], # Korean hangul
        0x212b35 => [0x3001, false], # Ideographic comma
        0x216b36 => [0x5235, false], # East Asian ideograph
        0x4b6268 => [0x9ed9, false], # East Asian ideograph
        0x3f404f => [0x638a, false], # East Asian ideograph
        0x695b7b => [0x6926, false], # East Asian ideograph
        0x222b38 => [0x601b, false], # East Asian ideograph
        0x216b39 => [0x5233, false], # East Asian ideograph
        0x6f485f => [0xac00, false], # Korean hangul
        0x276047 => [0x988a, false], # East Asian ideograph
        0x212b3a => [0xff1a, false], # Ideographic colon
        0x225d59 => [0x754b, false], # East Asian ideograph
        0x333f3f => [0x51f4, false], # East Asian ideograph
        0x212b3b => [0xff1f, false], # Ideographic question mark
        0x2d3852 => [0x51a2, false], # East Asian ideograph
        0x295739 => [0x9c9e, false], # East Asian ideograph
        0x222b3d => [0x6033, false], # East Asian ideograph
        0x276b3e => [0x522d, false], # East Asian ideograph
        0x6f515d => [0xbd89, false], # Korean hangul
        0x276048 => [0x9888, false], # East Asian ideograph
        0x275a28 => [0x8d42, false], # East Asian ideograph
        0x225d5a => [0x7548, false], # East Asian ideograph
        0x29462d => [0x9549, false], # East Asian ideograph
        0x6f4b45 => [0xb07d, false], # Korean hangul
        0x274f3c => [0x79f0, false], # East Asian ideograph
        0x706b42 => [0x80bc, false], # East Asian ideograph
        0x6f515e => [0xbd90, false], # Korean hangul
        0x276049 => [0x9891, false], # East Asian ideograph
        0x217b75 => [0x5aa0, false], # East Asian ideograph
        0x706b44 => [0x80bd, false], # East Asian ideograph
        0x33615a => [0x8eb0, false], # East Asian ideograph
        0x222b45 => [0x600d, false], # East Asian ideograph
        0x2d3854 => [0x5896, false], # East Asian ideograph
        0x274f3d => [0x79cd, false], # East Asian ideograph
        0x28533c => [0x709d, false], # East Asian ideograph
        0x69573b => [0x5f41, false], # East Asian ideograph
        0x216b47 => [0x5260, false], # East Asian ideograph
        0x6f5b51 => [0xd291, false], # Korean hangul
        0x6f515f => [0xbd91, false], # Korean hangul
        0x27604a => [0x9893, false], # East Asian ideograph
        0x213744 => [0x5678, false], # East Asian ideograph
        0x4b3057 => [0x4e99, false], # East Asian ideograph
        0x6f2477 => [0x3154, false], # Korean hangul
        0x2f4053 => [0x914f, false], # East Asian ideograph
        0x226b4b => [0x7b37, false], # East Asian ideograph
        0x29573c => [0x9c91, false], # East Asian ideograph
        0x706b4c => [0x80e9, false], # East Asian ideograph
        0x4b5f3a => [0x967a, false], # East Asian ideograph
        0x216b4d => [0x525e, false], # East Asian ideograph
        0x6f5160 => [0xbd93, false], # Korean hangul
        0x6f5940 => [0xccad, false], # Korean hangul
        0x29573d => [0x9c92, false], # East Asian ideograph
        0x6f5161 => [0xbd95, false], # Korean hangul
        0x216b53 => [0x5255, false], # East Asian ideograph
        0x705f50 => [0x549d, false], # East Asian ideograph
        0x2e6b54 => [0x7b04, false], # East Asian ideograph
        0x292b55 => [0x86f3, false], # East Asian ideograph
        0x6f555a => [0xc5e0, false], # Korean hangul
        0x70622a => [0x7339, false], # East Asian ideograph
        0x6f5162 => [0xbd99, false], # Korean hangul
        0x212b59 => [0xff0f, false], # Ideographic solidus
        0x2d562e => [0x8024, false], # East Asian ideograph
        0x213563 => [0x5439, false], # East Asian ideograph
        0x216b5b => [0x526e, false], # East Asian ideograph
        0x6f4b67 => [0xb0c8, false], # Korean hangul
        0x4b3d24 => [0x53a6, false], # East Asian ideograph
        0x6f5163 => [0xbd9c, false], # Korean hangul
        0x225d60 => [0x755b, false], # East Asian ideograph
        0x276b5f => [0x672d, false], # East Asian ideograph
        0x2d433e => [0x667b, false], # East Asian ideograph
        0x235053 => [0x98c6, false], # East Asian ideograph
        0x345452 => [0x7118, false], # East Asian ideograph
        0x6f5b40 => [0xd1d8, false], # Korean hangul
        0x213569 => [0x5462, false], # East Asian ideograph
        0x4c6b62 => [0x7b4c, false], # East Asian ideograph (variant of 226B62 which maps to 7B4C)
        0x394634 => [0x6b96, false], # East Asian ideograph (variant of 214634 which maps to 6B96)
        0x274171 => [0x629a, false], # East Asian ideograph
        0x6f5165 => [0xbdf0, false], # Korean hangul
        0x28356d => [0x6512, false], # East Asian ideograph
        0x274f44 => [0x79ef, false], # East Asian ideograph
        0x295742 => [0x9c95, false], # East Asian ideograph
        0x4b3d27 => [0x5ec3, false], # East Asian ideograph
        0x6f5166 => [0xbe0c, false], # Korean hangul
        0x6f4d23 => [0xb310, false], # Korean hangul
        0x213b61 => [0x5c64, false], # East Asian ideograph (variant of 4B3B61 which maps to 5C64)
        0x4b5277 => [0x8068, false], # East Asian ideograph
        0x336162 => [0x9a23, false], # East Asian ideograph
        0x705f51 => [0x54d0, false], # East Asian ideograph
        0x6f4a46 => [0xae50, false], # Korean hangul
        0x292b6e => [0x86f0, false], # East Asian ideograph
        0x2d4a60 => [0x6c02, false], # East Asian ideograph
        0x222b6f => [0x604c, false], # East Asian ideograph
        0x6f5167 => [0xbe0d, false], # Korean hangul
        0x276052 => [0x989b, false], # East Asian ideograph
        0x6f4d24 => [0xb311, false], # Korean hangul
        0x232b72 => [0x87ac, false], # East Asian ideograph
        0x6f496c => [0xad49, false], # Korean hangul
        0x216b74 => [0x5282, false], # East Asian ideograph
        0x2d5f2e => [0x661c, false], # East Asian ideograph
        0x216b75 => [0x5281, false], # East Asian ideograph
        0x6f5168 => [0xbe10, false], # Korean hangul
        0x215966 => [0x8c8c, false], # East Asian ideograph
        0x6f5621 => [0xc641, false], # Korean hangul
        0x33515c => [0x7dab, false], # East Asian ideograph
        0x275622 => [0x8427, false], # East Asian ideograph
        0x215623 => [0x859b, false], # East Asian ideograph
        0x226b79 => [0x7b72, false], # East Asian ideograph
        0x215624 => [0x8591, false], # East Asian ideograph
        0x2d496b => [0x70df, false], # East Asian ideograph
        0x226b7a => [0x7b78, false], # East Asian ideograph
        0x6f5169 => [0xbe14, false], # Korean hangul
        0x275626 => [0x8537, false], # East Asian ideograph
        0x23487c => [0x9481, false], # East Asian ideograph
        0x226b7c => [0x7b67, false], # East Asian ideograph
        0x6f5627 => [0xc654, false], # Korean hangul
        0x2d5635 => [0x846f, false], # East Asian ideograph
        0x215628 => [0x8587, false], # East Asian ideograph
        0x29352d => [0x8c30, false], # East Asian ideograph
        0x275629 => [0x84dd, false], # East Asian ideograph
        0x21562a => [0x85a9, false], # East Asian ideograph
        0x276055 => [0x613f, false], # East Asian ideograph
        0x6f4d27 => [0xb315, false], # Korean hangul
        0x225d67 => [0x7563, false], # East Asian ideograph
        0x273d2f => [0x5385, false], # East Asian ideograph
        0x335d23 => [0x8a76, false], # East Asian ideograph
        0x6f562d => [0xc678, false], # Korean hangul
        0x2e2968 => [0x5f51, false], # East Asian ideograph
        0x21562e => [0x85c9, false], # East Asian ideograph
        0x4b3d2c => [0x53b0, false], # East Asian ideograph
        0x21562f => [0x85b0, false], # East Asian ideograph
        0x4b527c => [0x8080, false], # East Asian ideograph
        0x233f4e => [0x9100, false], # East Asian ideograph
        0x4b4e39 => [0x5cfa, false], # East Asian ideograph
        0x275631 => [0x827a, false], # East Asian ideograph
        0x215632 => [0x85ea, false], # East Asian ideograph
        0x695633 => [0x5cbe, false], # East Asian ideograph
        0x6f516c => [0xbe1f, false], # Korean hangul
        0x21596a => [0x8ca0, false], # East Asian ideograph
        0x213751 => [0x5687, false], # East Asian ideograph
        0x275635 => [0x836f, false], # East Asian ideograph
        0x6f5529 => [0xc558, false], # Korean hangul
        0x235636 => [0x9b43, false], # East Asian ideograph
        0x275637 => [0x853c, false], # East Asian ideograph
        0x6f5c2e => [0xd39c, false], # Korean hangul
        0x4c5638 => [0x729f, false], # East Asian ideograph
        0x275639 => [0x853a, false], # East Asian ideograph
        0x21563a => [0x8606, false], # East Asian ideograph
        0x233f50 => [0x9107, false], # East Asian ideograph
        0x225927 => [0x73ea, false], # East Asian ideograph
        0x21563b => [0x860b, false], # East Asian ideograph
        0x6f5a22 => [0xceac, false], # Korean hangul
        0x21563c => [0x8607, false], # East Asian ideograph
        0x224c5e => [0x6f36, false], # East Asian ideograph
        0x21563d => [0x860a, false], # East Asian ideograph
        0x4b3d2f => [0x5ef0, false], # East Asian ideograph
        0x696576 => [0x7e90, false], # East Asian ideograph
        0x21563e => [0x862d, false], # East Asian ideograph
        0x276059 => [0x9885, false], # East Asian ideograph
        0x21596c => [0x8ca1, false], # East Asian ideograph
        0x2d563f => [0x6a97, false], # East Asian ideograph
        0x276023 => [0x5de9, false], # East Asian ideograph
        0x275640 => [0x85d3, false], # East Asian ideograph
        0x346126 => [0x6900, false], # East Asian ideograph
        0x2d3421 => [0x5294, false], # East Asian ideograph
        0x235641 => [0x9b4b, false], # East Asian ideograph
        0x215642 => [0x863f, false], # East Asian ideograph
        0x4b5f49 => [0x51cb, false], # East Asian ideograph
        0x2d4971 => [0x70a4, false], # East Asian ideograph
        0x395643 => [0x4e55, false], # East Asian ideograph
        0x6f4d2c => [0xb35c, false], # Korean hangul
        0x213754 => [0x5695, false], # East Asian ideograph
        0x275644 => [0x4e47, false], # East Asian ideograph
        0x70602d => [0x55b9, false], # East Asian ideograph
        0x692426 => [0x3046, false], # Hiragana letter U
        0x2d5a7e => [0x8e7b, false], # East Asian ideograph
        0x6f5645 => [0xc6cc, false], # Korean hangul
        0x6f5646 => [0xc6cd, false], # Korean hangul
        0x2d572b => [0x8797, false], # East Asian ideograph
        0x275647 => [0x5904, false], # East Asian ideograph
        0x215648 => [0x865c, false], # East Asian ideograph
        0x27605b => [0x98ce, false], # East Asian ideograph
        0x28645a => [0x7817, false], # East Asian ideograph
        0x6f4d2d => [0xb35f, false], # Korean hangul
        0x225d6d => [0x7579, false], # East Asian ideograph
        0x21564a => [0x865f, false], # East Asian ideograph
        0x2d563c => [0x8613, false], # East Asian ideograph
        0x45564b => [0x865e, false], # East Asian ideograph (variant of 21564B which maps to 865E)
        0x21564c => [0x8667, false], # East Asian ideograph
        0x6f5171 => [0xbe4c, false], # Korean hangul
        0x27605c => [0x98d2, false], # East Asian ideograph
        0x69564e => [0x5d76, false], # East Asian ideograph
        0x29302d => [0x88e3, false], # East Asian ideograph
        0x22564f => [0x72b4, false], # East Asian ideograph
        0x6f496e => [0xad6c, false], # Korean hangul
        0x277169 => [0x5522, false], # East Asian ideograph
        0x6f5650 => [0xc6ec, false], # Korean hangul
        0x6f5c2f => [0xd3a0, false], # Korean hangul
        0x235061 => [0x98e4, false], # East Asian ideograph
        0x4b5f4c => [0x9d8f, false], # East Asian ideograph
        0x225652 => [0x72b5, false], # East Asian ideograph
        0x27605d => [0x53f0, false], # East Asian ideograph (duplicate simplified)
        0x6f4d2f => [0xb365, false], # Korean hangul
        0x6f5653 => [0xc704, false], # Korean hangul
        0x294642 => [0x94e0, false], # East Asian ideograph
        0x692429 => [0x3049, false], # Hiragana letter small O
        0x333f55 => [0x5b3e, false], # East Asian ideograph
        0x6f5654 => [0xc705, false], # Korean hangul
        0x6f5655 => [0xc708, false], # Korean hangul
        0x225656 => [0x72bc, false], # East Asian ideograph
        0x695657 => [0x5d90, false], # East Asian ideograph
        0x27605e => [0x522e, false], # East Asian ideograph
        0x6f4d30 => [0xb367, false], # Korean hangul
        0x225658 => [0x72c3, false], # East Asian ideograph
        0x217747 => [0x5853, false], # East Asian ideograph
        0x6f5659 => [0xc719, false], # Korean hangul
        0x21565a => [0x86cb, false], # East Asian ideograph
        0x293537 => [0x8c20, false], # East Asian ideograph
        0x235063 => [0x98e5, false], # East Asian ideograph
        0x6f5174 => [0xbe55, false], # Korean hangul
        0x27605f => [0x98d3, false], # East Asian ideograph
        0x6f4d31 => [0xb368, false], # Korean hangul
        0x23565d => [0x9b74, false], # East Asian ideograph
        0x6f4b23 => [0xafb9, false], # Korean hangul
        0x4b306c => [0x96e0, false], # East Asian ideograph
        0x6f565e => [0xc730, false], # Korean hangul
        0x6f5638 => [0xc6a7, false], # Korean hangul
        0x6f565f => [0xc735, false], # Korean hangul
        0x4c6074 => [0x76b9, false], # East Asian ideograph
        0x224c65 => [0x6f2d, false], # East Asian ideograph
        0x215660 => [0x86df, false], # East Asian ideograph
        0x4b5052 => [0x7c56, false], # East Asian ideograph
        0x6f5175 => [0xbe57, false], # Korean hangul
        0x6f4d32 => [0xb369, false], # Korean hangul
        0x213b64 => [0x5c6f, false], # East Asian ideograph
        0x6f5662 => [0xc73d, false], # Korean hangul
        0x396223 => [0x9bfd, false], # East Asian ideograph (variant of 216223)
        0x333f58 => [0x61f4, false], # East Asian ideograph
        0x235663 => [0x9b68, false], # East Asian ideograph
        0x2d3428 => [0x5226, false], # East Asian ideograph
        0x2e5a40 => [0x73b3, false], # East Asian ideograph
        0x215664 => [0x86db, false], # East Asian ideograph
        0x6f583b => [0xc9e2, false], # Korean hangul
        0x293539 => [0x8c33, false], # East Asian ideograph
        0x215665 => [0x86e4, false], # East Asian ideograph
        0x4b5f50 => [0x96e3, false], # East Asian ideograph
        0x4b3b37 => [0x51a9, false], # East Asian ideograph
        0x6f5176 => [0xbe59, false], # Korean hangul
        0x2f2f5d => [0x7e48, false], # East Asian ideograph
        0x276061 => [0x98d5, false], # East Asian ideograph
        0x286460 => [0x7856, false], # East Asian ideograph
        0x21375b => [0x56b6, false], # East Asian ideograph
        0x215667 => [0x86f9, false], # East Asian ideograph
        0x225930 => [0x73db, false], # East Asian ideograph
        0x6f5668 => [0xc751, false], # Korean hangul
        0x22403d => [0x6a12, false], # East Asian ideograph
        0x6f5669 => [0xc758, false], # Korean hangul
        0x224c67 => [0x6f34, false], # East Asian ideograph
        0x4b566a => [0x8708, false], # East Asian ideograph (variant of 21566A which maps to 8708)
        0x21566b => [0x8700, false], # East Asian ideograph
        0x276062 => [0x98d8, false], # East Asian ideograph
        0x285e7a => [0x75d6, false], # East Asian ideograph
        0x6f4d34 => [0xb36b, false], # Korean hangul
        0x22566c => [0x72cc, false], # East Asian ideograph
        0x294647 => [0x954f, false], # East Asian ideograph
        0x6f566d => [0xc77c, false], # Korean hangul
        0x334730 => [0x6e5f, false], # East Asian ideograph
        0x6f5041 => [0xbabb, false], # Korean hangul
        0x22566e => [0x72db, false], # East Asian ideograph
        0x22566f => [0x72cd, false], # East Asian ideograph
        0x21356d => [0x5496, false], # East Asian ideograph
        0x276063 => [0x98de, false], # East Asian ideograph
        0x6f4d35 => [0xb36e, false], # Korean hangul
        0x21375d => [0x56c1, false], # East Asian ideograph
        0x225d75 => [0x7571, false], # East Asian ideograph
        0x333f5b => [0x6133, false], # East Asian ideograph
        0x235672 => [0x9b80, false], # East Asian ideograph
        0x235673 => [0x9b8c, false], # East Asian ideograph
        0x2f5e42 => [0x9ec9, false], # East Asian ideograph
        0x6f5674 => [0xc789, false], # Korean hangul
        0x6f543c => [0xc318, false], # Korean hangul
        0x2d5675 => [0x9f05, false], # East Asian ideograph
        0x6f4d36 => [0xb370, false], # Korean hangul
        0x21375e => [0x56c2, false], # East Asian ideograph
        0x275676 => [0x8680, false], # East Asian ideograph
        0x692430 => [0x3050, false], # Hiragana letter GU
        0x4d386f => [0x544b, false], # East Asian ideograph
        0x2d4122 => [0x6485, false], # East Asian ideograph
        0x224123 => [0x69f0, false], # East Asian ideograph
        0x295756 => [0x9ca9, false], # East Asian ideograph
        0x215679 => [0x8774, false], # East Asian ideograph
        0x224124 => [0x69f2, false], # East Asian ideograph
        0x21567a => [0x8766, false], # East Asian ideograph
        0x234125 => [0x9193, false], # East Asian ideograph
        0x6f4d37 => [0xb371, false], # Korean hangul
        0x23567b => [0x9b7d, false], # East Asian ideograph
        0x21774e => [0x5856, false], # East Asian ideograph
        0x4b3072 => [0x4eed, false], # East Asian ideograph
        0x6f4a4a => [0xae60, false], # Korean hangul
        0x33567c => [0x8671, false], # East Asian ideograph
        0x6f567d => [0xc798, false], # Korean hangul
        0x224128 => [0x6a14, false], # East Asian ideograph
        0x21567e => [0x8757, false], # East Asian ideograph
        0x224129 => [0x6a63, false], # East Asian ideograph
        0x295030 => [0x989e, false], # East Asian ideograph
        0x6f517b => [0xbe60, false], # Korean hangul
        0x4b412a => [0x6323, false], # East Asian ideograph
        0x6f4d38 => [0xb374, false], # Korean hangul
        0x225742 => [0x731e, false], # East Asian ideograph
        0x23412b => [0x919d, false], # East Asian ideograph
        0x212b33 => [0x3002, false], # Ideographic full stop
        0x23412c => [0x919a, false], # East Asian ideograph
        0x2d342e => [0x8274, false], # East Asian ideograph
        0x6f5538 => [0xc580, false], # Korean hangul
        0x276067 => [0x9968, false], # East Asian ideograph
        0x6f4d39 => [0xb378, false], # Korean hangul
        0x234130 => [0x91a2, false], # East Asian ideograph
        0x3f476f => [0x51c8, false], # East Asian ideograph
        0x334131 => [0x6425, false], # East Asian ideograph
        0x6f5042 => [0xbabd, false], # Korean hangul
        0x2d4132 => [0x642f, false], # East Asian ideograph
        0x287130 => [0x7edb, false], # East Asian ideograph
        0x234c29 => [0x9708, false], # East Asian ideograph
        0x6f517d => [0xbe64, false], # Korean hangul
        0x234134 => [0x919b, false], # East Asian ideograph (variant of 4D4134 which maps to 919B)
        0x6f4d3a => [0xb380, false], # Korean hangul
        0x213762 => [0x56c8, false], # East Asian ideograph
        0x336179 => [0x9c7b, false], # East Asian ideograph
        0x4c3474 => [0x631d, false], # East Asian ideograph
        0x235d36 => [0x9e15, false], # East Asian ideograph
        0x274136 => [0x62a1, false], # East Asian ideograph
        0x274f5c => [0x6d3c, false], # East Asian ideograph
        0x6f4f68 => [0xb9cc, false], # Korean hangul
        0x224137 => [0x6a67, false], # East Asian ideograph
        0x344138 => [0x8022, false], # East Asian ideograph
        0x6f517e => [0xbe68, false], # Korean hangul
        0x224139 => [0x6a43, false], # East Asian ideograph
        0x6f4d3b => [0xb383, false], # Korean hangul
        0x22413a => [0x6a33, false], # East Asian ideograph
        0x225938 => [0x73e3, false], # East Asian ideograph
        0x22413b => [0x6a32, false], # East Asian ideograph
        0x274f5d => [0x7a9d, false], # East Asian ideograph
        0x27413c => [0x62e3, false], # East Asian ideograph
        0x706247 => [0x9987, false], # East Asian ideograph
        0x23413d => [0x91aa, false], # East Asian ideograph
        0x27606a => [0x996e, false], # East Asian ideograph
        0x6f4d3c => [0xb385, false], # Korean hangul
        0x213b66 => [0x5c79, false], # East Asian ideograph
        0x213764 => [0x56d1, false], # East Asian ideograph
        0x22413f => [0x6a28, false], # East Asian ideograph
        0x213321 => [0x5167, false], # East Asian ideograph
        0x4c3f68 => [0x69c7, false], # East Asian ideograph
        0x224140 => [0x6a48, false], # East Asian ideograph
        0x224141 => [0x6a50, false], # East Asian ideograph
        0x224142 => [0x6a52, false], # East Asian ideograph
        0x334c2c => [0x754d, false], # East Asian ideograph
        0x336058 => [0x9855, false], # East Asian ideograph
        0x224143 => [0x6a72, false], # East Asian ideograph
        0x6f4d3d => [0xb38c, false], # Korean hangul
        0x274570 => [0x6743, false], # East Asian ideograph
        0x285836 => [0x7315, false], # East Asian ideograph
        0x224145 => [0x6a3e, false], # East Asian ideograph
        0x224146 => [0x6a77, false], # East Asian ideograph
        0x287134 => [0x7ed7, false], # East Asian ideograph
        0x224147 => [0x6a5b, false], # East Asian ideograph
        0x214148 => [0x63ea, false], # East Asian ideograph
        0x6f4d3e => [0xb3c4, false], # Korean hangul
        0x225d7e => [0x757f, false], # East Asian ideograph
        0x217755 => [0x589a, false], # East Asian ideograph
        0x2d3877 => [0x5900, false], # East Asian ideograph
        0x22414a => [0x6a5e, false], # East Asian ideograph
        0x21414b => [0x643e, false], # East Asian ideograph
        0x21414c => [0x6413, false], # East Asian ideograph
        0x23414d => [0x91b5, false], # East Asian ideograph
        0x3f4a60 => [0x7266, false], # East Asian ideograph
        0x6f4d3f => [0xb3c5, false], # Korean hangul
        0x6f4c78 => [0xb2f7, false], # Korean hangul
        0x335d3b => [0x57dc, false], # East Asian ideograph
        0x22414f => [0x6a51, false], # East Asian ideograph
        0x2d4150 => [0x6428, false], # East Asian ideograph
        0x29575f => [0x9ca0, false], # East Asian ideograph
        0x224151 => [0x6a56, false], # East Asian ideograph
        0x2d4152 => [0x6447, false], # East Asian ideograph
        0x6f4d40 => [0xb3c8, false], # Korean hangul
        0x224153 => [0x6a36, false], # East Asian ideograph
        0x2d4154 => [0x635c, false], # East Asian ideograph
        0x2d3436 => [0x52f3, false], # East Asian ideograph
        0x274155 => [0x62a2, false], # East Asian ideograph
        0x217c30 => [0x5a93, false], # East Asian ideograph
        0x224156 => [0x6a7a, false], # East Asian ideograph
        0x234157 => [0x91bd, false], # East Asian ideograph
        0x6f4d41 => [0xb3cb, false], # Korean hangul
        0x213769 => [0x56e4, false], # East Asian ideograph
        0x224158 => [0x6a3f, false], # East Asian ideograph
        0x4b4f7b => [0x7b7a, false], # East Asian ideograph
        0x21623b => [0x9d12, false], # East Asian ideograph (variant of 4B623B which maps to 9D12)
        0x4b523e => [0x7f9a, false], # East Asian ideograph (variant of 21523E which maps to 7F9A)
        0x23415a => [0x91c2, false], # East Asian ideograph
        0x293651 => [0x8d36, false], # East Asian ideograph
        0x23415b => [0x91c4, false], # East Asian ideograph
        0x33347d => [0x53c1, false], # East Asian ideograph
        0x23415c => [0x91c3, false], # East Asian ideograph
        0x275a30 => [0x8d24, false], # East Asian ideograph
        0x29415d => [0x917d, false], # East Asian ideograph
        0x29586a => [0x9ccb, false], # East Asian ideograph
        0x274f64 => [0x7a83, false], # East Asian ideograph
        0x27415f => [0x6402, false], # East Asian ideograph
        0x70624e => [0x9995, false], # East Asian ideograph
        0x224c76 => [0x6efa, false], # East Asian ideograph
        0x224833 => [0x6cb4, false], # East Asian ideograph
        0x4b5164 => [0x770c, false], # East Asian ideograph
        0x4c4146 => [0x8538, false], # East Asian ideograph
        0x234161 => [0x91d4, false], # East Asian ideograph
        0x284257 => [0x68bc, false], # East Asian ideograph
        0x294656 => [0x955b, false], # East Asian ideograph
        0x234162 => [0x91d3, false], # East Asian ideograph
        0x234163 => [0x91d5, false], # East Asian ideograph
        0x217639 => [0x580e, false], # East Asian ideograph
        0x234164 => [0x91d9, false], # East Asian ideograph
        0x274b22 => [0x72ef, false], # East Asian ideograph
        0x6f5b59 => [0xd2b8, false], # Korean hangul
        0x274165 => [0x635e, false], # East Asian ideograph
        0x286272 => [0x770d, false], # East Asian ideograph
        0x274166 => [0x62e8, false], # East Asian ideograph
        0x6f4d44 => [0xb3d4, false], # Korean hangul
        0x234168 => [0x91e2, false], # East Asian ideograph
        0x6f534a => [0xc1a9, false], # Korean hangul
        0x234169 => [0x91ed, false], # East Asian ideograph
        0x23416a => [0x91f7, false], # East Asian ideograph
        0x333d2f => [0x5e81, false], # East Asian ideograph
        0x23416b => [0x91fa, false], # East Asian ideograph
        0x6f4d45 => [0xb3d5, false], # Korean hangul
        0x21775c => [0x5889, false], # East Asian ideograph
        0x22416c => [0x69f9, false], # East Asian ideograph
        0x215543 => [0x83cc, false], # East Asian ideograph
        0x4b4e56 => [0x78fa, false], # East Asian ideograph
        0x22416d => [0x6a64, false], # East Asian ideograph
        0x295427 => [0x9a85, false], # East Asian ideograph
        0x27416e => [0x6251, false], # East Asian ideograph
        0x217c31 => [0x5aac, false], # East Asian ideograph
        0x23416f => [0x91f2, false], # East Asian ideograph
        0x234171 => [0x91e8, false], # East Asian ideograph
        0x294458 => [0x952c, false], # East Asian ideograph
        0x4b434d => [0x663f, false], # East Asian ideograph
        0x234172 => [0x91f6, false], # East Asian ideograph
        0x2d343c => [0x52a2, false], # East Asian ideograph
        0x334c3e => [0x8e08, false], # East Asian ideograph
        0x234173 => [0x91ee, false], # East Asian ideograph
        0x274174 => [0x62e5, false], # East Asian ideograph
        0x4b3d4b => [0x5f3e, false], # East Asian ideograph
        0x334c36 => [0x753b, false], # East Asian ideograph (variant of 274C36 which maps to 753B)
        0x4b4c67 => [0x761f, false], # East Asian ideograph (variant of 214C67 which maps to 761F)
        0x224175 => [0x6aa8, false], # East Asian ideograph
        0x29465a => [0x955f, false], # East Asian ideograph
        0x274176 => [0x51fb, false], # East Asian ideograph
        0x692441 => [0x3061, false], # Hiragana letter TI
        0x21332c => [0x5178, false], # East Asian ideograph
        0x235d43 => [0x9e7b, false], # East Asian ideograph
        0x224177 => [0x6aa5, false], # East Asian ideograph
        0x2d343d => [0x52e7, false], # East Asian ideograph
        0x224179 => [0x6a96, false], # East Asian ideograph
        0x4b3d4c => [0x5f25, false], # East Asian ideograph (variant of 273D4C)
        0x222c24 => [0x608a, false], # East Asian ideograph
        0x27417a => [0x6321, false], # East Asian ideograph
        0x6f4d48 => [0xb3db, false], # Korean hangul
        0x707360 => [0x7b7b, false], # East Asian ideograph
        0x286032 => [0x75ac, false], # East Asian ideograph
        0x21332d => [0x517c, false], # East Asian ideograph
        0x27417c => [0x636e, false], # East Asian ideograph
        0x216c27 => [0x5296, false], # East Asian ideograph
        0x27417d => [0x63b3, false], # East Asian ideograph
        0x284f26 => [0x6cf7, false], # East Asian ideograph
        0x224a44 => [0x6e12, false], # East Asian ideograph
        0x22417e => [0x6a7d, false], # East Asian ideograph
        0x6f5d6a => [0xd750, false], # Korean hangul
        0x226c29 => [0x7b73, false], # East Asian ideograph
        0x2e2b74 => [0x609b, false], # East Asian ideograph
        0x276077 => [0x997f, false], # East Asian ideograph
        0x6f4d49 => [0xb3fc, false], # Korean hangul
        0x217760 => [0x589b, false], # East Asian ideograph
        0x223378 => [0x647d, false], # East Asian ideograph
        0x232c2c => [0x87ee, false], # East Asian ideograph
        0x274b28 => [0x72de, false], # East Asian ideograph
        0x222c2f => [0x609e, false], # East Asian ideograph
        0x6f4b28 => [0xafc9, false], # Korean hangul
        0x217761 => [0x587c, false], # East Asian ideograph
        0x222c30 => [0x6083, false], # East Asian ideograph
        0x4b4e5b => [0x783f, false], # East Asian ideograph
        0x22483b => [0x6d28, false], # East Asian ideograph
        0x216c33 => [0x52ae, false], # East Asian ideograph
        0x4c4345 => [0x67a6, false], # East Asian ideograph
        0x222c34 => [0x60a7, false], # East Asian ideograph
        0x6f4d4b => [0xb410, false], # Korean hangul
        0x213773 => [0x5718, false], # East Asian ideograph
        0x233b2e => [0x8ec9, false], # East Asian ideograph
        0x216c38 => [0x52bc, false], # East Asian ideograph
        0x2d454e => [0x697d, false], # East Asian ideograph
        0x217763 => [0x5888, false], # East Asian ideograph
        0x692446 => [0x3066, false], # Hiragana letter TE
        0x232c3a => [0x87d6, false], # East Asian ideograph
        0x235d48 => [0x9e83, false], # East Asian ideograph
        0x39302d => [0x534b, false], # East Asian ideograph
        0x6f4d4d => [0xb41c, false], # Korean hangul
        0x213775 => [0x571f, false], # East Asian ideograph
        0x286037 => [0x763f, false], # East Asian ideograph
        0x692447 => [0x3067, false], # Hiragana letter DE
        0x223731 => [0x65c6, false], # East Asian ideograph
        0x294478 => [0x9522, false], # East Asian ideograph
        0x274b2c => [0x730e, false], # East Asian ideograph
        0x287144 => [0x7ee0, false], # East Asian ideograph
        0x234c3d => [0x971d, false], # East Asian ideograph
        0x706c43 => [0x70c0, false], # East Asian ideograph
        0x213333 => [0x5191, false], # East Asian ideograph
        0x333066 => [0x5fc8, false], # East Asian ideograph
        0x69613a => [0x7549, false], # East Asian ideograph
        0x6f4f6c => [0xb9d1, false], # Korean hangul
        0x216c46 => [0x52d4, false], # East Asian ideograph
        0x234c3e => [0x9719, false], # East Asian ideograph
        0x453666 => [0x5ad0, false], # East Asian ideograph
        0x232c48 => [0x87d3, false], # East Asian ideograph
        0x6f4d4f => [0xb428, false], # Korean hangul
        0x6f4b29 => [0xafcb, false], # Korean hangul
        0x294662 => [0x956a, false], # East Asian ideograph
        0x692449 => [0x3069, false], # Hiragana letter DO
        0x275f50 => [0x96be, false], # East Asian ideograph
        0x235d4b => [0x9e88, false], # East Asian ideograph
        0x274b2e => [0x732e, false], # East Asian ideograph
        0x235a6b => [0x9d3d, false], # East Asian ideograph
        0x292c4c => [0x866e, false], # East Asian ideograph
        0x4b346b => [0x5df5, false], # East Asian ideograph
        0x69595e => [0x63b5, false], # East Asian ideograph
        0x472c4d => [0x8801, false], # East Asian ideograph (variant of 232C4D which maps to 8801)
        0x28603a => [0x75c8, false], # East Asian ideograph
        0x454774 => [0x6e15, false], # East Asian ideograph
        0x6f5564 => [0xc5f7, false], # Korean hangul
        0x694664 => [0x51ea, false], # East Asian ideograph
        0x294221 => [0x9495, false], # East Asian ideograph
        0x6f4975 => [0xad7c, false], # Korean hangul
        0x292c55 => [0x86cf, false], # East Asian ideograph
        0x6f5762 => [0xc8c8, false], # Korean hangul
        0x4b5f6f => [0x970a, false], # East Asian ideograph
        0x2d4f37 => [0x7980, false], # East Asian ideograph
        0x216c58 => [0x52f0, false], # East Asian ideograph
        0x294222 => [0x9490, false], # East Asian ideograph
        0x6f5a48 => [0xcf78, false], # Korean hangul
        0x216c5a => [0x52f1, false], # East Asian ideograph
        0x2d632b => [0x5c28, false], # East Asian ideograph
        0x4b4135 => [0x6368, false], # East Asian ideograph
        0x275c3e => [0x8fc7, false], # East Asian ideograph
        0x223b7a => [0x67f6, false], # East Asian ideograph
        0x69244d => [0x306d, false], # Hiragana letter NE
        0x222c5d => [0x60c4, false], # East Asian ideograph
        0x3a284c => [0x53a9, false], # East Asian ideograph (variant of 4C284C)
        0x294223 => [0x94ad, false], # East Asian ideograph
        0x235d4f => [0x9e87, false], # East Asian ideograph
        0x3f5e60 => [0x9586, false], # East Asian ideograph
        0x395773 => [0x7daf, false], # East Asian ideograph
        0x4b5f71 => [0x9756, false], # East Asian ideograph
        0x224844 => [0x6d39, false], # East Asian ideograph
        0x292c61 => [0x86f4, false], # East Asian ideograph
        0x6f4d54 => [0xb451, false], # Korean hangul
        0x6f4b2a => [0xafcd, false], # Korean hangul
        0x69544b => [0x5870, false], # East Asian ideograph
        0x213339 => [0x51a5, false], # East Asian ideograph
        0x294224 => [0x94aa, false], # East Asian ideograph
        0x6f563f => [0xc6b9, false], # Korean hangul
        0x292c64 => [0x877e, false], # East Asian ideograph
        0x4b5f72 => [0x975b, false], # East Asian ideograph
        0x4b5365 => [0x8133, false], # East Asian ideograph
        0x222c66 => [0x60e2, false], # East Asian ideograph
        0x6f4a50 => [0xae6c, false], # Korean hangul
        0x294225 => [0x94ab, false], # East Asian ideograph
        0x2d5664 => [0x9f04, false], # East Asian ideograph
        0x6f542a => [0xc2ed, false], # Korean hangul
        0x4b5f73 => [0x975c, false], # East Asian ideograph (variant of 215F73 which maps to 975C)
        0x335228 => [0x94b5, false], # East Asian ideograph
        0x336c6b => [0x6031, false], # East Asian ideograph
        0x6f4d56 => [0xb458, false], # Korean hangul
        0x692450 => [0x3070, false], # Hiragana letter BA
        0x4b4e67 => [0x79d8, false], # East Asian ideograph
        0x6f5c37 => [0xd3bc, false], # Korean hangul
        0x28714d => [0x7ee1, false], # East Asian ideograph
        0x216c6f => [0x530b, false], # East Asian ideograph
        0x6f4d57 => [0xb460, false], # Korean hangul
        0x222c73 => [0x6103, false], # East Asian ideograph
        0x6f5a21 => [0xcea5, false], # Korean hangul
        0x4b3d5c => [0x5f83, false], # East Asian ideograph
        0x6f4d58 => [0xb461, false], # Korean hangul
        0x6f5425 => [0xc2e0, false], # Korean hangul
        0x692452 => [0x3072, false], # Hiragana letter HI
        0x21333d => [0x51b6, false], # East Asian ideograph
        0x215721 => [0x8759, false], # East Asian ideograph
        0x6f4f6e => [0xb9d9, false], # Korean hangul
        0x6f5723 => [0xc7a4, false], # Korean hangul
        0x225724 => [0x72f4, false], # East Asian ideograph
        0x6f5d74 => [0xd769, false], # Korean hangul
        0x6f4d59 => [0xb463, false], # Korean hangul
        0x215725 => [0x879e, false], # East Asian ideograph
        0x695438 => [0x57b3, false], # East Asian ideograph
        0x692453 => [0x3073, false], # Hiragana letter BI
        0x6f5726 => [0xc7a7, false], # Korean hangul
        0x6f5727 => [0xc7ac, false], # Korean hangul
        0x2f5e66 => [0x9b12, false], # East Asian ideograph
        0x4b4b71 => [0x7f3e, false], # East Asian ideograph (variant of 2D4B71 which maps to 7F3E)
        0x6f5728 => [0xc7ad, false], # Korean hangul
        0x225729 => [0x7302, false], # East Asian ideograph
        0x697323 => [0x9d64, false], # East Asian ideograph
        0x6f4d5a => [0xb465, false], # Korean hangul
        0x6f572a => [0xc7b4, false], # Korean hangul
        0x2e4873 => [0x6fa3, false], # East Asian ideograph
        0x21572b => [0x87b3, false], # East Asian ideograph
        0x333330 => [0x518a, false], # East Asian ideograph
        0x21572c => [0x87bb, false], # East Asian ideograph
        0x29577a => [0x9cad, false], # East Asian ideograph
        0x21572d => [0x87c8, false], # East Asian ideograph
        0x39563c => [0x56cc, false], # East Asian ideograph
        0x222632 => [0x5db8, false], # East Asian ideograph
        0x21572e => [0x87d2, false], # East Asian ideograph
        0x4b5d58 => [0x9234, false], # East Asian ideograph
        0x6f4d5b => [0xb46c, false], # Korean hangul
        0x21572f => [0x87ba, false], # East Asian ideograph
        0x2d5730 => [0x87c7, false], # East Asian ideograph
        0x235731 => [0x9b92, false], # East Asian ideograph
        0x6f5c38 => [0xd3c4, false], # Korean hangul
        0x275732 => [0x86f2, false], # East Asian ideograph
        0x6f4e75 => [0xb7ac, false], # Korean hangul
        0x275733 => [0x866b, false], # East Asian ideograph
        0x6f4d5c => [0xb480, false], # Korean hangul
        0x275734 => [0x8749, false], # East Asian ideograph
        0x215735 => [0x87fb, false], # East Asian ideograph
        0x215736 => [0x8805, false], # East Asian ideograph
        0x2d5228 => [0x9262, false], # East Asian ideograph
        0x29577c => [0x9cb0, false], # East Asian ideograph
        0x695737 => [0x5f16, false], # East Asian ideograph
        0x222634 => [0x5dbf, false], # East Asian ideograph
        0x213d21 => [0x5ebe, false], # East Asian ideograph
        0x6f4d5d => [0xb488, false], # Korean hangul
        0x235739 => [0x9b9d, false], # East Asian ideograph
        0x6f573a => [0xc811, false], # Korean hangul
        0x2d3453 => [0x758b, false], # East Asian ideograph
        0x21573b => [0x8822, false], # East Asian ideograph
        0x213132 => [0x4f5b, false], # East Asian ideograph
        0x21573c => [0x8823, false], # East Asian ideograph
        0x21573d => [0x8821, false], # East Asian ideograph
        0x23417a => [0x91f8, false], # East Asian ideograph
        0x6f4d5e => [0xb4a4, false], # Korean hangul
        0x21573e => [0x881f, false], # East Asian ideograph
        0x275e69 => [0x5173, false], # East Asian ideograph
        0x6f5b7a => [0xd330, false], # Korean hangul
        0x692458 => [0x3078, false], # Hiragana letter HE
        0x21573f => [0x8831, false], # East Asian ideograph
        0x235d5a => [0x9e95, false], # East Asian ideograph
        0x4b5740 => [0x8827, false], # East Asian ideograph
        0x2d572d => [0x8748, false], # East Asian ideograph
        0x215741 => [0x8836, false], # East Asian ideograph
        0x69533b => [0x555d, false], # East Asian ideograph
        0x275742 => [0x86ee, false], # East Asian ideograph
        0x27614f => [0x9a74, false], # East Asian ideograph
        0x6f4d5f => [0xb4b7, false], # Korean hangul
        0x215743 => [0x8840, false], # East Asian ideograph
        0x4c5c3a => [0x73f1, false], # East Asian ideograph
        0x6f5744 => [0xc82d, false], # Korean hangul
        0x694823 => [0x7872, false], # East Asian ideograph
        0x6f5745 => [0xc82f, false], # Korean hangul
        0x2d522b => [0x9475, false], # East Asian ideograph
        0x215746 => [0x8853, false], # East Asian ideograph (variant of 4B5746 which maps to 8853)
        0x275747 => [0x4e8d, false], # East Asian ideograph
        0x2d4562 => [0x681d, false], # East Asian ideograph
        0x275a36 => [0x8d56, false], # East Asian ideograph
        0x6f4d60 => [0xb4c0, false], # Korean hangul
        0x69245a => [0x307a, false], # Hiragana letter PE
        0x213345 => [0x51dd, false], # East Asian ideograph
        0x215749 => [0x885b, false], # East Asian ideograph
        0x235d5c => [0x9e91, false], # East Asian ideograph
        0x2d3164 => [0x7ae2, false], # East Asian ideograph
        0x4b4a2e => [0x55b6, false], # East Asian ideograph
        0x21574a => [0x885d, false], # East Asian ideograph
        0x474236 => [0x949a, false], # East Asian ideograph
        0x2d4425 => [0x686e, false], # East Asian ideograph
        0x29533d => [0x9a90, false], # East Asian ideograph
        0x21574c => [0x8862, false], # East Asian ideograph
        0x3b3922 => [0x8db5, false], # East Asian ideograph
        0x21574d => [0x8863, false], # East Asian ideograph
        0x23574e => [0x9ba0, false], # East Asian ideograph
        0x2d3457 => [0x62fe, false], # East Asian ideograph
        0x21574f => [0x8868, false], # East Asian ideograph
        0x6f5750 => [0xc885, false], # Korean hangul
        0x213d22 => [0x5eca, false], # East Asian ideograph
        0x2d4564 => [0x68b9, false], # East Asian ideograph
        0x215752 => [0x8881, false], # East Asian ideograph
        0x27602e => [0x97e9, false], # East Asian ideograph
        0x6f5753 => [0xc88b, false], # Korean hangul
        0x69613e => [0x7569, false], # East Asian ideograph
        0x6f5754 => [0xc88c, false], # Korean hangul
        0x215755 => [0x8888, false], # East Asian ideograph
        0x4b3d67 => [0x5f84, false], # East Asian ideograph (variant of 273D67)
        0x215756 => [0x88ab, false], # East Asian ideograph
        0x6f4d63 => [0xb4dd, false], # Korean hangul
        0x29367e => [0x8d59, false], # East Asian ideograph
        0x217337 => [0x568a, false], # East Asian ideograph
        0x215759 => [0x888d, false], # East Asian ideograph
        0x6f5967 => [0xce60, false], # Korean hangul
        0x21575a => [0x888b, false], # East Asian ideograph
        0x21575b => [0x889e, false], # East Asian ideograph
        0x4d3c6c => [0x8fb6, false], # East Asian ideograph
        0x21575c => [0x88c1, false], # East Asian ideograph
        0x273a36 => [0x5988, false], # East Asian ideograph
        0x6f4921 => [0xac70, false], # Korean hangul
        0x23575d => [0x9bc6, false], # East Asian ideograph
        0x23575e => [0x9bbf, false], # East Asian ideograph
        0x22575f => [0x733b, false], # East Asian ideograph
        0x2d5760 => [0x5e2c, false], # East Asian ideograph
        0x6f4d65 => [0xb4e3, false], # Korean hangul
        0x4c7265 => [0x7dfc, false], # East Asian ideograph
        0x69245f => [0x307f, false], # Hiragana letter MI
        0x6f4922 => [0xac71, false], # Korean hangul
        0x225762 => [0x733a, false], # East Asian ideograph
        0x6f5125 => [0xbc45, false], # Korean hangul
        0x2e4670 => [0x6cd0, false], # East Asian ideograph
        0x284539 => [0x6b9a, false], # East Asian ideograph
        0x2d345b => [0x6607, false], # East Asian ideograph
        0x275763 => [0x91cc, false], # East Asian ideograph
        0x393054 => [0x4f0d, false], # East Asian ideograph
        0x226260 => [0x777a, false], # East Asian ideograph
        0x6f5764 => [0xc8d4, false], # Korean hangul
        0x232739 => [0x85da, false], # East Asian ideograph
        0x215765 => [0x88dd, false], # East Asian ideograph
        0x395564 => [0x6726, false], # East Asian ideograph
        0x235766 => [0x9bb9, false], # East Asian ideograph
        0x6f4e2e => [0xb560, false], # Korean hangul
        0x6f5767 => [0xc8e0, false], # Korean hangul
        0x6f504b => [0xbb3b, false], # Korean hangul
        0x6f2469 => [0x3138, false], # Korean hangul
        0x215768 => [0x88f3, false], # East Asian ideograph
        0x2d5232 => [0x8fa0, false], # East Asian ideograph
        0x6f5769 => [0xc8f0, false], # Korean hangul
        0x6f5b60 => [0xd2cb, false], # Korean hangul
        0x69576a => [0x603a, false], # East Asian ideograph
        0x22576b => [0x7352, false], # East Asian ideograph
        0x21334c => [0x51f9, false], # East Asian ideograph
        0x27576c => [0x5236, false], # East Asian ideograph
        0x225521 => [0x721d, false], # East Asian ideograph
        0x2d345d => [0x5349, false], # East Asian ideograph
        0x6f576d => [0xc8fd, false], # Korean hangul
        0x2d5233 => [0x7f78, false], # East Asian ideograph
        0x23576e => [0x9bc0, false], # East Asian ideograph
        0x29312b => [0x89d1, false], # East Asian ideograph
        0x2d5361 => [0x811a, false], # East Asian ideograph
        0x4b576f => [0x8910, false], # East Asian ideograph (variant of 21576F which maps to 8910)
        0x6f4d68 => [0xb4ed, false], # Korean hangul
        0x6f4b2e => [0xafe9, false], # Korean hangul
        0x6f4925 => [0xac77, false], # Korean hangul
        0x275771 => [0x8934, false], # East Asian ideograph
        0x215772 => [0x8912, false], # East Asian ideograph
        0x294164 => [0x948b, false], # East Asian ideograph
        0x275773 => [0x88e4, false], # East Asian ideograph
        0x215774 => [0x892a, false], # East Asian ideograph
        0x6f4d69 => [0xb4ef, false], # Korean hangul
        0x29467c => [0x9546, false], # East Asian ideograph
        0x3f4926 => [0x6e08, false], # East Asian ideograph
        0x21334e => [0x51fd, false], # East Asian ideograph
        0x6f5776 => [0xc92c, false], # Korean hangul
        0x234221 => [0x91f9, false], # East Asian ideograph
        0x215777 => [0x893b, false], # East Asian ideograph
        0x224222 => [0x6a7f, false], # East Asian ideograph
        0x6f5a33 => [0xcf11, false], # Korean hangul
        0x234223 => [0x9204, false], # East Asian ideograph
        0x216231 => [0x9cf6, false], # East Asian ideograph
        0x215779 => [0x8938, false], # East Asian ideograph
        0x224224 => [0x6a91, false], # East Asian ideograph
        0x21577a => [0x8944, false], # East Asian ideograph
        0x214225 => [0x64e0, false], # East Asian ideograph
        0x6f4927 => [0xac79, false], # Korean hangul
        0x22577b => [0x7358, false], # East Asian ideograph
        0x224226 => [0x6a9f, false], # East Asian ideograph
        0x4b4a38 => [0x71d7, false], # East Asian ideograph
        0x21577c => [0x8960, false], # East Asian ideograph
        0x234227 => [0x920a, false], # East Asian ideograph
        0x27577d => [0x8884, false], # East Asian ideograph
        0x234228 => [0x9225, false], # East Asian ideograph
        0x295347 => [0x9a93, false], # East Asian ideograph
        0x216232 => [0x9cf4, false], # East Asian ideograph
        0x21577e => [0x8964, false], # East Asian ideograph
        0x274229 => [0x6401, false], # East Asian ideograph
        0x22422a => [0x6a92, false], # East Asian ideograph
        0x692465 => [0x3085, false], # Hiragana letter small YU
        0x22422b => [0x6aa3, false], # East Asian ideograph
        0x6f504c => [0xbb3c, false], # Korean hangul
        0x6f545b => [0xc42c, false], # Korean hangul
        0x23422c => [0x9228, false], # East Asian ideograph
        0x6f5a35 => [0xcf15, false], # Korean hangul
        0x6f5b61 => [0xd2d4, false], # Korean hangul
        0x29556c => [0x960b, false], # East Asian ideograph
        0x4b5b46 => [0x8f0c, false], # East Asian ideograph
        0x21422e => [0x64fe, false], # East Asian ideograph
        0x456260 => [0x5e7a, false], # East Asian ideograph
        0x275c57 => [0x8fd8, false], # East Asian ideograph
        0x23422f => [0x9203, false], # East Asian ideograph
        0x276030 => [0x827d, false], # East Asian ideograph
        0x692466 => [0x3086, false], # Hiragana letter YU
        0x274230 => [0x6446, false], # East Asian ideograph
        0x2d567b => [0x8717, false], # East Asian ideograph
        0x234231 => [0x9200, false], # East Asian ideograph
        0x393573 => [0x5611, false], # East Asian ideograph
        0x234232 => [0x9218, false], # East Asian ideograph
        0x295c3e => [0x9e37, false], # East Asian ideograph
        0x274233 => [0x62e6, false], # East Asian ideograph
        0x6f4d6d => [0xb518, false], # Korean hangul
        0x6f4b2f => [0xaff0, false], # Korean hangul
        0x274234 => [0x6400, false], # East Asian ideograph
        0x274235 => [0x6444, false], # East Asian ideograph
        0x234236 => [0x9208, false], # East Asian ideograph
        0x6f5a37 => [0xcf20, false], # Korean hangul
        0x224237 => [0x6a9b, false], # East Asian ideograph
        0x234238 => [0x921c, false], # East Asian ideograph
        0x2d6079 => [0x8218, false], # East Asian ideograph
        0x6f492b => [0xac83, false], # Korean hangul
        0x213353 => [0x5206, false], # East Asian ideograph
        0x27423a => [0x6405, false], # East Asian ideograph
        0x287349 => [0x7f30, false], # East Asian ideograph
        0x2d3464 => [0x613d, false], # East Asian ideograph
        0x23423b => [0x9224, false], # East Asian ideograph
        0x2d3021 => [0x5f0c, false], # East Asian ideograph
        0x6f5a38 => [0xcf24, false], # Korean hangul
        0x335772 => [0x8943, false], # East Asian ideograph
        0x293132 => [0x89cc, false], # East Asian ideograph
        0x226635 => [0x7911, false], # East Asian ideograph
        0x33423d => [0x53ce, false], # East Asian ideograph
        0x284d2b => [0x6d54, false], # East Asian ideograph
        0x2d486b => [0x6f82, false], # East Asian ideograph
        0x692469 => [0x3089, false], # Hiragana letter RA
        0x6f576b => [0xc8f5, false], # Korean hangul
        0x2d3c7c => [0x83f4, false], # East Asian ideograph
        0x4b516d => [0x7dcf, false], # East Asian ideograph
        0x216237 => [0x9d23, false], # East Asian ideograph
        0x224242 => [0x6aa0, false], # East Asian ideograph
        0x6f4d70 => [0xb524, false], # Korean hangul
        0x234243 => [0x9212, false], # East Asian ideograph
        0x69246a => [0x308a, false], # Hiragana letter RI
        0x334244 => [0x6559, false], # East Asian ideograph
        0x4b4a3e => [0x7235, false], # East Asian ideograph
        0x2f5e7d => [0x6641, false], # East Asian ideograph
        0x393577 => [0x9fa2, false], # East Asian ideograph
        0x6f5b62 => [0xd1f8, false], # Korean hangul
        0x214247 => [0x6557, false], # East Asian ideograph
        0x6f4d71 => [0xb525, false], # Korean hangul
        0x6f4c7a => [0xb2fa, false], # Korean hangul
        0x234248 => [0x91ff, false], # East Asian ideograph
        0x285323 => [0x8367, false], # East Asian ideograph
        0x69246b => [0x308b, false], # Hiragana letter RU
        0x217345 => [0x5699, false], # East Asian ideograph
        0x224249 => [0x6a9e, false], # East Asian ideograph
        0x22424a => [0x6a87, false], # East Asian ideograph
        0x6f5a3b => [0xcf2f, false], # Korean hangul
        0x22424b => [0x6a8e, false], # East Asian ideograph
        0x23424e => [0x9206, false], # East Asian ideograph
        0x27424f => [0x542f, false], # East Asian ideograph
        0x6f5a3c => [0xcf30, false], # Korean hangul
        0x21623a => [0x9d1b, false], # East Asian ideograph
        0x224251 => [0x6aab, false], # East Asian ideograph
        0x6f4d73 => [0xb528, false], # Korean hangul
        0x234252 => [0x9249, false], # East Asian ideograph
        0x6f4930 => [0xac89, false], # Korean hangul
        0x394243 => [0x4ff2, false], # East Asian ideograph
        0x705f61 => [0x54da, false], # East Asian ideograph
        0x234254 => [0x924d, false], # East Asian ideograph
        0x6f556b => [0xc606, false], # Korean hangul
        0x224255 => [0x6ac8, false], # East Asian ideograph
        0x224864 => [0x6d19, false], # East Asian ideograph
        0x4c4177 => [0x8223, false], # East Asian ideograph
        0x274256 => [0x655b, false], # East Asian ideograph
        0x6f4d74 => [0xb529, false], # Korean hangul
        0x224257 => [0x6aae, false], # East Asian ideograph
        0x6f4931 => [0xac8a, false], # Korean hangul
        0x6f497c => [0xad90, false], # Korean hangul
        0x234258 => [0x923a, false], # East Asian ideograph
        0x2d346a => [0x5918, false], # East Asian ideograph
        0x6f4879 => [0xac30, false], # Korean hangul
        0x2d3c7d => [0x53a2, false], # East Asian ideograph
        0x2d4f3e => [0x7a3e, false], # East Asian ideograph
        0x4d4134 => [0x919b, false], # East Asian ideograph
        0x23425c => [0x922e, false], # East Asian ideograph
        0x6f4932 => [0xac8b, false], # Korean hangul
        0x22425d => [0x6abf, false], # East Asian ideograph
        0x2d5241 => [0x7fa3, false], # East Asian ideograph
        0x6f5a3f => [0xcf58, false], # Korean hangul
        0x23425f => [0x9233, false], # East Asian ideograph
        0x294260 => [0x94b7, false], # East Asian ideograph
        0x234261 => [0x9266, false], # East Asian ideograph
        0x6f4933 => [0xac8c, false], # Korean hangul
        0x214263 => [0x65ac, false], # East Asian ideograph
        0x29446d => [0x951b, false], # East Asian ideograph
        0x6f5a40 => [0xcf5c, false], # Korean hangul
        0x224264 => [0x6aca, false], # East Asian ideograph
        0x224867 => [0x6d0e, false], # East Asian ideograph
        0x4b3938 => [0x5942, false], # East Asian ideograph
        0x214266 => [0x65b7, false], # East Asian ideograph
        0x6f4934 => [0xac90, false], # Korean hangul
        0x22375b => [0x65fb, false], # East Asian ideograph
        0x4b4a45 => [0x5c13, false], # East Asian ideograph
        0x234268 => [0x9235, false], # East Asian ideograph
        0x4b4b77 => [0x4ec0, false], # East Asian ideograph
        0x6f5a41 => [0xcf64, false], # Korean hangul
        0x4b5b52 => [0x8f42, false], # East Asian ideograph
        0x6f4d78 => [0xb530, false], # Korean hangul
        0x23426b => [0x9250, false], # East Asian ideograph
        0x692472 => [0x3092, false], # Hiragana letter WO
        0x21335d => [0x5228, false], # East Asian ideograph
        0x22375c => [0x65fc, false], # East Asian ideograph
        0x23426c => [0x926b, false], # East Asian ideograph
        0x23426d => [0x9239, false], # East Asian ideograph
        0x6f556c => [0xc607, false], # Korean hangul
        0x6f5a42 => [0xcf65, false], # Korean hangul
        0x6f4b6c => [0xb0e5, false], # Korean hangul
        0x23426f => [0x926d, false], # East Asian ideograph
        0x234270 => [0x926c, false], # East Asian ideograph
        0x6f4936 => [0xac9c, false], # Korean hangul
        0x234271 => [0x924f, false], # East Asian ideograph
        0x2d4272 => [0x65e3, false], # East Asian ideograph
        0x6f5c3e => [0xd3ed, false], # Korean hangul
        0x2d3c7e => [0x53a0, false], # East Asian ideograph
        0x6f5a43 => [0xcf67, false], # Korean hangul
        0x294274 => [0x94bf, false], # East Asian ideograph
        0x6f4d7a => [0xb532, false], # Korean hangul
        0x6f4937 => [0xac9f, false], # Korean hangul
        0x234277 => [0x9260, false], # East Asian ideograph
        0x2d302d => [0x4e17, false], # East Asian ideograph
        0x6f4d62 => [0xb4dc, false], # Korean hangul
        0x6f5a44 => [0xcf69, false], # Korean hangul
        0x234c6a => [0x9741, false], # East Asian ideograph
        0x4b5b55 => [0x8ee2, false], # East Asian ideograph
        0x224279 => [0x6ae6, false], # East Asian ideograph
        0x216d24 => [0x531c, false], # East Asian ideograph
        0x6f4d7b => [0xb534, false], # Korean hangul
        0x69562e => [0x5cbb, false], # East Asian ideograph
        0x6f4938 => [0xaca0, false], # Korean hangul
        0x6f5b3a => [0xd1a4, false], # Korean hangul
        0x275823 => [0x88ad, false], # East Asian ideograph
        0x29424b => [0x94a3, false], # East Asian ideograph
        0x235d77 => [0x9ead, false], # East Asian ideograph
        0x6f4f75 => [0xb9e5, false], # Korean hangul
        0x213d6a => [0x5f97, false], # East Asian ideograph
        0x2e3870 => [0x714a, false], # East Asian ideograph
        0x6f5a45 => [0xcf70, false], # Korean hangul
        0x22486c => [0x6d00, false], # East Asian ideograph
        0x234c6b => [0x9747, false], # East Asian ideograph
        0x23427e => [0x9236, false], # East Asian ideograph
        0x6f4d7c => [0xb537, false], # Korean hangul
        0x6f4b32 => [0xb00c, false], # Korean hangul
        0x222d2a => [0x610a, false], # East Asian ideograph
        0x22442a => [0x6b35, false], # East Asian ideograph
        0x216d2e => [0x532d, false], # East Asian ideograph
        0x6f4d7d => [0xb538, false], # Korean hangul
        0x6f493a => [0xaca8, false], # Korean hangul
        0x213362 => [0x5230, false], # East Asian ideograph
        0x276822 => [0x507b, false], # East Asian ideograph
        0x2d3473 => [0x5374, false], # East Asian ideograph
        0x6f5a47 => [0xcf74, false], # Korean hangul
        0x4c3f7a => [0x6922, false], # East Asian ideograph
        0x29535a => [0x9a9f, false], # East Asian ideograph
        0x222d32 => [0x6112, false], # East Asian ideograph
        0x4b5b58 => [0x8ee3, false], # East Asian ideograph
        0x4b393f => [0x5333, false], # East Asian ideograph
        0x216d33 => [0x5330, false], # East Asian ideograph
        0x2d486e => [0x6f97, false], # East Asian ideograph (not in Unicode)
        0x282d34 => [0x607d, false], # East Asian ideograph
        0x6f497e => [0xada4, false], # Korean hangul
        0x224e32 => [0x6fca, false], # East Asian ideograph
        0x6f5c3f => [0xd3f0, false], # Korean hangul
        0x2f5a48 => [0x9d44, false], # East Asian ideograph
        0x22486f => [0x6d33, false], # East Asian ideograph
        0x4c5f58 => [0x7640, false], # East Asian ideograph
        0x6f4e2f => [0xb561, false], # Korean hangul
        0x6f493c => [0xacaa, false], # Korean hangul
        0x39424f => [0x5554, false], # East Asian ideograph
        0x2d3032 => [0x7add, false], # East Asian ideograph
        0x33392f => [0x9029, false], # East Asian ideograph
        0x284f5d => [0x6ca3, false], # East Asian ideograph
        0x22442d => [0x6b3b, false], # East Asian ideograph
        0x216d3e => [0x533d, false], # East Asian ideograph
        0x51513b => [0x7e9f, false], # East Asian ideograph
        0x222d3f => [0x6121, false], # East Asian ideograph
        0x6f4f76 => [0xb9e8, false], # Korean hangul
        0x292768 => [0x8572, false], # East Asian ideograph
        0x274b5f => [0x7410, false], # East Asian ideograph
        0x6f5a4a => [0xcf85, false], # Korean hangul
        0x232d41 => [0x8841, false], # East Asian ideograph
        0x2d4277 => [0x65ee, false], # East Asian ideograph
        0x293a6b => [0x8e7f, false], # East Asian ideograph
        0x232a57 => [0x873e, false], # East Asian ideograph
        0x6f4b33 => [0xb00d, false], # Korean hangul
        0x222d43 => [0x6106, false], # East Asian ideograph
        0x294251 => [0x94c8, false], # East Asian ideograph
        0x3b2d44 => [0x8842, false], # East Asian ideograph
        0x295053 => [0x98d9, false], # East Asian ideograph
        0x287178 => [0x7ef6, false], # East Asian ideograph
        0x226d47 => [0x7c00, false], # East Asian ideograph
        0x2d4141 => [0x63b2, false], # East Asian ideograph
        0x6f493f => [0xacb0, false], # Korean hangul
        0x6f547e => [0xc548, false], # Korean hangul
        0x294252 => [0x94c9, false], # East Asian ideograph
        0x296028 => [0x9f86, false], # East Asian ideograph
        0x2f3833 => [0x8d91, false], # East Asian ideograph
        0x216d4b => [0x535d, false], # East Asian ideograph
        0x6f4940 => [0xacb8, false], # Korean hangul
        0x284f61 => [0x6ee0, false], # East Asian ideograph
        0x6f5c40 => [0xd3f4, false], # Korean hangul
        0x295360 => [0x9a98, false], # East Asian ideograph
        0x4b5b5e => [0x5f01, false], # East Asian ideograph
        0x232d51 => [0x884a, false], # East Asian ideograph
        0x6f4941 => [0xacb9, false], # Korean hangul
        0x294254 => [0x94cb, false], # East Asian ideograph
        0x286d54 => [0x7b5a, false], # East Asian ideograph
        0x222d56 => [0x53af, false], # East Asian ideograph
        0x232d57 => [0x8850, false], # East Asian ideograph
        0x21336a => [0x524c, false], # East Asian ideograph
        0x294255 => [0x94ca, false], # East Asian ideograph
        0x29602b => [0x9f85, false], # East Asian ideograph
        0x6f4f77 => [0xb9ec, false], # Korean hangul
        0x21313a => [0x4f38, false], # East Asian ideograph
        0x3f462b => [0x5e30, false], # East Asian ideograph
        0x274b64 => [0x7391, false], # East Asian ideograph
        0x6f5a4f => [0xcfb0, false], # Korean hangul
        0x696d5a => [0x8f4c, false], # East Asian ideograph
        0x29327e => [0x8c07, false], # East Asian ideograph
        0x6f4b34 => [0xb010, false], # Korean hangul
        0x6f4943 => [0xacbc, false], # Korean hangul
        0x21336b => [0x524b, false], # East Asian ideograph
        0x4d4862 => [0x9229, false], # East Asian ideograph
        0x222d5e => [0x6137, false], # East Asian ideograph
        0x28717d => [0x7efa, false], # East Asian ideograph
        0x6f5a50 => [0xcfc4, false], # Korean hangul
        0x6f4b75 => [0xb113, false], # Korean hangul
        0x6f4a5a => [0xaebd, false], # Korean hangul
        0x6f4944 => [0xacbd, false], # Korean hangul
        0x21735b => [0x56c3, false], # East Asian ideograph
        0x225541 => [0x723f, false], # East Asian ideograph
        0x226d63 => [0x7c20, false], # East Asian ideograph
        0x2d4147 => [0x6271, false], # East Asian ideograph
        0x216d66 => [0x5393, false], # East Asian ideograph
        0x6f4945 => [0xacc1, false], # Korean hangul
        0x21336d => [0x5247, false], # East Asian ideograph
        0x294258 => [0x94b0, false], # East Asian ideograph
        0x6f4b22 => [0xafb8, false], # Korean hangul
        0x335941 => [0x54d7, false], # East Asian ideograph
        0x274b67 => [0x73af, false], # East Asian ideograph
        0x234c78 => [0x975d, false], # East Asian ideograph
        0x234835 => [0x942b, false], # East Asian ideograph
        0x6f5052 => [0xbb4f, false], # Korean hangul
        0x276d6d => [0x538d, false], # East Asian ideograph
        0x274b68 => [0x7477, false], # East Asian ideograph
        0x6f5a53 => [0xcfe4, false], # Korean hangul
        0x225235 => [0x712f, false], # East Asian ideograph
        0x294f23 => [0x9880, false], # East Asian ideograph
        0x4b546d => [0x82d3, false], # East Asian ideograph (variant of 21546D which maps to 82D3)
        0x6f4c7b => [0xb2fb, false], # Korean hangul
        0x6f4947 => [0xacc4, false], # Korean hangul
        0x21336f => [0x5256, false], # East Asian ideograph
        0x225544 => [0x7242, false], # East Asian ideograph
        0x6f5724 => [0xc7a5, false], # Korean hangul
        0x274932 => [0x6cfb, false], # East Asian ideograph
        0x4b682e => [0x4ec2, false], # East Asian ideograph
        0x274b69 => [0x73ba, false], # East Asian ideograph
        0x6f5a54 => [0xcfe8, false], # Korean hangul
        0x234837 => [0x9441, false], # East Asian ideograph
        0x213d3f => [0x5f17, false], # East Asian ideograph
        0x4d2d75 => [0x8872, false], # East Asian ideograph (variant of 232D75 which maps to 8872)
        0x213370 => [0x525b, false], # East Asian ideograph
        0x69252c => [0x30ac, false], # Katakana letter GA
        0x225821 => [0x734b, false], # East Asian ideograph
        0x282d77 => [0x60ad, false], # East Asian ideograph
        0x215822 => [0x896f, false], # East Asian ideograph
        0x6f5a55 => [0xcff0, false], # Korean hangul
        0x6f557d => [0xc63b, false], # Korean hangul
        0x234c7b => [0x975f, false], # East Asian ideograph
        0x222d79 => [0x6164, false], # East Asian ideograph
        0x4b5824 => [0x897e, false], # East Asian ideograph
        0x696d7a => [0x9027, false], # East Asian ideograph
        0x6f4949 => [0xacd7, false], # Korean hangul
        0x215825 => [0x8981, false], # East Asian ideograph
        0x29425c => [0x94cc, false], # East Asian ideograph
        0x695c29 => [0x6925, false], # East Asian ideograph
        0x4b5826 => [0x8983, false], # East Asian ideograph (variant of 215826 which maps to 8983)
        0x232d7c => [0x8879, false], # East Asian ideograph
        0x295827 => [0x9cb2, false], # East Asian ideograph
        0x6f5a56 => [0xcff3, false], # Korean hangul
        0x295369 => [0x9a7a, false], # East Asian ideograph
        0x215828 => [0x898b, false], # East Asian ideograph
        0x225829 => [0x736c, false], # East Asian ideograph
        0x6f494a => [0xace0, false], # Korean hangul
        0x21582a => [0x8993, false], # East Asian ideograph
        0x21582b => [0x8996, false], # East Asian ideograph
        0x2d5259 => [0x98dc, false], # East Asian ideograph
        0x21582c => [0x89aa, false], # East Asian ideograph
        0x284f6b => [0x6f13, false], # East Asian ideograph
        0x29536a => [0x9a9d, false], # East Asian ideograph
        0x21582d => [0x89a6, false], # East Asian ideograph
        0x27582e => [0x89ca, false], # East Asian ideograph
        0x6f494b => [0xace1, false], # Korean hangul
        0x22582f => [0x736f, false], # East Asian ideograph
        0x275830 => [0x89c9, false], # East Asian ideograph
        0x215831 => [0x89bd, false], # East Asian ideograph
        0x6f5a58 => [0xcffc, false], # Korean hangul
        0x275832 => [0x89c2, false], # East Asian ideograph
        0x222871 => [0x5ee8, false], # East Asian ideograph
        0x22443c => [0x6b43, false], # East Asian ideograph
        0x33483b => [0x6cdd, false], # East Asian ideograph
        0x2d5833 => [0x752a, false], # East Asian ideograph
        0x276037 => [0x9875, false], # East Asian ideograph
        0x6f494c => [0xace4, false], # Korean hangul
        0x215834 => [0x89e3, false], # East Asian ideograph
        0x29425f => [0x94b6, false], # East Asian ideograph
        0x275835 => [0x89de, false], # East Asian ideograph
        0x274933 => [0x6e0e, false], # East Asian ideograph
        0x235b52 => [0x9d6f, false], # East Asian ideograph
        0x215836 => [0x89f8, false], # East Asian ideograph
        0x455837 => [0x8ba0, false], # East Asian ideograph
        0x4c5f69 => [0x75eb, false], # East Asian ideograph
        0x275838 => [0x8ba1, false], # East Asian ideograph
        0x6f4b36 => [0xb01c, false], # Korean hangul
        0x6f494d => [0xace7, false], # Korean hangul
        0x275839 => [0x8ba2, false], # East Asian ideograph
        0x4d2962 => [0x86c9, false], # East Asian ideograph (variant of 232962 which maps to 86C9)
        0x223331 => [0x640c, false], # East Asian ideograph
        0x22583b => [0x7381, false], # East Asian ideograph
        0x6f5a5a => [0xd018, false], # Korean hangul
        0x2e7431 => [0x7f48, false], # East Asian ideograph
        0x4b625c => [0x9eb8, false], # East Asian ideograph (variant of 27625C which maps to 9EB8)
        0x27583c => [0x8bb0, false], # East Asian ideograph
        0x22443e => [0x6b48, false], # East Asian ideograph
        0x23483d => [0x9467, false], # East Asian ideograph
        0x27583d => [0x8ba8, false], # East Asian ideograph
        0x273a63 => [0x5b6a, false], # East Asian ideograph
        0x6f494e => [0xace8, false], # Korean hangul
        0x27583e => [0x8ba7, false], # East Asian ideograph
        0x294261 => [0x94b2, false], # East Asian ideograph
        0x22583f => [0x7388, false], # East Asian ideograph
        0x2d525d => [0x6537, false], # East Asian ideograph
        0x224c3f => [0x6f26, false], # East Asian ideograph
        0x6f5a5b => [0xd02d, false], # Korean hangul
        0x215841 => [0x8a16, false], # East Asian ideograph
        0x215842 => [0x8a17, false], # East Asian ideograph
        0x6f494f => [0xacea, false], # Korean hangul
        0x275843 => [0x8bad, false], # East Asian ideograph
        0x275844 => [0x8bbf, false], # East Asian ideograph
        0x233732 => [0x8d0c, false], # East Asian ideograph
        0x2d3045 => [0x4e57, false], # East Asian ideograph
        0x275845 => [0x8bc0, false], # East Asian ideograph
        0x6f5a5c => [0xd034, false], # Korean hangul
        0x225846 => [0x7395, false], # East Asian ideograph
        0x294f2c => [0x988f, false], # East Asian ideograph
        0x215847 => [0x8a25, false], # East Asian ideograph
        0x6f4950 => [0xacec, false], # Korean hangul
        0x225848 => [0x7397, false], # East Asian ideograph
        0x6f5739 => [0xc810, false], # Korean hangul
        0x285c3a => [0x748e, false], # East Asian ideograph
        0x6f5054 => [0xbb54, false], # Korean hangul
        0x215849 => [0x8a2d, false], # East Asian ideograph
        0x21584a => [0x8a1b, false], # East Asian ideograph
        0x6f5a5d => [0xd035, false], # Korean hangul
        0x295370 => [0x9a9c, false], # East Asian ideograph
        0x286d47 => [0x7ba6, false], # East Asian ideograph
        0x21584b => [0x8a1f, false], # East Asian ideograph
        0x21584c => [0x8a3b, false], # East Asian ideograph
        0x2d4153 => [0x64e3, false], # East Asian ideograph
        0x276038 => [0x9876, false], # East Asian ideograph
        0x6f4951 => [0xacef, false], # Korean hangul
        0x22584d => [0x7394, false], # East Asian ideograph
        0x294264 => [0x94ba, false], # East Asian ideograph
        0x21584e => [0x8a55, false], # East Asian ideograph
        0x21584f => [0x8a5e, false], # East Asian ideograph
        0x4b576c => [0x523e, false], # East Asian ideograph
        0x27486d => [0x6da6, false], # East Asian ideograph
        0x275851 => [0x8bc2, false], # East Asian ideograph
        0x6f4b37 => [0xb01d, false], # Korean hangul
        0x6f4952 => [0xacf0, false], # Korean hangul
        0x225852 => [0x73a6, false], # East Asian ideograph
        0x227768 => [0x80b8, false], # East Asian ideograph
        0x223336 => [0x6415, false], # East Asian ideograph
        0x275854 => [0x8bc8, false], # East Asian ideograph
        0x6f5a5f => [0xd050, false], # Korean hangul
        0x275855 => [0x8bcb, false], # East Asian ideograph
        0x275856 => [0x8bc9, false], # East Asian ideograph
        0x6f4a5d => [0xaec4, false], # Korean hangul
        0x215857 => [0x8a3a, false], # East Asian ideograph
        0x21736a => [0x56d4, false], # East Asian ideograph
        0x33333c => [0x6c37, false], # East Asian ideograph
        0x215858 => [0x8a6b, false], # East Asian ideograph
        0x4b4621 => [0x6b53, false], # East Asian ideograph
        0x235859 => [0x9c12, false], # East Asian ideograph
        0x6f5a60 => [0xd06c, false], # Korean hangul
        0x27585a => [0x8be6, false], # East Asian ideograph
        0x27585b => [0x8bd5, false], # East Asian ideograph
        0x45456d => [0x6a10, false], # East Asian ideograph
        0x2d5f2c => [0x5826, false], # East Asian ideograph
        0x6f4954 => [0xacf3, false], # Korean hangul
        0x21585c => [0x8a69, false], # East Asian ideograph
        0x6f4b4a => [0xb08f, false], # Korean hangul
        0x225551 => [0x724f, false], # East Asian ideograph
        0x21585d => [0x8a70, false], # East Asian ideograph
        0x29443e => [0x94e4, false], # East Asian ideograph
        0x21585e => [0x8a63, false], # East Asian ideograph
        0x6f5a61 => [0xd070, false], # Korean hangul
        0x21625f => [0x9ebb, false], # East Asian ideograph
        0x21585f => [0x8a7c, false], # East Asian ideograph
        0x215860 => [0x8aa0, false], # East Asian ideograph
        0x6f4e30 => [0xb5a0, false], # Korean hangul
        0x2d5f2d => [0x964f, false], # East Asian ideograph
        0x275861 => [0x5938, false], # East Asian ideograph
        0x275840 => [0x8baf, false], # East Asian ideograph
        0x6f5055 => [0xbb58, false], # Korean hangul
        0x215862 => [0x8a85, false], # East Asian ideograph
        0x6f5441 => [0xc329, false], # Korean hangul
        0x225863 => [0x73a0, false], # East Asian ideograph
        0x6f5a62 => [0xd074, false], # Korean hangul
        0x695375 => [0x56ce, false], # East Asian ideograph
        0x6f5864 => [0xcb18, false], # Korean hangul
        0x215865 => [0x8a62, false], # East Asian ideograph
        0x2d575b => [0x886e, false], # East Asian ideograph
        0x3f4956 => [0x7832, false], # East Asian ideograph
        0x215866 => [0x8a71, false], # East Asian ideograph
        0x285c40 => [0x74d2, false], # East Asian ideograph
        0x215867 => [0x8a6e, false], # East Asian ideograph
        0x295132 => [0x997d, false], # East Asian ideograph
        0x233739 => [0x8d11, false], # East Asian ideograph
        0x2d5265 => [0x79d0, false], # East Asian ideograph
        0x225868 => [0x73cf, false], # East Asian ideograph
        0x6f5a63 => [0xd07c, false], # Korean hangul
        0x282d5e => [0x607a, false], # East Asian ideograph
        0x6f4a25 => [0xadd0, false], # Korean hangul
        0x275869 => [0x8bf4, false], # East Asian ideograph
        0x21586a => [0x8aa6, false], # East Asian ideograph
        0x6f4b38 => [0xb028, false], # Korean hangul
        0x21586b => [0x8aa1, false], # East Asian ideograph
        0x215155 => [0x7dd6, false], # East Asian ideograph
        0x22333b => [0x6422, false], # East Asian ideograph
        0x27586d => [0x5fd7, false], # East Asian ideograph
        0x22456f => [0x6be7, false], # East Asian ideograph
        0x23586e => [0x9c23, false], # East Asian ideograph
        0x27586f => [0x8bec, false], # East Asian ideograph
        0x6f4f36 => [0xb839, false], # Korean hangul
        0x6f4a5e => [0xaecc, false], # Korean hangul
        0x215870 => [0x8a8d, false], # East Asian ideograph
        0x21736f => [0x56e1, false], # East Asian ideograph
        0x294469 => [0x94fc, false], # East Asian ideograph
        0x215871 => [0x8aa4, false], # East Asian ideograph (variant of 4B5871 which maps to 8AA4)
        0x23373b => [0x8d12, false], # East Asian ideograph
        0x2d5267 => [0x79cf, false], # East Asian ideograph
        0x215872 => [0x8aa8, false], # East Asian ideograph
        0x2d4e24 => [0x6998, false], # East Asian ideograph
        0x215873 => [0x8aa5, false], # East Asian ideograph
        0x217e60 => [0x5bc9, false], # East Asian ideograph
        0x215874 => [0x8a98, false], # East Asian ideograph
        0x4b5d65 => [0x8217, false], # East Asian ideograph
        0x6f4959 => [0xacfd, false], # Korean hangul
        0x215875 => [0x8a91, false], # East Asian ideograph
        0x6f5130 => [0xbc9a, false], # Korean hangul
        0x275876 => [0x8c0a, false], # East Asian ideograph
        0x6f4b68 => [0xb0c9, false], # Korean hangul
        0x215877 => [0x8ac4, false], # East Asian ideograph
        0x6f5a66 => [0xd0a4, false], # Korean hangul
        0x4b5176 => [0x7e04, false], # East Asian ideograph
        0x295379 => [0x9a96, false], # East Asian ideograph
        0x235878 => [0x9c21, false], # East Asian ideograph
        0x234323 => [0x924e, false], # East Asian ideograph
        0x275879 => [0x8c08, false], # East Asian ideograph
        0x6f495a => [0xad00, false], # Korean hangul
        0x27587a => [0x8bf7, false], # East Asian ideograph
        0x224325 => [0x6acc, false], # East Asian ideograph
        0x21587b => [0x8af8, false], # East Asian ideograph
        0x23373d => [0x8d14, false], # East Asian ideograph
        0x21587c => [0x8ab2, false], # East Asian ideograph
        0x234327 => [0x9256, false], # East Asian ideograph
        0x29537a => [0x9aa2, false], # East Asian ideograph
        0x21587d => [0x8abf, false], # East Asian ideograph
        0x224328 => [0x6ad1, false], # East Asian ideograph
        0x21587e => [0x8ac9, false], # East Asian ideograph
        0x4c7d4d => [0x8343, false], # East Asian ideograph
        0x2d4329 => [0x668e, false], # East Asian ideograph
        0x6f495b => [0xad04, false], # Korean hangul
        0x6f5d31 => [0xd611, false], # Korean hangul
        0x6f4f7c => [0xb9f9, false], # Korean hangul
        0x23432b => [0x925a, false], # East Asian ideograph
        0x2d3051 => [0x5f0d, false], # East Asian ideograph
        0x6f5a68 => [0xd0a8, false], # Korean hangul
        0x216266 => [0x9ed1, false], # East Asian ideograph
        0x27432d => [0x65f6, false], # East Asian ideograph
        0x4b5736 => [0x877f, false], # East Asian ideograph
        0x23432e => [0x9241, false], # East Asian ideograph
        0x6f495c => [0xad0c, false], # Korean hangul
        0x285252 => [0x709c, false], # East Asian ideograph
        0x275847 => [0x8bb7, false], # East Asian ideograph
        0x23432f => [0x9283, false], # East Asian ideograph
        0x335958 => [0x8c4a, false], # East Asian ideograph
        0x394330 => [0x6644, false], # East Asian ideograph
        0x2d526b => [0x7085, false], # East Asian ideograph
        0x234331 => [0x92a5, false], # East Asian ideograph
        0x213065 => [0x4eca, false], # East Asian ideograph
        0x6f5626 => [0xc653, false], # Korean hangul
        0x274332 => [0x663c, false], # East Asian ideograph
        0x234333 => [0x9282, false], # East Asian ideograph
        0x2d5f35 => [0x78d2, false], # East Asian ideograph
        0x6f495d => [0xad0d, false], # Korean hangul
        0x275848 => [0x8bb8, false], # East Asian ideograph
        0x224334 => [0x6acd, false], # East Asian ideograph
        0x234335 => [0x92a8, false], # East Asian ideograph
        0x2d3053 => [0x4e3c, false], # East Asian ideograph
        0x29572b => [0x9c90, false], # East Asian ideograph
        0x6f5a6a => [0xd0b4, false], # Korean hangul
        0x224337 => [0x6aec, false], # East Asian ideograph
        0x215e25 => [0x938a, false], # East Asian ideograph
        0x234338 => [0x92a4, false], # East Asian ideograph
        0x6f495e => [0xad0f, false], # Korean hangul
        0x224339 => [0x6af3, false], # East Asian ideograph
        0x22433a => [0x6ae7, false], # East Asian ideograph
        0x2d433b => [0x6662, false], # East Asian ideograph
        0x226225 => [0x7735, false], # East Asian ideograph
        0x4b5b29 => [0x8e8d, false], # East Asian ideograph
        0x6f495f => [0xad11, false], # Korean hangul
        0x23433e => [0x9276, false], # East Asian ideograph
        0x227775 => [0x6711, false], # East Asian ideograph
        0x22433f => [0x6aeb, false], # East Asian ideograph
        0x224340 => [0x6aea, false], # East Asian ideograph
        0x21626a => [0x9ede, false], # East Asian ideograph
        0x274341 => [0x6655, false], # East Asian ideograph
        0x6f5428 => [0xc2eb, false], # Korean hangul
        0x234342 => [0x9288, false], # East Asian ideograph
        0x27603b => [0x987a, false], # East Asian ideograph
        0x6f4960 => [0xad18, false], # Korean hangul
        0x274343 => [0x7545, false], # East Asian ideograph
        0x6f4f7d => [0xb9fa, false], # Korean hangul
        0x223344 => [0x6430, false], # East Asian ideograph
        0x224344 => [0x6af1, false], # East Asian ideograph
        0x4c6c46 => [0x7b9f, false], # East Asian ideograph
        0x234345 => [0x928e, false], # East Asian ideograph
        0x6f562a => [0xc65d, false], # Korean hangul
        0x234346 => [0x92a0, false], # East Asian ideograph
        0x4b7954 => [0x5968, false], # East Asian ideograph
        0x6f4b3a => [0xb045, false], # Korean hangul
        0x234347 => [0x9277, false], # East Asian ideograph
        0x6f4961 => [0xad19, false], # Korean hangul
        0x274348 => [0x6653, false], # East Asian ideograph
        0x274349 => [0x5386, false], # East Asian ideograph (duplicate simplified)
        0x6f564f => [0xc6e9, false], # Korean hangul
        0x224571 => [0x6be8, false], # East Asian ideograph
        0x213066 => [0x4ec1, false], # East Asian ideograph
        0x6f562b => [0xc660, false], # Korean hangul
        0x27434b => [0x66a7, false], # East Asian ideograph
        0x6f4962 => [0xad1c, false], # Korean hangul
        0x27434d => [0x65f7, false], # East Asian ideograph
        0x6f4a60 => [0xaecf, false], # Korean hangul
        0x4b335b => [0x522b, false], # East Asian ideograph
        0x23595e => [0x9c6e, false], # East Asian ideograph
        0x22434e => [0x6afd, false], # East Asian ideograph
        0x2d3058 => [0x4e9c, false], # East Asian ideograph
        0x29434f => [0x94de, false], # East Asian ideograph
        0x6f562c => [0xc671, false], # Korean hangul
        0x224350 => [0x6afa, false], # East Asian ideograph
        0x347d24 => [0x83c7, false], # East Asian ideograph
        0x6f552e => [0xc561, false], # Korean hangul
        0x2d5f3b => [0x96a0, false], # East Asian ideograph
        0x6f4963 => [0xad20, false], # Korean hangul
        0x6f5132 => [0xbca1, false], # Korean hangul
        0x224352 => [0x6b01, false], # East Asian ideograph
        0x4b4a74 => [0x731c, false], # East Asian ideograph (variant of 214A74 which maps to 731C)
        0x234354 => [0x927e, false], # East Asian ideograph
        0x6f5c47 => [0xd45c, false], # Korean hangul
        0x274355 => [0x4e66, false], # East Asian ideograph
        0x6f4964 => [0xad28, false], # Korean hangul
        0x334357 => [0x6702, false], # East Asian ideograph
        0x223348 => [0x6435, false], # East Asian ideograph
        0x224358 => [0x6b03, false], # East Asian ideograph
        0x224359 => [0x6af8, false], # East Asian ideograph
        0x21605f => [0x98b6, false], # East Asian ideograph
        0x4d6047 => [0x816d, false], # East Asian ideograph
        0x27435a => [0x4f1a, false], # East Asian ideograph
        0x23435b => [0x9291, false], # East Asian ideograph
        0x27603c => [0x987b, false], # East Asian ideograph
        0x6f4965 => [0xad29, false], # Korean hangul
        0x6f4f7e => [0xba00, false], # Korean hangul
        0x23435d => [0x929b, false], # East Asian ideograph
        0x233748 => [0x8d6c, false], # East Asian ideograph
        0x2d305b => [0x4ebe, false], # East Asian ideograph
        0x217573 => [0x57d5, false], # East Asian ideograph
        0x6f562f => [0xc67c, false], # Korean hangul
        0x284b43 => [0x8365, false], # East Asian ideograph
        0x22435f => [0x6b0d, false], # East Asian ideograph
        0x6f4b3b => [0xb048, false], # Korean hangul
        0x224360 => [0x6b09, false], # East Asian ideograph
        0x355053 => [0x98c8, false], # East Asian ideograph
        0x6f4966 => [0xad2d, false], # Korean hangul
        0x224361 => [0x6b0e, false], # East Asian ideograph
        0x225563 => [0x726e, false], # East Asian ideograph
        0x234362 => [0x927f, false], # East Asian ideograph
        0x69243c => [0x305c, false], # Hiragana letter ZE
        0x6f5630 => [0xc680, false], # Korean hangul
        0x234364 => [0x92a3, false], # East Asian ideograph
        0x6f4967 => [0xad34, false], # Korean hangul
        0x275852 => [0x8bcf, false], # East Asian ideograph
        0x214366 => [0x6727, false], # East Asian ideograph
        0x224367 => [0x6b11, false], # East Asian ideograph
        0x2d4e33 => [0x78aa, false], # East Asian ideograph
        0x3f5631 => [0x517f, false], # East Asian ideograph
        0x334369 => [0x5932, false], # East Asian ideograph
        0x234857 => [0x9464, false], # East Asian ideograph
        0x21436a => [0x672b, false], # East Asian ideograph
        0x293032 => [0x7962, false], # East Asian ideograph
        0x6f4968 => [0xad38, false], # Korean hangul
        0x275853 => [0x8bc5, false], # East Asian ideograph
        0x33496a => [0x934a, false], # East Asian ideograph
        0x22436d => [0x6b19, false], # East Asian ideograph
        0x4b5179 => [0x7f0b, false], # East Asian ideograph
        0x6f5632 => [0xc68b, false], # Korean hangul
        0x23436f => [0x92d0, false], # East Asian ideograph
        0x6f4969 => [0xad3c, false], # Korean hangul
        0x2d4370 => [0x6736, false], # East Asian ideograph
        0x215167 => [0x7e46, false], # East Asian ideograph
        0x234371 => [0x92f1, false], # East Asian ideograph
        0x234372 => [0x92df, false], # East Asian ideograph
        0x275528 => [0x835a, false], # East Asian ideograph
        0x215e31 => [0x93c8, false], # East Asian ideograph
        0x6f496a => [0xad44, false], # Korean hangul
        0x214375 => [0x6750, false], # East Asian ideograph
        0x215168 => [0x7e37, false], # East Asian ideograph
        0x6f572b => [0xc7bc, false], # Korean hangul
        0x234376 => [0x92b6, false], # East Asian ideograph
        0x4b4638 => [0x6bb1, false], # East Asian ideograph
        0x234377 => [0x92c0, false], # East Asian ideograph
        0x6f5634 => [0xc694, false], # Korean hangul
        0x6f4d2b => [0xb35b, false], # Korean hangul
        0x6f4b3c => [0xb04a, false], # Korean hangul
        0x234379 => [0x92be, false], # East Asian ideograph
        0x6f572e => [0xc7c0, false], # Korean hangul
        0x2d3061 => [0x4eb0, false], # East Asian ideograph
        0x6f5a78 => [0xd0d4, false], # Korean hangul
        0x6f5635 => [0xc695, false], # Korean hangul
        0x29437d => [0x94d8, false], # East Asian ideograph
        0x696e28 => [0x9056, false], # East Asian ideograph
        0x4b5746 => [0x8853, false], # East Asian ideograph
        0x23437e => [0x92d5, false], # East Asian ideograph
        0x2d3d2b => [0x5ebf, false], # East Asian ideograph
        0x6f4a62 => [0xaed1, false], # Korean hangul
        0x276e2a => [0x53a3, false], # East Asian ideograph
        0x2d527b => [0x8074, false], # East Asian ideograph
        0x6f5a79 => [0xd0d5, false], # Korean hangul
        0x282d74 => [0x6004, false], # East Asian ideograph
        0x6f5636 => [0xc698, false], # Korean hangul
        0x23485c => [0x9465, false], # East Asian ideograph
        0x226233 => [0x774a, false], # East Asian ideograph
        0x6f496d => [0xad50, false], # Korean hangul
        0x6f5c49 => [0xd478, false], # Korean hangul
        0x6f5840 => [0xc9ed, false], # Korean hangul
        0x222e2f => [0x615c, false], # East Asian ideograph
        0x223351 => [0x640a, false], # East Asian ideograph
        0x21317d => [0x501a, false], # East Asian ideograph
        0x6f5a7a => [0xd0dc, false], # Korean hangul
        0x2e7451 => [0x7f58, false], # East Asian ideograph
        0x6f5637 => [0xc6a5, false], # Korean hangul
        0x215e35 => [0x93dd, false], # East Asian ideograph
        0x23485d => [0x9455, false], # East Asian ideograph
        0x6f4e31 => [0xb5a1, false], # Korean hangul
        0x225e2c => [0x7590, false], # East Asian ideograph
        0x2d3d2d => [0x5396, false], # East Asian ideograph
        0x275859 => [0x8be5, false], # East Asian ideograph
        0x21516c => [0x7e2b, false], # East Asian ideograph
        0x225128 => [0x70e0, false], # East Asian ideograph
        0x6f5a7b => [0xd0dd, false], # Korean hangul
        0x275529 => [0x830e, false], # East Asian ideograph
        0x22445f => [0x6b6e, false], # East Asian ideograph
        0x33485e => [0x67d2, false], # East Asian ideograph
        0x226235 => [0x7743, false], # East Asian ideograph
        0x2d4171 => [0x62ca, false], # East Asian ideograph
        0x6f496f => [0xad6d, false], # Korean hangul
        0x6f572c => [0xc7bd, false], # Korean hangul
        0x27493a => [0x6fd1, false], # East Asian ideograph
        0x23596b => [0x9c7a, false], # East Asian ideograph
        0x235b59 => [0x9da9, false], # East Asian ideograph
        0x27474e => [0x6cfe, false], # East Asian ideograph
        0x6f5639 => [0xc6a9, false], # Korean hangul
        0x215e37 => [0x93d8, false], # East Asian ideograph
        0x222e3d => [0x61a2, false], # East Asian ideograph
        0x2d3d2f => [0x539b, false], # East Asian ideograph
        0x6f5946 => [0xccd0, false], # Korean hangul
        0x345d6b => [0x756d, false], # East Asian ideograph
        0x285d6b => [0x7572, false], # East Asian ideograph
        0x222e40 => [0x61a8, false], # East Asian ideograph
        0x6f563a => [0xc6b0, false], # Korean hangul
        0x4b3c2b => [0x67c3, false], # East Asian ideograph (Version J extension)
        0x6f4f37 => [0xb840, false], # Korean hangul
        0x6f4971 => [0xad73, false], # Korean hangul
        0x6f4a63 => [0xaed8, false], # Korean hangul
        0x22512b => [0x70d4, false], # East Asian ideograph
        0x23552a => [0x9aef, false], # East Asian ideograph
        0x6f5a7e => [0xd0ec, false], # Korean hangul
        0x222e45 => [0x6196, false], # East Asian ideograph
        0x6f563b => [0xc6b1, false], # Korean hangul
        0x6f4972 => [0xad74, false], # Korean hangul
        0x6f563c => [0xc6b4, false], # Korean hangul
        0x234862 => [0x946a, false], # East Asian ideograph
        0x282e4c => [0x6126, false], # East Asian ideograph
        0x2d5f4b => [0x96d1, false], # East Asian ideograph
        0x6f4973 => [0xad75, false], # Korean hangul
        0x215171 => [0x7e54, false], # East Asian ideograph
        0x6f563d => [0xc6b7, false], # Korean hangul
        0x215e3b => [0x93fd, false], # East Asian ideograph
        0x2d4176 => [0x6483, false], # East Asian ideograph
        0x2d5f4c => [0x9dc4, false], # East Asian ideograph
        0x6f4974 => [0xad76, false], # Korean hangul
        0x6f5d32 => [0xd613, false], # Korean hangul
        0x282e52 => [0x6003, false], # East Asian ideograph
        0x27493b => [0x6ca5, false], # East Asian ideograph
        0x233941 => [0x8dfd, false], # East Asian ideograph
        0x6f563e => [0xc6b8, false], # Korean hangul
        0x4b5773 => [0x7ed4, false], # East Asian ideograph
        0x213c23 => [0x5d22, false], # East Asian ideograph
        0x286e56 => [0x7ba8, false], # East Asian ideograph
        0x2d3d34 => [0x5efe, false], # East Asian ideograph
        0x215173 => [0x7e5e, false], # East Asian ideograph
        0x223359 => [0x6407, false], # East Asian ideograph
        0x216e58 => [0x535f, false], # East Asian ideograph
        0x707523 => [0x9170, false], # East Asian ideograph
        0x6f4b77 => [0xb119, false], # Korean hangul
        0x222e5a => [0x61cb, false], # East Asian ideograph
        0x213c24 => [0x5d29, false], # East Asian ideograph
        0x33355c => [0x5449, false], # East Asian ideograph
        0x273648 => [0x95ee, false], # East Asian ideograph
        0x282e5c => [0x603f, false], # East Asian ideograph
        0x27514c => [0x7eff, false], # East Asian ideograph
        0x6f5579 => [0xc635, false], # Korean hangul
        0x6f5640 => [0xc6ba, false], # Korean hangul
        0x6f5951 => [0xcd5c, false], # Korean hangul
        0x4b397b => [0x5a2f, false], # East Asian ideograph
        0x29402c => [0x90d0, false], # East Asian ideograph
        0x22623d => [0x7760, false], # East Asian ideograph
        0x6f4977 => [0xad7f, false], # Korean hangul
        0x6f5136 => [0xbcb0, false], # Korean hangul
        0x216e61 => [0x5414, false], # East Asian ideograph
        0x22335b => [0x643b, false], # East Asian ideograph
        0x6f4a2e => [0xadfc, false], # Korean hangul
        0x6f5641 => [0xc6c0, false], # Korean hangul
        0x213c26 => [0x5d19, false], # East Asian ideograph
        0x275863 => [0x8be1, false], # East Asian ideograph
        0x695a7e => [0x66bc, false], # East Asian ideograph
        0x287e61 => [0x82cc, false], # East Asian ideograph
        0x232e68 => [0x88d2, false], # East Asian ideograph
        0x6f5642 => [0xc6c1, false], # Korean hangul
        0x234868 => [0x946b, false], # East Asian ideograph
        0x6f5429 => [0xc2ec, false], # Korean hangul
        0x334425 => [0x76c3, false], # East Asian ideograph
        0x6f4979 => [0xad82, false], # Korean hangul
        0x296062 => [0x9f9b, false], # East Asian ideograph
        0x22335d => [0x643f, false], # East Asian ideograph
        0x23375c => [0x8d7a, false], # East Asian ideograph
        0x6f5643 => [0xc6c3, false], # Korean hangul
        0x213c28 => [0x5d50, false], # East Asian ideograph
        0x216e6f => [0x541a, false], # East Asian ideograph
        0x6f497a => [0xad88, false], # Korean hangul
        0x275422 => [0x810f, false], # East Asian ideograph (duplicate simplified)
        0x222e71 => [0x61e0, false], # East Asian ideograph
        0x2d3e40 => [0x6052, false], # East Asian ideograph
        0x6f5644 => [0xc6c5, false], # Korean hangul
        0x28702e => [0x56e2, false], # East Asian ideograph (duplicate simplified)
        0x6f497b => [0xad8c, false], # Korean hangul
        0x6f4a65 => [0xaef4, false], # Korean hangul
        0x455164 => [0x53bf, false], # East Asian ideograph
        0x215921 => [0x8ac2, false], # East Asian ideograph
        0x222e77 => [0x61e5, false], # East Asian ideograph
        0x275922 => [0x8c01, false], # East Asian ideograph
        0x275923 => [0x8bde, false], # East Asian ideograph
        0x232652 => [0x85a2, false], # East Asian ideograph
        0x6f552f => [0xc564, false], # Korean hangul
        0x216e79 => [0x5454, false], # East Asian ideograph
        0x275924 => [0x8bba, false], # East Asian ideograph
        0x235925 => [0x9c32, false], # East Asian ideograph
        0x215926 => [0x8afa, false], # East Asian ideograph
        0x275927 => [0x8c0f, false], # East Asian ideograph
        0x216e7d => [0x543d, false], # East Asian ideograph
        0x235928 => [0x9c48, false], # East Asian ideograph
        0x282e7e => [0x603c, false], # East Asian ideograph
        0x215929 => [0x8ae7, false], # East Asian ideograph
        0x275868 => [0x8bdf, false], # East Asian ideograph
        0x6f505d => [0xbbc8, false], # Korean hangul
        0x23592a => [0x9c33, false], # East Asian ideograph
        0x21592b => [0x8b00, false], # East Asian ideograph
        0x6f5b72 => [0xd31d, false], # Korean hangul
        0x27592c => [0x8c12, false], # East Asian ideograph
        0x6f5647 => [0xc6d0, false], # Korean hangul
        0x29416b => [0x948e, false], # East Asian ideograph
        0x213e47 => [0x6064, false], # East Asian ideograph
        0x23486d => [0x9471, false], # East Asian ideograph
        0x27592e => [0x8bfa, false], # East Asian ideograph
        0x22592f => [0x73d4, false], # East Asian ideograph
        0x27493d => [0x6f47, false], # East Asian ideograph
        0x233761 => [0x8d84, false], # East Asian ideograph
        0x215930 => [0x8aed, false], # East Asian ideograph
        0x335a7b => [0x8e28, false], # East Asian ideograph
        0x275931 => [0x8c24, false], # East Asian ideograph
        0x6f5648 => [0xc6d4, false], # Korean hangul
        0x697174 => [0x9ade, false], # East Asian ideograph
        0x235932 => [0x9c35, false], # East Asian ideograph
        0x275933 => [0x8c1c, false], # East Asian ideograph
        0x27586a => [0x8bf5, false], # East Asian ideograph
        0x215934 => [0x8b1b, false], # East Asian ideograph
        0x215935 => [0x8b0a, false], # East Asian ideograph
        0x225936 => [0x73e7, false], # East Asian ideograph
        0x6f5649 => [0xc6dc, false], # Korean hangul
        0x275937 => [0x8a8a, false], # East Asian ideograph
        0x215938 => [0x8b1d, false], # East Asian ideograph
        0x27586b => [0x8beb, false], # East Asian ideograph
        0x6f4a66 => [0xaf0d, false], # Korean hangul
        0x282f66 => [0x6217, false], # East Asian ideograph
        0x215939 => [0x8b39, false], # East Asian ideograph
        0x22513a => [0x70d0, false], # East Asian ideograph
        0x21593a => [0x8b2c, false], # East Asian ideograph
        0x21593b => [0x8b28, false], # East Asian ideograph
        0x6f564a => [0xc6dd, false], # Korean hangul
        0x224471 => [0x6b82, false], # East Asian ideograph
        0x21593c => [0x8b58, false], # East Asian ideograph
        0x27593d => [0x8c31, false], # East Asian ideograph
        0x6f5138 => [0xbcb3, false], # Korean hangul
        0x27586c => [0x8bed, false], # East Asian ideograph
        0x27593e => [0x8c32, false], # East Asian ideograph
        0x22513b => [0x70c7, false], # East Asian ideograph
        0x22593f => [0x73e9, false], # East Asian ideograph
        0x274e5b => [0x77ff, false], # East Asian ideograph
        0x215940 => [0x8b5a, false], # East Asian ideograph
        0x6f564b => [0xc6df, false], # Korean hangul
        0x395577 => [0x854b, false], # East Asian ideograph
        0x213c30 => [0x5dcd, false], # East Asian ideograph
        0x215942 => [0x8b4f, false], # East Asian ideograph
        0x6f573b => [0xc813, false], # Korean hangul
        0x275943 => [0x8bae, false], # East Asian ideograph
        0x22472c => [0x6c54, false], # East Asian ideograph
        0x22513c => [0x70da, false], # East Asian ideograph
        0x6f5944 => [0xccbc, false], # Korean hangul
        0x2d435f => [0x6716, false], # East Asian ideograph
        0x6f5b73 => [0xd31f, false], # Korean hangul
        0x235945 => [0x9c51, false], # East Asian ideograph
        0x6f564c => [0xc6e0, false], # Korean hangul
        0x69255a => [0x30da, false], # Katakana letter PE
        0x213c31 => [0x5dd2, false], # East Asian ideograph
        0x215947 => [0x8b74, false], # East Asian ideograph
        0x215948 => [0x8b77, false], # East Asian ideograph
        0x2e4c7b => [0x6e86, false], # East Asian ideograph
        0x22513d => [0x70c6, false], # East Asian ideograph
        0x235949 => [0x9c63, false], # East Asian ideograph
        0x333323 => [0x4e21, false], # East Asian ideograph
        0x22594a => [0x73f8, false], # East Asian ideograph
        0x6f564d => [0xc6e1, false], # Korean hangul
        0x6f5275 => [0xc0db, false], # Korean hangul
        0x27594b => [0x53d8, false], # East Asian ideograph
        0x21594c => [0x8b93, false], # East Asian ideograph
        0x27594d => [0x8c36, false], # East Asian ideograph
        0x28582b => [0x7303, false], # East Asian ideograph
        0x27594e => [0x8c17, false], # East Asian ideograph
        0x21594f => [0x8b9a, false], # East Asian ideograph
        0x6f564e => [0xc6e8, false], # Korean hangul
        0x4b3c2f => [0x5dba, false], # East Asian ideograph
        0x287030 => [0x7c9d, false], # East Asian ideograph
        0x213c33 => [0x5dd6, false], # East Asian ideograph
        0x6f5a2c => [0xcef9, false], # Korean hangul
        0x2d3253 => [0x50e3, false], # East Asian ideograph
        0x6f4a67 => [0xaf2c, false], # Korean hangul
        0x6f5952 => [0xcd78, false], # Korean hangul
        0x333768 => [0x8ff4, false], # East Asian ideograph
        0x21373f => [0x5659, false], # East Asian ideograph
        0x393439 => [0x61c3, false], # East Asian ideograph
        0x215954 => [0x8c48, false], # East Asian ideograph
        0x395652 => [0x87a1, false], # East Asian ideograph
        0x215955 => [0x8c49, false], # East Asian ideograph
        0x215956 => [0x8c4c, false], # East Asian ideograph
        0x396167 => [0x9b2a, false], # East Asian ideograph
        0x706b5b => [0x810e, false], # East Asian ideograph
        0x275957 => [0x7ad6, false], # East Asian ideograph
        0x215958 => [0x8c50, false], # East Asian ideograph
        0x474931 => [0x95f6, false], # East Asian ideograph
        0x2d5959 => [0x8277, false], # East Asian ideograph
        0x213665 => [0x558a, false], # East Asian ideograph
        0x22595a => [0x73fd, false], # East Asian ideograph
        0x6f4e32 => [0xb5a4, false], # Korean hangul
        0x217c24 => [0x5aa7, false], # East Asian ideograph
        0x2f5973 => [0x9cec, false], # East Asian ideograph
        0x6f595b => [0xcdb0, false], # Korean hangul
        0x21595c => [0x8c62, false], # East Asian ideograph
        0x4b4655 => [0x6c17, false], # East Asian ideograph
        0x6f595d => [0xcdcc, false], # Korean hangul
        0x455d3e => [0x9485, false], # East Asian ideograph
        0x6f5b74 => [0xd320, false], # Korean hangul
        0x21595e => [0x8c6b, false], # East Asian ideograph
        0x6f5651 => [0xc6f0, false], # Korean hangul
        0x69717d => [0x9af1, false], # East Asian ideograph
        0x2d595f => [0x732a, false], # East Asian ideograph
        0x2d5960 => [0x72b2, false], # East Asian ideograph
        0x6f5731 => [0xc7c9, false], # Korean hangul
        0x513a47 => [0x8885, false], # East Asian ideograph
        0x6f5962 => [0xce30, false], # Korean hangul
        0x39505b => [0x9b3b, false], # East Asian ideograph
        0x215963 => [0x8c8a, false], # East Asian ideograph
        0x6f5652 => [0xc6f8, false], # Korean hangul
        0x224479 => [0x6b8d, false], # East Asian ideograph
        0x4b5964 => [0x72e2, false], # East Asian ideograph
        0x234435 => [0x92dd, false], # East Asian ideograph
        0x2d5965 => [0x72f8, false], # East Asian ideograph
        0x2d3d48 => [0x5f4a, false], # East Asian ideograph
        0x275966 => [0x7683, false], # East Asian ideograph
        0x213f79 => [0x6249, false], # East Asian ideograph
        0x215967 => [0x8c93, false], # East Asian ideograph
        0x215968 => [0x8c9d, false], # East Asian ideograph
        0x295b77 => [0x9e63, false], # East Asian ideograph
        0x215969 => [0x8c9e, false], # East Asian ideograph
        0x217c27 => [0x5a9c, false], # East Asian ideograph
        0x6f5c2d => [0xd399, false], # Korean hangul
        0x27596a => [0x8d1f, false], # East Asian ideograph
        0x6f4a68 => [0xaf2d, false], # Korean hangul
        0x706b5f => [0x8112, false], # East Asian ideograph
        0x21596b => [0x8ca2, false], # East Asian ideograph
        0x52735d => [0x7e8a, false], # East Asian ideograph (variant of 22735D which maps to 7E8A)
        0x695c30 => [0x6923, false], # East Asian ideograph
        0x225144 => [0x7104, false], # East Asian ideograph
        0x22596c => [0x7430, false], # East Asian ideograph
        0x33332a => [0x4e93, false], # East Asian ideograph
        0x21596d => [0x8cac, false], # East Asian ideograph
        0x21596e => [0x8cab, false], # East Asian ideograph
        0x217c28 => [0x5a7c, false], # East Asian ideograph
        0x697260 => [0x9c30, false], # East Asian ideograph
        0x27596f => [0x8d27, false], # East Asian ideograph
        0x215970 => [0x8caa, false], # East Asian ideograph
        0x695c31 => [0x6921, false], # East Asian ideograph
        0x215971 => [0x8ca7, false], # East Asian ideograph
        0x6f5c4f => [0xd48b, false], # Korean hangul
        0x275e46 => [0x9576, false], # East Asian ideograph
        0x215972 => [0x8ca9, false], # East Asian ideograph
        0x215973 => [0x8caf, false], # East Asian ideograph
        0x217c29 => [0x5a96, false], # East Asian ideograph
        0x6f5974 => [0xce85, false], # Korean hangul
        0x275975 => [0x8d39, false], # East Asian ideograph
        0x6f5060 => [0xbbf9, false], # Korean hangul
        0x4b465a => [0x6c32, false], # East Asian ideograph
        0x275976 => [0x8d32, false], # East Asian ideograph
        0x234421 => [0x92c6, false], # East Asian ideograph
        0x215977 => [0x8cc0, false], # East Asian ideograph
        0x6f5656 => [0xc70c, false], # Korean hangul
        0x277748 => [0x57b2, false], # East Asian ideograph
        0x215978 => [0x8cb4, false], # East Asian ideograph
        0x275979 => [0x8d34, false], # East Asian ideograph
        0x295b2a => [0x9e46, false], # East Asian ideograph
        0x6f503e => [0xbab0, false], # Korean hangul
        0x21597a => [0x8cb7, false], # East Asian ideograph
        0x234425 => [0x92f4, false], # East Asian ideograph
        0x27597b => [0x8d2c, false], # East Asian ideograph
        0x274426 => [0x4e1c, false], # East Asian ideograph
        0x27597c => [0x8d3b, false], # East Asian ideograph
        0x6f5657 => [0xc714, false], # Korean hangul
        0x234427 => [0x92cf, false], # East Asian ideograph
        0x292d51 => [0x8511, false], # East Asian ideograph
        0x21597d => [0x8cb8, false], # East Asian ideograph
        0x23443a => [0x92ca, false], # East Asian ideograph
        0x27597e => [0x8d38, false], # East Asian ideograph
        0x23442a => [0x92b2, false], # East Asian ideograph
        0x225148 => [0x70f3, false], # East Asian ideograph
        0x29442b => [0x9503, false], # East Asian ideograph
        0x6f5324 => [0xc120, false], # Korean hangul
        0x6f5658 => [0xc717, false], # Korean hangul
        0x23442c => [0x92e7, false], # East Asian ideograph
        0x294f6b => [0x98a1, false], # East Asian ideograph
        0x696d41 => [0x8ec8, false], # East Asian ideograph
        0x23442d => [0x92c7, false], # East Asian ideograph
        0x277d40 => [0x5af1, false], # East Asian ideograph
        0x2d3d4e => [0x7bf2, false], # East Asian ideograph
        0x23442e => [0x92f0, false], # East Asian ideograph
        0x6f4a69 => [0xaf30, false], # Korean hangul
        0x23442f => [0x92db, false], # East Asian ideograph
        0x6f4e5e => [0xb768, false], # Korean hangul
        0x234430 => [0x92dc, false], # East Asian ideograph
        0x2d4e5b => [0x945b, false], # East Asian ideograph
        0x234431 => [0x92d8, false], # East Asian ideograph
        0x224432 => [0x6b39, false], # East Asian ideograph
        0x234433 => [0x92e9, false], # East Asian ideograph
        0x224435 => [0x6b3f, false], # East Asian ideograph
        0x274e5e => [0x783e, false], # East Asian ideograph
        0x6f565a => [0xc720, false], # Korean hangul
        0x27375a => [0x4e25, false], # East Asian ideograph
        0x224437 => [0x6b46, false], # East Asian ideograph
        0x2d3d50 => [0x5f5c, false], # East Asian ideograph
        0x224438 => [0x6b41, false], # East Asian ideograph
        0x234439 => [0x92d1, false], # East Asian ideograph
        0x6f5061 => [0xbbfc, false], # Korean hangul
        0x283561 => [0x64ba, false], # East Asian ideograph
        0x22443a => [0x6b40, false], # East Asian ideograph
        0x6f565b => [0xc721, false], # Korean hangul
        0x22443b => [0x6b42, false], # East Asian ideograph
        0x6f5973 => [0xce84, false], # Korean hangul
        0x6f4c7e => [0xb301, false], # Korean hangul
        0x23443c => [0x92c2, false], # East Asian ideograph
        0x6f4f29 => [0xb810, false], # Korean hangul
        0x454c3c => [0x7589, false], # East Asian ideograph
        0x6f5733 => [0xc7d8, false], # Korean hangul
        0x23443e => [0x92cc, false], # East Asian ideograph
        0x22443f => [0x6b4a, false], # East Asian ideograph
        0x235b60 => [0x9d98, false], # East Asian ideograph
        0x2e525d => [0x715b, false], # East Asian ideograph
        0x6f565c => [0xc724, false], # Korean hangul
        0x234440 => [0x92ef, false], # East Asian ideograph
        0x213c41 => [0x5df7, false], # East Asian ideograph
        0x234441 => [0x92e8, false], # East Asian ideograph
        0x6f5d35 => [0xd61c, false], # Korean hangul
        0x287739 => [0x8069, false], # East Asian ideograph
        0x27587e => [0x8bff, false], # East Asian ideograph
        0x234443 => [0x92eb, false], # East Asian ideograph
        0x695c39 => [0x697e, false], # East Asian ideograph
        0x295d36 => [0x9e2c, false], # East Asian ideograph
        0x2d4444 => [0x69c5, false], # East Asian ideograph
        0x6f565d => [0xc728, false], # Korean hangul
        0x234445 => [0x92f5, false], # East Asian ideograph
        0x224446 => [0x6b4e, false], # East Asian ideograph (variant of 4C4446 which maps to 6B4E)
        0x6f4a6a => [0xaf34, false], # Korean hangul
        0x234448 => [0x92f2, false], # East Asian ideograph
        0x28422b => [0x6a2f, false], # East Asian ideograph
        0x334449 => [0x6144, false], # East Asian ideograph
        0x22444a => [0x6b57, false], # East Asian ideograph
        0x2d444b => [0x6852, false], # East Asian ideograph
        0x6f5530 => [0xc568, false], # Korean hangul
        0x6f513c => [0xbcc0, false], # Korean hangul
        0x22444c => [0x6b54, false], # East Asian ideograph
        0x23444d => [0x9307, false], # East Asian ideograph
        0x22444e => [0x6b55, false], # East Asian ideograph
        0x6f5c51 => [0xd4cc, false], # Korean hangul
        0x515e5d => [0x9616, false], # East Asian ideograph
        0x2d4450 => [0x8308, false], # East Asian ideograph
        0x224451 => [0x6b5c, false], # East Asian ideograph
        0x287a56 => [0x8114, false], # East Asian ideograph
        0x6f5062 => [0xbbff, false], # Korean hangul
        0x212b38 => [0xff0c, false], # Ideographic variant comma
        0x225150 => [0x70f4, false], # East Asian ideograph
        0x23554f => [0x9b10, false], # East Asian ideograph
        0x224453 => [0x6b5e, false], # East Asian ideograph
        0x6f5b77 => [0xd328, false], # Korean hangul
        0x224454 => [0x6b60, false], # East Asian ideograph
        0x22625d => [0x777e, false], # East Asian ideograph
        0x217c34 => [0x5aae, false], # East Asian ideograph
        0x4b4456 => [0x6813, false], # East Asian ideograph
        0x6f5734 => [0xc800, false], # Korean hangul
        0x294457 => [0x9529, false], # East Asian ideograph
        0x212b39 => [0xff1b, false], # Ideographic semicolon
        0x234458 => [0x931f, false], # East Asian ideograph
        0x6f5661 => [0xc73c, false], # Korean hangul
        0x23445a => [0x9331, false], # East Asian ideograph
        0x4d5f70 => [0x9f44, false], # East Asian ideograph
        0x22445b => [0x6b6b, false], # East Asian ideograph
        0x22736b => [0x7e95, false], # East Asian ideograph
        0x22445d => [0x6b6c, false], # East Asian ideograph
        0x4c284c => [0x53a9, false], # East Asian ideograph
        0x4b3c33 => [0x5dcc, false], # East Asian ideograph
        0x215e60 => [0x95bb, false], # East Asian ideograph
        0x23445f => [0x930f, false], # East Asian ideograph
        0x6f4a6b => [0xaf3c, false], # Korean hangul
        0x224461 => [0x6b71, false], # East Asian ideograph
        0x6f5d2c => [0xd600, false], # Korean hangul
        0x234462 => [0x9302, false], # East Asian ideograph
        0x6f5170 => [0xbe4b, false], # Korean hangul
        0x274463 => [0x6761, false], # East Asian ideograph
        0x234464 => [0x9324, false], # East Asian ideograph
        0x2d5b2f => [0x8eb1, false], # East Asian ideograph
        0x214466 => [0x6885, false], # East Asian ideograph
        0x274468 => [0x67ad, false], # East Asian ideograph
        0x294f77 => [0x989f, false], # East Asian ideograph
        0x6f5221 => [0xbe70, false], # Korean hangul
        0x6f4b7a => [0xb11d, false], # Korean hangul
        0x274469 => [0x6800, false], # East Asian ideograph
        0x2d5f73 => [0x975a, false], # Unrelated variant of EACC 234C76 which maps to 975A
        0x23446a => [0x9323, false], # East Asian ideograph
        0x22446b => [0x6b7e, false], # East Asian ideograph
        0x212b3d => [0xff01, false], # Ideographic exclamation point
        0x225155 => [0x7111, false], # East Asian ideograph
        0x23446c => [0x9321, false], # East Asian ideograph
        0x4c683e => [0x79eb, false], # East Asian ideograph
        0x27446d => [0x5f03, false], # East Asian ideograph
        0x6f5222 => [0xbe71, false], # Korean hangul
        0x27446e => [0x6816, false], # East Asian ideograph
        0x2d4e79 => [0x5fa1, false], # East Asian ideograph
        0x6f5735 => [0xc801, false], # Korean hangul
        0x2d4b43 => [0x746f, false], # East Asian ideograph
        0x274471 => [0x680b, false], # East Asian ideograph
        0x4b5a68 => [0x8df5, false], # East Asian ideograph (variant of 275A68 which maps to 8DF5)
        0x234472 => [0x9301, false], # East Asian ideograph
        0x6f5223 => [0xbe73, false], # Korean hangul
        0x6f5326 => [0xc124, false], # Korean hangul
        0x224473 => [0x6b84, false], # East Asian ideograph
        0x2d3332 => [0x5190, false], # East Asian ideograph
        0x234474 => [0x9315, false], # East Asian ideograph
        0x47366f => [0x8d4d, false], # East Asian ideograph
        0x294475 => [0x9494, false], # East Asian ideograph
        0x235556 => [0x9b1d, false], # East Asian ideograph
        0x234476 => [0x9329, false], # East Asian ideograph
        0x23386f => [0x8dba, false], # East Asian ideograph
        0x232f21 => [0x88fc, false], # East Asian ideograph
        0x2d4a26 => [0x713c, false], # East Asian ideograph
        0x287035 => [0x7c74, false], # East Asian ideograph
        0x6f5224 => [0xbe74, false], # Korean hangul
        0x234478 => [0x932e, false], # East Asian ideograph
        0x234479 => [0x932a, false], # East Asian ideograph
        0x6f4a6c => [0xaf3d, false], # Korean hangul
        0x27447a => [0x67a3, false], # East Asian ideograph
        0x6f5d2d => [0xd601, false], # Korean hangul
        0x22447b => [0x6b95, false], # East Asian ideograph
        0x21447c => [0x6912, false], # East Asian ideograph
        0x216f27 => [0x5423, false], # East Asian ideograph
        0x213c4d => [0x5e11, false], # East Asian ideograph
        0x2d447d => [0x684c, false], # East Asian ideograph
        0x2d3d5e => [0x9af4, false], # East Asian ideograph
        0x23447e => [0x9335, false], # East Asian ideograph
        0x706058 => [0x562d, false], # East Asian ideograph
        0x273671 => [0x54df, false], # East Asian ideograph
        0x6f5226 => [0xbe7b, false], # Korean hangul
        0x232f2d => [0x8909, false], # East Asian ideograph
        0x23444c => [0x9303, false], # East Asian ideograph
        0x4d5b35 => [0x9dab, false], # East Asian ideograph
        0x232f2f => [0x8918, false], # East Asian ideograph
        0x6f5b79 => [0xd32c, false], # Korean hangul
        0x213d3c => [0x5f15, false], # East Asian ideograph
        0x6f566a => [0xc774, false], # Korean hangul
        0x6f5227 => [0xbe7c, false], # Korean hangul
        0x213c4f => [0x5e25, false], # East Asian ideograph
        0x2d3632 => [0x8a7b, false], # East Asian ideograph
        0x346622 => [0x589d, false], # East Asian ideograph
        0x295c47 => [0x9e68, false], # East Asian ideograph
        0x293a2e => [0x8dc4, false], # East Asian ideograph
        0x2e6f35 => [0x6cd4, false], # East Asian ideograph
        0x6f566b => [0xc775, false], # Korean hangul
        0x6f5228 => [0xbe7d, false], # Korean hangul
        0x22516d => [0x7134, false], # East Asian ideograph
        0x23444e => [0x931e, false], # East Asian ideograph
        0x2f5d49 => [0x9ea4, false], # East Asian ideograph
        0x2e313a => [0x6332, false], # East Asian ideograph
        0x216f3a => [0x546d, false], # East Asian ideograph
        0x6f566c => [0xc778, false], # Korean hangul
        0x282458 => [0x5d03, false], # East Asian ideograph
        0x216f3b => [0x5491, false], # East Asian ideograph
        0x6f5229 => [0xbe80, false], # Korean hangul
        0x4d5f7b => [0x97f2, false], # East Asian ideograph
        0x222f3d => [0x6201, false], # East Asian ideograph
        0x295c49 => [0x9e47, false], # East Asian ideograph
        0x4b577e => [0x7e7f, false], # East Asian ideograph
        0x217c41 => [0x5ac4, false], # East Asian ideograph
        0x215a28 => [0x8cc2, false], # East Asian ideograph
        0x697265 => [0x9c5a, false], # East Asian ideograph
        0x216f42 => [0x5494, false], # East Asian ideograph
        0x232f43 => [0x8915, false], # East Asian ideograph
        0x333344 => [0x51db, false], # East Asian ideograph
        0x6f566e => [0xc77d, false], # Korean hangul
        0x274340 => [0x6656, false], # East Asian ideograph
        0x6f522b => [0xbe8c, false], # Korean hangul
        0x213c53 => [0x5e36, false], # East Asian ideograph
        0x4c5175 => [0x8315, false], # East Asian ideograph
        0x225e37 => [0x75a2, false], # East Asian ideograph
        0x222f47 => [0x6214, false], # East Asian ideograph
        0x2d3921 => [0x591f, false], # East Asian ideograph
        0x233345 => [0x8ae2, false], # East Asian ideograph
        0x216f49 => [0x548d, false], # East Asian ideograph
        0x355d5c => [0x8c8e, false], # East Asian ideograph
        0x6f566f => [0xc783, false], # Korean hangul
        0x216f4a => [0x5463, false], # East Asian ideograph
        0x6f522c => [0xbe8f, false], # Korean hangul
        0x6f5737 => [0xc808, false], # Korean hangul
        0x225160 => [0x70f6, false], # East Asian ideograph
        0x6f5670 => [0xc784, false], # Korean hangul
        0x234453 => [0x931d, false], # East Asian ideograph
        0x216f7b => [0x551a, false], # East Asian ideograph
        0x6f5d68 => [0xd744, false], # Korean hangul
        0x6f5671 => [0xc785, false], # Korean hangul
        0x6f522e => [0xbe91, false], # Korean hangul
        0x294427 => [0x94d7, false], # East Asian ideograph
        0x234454 => [0x92fa, false], # East Asian ideograph
        0x2d3d67 => [0x9015, false], # East Asian ideograph
        0x6f4a6e => [0xaf41, false], # Korean hangul
        0x222f56 => [0x6223, false], # East Asian ideograph
        0x6f4f43 => [0xb8e8, false], # Korean hangul
        0x6f5d2f => [0xd608, false], # Korean hangul
        0x335561 => [0x8462, false], # East Asian ideograph
        0x216f58 => [0x54a1, false], # East Asian ideograph
        0x6f5672 => [0xc787, false], # Korean hangul
        0x274344 => [0x6682, false], # East Asian ideograph
        0x6f522f => [0xbe98, false], # Korean hangul
        0x213c57 => [0x5e3d, false], # East Asian ideograph
        0x4b356a => [0x55ec, false], # East Asian ideograph
        0x23223c => [0x83f3, false], # East Asian ideograph
        0x696f5b => [0x958a, false], # East Asian ideograph
        0x273238 => [0x4fa6, false], # East Asian ideograph
        0x695c4f => [0x69dd, false], # East Asian ideograph
        0x222f5d => [0x6224, false], # East Asian ideograph
        0x6f5673 => [0xc788, false], # Korean hangul
        0x216f5e => [0x54be, false], # East Asian ideograph
        0x6f5230 => [0xbea8, false], # Korean hangul
        0x213c58 => [0x5e40, false], # East Asian ideograph
        0x692433 => [0x3053, false], # Hiragana letter KO
        0x292f60 => [0x88e2, false], # East Asian ideograph
        0x6f5066 => [0xbc0b, false], # Korean hangul
        0x4c2f61 => [0x622c, false], # East Asian ideograph
        0x4b4235 => [0x6442, false], # East Asian ideograph
        0x6f5b7b => [0xd338, false], # Korean hangul
        0x6f5c32 => [0xd3ab, false], # Korean hangul
        0x6f5231 => [0xbed0, false], # Korean hangul
        0x213c59 => [0x5e4c, false], # East Asian ideograph
        0x216f64 => [0x54b5, false], # East Asian ideograph
        0x225e2e => [0x7594, false], # East Asian ideograph
        0x6f5738 => [0xc80a, false], # Korean hangul
        0x2d3f24 => [0x661a, false], # East Asian ideograph
        0x226f66 => [0x7cce, false], # East Asian ideograph
        0x4b4236 => [0x643a, false], # East Asian ideograph
        0x6f5a30 => [0xcf04, false], # Korean hangul
        0x2d4a34 => [0x718f, false], # East Asian ideograph
        0x226f68 => [0x7cc8, false], # East Asian ideograph
        0x6f5a28 => [0xcef4, false], # Korean hangul
        0x6f5232 => [0xbed1, false], # Korean hangul
        0x222f69 => [0x97ef, false], # East Asian ideograph
        0x333428 => [0x523c, false], # East Asian ideograph
        0x6f5676 => [0xc78e, false], # Korean hangul
        0x216f6d => [0x54ae, false], # East Asian ideograph
        0x6f5233 => [0xbed4, false], # Korean hangul
        0x2d3d6c => [0x5f93, false], # East Asian ideograph
        0x6f4a6f => [0xaf42, false], # Korean hangul
        0x232f6f => [0x894f, false], # East Asian ideograph
        0x2d5b42 => [0x8f19, false], # East Asian ideograph
        0x393e7d => [0x7609, false], # East Asian ideograph
        0x695c53 => [0x6a2e, false], # East Asian ideograph
        0x225167 => [0x70ef, false], # East Asian ideograph
        0x235566 => [0x9b23, false], # East Asian ideograph
        0x216f71 => [0x54bf, false], # East Asian ideograph
        0x6f5677 => [0xc655, false], # Korean hangul
        0x292f72 => [0x88e5, false], # East Asian ideograph
        0x6f5234 => [0xbed7, false], # Korean hangul
        0x213c5c => [0x5e57, false], # East Asian ideograph
        0x4d2f73 => [0x7e5d, false], # East Asian ideograph
        0x6f4f4c => [0xb93c, false], # Korean hangul
        0x2d5b43 => [0x8efd, false], # East Asian ideograph
        0x226f75 => [0x7cd7, false], # East Asian ideograph
        0x225168 => [0x7100, false], # East Asian ideograph
        0x33334e => [0x51fe, false], # East Asian ideograph
        0x225a21 => [0x7428, false], # East Asian ideograph
        0x215a22 => [0x8cc7, false], # East Asian ideograph
        0x23445b => [0x9306, false], # East Asian ideograph
        0x215a23 => [0x8cca, false], # East Asian ideograph
        0x6f536c => [0xc274, false], # Korean hangul
        0x4b312d => [0x4f2b, false], # East Asian ideograph
        0x235a24 => [0x9d04, false], # East Asian ideograph
        0x21322a => [0x5003, false], # East Asian ideograph
        0x222f7a => [0x6250, false], # East Asian ideograph
        0x215a25 => [0x8cc4, false], # East Asian ideograph
        0x6f5d60 => [0xd71c, false], # Korean hangul
        0x335568 => [0x8406, false], # East Asian ideograph
        0x226f7b => [0x7ce8, false], # East Asian ideograph (variant of 4C6F7B which maps to 7CE8)
        0x6f5b7c => [0xd339, false], # Korean hangul
        0x275a26 => [0x8d40, false], # East Asian ideograph
        0x6f5679 => [0xc790, false], # Korean hangul
        0x2e2f7c => [0x634d, false], # East Asian ideograph
        0x215a27 => [0x8cc3, false], # East Asian ideograph
        0x213c5e => [0x5e63, false], # East Asian ideograph
        0x276121 => [0x998a, false], # East Asian ideograph
        0x2d6f7d => [0x8123, false], # East Asian ideograph
        0x235a28 => [0x9d07, false], # East Asian ideograph
        0x2d4472 => [0x68ca, false], # East Asian ideograph
        0x215a29 => [0x8cd3, false], # East Asian ideograph
        0x215a2a => [0x8cd1, false], # East Asian ideograph
        0x217d2e => [0x5b0d, false], # East Asian ideograph
        0x215a2b => [0x8cd2, false], # East Asian ideograph
        0x395063 => [0x9939, false], # East Asian ideograph
        0x6f567a => [0xc791, false], # Korean hangul
        0x275a2c => [0x8d54, false], # East Asian ideograph
        0x23445d => [0x92f9, false], # East Asian ideograph
        0x215a2d => [0x8ce6, false], # East Asian ideograph
        0x295c57 => [0x9e6b, false], # East Asian ideograph
        0x215a2f => [0x8ce3, false], # East Asian ideograph
        0x213076 => [0x4ed9, false], # East Asian ideograph
        0x215a30 => [0x8ce2, false], # East Asian ideograph
        0x4b3c38 => [0x949c, false], # East Asian ideograph
        0x275a31 => [0x8d31, false], # East Asian ideograph
        0x276123 => [0x9992, false], # East Asian ideograph
        0x225a32 => [0x743b, false], # East Asian ideograph
        0x4b3130 => [0x4fab, false], # East Asian ideograph
        0x6f4a70 => [0xaf43, false], # Korean hangul
        0x215a33 => [0x8cdc, false], # East Asian ideograph
        0x275a34 => [0x8d28, false], # East Asian ideograph
        0x215a35 => [0x8ced, false], # East Asian ideograph
        0x2d4a3b => [0x4e89, false], # East Asian ideograph
        0x225a36 => [0x7444, false], # East Asian ideograph
        0x275a37 => [0x8d5b, false], # East Asian ideograph
        0x215a38 => [0x8cfa, false], # East Asian ideograph
        0x215a39 => [0x8d05, false], # East Asian ideograph
        0x293a40 => [0x8df8, false], # East Asian ideograph
        0x23556c => [0x9b29, false], # East Asian ideograph
        0x215a3a => [0x8cfc, false], # East Asian ideograph
        0x215a3b => [0x8d08, false], # East Asian ideograph (variant of 4B5A3B which maps to 8D08)
        0x6f5352 => [0xc1f0, false], # Korean hangul
        0x215a3c => [0x8d0b, false], # East Asian ideograph
        0x215a3d => [0x8d0a, false], # East Asian ideograph
        0x215a3e => [0x8d0f, false], # East Asian ideograph
        0x6f5b7d => [0xd33b, false], # Korean hangul
        0x215a3f => [0x8d0d, false], # East Asian ideograph
        0x6f567e => [0xc7a0, false], # Korean hangul
        0x213d40 => [0x5f1b, false], # East Asian ideograph
        0x215a40 => [0x8d13, false], # East Asian ideograph
        0x276126 => [0x990d, false], # East Asian ideograph
        0x215a41 => [0x8d16, false], # East Asian ideograph
        0x215a42 => [0x8d1b, false], # East Asian ideograph
        0x295c5b => [0x9e6c, false], # East Asian ideograph
        0x225a43 => [0x7458, false], # East Asian ideograph
        0x235a44 => [0x9d1d, false], # East Asian ideograph
        0x225a45 => [0x7442, false], # East Asian ideograph
        0x6f5a46 => [0xcf71, false], # Korean hangul
        0x2d3d75 => [0x60ea, false], # East Asian ideograph
        0x2e4174 => [0x6aa9, false], # East Asian ideograph
        0x225a47 => [0x744b, false], # East Asian ideograph
        0x215a48 => [0x8d70, false], # East Asian ideograph
        0x6f5660 => [0xc737, false], # Korean hangul
        0x337345 => [0x9f67, false], # East Asian ideograph
        0x6f5a49 => [0xcf80, false], # Korean hangul
        0x225a4a => [0x744a, false], # East Asian ideograph
        0x213c65 => [0x5e74, false], # East Asian ideograph
        0x6f5a4b => [0xcf8c, false], # Korean hangul
        0x6f5938 => [0xcc71, false], # Korean hangul
        0x2d3d76 => [0x5fb4, false], # East Asian ideograph
        0x2d5476 => [0x8318, false], # East Asian ideograph
        0x6f5a4c => [0xcf8d, false], # Korean hangul
        0x6f5573 => [0xc628, false], # Korean hangul
        0x6f5a4d => [0xcfa1, false], # Korean hangul
        0x2d5a4e => [0x8d82, false], # East Asian ideograph
        0x215a4f => [0x8d99, false], # East Asian ideograph
        0x4b3864 => [0x58c7, false], # East Asian ideograph
        0x275a50 => [0x8d76, false], # East Asian ideograph
        0x2d392f => [0x7287, false], # East Asian ideograph
        0x6f5a51 => [0xcfe0, false], # Korean hangul
        0x704c2a => [0x915e, false], # East Asian ideograph
        0x224e37 => [0x6faf, false], # East Asian ideograph
        0x6f5a52 => [0xcfe1, false], # Korean hangul
        0x6f5c58 => [0xd515, false], # Korean hangul
        0x215a53 => [0x8da8, false], # East Asian ideograph
        0x6f492e => [0xac86, false], # Korean hangul
        0x6f537d => [0xc2b4, false], # Korean hangul
        0x6f523f => [0xbf40, false], # Korean hangul
        0x225a55 => [0x7457, false], # East Asian ideograph
        0x225a56 => [0x7451, false], # East Asian ideograph
        0x6f5069 => [0xbc0f, false], # Korean hangul
        0x6f5a57 => [0xcff5, false], # Korean hangul
        0x293a46 => [0x8e70, false], # East Asian ideograph
        0x23512f => [0x9916, false], # East Asian ideograph
        0x4b3642 => [0x8bf6, false], # East Asian ideograph
        0x2e4e41 => [0x7032, false], # East Asian ideograph
        0x215a59 => [0x8ddb, false], # East Asian ideograph
        0x6f5240 => [0xbf41, false], # Korean hangul
        0x706131 => [0x5c9c, false], # East Asian ideograph
        0x4b357b => [0x54cc, false], # East Asian ideograph
        0x225a5a => [0x745d, false], # East Asian ideograph
        0x225a5b => [0x7454, false], # East Asian ideograph
        0x225174 => [0x7131, false], # East Asian ideograph
        0x235573 => [0x9b2d, false], # East Asian ideograph
        0x235130 => [0x9914, false], # East Asian ideograph
        0x33386e => [0x576f, false], # East Asian ideograph
        0x6f5a5e => [0xd038, false], # Korean hangul
        0x6f5241 => [0xbf44, false], # Korean hangul
        0x2d5a5f => [0x8e5f, false], # East Asian ideograph
        0x6f5949 => [0xcd09, false], # Korean hangul
        0x225a60 => [0x746d, false], # East Asian ideograph
        0x277239 => [0x54d4, false], # East Asian ideograph
        0x225a61 => [0x7462, false], # East Asian ideograph
        0x235574 => [0x9b2e, false], # East Asian ideograph (not in Unicode)
        0x29506c => [0x996b, false], # East Asian ideograph
        0x6f532f => [0xc131, false], # Korean hangul
        0x215a62 => [0x8df3, false], # East Asian ideograph
        0x215a63 => [0x8dfa, false], # East Asian ideograph
        0x6f5242 => [0xbf48, false], # Korean hangul
        0x27612d => [0x51af, false], # East Asian ideograph
        0x6f5a64 => [0xd07d, false], # Korean hangul
        0x4c796b => [0x815f, false], # East Asian ideograph
        0x235a65 => [0x9d30, false], # East Asian ideograph
        0x275021 => [0x7b0b, false], # East Asian ideograph
        0x235132 => [0x9911, false], # East Asian ideograph
        0x2f3b63 => [0x5e32, false], # East Asian ideograph (not in Unicode)
        0x2d4a45 => [0x5c12, false], # East Asian ideograph
        0x275a68 => [0x8df5, false], # East Asian ideograph
        0x6f5243 => [0xbf50, false], # Korean hangul
        0x213c6b => [0x5e7e, false], # East Asian ideograph
        0x215a69 => [0x8e22, false], # East Asian ideograph
        0x225a6a => [0x7471, false], # East Asian ideograph
        0x225a6b => [0x7468, false], # East Asian ideograph
        0x4b5936 => [0x8b20, false], # East Asian ideograph
        0x4d5a6c => [0x9d46, false], # East Asian ideograph
        0x216035 => [0x97fb, false], # East Asian ideograph
        0x2d4a46 => [0x58bb, false], # East Asian ideograph
        0x6f5a6d => [0xd0b9, false], # Korean hangul
        0x4b4857 => [0x6f22, false], # East Asian ideograph
        0x235a70 => [0x9d5c, false], # East Asian ideograph
        0x224d35 => [0x6f90, false], # East Asian ideograph
        0x282951 => [0x5f2a, false], # East Asian ideograph
        0x275a71 => [0x8e0a, false], # East Asian ideograph
        0x6f5a72 => [0xd0c4, false], # Korean hangul
        0x6f5245 => [0xbf55, false], # Korean hangul
        0x276130 => [0x9a6e, false], # East Asian ideograph
        0x6f4c27 => [0xb141, false], # Korean hangul
        0x695a73 => [0x6683, false], # East Asian ideograph
        0x454b7a => [0x7523, false], # East Asian ideograph
        0x6f5a74 => [0xd0c9, false], # Korean hangul
        0x2d377c => [0x962c, false], # East Asian ideograph
        0x215a75 => [0x8e4b, false], # East Asian ideograph
        0x295822 => [0x9cae, false], # East Asian ideograph
        0x6f5a76 => [0xd0d1, false], # Korean hangul
        0x6f5a77 => [0xd0d3, false], # Korean hangul
        0x6f5246 => [0xbfb0, false], # Korean hangul
        0x234522 => [0x9314, false], # East Asian ideograph
        0x225a78 => [0x7460, false], # East Asian ideograph
        0x225a79 => [0x7472, false], # East Asian ideograph
        0x225a7a => [0x7484, false], # East Asian ideograph
        0x224525 => [0x6b99, false], # East Asian ideograph
        0x224d37 => [0x6f8d, false], # East Asian ideograph
        0x225a7b => [0x7487, false], # East Asian ideograph
        0x274526 => [0x6781, false], # East Asian ideograph
        0x6f5a7c => [0xd0e0, false], # Korean hangul
        0x6f5247 => [0xbfc0, false], # Korean hangul
        0x276132 => [0x9a73, false], # East Asian ideograph
        0x6f5a7d => [0xd0e4, false], # Korean hangul
        0x234528 => [0x92fe, false], # East Asian ideograph
        0x215a7e => [0x8e7a, false], # East Asian ideograph
        0x224529 => [0x6b9b, false], # East Asian ideograph
        0x27452a => [0x6768, false], # East Asian ideograph
        0x3f4c3c => [0x7582, false], # East Asian ideograph
        0x27452b => [0x6862, false], # East Asian ideograph
        0x2e7062 => [0x7d4f, false], # East Asian ideograph
        0x6f5248 => [0xbfc5, false], # Korean hangul
        0x276133 => [0x9a7b, false], # East Asian ideograph
        0x4b3b61 => [0x5c64, false], # East Asian ideograph
        0x69242b => [0x304b, false], # Hiragana letter KA
        0x27452d => [0x4e1a, false], # East Asian ideograph
        0x2d3931 => [0x67f0, false], # East Asian ideograph
        0x2d7e6a => [0x51a4, false], # East Asian ideograph
        0x27452f => [0x67ab, false], # East Asian ideograph
        0x6f5c5a => [0xd53d, false], # Korean hangul
        0x224d39 => [0x6f92, false], # East Asian ideograph
        0x692434 => [0x3054, false], # Hiragana letter GO
        0x6f5249 => [0xbfcc, false], # Korean hangul
        0x234531 => [0x9341, false], # East Asian ideograph
        0x217c60 => [0x5adc, false], # East Asian ideograph
        0x273764 => [0x5631, false], # East Asian ideograph
        0x234532 => [0x9319, false], # East Asian ideograph
        0x6f506b => [0xbbb4, false], # Korean hangul
        0x4b4534 => [0x6994, false], # East Asian ideograph (variant of 214534)
        0x224d3a => [0x6f89, false], # East Asian ideograph
        0x234535 => [0x934c, false], # East Asian ideograph
        0x6f524a => [0xbfcd, false], # Korean hangul
        0x224536 => [0x6ba2, false], # East Asian ideograph
        0x6f4c28 => [0xb144, false], # Korean hangul
        0x21382f => [0x577c, false], # East Asian ideograph
        0x274537 => [0x8363, false], # East Asian ideograph
        0x224538 => [0x6baa, false], # East Asian ideograph
        0x274539 => [0x6784, false], # East Asian ideograph
        0x235b6a => [0x9da1, false], # East Asian ideograph
        0x2d453a => [0x6760, false], # East Asian ideograph
        0x22453b => [0x6bad, false], # East Asian ideograph
        0x234471 => [0x9340, false], # East Asian ideograph
        0x692531 => [0x30b1, false], # Katakana letter KE
        0x22453d => [0x6bb0, false], # East Asian ideograph
        0x6f4f66 => [0xb9c8, false], # Korean hangul
        0x6f5663 => [0xc740, false], # Korean hangul
        0x274871 => [0x6d47, false], # East Asian ideograph
        0x224d3c => [0x6f8c, false], # East Asian ideograph
        0x22453f => [0x6bb3, false], # East Asian ideograph
        0x274540 => [0x67aa, false], # East Asian ideograph
        0x234541 => [0x9379, false], # East Asian ideograph
        0x4b3144 => [0x4f36, false], # East Asian ideograph (variant of 213144 which maps to 4F36)
        0x295c6c => [0x9e6a, false], # East Asian ideograph
        0x2d4543 => [0x6901, false], # East Asian ideograph
        0x224d3d => [0x6f62, false], # East Asian ideograph (variant of 4C4D3D which maps to 6F62)
        0x33513c => [0x7d4c, false], # East Asian ideograph
        0x214544 => [0x6a23, false], # East Asian ideograph
        0x2d5036 => [0x84d1, false], # East Asian ideograph
        0x6f524d => [0xbfdc, false], # Korean hangul
        0x234a5e => [0x967f, false], # East Asian ideograph
        0x223553 => [0x6509, false], # East Asian ideograph
        0x4b7577 => [0x57d3, false], # East Asian ideograph
        0x225e4a => [0x75c2, false], # East Asian ideograph
        0x214546 => [0x6a01, false], # East Asian ideograph
        0x2d3932 => [0x7ad2, false], # East Asian ideograph
        0x274547 => [0x6807, false], # East Asian ideograph
        0x234548 => [0x935c, false], # East Asian ideograph
        0x6f5c5b => [0xd540, false], # Korean hangul
        0x216037 => [0x9801, false], # East Asian ideograph
        0x274549 => [0x67a2, false], # East Asian ideograph
        0x6f524e => [0xbfdd, false], # Korean hangul
        0x27454a => [0x697c, false], # East Asian ideograph
        0x213833 => [0x57ae, false], # East Asian ideograph
        0x4b602d => [0x9771, false], # East Asian ideograph
        0x2d5b5d => [0x8fa2, false], # East Asian ideograph
        0x2d3944 => [0x511e, false], # East Asian ideograph
        0x27454c => [0x6868, false], # East Asian ideograph
        0x23454d => [0x9347, false], # East Asian ideograph
        0x293373 => [0x8c21, false], # East Asian ideograph
        0x27454e => [0x4e50, false], # East Asian ideograph
        0x6f524f => [0xbfe1, false], # Korean hangul
        0x27454f => [0x679e, false], # East Asian ideograph
        0x2d3f2a => [0x5abf, false], # East Asian ideograph
        0x2d4550 => [0x58ab, false], # East Asian ideograph
        0x2d5b5e => [0x8fa7, false], # East Asian ideograph
        0x234551 => [0x937a, false], # East Asian ideograph
        0x29582c => [0x9cb1, false], # East Asian ideograph
        0x274553 => [0x692d, false], # East Asian ideograph
        0x27767a => [0x57da, false], # East Asian ideograph
        0x224554 => [0x6bc8, false], # East Asian ideograph
        0x274555 => [0x6811, false], # East Asian ideograph
        0x234556 => [0x937c, false], # East Asian ideograph
        0x293a57 => [0x8dfb, false], # East Asian ideograph
        0x274557 => [0x6866, false], # East Asian ideograph
        0x29582d => [0x9cb7, false], # East Asian ideograph
        0x235140 => [0x991c, false], # East Asian ideograph
        0x274558 => [0x6734, false], # East Asian ideograph
        0x474e5c => [0x97de, false], # East Asian ideograph (variant of 234E5C which maps to 97DE)
        0x274366 => [0x80e7, false], # East Asian ideograph
        0x6f5251 => [0xc059, false], # Korean hangul
        0x27613c => [0x9a86, false], # East Asian ideograph
        0x2d3261 => [0x5039, false], # East Asian ideograph
        0x27455b => [0x6865, false], # East Asian ideograph
        0x275030 => [0x8303, false], # East Asian ideograph
        0x23455c => [0x9377, false], # East Asian ideograph
        0x6f5172 => [0xbe4e, false], # Korean hangul
        0x27455d => [0x673a, false], # East Asian ideograph
        0x213f50 => [0x61ca, false], # East Asian ideograph
        0x6f5252 => [0xc05c, false], # Korean hangul
        0x23455e => [0x9358, false], # East Asian ideograph
        0x2d5f63 => [0x873a, false], # East Asian ideograph
        0x27455f => [0x6863, false], # East Asian ideograph
        0x224560 => [0x6bda, false], # East Asian ideograph
        0x33327a => [0x5150, false], # East Asian ideograph
        0x274561 => [0x68c0, false], # East Asian ideograph
        0x29582f => [0x9cb5, false], # East Asian ideograph
        0x274562 => [0x6867, false], # East Asian ideograph
        0x3f347d => [0x84e1, false], # East Asian ideograph
        0x6f5253 => [0xc060, false], # Korean hangul
        0x274563 => [0x67e0, false], # East Asian ideograph
        0x235131 => [0x9917, false], # East Asian ideograph
        0x234564 => [0x9376, false], # East Asian ideograph
        0x274565 => [0x67dc, false], # East Asian ideograph
        0x213230 => [0x5065, false], # East Asian ideograph
        0x224a4c => [0x6e1f, false], # East Asian ideograph
        0x274566 => [0x69db, false], # East Asian ideograph
        0x294567 => [0x9537, false], # East Asian ideograph
        0x6f5254 => [0xc068, false], # Korean hangul
        0x27613f => [0x9a88, false], # East Asian ideograph
        0x6f4c2a => [0xb151, false], # Korean hangul
        0x2d4569 => [0x6ac9, false], # East Asian ideograph
        0x4b314c => [0x5f95, false], # East Asian ideograph
        0x6f573f => [0xc81c, false], # Korean hangul
        0x23456a => [0x9348, false], # East Asian ideograph
        0x27325d => [0x4ef7, false], # East Asian ideograph
        0x27456b => [0x691f, false], # East Asian ideograph
        0x213f3b => [0x616b, false], # East Asian ideograph
        0x295831 => [0x9cb6, false], # East Asian ideograph
        0x27456c => [0x6809, false], # East Asian ideograph
        0x2e4e56 => [0x9800, false], # East Asian ideograph
        0x6f5255 => [0xc069, false], # Korean hangul
        0x27456d => [0x6a79, false], # East Asian ideograph
        0x276140 => [0x9a91, false], # East Asian ideograph
        0x23447b => [0x933f, false], # East Asian ideograph
        0x27456e => [0x680f, false], # East Asian ideograph
        0x27456f => [0x6a31, false], # East Asian ideograph
        0x224570 => [0x6bea, false], # East Asian ideograph
        0x3f456d => [0x8263, false], # East Asian ideograph
        0x235145 => [0x9927, false], # East Asian ideograph
        0x274571 => [0x6984, false], # East Asian ideograph
        0x2d4a58 => [0x7f9d, false], # East Asian ideograph
        0x6f5256 => [0xc090, false], # Korean hangul
        0x217c6d => [0x5ae5, false], # East Asian ideograph
        0x286540 => [0x7800, false], # East Asian ideograph
        0x23447c => [0x933a, false], # East Asian ideograph
        0x234573 => [0x9360, false], # East Asian ideograph
        0x2d4574 => [0x5ffb, false], # East Asian ideograph
        0x6f5d37 => [0xd639, false], # Korean hangul
        0x233021 => [0x894d, false], # East Asian ideograph
        0x6f5257 => [0xc091, false], # Korean hangul
        0x234577 => [0x936e, false], # East Asian ideograph
        0x287022 => [0x7cc1, false], # East Asian ideograph
        0x274578 => [0x94a6, false], # East Asian ideograph
        0x6f5844 => [0xc9f8, false], # Korean hangul
        0x213023 => [0x4e03, false], # East Asian ideograph
        0x225b2d => [0x7486, false], # East Asian ideograph
        0x234579 => [0x938f, false], # East Asian ideograph
        0x233024 => [0x895a, false], # East Asian ideograph
        0x293a5e => [0x8df9, false], # East Asian ideograph
        0x23457a => [0x93ac, false], # East Asian ideograph
        0x6f5c5d => [0xd54c, false], # Korean hangul
        0x233025 => [0x895e, false], # East Asian ideograph
        0x335147 => [0x7eee, false], # East Asian ideograph
        0x23457b => [0x9395, false], # East Asian ideograph
        0x213026 => [0x4e0a, false], # East Asian ideograph
        0x4b4e7b => [0x7985, false], # East Asian ideograph (variant of 274E7B which maps to 7985)
        0x6f5258 => [0xc094, false], # Korean hangul
        0x27457c => [0x6b27, false], # East Asian ideograph
        0x295175 => [0x9993, false], # East Asian ideograph
        0x21383d => [0x57df, false], # East Asian ideograph
        0x2e3028 => [0x640b, false], # East Asian ideograph
        0x27457e => [0x6b24, false], # East Asian ideograph
        0x213029 => [0x4e10, false], # East Asian ideograph
        0x293a5f => [0x8dde, false], # East Asian ideograph
        0x2d4a5b => [0x7282, false], # East Asian ideograph
        0x22302b => [0x625a, false], # East Asian ideograph
        0x6f5259 => [0xc098, false], # Korean hangul
        0x276144 => [0x817e, false], # East Asian ideograph
        0x21302c => [0x4e19, false], # East Asian ideograph
        0x21383e => [0x580a, false], # East Asian ideograph
        0x22302d => [0x6266, false], # East Asian ideograph
        0x22702e => [0x7cf0, false], # East Asian ideograph
        0x6f5664 => [0xc744, false], # Korean hangul
        0x293a60 => [0x8e2c, false], # East Asian ideograph
        0x21302f => [0x4e18, false], # East Asian ideograph
        0x217030 => [0x5504, false], # East Asian ideograph
        0x6f525a => [0xc0a0, false], # Korean hangul
        0x234469 => [0x9338, false], # East Asian ideograph
        0x692126 => [0x30fb, false], # Ideographic centered point
        0x223031 => [0x6286, false], # East Asian ideograph
        0x21383f => [0x5805, false], # East Asian ideograph
        0x273032 => [0x5e76, false], # East Asian ideograph
        0x6f594a => [0xcd0c, false], # Korean hangul
        0x2d5b69 => [0x5ef5, false], # East Asian ideograph
        0x6f4c55 => [0xb284, false], # Korean hangul
        0x275039 => [0x7b03, false], # East Asian ideograph
        0x6f5666 => [0xc74c, false], # Korean hangul
        0x692139 => [0x3005, false], # Ideographic iteration mark
        0x69213c => [0x30fc, false], # Vowel elongation mark for kana
        0x213035 => [0x4e32, false], # East Asian ideograph
        0x695130 => [0x5116, false], # East Asian ideograph
        0x6f525b => [0xc0a3, false], # Korean hangul
        0x276146 => [0x9aa0, false], # East Asian ideograph
        0x217c72 => [0x5aea, false], # East Asian ideograph
        0x6f4a77 => [0xaf64, false], # Korean hangul
        0x23403e => [0x9146, false], # East Asian ideograph
        0x2d3263 => [0x4fad, false], # East Asian ideograph
        0x4b4f29 => [0x7a50, false], # East Asian ideograph
        0x692152 => [0x3008, false], # Ideographic less than sign
        0x27503a => [0x7b51, false], # East Asian ideograph
        0x692154 => [0x300a, false], # Ideographic left double angle bracket
        0x692155 => [0x300b, false], # Ideographic right double angle bracket
        0x217039 => [0x54f7, false], # East Asian ideograph
        0x21303a => [0x4e43, false], # East Asian ideograph
        0x2e4e5d => [0x6de0, false], # East Asian ideograph
        0x6f525c => [0xc0a5, false], # Korean hangul
        0x21303b => [0x4e45, false], # East Asian ideograph
        0x213841 => [0x5806, false], # East Asian ideograph
        0x217830 => [0x58a3, false], # East Asian ideograph
        0x6f576a => [0xc8f1, false], # Korean hangul
        0x233376 => [0x8b1f, false], # East Asian ideograph
        0x33514c => [0x7dd1, false], # East Asian ideograph
        0x213e21 => [0x5fcd, false], # East Asian ideograph
        0x276148 => [0x84e6, false], # East Asian ideograph
        0x2d4b71 => [0x7f3e, false], # East Asian ideograph
        0x223041 => [0x62a3, false], # East Asian ideograph
        0x213042 => [0x4e52, false], # East Asian ideograph
        0x277255 => [0x54d3, false], # East Asian ideograph
        0x213043 => [0x4e53, false], # East Asian ideograph
        0x227044 => [0x7d03, false], # East Asian ideograph
        0x234544 => [0x9386, false], # East Asian ideograph
        0x287045 => [0x7ea8, false], # East Asian ideograph
        0x227c31 => [0x82af, false], # East Asian ideograph
        0x234041 => [0x9147, false], # East Asian ideograph
        0x2d3954 => [0x7385, false], # East Asian ideograph
        0x213047 => [0x4e5d, false], # East Asian ideograph
        0x213049 => [0x4e5e, false], # East Asian ideograph
        0x6f525f => [0xc0ac, false], # Korean hangul
        0x28255a => [0x5d5d, false], # East Asian ideograph
        0x273f31 => [0x60ed, false], # East Asian ideograph
        0x225e5c => [0x75cf, false], # East Asian ideograph
        0x29472f => [0x94e9, false], # East Asian ideograph
        0x21304b => [0x4e73, false], # East Asian ideograph
        0x21304c => [0x4e7e, false], # East Asian ideograph
        0x27503e => [0x7bd3, false], # East Asian ideograph
        0x6f5667 => [0xc74d, false], # Korean hangul
        0x27304d => [0x4e71, false], # East Asian ideograph
        0x23514f => [0x992e, false], # East Asian ideograph
        0x6f5870 => [0xcbd4, false], # Korean hangul
        0x275b36 => [0x8f69, false], # East Asian ideograph
        0x6f5260 => [0xc0ad, false], # Korean hangul
        0x27614b => [0x60ca, false], # East Asian ideograph
        0x6f4a78 => [0xaf65, false], # Korean hangul
        0x227050 => [0x7d18, false], # East Asian ideograph
        0x227051 => [0x7d1e, false], # East Asian ideograph
        0x277258 => [0x6076, false], # East Asian ideograph (duplicate simplified)
        0x393052 => [0x65bc, false], # East Asian ideograph
        0x235150 => [0x992c, false], # East Asian ideograph
        0x213053 => [0x4e95, false], # East Asian ideograph
        0x6f5261 => [0xc0ae, false], # Korean hangul
        0x294040 => [0x90e6, false], # East Asian ideograph
        0x213055 => [0x4e92, false], # East Asian ideograph
        0x2e5f6f => [0x75b8, false], # East Asian ideograph
        0x27326a => [0x4fea, false], # East Asian ideograph
        0x223057 => [0x62d1, false], # East Asian ideograph
        0x235151 => [0x992a, false], # East Asian ideograph
        0x213058 => [0x4e9e, false], # East Asian ideograph
        0x2d4621 => [0x61fd, false], # East Asian ideograph
        0x213059 => [0x4e9b, false], # East Asian ideograph
        0x284333 => [0x680e, false], # East Asian ideograph
        0x294732 => [0x94f4, false], # East Asian ideograph
        0x33625e => [0x9eaa, false], # East Asian ideograph
        0x22705a => [0x7d3d, false], # East Asian ideograph
        0x6f5070 => [0xbc18, false], # Korean hangul
        0x232223 => [0x83fd, false], # East Asian ideograph
        0x222224 => [0x5bf0, false], # East Asian ideograph
        0x232225 => [0x841e, false], # East Asian ideograph
        0x693c36 => [0x96eb, false], # East Asian ideograph
        0x232229 => [0x83c9, false], # East Asian ideograph
        0x23222a => [0x83df, false], # East Asian ideograph
        0x23222c => [0x841f, false], # East Asian ideograph
        0x23222e => [0x840f, false], # East Asian ideograph
        0x69705d => [0x9786, false], # East Asian ideograph
        0x232230 => [0x8411, false], # East Asian ideograph
        0x222233 => [0x5c00, false], # East Asian ideograph
        0x222235 => [0x5c57, false], # East Asian ideograph
        0x232236 => [0x839a, false], # East Asian ideograph
        0x707438 => [0x823e, false], # East Asian ideograph
        0x33625f => [0x8534, false], # East Asian ideograph
        0x21305f => [0x4ea8, false], # East Asian ideograph
        0x22223c => [0x5c15, false], # East Asian ideograph
        0x333060 => [0x4eaf, false], # East Asian ideograph
        0x6f5742 => [0xc824, false], # Korean hangul
        0x232243 => [0x83d1, false], # East Asian ideograph
        0x275042 => [0x7bab, false], # East Asian ideograph
        0x222246 => [0x5c22, false], # East Asian ideograph
        0x223061 => [0x62e4, false], # East Asian ideograph
        0x222248 => [0x5c25, false], # East Asian ideograph
        0x23224a => [0x848e, false], # East Asian ideograph
        0x22224b => [0x5c2a, false], # East Asian ideograph
        0x23224c => [0x8439, false], # East Asian ideograph
        0x23224d => [0x8476, false], # East Asian ideograph
        0x23224e => [0x8479, false], # East Asian ideograph
        0x213c6e => [0x5e8a, false], # East Asian ideograph
        0x222252 => [0x5c2f, false], # East Asian ideograph
        0x217c7b => [0x5ada, false], # East Asian ideograph
        0x284335 => [0x6a7c, false], # East Asian ideograph
        0x4c3b22 => [0x6860, false], # East Asian ideograph
        0x294734 => [0x9566, false], # East Asian ideograph
        0x213064 => [0x4eba, false], # East Asian ideograph
        0x22225b => [0x5c32, false], # East Asian ideograph
        0x23225c => [0x8451, false], # East Asian ideograph
        0x23225f => [0x847d, false], # East Asian ideograph
        0x232262 => [0x845a, false], # East Asian ideograph
        0x222263 => [0x5c3b, false], # East Asian ideograph
        0x222265 => [0x5c44, false], # East Asian ideograph
        0x232266 => [0x8459, false], # East Asian ideograph
        0x222267 => [0x5c49, false], # East Asian ideograph
        0x232269 => [0x8473, false], # East Asian ideograph
        0x23226e => [0x843e, false], # East Asian ideograph
        0x6f5265 => [0xc0b4, false], # Korean hangul
        0x276150 => [0x9aa5, false], # East Asian ideograph
        0x232271 => [0x846d, false], # East Asian ideograph
        0x394735 => [0x6c3e, false], # East Asian ideograph
        0x223069 => [0x62b6, false], # East Asian ideograph
        0x232278 => [0x847a, false], # East Asian ideograph
        0x222279 => [0x5c59, false], # East Asian ideograph
        0x22227b => [0x5c5d, false], # East Asian ideograph
        0x22227c => [0x5c5f, false], # East Asian ideograph
        0x22706a => [0x7d3f, false], # East Asian ideograph
        0x21306b => [0x4ecd, false], # East Asian ideograph
        0x2d306c => [0x8b8e, false], # East Asian ideograph
        0x6f5266 => [0xc0b5, false], # Korean hangul
        0x276151 => [0x9a8a, false], # East Asian ideograph
        0x284337 => [0x6987, false], # East Asian ideograph
        0x225e63 => [0x75e7, false], # East Asian ideograph
        0x33512e => [0x7e8d, false], # East Asian ideograph
        0x4b306e => [0x4ee4, false], # East Asian ideograph (variant of 21306E which maps to 4EE4)
        0x21306f => [0x4ed8, false], # East Asian ideograph
        0x235156 => [0x992b, false], # East Asian ideograph
        0x454146 => [0x63db, false], # East Asian ideograph
        0x213071 => [0x4ed6, false], # East Asian ideograph
        0x6f5267 => [0xc0b6, false], # Korean hangul
        0x213072 => [0x4ede, false], # East Asian ideograph
        0x6f4e24 => [0xb544, false], # Korean hangul
        0x6f5a24 => [0xcee4, false], # Korean hangul
        0x225254 => [0x714f, false], # East Asian ideograph
        0x215b21 => [0x8e76, false], # East Asian ideograph
        0x6f5268 => [0xc0bc, false], # Korean hangul
        0x276153 => [0x80ae, false], # East Asian ideograph
        0x213077 => [0x4ee5, false], # East Asian ideograph
        0x215b22 => [0x8e7c, false], # East Asian ideograph
        0x21384d => [0x5857, false], # East Asian ideograph
        0x333078 => [0x5f77, false], # East Asian ideograph
        0x225d69 => [0x756f, false], # East Asian ideograph
        0x335e21 => [0x9221, false], # East Asian ideograph
        0x213079 => [0x4f09, false], # East Asian ideograph
        0x6f5b24 => [0xd0f1, false], # Korean hangul
        0x6f5b25 => [0xd130, false], # Korean hangul
        0x235158 => [0x9931, false], # East Asian ideograph
        0x4b3e2a => [0x6035, false], # East Asian ideograph
        0x275b26 => [0x8db8, false], # East Asian ideograph
        0x6f5269 => [0xc0bd, false], # Korean hangul
        0x6f4b2c => [0xafd4, false], # Korean hangul
        0x225b27 => [0x7482, false], # East Asian ideograph
        0x21384e => [0x5858, false], # East Asian ideograph
        0x21307d => [0x4f0a, false], # East Asian ideograph
        0x215b28 => [0x8e8a, false], # East Asian ideograph
        0x215b29 => [0x8e8d, false], # East Asian ideograph (variant of 4B5B29 which maps to 8E8D)
        0x275048 => [0x5e18, false], # East Asian ideograph
        0x275b2a => [0x8e2f, false], # East Asian ideograph
        0x4b6044 => [0x9818, false], # East Asian ideograph
        0x4d4a6c => [0x9667, false], # East Asian ideograph
        0x275b2b => [0x8e51, false], # East Asian ideograph
        0x6f526a => [0xc0bf, false], # Korean hangul
        0x293b7a => [0x8f8f, false], # East Asian ideograph
        0x215a68 => [0x8e10, false], # East Asian ideograph
        0x23404d => [0x9156, false], # East Asian ideograph
        0x215b2d => [0x8eab, false], # East Asian ideograph
        0x235b2e => [0x9d8a, false], # East Asian ideograph
        0x3f5f34 => [0x9699, false], # East Asian ideograph (not in Unicode)
        0x274224 => [0x6361, false], # East Asian ideograph
        0x212320 => [0x3000, false], # Ideographic space in some implementations
        0x215b30 => [0x8eba, false], # East Asian ideograph
        0x222323 => [0x5c63, false], # East Asian ideograph
        0x232324 => [0x8432, false], # East Asian ideograph
        0x225772 => [0x735e, false], # East Asian ideograph
        0x212328 => [0xff08, false], # Ideographic left parenthesis
        0x222329 => [0x5c67, false], # East Asian ideograph
        0x22232b => [0x5c68, false], # East Asian ideograph
        0x23232d => [0x842a, false], # East Asian ideograph
        0x23232e => [0x8429, false], # East Asian ideograph
        0x222330 => [0x5c6d, false], # East Asian ideograph
        0x222331 => [0x5c6e, false], # East Asian ideograph
        0x232332 => [0x8471, false], # East Asian ideograph
        0x215b33 => [0x8ecb, false], # East Asian ideograph
        0x232335 => [0x845f, false], # East Asian ideograph
        0x232336 => [0x8460, false], # East Asian ideograph
        0x222337 => [0x5c74, false], # East Asian ideograph
        0x222339 => [0x5c73, false], # East Asian ideograph
        0x23233a => [0x8446, false], # East Asian ideograph
        0x22233b => [0x5c77, false], # East Asian ideograph
        0x22233c => [0x5c7a, false], # East Asian ideograph
        0x29233d => [0x836e, false], # East Asian ideograph
        0x235b35 => [0x9d87, false], # East Asian ideograph
        0x222340 => [0x5c7c, false], # East Asian ideograph
        0x6f526c => [0xc0c1, false], # Korean hangul
        0x232345 => [0x844e, false], # East Asian ideograph
        0x222346 => [0x5c8f, false], # East Asian ideograph
        0x29473c => [0x9568, false], # East Asian ideograph
        0x222349 => [0x5c88, false], # East Asian ideograph
        0x275b37 => [0x8f6b, false], # East Asian ideograph
        0x22234d => [0x5c99, false], # East Asian ideograph
        0x232350 => [0x84a1, false], # East Asian ideograph
        0x225b38 => [0x7480, false], # East Asian ideograph
        0x232353 => [0x849f, false], # East Asian ideograph
        0x222355 => [0x5ca6, false], # East Asian ideograph
        0x232356 => [0x84ba, false], # East Asian ideograph
        0x222357 => [0x5ca0, false], # East Asian ideograph
        0x23515c => [0x993b, false], # East Asian ideograph
        0x69255e => [0x30de, false], # Katakana letter MA
        0x22235c => [0x5ca2, false], # East Asian ideograph
        0x275b3a => [0x8f72, false], # East Asian ideograph
        0x23235e => [0x84c1, false], # East Asian ideograph
        0x23235f => [0x84bb, false], # East Asian ideograph
        0x222360 => [0x5cb5, false], # East Asian ideograph
        0x222361 => [0x5ca7, false], # East Asian ideograph
        0x215b3b => [0x8ef8, false], # East Asian ideograph
        0x222366 => [0x5ca8, false], # East Asian ideograph
        0x222367 => [0x5cac, false], # East Asian ideograph
        0x234050 => [0x9158, false], # East Asian ideograph
        0x225b3c => [0x7481, false], # East Asian ideograph
        0x22236b => [0x5ca3, false], # East Asian ideograph
        0x22236c => [0x5cb6, false], # East Asian ideograph
        0x22236d => [0x5cc1, false], # East Asian ideograph
        0x23456e => [0x9351, false], # East Asian ideograph
        0x22236f => [0x5cad, false], # East Asian ideograph
        0x232370 => [0x84b1, false], # East Asian ideograph
        0x232371 => [0x849d, false], # East Asian ideograph
        0x232372 => [0x84d0, false], # East Asian ideograph
        0x224666 => [0x6c2d, false], # East Asian ideograph
        0x232375 => [0x8494, false], # East Asian ideograph
        0x222378 => [0x5cd3, false], # East Asian ideograph
        0x232379 => [0x84c7, false], # East Asian ideograph
        0x23237a => [0x84bd, false], # East Asian ideograph
        0x215b3f => [0x8f09, false], # East Asian ideograph
        0x23237c => [0x84c2, false], # East Asian ideograph
        0x6f526e => [0xc0c8, false], # Korean hangul
        0x282569 => [0x5d02, false], # East Asian ideograph
        0x225b40 => [0x7497, false], # East Asian ideograph
        0x29473e => [0x94f9, false], # East Asian ideograph
        0x23572e => [0x9b93, false], # East Asian ideograph
        0x275b41 => [0x8f85, false], # East Asian ideograph
        0x215b42 => [0x8f12, false], # East Asian ideograph
        0x27504d => [0x7b79, false], # East Asian ideograph
        0x224d5f => [0x6f72, false], # East Asian ideograph
        0x225b43 => [0x7498, false], # East Asian ideograph
        0x6f5b44 => [0xd22d, false], # Korean hangul
        0x6f526f => [0xc0c9, false], # Korean hangul
        0x2e6c46 => [0x7be6, false], # East Asian ideograph
        0x225b45 => [0x749a, false], # East Asian ideograph
        0x284340 => [0x67a5, false], # East Asian ideograph
        0x692526 => [0x30a6, false], # Katakana letter U
        0x275b46 => [0x8f86, false], # East Asian ideograph
        0x2d573b => [0x60f7, false], # East Asian ideograph
        0x215b47 => [0x8f1f, false], # East Asian ideograph
        0x275b48 => [0x8f89, false], # East Asian ideograph
        0x275b49 => [0x8f88, false], # East Asian ideograph
        0x6f5270 => [0xc0cc, false], # Korean hangul
        0x27615b => [0x810f, false], # East Asian ideograph
        0x275b4a => [0x8f6e, false], # East Asian ideograph
        0x234a65 => [0x9688, false], # East Asian ideograph
        0x6f5845 => [0xc9f9, false], # Korean hangul
        0x215b4b => [0x8f1c, false], # East Asian ideograph
        0x33422a => [0x62e1, false], # East Asian ideograph
        0x275b4c => [0x8f90, false], # East Asian ideograph
        0x225b4d => [0x74a4, false], # East Asian ideograph
        0x235160 => [0x993a, false], # East Asian ideograph
        0x275b4e => [0x8f93, false], # East Asian ideograph
        0x6f516b => [0xbe1d, false], # Korean hangul
        0x276131 => [0x9a6f, false], # East Asian ideograph
        0x215b4f => [0x8f44, false], # East Asian ideograph
        0x225a2b => [0x7424, false], # East Asian ideograph
        0x2d3b40 => [0x5c06, false], # East Asian ideograph (variant of 273B40 which maps to 5C06)
        0x275b51 => [0x8f95, false], # East Asian ideograph
        0x28544f => [0x70ec, false], # East Asian ideograph
        0x224d62 => [0x6f57, false], # East Asian ideograph
        0x29337a => [0x8bcc, false], # East Asian ideograph
        0x235161 => [0x9941, false], # East Asian ideograph
        0x275b53 => [0x8206, false], # East Asian ideograph
        0x215b54 => [0x8f4d, false], # East Asian ideograph
        0x692533 => [0x30b3, false], # Katakana letter KO
        0x4b316a => [0x723c, false], # East Asian ideograph
        0x215b55 => [0x8f49, false], # East Asian ideograph
        0x2d3f31 => [0x6159, false], # East Asian ideograph
        0x6f5665 => [0xc74a, false], # Korean hangul
        0x225b56 => [0x748d, false], # East Asian ideograph
        0x224667 => [0x6c30, false], # East Asian ideograph
        0x215b57 => [0x8f4e, false], # East Asian ideograph
        0x275b58 => [0x8f70, false], # East Asian ideograph
        0x213c71 => [0x5e96, false], # East Asian ideograph
        0x275b59 => [0x8f94, false], # East Asian ideograph
        0x234056 => [0x9164, false], # East Asian ideograph
        0x225a2d => [0x742d, false], # East Asian ideograph
        0x6f4c56 => [0xb289, false], # Korean hangul
        0x232421 => [0x8495, false], # East Asian ideograph
        0x692422 => [0x3042, false], # Hiragana letter A
        0x692423 => [0x3043, false], # Hiragana letter small I
        0x692424 => [0x3044, false], # Hiragana letter I
        0x692425 => [0x3045, false], # Hiragana letter small U
        0x222426 => [0x5ce0, false], # East Asian ideograph
        0x232427 => [0x84af, false], # East Asian ideograph
        0x222428 => [0x5cd2, false], # East Asian ideograph
        0x232429 => [0x84ad, false], # East Asian ideograph
        0x69242a => [0x304a, false], # Hiragana letter O
        0x22242b => [0x5ccb, false], # East Asian ideograph
        0x69242c => [0x304c, false], # Hiragana letter GA
        0x69242d => [0x304d, false], # Hiragana letter KI
        0x69242e => [0x304e, false], # Hiragana letter GI
        0x215b5d => [0x8fa3, false], # East Asian ideograph
        0x222430 => [0x5cc7, false], # East Asian ideograph
        0x222431 => [0x5cdc, false], # East Asian ideograph
        0x232432 => [0x84a8, false], # East Asian ideograph
        0x232433 => [0x84d6, false], # East Asian ideograph
        0x222434 => [0x5d00, false], # East Asian ideograph
        0x215b5e => [0x8fa8, false], # East Asian ideograph
        0x213859 => [0x5875, false], # East Asian ideograph
        0x692437 => [0x3057, false], # Hiragana letter SI
        0x692438 => [0x3058, false], # Hiragana letter ZI
        0x692439 => [0x3059, false], # Hiragana letter SU
        0x23243a => [0x8493, false], # East Asian ideograph
        0x22243b => [0x5cff, false], # East Asian ideograph
        0x22243c => [0x5ceb, false], # East Asian ideograph
        0x69243d => [0x305d, false], # Hiragana letter SO
        0x69243e => [0x305e, false], # Hiragana letter ZO
        0x23243f => [0x84cf, false], # East Asian ideograph
        0x692440 => [0x3060, false], # Hiragana letter DA
        0x232441 => [0x84ca, false], # East Asian ideograph
        0x692442 => [0x3062, false], # Hiragana letter DI
        0x692443 => [0x3063, false], # Hiragana letter small TU
        0x692444 => [0x3064, false], # Hiragana letter TU
        0x692445 => [0x3065, false], # Hiragana letter DU
        0x232446 => [0x8506, false], # East Asian ideograph
        0x232447 => [0x850b, false], # East Asian ideograph
        0x692448 => [0x3068, false], # Hiragana letter TO
        0x222449 => [0x5d1e, false], # East Asian ideograph
        0x22244a => [0x5d12, false], # East Asian ideograph
        0x69244b => [0x306b, false], # Hiragana letter NI
        0x69244c => [0x306c, false], # Hiragana letter NU
        0x23244d => [0x8500, false], # East Asian ideograph
        0x69244e => [0x306e, false], # Hiragana letter NO
        0x69244f => [0x306f, false], # Hiragana letter HA
        0x222450 => [0x5d1a, false], # East Asian ideograph
        0x692451 => [0x3071, false], # Hiragana letter PA
        0x222452 => [0x5d0c, false], # East Asian ideograph
        0x222453 => [0x5d20, false], # East Asian ideograph
        0x222454 => [0x5d21, false], # East Asian ideograph
        0x692455 => [0x3075, false], # Hiragana letter HU
        0x692456 => [0x3076, false], # Hiragana letter BU
        0x222457 => [0x5d27, false], # East Asian ideograph
        0x222458 => [0x5d0d, false], # East Asian ideograph
        0x232459 => [0x851f, false], # East Asian ideograph
        0x22245a => [0x5d26, false], # East Asian ideograph
        0x69245b => [0x307b, false], # Hiragana letter HO
        0x23245c => [0x853b, false], # East Asian ideograph
        0x22245d => [0x5d2e, false], # East Asian ideograph
        0x69245e => [0x307e, false], # Hiragana letter MA
        0x23245f => [0x84ea, false], # East Asian ideograph
        0x692460 => [0x3080, false], # Hiragana letter MU
        0x692461 => [0x3081, false], # Hiragana letter ME
        0x692462 => [0x3082, false], # Hiragana letter MO
        0x692463 => [0x3083, false], # Hiragana letter small YA
        0x692464 => [0x3084, false], # Hiragana letter YA
        0x235b66 => [0x9da4, false], # East Asian ideograph
        0x232466 => [0x84f4, false], # East Asian ideograph
        0x692467 => [0x3087, false], # Hiragana letter small YO
        0x692468 => [0x3088, false], # Hiragana letter YO
        0x222469 => [0x5d24, false], # East Asian ideograph
        0x23246a => [0x850c, false], # East Asian ideograph
        0x215b67 => [0x8fc5, false], # East Asian ideograph
        0x69246c => [0x308c, false], # Hiragana letter RE
        0x69246d => [0x308d, false], # Hiragana letter RO
        0x69246e => [0x308e, false], # Hiragana letter small WA
        0x69246f => [0x308f, false], # Hiragana letter WA
        0x692470 => [0x3090, false], # Hiragana letter WI
        0x222471 => [0x5d36, false], # East Asian ideograph
        0x222472 => [0x5d3e, false], # East Asian ideograph
        0x692473 => [0x3093, false], # Hiragana letter N
        0x222474 => [0x5d4b, false], # East Asian ideograph
        0x232475 => [0x8515, false], # East Asian ideograph
        0x222476 => [0x5d57, false], # East Asian ideograph
        0x222477 => [0x5d34, false], # East Asian ideograph
        0x6f2478 => [0x3155, false], # Korean hangul
        0x335e2f => [0x5257, false], # East Asian ideograph
        0x23247a => [0x84fc, false], # East Asian ideograph
        0x6f247b => [0x3158, false], # Korean hangul
        0x23247c => [0x84eb, false], # East Asian ideograph
        0x23247d => [0x84fd, false], # East Asian ideograph
        0x6f247e => [0x315b, false], # Korean hangul
        0x235b6b => [0x9d9a, false], # East Asian ideograph
        0x235166 => [0x993c, false], # East Asian ideograph
        0x225b6c => [0x74a5, false], # East Asian ideograph
        0x6f5277 => [0xc0dd, false], # Korean hangul
        0x6f4c31 => [0xb179, false], # Korean hangul
        0x215b6d => [0x8ff0, false], # East Asian ideograph (variant of 275B6D which maps to 8FF0)
        0x21385c => [0x5885, false], # East Asian ideograph
        0x69252e => [0x30ae, false], # Katakana letter GI
        0x225b6e => [0x74a8, false], # East Asian ideograph
        0x235e30 => [0x9ec1, false], # East Asian ideograph
        0x295921 => [0x9cd9, false], # East Asian ideograph
        0x6f5b6f => [0xd310, false], # Korean hangul
        0x295854 => [0x9cc4, false], # East Asian ideograph
        0x215b70 => [0x8fea, false], # East Asian ideograph
        0x705b71 => [0x57b4, false], # East Asian ideograph
        0x6f5278 => [0xc0e4, false], # Korean hangul
        0x276163 => [0x677e, false], # East Asian ideograph (duplicate simplified)
        0x6f4e35 => [0xb5b0, false], # Korean hangul
        0x69252f => [0x30af, false], # Katakana letter KU
        0x234b66 => [0x96d8, false], # East Asian ideograph
        0x235b74 => [0x9db1, false], # East Asian ideograph
        0x6f4b7c => [0xb123, false], # Korean hangul
        0x4b6053 => [0x985e, false], # East Asian ideograph
        0x224926 => [0x6d6f, false], # East Asian ideograph
        0x235b76 => [0x9db6, false], # East Asian ideograph
        0x234621 => [0x93b5, false], # East Asian ideograph
        0x276164 => [0x80e1, false], # East Asian ideograph (duplicate simplified)
        0x235b77 => [0x9dbc, false], # East Asian ideograph
        0x336275 => [0x76bc, false], # East Asian ideograph
        0x234623 => [0x9388, false], # East Asian ideograph
        0x295b79 => [0x9e5a, false], # East Asian ideograph
        0x51496b => [0x852b, false], # East Asian ideograph
        0x215b7a => [0x9003, false], # East Asian ideograph
        0x234625 => [0x93b9, false], # East Asian ideograph
        0x215b7b => [0x8ffd, false], # East Asian ideograph
        0x6f5173 => [0xbe54, false], # Korean hangul
        0x6f527a => [0xc0e8, false], # Korean hangul
        0x276165 => [0x987b, false], # East Asian ideograph (duplicate simplified)
        0x235b7c => [0x9dba, false], # East Asian ideograph
        0x21385f => [0x5880, false], # East Asian ideograph
        0x234627 => [0x93a1, false], # East Asian ideograph
        0x215635 => [0x85e5, false], # East Asian ideograph
        0x275b7d => [0x8fd9, false], # East Asian ideograph
        0x234628 => [0x93b0, false], # East Asian ideograph
        0x235b7e => [0x9dcf, false], # East Asian ideograph
        0x234629 => [0x93a3, false], # East Asian ideograph
        0x21462a => [0x6b77, false], # East Asian ideograph
        0x224928 => [0x6d61, false], # East Asian ideograph
        0x23462b => [0x939b, false], # East Asian ideograph
        0x276166 => [0x9b13, false], # East Asian ideograph
        0x4b3f50 => [0x61ca, false], # East Asian ideograph (variant of 213F50)
        0x22462c => [0x6bf3, false], # East Asian ideograph
        0x692532 => [0x30b2, false], # Katakana letter GE
        0x23462d => [0x9398, false], # East Asian ideograph
        0x213238 => [0x5075, false], # East Asian ideograph
        0x282626 => [0x5cc4, false], # East Asian ideograph
        0x4b462e => [0x6b81, false], # East Asian ideograph
        0x2f5f45 => [0x86a1, false], # East Asian ideograph
        0x2d4b22 => [0x736a, false], # East Asian ideograph
        0x29576e => [0x9ca7, false], # East Asian ideograph
        0x692521 => [0x30a1, false], # Katakana letter small A
        0x692522 => [0x30a2, false], # Katakana letter A
        0x276167 => [0x6597, false], # East Asian ideograph
        0x232524 => [0x851e, false], # East Asian ideograph
        0x222525 => [0x5d3f, false], # East Asian ideograph
        0x222526 => [0x5d52, false], # East Asian ideograph
        0x222527 => [0x5d3d, false], # East Asian ideograph
        0x222528 => [0x5d4e, false], # East Asian ideograph
        0x692529 => [0x30a9, false], # Katakana letter small O
        0x23252a => [0x8518, false], # East Asian ideograph
        0x69252b => [0x30ab, false], # Katakana letter KA
        0x22252c => [0x5d59, false], # East Asian ideograph
        0x23252d => [0x8526, false], # East Asian ideograph
        0x23252e => [0x8507, false], # East Asian ideograph (variant of 2F252E which maps to 8507)
        0x22252f => [0x5d32, false], # East Asian ideograph
        0x692530 => [0x30b0, false], # Katakana letter GU
        0x222531 => [0x5d42, false], # East Asian ideograph
        0x4c2532 => [0x5d5b, false], # East Asian ideograph
        0x224633 => [0x6bf8, false], # East Asian ideograph
        0x232534 => [0x84f0, false], # East Asian ideograph
        0x232535 => [0x84ef, false], # East Asian ideograph
        0x232536 => [0x8556, false], # East Asian ideograph
        0x692537 => [0x30b7, false], # Katakana letter SI
        0x692538 => [0x30b8, false], # Katakana letter ZI
        0x222539 => [0x5d6f, false], # East Asian ideograph
        0x22253a => [0x5d6b, false], # East Asian ideograph
        0x696136 => [0x753c, false], # East Asian ideograph
        0x69253c => [0x30bc, false], # Katakana letter ZE
        0x69253d => [0x30bd, false], # Katakana letter SO
        0x69253e => [0x30be, false], # Katakana letter ZO
        0x274635 => [0x6b87, false], # East Asian ideograph
        0x692540 => [0x30c0, false], # Katakana letter DA
        0x276168 => [0x95f9, false], # East Asian ideograph
        0x692542 => [0x30c2, false], # Katakana letter DI
        0x692543 => [0x30c3, false], # Katakana letter small TU
        0x222544 => [0x5d4a, false], # East Asian ideograph
        0x225e7a => [0x7602, false], # East Asian ideograph
        0x232546 => [0x8541, false], # East Asian ideograph
        0x692534 => [0x30b4, false], # Katakana letter GO
        0x692548 => [0x30c8, false], # Katakana letter TO
        0x222549 => [0x5d6c, false], # East Asian ideograph
        0x22254a => [0x5d62, false], # East Asian ideograph
        0x23254b => [0x8558, false], # East Asian ideograph
        0x69254c => [0x30cc, false], # Katakana letter NU
        0x22254d => [0x5d82, false], # East Asian ideograph
        0x23254e => [0x8561, false], # East Asian ideograph
        0x23254f => [0x8540, false], # East Asian ideograph
        0x222550 => [0x5d79, false], # East Asian ideograph
        0x224638 => [0x6bf9, false], # East Asian ideograph
        0x692552 => [0x30d2, false], # Katakana letter HI
        0x692553 => [0x30d3, false], # Katakana letter BI
        0x692554 => [0x30d4, false], # Katakana letter PI
        0x287231 => [0x7f03, false], # East Asian ideograph
        0x692556 => [0x30d6, false], # Katakana letter BU
        0x33516d => [0x6374, false], # East Asian ideograph
        0x4d222a => [0x83b5, false], # East Asian ideograph
        0x692559 => [0x30d9, false], # Katakana letter BE
        0x22255a => [0x5d81, false], # East Asian ideograph
        0x69255b => [0x30db, false], # Katakana letter HO
        0x23255c => [0x8564, false], # East Asian ideograph
        0x23255d => [0x855e, false], # East Asian ideograph
        0x23255e => [0x8573, false], # East Asian ideograph
        0x23255f => [0x8551, false], # East Asian ideograph
        0x222560 => [0x5d7e, false], # East Asian ideograph
        0x692561 => [0x30e1, false], # Katakana letter ME
        0x692562 => [0x30e2, false], # Katakana letter MO
        0x27463b => [0x6740, false], # East Asian ideograph
        0x232564 => [0x8562, false], # East Asian ideograph
        0x292535 => [0x82c1, false], # East Asian ideograph
        0x222566 => [0x5d92, false], # East Asian ideograph
        0x292567 => [0x836c, false], # East Asian ideograph
        0x222568 => [0x5d99, false], # East Asian ideograph
        0x27463c => [0x58f3, false], # East Asian ideograph
        0x22256a => [0x5da2, false], # East Asian ideograph
        0x23256b => [0x8563, false], # East Asian ideograph
        0x23256c => [0x848d, false], # East Asian ideograph
        0x23256d => [0x8542, false], # East Asian ideograph
        0x69256e => [0x30ee, false], # Katakana letter small WA
        0x23463d => [0x93a4, false], # East Asian ideograph
        0x692570 => [0x30f0, false], # Katakana letter WI
        0x232571 => [0x854e, false], # East Asian ideograph
        0x692572 => [0x30f2, false], # Katakana letter WO
        0x222573 => [0x5da1, false], # East Asian ideograph
        0x232574 => [0x8555, false], # East Asian ideograph
        0x222575 => [0x5d93, false], # East Asian ideograph
        0x232576 => [0x855d, false], # East Asian ideograph
        0x222577 => [0x5da0, false], # East Asian ideograph
        0x4b3e40 => [0x6046, false], # East Asian ideograph
        0x22257b => [0x5d94, false], # East Asian ideograph
        0x27616a => [0x90c1, false], # East Asian ideograph
        0x22257e => [0x5dac, false], # East Asian ideograph
        0x6f4e3c => [0xb5c0, false], # Korean hangul
        0x284350 => [0x68c2, false], # East Asian ideograph
        0x234640 => [0x93bc, false], # East Asian ideograph
        0x692536 => [0x30b6, false], # Katakana letter ZA
        0x513421 => [0x91d6, false], # East Asian ideograph
        0x224642 => [0x6bff, false], # East Asian ideograph
        0x3f5f49 => [0x7431, false], # East Asian ideograph
        0x29585c => [0x9cc7, false], # East Asian ideograph
        0x6f5c65 => [0xd55c, false], # Korean hangul
        0x224644 => [0x6c06, false], # East Asian ideograph
        0x276134 => [0x9a7c, false], # East Asian ideograph
        0x28656a => [0x789b, false], # East Asian ideograph
        0x213865 => [0x58c5, false], # East Asian ideograph
        0x294750 => [0x950e, false], # East Asian ideograph
        0x4b3178 => [0x5029, false], # East Asian ideograph (variant of 213178 which maps to 5029)
        0x234647 => [0x93a6, false], # East Asian ideograph
        0x29337d => [0x8c27, false], # East Asian ideograph
        0x224648 => [0x6c04, false], # East Asian ideograph
        0x22492e => [0x6d8a, false], # East Asian ideograph
        0x453755 => [0x56ae, false], # East Asian ideograph
        0x2d516a => [0x7db3, false], # East Asian ideograph
        0x6f4e3e => [0xb5cc, false], # Korean hangul
        0x23464a => [0x93aa, false], # East Asian ideograph
        0x294751 => [0x950f, false], # East Asian ideograph
        0x33627d => [0x6589, false], # East Asian ideograph
        0x213423 => [0x5291, false], # East Asian ideograph
        0x235060 => [0x98e3, false], # East Asian ideograph
        0x333c21 => [0x7895, false], # East Asian ideograph
        0x6f5748 => [0xc84c, false], # Korean hangul
        0x295153 => [0x9967, false], # East Asian ideograph
        0x22464c => [0x6c08, false], # East Asian ideograph
        0x23464d => [0x939e, false], # East Asian ideograph
        0x213c74 => [0x5ea0, false], # East Asian ideograph
        0x6f4e3f => [0xb5cf, false], # Korean hangul
        0x23464f => [0x9397, false], # East Asian ideograph
        0x692539 => [0x30b9, false], # Katakana letter SU
        0x27727a => [0x54d5, false], # East Asian ideograph
        0x234651 => [0x93bb, false], # East Asian ideograph
        0x295825 => [0x9cba, false], # East Asian ideograph
        0x224d73 => [0x6fb6, false], # East Asian ideograph
        0x224652 => [0x6c0d, false], # East Asian ideograph
        0x234653 => [0x93f1, false], # East Asian ideograph
        0x225851 => [0x739e, false], # East Asian ideograph
        0x6f4e40 => [0xb5d1, false], # Korean hangul
        0x213868 => [0x58d5, false], # East Asian ideograph
        0x69253a => [0x30ba, false], # Katakana letter ZU
        0x274655 => [0x6c14, false], # East Asian ideograph
        0x234656 => [0x93de, false], # East Asian ideograph
        0x234657 => [0x93ee, false], # East Asian ideograph
        0x234d30 => [0x976e, false], # East Asian ideograph
        0x274658 => [0x6c22, false], # East Asian ideograph
        0x27384a => [0x573a, false], # East Asian ideograph
        0x223244 => [0x63e5, false], # East Asian ideograph
        0x224659 => [0x6c15, false], # East Asian ideograph
        0x51563f => [0x8616, false], # East Asian ideograph
        0x23465a => [0x93c7, false], # East Asian ideograph
        0x23465b => [0x93f2, false], # East Asian ideograph
        0x232625 => [0x8580, false], # East Asian ideograph
        0x222626 => [0x5da7, false], # East Asian ideograph
        0x232628 => [0x858f, false], # East Asian ideograph
        0x22465c => [0x6c1a, false], # East Asian ideograph
        0x22262a => [0x5db0, false], # East Asian ideograph
        0x23262d => [0x8579, false], # East Asian ideograph
        0x22262e => [0x5db4, false], # East Asian ideograph
        0x23465d => [0x93d4, false], # East Asian ideograph
        0x222630 => [0x5db6, false], # East Asian ideograph
        0x232632 => [0x857f, false], # East Asian ideograph
        0x232633 => [0x8577, false], # East Asian ideograph
        0x232634 => [0x8578, false], # East Asian ideograph
        0x22465e => [0x6c1d, false], # East Asian ideograph
        0x222636 => [0x5db7, false], # East Asian ideograph
        0x6f4c37 => [0xb18b, false], # Korean hangul
        0x2d6132 => [0x99ee, false], # East Asian ideograph
        0x23263d => [0x85a4, false], # East Asian ideograph
        0x22263e => [0x5dc3, false], # East Asian ideograph
        0x224660 => [0x6c20, false], # East Asian ideograph
        0x232642 => [0x857a, false], # East Asian ideograph
        0x222644 => [0x5dc7, false], # East Asian ideograph
        0x232645 => [0x8557, false], # East Asian ideograph
        0x222646 => [0x5dc9, false], # East Asian ideograph
        0x222647 => [0x5dcb, false], # East Asian ideograph
        0x232649 => [0x85a8, false], # East Asian ideograph
        0x213d4f => [0x5f59, false], # East Asian ideograph
        0x234d32 => [0x9778, false], # East Asian ideograph
        0x224662 => [0x6c21, false], # East Asian ideograph
        0x22264e => [0x5dd8, false], # East Asian ideograph
        0x232650 => [0x8599, false], # East Asian ideograph
        0x232651 => [0x858a, false], # East Asian ideograph
        0x222652 => [0x5ddc, false], # East Asian ideograph
        0x234663 => [0x93ca, false], # East Asian ideograph
        0x232654 => [0x8590, false], # East Asian ideograph
        0x232656 => [0x8585, false], # East Asian ideograph
        0x232657 => [0x8588, false], # East Asian ideograph
        0x225a40 => [0x7447, false], # East Asian ideograph
        0x224664 => [0x6c2a, false], # East Asian ideograph
        0x23265a => [0x85b8, false], # East Asian ideograph
        0x6f5749 => [0xc870, false], # Korean hangul
        0x23265d => [0x85c1, false], # East Asian ideograph
        0x334665 => [0x6c61, false], # East Asian ideograph
        0x232661 => [0x85ba, false], # East Asian ideograph
        0x222662 => [0x5e00, false], # East Asian ideograph
        0x222664 => [0x51e7, false], # East Asian ideograph
        0x234666 => [0x93e8, false], # East Asian ideograph
        0x224934 => [0x6d79, false], # East Asian ideograph
        0x232668 => [0x85ce, false], # East Asian ideograph
        0x23266a => [0x85c2, false], # East Asian ideograph
        0x23266b => [0x85b7, false], # East Asian ideograph
        0x23266c => [0x85b9, false], # East Asian ideograph
        0x23266e => [0x85b3, false], # East Asian ideograph
        0x23266f => [0x85bd, false], # East Asian ideograph
        0x232670 => [0x85c4, false], # East Asian ideograph
        0x224668 => [0x6c2c, false], # East Asian ideograph
        0x222672 => [0x5e14, false], # East Asian ideograph
        0x222673 => [0x5e17, false], # East Asian ideograph
        0x232675 => [0x85be, false], # East Asian ideograph
        0x222676 => [0x5e19, false], # East Asian ideograph
        0x224669 => [0x6c31, false], # East Asian ideograph (not in Unicode)
        0x222678 => [0x5e1f, false], # East Asian ideograph
        0x22267a => [0x5e23, false], # East Asian ideograph
        0x22267b => [0x5e21, false], # East Asian ideograph
        0x23267e => [0x85b6, false], # East Asian ideograph
        0x295421 => [0x9aa3, false], # East Asian ideograph
        0x2d4647 => [0x6bd8, false], # East Asian ideograph
        0x284359 => [0x6989, false], # East Asian ideograph
        0x2d466d => [0x51b3, false], # East Asian ideograph
        0x294758 => [0x9561, false], # East Asian ideograph
        0x69253f => [0x30bf, false], # Katakana letter TA
        0x227c5b => [0x82d0, false], # East Asian ideograph
        0x28723c => [0x7f08, false], # East Asian ideograph
        0x224670 => [0x6c3b, false], # East Asian ideograph
        0x295422 => [0x9a81, false], # East Asian ideograph
        0x234d35 => [0x9773, false], # East Asian ideograph
        0x276174 => [0x9c7c, false], # East Asian ideograph
        0x234672 => [0x93da, false], # East Asian ideograph
        0x234673 => [0x93d0, false], # East Asian ideograph
        0x335e42 => [0x9452, false], # East Asian ideograph
        0x2d353c => [0x6b62, false], # East Asian ideograph
        0x234674 => [0x93ef, false], # East Asian ideograph
        0x6f4e37 => [0xb5b3, false], # Korean hangul
        0x4b4676 => [0x6c89, false], # East Asian ideograph
        0x213121 => [0x4f11, false], # East Asian ideograph
        0x276136 => [0x9a77, false], # East Asian ideograph
        0x21386f => [0x58e2, false], # East Asian ideograph
        0x223c6e => [0x68b2, false], # East Asian ideograph
        0x6f2472 => [0x314f, false], # Korean hangul
        0x224678 => [0x6c46, false], # East Asian ideograph
        0x6f5078 => [0xbc29, false], # Korean hangul
        0x28723e => [0x7f0c, false], # East Asian ideograph
        0x29364e => [0x8d33, false], # East Asian ideograph
        0x22467a => [0x6c52, false], # East Asian ideograph
        0x213125 => [0x4f01, false], # East Asian ideograph
        0x234d37 => [0x9783, false], # East Asian ideograph
        0x215f69 => [0x9739, false], # East Asian ideograph
        0x276176 => [0x9c81, false], # East Asian ideograph
        0x6f4e48 => [0xb69d, false], # Korean hangul
        0x23467c => [0x93cc, false], # East Asian ideograph
        0x6f574a => [0xc871, false], # Korean hangul
        0x224d7c => [0x6fc6, false], # East Asian ideograph
        0x23517b => [0x9954, false], # East Asian ideograph
        0x21312a => [0x4f4f, false], # East Asian ideograph
        0x234d38 => [0x977a, false], # East Asian ideograph
        0x213c76 => [0x5eab, false], # East Asian ideograph
        0x21312b => [0x4f4d, false], # East Asian ideograph
        0x6f4e49 => [0xb6a4, false], # Korean hangul
        0x213871 => [0x58e9, false], # East Asian ideograph
        0x21312c => [0x4f34, false], # East Asian ideograph
        0x6f594c => [0xcd18, false], # Korean hangul
        0x21342e => [0x52c3, false], # East Asian ideograph
        0x21312d => [0x4f47, false], # East Asian ideograph
        0x2d5758 => [0x890e, false], # East Asian ideograph
        0x21312f => [0x4f3a, false], # East Asian ideograph
        0x275b3f => [0x8f7d, false], # East Asian ideograph
        0x6f4f3d => [0xb86d, false], # Korean hangul
        0x28704a => [0x7ebe, false], # East Asian ideograph
        0x222722 => [0x5e22, false], # East Asian ideograph
        0x286577 => [0x789c, false], # East Asian ideograph
        0x222724 => [0x5e28, false], # East Asian ideograph
        0x213872 => [0x58eb, false], # East Asian ideograph
        0x232728 => [0x85f7, false], # East Asian ideograph
        0x6f5424 => [0xc2dd, false], # Korean hangul
        0x23272c => [0x85e6, false], # East Asian ideograph
        0x223132 => [0x6360, false], # East Asian ideograph
        0x23272e => [0x85d4, false], # East Asian ideograph
        0x232731 => [0x85ed, false], # East Asian ideograph
        0x6f5d42 => [0xd65c, false], # Korean hangul
        0x222735 => [0x5e44, false], # East Asian ideograph
        0x222736 => [0x5e43, false], # East Asian ideograph
        0x222739 => [0x5e42, false], # East Asian ideograph
        0x22273f => [0x5e4e, false], # East Asian ideograph
        0x6f4e4b => [0xb6ac, false], # Korean hangul
        0x232743 => [0x85df, false], # East Asian ideograph
        0x232745 => [0x85d8, false], # East Asian ideograph
        0x692545 => [0x30c5, false], # Katakana letter DU
        0x222747 => [0x5e58, false], # East Asian ideograph
        0x222748 => [0x5e48, false], # East Asian ideograph
        0x513b52 => [0x6c3d, false], # East Asian ideograph
        0x213137 => [0x4f3d, false], # East Asian ideograph
        0x23274c => [0x85dc, false], # East Asian ideograph
        0x23274e => [0x85f5, false], # East Asian ideograph
        0x273138 => [0x5e03, false], # East Asian ideograph
        0x232752 => [0x8622, false], # East Asian ideograph
        0x232754 => [0x8610, false], # East Asian ideograph
        0x285029 => [0x6edf, false], # East Asian ideograph
        0x232757 => [0x85fc, false], # East Asian ideograph
        0x222758 => [0x5e61, false], # East Asian ideograph
        0x23275b => [0x85ff, false], # East Asian ideograph
        0x23313a => [0x89d6, false], # East Asian ideograph
        0x23275e => [0x85fe, false], # East Asian ideograph
        0x22275f => [0x5e6c, false], # East Asian ideograph
        0x222760 => [0x5e6a, false], # East Asian ideograph
        0x222763 => [0x5e6e, false], # East Asian ideograph
        0x222764 => [0x5e6d, false], # East Asian ideograph
        0x222765 => [0x5e70, false], # East Asian ideograph
        0x232768 => [0x8604, false], # East Asian ideograph
        0x27313c => [0x5360, false], # East Asian ideograph
        0x227c6e => [0x8314, false], # East Asian ideograph
        0x22276d => [0x5e75, false], # East Asian ideograph
        0x232771 => [0x8605, false], # East Asian ideograph
        0x216757 => [0x50a3, false], # East Asian ideograph
        0x232775 => [0x862b, false], # East Asian ideograph
        0x213d51 => [0x5f62, false], # East Asian ideograph
        0x222777 => [0x5e80, false], # East Asian ideograph
        0x21313f => [0x4f5c, false], # East Asian ideograph
        0x22277e => [0x5e8b, false], # East Asian ideograph
        0x275d38 => [0x91ca, false], # East Asian ideograph
        0x294760 => [0x9563, false], # East Asian ideograph
        0x213432 => [0x52d2, false], # East Asian ideograph
        0x33572e => [0x880e, false], # East Asian ideograph
        0x2d3543 => [0x4edd, false], # East Asian ideograph
        0x27506f => [0x7ea0, false], # East Asian ideograph
        0x215b27 => [0x8e85, false], # East Asian ideograph
        0x213142 => [0x4f4e, false], # East Asian ideograph
        0x217d40 => [0x5b19, false], # East Asian ideograph
        0x213143 => [0x4f5d, false], # East Asian ideograph
        0x213c77 => [0x5ea7, false], # East Asian ideograph
        0x213144 => [0x4f36, false], # East Asian ideograph
        0x6f4b5c => [0xb0af, false], # Korean hangul
        0x6f4e4e => [0xb6f4, false], # Korean hangul
        0x213876 => [0x58fa, false], # East Asian ideograph
        0x223145 => [0x6335, false], # East Asian ideograph
        0x706067 => [0x567b, false], # East Asian ideograph
        0x223832 => [0x665f, false], # East Asian ideograph
        0x295828 => [0x9cb4, false], # East Asian ideograph
        0x233147 => [0x89e1, false], # East Asian ideograph
        0x29586e => [0x9ca5, false], # East Asian ideograph
        0x213148 => [0x4f8d, false], # East Asian ideograph
        0x275b40 => [0x8f7e, false], # East Asian ideograph
        0x6f4e4f => [0xb6f8, false], # Korean hangul
        0x275736 => [0x8747, false], # East Asian ideograph
        0x21314a => [0x4f7f, false], # East Asian ideograph
        0x692549 => [0x30c9, false], # Katakana letter DO
        0x2f5476 => [0x9ae1, false], # East Asian ideograph
        0x21314b => [0x4f9b, false], # East Asian ideograph
        0x275071 => [0x7ea3, false], # East Asian ideograph
        0x21314c => [0x4f86, false], # East Asian ideograph
        0x2d3730 => [0x751e, false], # East Asian ideograph
        0x21314d => [0x4f6c, false], # East Asian ideograph
        0x6f4e50 => [0xb700, false], # Korean hangul
        0x21314f => [0x4f96, false], # East Asian ideograph
        0x213435 => [0x52de, false], # East Asian ideograph
        0x233c33 => [0x8f47, false], # East Asian ideograph
        0x213151 => [0x4f83, false], # East Asian ideograph
        0x287247 => [0x7f11, false], # East Asian ideograph
        0x697152 => [0x99f2, false], # East Asian ideograph
        0x453768 => [0x5efb, false], # East Asian ideograph
        0x213153 => [0x4f88, false], # East Asian ideograph
        0x276138 => [0x9a79, false], # East Asian ideograph
        0x4b3f51 => [0x61d1, false], # East Asian ideograph
        0x6f4e51 => [0xb701, false], # Korean hangul
        0x213154 => [0x4f69, false], # East Asian ideograph
        0x69254b => [0x30cb, false], # Katakana letter NI
        0x213436 => [0x52db, false], # East Asian ideograph
        0x6f5826 => [0xc998, false], # Korean hangul
        0x275073 => [0x7eab, false], # East Asian ideograph
        0x354156 => [0x91be, false], # East Asian ideograph
        0x295871 => [0x9cce, false], # East Asian ideograph
        0x287248 => [0x7f0f, false], # East Asian ideograph
        0x4b606f => [0x991d, false], # East Asian ideograph
        0x233158 => [0x89f1, false], # East Asian ideograph
        0x294531 => [0x9528, false], # East Asian ideograph
        0x6f4e52 => [0xb728, false], # Korean hangul
        0x284366 => [0x6924, false], # East Asian ideograph
        0x217159 => [0x55d0, false], # East Asian ideograph
        0x225a4f => [0x7452, false], # East Asian ideograph
        0x21315a => [0x4faf, false], # East Asian ideograph
        0x232822 => [0x8627, false], # East Asian ideograph
        0x21715b => [0x55cd, false], # East Asian ideograph
        0x274c31 => [0x7544, false], # East Asian ideograph
        0x232826 => [0x8629, false], # East Asian ideograph
        0x23315c => [0x89f3, false], # East Asian ideograph
        0x224943 => [0x6d94, false], # East Asian ideograph
        0x213a61 => [0x5b7a, false], # East Asian ideograph
        0x21315d => [0x4fe0, false], # East Asian ideograph
        0x6f4b5d => [0xb0b1, false], # Korean hangul
        0x232832 => [0x8637, false], # East Asian ideograph
        0x395230 => [0x5bd8, false], # East Asian ideograph
        0x222835 => [0x5ea5, false], # East Asian ideograph
        0x222836 => [0x5eaf, false], # East Asian ideograph
        0x213438 => [0x52e2, false], # East Asian ideograph
        0x232838 => [0x8636, false], # East Asian ideograph
        0x21315f => [0x4fb6, false], # East Asian ideograph
        0x23283e => [0x863c, false], # East Asian ideograph
        0x23283f => [0x8640, false], # East Asian ideograph
        0x232840 => [0x863a, false], # East Asian ideograph
        0x233160 => [0x89f6, false], # East Asian ideograph
        0x222842 => [0x5eb9, false], # East Asian ideograph
        0x39365a => [0x8ae0, false], # East Asian ideograph
        0x227161 => [0x7da3, false], # East Asian ideograph
        0x22284b => [0x5eb3, false], # East Asian ideograph
        0x22284c => [0x5ec4, false], # East Asian ideograph
        0x217162 => [0x55dd, false], # East Asian ideograph
        0x6f4e54 => [0xb72c, false], # Korean hangul
        0x275d3f => [0x9488, false], # East Asian ideograph
        0x294767 => [0x94e7, false], # East Asian ideograph
        0x69254e => [0x30ce, false], # Katakana letter NO
        0x222855 => [0x5ecb, false], # East Asian ideograph
        0x222857 => [0x5ecd, false], # East Asian ideograph
        0x213164 => [0x4fdf, false], # East Asian ideograph
        0x22285a => [0x5ed2, false], # East Asian ideograph
        0x22285b => [0x5ed1, false], # East Asian ideograph
        0x22285c => [0x5ed5, false], # East Asian ideograph
        0x23285e => [0x8659, false], # East Asian ideograph
        0x22285f => [0x5ed4, false], # East Asian ideograph
        0x222860 => [0x5ed9, false], # East Asian ideograph
        0x222861 => [0x5ece, false], # East Asian ideograph
        0x21232d => [0xff0d, false], # Ideographic hyphen minus
        0x232866 => [0x8661, false], # East Asian ideograph
        0x4c2867 => [0x5edb, false], # East Asian ideograph
        0x222868 => [0x5ee1, false], # East Asian ideograph
        0x232869 => [0x8662, false], # East Asian ideograph
        0x23286a => [0x8663, false], # East Asian ideograph
        0x287167 => [0x7eef, false], # East Asian ideograph
        0x22286d => [0x5ee7, false], # East Asian ideograph
        0x232871 => [0x8669, false], # East Asian ideograph
        0x69254f => [0x30cf, false], # Katakana letter HA
        0x2d5b7a => [0x8fef, false], # East Asian ideograph
        0x217169 => [0x55e9, false], # East Asian ideograph
        0x232878 => [0x866c, false], # East Asian ideograph
        0x23287b => [0x8672, false], # East Asian ideograph
        0x22287c => [0x5eed, false], # East Asian ideograph
        0x21316a => [0x4fce, false], # East Asian ideograph
        0x23287e => [0x867b, false], # East Asian ideograph
        0x274c34 => [0x5f02, false], # East Asian ideograph
        0x23462a => [0x93b7, false], # East Asian ideograph
        0x21316b => [0x4fd7, false], # East Asian ideograph
        0x23316c => [0x8a06, false], # East Asian ideograph
        0x276139 => [0x9a78, false], # East Asian ideograph
        0x6f4e56 => [0xb730, false], # Korean hangul
        0x294769 => [0x9564, false], # East Asian ideograph
        0x6f507b => [0xbc31, false], # Korean hangul
        0x21316e => [0x500d, false], # East Asian ideograph
        0x4b5861 => [0x4f89, false], # East Asian ideograph
        0x21716f => [0x55cf, false], # East Asian ideograph
        0x213170 => [0x5026, false], # East Asian ideograph
        0x4b3e5b => [0x60c5, false], # East Asian ideograph
        0x213171 => [0x500c, false], # East Asian ideograph
        0x6f4e57 => [0xb738, false], # Korean hangul
        0x223172 => [0x639e, false], # East Asian ideograph
        0x692551 => [0x30d1, false], # Katakana letter PA
        0x21343c => [0x52f5, false], # East Asian ideograph
        0x273173 => [0x4eec, false], # East Asian ideograph
        0x4c4f24 => [0x6f46, false], # East Asian ideograph
        0x235b7a => [0x9dc1, false], # East Asian ideograph
        0x287174 => [0x7ef2, false], # East Asian ideograph
        0x274c36 => [0x753b, false], # East Asian ideograph
        0x39365e => [0x559e, false], # East Asian ideograph
        0x6f5b21 => [0xd0ed, false], # Korean hangul
        0x4b5c32 => [0x9038, false], # East Asian ideograph
        0x6f4b5e => [0xb0b3, false], # Korean hangul
        0x6f4e58 => [0xb739, false], # Korean hangul
        0x2d3177 => [0x5e78, false], # East Asian ideograph
        0x21343d => [0x52f8, false], # East Asian ideograph
        0x217178 => [0x55c1, false], # East Asian ideograph
        0x6f5c23 => [0xd37c, false], # Korean hangul
        0x273179 => [0x4fe9, false], # East Asian ideograph
        0x6f5c24 => [0xd37d, false], # Korean hangul
        0x29365f => [0x8d47, false], # East Asian ideograph
        0x6f5b22 => [0xd0ef, false], # Korean hangul
        0x6f5c25 => [0xd380, false], # Korean hangul
        0x21317b => [0x5012, false], # East Asian ideograph
        0x6f5c26 => [0xd384, false], # Korean hangul
        0x6f4e59 => [0xb73b, false], # Korean hangul
        0x4b317c => [0x5024, false], # East Asian ideograph
        0x235c27 => [0x9dc3, false], # East Asian ideograph
        0x22317d => [0x63ab, false], # East Asian ideograph
        0x215c28 => [0x901a, false], # East Asian ideograph
        0x4c4f26 => [0x6edd, false], # East Asian ideograph
        0x69717e => [0x9af7, false], # East Asian ideograph
        0x215c29 => [0x9020, false], # East Asian ideograph
        0x216764 => [0x5095, false], # East Asian ideograph
        0x6f5c2a => [0xd390, false], # Korean hangul
        0x6f5c2b => [0xd391, false], # Korean hangul
        0x6f4e5a => [0xb744, false], # Korean hangul
        0x6f5c2c => [0xd398, false], # Korean hangul
        0x695c2d => [0x6928, false], # East Asian ideograph
        0x4b372f => [0x5c1c, false], # East Asian ideograph
        0x274c39 => [0x5f53, false], # East Asian ideograph
        0x287251 => [0x7f1f, false], # East Asian ideograph
        0x6f5c6b => [0xd56c, false], # Korean hangul
        0x215c2f => [0x902e, false], # East Asian ideograph
        0x27613a => [0x9a7d, false], # East Asian ideograph
        0x225c30 => [0x74c8, false], # East Asian ideograph
        0x6f4e5b => [0xb748, false], # Korean hangul
        0x222923 => [0x5ef4, false], # East Asian ideograph
        0x232925 => [0x867a, false], # East Asian ideograph
        0x232926 => [0x8673, false], # East Asian ideograph
        0x225c31 => [0x74c5, false], # East Asian ideograph
        0x29432b => [0x94c6, false], # East Asian ideograph
        0x6f5828 => [0xc99d, false], # Korean hangul
        0x215c32 => [0xfa25, false], # East Asian ideograph
        0x23292e => [0x8696, false], # East Asian ideograph
        0x27507d => [0x7eaf, false], # East Asian ideograph
        0x217671 => [0x5827, false], # East Asian ideograph
        0x29333b => [0x8c00, false], # East Asian ideograph
        0x215c33 => [0x9032, false], # East Asian ideograph
        0x222935 => [0x5f07, false], # East Asian ideograph
        0x232936 => [0x8691, false], # East Asian ideograph
        0x232937 => [0x869c, false], # East Asian ideograph
        0x235c34 => [0x9dac, false], # East Asian ideograph
        0x22293a => [0x5f0b, false], # East Asian ideograph
        0x23293c => [0x868d, false], # East Asian ideograph
        0x23293d => [0x868b, false], # East Asian ideograph
        0x6f5c35 => [0xd3b5, false], # Korean hangul
        0x232940 => [0x86a6, false], # East Asian ideograph
        0x232942 => [0x869d, false], # East Asian ideograph
        0x39476f => [0x51c0, false], # East Asian ideograph
        0x235c36 => [0x9db2, false], # East Asian ideograph
        0x232946 => [0x86a0, false], # East Asian ideograph
        0x2d3f3a => [0x6185, false], # East Asian ideograph
        0x232948 => [0x86a7, false], # East Asian ideograph
        0x22294a => [0x5f28, false], # East Asian ideograph
        0x22294b => [0x5f22, false], # East Asian ideograph
        0x22294c => [0x5f23, false], # East Asian ideograph
        0x22294d => [0x5f24, false], # East Asian ideograph
        0x235b7b => [0x9db8, false], # East Asian ideograph
        0x225c38 => [0x74d6, false], # East Asian ideograph
        0x222952 => [0x5f30, false], # East Asian ideograph
        0x6f5a27 => [0xceec, false], # Korean hangul
        0x215c39 => [0x9054, false], # East Asian ideograph
        0x232958 => [0x86ba, false], # East Asian ideograph
        0x232959 => [0x86b0, false], # East Asian ideograph
        0x22295c => [0x5f40, false], # East Asian ideograph
        0x215c3a => [0x9055, false], # East Asian ideograph
        0x6f4e5d => [0xb764, false], # Korean hangul
        0x22295f => [0x5f44, false], # East Asian ideograph
        0x232960 => [0x86b3, false], # East Asian ideograph
        0x232962 => [0x86c9, false], # East Asian ideograph
        0x215c3b => [0x903c, false], # East Asian ideograph
        0x232967 => [0x86d8, false], # East Asian ideograph
        0x222968 => [0x5f50, false], # East Asian ideograph
        0x215c3c => [0x9047, false], # East Asian ideograph
        0x22296a => [0x5f56, false], # East Asian ideograph
        0x22296c => [0x5f58, false], # East Asian ideograph
        0x2d3e60 => [0x6075, false], # East Asian ideograph
        0x23296e => [0x86e3, false], # East Asian ideograph
        0x225c3d => [0x74d8, false], # East Asian ideograph
        0x222970 => [0x5f60, false], # East Asian ideograph
        0x232971 => [0x86ec, false], # East Asian ideograph
        0x222972 => [0x5f63, false], # East Asian ideograph
        0x222973 => [0x809c, false], # East Asian ideograph
        0x222974 => [0x5f67, false], # East Asian ideograph
        0x215c3e => [0x904e, false], # East Asian ideograph
        0x232977 => [0x86d0, false], # East Asian ideograph
        0x222978 => [0x5f72, false], # East Asian ideograph
        0x222979 => [0x5f73, false], # East Asian ideograph
        0x23297a => [0x86d1, false], # East Asian ideograph
        0x2d5c3f => [0x5fa7, false], # East Asian ideograph
        0x22297c => [0x5f74, false], # East Asian ideograph
        0x23297e => [0x86de, false], # East Asian ideograph
        0x225c40 => [0x74da, false], # East Asian ideograph
        0x213443 => [0x5308, false], # East Asian ideograph
        0x215c41 => [0x9041, false], # East Asian ideograph
        0x4c4f2b => [0x701e, false], # East Asian ideograph
        0x6f5c42 => [0xd3fd, false], # Korean hangul
        0x333251 => [0x5fba, false], # East Asian ideograph
        0x6f5b28 => [0xd138, false], # Korean hangul
        0x22494f => [0x6d96, false], # East Asian ideograph
        0x695c43 => [0x6981, false], # East Asian ideograph
        0x4b5c39 => [0x9039, false], # East Asian ideograph
        0x215c44 => [0x9060, false], # East Asian ideograph
        0x6f4e5f => [0xb770, false], # Korean hangul
        0x273b31 => [0x5b9e, false], # East Asian ideograph
        0x215c45 => [0x905c, false], # East Asian ideograph
        0x29432f => [0x94f3, false], # East Asian ideograph
        0x696325 => [0x7907, false], # East Asian ideograph
        0x235c46 => [0x9dde, false], # East Asian ideograph
        0x215c47 => [0x9065, false], # East Asian ideograph
        0x6f5b29 => [0xd140, false], # Korean hangul
        0x215c48 => [0x905e, false], # East Asian ideograph
        0x6f4e38 => [0xb5b4, false], # Korean hangul
        0x27613b => [0x9a87, false], # East Asian ideograph
        0x275c49 => [0x9002, false], # East Asian ideograph
        0x6f4e60 => [0xb771, false], # Korean hangul
        0x273b32 => [0x5b81, false], # East Asian ideograph
        0x29255a => [0x8487, false], # East Asian ideograph
        0x6f5c4a => [0xd479, false], # Korean hangul
        0x225a5d => [0x7440, false], # East Asian ideograph
        0x235e5c => [0x9ee7, false], # East Asian ideograph
        0x6f5c4b => [0xd47c, false], # Korean hangul
        0x2d3556 => [0x7343, false], # East Asian ideograph
        0x2d532c => [0x6bd3, false], # East Asian ideograph
        0x6f5c4c => [0xd480, false], # Korean hangul
        0x336321 => [0x6b6f, false], # East Asian ideograph
        0x6f5b2a => [0xd141, false], # Korean hangul
        0x215c4d => [0x9075, false], # East Asian ideograph
        0x6f4c3a => [0xb193, false], # Korean hangul
        0x6f5c4e => [0xd489, false], # Korean hangul
        0x6f4e61 => [0xb775, false], # Korean hangul
        0x294774 => [0x9571, false], # East Asian ideograph
        0x215c4f => [0x9078, false], # East Asian ideograph
        0x217435 => [0x571a, false], # East Asian ideograph
        0x215c50 => [0x9072, false], # East Asian ideograph
        0x4b3231 => [0x4eee, false], # East Asian ideograph
        0x275c51 => [0x8fc1, false], # East Asian ideograph
        0x6f5b2b => [0xd143, false], # Korean hangul
        0x275c52 => [0x8fbd, false], # East Asian ideograph
        0x215c53 => [0x907a, false], # East Asian ideograph
        0x4d503a => [0x98d1, false], # East Asian ideograph
        0x69255c => [0x30dc, false], # Katakana letter BO
        0x225c54 => [0x74e9, false], # East Asian ideograph
        0x217436 => [0x571b, false], # East Asian ideograph
        0x6f5c55 => [0xd508, false], # Korean hangul
        0x213a36 => [0x5abd, false], # East Asian ideograph
        0x215c56 => [0x9081, false], # East Asian ideograph
        0x6f5b2c => [0xd144, false], # Korean hangul
        0x455f35 => [0x9668, false], # East Asian ideograph (Version J extension)
        0x215c57 => [0x9084, false], # East Asian ideograph
        0x6f4f3e => [0xb86f, false], # Korean hangul
        0x6f5c33 => [0xd3ad, false], # Korean hangul
        0x225c58 => [0x74f1, false], # East Asian ideograph
        0x69255d => [0x30dd, false], # Katakana letter PO
        0x6f5c59 => [0xd53c, false], # Korean hangul
        0x215c5a => [0x9087, false], # East Asian ideograph
        0x212a21 => [0xe8d0, false], # EACC component character
        0x212a22 => [0xe8d1, false], # EACC component character
        0x215c5b => [0x908a, false], # East Asian ideograph
        0x212a24 => [0xe8d3, false], # EACC component character
        0x232a25 => [0x870b, false], # East Asian ideograph
        0x212a26 => [0xe8d5, false], # EACC component character
        0x222a27 => [0x5f89, false], # East Asian ideograph
        0x212a28 => [0xe8d6, false], # EACC component character
        0x215c5c => [0x9090, false], # East Asian ideograph
        0x212a2a => [0xe8d8, false], # EACC component character
        0x222a2b => [0x5f94, false], # East Asian ideograph
        0x212a2c => [0xe8da, false], # EACC component character
        0x212a2d => [0xe8db, false], # EACC component character
        0x69727e => [0x9d48, false], # East Asian ideograph
        0x215c5d => [0x908f, false], # East Asian ideograph
        0x2e284c => [0x5ecf, false], # East Asian ideograph
        0x6f4e64 => [0xb77c, false], # Korean hangul
        0x275d4f => [0x94b4, false], # East Asian ideograph
        0x232a33 => [0x86f8, false], # East Asian ideograph
        0x232a34 => [0x8706, false], # East Asian ideograph
        0x4b5c5e => [0x961d, false], # East Asian ideograph
        0x232a36 => [0x870e, false], # East Asian ideograph
        0x212a37 => [0xe8e4, false], # EACC component character
        0x232a38 => [0x8709, false], # East Asian ideograph
        0x222a39 => [0x5f9c, false], # East Asian ideograph
        0x232a3a => [0x870a, false], # East Asian ideograph
        0x235c5f => [0x9deb, false], # East Asian ideograph
        0x212a3c => [0xe8e9, false], # EACC component character
        0x222a3d => [0x5f9a, false], # East Asian ideograph
        0x232a3e => [0x870d, false], # East Asian ideograph
        0x212a3f => [0xe8ec, false], # EACC component character
        0x212a40 => [0xe8ed, false], # EACC component character
        0x6f5c60 => [0xd551, false], # Korean hangul
        0x232a42 => [0x874a, false], # East Asian ideograph
        0x232a43 => [0x8723, false], # East Asian ideograph
        0x232a44 => [0x8737, false], # East Asian ideograph
        0x232a45 => [0x8728, false], # East Asian ideograph
        0x222a46 => [0x5faf, false], # East Asian ideograph
        0x225c61 => [0x74f4, false], # East Asian ideograph
        0x232a49 => [0x8740, false], # East Asian ideograph
        0x232a4b => [0x872e, false], # East Asian ideograph
        0x232a4c => [0x873d, false], # East Asian ideograph
        0x232a4e => [0x871e, false], # East Asian ideograph
        0x6f4e65 => [0xb77d, false], # Korean hangul
        0x222a50 => [0x5fbc, false], # East Asian ideograph
        0x69255f => [0x30df, false], # Katakana letter MI
        0x232a53 => [0x8743, false], # East Asian ideograph
        0x232a55 => [0x8744, false], # East Asian ideograph
        0x6f507e => [0xbc38, false], # Korean hangul
        0x222a57 => [0x5fc9, false], # East Asian ideograph
        0x232a59 => [0x8729, false], # East Asian ideograph
        0x232a5a => [0x8739, false], # East Asian ideograph
        0x213241 => [0x5091, false], # East Asian ideograph
        0x232a5f => [0x871a, false], # East Asian ideograph
        0x222a61 => [0x5fd2, false], # East Asian ideograph
        0x222a63 => [0x5fd0, false], # East Asian ideograph
        0x232a64 => [0x8731, false], # East Asian ideograph
        0x232a65 => [0x8711, false], # East Asian ideograph
        0x232a66 => [0x8712, false], # East Asian ideograph
        0x222a67 => [0x5fce, false], # East Asian ideograph
        0x222a68 => [0x5fed, false], # East Asian ideograph
        0x6f4c3b => [0xb194, false], # Korean hangul
        0x232a6b => [0x874f, false], # East Asian ideograph
        0x232a6c => [0x8771, false], # East Asian ideograph
        0x232a6d => [0x8763, false], # East Asian ideograph
        0x275d51 => [0x94b8, false], # East Asian ideograph
        0x232a71 => [0x8764, false], # East Asian ideograph
        0x222a72 => [0x5fee, false], # East Asian ideograph
        0x232a73 => [0x8765, false], # East Asian ideograph
        0x232a74 => [0x877d, false], # East Asian ideograph
        0x6f5c69 => [0xd569, false], # Korean hangul
        0x222a78 => [0x5fe1, false], # East Asian ideograph
        0x232a79 => [0x8758, false], # East Asian ideograph
        0x222a7b => [0x5fe4, false], # East Asian ideograph
        0x215c6a => [0x90e1, false], # East Asian ideograph
        0x28725d => [0x7f1c, false], # East Asian ideograph
        0x6f5b30 => [0xd150, false], # Korean hangul
        0x275c6b => [0x5369, false], # East Asian ideograph
        0x3f516d => [0x6403, false], # East Asian ideograph
        0x235c6c => [0x9de6, false], # East Asian ideograph
        0x6f4e67 => [0xb784, false], # Korean hangul
        0x275d52 => [0x94c0, false], # East Asian ideograph
        0x215c6d => [0x90f5, false], # East Asian ideograph
        0x33303a => [0x8ffa, false], # East Asian ideograph
        0x6f5c6e => [0xd574, false], # Korean hangul
        0x6f5c6f => [0xd575, false], # Korean hangul
        0x28725e => [0x7f19, false], # East Asian ideograph
        0x6f5873 => [0xcc0c, false], # Korean hangul
        0x705f30 => [0x7519, false], # East Asian ideograph
        0x215c70 => [0x9109, false], # East Asian ideograph
        0x215c71 => [0x9112, false], # East Asian ideograph
        0x6f4e68 => [0xb78c, false], # Korean hangul
        0x275e68 => [0x9617, false], # East Asian ideograph
        0x4b5c72 => [0x9119, false], # East Asian ideograph (variant of 215C72 which maps to 9119)
        0x275153 => [0x7eac, false], # East Asian ideograph
        0x215c73 => [0x912d, false], # East Asian ideograph
        0x235a21 => [0x9d02, false], # East Asian ideograph
        0x215c74 => [0x9130, false], # East Asian ideograph
        0x28725f => [0x7f1b, false], # East Asian ideograph
        0x6f5b32 => [0xd15c, false], # Korean hangul
        0x224959 => [0x6dab, false], # East Asian ideograph
        0x215c75 => [0x9127, false], # East Asian ideograph
        0x6f5c76 => [0xd589, false], # Korean hangul
        0x2d4228 => [0x5117, false], # East Asian ideograph
        0x215c77 => [0x9139, false], # East Asian ideograph (variant of 4B5C77 which maps to 9139)
        0x455e60 => [0x95eb, false], # East Asian ideograph (Version J extension)
        0x6f5c78 => [0xd5a5, false], # Korean hangul
        0x235a22 => [0x9d03, false], # East Asian ideograph
        0x2f3d5d => [0x900e, false], # East Asian ideograph
        0x6f5c79 => [0xd5c8, false], # Korean hangul
        0x224724 => [0x6c5c, false], # East Asian ideograph
        0x6f5c7a => [0xd5c9, false], # Korean hangul
        0x27613d => [0x9a8b, false], # East Asian ideograph
        0x2e4a6b => [0x6ea6, false], # East Asian ideograph
        0x224726 => [0x6c5b, false], # East Asian ideograph
        0x275d55 => [0x94c5, false], # East Asian ideograph
        0x292564 => [0x8489, false], # East Asian ideograph
        0x224727 => [0x6c4d, false], # East Asian ideograph
        0x225c7d => [0x7507, false], # East Asian ideograph
        0x22474d => [0x6c93, false], # East Asian ideograph
        0x235a23 => [0x9cf7, false], # East Asian ideograph
        0x235c7e => [0x9dfd, false], # East Asian ideograph
        0x2d4729 => [0x6d29, false], # East Asian ideograph
        0x213d57 => [0x5f6d, false], # East Asian ideograph
        0x234d5a => [0x979f, false], # East Asian ideograph
        0x6f4c3c => [0xb1a8, false], # Korean hangul
        0x294532 => [0x9531, false], # East Asian ideograph
        0x22472b => [0x6c4b, false], # East Asian ideograph
        0x3f4a28 => [0x9df0, false], # East Asian ideograph
        0x225a68 => [0x7474, false], # East Asian ideograph
        0x6f7649 => [0xe8bb, false], # Korean hangul
        0x6f5751 => [0xc886, false], # Korean hangul
        0x22472d => [0x6c63, false], # East Asian ideograph
        0x6f5b35 => [0xd160, false], # Korean hangul
        0x4b3d2a => [0x5ee3, false], # East Asian ideograph
        0x23472f => [0x93a9, false], # East Asian ideograph
        0x28773f => [0x804d, false], # East Asian ideograph
        0x232b21 => [0x8761, false], # East Asian ideograph
        0x692535 => [0x30b5, false], # Katakana letter SA
        0x222b24 => [0x5fea, false], # East Asian ideograph
        0x692566 => [0x30e6, false], # Katakana letter YU
        0x212b26 => [0x300d, false], # Ideographic right corner bracket
        0x225a69 => [0x746e, false], # East Asian ideograph
        0x232b28 => [0x875f, false], # East Asian ideograph
        0x222b2a => [0x6026, false], # East Asian ideograph
        0x222b2c => [0x6029, false], # East Asian ideograph
        0x232b2d => [0x876f, false], # East Asian ideograph
        0x232b2e => [0x875d, false], # East Asian ideograph
        0x232b30 => [0x876e, false], # East Asian ideograph
        0x222b31 => [0x6008, false], # East Asian ideograph
        0x212b32 => [0xff3d, false], # Ideographic right square bracket
        0x224733 => [0x6c76, false], # East Asian ideograph
        0x212b34 => [0xff0e, false], # Ideographic variant full stop
        0x232b35 => [0x8753, false], # East Asian ideograph
        0x222b36 => [0x600a, false], # East Asian ideograph
        0x222b37 => [0x600c, false], # East Asian ideograph
        0x234d5c => [0x979a, false], # East Asian ideograph
        0x214734 => [0x6cbb, false], # East Asian ideograph
        0x232b3a => [0x87a3, false], # East Asian ideograph
        0x4b5e69 => [0x95a2, false], # East Asian ideograph
        0x222b3c => [0x6017, false], # East Asian ideograph
        0x232b3d => [0x8793, false], # East Asian ideograph
        0x6f245f => [0x3148, false], # Korean hangul
        0x2d4735 => [0x6c4e, false], # East Asian ideograph
        0x275d58 => [0x94c3, false], # East Asian ideograph
        0x273b3f => [0x4e13, false], # East Asian ideograph
        0x692567 => [0x30e7, false], # Katakana letter small YO
        0x213452 => [0x5331, false], # East Asian ideograph
        0x232b45 => [0x8799, false], # East Asian ideograph
        0x222b46 => [0x6010, false], # East Asian ideograph
        0x232b48 => [0x8788, false], # East Asian ideograph
        0x222b4b => [0x6039, false], # East Asian ideograph
        0x232b4c => [0x8798, false], # East Asian ideograph
        0x222b50 => [0x6013, false], # East Asian ideograph
        0x224738 => [0x6c6c, false], # East Asian ideograph
        0x222b53 => [0x6054, false], # East Asian ideograph
        0x232b54 => [0x878b, false], # East Asian ideograph
        0x232b55 => [0x8784, false], # East Asian ideograph
        0x222b57 => [0x605d, false], # East Asian ideograph
        0x232b58 => [0x87a9, false], # East Asian ideograph
        0x336b33 => [0x524f, false], # East Asian ideograph
        0x222b5a => [0x6047, false], # East Asian ideograph
        0x2e2b5b => [0x605a, false], # East Asian ideograph
        0x232b5d => [0x8789, false], # East Asian ideograph
        0x222b5e => [0x6049, false], # East Asian ideograph
        0x222b5f => [0x6053, false], # East Asian ideograph
        0x232b60 => [0x87ad, false], # East Asian ideograph
        0x4b4e37 => [0x7814, false], # East Asian ideograph
        0x23473b => [0x940f, false], # East Asian ideograph
        0x232b66 => [0x87be, false], # East Asian ideograph
        0x222b68 => [0x6067, false], # East Asian ideograph
        0x23473c => [0x9420, false], # East Asian ideograph (not in Unicode)
        0x232b6e => [0x87c4, false], # East Asian ideograph
        0x232b6f => [0x87af, false], # East Asian ideograph
        0x222b71 => [0x6041, false], # East Asian ideograph
        0x222b72 => [0x6077, false], # East Asian ideograph
        0x222b74 => [0x6042, false], # East Asian ideograph
        0x22473e => [0x6c94, false], # East Asian ideograph
        0x222b76 => [0x605f, false], # East Asian ideograph
        0x232b78 => [0x87ae, false], # East Asian ideograph
        0x2e6c27 => [0x7b2e, false], # East Asian ideograph
        0x222b7a => [0x6061, false], # East Asian ideograph
        0x6f4e6f => [0xb799, false], # Korean hangul
        0x232b7e => [0x87bf, false], # East Asian ideograph
        0x692569 => [0x30e9, false], # Katakana letter RA
        0x224740 => [0x6c8f, false], # East Asian ideograph
        0x333c52 => [0x8cec, false], # East Asian ideograph
        0x4b4741 => [0x51bd, false], # East Asian ideograph
        0x23233c => [0x8452, false], # East Asian ideograph
        0x224742 => [0x6c65, false], # East Asian ideograph
        0x213d58 => [0x5f70, false], # East Asian ideograph
        0x2d5e61 => [0x6ff6, false], # East Asian ideograph
        0x6f4c3d => [0xb1cc, false], # Korean hangul
        0x69256a => [0x30ea, false], # Katakana letter RI
        0x294340 => [0x94d6, false], # East Asian ideograph
        0x6f5752 => [0xc887, false], # Korean hangul
        0x4b4b3e => [0xf9ad, false], # East Asian ideograph
        0x2d4746 => [0x6c79, false], # East Asian ideograph
        0x2d6147 => [0x99c8, false], # East Asian ideograph
        0x224747 => [0x6c6f, false], # East Asian ideograph
        0x705f39 => [0x5416, false], # East Asian ideograph
        0x224749 => [0x6c9d, false], # East Asian ideograph
        0x275d5c => [0x94dc, false], # East Asian ideograph
        0x295433 => [0x9aa7, false], # East Asian ideograph
        0x2f4a2e => [0x90b4, false], # East Asian ideograph
        0x22474a => [0x6c69, false], # East Asian ideograph
        0x22474b => [0x6c9a, false], # East Asian ideograph
        0x21383b => [0x57f7, false], # East Asian ideograph
        0x22474c => [0x6c6d, false], # East Asian ideograph
        0x23474d => [0x9419, false], # East Asian ideograph
        0x275b47 => [0x8f8d, false], # East Asian ideograph
        0x23474e => [0x940d, false], # East Asian ideograph
        0x275d5d => [0x94ed, false], # East Asian ideograph
        0x6f4a2f => [0xadff, false], # Korean hangul
        0x234750 => [0x9426, false], # East Asian ideograph
        0x6f5d4a => [0xd68c, false], # Korean hangul
        0x224751 => [0x6c87, false], # East Asian ideograph
        0x39345b => [0x965e, false], # East Asian ideograph
        0x224752 => [0x6c6e, false], # East Asian ideograph
        0x6f4e73 => [0xb7a9, false], # Korean hangul
        0x69256d => [0x30ed, false], # Katakana letter RO
        0x4d4754 => [0x9544, false], # East Asian ideograph
        0x294343 => [0x94d2, false], # East Asian ideograph
        0x276d2e => [0x5326, false], # East Asian ideograph
        0x235a2c => [0x9cf8, false], # East Asian ideograph
        0x224756 => [0x6c95, false], # East Asian ideograph
        0x234758 => [0x9414, false], # East Asian ideograph
        0x275d5f => [0x94ec, false], # East Asian ideograph
        0x6f4a31 => [0xae01, false], # Korean hangul
        0x274759 => [0x6cea, false], # East Asian ideograph
        0x6f582d => [0xc9c8, false], # Korean hangul
        0x4c715a => [0x7ee6, false], # East Asian ideograph
        0x22475a => [0x6c82, false], # East Asian ideograph
        0x2d5340 => [0x812c, false], # East Asian ideograph
        0x2d3b6e => [0x5d17, false], # East Asian ideograph
        0x232c24 => [0x87bd, false], # East Asian ideograph
        0x23475c => [0x9422, false], # East Asian ideograph
        0x222c2b => [0x6092, false], # East Asian ideograph
        0x222c2c => [0x609d, false], # East Asian ideograph
        0x222c2d => [0x6081, false], # East Asian ideograph
        0x23475d => [0x9406, false], # East Asian ideograph
        0x232c30 => [0x87f3, false], # East Asian ideograph
        0x232c31 => [0x87f0, false], # East Asian ideograph
        0x222c32 => [0x6097, false], # East Asian ideograph
        0x215673 => [0x8725, false], # East Asian ideograph
        0x232c34 => [0x87ea, false], # East Asian ideograph
        0x29475e => [0x9562, false], # East Asian ideograph
        0x232c36 => [0x87db, false], # East Asian ideograph
        0x232c37 => [0x87e2, false], # East Asian ideograph
        0x232c39 => [0x87eb, false], # East Asian ideograph
        0x222c3a => [0x6095, false], # East Asian ideograph
        0x2d475f => [0x51c4, false], # East Asian ideograph
        0x4d2c3c => [0x87e5, false], # East Asian ideograph
        0x222c3e => [0x60c7, false], # East Asian ideograph
        0x232c3f => [0x87f5, false], # East Asian ideograph
        0x217d48 => [0x5b21, false], # East Asian ideograph
        0x234760 => [0x9410, false], # East Asian ideograph
        0x222c42 => [0x60b0, false], # East Asian ideograph
        0x6f4d33 => [0xb36a, false], # Korean hangul
        0x222c46 => [0x60be, false], # East Asian ideograph
        0x232c47 => [0x87e0, false], # East Asian ideograph
        0x222c48 => [0x60d4, false], # East Asian ideograph
        0x232c49 => [0x87dc, false], # East Asian ideograph
        0x232c4c => [0x87e3, false], # East Asian ideograph
        0x232c4d => [0x8801, false], # East Asian ideograph
        0x222c4e => [0x60ce, false], # East Asian ideograph
        0x232c4f => [0x8803, false], # East Asian ideograph
        0x232c50 => [0x880a, false], # East Asian ideograph
        0x222c51 => [0x60cf, false], # East Asian ideograph
        0x222c53 => [0x60d9, false], # East Asian ideograph
        0x222c54 => [0x60b3, false], # East Asian ideograph
        0x232c55 => [0x87f6, false], # East Asian ideograph
        0x222c56 => [0x60dd, false], # East Asian ideograph
        0x232c57 => [0x87f7, false], # East Asian ideograph
        0x235a2f => [0x9d2a, false], # East Asian ideograph
        0x232c5c => [0x880b, false], # East Asian ideograph
        0x232c5d => [0x8806, false], # East Asian ideograph
        0x232c5f => [0x87fe, false], # East Asian ideograph
        0x222c60 => [0x60b1, false], # East Asian ideograph
        0x232c61 => [0x8810, false], # East Asian ideograph
        0x222c62 => [0x60e3, false], # East Asian ideograph
        0x232c63 => [0x8819, false], # East Asian ideograph
        0x232c64 => [0x8811, false], # East Asian ideograph
        0x224766 => [0x6cef, false], # East Asian ideograph
        0x232c66 => [0x8818, false], # East Asian ideograph
        0x222c67 => [0x60e5, false], # East Asian ideograph
        0x222c69 => [0x60db, false], # East Asian ideograph
        0x232c6a => [0x8813, false], # East Asian ideograph
        0x232c6b => [0x8816, false], # East Asian ideograph
        0x6f5236 => [0xbee0, false], # Korean hangul
        0x275d62 => [0x950c, false], # East Asian ideograph
        0x222c6e => [0x60e9, false], # East Asian ideograph
        0x692571 => [0x30f1, false], # Katakana letter WE
        0x222c70 => [0x6114, false], # East Asian ideograph
        0x274768 => [0x6d45, false], # East Asian ideograph
        0x232c72 => [0x8834, false], # East Asian ideograph
        0x232c73 => [0x881c, false], # East Asian ideograph
        0x222c75 => [0x6119, false], # East Asian ideograph
        0x234769 => [0x93f7, false], # East Asian ideograph
        0x232c7a => [0x881b, false], # East Asian ideograph
        0x222c7c => [0x60fd, false], # East Asian ideograph
        0x222c7d => [0x610d, false], # East Asian ideograph
        0x6f5b41 => [0xd1f4, false], # Korean hangul
        0x29323b => [0x8bce, false], # East Asian ideograph
        0x287042 => [0x7ea1, false], # East Asian ideograph
        0x4b476c => [0x51c5, false], # East Asian ideograph
        0x275d63 => [0x9511, false], # East Asian ideograph
        0x6f4a35 => [0xae0d, false], # Korean hangul
        0x223e61 => [0x6971, false], # East Asian ideograph
        0x22476e => [0x6cad, false], # East Asian ideograph (variant of 4C476E which maps to 6CAD)
        0x2d5344 => [0x8107, false], # East Asian ideograph
        0x23476f => [0x940e, false], # East Asian ideograph
        0x6f5b2e => [0xd14c, false], # Korean hangul
        0x29323c => [0x8bd2, false], # East Asian ideograph
        0x6f4e39 => [0xb5b5, false], # Korean hangul
        0x224770 => [0x6caf, false], # East Asian ideograph
        0x295925 => [0x9ccc, false], # East Asian ideograph
        0x234771 => [0x9411, false], # East Asian ideograph
        0x692573 => [0x30f3, false], # Katakana letter N
        0x275921 => [0x8c04, false], # East Asian ideograph
        0x294349 => [0x94d5, false], # East Asian ideograph
        0x2d386e => [0x58ca, false], # East Asian ideograph
        0x217e23 => [0x5b62, false], # East Asian ideograph
        0x274774 => [0x6e0a, false], # East Asian ideograph
        0x6f5b43 => [0xd22c, false], # Korean hangul
        0x275551 => [0x80e1, false], # East Asian ideograph (duplicate simplified)
        0x22496a => [0x6dac, false], # East Asian ideograph
        0x6f5b3e => [0xd1b3, false], # Korean hangul
        0x225265 => [0x7168, false], # East Asian ideograph
        0x2d467c => [0x6cb2, false], # East Asian ideograph
        0x29464a => [0x953c, false], # East Asian ideograph
        0x3f3e47 => [0x5379, false], # East Asian ideograph
        0x21345f => [0x5352, false], # East Asian ideograph
        0x274777 => [0x6ca6, false], # East Asian ideograph
        0x2d6159 => [0x9ac4, false], # East Asian ideograph
        0x213223 => [0x5021, false], # East Asian ideograph
        0x234779 => [0x9429, false], # East Asian ideograph
        0x213224 => [0x500b, false], # East Asian ideograph
        0x695457 => [0x58b8, false], # East Asian ideograph
        0x22477a => [0x6cba, false], # East Asian ideograph
        0x223225 => [0x6387, false], # East Asian ideograph
        0x22477b => [0x7553, false], # East Asian ideograph
        0x223226 => [0x637a, false], # East Asian ideograph
        0x6f4b65 => [0xb0c5, false], # Korean hangul
        0x6f4a38 => [0xae34, false], # Korean hangul
        0x213460 => [0x5354, false], # East Asian ideograph
        0x233227 => [0x8a51, false], # East Asian ideograph
        0x27477d => [0x6d8c, false], # East Asian ideograph
        0x213228 => [0x4ff3, false], # East Asian ideograph
        0x6f5330 => [0xc136, false], # Korean hangul
        0x3f3d6f => [0x8986, false], # East Asian ideograph
        0x287272 => [0x7f21, false], # East Asian ideograph
        0x6f4c44 => [0xb205, false], # Korean hangul
        0x213229 => [0x502d, false], # East Asian ideograph
        0x28742e => [0x7f42, false], # East Asian ideograph
        0x6f4f3f => [0xb871, false], # Korean hangul
        0x22322a => [0x6386, false], # East Asian ideograph
        0x4c5c61 => [0x74f4, false], # East Asian ideograph (variant of 225C61 which maps to 74F4)
        0x6f4e7c => [0xb7f0, false], # Korean hangul
        0x6f5237 => [0xbee3, false], # Korean hangul
        0x21722b => [0x55f9, false], # East Asian ideograph
        0x692576 => [0x30f6, false], # Katakana letter small KE
        0x223860 => [0x6673, false], # East Asian ideograph
        0x21322d => [0x502b, false], # East Asian ideograph
        0x6f5d4c => [0xd69f, false], # Korean hangul
        0x6f5b46 => [0xd234, false], # Korean hangul
        0x21322e => [0x505c, false], # East Asian ideograph
        0x22496d => [0x6dd5, false], # East Asian ideograph
        0x232b53 => [0x8785, false], # East Asian ideograph
        0x21322f => [0x504f, false], # East Asian ideograph
        0x225f2f => [0x760f, false], # East Asian ideograph
        0x292657 => [0x835f, false], # East Asian ideograph
        0x233230 => [0x8a56, false], # East Asian ideograph
        0x232d23 => [0x8828, false], # East Asian ideograph
        0x217231 => [0x560c, false], # East Asian ideograph
        0x232d2a => [0x8832, false], # East Asian ideograph
        0x222d2c => [0x6110, false], # East Asian ideograph
        0x232d2e => [0x882e, false], # East Asian ideograph
        0x213e35 => [0x600e, false], # East Asian ideograph
        0x232d32 => [0x882d, false], # East Asian ideograph
        0x213233 => [0x5049, false], # East Asian ideograph
        0x222d34 => [0x60f2, false], # East Asian ideograph
        0x222d37 => [0x6125, false], # East Asian ideograph
        0x277234 => [0x551b, false], # East Asian ideograph
        0x222d3b => [0x60f8, false], # East Asian ideograph
        0x232d3c => [0x883c, false], # East Asian ideograph
        0x6f4e7e => [0xb7fc, false], # Korean hangul
        0x273235 => [0x4fa7, false], # East Asian ideograph
        0x222d41 => [0x60fc, false], # East Asian ideograph
        0x232d42 => [0x4610, false], # East Asian ideograph (not in Unicode)
        0x275926 => [0x8c1a, false], # East Asian ideograph
        0x232d44 => [0x8844, false], # East Asian ideograph
        0x227236 => [0x7df9, false], # East Asian ideograph
        0x222d48 => [0x6149, false], # East Asian ideograph
        0x222d4a => [0x614a, false], # East Asian ideograph
        0x232d4b => [0x8847, false], # East Asian ideograph
        0x222d4e => [0x612b, false], # East Asian ideograph
        0x287275 => [0x7d77, false], # East Asian ideograph
        0x222d50 => [0x6129, false], # East Asian ideograph
        0x222d51 => [0x6150, false], # East Asian ideograph
        0x232d53 => [0x884e, false], # East Asian ideograph
        0x232d56 => [0x8852, false], # East Asian ideograph
        0x233239 => [0x8a48, false], # East Asian ideograph
        0x222d58 => [0x6130, false], # East Asian ideograph
        0x232d59 => [0x8856, false], # East Asian ideograph
        0x232d5a => [0x8855, false], # East Asian ideograph
        0x222d5b => [0x6141, false], # East Asian ideograph
        0x21323a => [0x5055, false], # East Asian ideograph
        0x232d5e => [0x885c, false], # East Asian ideograph
        0x232d5f => [0x885a, false], # East Asian ideograph
        0x222d61 => [0x6146, false], # East Asian ideograph
        0x22323b => [0x6390, false], # East Asian ideograph
        0x222d66 => [0x615e, false], # East Asian ideograph
        0x222d67 => [0x6175, false], # East Asian ideograph
        0x222d68 => [0x6174, false], # East Asian ideograph
        0x232d69 => [0x8869, false], # East Asian ideograph
        0x21316c => [0x5009, false], # East Asian ideograph
        0x222d6b => [0x6183, false], # East Asian ideograph
        0x232d6d => [0x886d, false], # East Asian ideograph
        0x232d6e => [0x887a, false], # East Asian ideograph
        0x21323d => [0x508d, false], # East Asian ideograph
        0x222d70 => [0x6171, false], # East Asian ideograph
        0x232d71 => [0x8875, false], # East Asian ideograph
        0x222757 => [0x5e5e, false], # East Asian ideograph
        0x222d74 => [0x616a, false], # East Asian ideograph
        0x232d75 => [0x8872, false], # East Asian ideograph
        0x6f5045 => [0xbb0f, false], # Korean hangul
        0x222d77 => [0x6173, false], # East Asian ideograph
        0x232d79 => [0x887d, false], # East Asian ideograph
        0x455564 => [0x6fdb, false], # East Asian ideograph (Version J extension)
        0x222d7b => [0x6153, false], # East Asian ideograph
        0x224234 => [0x6a99, false], # East Asian ideograph
        0x232d7d => [0x887f, false], # East Asian ideograph
        0x232d7e => [0x887e, false], # East Asian ideograph
        0x275928 => [0x8bb3, false], # East Asian ideograph
        0x294350 => [0x94df, false], # East Asian ideograph
        0x233240 => [0x8a3d, false], # East Asian ideograph
        0x696126 => [0x74f2, false], # East Asian ideograph
        0x6f567b => [0xc794, false], # Korean hangul
        0x273241 => [0x6770, false], # East Asian ideograph
        0x6f5874 => [0xcc0d, false], # Korean hangul
        0x6f5b4a => [0xd241, false], # Korean hangul
        0x213242 => [0x5080, false], # East Asian ideograph
        0x4b5c5b => [0x8fba, false], # East Asian ideograph
        0x223243 => [0x63de, false], # East Asian ideograph
        0x6f5238 => [0xbee4, false], # Korean hangul
        0x213244 => [0x5098, false], # East Asian ideograph
        0x6f4a3e => [0xae44, false], # Korean hangul
        0x275929 => [0x8c10, false], # East Asian ideograph
        0x2d4539 => [0x6406, false], # East Asian ideograph
        0x213246 => [0x50b3, false], # East Asian ideograph
        0x274c60 => [0x75a1, false], # East Asian ideograph
        0x6f5b4b => [0xd264, false], # Korean hangul
        0x273247 => [0x503a, false], # East Asian ideograph
        0x227248 => [0x7df6, false], # East Asian ideograph
        0x23492e => [0x9585, false], # East Asian ideograph
        0x213249 => [0x50c5, false], # East Asian ideograph
        0x6f4a3f => [0xae45, false], # Korean hangul
        0x27592a => [0x8c0d, false], # East Asian ideograph
        0x223866 => [0x666d, false], # East Asian ideograph
        0x21724b => [0x564b, false], # East Asian ideograph
        0x274c61 => [0x759f, false], # East Asian ideograph
        0x6f5b4c => [0xd277, false], # Korean hangul
        0x21324c => [0x50b7, false], # East Asian ideograph
        0x393246 => [0x4f1d, false], # East Asian ideograph
        0x21324d => [0x50af, false], # East Asian ideograph
        0x275d6e => [0x9530, false], # East Asian ideograph
        0x6f4a40 => [0xae4a, false], # Korean hangul
        0x27592b => [0x8c0b, false], # East Asian ideograph
        0x6f5830 => [0xc9d1, false], # Korean hangul
        0x21324f => [0x50ee, false], # East Asian ideograph
        0x213250 => [0x50f1, false], # East Asian ideograph
        0x274c62 => [0x75ea, false], # East Asian ideograph
        0x333963 => [0x59c9, false], # East Asian ideograph
        0x213251 => [0x50e5, false], # East Asian ideograph
        0x217252 => [0x5640, false], # East Asian ideograph
        0x292433 => [0x8298, false], # East Asian ideograph (duplicate simplified)
        0x69515e => [0x51e9, false], # East Asian ideograph
        0x287253 => [0x7f12, false], # East Asian ideograph
        0x6f575a => [0xc89f, false], # Korean hangul
        0x223b63 => [0x67d8, false], # East Asian ideograph
        0x213255 => [0x50d5, false], # East Asian ideograph
        0x274c63 => [0x75af, false], # East Asian ideograph
        0x4c523a => [0x717a, false], # East Asian ideograph
        0x213256 => [0x507d, false], # East Asian ideograph
        0x234d74 => [0x97ab, false], # East Asian ideograph
        0x22674b => [0x798a, false], # East Asian ideograph
        0x213257 => [0x50cf, false], # East Asian ideograph
        0x234931 => [0x958c, false], # East Asian ideograph
        0x213258 => [0x50d1, false], # East Asian ideograph
        0x224235 => [0x6a9d, false], # East Asian ideograph
        0x27592d => [0x8c13, false], # East Asian ideograph
        0x294355 => [0x94eb, false], # East Asian ideograph
        0x273259 => [0x4eea, false], # East Asian ideograph
        0x6f567c => [0xc796, false], # Korean hangul
        0x21325a => [0x5104, false], # East Asian ideograph
        0x6f5b4f => [0xd288, false], # Korean hangul
        0x222e23 => [0x618b, false], # East Asian ideograph
        0x2d5129 => [0x7d25, false], # East Asian ideograph
        0x232e28 => [0x88a2, false], # East Asian ideograph
        0x21325c => [0x50f5, false], # East Asian ideograph
        0x232e2a => [0x88a4, false], # East Asian ideograph
        0x222e2c => [0x616f, false], # East Asian ideograph
        0x222e2d => [0x6165, false], # East Asian ideograph
        0x6f5239 => [0xbee5, false], # Korean hangul
        0x232e2f => [0x88aa, false], # East Asian ideograph
        0x276135 => [0x9a7e, false], # East Asian ideograph
        0x222e32 => [0x619d, false], # East Asian ideograph
        0x222e33 => [0x61a6, false], # East Asian ideograph
        0x232e34 => [0x889a, false], # East Asian ideograph
        0x27325e => [0x4fac, false], # East Asian ideograph
        0x235a3f => [0x9d1e, false], # East Asian ideograph
        0x232e3a => [0x8890, false], # East Asian ideograph
        0x232e3b => [0x888c, false], # East Asian ideograph
        0x232e3d => [0x88a0, false], # East Asian ideograph
        0x232e40 => [0x8899, false], # East Asian ideograph
        0x213260 => [0x5108, false], # East Asian ideograph
        0x222e42 => [0x619c, false], # East Asian ideograph
        0x222e43 => [0x61af, false], # East Asian ideograph
        0x232e45 => [0x8897, false], # East Asian ideograph
        0x222e46 => [0x6197, false], # East Asian ideograph
        0x222e47 => [0x61ad, false], # East Asian ideograph
        0x232e48 => [0x88c9, false], # East Asian ideograph
        0x232e49 => [0x88bf, false], # East Asian ideograph
        0x232e4a => [0x88ba, false], # East Asian ideograph
        0x222e4c => [0x6192, false], # East Asian ideograph
        0x213262 => [0x5110, false], # East Asian ideograph
        0x232e4f => [0x88c0, false], # East Asian ideograph
        0x6f4a44 => [0xae4d, false], # Korean hangul
        0x232e51 => [0x88b2, false], # East Asian ideograph
        0x222e52 => [0x61ae, false], # East Asian ideograph
        0x213263 => [0x5118, false], # East Asian ideograph
        0x232e54 => [0x88bc, false], # East Asian ideograph
        0x222e55 => [0x618d, false], # East Asian ideograph
        0x232e57 => [0x88b7, false], # East Asian ideograph
        0x232e59 => [0x88bd, false], # East Asian ideograph
        0x232e5a => [0x88c4, false], # East Asian ideograph
        0x2d313a => [0x62bb, false], # East Asian ideograph
        0x222e5c => [0x61cc, false], # East Asian ideograph
        0x222e5d => [0x61c6, false], # East Asian ideograph
        0x232e5e => [0x88cb, false], # East Asian ideograph
        0x213265 => [0x5114, false], # East Asian ideograph
        0x232e60 => [0x88cc, false], # East Asian ideograph
        0x232e62 => [0x88db, false], # East Asian ideograph
        0x232e64 => [0x88ce, false], # East Asian ideograph
        0x224535 => [0x6ba3, false], # East Asian ideograph
        0x234934 => [0x9597, false], # East Asian ideograph
        0x222e68 => [0x61ba, false], # East Asian ideograph
        0x222e6a => [0x61b8, false], # East Asian ideograph
        0x273267 => [0x507f, false], # East Asian ideograph
        0x6f4a45 => [0xae4e, false], # Korean hangul
        0x275930 => [0x8c15, false], # East Asian ideograph
        0x294358 => [0x94ef, false], # East Asian ideograph
        0x232e71 => [0x88f1, false], # East Asian ideograph
        0x232e72 => [0x88fe, false], # East Asian ideograph
        0x6f5d66 => [0xd734, false], # Korean hangul
        0x232e75 => [0x88f2, false], # East Asian ideograph
        0x233269 => [0x8a8f, false], # East Asian ideograph
        0x232e78 => [0x8900, false], # East Asian ideograph
        0x213f2e => [0x6176, false], # East Asian ideograph
        0x232e7a => [0x88f0, false], # East Asian ideograph
        0x6f5b52 => [0xd293, false], # Korean hangul
        0x222e7d => [0x61dc, false], # East Asian ideograph
        0x222e7e => [0x61df, false], # East Asian ideograph
        0x69723b => [0x9b96, false], # East Asian ideograph
        0x21326b => [0x513c, false], # East Asian ideograph
        0x276069 => [0x996a, false], # East Asian ideograph
        0x294359 => [0x94e5, false], # East Asian ideograph
        0x4b5434 => [0x6319, false], # East Asian ideograph
        0x6f5b53 => [0xd295, false], # Korean hangul
        0x21326f => [0x5145, false], # East Asian ideograph
        0x223270 => [0x63be, false], # East Asian ideograph
        0x234936 => [0x958e, false], # East Asian ideograph
        0x2d4249 => [0x53d9, false], # East Asian ideograph
        0x213271 => [0x5146, false], # East Asian ideograph
        0x224236 => [0x6a7e, false], # East Asian ideograph
        0x6f4a47 => [0xae54, false], # Korean hangul
        0x275932 => [0x8c26, false], # East Asian ideograph
        0x217272 => [0x5660, false], # East Asian ideograph
        0x295834 => [0x9cbb, false], # East Asian ideograph
        0x223273 => [0x63dd, false], # East Asian ideograph
        0x6f5b54 => [0xd29c, false], # Korean hangul
        0x22497b => [0x6dbf, false], # East Asian ideograph
        0x213275 => [0x514c, false], # East Asian ideograph
        0x6f523a => [0xbeec, false], # Korean hangul
        0x2d3a26 => [0x5a3f, false], # East Asian ideograph
        0x6f5d21 => [0xd5d9, false], # Korean hangul
        0x283f5c => [0x6769, false], # East Asian ideograph
        0x29435b => [0x94e3, false], # East Asian ideograph
        0x213277 => [0x514d, false], # East Asian ideograph
        0x6f5d22 => [0xd5db, false], # Korean hangul
        0x2d5d23 => [0x9167, false], # East Asian ideograph
        0x274c6a => [0x75ae, false], # East Asian ideograph
        0x6f4c2e => [0xb158, false], # Korean hangul
        0x213279 => [0x5154, false], # East Asian ideograph
        0x705f54 => [0x54b4, false], # East Asian ideograph
        0x6f5d24 => [0xd5e4, false], # Korean hangul
        0x29324f => [0x8bd6, false], # East Asian ideograph
        0x393460 => [0x604a, false], # East Asian ideograph
        0x273859 => [0x5c18, false], # East Asian ideograph
        0x225d25 => [0x750e, false], # East Asian ideograph
        0x21327b => [0x5157, false], # East Asian ideograph
        0x6f5d26 => [0xd5e8, false], # Korean hangul
        0x275934 => [0x8bb2, false], # East Asian ideograph
        0x223870 => [0x6684, false], # East Asian ideograph
        0x235d27 => [0x9e0e, false], # East Asian ideograph
        0x6f577c => [0xc961, false], # Korean hangul
        0x21327d => [0x5162, false], # East Asian ideograph
        0x225d28 => [0x750d, false], # East Asian ideograph
        0x213e38 => [0x6059, false], # East Asian ideograph
        0x23327e => [0x8ab6, false], # East Asian ideograph
        0x295d29 => [0x9e71, false], # East Asian ideograph
        0x293250 => [0x8bd3, false], # East Asian ideograph
        0x275d2a => [0x9154, false], # East Asian ideograph
        0x235d2b => [0x9e11, false], # East Asian ideograph
        0x2f4a4a => [0x5f8f, false], # East Asian ideograph
        0x275935 => [0x8c0e, false], # East Asian ideograph
        0x225d2c => [0x7511, false], # East Asian ideograph
        0x225d2d => [0x750f, false], # East Asian ideograph
        0x2d3140 => [0x4f32, false], # East Asian ideograph
        0x6f5b57 => [0xd2ac, false], # Korean hangul
        0x6f5d2e => [0xd604, false], # Korean hangul
        0x215d2f => [0x919e, false], # East Asian ideograph
        0x275d79 => [0x952e, false], # East Asian ideograph
        0x275d30 => [0x4e11, false], # East Asian ideograph
        0x6f4a4b => [0xae61, false], # Korean hangul
        0x232f23 => [0x88ef, false], # East Asian ideograph
        0x232f24 => [0x8903, false], # East Asian ideograph
        0x39456d => [0x826b, false], # East Asian ideograph
        0x6f5758 => [0xc89c, false], # Korean hangul
        0x215d31 => [0x91ab, false], # East Asian ideograph
        0x225648 => [0x72a8, false], # East Asian ideograph
        0x222f29 => [0x61f3, false], # East Asian ideograph
        0x275d32 => [0x9171, false], # East Asian ideograph
        0x274c6d => [0x75e8, false], # East Asian ideograph
        0x212f30 => [0x3007, false], # East Asian ideograph (number zero)
        0x225d33 => [0x7513, false], # East Asian ideograph
        0x232f35 => [0x8906, false], # East Asian ideograph
        0x232f36 => [0x890c, false], # East Asian ideograph
        0x232f37 => [0x8919, false], # East Asian ideograph
        0x215d34 => [0x91c0, false], # East Asian ideograph
        0x232f3d => [0x890a, false], # East Asian ideograph
        0x6f4b69 => [0xb0d0, false], # Korean hangul
        0x215d35 => [0x91c1, false], # East Asian ideograph
        0x6f4a4c => [0xae62, false], # Korean hangul
        0x222f41 => [0x6204, false], # East Asian ideograph
        0x235742 => [0x9b9e, false], # East Asian ideograph
        0x222f43 => [0x6207, false], # East Asian ideograph
        0x222f44 => [0x6209, false], # East Asian ideograph
        0x232f45 => [0x892f, false], # East Asian ideograph
        0x232f47 => [0x8930, false], # East Asian ideograph
        0x345e47 => [0x75fe, false], # East Asian ideograph
        0x235d37 => [0x9e18, false], # East Asian ideograph
        0x274c6e => [0x7597, false], # East Asian ideograph
        0x232f4e => [0x8921, false], # East Asian ideograph
        0x232f4f => [0x8927, false], # East Asian ideograph
        0x232f51 => [0x891f, false], # East Asian ideograph
        0x232f53 => [0x8931, false], # East Asian ideograph
        0x232f54 => [0x891e, false], # East Asian ideograph
        0x295029 => [0x98a5, false], # East Asian ideograph
        0x232f56 => [0x8926, false], # East Asian ideograph
        0x232f57 => [0x8922, false], # East Asian ideograph
        0x232f5a => [0x8935, false], # East Asian ideograph
        0x222f5b => [0x6225, false], # East Asian ideograph
        0x232f5d => [0x8941, false], # East Asian ideograph
        0x275938 => [0x8c22, false], # East Asian ideograph
        0x232f60 => [0x8933, false], # East Asian ideograph
        0x222f61 => [0x6229, false], # East Asian ideograph
        0x235d3b => [0x9e1d, false], # East Asian ideograph
        0x232f66 => [0x8954, false], # East Asian ideograph
        0x222f67 => [0x622d, false], # East Asian ideograph
        0x6f5d50 => [0xd6c5, false], # Korean hangul
        0x215d3c => [0x91cf, false], # East Asian ideograph
        0x6f5b5a => [0xd2b9, false], # Korean hangul
        0x222f6e => [0x6239, false], # East Asian ideograph
        0x222f6f => [0x623a, false], # East Asian ideograph
        0x222f70 => [0x623d, false], # East Asian ideograph
        0x232f72 => [0x8947, false], # East Asian ideograph
        0x27385a => [0x57ab, false], # East Asian ideograph
        0x222f75 => [0x6243, false], # East Asian ideograph
        0x222f77 => [0x6246, false], # East Asian ideograph
        0x222f78 => [0x6245, false], # East Asian ideograph
        0x222f79 => [0x624a, false], # East Asian ideograph
        0x232f7a => [0x894c, false], # East Asian ideograph
        0x232f7b => [0x8946, false], # East Asian ideograph
        0x222f7c => [0x625e, false], # East Asian ideograph
        0x295859 => [0x9cc6, false], # East Asian ideograph
        0x275d40 => [0x9489, false], # East Asian ideograph
        0x275d41 => [0x948a, false], # East Asian ideograph
        0x6f5b5b => [0xd2bc, false], # Korean hangul
        0x215d42 => [0x91dc, false], # East Asian ideograph
        0x6f4e3a => [0xb5bb, false], # Korean hangul
        0x275d43 => [0x9497, false], # East Asian ideograph
        0x45604e => [0x984f, false], # East Asian ideograph
        0x215d44 => [0x91e6, false], # East Asian ideograph
        0x6f4a4f => [0xae69, false], # Korean hangul
        0x27593a => [0x8c2c, false], # East Asian ideograph
        0x273721 => [0x545c, false], # East Asian ideograph
        0x275d45 => [0x9493, false], # East Asian ideograph
        0x2d535e => [0x8193, false], # East Asian ideograph
        0x275d46 => [0x948f, false], # East Asian ideograph
        0x33632b => [0x7adc, false], # East Asian ideograph
        0x6f5b5c => [0xd2bf, false], # Korean hangul
        0x705f5b => [0x54a3, false], # East Asian ideograph
        0x215d47 => [0x9223, false], # East Asian ideograph
        0x293256 => [0x8be9, false], # East Asian ideograph
        0x2e493b => [0x6e7c, false], # East Asian ideograph
        0x275d48 => [0x949d, false], # East Asian ideograph
        0x4b4921 => [0x6ca2, false], # East Asian ideograph
        0x235d49 => [0x9e84, false], # East Asian ideograph
        0x27606b => [0x996d, false], # East Asian ideograph
        0x27593b => [0x8c1f, false], # East Asian ideograph
        0x6f5759 => [0xc89d, false], # Korean hangul
        0x275d4a => [0x94a0, false], # East Asian ideograph
        0x215d4b => [0x9214, false], # East Asian ideograph
        0x6f5b5d => [0xd2c0, false], # Korean hangul
        0x275d4c => [0x94a7, false], # East Asian ideograph
        0x284c2e => [0x6d52, false], # East Asian ideograph
        0x697246 => [0x9bd1, false], # East Asian ideograph
        0x275d4d => [0x94a4, false], # East Asian ideograph
        0x2d3356 => [0x5211, false], # East Asian ideograph (not in Unicode)
        0x6f4b6a => [0xb0d1, false], # Korean hangul
        0x6f5d4e => [0xd6a8, false], # Korean hangul
        0x6f4a51 => [0xae70, false], # Korean hangul
        0x27593c => [0x8bc6, false], # East Asian ideograph
        0x294364 => [0x94f7, false], # East Asian ideograph
        0x233c77 => [0x8fcb, false], # East Asian ideograph
        0x213a38 => [0x5ac2, false], # East Asian ideograph
        0x275d50 => [0x94b9, false], # East Asian ideograph
        0x2d3147 => [0x5002, false], # East Asian ideograph
        0x6f5b5e => [0xd2c8, false], # Korean hangul
        0x215d51 => [0x923d, false], # East Asian ideograph
        0x396074 => [0x55b0, false], # East Asian ideograph
        0x215d52 => [0x923e, false], # East Asian ideograph
        0x6f523c => [0xbf09, false], # Korean hangul
        0x275d53 => [0x94be, false], # East Asian ideograph
        0x21347a => [0x53ad, false], # East Asian ideograph
        0x213037 => [0x4e38, false], # East Asian ideograph
        0x4b4b63 => [0x749c, false], # East Asian ideograph
        0x215d55 => [0x925b, false], # East Asian ideograph
        0x6f5b5f => [0xd2c9, false], # Korean hangul
        0x275d56 => [0x94a9, false], # East Asian ideograph
        0x22675c => [0x799a, false], # East Asian ideograph
        0x275d57 => [0x94c2, false], # East Asian ideograph
        0x234942 => [0x95ac, false], # East Asian ideograph
        0x225d58 => [0x7547, false], # East Asian ideograph
        0x6f4a53 => [0xae79, false], # Korean hangul
        0x224830 => [0x6cd1, false], # East Asian ideograph
        0x275d59 => [0x94f0, false], # East Asian ideograph
        0x235a4f => [0x9d41, false], # East Asian ideograph
        0x275d5a => [0x94f6, false], # East Asian ideograph
        0x274c75 => [0x765e, false], # East Asian ideograph
        0x213021 => [0x4e00, false], # East Asian ideograph
        0x213022 => [0x4e01, false], # East Asian ideograph
        0x215d5b => [0x92ac, false], # East Asian ideograph
        0x213024 => [0x4e09, false], # East Asian ideograph
        0x213025 => [0x4e0b, false], # East Asian ideograph
        0x223026 => [0x6268, false], # East Asian ideograph
        0x213027 => [0x4e08, false], # East Asian ideograph
        0x223028 => [0x6260, false], # East Asian ideograph
        0x233029 => [0x895b, false], # East Asian ideograph
        0x21302a => [0x4e0d, false], # East Asian ideograph
        0x21302b => [0x4e14, false], # East Asian ideograph
        0x22302c => [0x6262, false], # East Asian ideograph
        0x21302d => [0x4e16, false], # East Asian ideograph
        0x21302e => [0x4e15, false], # East Asian ideograph
        0x215d5d => [0x9298, false], # East Asian ideograph
        0x213030 => [0x4e22, false], # East Asian ideograph
        0x233031 => [0x8966, false], # East Asian ideograph
        0x223032 => [0x628e, false], # East Asian ideograph
        0x213034 => [0x4e2d, false], # East Asian ideograph
        0x275d5e => [0x94e2, false], # East Asian ideograph
        0x213036 => [0x51e1, false], # East Asian ideograph
        0x233037 => [0x896d, false], # East Asian ideograph
        0x213038 => [0x4e39, false], # East Asian ideograph
        0x213039 => [0x4e3b, false], # East Asian ideograph
        0x23303a => [0x896b, false], # East Asian ideograph
        0x23303b => [0x896e, false], # East Asian ideograph
        0x23303c => [0x896c, false], # East Asian ideograph
        0x21303d => [0x4e4b, false], # East Asian ideograph
        0x21303e => [0x5c39, false], # East Asian ideograph
        0x21303f => [0x4e4f, false], # East Asian ideograph
        0x213040 => [0x4e4e, false], # East Asian ideograph
        0x233041 => [0x8976, false], # East Asian ideograph
        0x233042 => [0x8974, false], # East Asian ideograph
        0x223043 => [0x6282, false], # East Asian ideograph
        0x213044 => [0x4e56, false], # East Asian ideograph
        0x213045 => [0x4e58, false], # East Asian ideograph
        0x213046 => [0x4e59, false], # East Asian ideograph
        0x215d61 => [0x929c, false], # East Asian ideograph
        0x213048 => [0x4e5f, false], # East Asian ideograph
        0x233049 => [0x897b, false], # East Asian ideograph
        0x23304a => [0x897c, false], # East Asian ideograph
        0x22304b => [0x629d, false], # East Asian ideograph
        0x27304c => [0x5e72, false], # East Asian ideograph
        0x225d62 => [0x7564, false], # East Asian ideograph
        0x6f4a55 => [0xae7c, false], # Korean hangul
        0x213050 => [0x4e8b, false], # East Asian ideograph
        0x213051 => [0x4e8c, false], # East Asian ideograph
        0x213052 => [0x4e8e, false], # East Asian ideograph
        0x233053 => [0x8984, false], # East Asian ideograph
        0x213054 => [0x4e94, false], # East Asian ideograph
        0x233055 => [0x8985, false], # East Asian ideograph
        0x223056 => [0x62a6, false], # East Asian ideograph
        0x213057 => [0x4e99, false], # East Asian ideograph (variant of 4B3057 which maps to 4E99)
        0x273058 => [0x4e9a, false], # East Asian ideograph
        0x215d64 => [0x92b3, false], # East Asian ideograph
        0x21305a => [0x4e9f, false], # East Asian ideograph
        0x274c77 => [0x7663, false], # East Asian ideograph
        0x21305c => [0x4ea6, false], # East Asian ideograph
        0x21305d => [0x4ea5, false], # East Asian ideograph
        0x21305e => [0x4ea4, false], # East Asian ideograph
        0x215d65 => [0x92ea, false], # East Asian ideograph
        0x213060 => [0x4eab, false], # East Asian ideograph
        0x213061 => [0x4eac, false], # East Asian ideograph
        0x233062 => [0x8991, false], # East Asian ideograph
        0x213063 => [0x4eae, false], # East Asian ideograph
        0x233064 => [0x8997, false], # East Asian ideograph
        0x215d66 => [0x92b7, false], # East Asian ideograph
        0x233066 => [0x8998, false], # East Asian ideograph
        0x4b5830 => [0x899a, false], # East Asian ideograph
        0x213068 => [0x4ec3, false], # East Asian ideograph
        0x213069 => [0x4ec4, false], # East Asian ideograph
        0x22306a => [0x62c3, false], # East Asian ideograph
        0x23306b => [0x899c, false], # East Asian ideograph
        0x21306c => [0x4ec7, false], # East Asian ideograph
        0x21306d => [0x4ecb, false], # East Asian ideograph
        0x21306e => [0x4ee4, false], # East Asian ideograph
        0x23306f => [0x89a1, false], # East Asian ideograph
        0x213070 => [0x4ed5, false], # East Asian ideograph
        0x275d68 => [0x9504, false], # East Asian ideograph
        0x223072 => [0x630d, false], # East Asian ideograph
        0x213073 => [0x4ee3, false], # East Asian ideograph
        0x213074 => [0x4ed4, false], # East Asian ideograph
        0x213075 => [0x4ed7, false], # East Asian ideograph
        0x233076 => [0x89a5, false], # East Asian ideograph
        0x275d69 => [0x9509, false], # East Asian ideograph
        0x213078 => [0x4eff, false], # East Asian ideograph
        0x233079 => [0x89a9, false], # East Asian ideograph
        0x6f5b63 => [0xd2f0, false], # Korean hangul
        0x21307c => [0x4efb, false], # East Asian ideograph
        0x275d6a => [0x950b, false], # East Asian ideograph
        0x21307e => [0x4f15, false], # East Asian ideograph
        0x224547 => [0x6bbd, false], # East Asian ideograph
        0x215d6b => [0x9320, false], # East Asian ideograph
        0x292a2f => [0x86f1, false], # East Asian ideograph
        0x6f5d6c => [0xd754, false], # Korean hangul
        0x29436a => [0x9512, false], # East Asian ideograph
        0x215d6d => [0x92f8, false], # East Asian ideograph
        0x235a53 => [0x9d36, false], # East Asian ideograph
        0x225d6e => [0x757a, false], # East Asian ideograph
        0x6f535b => [0xc218, false], # Korean hangul
        0x274c79 => [0x766b, false], # East Asian ideograph
        0x6f5b64 => [0xd2f1, false], # Korean hangul
        0x275d6f => [0x9519, false], # East Asian ideograph
        0x29325e => [0x8bdc, false], # East Asian ideograph
        0x6f5721 => [0xc7a1, false], # Korean hangul
        0x2f575f => [0x9abe, false], # East Asian ideograph
        0x275d70 => [0x94b1, false], # East Asian ideograph
        0x4b5832 => [0x89b3, false], # East Asian ideograph
        0x225d71 => [0x7577, false], # East Asian ideograph
        0x6f4a58 => [0xae85, false], # Korean hangul
        0x275d72 => [0x9521, false], # East Asian ideograph
        0x22343c => [0x649d, false], # East Asian ideograph
        0x275d73 => [0x94ee, false], # East Asian ideograph
        0x6f5b65 => [0xd2f4, false], # Korean hangul
        0x275d74 => [0x5f55, false], # East Asian ideograph
        0x6f5722 => [0xc7a3, false], # Korean hangul
        0x69724e => [0x9bf2, false], # East Asian ideograph
        0x215d75 => [0x9310, false], # East Asian ideograph
        0x234948 => [0x95bc, false], # East Asian ideograph
        0x4b325f => [0x50bb, false], # East Asian ideograph
        0x215d76 => [0x9326, false], # East Asian ideograph
        0x6f5835 => [0xc9da, false], # Korean hangul
        0x692153 => [0x3009, false], # Ideographic greater than sign
        0x215d77 => [0x934d, false], # East Asian ideograph
        0x2d632d => [0x4e80, false], # East Asian ideograph
        0x215d78 => [0x9382, false], # East Asian ideograph
        0x274c7b => [0x53d1, false], # East Asian ideograph
        0x6f5b66 => [0xd2f8, false], # Korean hangul
        0x225d79 => [0x757d, false], # East Asian ideograph
        0x6f5432 => [0xc2fc, false], # Korean hangul
        0x224824 => [0x6cd8, false], # East Asian ideograph
        0x4b5c77 => [0x9139, false], # East Asian ideograph
        0x235d7a => [0x9eb0, false], # East Asian ideograph
        0x6f5d7b => [0xd790, false], # Korean hangul
        0x27606d => [0x9974, false], # East Asian ideograph
        0x224826 => [0x6cc6, false], # East Asian ideograph
        0x6f575b => [0xc8a0, false], # Korean hangul
        0x275d7c => [0x9505, false], # East Asian ideograph
        0x2e5452 => [0x71fe, false], # East Asian ideograph
        0x234827 => [0x93f4, false], # East Asian ideograph
        0x275d7d => [0x951a, false], # East Asian ideograph
        0x234828 => [0x9436, false], # East Asian ideograph
        0x6f5b67 => [0xd300, false], # Korean hangul
        0x275d7e => [0x953e, false], # East Asian ideograph
        0x224829 => [0x6ce9, false], # East Asian ideograph
        0x226764 => [0x799d, false], # East Asian ideograph
        0x23494a => [0x95cd, false], # East Asian ideograph
        0x395e3d => [0x9295, false], # East Asian ideograph
        0x6f4a5b => [0xaebe, false], # Korean hangul
        0x23482b => [0x943b, false], # East Asian ideograph
        0x275946 => [0x8bd1, false], # East Asian ideograph
        0x22527c => [0x717b, false], # East Asian ideograph
        0x23482d => [0x9424, false], # East Asian ideograph
        0x6f5b68 => [0xd301, false], # Korean hangul
        0x6f5725 => [0xc7a6, false], # Korean hangul
        0x284e42 => [0x6d4d, false], # East Asian ideograph
        0x21482f => [0x6e34, false], # East Asian ideograph
        0x6f523e => [0xbf1d, false], # Korean hangul
        0x6f4a5c => [0xaec0, false], # Korean hangul
        0x234830 => [0x9437, false], # East Asian ideograph
        0x213122 => [0x4f10, false], # East Asian ideograph
        0x213123 => [0x4f0f, false], # East Asian ideograph
        0x213124 => [0x4ef2, false], # East Asian ideograph
        0x223125 => [0x62f5, false], # East Asian ideograph
        0x213126 => [0x4ef3, false], # East Asian ideograph
        0x213127 => [0x4ef6, false], # East Asian ideograph
        0x213128 => [0x4ef0, false], # East Asian ideograph
        0x23312a => [0x89b8, false], # East Asian ideograph
        0x23312b => [0x89b7, false], # East Asian ideograph
        0x23312c => [0x89b6, false], # East Asian ideograph
        0x234832 => [0x9440, false], # East Asian ideograph
        0x21312e => [0x4f57, false], # East Asian ideograph
        0x23312f => [0x89bc, false], # East Asian ideograph
        0x213130 => [0x4f5e, false], # East Asian ideograph
        0x223131 => [0x630c, false], # East Asian ideograph
        0x233132 => [0x89bf, false], # East Asian ideograph
        0x213133 => [0x4f55, false], # East Asian ideograph
        0x213134 => [0x4f30, false], # East Asian ideograph
        0x213135 => [0x4f50, false], # East Asian ideograph
        0x213136 => [0x4f51, false], # East Asian ideograph
        0x223137 => [0x62f6, false], # East Asian ideograph
        0x213138 => [0x4f48, false], # East Asian ideograph
        0x213139 => [0x4f46, false], # East Asian ideograph
        0x22313a => [0x6331, false], # East Asian ideograph
        0x23313b => [0x89d5, false], # East Asian ideograph
        0x21313c => [0x4f54, false], # East Asian ideograph
        0x21313d => [0x4f3c, false], # East Asian ideograph
        0x21313e => [0x4f63, false], # East Asian ideograph
        0x23313f => [0x89da, false], # East Asian ideograph
        0x213140 => [0x4f60, false], # East Asian ideograph
        0x213141 => [0x4f2f, false], # East Asian ideograph
        0x223142 => [0x6345, false], # East Asian ideograph
        0x233143 => [0x89e5, false], # East Asian ideograph
        0x223144 => [0x6343, false], # East Asian ideograph
        0x234836 => [0x942d, false], # East Asian ideograph
        0x213146 => [0x4f6f, false], # East Asian ideograph
        0x223147 => [0x6353, false], # East Asian ideograph
        0x223148 => [0x6364, false], # East Asian ideograph
        0x223149 => [0x6336, false], # East Asian ideograph
        0x22314a => [0x6344, false], # East Asian ideograph
        0x224837 => [0x6d1d, false], # East Asian ideograph
        0x23314c => [0x89e9, false], # East Asian ideograph
        0x23314d => [0x89eb, false], # East Asian ideograph
        0x21314e => [0x4f8b, false], # East Asian ideograph
        0x27314f => [0x4ed1, false], # East Asian ideograph
        0x234838 => [0x9431, false], # East Asian ideograph
        0x213152 => [0x4f7b, false], # East Asian ideograph
        0x233153 => [0x89ed, false], # East Asian ideograph
        0x223154 => [0x6339, false], # East Asian ideograph
        0x213155 => [0x4f8f, false], # East Asian ideograph
        0x213156 => [0x4f7e, false], # East Asian ideograph
        0x213157 => [0x4fe1, false], # East Asian ideograph
        0x223158 => [0x6357, false], # East Asian ideograph
        0x213159 => [0x4fb5, false], # East Asian ideograph
        0x22315a => [0x633c, false], # East Asian ideograph
        0x22315b => [0x6358, false], # East Asian ideograph
        0x21315c => [0x4fde, false], # East Asian ideograph
        0x27315d => [0x4fa0, false], # East Asian ideograph
        0x21315e => [0x4fcf, false], # East Asian ideograph
        0x22315f => [0x6354, false], # East Asian ideograph
        0x213160 => [0x4fda, false], # East Asian ideograph
        0x213161 => [0x4fdd, false], # East Asian ideograph
        0x213162 => [0x4fc3, false], # East Asian ideograph
        0x213163 => [0x4fd8, false], # East Asian ideograph
        0x233164 => [0x89f7, false], # East Asian ideograph
        0x213165 => [0x4fca, false], # East Asian ideograph
        0x213166 => [0x4fae, false], # East Asian ideograph
        0x213167 => [0x4fd0, false], # East Asian ideograph
        0x223168 => [0x637d, false], # East Asian ideograph
        0x273169 => [0x7cfb, false], # East Asian ideograph (duplicate simplified)
        0x22316a => [0x63b6, false], # East Asian ideograph
        0x22316b => [0x6382, false], # East Asian ideograph
        0x27316c => [0x4ed3, false], # East Asian ideograph
        0x23316d => [0x8a07, false], # East Asian ideograph
        0x22316e => [0x639f, false], # East Asian ideograph
        0x21483d => [0x6e9d, false], # East Asian ideograph
        0x233170 => [0x8a0f, false], # East Asian ideograph
        0x233171 => [0x8a11, false], # East Asian ideograph
        0x233172 => [0x8a12, false], # East Asian ideograph
        0x233173 => [0x8a0d, false], # East Asian ideograph
        0x213174 => [0x4ff8, false], # East Asian ideograph
        0x213175 => [0x5028, false], # East Asian ideograph
        0x213176 => [0x5014, false], # East Asian ideograph
        0x213177 => [0x5016, false], # East Asian ideograph
        0x213178 => [0x5029, false], # East Asian ideograph
        0x223179 => [0x6381, false], # East Asian ideograph
        0x23317a => [0x8a27, false], # East Asian ideograph
        0x22317b => [0x6397, false], # East Asian ideograph
        0x21317c => [0x503c, false], # East Asian ideograph
        0x23317d => [0x8a29, false], # East Asian ideograph
        0x21317e => [0x4ffa, false], # East Asian ideograph
        0x234840 => [0x9445, false], # East Asian ideograph
        0x274841 => [0x6c85, false], # East Asian ideograph
        0x6f5b6c => [0xd30c, false], # Korean hangul
        0x234842 => [0x9450, false], # East Asian ideograph
        0x273266 => [0x4f18, false], # East Asian ideograph
        0x4b513b => [0x7cf8, false], # East Asian ideograph
        0x6f4b6d => [0xb0ec, false], # Korean hangul
        0x274844 => [0x6e7f, false], # East Asian ideograph
        0x2d4845 => [0x6e29, false], # East Asian ideograph
        0x4b4846 => [0x78c6, false], # East Asian ideograph
        0x226f69 => [0x7cc5, false], # East Asian ideograph
        0x274848 => [0x6ca7, false], # East Asian ideograph
        0x4b3622 => [0x8c18, false], # East Asian ideograph
        0x6f4a61 => [0xaed0, false], # Korean hangul
        0x273733 => [0x5578, false], # East Asian ideograph
        0x23484a => [0x944a, false], # East Asian ideograph
        0x27484b => [0x51c6, false], # East Asian ideograph
        0x6f5b6e => [0xd30e, false], # Korean hangul
        0x2f3639 => [0x8c7c, false], # East Asian ideograph
        0x4b484c => [0x6f91, false], # East Asian ideograph
        0x22484d => [0x6d26, false], # East Asian ideograph
        0x22484e => [0x6d27, false], # East Asian ideograph
        0x294375 => [0x9514, false], # East Asian ideograph
        0x22484f => [0x6d0f, false], # East Asian ideograph
        0x224850 => [0x6d0a, false], # East Asian ideograph
        0x2d4466 => [0x6973, false], # East Asian ideograph
        0x224851 => [0x6d3f, false], # East Asian ideograph
        0x226329 => [0x77be, false], # East Asian ideograph
        0x234853 => [0x9466, false], # East Asian ideograph
        0x47594e => [0x9c3a, false], # East Asian ideograph
        0x274854 => [0x6e0d, false], # East Asian ideograph
        0x514e5b => [0x9271, false], # East Asian ideograph
        0x274855 => [0x6da8, false], # East Asian ideograph
        0x6f5b70 => [0xd314, false], # Korean hangul
        0x274842 => [0x706d, false], # East Asian ideograph
        0x6f572d => [0xc7bf, false], # Korean hangul
        0x284c41 => [0x6ca4, false], # East Asian ideograph
        0x234560 => [0x93be, false], # East Asian ideograph (not in Unicode)
        0x29243a => [0x83bc, false], # East Asian ideograph
        0x274857 => [0x6c49, false], # East Asian ideograph
        0x273b79 => [0x5c9b, false], # East Asian ideograph
        0x234858 => [0x9462, false], # East Asian ideograph
        0x2f252d => [0x6a22, false], # East Asian ideograph
        0x6f575d => [0xc8a8, false], # Korean hangul
        0x3f4621 => [0x9a69, false], # East Asian ideograph
        0x274859 => [0x6d9f, false], # East Asian ideograph
        0x286622 => [0x7857, false], # East Asian ideograph
        0x22485a => [0x6d07, false], # East Asian ideograph
        0x4b4d7b => [0x77d7, false], # East Asian ideograph (variant of 214D7B which maps to 77D7)
        0x6f5b71 => [0xd31c, false], # Korean hangul
        0x213221 => [0x5018, false], # East Asian ideograph
        0x213222 => [0x4ff1, false], # East Asian ideograph
        0x22485b => [0x6d04, false], # East Asian ideograph
        0x273224 => [0x4e2a, false], # East Asian ideograph
        0x213225 => [0x5019, false], # East Asian ideograph
        0x273226 => [0x4f25, false], # East Asian ideograph
        0x223227 => [0x638e, false], # East Asian ideograph
        0x233228 => [0x8a4a, false], # East Asian ideograph
        0x22485c => [0x6cda, false], # East Asian ideograph
        0x23322a => [0x8a4e, false], # East Asian ideograph
        0x21322b => [0x4ffe, false], # East Asian ideograph
        0x21322c => [0x502a, false], # East Asian ideograph
        0x27322d => [0x4f26, false], # East Asian ideograph
        0x27322e => [0x4ec3, false], # East Asian ideograph (duplicate simplified)
        0x22322f => [0x6375, false], # East Asian ideograph
        0x223230 => [0x63af, false], # East Asian ideograph
        0x213231 => [0x5047, false], # East Asian ideograph
        0x213232 => [0x505a, false], # East Asian ideograph
        0x273233 => [0x4f1f, false], # East Asian ideograph
        0x213234 => [0x5043, false], # East Asian ideograph
        0x23485e => [0x945e, false], # East Asian ideograph
        0x213236 => [0x5076, false], # East Asian ideograph
        0x213237 => [0x504e, false], # East Asian ideograph
        0x223238 => [0x63b0, false], # East Asian ideograph
        0x223239 => [0x63ae, false], # East Asian ideograph
        0x22323a => [0x637c, false], # East Asian ideograph
        0x27485f => [0x6ede, false], # East Asian ideograph
        0x21323c => [0x5077, false], # East Asian ideograph
        0x22323d => [0x63ad, false], # East Asian ideograph
        0x27323e => [0x5bb6, false], # East Asian ideograph
        0x21323f => [0x5085, false], # East Asian ideograph
        0x273240 => [0x5907, false], # East Asian ideograph
        0x224860 => [0x6d2e, false], # East Asian ideograph
        0x233242 => [0x8a45, false], # East Asian ideograph
        0x273243 => [0x4f27, false], # East Asian ideograph
        0x273244 => [0x4f1e, false], # East Asian ideograph
        0x213245 => [0x50ad, false], # East Asian ideograph
        0x273246 => [0x4f20, false], # East Asian ideograph
        0x224861 => [0x6d35, false], # East Asian ideograph
        0x213248 => [0x50b2, false], # East Asian ideograph
        0x273249 => [0x4ec5, false], # East Asian ideograph
        0x27324a => [0x503e, false], # East Asian ideograph
        0x21324b => [0x50ac, false], # East Asian ideograph
        0x27324c => [0x4f24, false], # East Asian ideograph
        0x224862 => [0x6d3a, false], # East Asian ideograph
        0x21324e => [0x50e7, false], # East Asian ideograph
        0x22324f => [0x63bd, false], # East Asian ideograph
        0x223250 => [0x63c3, false], # East Asian ideograph
        0x273251 => [0x4fa5, false], # East Asian ideograph
        0x223252 => [0x63f5, false], # East Asian ideograph
        0x213253 => [0x50ed, false], # East Asian ideograph
        0x213254 => [0x50da, false], # East Asian ideograph
        0x273255 => [0x4ec6, false], # East Asian ideograph
        0x273256 => [0x4f2a, false], # East Asian ideograph
        0x273257 => [0x8c61, false], # East Asian ideograph
        0x273258 => [0x4fa8, false], # East Asian ideograph
        0x233259 => [0x8a82, false], # East Asian ideograph
        0x27325a => [0x4ebf, false], # East Asian ideograph
        0x22325b => [0x63e0, false], # East Asian ideograph
        0x22325c => [0x63d5, false], # East Asian ideograph
        0x23325d => [0x8a84, false], # East Asian ideograph
        0x23325e => [0x8a75, false], # East Asian ideograph
        0x274865 => [0x6e14, false], # East Asian ideograph
        0x273260 => [0x4fa9, false], # East Asian ideograph
        0x273261 => [0x4fed, false], # East Asian ideograph
        0x273262 => [0x50a7, false], # East Asian ideograph
        0x273263 => [0x5c3d, false], # East Asian ideograph (duplicate simplified)
        0x213264 => [0x5112, false], # East Asian ideograph
        0x273265 => [0x4fe6, false], # East Asian ideograph
        0x223266 => [0x63c5, false], # East Asian ideograph (not in Unicode)
        0x213267 => [0x511f, false], # East Asian ideograph
        0x213268 => [0x5121, false], # East Asian ideograph
        0x273269 => [0x50a8, false], # East Asian ideograph
        0x21326a => [0x5137, false], # East Asian ideograph
        0x27326b => [0x4fe8, false], # East Asian ideograph
        0x21326c => [0x5140, false], # East Asian ideograph
        0x21326d => [0x5143, false], # East Asian ideograph
        0x21326e => [0x5141, false], # East Asian ideograph
        0x23326f => [0x8a96, false], # East Asian ideograph
        0x213270 => [0x5144, false], # East Asian ideograph
        0x233271 => [0x8a9a, false], # East Asian ideograph
        0x213272 => [0x5149, false], # East Asian ideograph
        0x273273 => [0x51f6, false], # East Asian ideograph
        0x213274 => [0x5148, false], # East Asian ideograph
        0x274c3c => [0x8fed, false], # East Asian ideograph
        0x223276 => [0x63d1, false], # East Asian ideograph (not in Unicode)
        0x234869 => [0x946d, false], # East Asian ideograph
        0x213278 => [0x5155, false], # East Asian ideograph
        0x223279 => [0x63c4, false], # East Asian ideograph
        0x27327a => [0x513f, false], # East Asian ideograph
        0x27327b => [0x5156, false], # East Asian ideograph
        0x21327c => [0x515c, false], # East Asian ideograph
        0x22486a => [0x6d2b, false], # East Asian ideograph
        0x22327e => [0x6412, false], # East Asian ideograph
        0x2d4f7c => [0x7b5e, false], # East Asian ideograph
        0x22486b => [0x6d11, false], # East Asian ideograph
        0x27486c => [0x6cfc, false], # East Asian ideograph
        0x6f5838 => [0xc9dc, false], # Korean hangul
        0x21304d => [0x4e82, false], # East Asian ideograph
        0x22486d => [0x6d24, false], # East Asian ideograph
        0x6f4e5c => [0xb760, false], # Korean hangul
        0x235c65 => [0x9def, false], # East Asian ideograph
        0x293b3f => [0x8f7a, false], # East Asian ideograph
        0x27486e => [0x6da7, false], # East Asian ideograph
        0x6f5b75 => [0xd321, false], # Korean hangul
        0x27486f => [0x6d01, false], # East Asian ideograph
        0x6f4870 => [0xac1b, false], # Korean hangul
        0x6f4c49 => [0xb214, false], # Korean hangul
        0x234871 => [0x9477, false], # East Asian ideograph
        0x275954 => [0x5c82, false], # East Asian ideograph
        0x2f252e => [0x8507, false], # East Asian ideograph
        0x6f4872 => [0xac1d, false], # Korean hangul
        0x2d315f => [0x4fa3, false], # East Asian ideograph
        0x235622 => [0x9b35, false], # East Asian ideograph
        0x6f5b76 => [0xd325, false], # Korean hangul
        0x2d4874 => [0x6f5c, false], # East Asian ideograph
        0x6f4875 => [0xac24, false], # Korean hangul
        0x29457a => [0x9550, false], # East Asian ideograph
        0x6f4876 => [0xac2c, false], # Korean hangul
        0x227321 => [0x7e35, false], # East Asian ideograph
        0x224877 => [0x6da5, false], # East Asian ideograph
        0x213322 => [0x5168, false], # East Asian ideograph
        0x4b5361 => [0x89d2, false], # East Asian ideograph (duplicate simplified)
        0x274878 => [0x6e83, false], # East Asian ideograph
        0x213323 => [0x5169, false], # East Asian ideograph
        0x455e21 => [0x953a, false], # East Asian ideograph
        0x293271 => [0x8bee, false], # East Asian ideograph
        0x213324 => [0x516b, false], # East Asian ideograph
        0x22455b => [0x6bd6, false], # East Asian ideograph
        0x6f487a => [0xac31, false], # Korean hangul
        0x213325 => [0x516d, false], # East Asian ideograph
        0x23487b => [0x9482, false], # East Asian ideograph
        0x27373d => [0x5480, false], # East Asian ideograph
        0x27487c => [0x6d53, false], # East Asian ideograph
        0x213327 => [0x516c, false], # East Asian ideograph
        0x6f5d56 => [0xd6e0, false], # Korean hangul
        0x22487d => [0x6d92, false], # East Asian ideograph
        0x217328 => [0x568c, false], # East Asian ideograph
        0x6f487e => [0xac54, false], # Korean hangul
        0x213329 => [0x5175, false], # East Asian ideograph
        0x215f33 => [0x9694, false], # East Asian ideograph
        0x276225 => [0x9ccf, false], # East Asian ideograph
        0x2d332a => [0x4e0c, false], # East Asian ideograph
        0x2d3e2b => [0x6060, false], # East Asian ideograph
        0x21332b => [0x5177, false], # East Asian ideograph
        0x4b5f62 => [0x7668, false], # East Asian ideograph
        0x3f4629 => [0x4e97, false], # East Asian ideograph
        0x513051 => [0x8cae, false], # East Asian ideograph
        0x22332c => [0x6424, false], # East Asian ideograph
        0x274c3b => [0x7574, false], # East Asian ideograph
        0x23463c => [0x9389, false], # East Asian ideograph
        0x22732d => [0x7e52, false], # East Asian ideograph
        0x22534a => [0x719b, false], # East Asian ideograph
        0x6f5736 => [0xc804, false], # Korean hangul
        0x215f34 => [0x9699, false], # East Asian ideograph
        0x21332f => [0x5189, false], # East Asian ideograph
        0x6f4a6d => [0xaf3f, false], # Korean hangul
        0x275958 => [0x4e30, false], # East Asian ideograph
        0x233321 => [0x8abe, false], # East Asian ideograph
        0x223322 => [0x6410, false], # East Asian ideograph
        0x273323 => [0x4e24, false], # East Asian ideograph
        0x223324 => [0x6434, false], # East Asian ideograph
        0x233325 => [0x8acf, false], # East Asian ideograph
        0x213326 => [0x516e, false], # East Asian ideograph
        0x233327 => [0x8ac6, false], # East Asian ideograph
        0x213328 => [0x5171, false], # East Asian ideograph
        0x223329 => [0x641b, false], # East Asian ideograph
        0x21332a => [0x5176, false], # East Asian ideograph
        0x22332b => [0x6420, false], # East Asian ideograph
        0x23332c => [0x8ad1, false], # East Asian ideograph
        0x23332d => [0x8ad3, false], # East Asian ideograph
        0x21332e => [0x5180, false], # East Asian ideograph
        0x22332f => [0x6426, false], # East Asian ideograph
        0x213330 => [0x518c, false], # East Asian ideograph
        0x233331 => [0x8aaf, false], # East Asian ideograph
        0x213332 => [0x5192, false], # East Asian ideograph
        0x233333 => [0x8ad4, false], # East Asian ideograph
        0x213334 => [0x5195, false], # East Asian ideograph
        0x213335 => [0x6700, false], # East Asian ideograph
        0x213336 => [0x5197, false], # East Asian ideograph
        0x213337 => [0x51a0, false], # East Asian ideograph
        0x233338 => [0x8ab9, false], # East Asian ideograph
        0x223339 => [0x3013, false], # East Asian ideograph (not found in unified han)
        0x23333b => [0x8adb, false], # East Asian ideograph
        0x21333c => [0x51b0, false], # East Asian ideograph
        0x22333d => [0x6421, false], # East Asian ideograph
        0x21333e => [0x51b7, false], # East Asian ideograph
        0x23333f => [0x8ad0, false], # East Asian ideograph
        0x233340 => [0x8ad7, false], # East Asian ideograph
        0x213341 => [0x51cc, false], # East Asian ideograph
        0x233344 => [0x8af3, false], # East Asian ideograph
        0x233336 => [0x8acd, false], # East Asian ideograph
        0x213347 => [0x51f0, false], # East Asian ideograph
        0x273348 => [0x51ef, false], # East Asian ideograph
        0x233349 => [0x8b4c, false], # East Asian ideograph
        0x223337 => [0x6418, false], # East Asian ideograph
        0x22334c => [0x6409, false], # East Asian ideograph
        0x21334d => [0x51f8, false], # East Asian ideograph
        0x23334e => [0x8af6, false], # East Asian ideograph
        0x21334f => [0x5200, false], # East Asian ideograph
        0x213350 => [0x5201, false], # East Asian ideograph
        0x223338 => [0x640e, false], # East Asian ideograph
        0x213352 => [0x5207, false], # East Asian ideograph
        0x223353 => [0x6440, false], # East Asian ideograph
        0x213354 => [0x5208, false], # East Asian ideograph
        0x213355 => [0x520a, false], # East Asian ideograph
        0x233356 => [0x8b03, false], # East Asian ideograph
        0x233357 => [0x8ae4, false], # East Asian ideograph
        0x233359 => [0x8b14, false], # East Asian ideograph
        0x21335a => [0x5224, false], # East Asian ideograph
        0x21335b => [0x5225, false], # East Asian ideograph
        0x235749 => [0x9b86, false], # East Asian ideograph
        0x23335d => [0x8afc, false], # East Asian ideograph
        0x21335e => [0x5229, false], # East Asian ideograph
        0x21335f => [0x5238, false], # East Asian ideograph
        0x213360 => [0x523b, false], # East Asian ideograph
        0x213361 => [0x5237, false], # East Asian ideograph
        0x233362 => [0x8ade, false], # East Asian ideograph
        0x233363 => [0x8ae1, false], # East Asian ideograph
        0x233364 => [0x8b07, false], # East Asian ideograph
        0x2d537e => [0x81c8, false], # East Asian ideograph
        0x213366 => [0x5241, false], # East Asian ideograph
        0x213367 => [0x5239, false], # East Asian ideograph
        0x223368 => [0x645b, false], # East Asian ideograph
        0x213369 => [0x524d, false], # East Asian ideograph
        0x22336a => [0x644f, false], # East Asian ideograph
        0x27336b => [0x514b, false], # East Asian ideograph
        0x21336c => [0x524a, false], # East Asian ideograph
        0x27336d => [0x5219, false], # East Asian ideograph
        0x21336e => [0x525c, false], # East Asian ideograph
        0x22336f => [0x6476, false], # East Asian ideograph
        0x273370 => [0x521a, false], # East Asian ideograph
        0x215f37 => [0x969b, false], # East Asian ideograph
        0x213372 => [0x525d, false], # East Asian ideograph
        0x233373 => [0x8b16, false], # East Asian ideograph
        0x213374 => [0x526f, false], # East Asian ideograph
        0x213375 => [0x5272, false], # East Asian ideograph
        0x223376 => [0x6474, false], # East Asian ideograph
        0x213377 => [0x5269, false], # East Asian ideograph
        0x273378 => [0x521b, false], # East Asian ideograph
        0x233379 => [0x8b06, false], # East Asian ideograph
        0x23337a => [0x8b05, false], # East Asian ideograph
        0x21337b => [0x527f, false], # East Asian ideograph
        0x27337c => [0x5212, false], # East Asian ideograph
        0x21337d => [0x5288, false], # East Asian ideograph
        0x27337e => [0x5267, false], # East Asian ideograph
        0x213340 => [0x51cd, false], # East Asian ideograph
        0x217341 => [0x569c, false], # East Asian ideograph
        0x27484f => [0x6caa, false], # East Asian ideograph
        0x276226 => [0x9ca2, false], # East Asian ideograph
        0x234960 => [0x95d5, false], # East Asian ideograph
        0x6f773b => [0xc6fd, false], # Korean hangul
        0x213344 => [0x51dc, false], # East Asian ideograph
        0x23337c => [0x8b0f, false], # East Asian ideograph
        0x223345 => [0x6441, false], # East Asian ideograph
        0x6f5b7e => [0xd33c, false], # Korean hangul
        0x282e79 => [0x6079, false], # East Asian ideograph
        0x213e40 => [0x6046, false], # East Asian ideograph (variant of 4B3E40 which maps to 6046)
        0x286e68 => [0x7b3e, false], # East Asian ideograph
        0x22677b => [0x79b4, false], # East Asian ideograph
        0x224562 => [0x6bdc, false], # East Asian ideograph
        0x213348 => [0x51f1, false], # East Asian ideograph
        0x695626 => [0x4e62, false], # East Asian ideograph
        0x6f4a72 => [0xaf49, false], # Korean hangul
        0x213349 => [0x51f3, false], # East Asian ideograph
        0x513057 => [0x4e98, false], # East Asian ideograph
        0x21334b => [0x51fa, false], # East Asian ideograph
        0x6f573c => [0xc814, false], # Korean hangul
        0x23334c => [0x8add, false], # East Asian ideograph
        0x224563 => [0x6bdd, false], # East Asian ideograph
        0x234962 => [0x95d2, false], # East Asian ideograph
        0x6f2525 => [0x3160, false], # Korean hangul
        0x4c3a33 => [0x80ad, false], # East Asian ideograph (variant of 2E3A33 which maps to 80AD)
        0x276072 => [0x9975, false], # East Asian ideograph
        0x27595e => [0x4e88, false], # East Asian ideograph
        0x21734e => [0x56ac, false], # East Asian ideograph
        0x273745 => [0x55b7, false], # East Asian ideograph
        0x23334f => [0x8af4, false], # East Asian ideograph
        0x233350 => [0x8af5, false], # East Asian ideograph
        0x6f573d => [0xc815, false], # Korean hangul
        0x213351 => [0x5203, false], # East Asian ideograph
        0x395050 => [0x7bed, false], # East Asian ideograph
        0x22633a => [0x77d1, false], # East Asian ideograph
        0x287352 => [0x7f34, false], # East Asian ideograph
        0x6f4b71 => [0xb10b, false], # Korean hangul
        0x6f4a74 => [0xaf58, false], # Korean hangul
        0x233353 => [0x8adf, false], # East Asian ideograph
        0x4b3354 => [0x82c5, false], # East Asian ideograph
        0x4b3355 => [0x520b, false], # East Asian ideograph
        0x6f573e => [0xc816, false], # Korean hangul
        0x213356 => [0x5211, false], # East Asian ideograph
        0x224565 => [0x6bdf, false], # East Asian ideograph
        0x213357 => [0x5217, false], # East Asian ideograph
        0x2d5c48 => [0x9013, false], # East Asian ideograph
        0x273747 => [0x54dd, false], # East Asian ideograph
        0x213359 => [0x520e, false], # East Asian ideograph
        0x6f5d58 => [0xd6e8, false], # Korean hangul
        0x27735a => [0x55be, false], # East Asian ideograph
        0x2d4f41 => [0x4e69, false], # East Asian ideograph
        0x4b5d2b => [0x9162, false], # East Asian ideograph
        0x273421 => [0x5251, false], # East Asian ideograph
        0x213422 => [0x5289, false], # East Asian ideograph
        0x273423 => [0x5242, false], # East Asian ideograph
        0x223424 => [0x6464, false], # East Asian ideograph
        0x213425 => [0x529f, false], # East Asian ideograph
        0x213426 => [0x52a0, false], # East Asian ideograph
        0x223427 => [0x6482, false], # East Asian ideograph
        0x223428 => [0x645e, false], # East Asian ideograph
        0x21335c => [0x5220, false], # East Asian ideograph
        0x21342a => [0x52ac, false], # East Asian ideograph
        0x21342b => [0x52aa, false], # East Asian ideograph
        0x22342c => [0x647b, false], # East Asian ideograph
        0x23342d => [0x8b26, false], # East Asian ideograph
        0x22342e => [0x645c, false], # East Asian ideograph
        0x27342f => [0x52b2, false], # East Asian ideograph
        0x233430 => [0x8b33, false], # East Asian ideograph
        0x213431 => [0x52d8, false], # East Asian ideograph
        0x21305b => [0x4ea1, false], # East Asian ideograph
        0x273433 => [0x52a1, false], # East Asian ideograph
        0x273434 => [0x52a8, false], # East Asian ideograph
        0x273435 => [0x52b3, false], # East Asian ideograph
        0x273436 => [0x52cb, false], # East Asian ideograph
        0x213437 => [0x52dd, false], # East Asian ideograph
        0x273438 => [0x52bf, false], # East Asian ideograph
        0x213439 => [0x52e4, false], # East Asian ideograph
        0x23343a => [0x8b29, false], # East Asian ideograph
        0x2d335f => [0x52b5, false], # East Asian ideograph
        0x27343c => [0x52b1, false], # East Asian ideograph
        0x27343d => [0x529d, false], # East Asian ideograph
        0x21343e => [0x52fb, false], # East Asian ideograph
        0x22343f => [0x6499, false], # East Asian ideograph
        0x213440 => [0x52ff, false], # East Asian ideograph
        0x217360 => [0x56c5, false], # East Asian ideograph
        0x233442 => [0x8b48, false], # East Asian ideograph
        0x215f3e => [0x96bb, false], # East Asian ideograph
        0x213444 => [0x530d, false], # East Asian ideograph
        0x234966 => [0x95da, false], # East Asian ideograph
        0x213446 => [0x530f, false], # East Asian ideograph
        0x213447 => [0x5315, false], # East Asian ideograph
        0x213448 => [0x5316, false], # East Asian ideograph
        0x213449 => [0x5317, false], # East Asian ideograph
        0x23344a => [0x8b46, false], # East Asian ideograph
        0x21344b => [0x53f5, false], # East Asian ideograph
        0x21344c => [0x531d, false], # East Asian ideograph
        0x22344d => [0x6496, false], # East Asian ideograph
        0x21344e => [0x5320, false], # East Asian ideograph
        0x21344f => [0x5323, false], # East Asian ideograph
        0x213450 => [0x532a, false], # East Asian ideograph
        0x273451 => [0x6c47, false], # East Asian ideograph
        0x273452 => [0x532e, false], # East Asian ideograph
        0x213363 => [0x523a, false], # East Asian ideograph
        0x213454 => [0x533e, false], # East Asian ideograph
        0x273455 => [0x533a, false], # East Asian ideograph
        0x213456 => [0x533f, false], # East Asian ideograph
        0x213457 => [0x5341, false], # East Asian ideograph
        0x213458 => [0x5343, false], # East Asian ideograph
        0x213459 => [0x5345, false], # East Asian ideograph
        0x21345a => [0x5348, false], # East Asian ideograph
        0x22345b => [0x64b6, false], # East Asian ideograph
        0x21345c => [0x534a, false], # East Asian ideograph
        0x21345d => [0x5349, false], # East Asian ideograph (variant of 2D345D which maps to 5349)
        0x6f5741 => [0xc820, false], # Korean hangul
        0x27345f => [0x5346, false], # East Asian ideograph
        0x273460 => [0x534f, false], # East Asian ideograph
        0x213461 => [0x5353, false], # East Asian ideograph
        0x223462 => [0x649f, false], # East Asian ideograph
        0x213463 => [0x5357, false], # East Asian ideograph
        0x213464 => [0x535a, false], # East Asian ideograph
        0x223465 => [0x64a7, false], # East Asian ideograph (not in Unicode)
        0x213466 => [0x535e, false], # East Asian ideograph
        0x213467 => [0x5361, false], # East Asian ideograph
        0x233468 => [0x8b6b, false], # East Asian ideograph
        0x213469 => [0x5366, false], # East Asian ideograph
        0x22346a => [0x64d7, false], # East Asian ideograph
        0x21346b => [0x536e, false], # East Asian ideograph
        0x21346c => [0x5370, false], # East Asian ideograph
        0x21346d => [0x5371, false], # East Asian ideograph
        0x21346e => [0x537d, false], # East Asian ideograph
        0x21346f => [0x5375, false], # East Asian ideograph
        0x233470 => [0x8b78, false], # East Asian ideograph
        0x213368 => [0x5243, false], # East Asian ideograph
        0x213473 => [0x537b, false], # East Asian ideograph
        0x223474 => [0x64be, false], # East Asian ideograph
        0x223475 => [0x64d0, false], # East Asian ideograph
        0x213476 => [0x539a, false], # East Asian ideograph
        0x213477 => [0x539d, false], # East Asian ideograph
        0x213478 => [0x539f, false], # East Asian ideograph
        0x233479 => [0x8b81, false], # East Asian ideograph
        0x27347a => [0x538c, false], # East Asian ideograph
        0x27347b => [0x5389, false], # East Asian ideograph
        0x21347c => [0x53bb, false], # East Asian ideograph
        0x27347d => [0x53c2, false], # East Asian ideograph
        0x21347e => [0x53c8, false], # East Asian ideograph
        0x334968 => [0x7133, false], # East Asian ideograph
        0x23336b => [0x8b0c, false], # East Asian ideograph
        0x6f4b72 => [0xb10c, false], # Korean hangul
        0x6f4a79 => [0xaf79, false], # Korean hangul
        0x22336c => [0x646b, false], # East Asian ideograph
        0x225676 => [0x72eb, false], # East Asian ideograph
        0x29583e => [0x9cca, false], # East Asian ideograph
        0x21736d => [0x56dd, false], # East Asian ideograph
        0x2d4f45 => [0x9834, false], # East Asian ideograph
        0x274858 => [0x6ee1, false], # East Asian ideograph
        0x6f5743 => [0xc82c, false], # Korean hangul
        0x23336f => [0x8b1c, false], # East Asian ideograph
        0x234969 => [0x95de, false], # East Asian ideograph
        0x694677 => [0x5302, false], # East Asian ideograph
        0x217370 => [0x56df, false], # East Asian ideograph
        0x6f4a7a => [0xaf80, false], # Korean hangul
        0x213371 => [0x5254, false], # East Asian ideograph
        0x333377 => [0x5270, false], # East Asian ideograph
        0x2d3372 => [0x5265, false], # East Asian ideograph
        0x6f5d59 => [0xd6f0, false], # Korean hangul
        0x6f502a => [0xba53, false], # Korean hangul
        0x696f27 => [0x933b, false], # East Asian ideograph
        0x295c65 => [0x9e69, false], # East Asian ideograph
        0x213373 => [0x526a, false], # East Asian ideograph
        0x395e2f => [0x5277, false], # East Asian ideograph
        0x287374 => [0x7f35, false], # East Asian ideograph
        0x225f3c => [0x760a, false], # East Asian ideograph
        0x23496a => [0x95e0, false], # East Asian ideograph
        0x217375 => [0x56eb, false], # East Asian ideograph
        0x334527 => [0x6918, false], # East Asian ideograph
        0x273376 => [0x5240, false], # East Asian ideograph
        0x6f516a => [0xbe1c, false], # Korean hangul
        0x275e21 => [0x949f, false], # East Asian ideograph
        0x2d3377 => [0x8cf8, false], # East Asian ideograph
        0x215e22 => [0x9318, false], # East Asian ideograph
        0x27615f => [0x53d1, false], # East Asian ideograph (duplicate simplified)
        0x233378 => [0x8b0b, false], # East Asian ideograph
        0x215e23 => [0x936c, false], # East Asian ideograph
        0x27485a => [0x6e10, false], # East Asian ideograph
        0x275e24 => [0x953b, false], # East Asian ideograph
        0x21337a => [0x527d, false], # East Asian ideograph
        0x225e25 => [0x7583, false], # East Asian ideograph
        0x6f583c => [0xc9e4, false], # Korean hangul
        0x22337b => [0x6473, false], # East Asian ideograph
        0x27374e => [0x549b, false], # East Asian ideograph
        0x2d5e26 => [0x7194, false], # East Asian ideograph
        0x293f4c => [0x90d3, false], # East Asian ideograph
        0x21337c => [0x5283, false], # East Asian ideograph
        0x215e27 => [0x93ae, false], # East Asian ideograph
        0x335635 => [0x85ac, false], # East Asian ideograph
        0x3f3f24 => [0x614e, false], # East Asian ideograph
        0x23337d => [0x8b10, false], # East Asian ideograph
        0x215e28 => [0x9396, false], # East Asian ideograph
        0x6f5746 => [0xc838, false], # Korean hangul
        0x21337e => [0x5287, false], # East Asian ideograph
        0x275e29 => [0x94a8, false], # East Asian ideograph
        0x6f4c4d => [0xb233, false], # Korean hangul
        0x215e2a => [0x93b3, false], # East Asian ideograph
        0x215e2b => [0x93e1, false], # East Asian ideograph
        0x213062 => [0x4ead, false], # East Asian ideograph
        0x692564 => [0x30e4, false], # Katakana letter YA
        0x223461 => [0x6498, false], # East Asian ideograph
        0x29516d => [0x9991, false], # East Asian ideograph
        0x215e2c => [0x93d1, false], # East Asian ideograph
        0x225e2d => [0x7592, false], # East Asian ideograph
        0x4c4d63 => [0x6f99, false], # East Asian ideograph
        0x6f5747 => [0xc83c, false], # Korean hangul
        0x215e2e => [0x93c3, false], # East Asian ideograph
        0x213d2c => [0x5ee0, false], # East Asian ideograph
        0x275e2f => [0x94f2, false], # East Asian ideograph
        0x2d6056 => [0x980b, false], # East Asian ideograph
        0x6f4a7e => [0xaf95, false], # Korean hangul
        0x275969 => [0x8d1e, false], # East Asian ideograph
        0x215e30 => [0x93d7, false], # East Asian ideograph
        0x213522 => [0x53cb, false], # East Asian ideograph
        0x233523 => [0x8b8b, false], # East Asian ideograph
        0x213524 => [0x53cd, false], # East Asian ideograph
        0x213525 => [0x53d6, false], # East Asian ideograph
        0x233526 => [0x8b87, false], # East Asian ideograph
        0x225e31 => [0x7595, false], # East Asian ideograph
        0x213528 => [0x53db, false], # East Asian ideograph
        0x213529 => [0x53df, false], # East Asian ideograph
        0x22352a => [0x64ef, false], # East Asian ideograph
        0x27352b => [0x4e1b, false], # East Asian ideograph
        0x21352c => [0x53e3, false], # East Asian ideograph
        0x22352d => [0x64e1, false], # East Asian ideograph
        0x22352e => [0x64e5, false], # East Asian ideograph
        0x224873 => [0x6d63, false], # East Asian ideograph
        0x213530 => [0x53ef, false], # East Asian ideograph
        0x213531 => [0x53e9, false], # East Asian ideograph
        0x213532 => [0x53f3, false], # East Asian ideograph
        0x223533 => [0x64e2, false], # East Asian ideograph
        0x213534 => [0x53e8, false], # East Asian ideograph
        0x213535 => [0x53e6, false], # East Asian ideograph
        0x223536 => [0x64ed, false], # East Asian ideograph
        0x233537 => [0x8b9c, false], # East Asian ideograph
        0x223538 => [0x64e4, false], # East Asian ideograph
        0x275e34 => [0x9542, false], # East Asian ideograph
        0x21353a => [0x53f1, false], # East Asian ideograph
        0x21353b => [0x53ed, false], # East Asian ideograph
        0x21353c => [0x53ea, false], # East Asian ideograph
        0x23353d => [0x8c3a, false], # East Asian ideograph
        0x235e35 => [0x9ec6, false], # East Asian ideograph
        0x213540 => [0x5409, false], # East Asian ideograph
        0x213541 => [0x5410, false], # East Asian ideograph
        0x213542 => [0x540f, false], # East Asian ideograph
        0x235a7b => [0x9d59, false], # East Asian ideograph
        0x233544 => [0x8c40, false], # East Asian ideograph
        0x233545 => [0x8c42, false], # East Asian ideograph
        0x213546 => [0x5404, false], # East Asian ideograph
        0x213547 => [0x5403, false], # East Asian ideograph
        0x213548 => [0x5412, false], # East Asian ideograph
        0x2d7164 => [0x55d4, false], # East Asian ideograph (variant of 217164 which maps to 55D4)
        0x21354a => [0x5406, false], # East Asian ideograph (not in Unicode)
        0x23354b => [0x8c47, false], # East Asian ideograph
        0x21354d => [0x542d, false], # East Asian ideograph
        0x21354e => [0x541d, false], # East Asian ideograph
        0x21354f => [0x541e, false], # East Asian ideograph
        0x213550 => [0x541b, false], # East Asian ideograph
        0x213551 => [0x544e, false], # East Asian ideograph
        0x233552 => [0x8c55, false], # East Asian ideograph
        0x23496f => [0x95e5, false], # East Asian ideograph
        0x233554 => [0x8c57, false], # East Asian ideograph
        0x213555 => [0x5431, false], # East Asian ideograph
        0x233556 => [0x8c5d, false], # East Asian ideograph
        0x213557 => [0x543c, false], # East Asian ideograph
        0x213558 => [0x5443, false], # East Asian ideograph
        0x213559 => [0x5426, false], # East Asian ideograph
        0x21355a => [0x5420, false], # East Asian ideograph
        0x22355b => [0x6516, false], # East Asian ideograph
        0x23355c => [0x86c3, false], # East Asian ideograph
        0x21355d => [0x5435, false], # East Asian ideograph
        0x234e26 => [0x97b5, false], # East Asian ideograph
        0x21355f => [0x544a, false], # East Asian ideograph
        0x213560 => [0x5448, false], # East Asian ideograph
        0x223561 => [0x651b, false], # East Asian ideograph
        0x213562 => [0x5438, false], # East Asian ideograph
        0x233563 => [0x8c68, false], # East Asian ideograph
        0x213564 => [0x5442, false], # East Asian ideograph
        0x233565 => [0x8c6d, false], # East Asian ideograph
        0x213566 => [0x541f, false], # East Asian ideograph
        0x213567 => [0x5429, false], # East Asian ideograph
        0x213568 => [0x5473, false], # East Asian ideograph
        0x223569 => [0x6527, false], # East Asian ideograph
        0x21356a => [0x5475, false], # East Asian ideograph
        0x21356b => [0x5495, false], # East Asian ideograph
        0x21356c => [0x5478, false], # East Asian ideograph
        0x22356d => [0x6522, false], # East Asian ideograph
        0x21356e => [0x5477, false], # East Asian ideograph
        0x22356f => [0x6529, false], # East Asian ideograph
        0x213571 => [0x5492, false], # East Asian ideograph
        0x223572 => [0x6525, false], # East Asian ideograph
        0x213573 => [0x547c, false], # East Asian ideograph
        0x233574 => [0x8c76, false], # East Asian ideograph
        0x225e3e => [0x75ba, false], # East Asian ideograph
        0x213576 => [0x548b, false], # East Asian ideograph
        0x213577 => [0x548c, false], # East Asian ideograph
        0x213578 => [0x5490, false], # East Asian ideograph
        0x213579 => [0x547d, false], # East Asian ideograph
        0x21357a => [0x5476, false], # East Asian ideograph
        0x23357b => [0x8c78, false], # East Asian ideograph
        0x22357c => [0x6541, false], # East Asian ideograph
        0x23357d => [0x8c7b, false], # East Asian ideograph
        0x21357e => [0x54a9, false], # East Asian ideograph
        0x275e40 => [0x956f, false], # East Asian ideograph
        0x23563a => [0x9b48, false], # East Asian ideograph
        0x333421 => [0x91fc, false], # East Asian ideograph
        0x6f574b => [0xc874, false], # Korean hangul
        0x234566 => [0x9355, false], # East Asian ideograph
        0x235e42 => [0x9ecc, false], # East Asian ideograph
        0x213d30 => [0x5ef7, false], # East Asian ideograph
        0x6f4c4e => [0xb234, false], # Korean hangul
        0x225e43 => [0x75b0, false], # East Asian ideograph
        0x225e44 => [0x75c3, false], # East Asian ideograph
        0x27552a => [0x82cb, false], # East Asian ideograph
        0x6f5763 => [0xc8cc, false], # Korean hangul
        0x275e45 => [0x94c4, false], # East Asian ideograph
        0x225e46 => [0x75bf, false], # East Asian ideograph
        0x2d5927 => [0x8acc, false], # East Asian ideograph
        0x234c38 => [0x971b, false], # East Asian ideograph
        0x6f574c => [0xc878, false], # Korean hangul
        0x225e47 => [0x75b4, false], # East Asian ideograph
        0x6f5d29 => [0xd5f5, false], # Korean hangul
        0x6f4b74 => [0xb110, false], # Korean hangul
        0x23452f => [0x9342, false], # East Asian ideograph
        0x27596e => [0x8d2f, false], # East Asian ideograph
        0x273755 => [0x5411, false], # East Asian ideograph
        0x275e49 => [0x9523, false], # East Asian ideograph
        0x215e4a => [0x947d, false], # East Asian ideograph
        0x23563c => [0x9b4e, false], # East Asian ideograph
        0x333423 => [0x5264, false], # East Asian ideograph
        0x275e4b => [0x51ff, false], # East Asian ideograph
        0x6f574d => [0xc87a, false], # Korean hangul
        0x235e4c => [0x9ed3, false], # East Asian ideograph
        0x275e4d => [0x95e8, false], # East Asian ideograph
        0x34492f => [0x6d34, false], # East Asian ideograph
        0x225e4e => [0x75c1, false], # East Asian ideograph
        0x277745 => [0x57d9, false], # East Asian ideograph
        0x275e4f => [0x95ea, false], # East Asian ideograph
        0x4c6f43 => [0x7ccd, false], # East Asian ideograph
        0x225e50 => [0x75b1, false], # East Asian ideograph
        0x274863 => [0x6d46, false], # East Asian ideograph
        0x6f574e => [0xc880, false], # Korean hangul
        0x284c62 => [0x988d, false], # East Asian ideograph
        0x225e51 => [0x75c4, false], # East Asian ideograph
        0x213d33 => [0x5efa, false], # East Asian ideograph
        0x275e52 => [0x95f0, false], # East Asian ideograph
        0x275970 => [0x8d2a, false], # East Asian ideograph
        0x215e53 => [0x958b, false], # East Asian ideograph
        0x275e54 => [0x95f2, false], # East Asian ideograph
        0x23563e => [0x9b4d, false], # East Asian ideograph
        0x215e55 => [0x9593, false], # East Asian ideograph
        0x274864 => [0x6e17, false], # East Asian ideograph
        0x6f574f => [0xc881, false], # Korean hangul
        0x6f4d29 => [0xb355, false], # Korean hangul
        0x235e57 => [0x9ee3, false], # East Asian ideograph
        0x275971 => [0x8d2b, false], # East Asian ideograph
        0x225e58 => [0x75cd, false], # East Asian ideograph
        0x215e59 => [0x95a8, false], # East Asian ideograph
        0x275e5a => [0x95fd, false], # East Asian ideograph
        0x6f7727 => [0xadd5, false], # Korean hangul
        0x213621 => [0x54aa, false], # East Asian ideograph
        0x213622 => [0x54a8, false], # East Asian ideograph
        0x213623 => [0x54ac, false], # East Asian ideograph
        0x213624 => [0x54c0, false], # East Asian ideograph
        0x213625 => [0x54b3, false], # East Asian ideograph
        0x213626 => [0x54a6, false], # East Asian ideograph
        0x213627 => [0x54ab, false], # East Asian ideograph
        0x213628 => [0x54c7, false], # East Asian ideograph
        0x213629 => [0x54c9, false], # East Asian ideograph
        0x21362a => [0x54c4, false], # East Asian ideograph
        0x21362b => [0x54c2, false], # East Asian ideograph
        0x22362c => [0x6538, false], # East Asian ideograph
        0x21362d => [0x54c1, false], # East Asian ideograph
        0x23362e => [0x8c88, false], # East Asian ideograph
        0x21362f => [0x54ce, false], # East Asian ideograph
        0x213630 => [0x54b1, false], # East Asian ideograph
        0x213631 => [0x54bb, false], # East Asian ideograph
        0x213632 => [0x54af, false], # East Asian ideograph
        0x213633 => [0x54c8, false], # East Asian ideograph
        0x223634 => [0x6542, false], # East Asian ideograph
        0x225e5e => [0x75cc, false], # East Asian ideograph
        0x213636 => [0x5510, false], # East Asian ideograph
        0x213637 => [0x54ea, false], # East Asian ideograph
        0x213638 => [0x5514, false], # East Asian ideograph
        0x233639 => [0x8c94, false], # East Asian ideograph
        0x21363a => [0x54e5, false], # East Asian ideograph
        0x225e5f => [0x75d0, false], # East Asian ideograph
        0x21363c => [0x54f2, false], # East Asian ideograph
        0x21363d => [0x54e8, false], # East Asian ideograph
        0x21363e => [0x54e1, false], # East Asian ideograph
        0x22363f => [0x6555, false], # East Asian ideograph
        0x213640 => [0x54ed, false], # East Asian ideograph
        0x233641 => [0x8c9b, false], # East Asian ideograph
        0x213642 => [0x5509, false], # East Asian ideograph
        0x213643 => [0x54e6, false], # East Asian ideograph
        0x233644 => [0x8ca4, false], # East Asian ideograph
        0x223645 => [0x6567, false], # East Asian ideograph
        0x213646 => [0x5546, false], # East Asian ideograph
        0x223647 => [0x6561, false], # East Asian ideograph
        0x213648 => [0x554f, false], # East Asian ideograph
        0x273649 => [0x54d1, false], # East Asian ideograph
        0x21364a => [0x5566, false], # East Asian ideograph
        0x21364b => [0x556a, false], # East Asian ideograph
        0x21364c => [0x554a, false], # East Asian ideograph
        0x21364d => [0x5544, false], # East Asian ideograph
        0x21364e => [0x555c, false], # East Asian ideograph
        0x22364f => [0x656d, false], # East Asian ideograph
        0x213650 => [0x5543, false], # East Asian ideograph
        0x213651 => [0x552c, false], # East Asian ideograph
        0x213652 => [0x5561, false], # East Asian ideograph
        0x233653 => [0x8cb9, false], # East Asian ideograph
        0x223654 => [0x657a, false], # East Asian ideograph
        0x213655 => [0x5555, false], # East Asian ideograph
        0x213656 => [0x552f, false], # East Asian ideograph
        0x233657 => [0x8ccd, false], # East Asian ideograph
        0x213658 => [0x5564, false], # East Asian ideograph
        0x213659 => [0x5538, false], # East Asian ideograph
        0x21365a => [0x55a7, false], # East Asian ideograph
        0x21365b => [0x5580, false], # East Asian ideograph
        0x21365c => [0x557b, false], # East Asian ideograph
        0x21365d => [0x557c, false], # East Asian ideograph
        0x21365e => [0x5527, false], # East Asian ideograph
        0x21365f => [0x5594, false], # East Asian ideograph
        0x213660 => [0x5587, false], # East Asian ideograph
        0x213661 => [0x559c, false], # East Asian ideograph
        0x213662 => [0x558b, false], # East Asian ideograph
        0x273663 => [0x4e27, false], # East Asian ideograph
        0x213664 => [0x55b3, false], # East Asian ideograph
        0x225e66 => [0x75e1, false], # East Asian ideograph
        0x213666 => [0x5583, false], # East Asian ideograph
        0x213667 => [0x55b1, false], # East Asian ideograph
        0x273668 => [0x5355, false], # East Asian ideograph
        0x213669 => [0x5582, false], # East Asian ideograph
        0x21366a => [0x559f, false], # East Asian ideograph
        0x225e67 => [0x75e6, false], # East Asian ideograph
        0x21366c => [0x5598, false], # East Asian ideograph
        0x21366d => [0x559a, false], # East Asian ideograph
        0x22366e => [0x658c, false], # East Asian ideograph
        0x27366f => [0x4e54, false], # East Asian ideograph
        0x223670 => [0x6592, false], # East Asian ideograph
        0x213671 => [0x55b2, false], # East Asian ideograph
        0x233672 => [0x8cdd, false], # East Asian ideograph
        0x213673 => [0x55e8, false], # East Asian ideograph
        0x233674 => [0x8cd9, false], # East Asian ideograph
        0x223675 => [0x659b, false], # East Asian ideograph
        0x213676 => [0x55dc, false], # East Asian ideograph
        0x223677 => [0x659d, false], # East Asian ideograph
        0x213678 => [0x55c7, false], # East Asian ideograph
        0x213679 => [0x55d3, false], # East Asian ideograph
        0x21367a => [0x55ce, false], # East Asian ideograph
        0x21367b => [0x55e3, false], # East Asian ideograph
        0x23367c => [0x8cf5, false], # East Asian ideograph
        0x21367d => [0x55e4, false], # East Asian ideograph
        0x23367e => [0x8cfb, false], # East Asian ideograph
        0x232760 => [0x8600, false], # East Asian ideograph
        0x275e6b => [0x8f9f, false], # East Asian ideograph (duplicate simplified)
        0x285424 => [0x70e8, false], # East Asian ideograph
        0x27375c => [0x556d, false], # East Asian ideograph
        0x4b5e6c => [0x961d, false], # East Asian ideograph (duplicate simplified)
        0x293f5a => [0x90e7, false], # East Asian ideograph
        0x235e6f => [0x9ef6, false], # East Asian ideograph
        0x4b5422 => [0x81d3, false], # East Asian ideograph
        0x6f583f => [0xc9ec, false], # Korean hangul
        0x27375d => [0x55eb, false], # East Asian ideograph
        0x225e71 => [0x75e4, false], # East Asian ideograph
        0x225e72 => [0x75e0, false], # East Asian ideograph
        0x4b4759 => [0x6d99, false], # East Asian ideograph
        0x6f4b66 => [0xb0c7, false], # Korean hangul
        0x225e73 => [0x75d7, false], # East Asian ideograph
        0x6f5755 => [0xc88d, false], # Korean hangul
        0x235e74 => [0x9ef9, false], # East Asian ideograph
        0x275977 => [0x8d3a, false], # East Asian ideograph
        0x27375e => [0x56a3, false], # East Asian ideograph
        0x235e76 => [0x9efb, false], # East Asian ideograph
        0x274921 => [0x6cfd, false], # East Asian ideograph
        0x293f5c => [0x90ac, false], # East Asian ideograph
        0x2d616a => [0x6b1d, false], # East Asian ideograph
        0x215e77 => [0x964c, false], # East Asian ideograph
        0x274922 => [0x6d4a, false], # East Asian ideograph
        0x6f5756 => [0xc890, false], # Korean hangul
        0x6f4924 => [0xac74, false], # Korean hangul
        0x6f4b76 => [0xb118, false], # Korean hangul
        0x215e7a => [0x9662, false], # East Asian ideograph
        0x224925 => [0x6d6d, false], # East Asian ideograph
        0x275978 => [0x8d35, false], # East Asian ideograph
        0x235e7b => [0x9efe, false], # East Asian ideograph (not in Unicode)
        0x274926 => [0x6d4e, false], # East Asian ideograph
        0x215e7c => [0x965b, false], # East Asian ideograph
        0x274927 => [0x6cde, false], # East Asian ideograph
        0x235e7d => [0x9f02, false], # East Asian ideograph
        0x274928 => [0x6ee8, false], # East Asian ideograph
        0x6f5757 => [0xc894, false], # Korean hangul
        0x215e7e => [0x965d, false], # East Asian ideograph
        0x224929 => [0x6d91, false], # East Asian ideograph
        0x287065 => [0x7ec2, false], # East Asian ideograph
        0x2f4231 => [0x8019, false], # Unrelated variant of EACC 215266 which maps to 8019
        0x6f492a => [0xac81, false], # Korean hangul
        0x33492e => [0x6f81, false], # East Asian ideograph
        0x27492b => [0x6ee5, false], # East Asian ideograph
        0x33337b => [0x52e6, false], # East Asian ideograph
        0x233871 => [0x8dc2, false], # East Asian ideograph
        0x22492c => [0x6d81, false], # East Asian ideograph
        0x235647 => [0x9b51, false], # East Asian ideograph
        0x33463c => [0x6bbb, false], # East Asian ideograph
        0x27492d => [0x6d9b, false], # East Asian ideograph
        0x6f553a => [0xc587, false], # Korean hangul
        0x27492e => [0x6da9, false], # East Asian ideograph
        0x22413c => [0x6a5a, false], # East Asian ideograph
        0x2e492f => [0x6cd9, false], # East Asian ideograph
        0x27597a => [0x4e70, false], # East Asian ideograph
        0x213331 => [0x518d, false], # East Asian ideograph
        0x273761 => [0x7f57, false], # East Asian ideograph (duplicate simplified)
        0x213721 => [0x55da, false], # East Asian ideograph
        0x223722 => [0x65a8, false], # East Asian ideograph
        0x223723 => [0x65a6, false], # East Asian ideograph
        0x213724 => [0x5600, false], # East Asian ideograph
        0x233725 => [0x8d04, false], # East Asian ideograph
        0x213726 => [0x55fe, false], # East Asian ideograph
        0x273727 => [0x5567, false], # East Asian ideograph
        0x213728 => [0x55f7, false], # East Asian ideograph
        0x213729 => [0x5608, false], # East Asian ideograph
        0x22372a => [0x65b6, false], # East Asian ideograph
        0x21372b => [0x55fd, false], # East Asian ideograph
        0x22372c => [0x65b8, false], # East Asian ideograph
        0x23372d => [0x8d09, false], # East Asian ideograph
        0x21372e => [0x5614, false], # East Asian ideograph
        0x22372f => [0x65bf, false], # East Asian ideograph
        0x273730 => [0x5c1d, false], # East Asian ideograph
        0x273731 => [0x55bd, false], # East Asian ideograph
        0x273732 => [0x5520, false], # East Asian ideograph
        0x213733 => [0x562f, false], # East Asian ideograph
        0x223734 => [0x65c2, false], # East Asian ideograph
        0x213735 => [0x5636, false], # East Asian ideograph
        0x213736 => [0x5632, false], # East Asian ideograph
        0x213737 => [0x563b, false], # East Asian ideograph
        0x213738 => [0x5639, false], # East Asian ideograph
        0x274934 => [0x6e85, false], # East Asian ideograph
        0x23373a => [0x8d10, false], # East Asian ideograph
        0x22373b => [0x65d0, false], # East Asian ideograph
        0x22373c => [0x65d2, false], # East Asian ideograph
        0x21373d => [0x5634, false], # East Asian ideograph
        0x23373e => [0x8d18, false], # East Asian ideograph
        0x224935 => [0x6def, false], # East Asian ideograph
        0x213740 => [0x5630, false], # East Asian ideograph
        0x213741 => [0x566b, false], # East Asian ideograph
        0x213742 => [0x5664, false], # East Asian ideograph
        0x213743 => [0x5669, false], # East Asian ideograph
        0x223744 => [0x65db, false], # East Asian ideograph
        0x213745 => [0x5674, false], # East Asian ideograph
        0x273746 => [0x5f53, false], # East Asian ideograph (duplicate simplified)
        0x213747 => [0x5665, false], # East Asian ideograph
        0x213748 => [0x566a, false], # East Asian ideograph
        0x213749 => [0x5668, false], # East Asian ideograph
        0x22374a => [0x65e1, false], # East Asian ideograph
        0x27374b => [0x55f3, false], # East Asian ideograph
        0x225a23 => [0x7429, false], # East Asian ideograph
        0x21374d => [0x566c, false], # East Asian ideograph
        0x21374e => [0x5680, false], # East Asian ideograph
        0x21374f => [0x568e, false], # East Asian ideograph
        0x213750 => [0x5685, false], # East Asian ideograph
        0x214938 => [0x701b, false], # East Asian ideograph
        0x233752 => [0x8d78, false], # East Asian ideograph
        0x213753 => [0x568f, false], # East Asian ideograph
        0x223754 => [0x65f4, false], # East Asian ideograph
        0x213755 => [0x56ae, false], # East Asian ideograph (variant of 453755 which maps to 56AE)
        0x273756 => [0x5499, false], # East Asian ideograph
        0x224939 => [0x6d7f, false], # East Asian ideograph
        0x213758 => [0x56a5, false], # East Asian ideograph
        0x213759 => [0x56b7, false], # East Asian ideograph
        0x22375a => [0x6609, false], # East Asian ideograph
        0x27375b => [0x5624, false], # East Asian ideograph
        0x21375c => [0x56c0, false], # East Asian ideograph
        0x21493a => [0x7028, false], # East Asian ideograph
        0x22375e => [0x660a, false], # East Asian ideograph
        0x21375f => [0x56bc, false], # East Asian ideograph
        0x213760 => [0x56ca, false], # East Asian ideograph
        0x213761 => [0x56c9, false], # East Asian ideograph
        0x273762 => [0x5453, false], # East Asian ideograph
        0x22493b => [0x6d85, false], # East Asian ideograph
        0x223764 => [0x6603, false], # East Asian ideograph
        0x213765 => [0x56db, false], # East Asian ideograph
        0x213766 => [0x56da, false], # East Asian ideograph
        0x213767 => [0x56e0, false], # East Asian ideograph
        0x213768 => [0x56de, false], # East Asian ideograph
        0x21493c => [0x7015, false], # East Asian ideograph
        0x22376a => [0x6611, false], # East Asian ideograph
        0x22376b => [0x6615, false], # East Asian ideograph
        0x21376c => [0x56fa, false], # East Asian ideograph
        0x22376d => [0x6604, false], # East Asian ideograph
        0x22376e => [0x6631, false], # East Asian ideograph
        0x21376f => [0x570b, false], # East Asian ideograph
        0x213770 => [0x570d, false], # East Asian ideograph
        0x233771 => [0x8d94, false], # East Asian ideograph
        0x223772 => [0x6621, false], # East Asian ideograph
        0x273773 => [0x56e2, false], # East Asian ideograph
        0x273774 => [0x56fe, false], # East Asian ideograph
        0x223775 => [0x662c, false], # East Asian ideograph
        0x223777 => [0x6635, false], # East Asian ideograph
        0x213778 => [0x572f, false], # East Asian ideograph
        0x213779 => [0x5730, false], # East Asian ideograph
        0x21377a => [0x5728, false], # East Asian ideograph
        0x21377b => [0x5733, false], # East Asian ideograph
        0x22377c => [0x661e, false], # East Asian ideograph
        0x22377d => [0x663a, false], # East Asian ideograph
        0x224940 => [0x6d67, false], # East Asian ideograph
        0x274941 => [0x6d12, false], # East Asian ideograph
        0x2e3144 => [0x651f, false], # East Asian ideograph
        0x274942 => [0x6ee9, false], # East Asian ideograph
        0x212a34 => [0xe8e1, false], # EACC component character
        0x214943 => [0x7063, false], # East Asian ideograph
        0x274944 => [0x6ee6, false], # East Asian ideograph
        0x4b5a3b => [0x8d08, false], # East Asian ideograph
        0x4b4761 => [0x6e05, false], # East Asian ideograph
        0x213f21 => [0x6148, false], # East Asian ideograph
        0x224946 => [0x6d60, false], # East Asian ideograph
        0x2d4b35 => [0x73c9, false], # East Asian ideograph
        0x2d4947 => [0x7ac8, false], # East Asian ideograph
        0x394c2d => [0x7546, false], # East Asian ideograph
        0x224948 => [0x6d98, false], # East Asian ideograph
        0x6f516f => [0xbe48, false], # Korean hangul
        0x234949 => [0x95be, false], # East Asian ideograph
        0x217068 => [0x5530, false], # East Asian ideograph
        0x295d3a => [0x9e73, false], # East Asian ideograph
        0x27494a => [0x707e, false], # East Asian ideograph
        0x225352 => [0x71a0, false], # East Asian ideograph
        0x234644 => [0x93bd, false], # East Asian ideograph
        0x22494b => [0x6d7c, false], # East Asian ideograph
        0x6f575e => [0xc8ac, false], # Korean hangul
        0x22494c => [0x6d70, false], # East Asian ideograph
        0x21494d => [0x7092, false], # East Asian ideograph
        0x23494e => [0x95ba, false], # East Asian ideograph
        0x295d3b => [0x9e42, false], # East Asian ideograph
        0x23494f => [0x95b6, false], # East Asian ideograph
        0x2e3e3f => [0x7ba0, false], # East Asian ideograph
        0x234950 => [0x95bf, false], # East Asian ideograph
        0x225278 => [0x7178, false], # East Asian ideograph
        0x274951 => [0x4e3a, false], # East Asian ideograph
        0x213d44 => [0x5f29, false], # East Asian ideograph
        0x334674 => [0x76c5, false], # East Asian ideograph
        0x234952 => [0x95bd, false], # East Asian ideograph
        0x6f4953 => [0xacf1, false], # Korean hangul
        0x21392a => [0x592e, false], # East Asian ideograph
        0x295d3c => [0x5364, false], # East Asian ideograph
        0x2d4954 => [0x70f1, false], # East Asian ideograph
        0x6f4955 => [0xacf5, false], # Korean hangul
        0x22526b => [0x7153, false], # East Asian ideograph
        0x6f5760 => [0xc8b8, false], # Korean hangul
        0x2d4956 => [0x70b0, false], # East Asian ideograph
        0x213d45 => [0x5f2d, false], # East Asian ideograph
        0x4b5871 => [0x8aa4, false], # East Asian ideograph
        0x6f4b78 => [0xb11b, false], # Korean hangul
        0x6f4957 => [0xacfa, false], # Korean hangul
        0x6f4958 => [0xacfc, false], # Korean hangul
        0x284339 => [0x680a, false], # East Asian ideograph
        0x22746a => [0x7f7f, false], # East Asian ideograph
        0x225251 => [0x7146, false], # East Asian ideograph
        0x234959 => [0x95c9, false], # East Asian ideograph
        0x22495a => [0x6db4, false], # East Asian ideograph
        0x6f5761 => [0xc8c4, false], # Korean hangul
        0x213821 => [0x5740, false], # East Asian ideograph
        0x233822 => [0x8d96, false], # East Asian ideograph
        0x213823 => [0x574d, false], # East Asian ideograph
        0x213824 => [0x573e, false], # East Asian ideograph
        0x213825 => [0x574e, false], # East Asian ideograph
        0x223827 => [0x6633, false], # East Asian ideograph
        0x223828 => [0x662b, false], # East Asian ideograph
        0x22495c => [0x6daa, false], # East Asian ideograph
        0x21382a => [0x5777, false], # East Asian ideograph
        0x22382b => [0x6634, false], # East Asian ideograph
        0x22382c => [0x6624, false], # East Asian ideograph
        0x21382d => [0x5766, false], # East Asian ideograph
        0x21382e => [0x5782, false], # East Asian ideograph
        0x21495d => [0x70cf, false], # East Asian ideograph
        0x213830 => [0x57a0, false], # East Asian ideograph
        0x223831 => [0x6645, false], # East Asian ideograph
        0x213832 => [0x57a3, false], # East Asian ideograph
        0x233833 => [0x8da6, false], # East Asian ideograph
        0x213834 => [0x57a2, false], # East Asian ideograph
        0x213835 => [0x57d4, false], # East Asian ideograph
        0x213836 => [0x57c2, false], # East Asian ideograph
        0x213837 => [0x57ce, false], # East Asian ideograph
        0x213838 => [0x57cb, false], # East Asian ideograph
        0x213839 => [0x57c3, false], # East Asian ideograph
        0x21383a => [0x57f9, false], # East Asian ideograph
        0x27383b => [0x6267, false], # East Asian ideograph
        0x21383c => [0x57fa, false], # East Asian ideograph
        0x22383d => [0x6665, false], # East Asian ideograph
        0x22383e => [0x665c, false], # East Asian ideograph
        0x22383f => [0x6661, false], # East Asian ideograph
        0x213840 => [0x5802, false], # East Asian ideograph
        0x224960 => [0x6dec, false], # East Asian ideograph
        0x213842 => [0x57e4, false], # East Asian ideograph
        0x213843 => [0x57e0, false], # East Asian ideograph
        0x273844 => [0x62a5, false], # East Asian ideograph
        0x273845 => [0x5c27, false], # East Asian ideograph
        0x213846 => [0x5835, false], # East Asian ideograph
        0x213847 => [0x582a, false], # East Asian ideograph
        0x223848 => [0x665b, false], # East Asian ideograph
        0x223849 => [0x6659, false], # East Asian ideograph
        0x22384a => [0x6667, false], # East Asian ideograph
        0x21384b => [0x5821, false], # East Asian ideograph
        0x21384c => [0x585e, false], # East Asian ideograph
        0x22384d => [0x6657, false], # East Asian ideograph
        0x275541 => [0x83b1, false], # East Asian ideograph
        0x21384f => [0x5851, false], # East Asian ideograph
        0x213850 => [0x586b, false], # East Asian ideograph
        0x223851 => [0x666c, false], # East Asian ideograph
        0x233852 => [0x8dab, false], # East Asian ideograph
        0x234963 => [0x95d3, false], # East Asian ideograph
        0x213854 => [0x5854, false], # East Asian ideograph
        0x273855 => [0x575e, false], # East Asian ideograph
        0x213856 => [0x584a, false], # East Asian ideograph
        0x213857 => [0x5883, false], # East Asian ideograph
        0x213858 => [0x587e, false], # East Asian ideograph
        0x234964 => [0x95d1, false], # East Asian ideograph
        0x23385a => [0x8db0, false], # East Asian ideograph
        0x27385b => [0x5811, false], # East Asian ideograph
        0x216061 => [0x98bc, false], # East Asian ideograph
        0x21385d => [0x5893, false], # East Asian ideograph
        0x21385e => [0x589e, false], # East Asian ideograph
        0x234965 => [0x95c3, false], # East Asian ideograph
        0x273860 => [0x575f, false], # East Asian ideograph
        0x273861 => [0x5760, false], # East Asian ideograph
        0x273862 => [0x5815, false], # East Asian ideograph
        0x213863 => [0x589f, false], # East Asian ideograph
        0x273864 => [0x575b, false], # East Asian ideograph
        0x274966 => [0x65e0, false], # East Asian ideograph
        0x233866 => [0x8db2, false], # East Asian ideograph
        0x273867 => [0x57a6, false], # East Asian ideograph
        0x223868 => [0x6677, false], # East Asian ideograph
        0x273869 => [0x538b, false], # East Asian ideograph
        0x21386a => [0x58d1, false], # East Asian ideograph
        0x27386b => [0x5739, false], # East Asian ideograph
        0x21386c => [0x58d8, false], # East Asian ideograph
        0x27386d => [0x5784, false], # East Asian ideograph
        0x23386e => [0x8dbc, false], # East Asian ideograph
        0x27386f => [0x575c, false], # East Asian ideograph
        0x233870 => [0x8db9, false], # East Asian ideograph
        0x223871 => [0x668c, false], # East Asian ideograph
        0x233872 => [0x8dc1, false], # East Asian ideograph
        0x213873 => [0x58ec, false], # East Asian ideograph
        0x213874 => [0x58ef, false], # East Asian ideograph
        0x223875 => [0x668b, false], # East Asian ideograph
        0x273876 => [0x58f6, false], # East Asian ideograph
        0x273877 => [0x5bff, false], # East Asian ideograph
        0x213878 => [0x590f, false], # East Asian ideograph
        0x223879 => [0x6694, false], # East Asian ideograph
        0x22387a => [0x668a, false], # East Asian ideograph
        0x21387b => [0x5916, false], # East Asian ideograph
        0x22387c => [0x6698, false], # East Asian ideograph
        0x22387d => [0x668d, false], # East Asian ideograph
        0x21387e => [0x591c, false], # East Asian ideograph
        0x234547 => [0x936a, false], # East Asian ideograph
        0x22496b => [0x6db7, false], # East Asian ideograph
        0x2d3f54 => [0x61d0, false], # East Asian ideograph
        0x22496c => [0x6de2, false], # East Asian ideograph
        0x6f5768 => [0xc8e4, false], # Korean hangul
        0x4b393e => [0x5965, false], # East Asian ideograph
        0x27496d => [0x70e6, false], # East Asian ideograph
        0x22496e => [0x6de9, false], # East Asian ideograph
        0x6f5765 => [0xc8d5, false], # Korean hangul
        0x27496f => [0x7080, false], # East Asian ideograph
        0x21763e => [0x5810, false], # East Asian ideograph
        0x6f4b79 => [0xb11c, false], # Korean hangul
        0x4d5053 => [0x98da, false], # East Asian ideograph
        0x6f4970 => [0xad70, false], # Korean hangul
        0x224247 => [0x6a90, false], # East Asian ideograph
        0x224971 => [0x6df6, false], # East Asian ideograph
        0x28433a => [0x69e0, false], # East Asian ideograph
        0x295d42 => [0x9e7e, false], # East Asian ideograph
        0x234972 => [0x95e4, false], # East Asian ideograph
        0x6f7622 => [0x3186, false], # Korean hangul
        0x27487b => [0x6dc0, false], # East Asian ideograph
        0x6f5766 => [0xc8d7, false], # Korean hangul
        0x215f64 => [0x971c, false], # East Asian ideograph
        0x213d4b => [0x5f48, false], # East Asian ideograph
        0x274975 => [0x6247, false], # East Asian ideograph
        0x6f4976 => [0xad7d, false], # Korean hangul
        0x213421 => [0x528d, false], # East Asian ideograph
        0x225257 => [0x7160, false], # East Asian ideograph
        0x4b4977 => [0x7188, false], # East Asian ideograph
        0x233422 => [0x8b2b, false], # East Asian ideograph
        0x6f4978 => [0xad81, false], # Korean hangul
        0x4b4339 => [0x6674, false], # East Asian ideograph
        0x223423 => [0x644e, false], # East Asian ideograph
        0x223d6a => [0x6910, false], # East Asian ideograph
        0x224979 => [0x6e0f, false], # East Asian ideograph
        0x213424 => [0x529b, false], # East Asian ideograph
        0x22414b => [0x6a5c, false], # East Asian ideograph
        0x23497a => [0x961e, false], # East Asian ideograph
        0x283671 => [0x6593, false], # East Asian ideograph
        0x23497b => [0x9624, false], # East Asian ideograph
        0x23497c => [0x9622, false], # East Asian ideograph
        0x213427 => [0x52a3, false], # East Asian ideograph
        0x4b5963 => [0x734f, false], # East Asian ideograph
        0x27497d => [0x70ed, false], # East Asian ideograph
        0x213428 => [0x52ab, false], # East Asian ideograph
        0x27497e => [0x70eb, false], # East Asian ideograph
        0x213429 => [0x52a9, false], # East Asian ideograph
        0x275d47 => [0x9499, false], # East Asian ideograph
        0x23342a => [0x8b37, false], # East Asian ideograph
        0x273771 => [0x56ed, false], # East Asian ideograph
        0x6f5843 => [0xc9f1, false], # Korean hangul
        0x21342c => [0x52be, false], # East Asian ideograph
        0x2d4f6b => [0x7af8, false], # East Asian ideograph
        0x4c2962 => [0x5f4d, false], # East Asian ideograph (variant of 222962 which maps to 5F4D)
        0x21342d => [0x52c7, false], # East Asian ideograph
        0x334c37 => [0x8e6f, false], # East Asian ideograph
        0x215f67 => [0x9727, false], # East Asian ideograph
        0x21742e => [0x5707, false], # East Asian ideograph
        0x23454c => [0x934f, false], # East Asian ideograph
        0x2d6078 => [0x9920, false], # East Asian ideograph
        0x21342f => [0x52c1, false], # East Asian ideograph
        0x273772 => [0x5706, false], # East Asian ideograph
        0x233921 => [0x8dcf, false], # East Asian ideograph
        0x233922 => [0x8dd6, false], # East Asian ideograph
        0x273923 => [0x4f19, false], # East Asian ideograph
        0x223924 => [0x7a25, false], # East Asian ideograph
        0x213925 => [0x5927, false], # East Asian ideograph
        0x213926 => [0x592a, false], # East Asian ideograph
        0x233927 => [0x8dd0, false], # East Asian ideograph
        0x213928 => [0x5929, false], # East Asian ideograph
        0x213929 => [0x592d, false], # East Asian ideograph
        0x22392a => [0x66a0, false], # East Asian ideograph
        0x23392b => [0x8dc5, false], # East Asian ideograph
        0x21392c => [0x5937, false], # East Asian ideograph
        0x217432 => [0x5714, false], # East Asian ideograph
        0x27392e => [0x5939, false], # East Asian ideograph
        0x23392f => [0x8de4, false], # East Asian ideograph
        0x223930 => [0x5c21, false], # East Asian ideograph
        0x213931 => [0x5948, false], # East Asian ideograph
        0x223932 => [0x669d, false], # East Asian ideograph
        0x213433 => [0x52d9, false], # East Asian ideograph
        0x213934 => [0x5955, false], # East Asian ideograph
        0x233935 => [0x8deb, false], # East Asian ideograph
        0x233936 => [0x8df4, false], # East Asian ideograph
        0x213937 => [0x594f, false], # East Asian ideograph
        0x233938 => [0x8de9, false], # East Asian ideograph
        0x213434 => [0x52d5, false], # East Asian ideograph
        0x22393a => [0x66b2, false], # East Asian ideograph
        0x23393b => [0x8de3, false], # East Asian ideograph
        0x21393c => [0x5960, false], # East Asian ideograph
        0x23393d => [0x8de7, false], # East Asian ideograph
        0x21393e => [0x5967, false], # East Asian ideograph
        0x23393f => [0x8e09, false], # East Asian ideograph
        0x223940 => [0x66b5, false], # East Asian ideograph
        0x273941 => [0x594b, false], # East Asian ideograph
        0x213942 => [0x5973, false], # East Asian ideograph
        0x223943 => [0x66ac, false], # East Asian ideograph
        0x233944 => [0x8dff, false], # East Asian ideograph
        0x213945 => [0x5984, false], # East Asian ideograph
        0x233946 => [0x8e05, false], # East Asian ideograph
        0x223947 => [0x66b1, false], # East Asian ideograph
        0x213948 => [0x597d, false], # East Asian ideograph
        0x233949 => [0x8e01, false], # East Asian ideograph
        0x21394a => [0x5982, false], # East Asian ideograph
        0x21394b => [0x5981, false], # East Asian ideograph
        0x21394c => [0x59a8, false], # East Asian ideograph
        0x21394d => [0x5992, false], # East Asian ideograph
        0x23394e => [0x8e04, false], # East Asian ideograph
        0x22394f => [0x66be, false], # East Asian ideograph
        0x233950 => [0x8e06, false], # East Asian ideograph
        0x233438 => [0x8b3e, false], # East Asian ideograph
        0x233952 => [0x8e2a, false], # East Asian ideograph
        0x273953 => [0x5986, false], # East Asian ideograph
        0x223954 => [0x66c0, false], # East Asian ideograph
        0x223955 => [0x66c7, false], # East Asian ideograph
        0x213956 => [0x598a, false], # East Asian ideograph
        0x233957 => [0x8e2e, false], # East Asian ideograph
        0x233958 => [0x8e21, false], # East Asian ideograph
        0x213959 => [0x59bb, false], # East Asian ideograph
        0x22395a => [0x66bb, false], # East Asian ideograph
        0x21395b => [0x59d1, false], # East Asian ideograph
        0x22395c => [0x66c4, false], # East Asian ideograph
        0x21343a => [0x52df, false], # East Asian ideograph
        0x21395e => [0x59d0, false], # East Asian ideograph
        0x21395f => [0x59d7, false], # East Asian ideograph
        0x223960 => [0x66cf, false], # East Asian ideograph
        0x213961 => [0x59d2, false], # East Asian ideograph
        0x213962 => [0x59d3, false], # East Asian ideograph
        0x213963 => [0x59ca, false], # East Asian ideograph
        0x233964 => [0x8e16, false], # East Asian ideograph
        0x213965 => [0x59cb, false], # East Asian ideograph
        0x233966 => [0x8e26, false], # East Asian ideograph
        0x213967 => [0x59e3, false], # East Asian ideograph
        0x233968 => [0x8e14, false], # East Asian ideograph
        0x213969 => [0x59ff, false], # East Asian ideograph
        0x21396a => [0x59d8, false], # East Asian ideograph
        0x21396b => [0x5a03, false], # East Asian ideograph
        0x21396c => [0x59e8, false], # East Asian ideograph
        0x21396d => [0x59e5, false], # East Asian ideograph
        0x21396e => [0x59ea, false], # East Asian ideograph
        0x23396f => [0x8e41, false], # East Asian ideograph
        0x213970 => [0x59fb, false], # East Asian ideograph
        0x223971 => [0x66da, false], # East Asian ideograph
        0x223972 => [0x66db, false], # East Asian ideograph
        0x223973 => [0x66e2, false], # East Asian ideograph
        0x213974 => [0x5a18, false], # East Asian ideograph
        0x213975 => [0x5a23, false], # East Asian ideograph
        0x223976 => [0x66e1, false], # East Asian ideograph
        0x233977 => [0x8e40, false], # East Asian ideograph
        0x223978 => [0x66e8, false], # East Asian ideograph
        0x233979 => [0x8e36, false], # East Asian ideograph
        0x21397a => [0x5a1f, false], # East Asian ideograph
        0x21397b => [0x5a1b, false], # East Asian ideograph
        0x22397c => [0x66e9, false], # East Asian ideograph
        0x21397d => [0x5a29, false], # East Asian ideograph
        0x23397e => [0x8e3d, false], # East Asian ideograph
        0x27785a => [0x5785, false], # East Asian ideograph
        0x217441 => [0x5724, false], # East Asian ideograph
        0x6f532a => [0xc12a, false], # Korean hangul
        0x213442 => [0x5306, false], # East Asian ideograph
        0x334550 => [0x7f47, false], # East Asian ideograph
        0x232337 => [0x846e, false], # East Asian ideograph
        0x217443 => [0x5729, false], # East Asian ideograph
        0x4b5521 => [0x8332, false], # East Asian ideograph
        0x233444 => [0x8b54, false], # East Asian ideograph
        0x235c71 => [0x9e07, false], # East Asian ideograph
        0x22525e => [0x7176, false], # East Asian ideograph
        0x213445 => [0x5310, false], # East Asian ideograph
        0x2d4b45 => [0x6bec, false], # East Asian ideograph
        0x6f5435 => [0xc309, false], # Korean hangul
        0x22527b => [0x7187, false], # East Asian ideograph
        0x6f576e => [0xc900, false], # Korean hangul
        0x6f532b => [0xc12c, false], # Korean hangul
        0x6f2527 => [0x3162, false], # Korean hangul
        0x227447 => [0x7f63, false], # East Asian ideograph
        0x4b3666 => [0x5a1a, false], # East Asian ideograph
        0x233448 => [0x8b53, false], # East Asian ideograph
        0x233449 => [0x8b4a, false], # East Asian ideograph
        0x275124 => [0x7eb8, false], # East Asian ideograph
        0x223046 => [0x6285, false], # East Asian ideograph
        0x21344a => [0x5319, false], # East Asian ideograph
        0x6f576f => [0xc904, false], # Korean hangul
        0x6f532c => [0xc12d, false], # Korean hangul
        0x23356f => [0x8c74, false], # East Asian ideograph
        0x6f4b7b => [0xb11e, false], # Korean hangul
        0x215b2a => [0x8e91, false], # East Asian ideograph
        0x6f4f6f => [0xb9db, false], # Korean hangul
        0x21344d => [0x5321, false], # East Asian ideograph
        0x22344e => [0x64a2, false], # East Asian ideograph
        0x23344f => [0x8b3f, false], # East Asian ideograph
        0x2e7450 => [0x7f82, false], # East Asian ideograph
        0x213451 => [0x532f, false], # East Asian ideograph
        0x335230 => [0x7f6e, false], # East Asian ideograph (variant of 215230 which maps to 7F6E)
        0x4b3668 => [0x5358, false], # East Asian ideograph
        0x234553 => [0x9356, false], # East Asian ideograph
        0x21725d => [0x5620, false], # East Asian ideograph
        0x27554f => [0x53f6, false], # East Asian ideograph
        0x213453 => [0x5339, false], # East Asian ideograph
        0x223454 => [0x6490, false], # East Asian ideograph
        0x213455 => [0x5340, false], # East Asian ideograph
        0x215f6f => [0x9748, false], # East Asian ideograph
        0x215b2c => [0x8eaa, false], # East Asian ideograph
        0x234554 => [0x9371, false], # East Asian ideograph
        0x6f5028 => [0xba4d, false], # Korean hangul
        0x283457 => [0x63b8, false], # East Asian ideograph
        0x274b2d => [0x736d, false], # East Asian ideograph
        0x2d3458 => [0x4edf, false], # East Asian ideograph
        0x6f4c65 => [0xb2d0, false], # Korean hangul
        0x284934 => [0x6d43, false], # East Asian ideograph
        0x233459 => [0x8b59, false], # East Asian ideograph
        0x6f5772 => [0xc90d, false], # Korean hangul
        0x233a21 => [0x8e30, false], # East Asian ideograph
        0x213a22 => [0x5a49, false], # East Asian ideograph
        0x21345b => [0x5347, false], # East Asian ideograph
        0x233a24 => [0x8e47, false], # East Asian ideograph
        0x213a25 => [0x5a4a, false], # East Asian ideograph
        0x233a26 => [0x8e46, false], # East Asian ideograph
        0x273a27 => [0x5987, false], # East Asian ideograph
        0x273a28 => [0x5a04, false], # East Asian ideograph
        0x213a29 => [0x5a3c, false], # East Asian ideograph
        0x213a2a => [0x5a62, false], # East Asian ideograph
        0x213a2b => [0x5a5a, false], # East Asian ideograph
        0x213a2c => [0x5a77, false], # East Asian ideograph
        0x213a2d => [0x5a9a, false], # East Asian ideograph
        0x233a2e => [0x8e4c, false], # East Asian ideograph
        0x213a2f => [0x5a7f, false], # East Asian ideograph
        0x223a30 => [0x670f, false], # East Asian ideograph
        0x224767 => [0x6cac, false], # East Asian ideograph
        0x233a32 => [0x8e4f, false], # East Asian ideograph
        0x223a33 => [0x6712, false], # East Asian ideograph
        0x223a34 => [0x6713, false], # East Asian ideograph
        0x233a35 => [0x8e62, false], # East Asian ideograph
        0x233a36 => [0x8e60, false], # East Asian ideograph
        0x213a37 => [0x5ab2, false], # East Asian ideograph
        0x223a38 => [0x6719, false], # East Asian ideograph
        0x223a39 => [0x6718, false], # East Asian ideograph
        0x233a3a => [0x8e54, false], # East Asian ideograph
        0x273a3b => [0x59aa, false], # East Asian ideograph
        0x213a3c => [0x5ad6, false], # East Asian ideograph
        0x213a3d => [0x5ae3, false], # East Asian ideograph
        0x233a3e => [0x8e5a, false], # East Asian ideograph
        0x233a3f => [0x8e5e, false], # East Asian ideograph
        0x233a40 => [0x8e55, false], # East Asian ideograph
        0x273a41 => [0x5a34, false], # East Asian ideograph
        0x213a42 => [0x5b09, false], # East Asian ideograph
        0x273a43 => [0x5a75, false], # East Asian ideograph
        0x273a44 => [0x5a07, false], # East Asian ideograph
        0x273a45 => [0x59a9, false], # East Asian ideograph
        0x233a46 => [0x8e95, false], # East Asian ideograph
        0x223a47 => [0x6723, false], # East Asian ideograph
        0x233a48 => [0x8e6d, false], # East Asian ideograph
        0x213a49 => [0x5b24, false], # East Asian ideograph
        0x273a4a => [0x5a74, false], # East Asian ideograph
        0x273a4b => [0x5a76, false], # East Asian ideograph
        0x223a4c => [0x673e, false], # East Asian ideograph
        0x213462 => [0x5351, false], # East Asian ideograph
        0x223a4e => [0x673f, false], # East Asian ideograph
        0x213a4f => [0x5b53, false], # East Asian ideograph
        0x213a50 => [0x5b54, false], # East Asian ideograph
        0x213a51 => [0x5b55, false], # East Asian ideograph
        0x213a52 => [0x5b57, false], # East Asian ideograph
        0x213a53 => [0x5b58, false], # East Asian ideograph
        0x213a54 => [0x5b5d, false], # East Asian ideograph
        0x213a55 => [0x5b5c, false], # East Asian ideograph
        0x233a57 => [0x8e8b, false], # East Asian ideograph
        0x223a58 => [0x6757, false], # East Asian ideograph
        0x213a59 => [0x5b64, false], # East Asian ideograph
        0x213a5a => [0x5b69, false], # East Asian ideograph
        0x273a5b => [0x5b59, false], # East Asian ideograph
        0x223a5c => [0x6747, false], # East Asian ideograph
        0x213a5d => [0x5b73, false], # East Asian ideograph
        0x233a5e => [0x8e9a, false], # East Asian ideograph
        0x273a5f => [0x5b5a, false], # East Asian ideograph
        0x273a60 => [0x5b66, false], # East Asian ideograph
        0x223a61 => [0x6755, false], # East Asian ideograph
        0x213a62 => [0x5b7d, false], # East Asian ideograph
        0x233a63 => [0x8e98, false], # East Asian ideograph
        0x233a64 => [0x8e9e, false], # East Asian ideograph
        0x223466 => [0x64b3, false], # East Asian ideograph
        0x223a66 => [0x674c, false], # East Asian ideograph
        0x223a67 => [0x6759, false], # East Asian ideograph
        0x223a68 => [0x6748, false], # East Asian ideograph
        0x213a69 => [0x5b8c, false], # East Asian ideograph
        0x275553 => [0x8364, false], # East Asian ideograph
        0x233a6b => [0x8ea5, false], # East Asian ideograph
        0x213a6c => [0x5b97, false], # East Asian ideograph
        0x213a6d => [0x5b9a, false], # East Asian ideograph
        0x213a6e => [0x5b9c, false], # East Asian ideograph
        0x233a6f => [0x8ea7, false], # East Asian ideograph
        0x213a70 => [0x5b99, false], # East Asian ideograph
        0x223a71 => [0x674a, false], # East Asian ideograph
        0x233a72 => [0x8e99, false], # East Asian ideograph
        0x213a73 => [0x5ba3, false], # East Asian ideograph
        0x213a74 => [0x5ba6, false], # East Asian ideograph
        0x213a75 => [0x5ba4, false], # East Asian ideograph
        0x213a76 => [0x5ba2, false], # East Asian ideograph
        0x213a77 => [0x5bb0, false], # East Asian ideograph
        0x213a78 => [0x5bb8, false], # East Asian ideograph
        0x233a7a => [0x8ebc, false], # East Asian ideograph
        0x213a7b => [0x5bb4, false], # East Asian ideograph
        0x223a7c => [0x6785, false], # East Asian ideograph
        0x213a7d => [0x5bb9, false], # East Asian ideograph
        0x213a7e => [0x5bb3, false], # East Asian ideograph
        0x23233f => [0x844a, false], # East Asian ideograph
        0x4b763d => [0x57f4, false], # East Asian ideograph (variant of 21763D which maps to 57F4)
        0x22746b => [0x7f7e, false], # East Asian ideograph
        0x283b7d => [0x53f0, false], # East Asian ideograph (duplicate simplified)
        0x22346c => [0x64d3, false], # East Asian ideograph
        0x6f5927 => [0xcc39, false], # Korean hangul
        0x393b39 => [0x5bf3, false], # East Asian ideograph
        0x213f26 => [0x614c, false], # East Asian ideograph
        0x235222 => [0x9957, false], # East Asian ideograph (variant of 475222 which maps to 9957)
        0x2d346e => [0x5373, false], # East Asian ideograph
        0x276232 => [0x9e23, false], # East Asian ideograph
        0x6f5333 => [0xc13c, false], # Korean hangul
        0x213d5b => [0x5f79, false], # East Asian ideograph
        0x213471 => [0x5378, false], # East Asian ideograph
        0x287472 => [0x7f74, false], # East Asian ideograph
        0x23344d => [0x8b56, false], # East Asian ideograph
        0x335223 => [0x7e8e, false], # East Asian ideograph
        0x233473 => [0x8b45, false], # East Asian ideograph
        0x273f3f => [0x51ed, false], # East Asian ideograph
        0x213474 => [0x537f, false], # East Asian ideograph
        0x213475 => [0x5384, false], # East Asian ideograph
        0x21325d => [0x50f9, false], # East Asian ideograph
        0x225f21 => [0x75f9, false], # East Asian ideograph
        0x217477 => [0x576d, false], # East Asian ideograph
        0x225f22 => [0x75fc, false], # East Asian ideograph
        0x23456f => [0x9364, false], # East Asian ideograph
        0x275f23 => [0x9648, false], # East Asian ideograph
        0x213479 => [0x53a5, false], # East Asian ideograph
        0x275f24 => [0x9646, false], # East Asian ideograph
        0x22747a => [0x7f91, false], # East Asian ideograph
        0x21347b => [0x53b2, false], # East Asian ideograph
        0x21392f => [0x5954, false], # East Asian ideograph
        0x225269 => [0x7150, false], # East Asian ideograph
        0x4b4835 => [0x6da3, false], # East Asian ideograph
        0x21347d => [0x53c3, false], # East Asian ideograph
        0x2d5f28 => [0x9665, false], # East Asian ideograph
        0x6f5336 => [0xc149, false], # Korean hangul
        0x6f5329 => [0xc127, false], # Korean hangul
        0x225f29 => [0x7616, false], # East Asian ideograph
        0x275f2a => [0x9634, false], # East Asian ideograph
        0x275f2b => [0x961f, false], # East Asian ideograph
        0x216c41 => [0x52d1, false], # East Asian ideograph
        0x225f2c => [0x7608, false], # East Asian ideograph
        0x6f577a => [0xc958, false], # Korean hangul
        0x225f2d => [0x7615, false], # East Asian ideograph
        0x295731 => [0x9c8b, false], # East Asian ideograph
        0x276222 => [0x9cc5, false], # East Asian ideograph
        0x225f2e => [0x760c, false], # East Asian ideograph
        0x23455d => [0x9349, false], # East Asian ideograph
        0x6f5567 => [0xc5fe, false], # Korean hangul
        0x215f2f => [0x9685, false], # East Asian ideograph
        0x273340 => [0x51bb, false], # East Asian ideograph
        0x223b21 => [0x677b, false], # East Asian ideograph
        0x223b22 => [0x6792, false], # East Asian ideograph
        0x223b23 => [0x6776, false], # East Asian ideograph
        0x213b24 => [0x5bc4, false], # East Asian ideograph
        0x223b25 => [0x6791, false], # East Asian ideograph
        0x223b26 => [0x6799, false], # East Asian ideograph
        0x215f31 => [0x968d, false], # East Asian ideograph
        0x223b28 => [0x67a4, false], # East Asian ideograph
        0x213b29 => [0x5bd0, false], # East Asian ideograph
        0x213b2a => [0x5bd3, false], # East Asian ideograph
        0x213b2b => [0x5be1, false], # East Asian ideograph
        0x213b2c => [0x5be5, false], # East Asian ideograph
        0x215f32 => [0x9698, false], # East Asian ideograph
        0x223b2e => [0x678f, false], # East Asian ideograph
        0x233b2f => [0x8ecf, false], # East Asian ideograph
        0x223b30 => [0x6772, false], # East Asian ideograph
        0x223b31 => [0x6798, false], # East Asian ideograph (variant of 4C3B31 which maps to 6798)
        0x223b32 => [0x676a, false], # East Asian ideograph
        0x233b33 => [0x8ed5, false], # East Asian ideograph
        0x213b34 => [0x5bee, false], # East Asian ideograph
        0x273b35 => [0x5bbd, false], # East Asian ideograph
        0x273b36 => [0x5ba1, false], # East Asian ideograph
        0x273b37 => [0x5199, false], # East Asian ideograph
        0x273b38 => [0x5ba0, false], # East Asian ideograph
        0x223b39 => [0x67ac, false], # East Asian ideograph
        0x213b3a => [0x5bf8, false], # East Asian ideograph
        0x223b3b => [0x67a0, false], # East Asian ideograph
        0x213b3c => [0x5c01, false], # East Asian ideograph
        0x213b3d => [0x5c04, false], # East Asian ideograph
        0x213b3e => [0x5c09, false], # East Asian ideograph
        0x233b3f => [0x8efa, false], # East Asian ideograph
        0x273b40 => [0x5c06, false], # East Asian ideograph
        0x213b41 => [0x5c0a, false], # East Asian ideograph
        0x233b42 => [0x8ef9, false], # East Asian ideograph
        0x273b43 => [0x5bf9, false], # East Asian ideograph
        0x223b44 => [0x67f9, false], # East Asian ideograph
        0x213b45 => [0x5c0f, false], # East Asian ideograph
        0x213b46 => [0x5c11, false], # East Asian ideograph
        0x213b47 => [0x5c16, false], # East Asian ideograph
        0x223b48 => [0x678d, false], # East Asian ideograph
        0x223b49 => [0x678c, false], # East Asian ideograph
        0x213b4a => [0x5c2c, false], # East Asian ideograph
        0x233b4b => [0x8ee8, false], # East Asian ideograph
        0x223b4c => [0x67fc, false], # East Asian ideograph
        0x213b4d => [0x5c38, false], # East Asian ideograph
        0x223b4e => [0x6810, false], # East Asian ideograph
        0x233b4f => [0x8eeb, false], # East Asian ideograph
        0x213b50 => [0x5c40, false], # East Asian ideograph
        0x223b51 => [0x67c8, false], # East Asian ideograph
        0x23455f => [0x935a, false], # East Asian ideograph
        0x213b53 => [0x5c3e, false], # East Asian ideograph
        0x223b54 => [0x67cc, false], # East Asian ideograph
        0x213b55 => [0x5c45, false], # East Asian ideograph
        0x233b56 => [0x8f00, false], # East Asian ideograph
        0x213b57 => [0x5c4e, false], # East Asian ideograph
        0x223b58 => [0x67c5, false], # East Asian ideograph
        0x233b59 => [0x8f05, false], # East Asian ideograph
        0x233b5a => [0x8f08, false], # East Asian ideograph
        0x233b5b => [0x8f07, false], # East Asian ideograph
        0x223b5c => [0x67bb, false], # East Asian ideograph
        0x213b5d => [0x5c5b, false], # East Asian ideograph (not in Unicode)
        0x213b5e => [0x5c60, false], # East Asian ideograph
        0x223b5f => [0x67b0, false], # East Asian ideograph
        0x223b60 => [0x6803, false], # East Asian ideograph
        0x223b61 => [0x67f8, false], # East Asian ideograph
        0x213b62 => [0x5c65, false], # East Asian ideograph
        0x273b63 => [0x5c5e, false], # East Asian ideograph
        0x233b64 => [0x8f2c, false], # East Asian ideograph
        0x213b65 => [0x5c71, false], # East Asian ideograph
        0x225a2a => [0x741b, false], # East Asian ideograph
        0x213b67 => [0x5c90, false], # East Asian ideograph
        0x213b68 => [0x5c8c, false], # East Asian ideograph
        0x213b69 => [0x5c91, false], # East Asian ideograph
        0x213b6a => [0x5c94, false], # East Asian ideograph
        0x233b6b => [0x8f1e, false], # East Asian ideograph
        0x213b6c => [0x5cb8, false], # East Asian ideograph
        0x233b6d => [0x8f25, false], # East Asian ideograph
        0x233b6e => [0x8f20, false], # East Asian ideograph
        0x223b6f => [0x67e4, false], # East Asian ideograph
        0x223b70 => [0x67d9, false], # East Asian ideograph
        0x223b71 => [0x67db, false], # East Asian ideograph
        0x223b72 => [0x67b5, false], # East Asian ideograph
        0x213b73 => [0x5d01, false], # East Asian ideograph
        0x273b74 => [0x5ce1, false], # East Asian ideograph
        0x223b75 => [0x67f7, false], # East Asian ideograph
        0x213b76 => [0x5cfb, false], # East Asian ideograph
        0x223b77 => [0x67b3, false], # East Asian ideograph
        0x233b78 => [0x8f36, false], # East Asian ideograph
        0x233b79 => [0x8f2e, false], # East Asian ideograph
        0x233b7a => [0x8f33, false], # East Asian ideograph
        0x215f3f => [0x96c0, false], # East Asian ideograph
        0x223b7c => [0x67ee, false], # East Asian ideograph
        0x223b7d => [0x6aaf, false], # East Asian ideograph
        0x223b7e => [0x67b2, false], # East Asian ideograph
        0x225f40 => [0x761b, false], # East Asian ideograph
        0x6f577e => [0xc970, false], # Korean hangul
        0x6f533b => [0xc158, false], # Korean hangul
        0x213d63 => [0x5f8a, false], # East Asian ideograph
        0x6f4b7e => [0xb125, false], # Korean hangul
        0x2d5d2f => [0x9196, false], # East Asian ideograph
        0x2d5f43 => [0x9ceb, false], # East Asian ideograph
        0x6f2459 => [0x3137, false], # Korean hangul
        0x293b42 => [0x8f75, false], # East Asian ideograph
        0x235f45 => [0x9f22, false], # East Asian ideograph
        0x2d5f46 => [0x96bd, false], # East Asian ideograph
        0x6f533c => [0xc167, false], # Korean hangul
        0x213d64 => [0x5f87, false], # East Asian ideograph
        0x225f47 => [0x7619, false], # East Asian ideograph
        0x234562 => [0x935f, false], # East Asian ideograph
        0x235f48 => [0x9f2b, false], # East Asian ideograph
        0x2e604a => [0x7690, false], # East Asian ideograph
        0x235f49 => [0x9f26, false], # East Asian ideograph
        0x225270 => [0x7144, false], # East Asian ideograph
        0x6f5d65 => [0xd72d, false], # Korean hangul
        0x224e2d => [0x6fc9, false], # East Asian ideograph
        0x2d4b3f => [0x73ce, false], # East Asian ideograph
        0x275f4b => [0x6742, false], # East Asian ideograph
        0x276234 => [0x9e29, false], # East Asian ideograph
        0x6f533d => [0xc168, false], # Korean hangul
        0x225f4c => [0x761d, false], # East Asian ideograph
        0x275f4d => [0x96cf, false], # East Asian ideograph
        0x6f5773 => [0xc90f, false], # Korean hangul
        0x6f245b => [0x3141, false], # Korean hangul
        0x275f4e => [0x53cc, false], # East Asian ideograph
        0x216c48 => [0x52d6, false], # East Asian ideograph
        0x275f4f => [0x79bb, false], # East Asian ideograph
        0x215f50 => [0x96e3, false], # East Asian ideograph (variant of 4B5F50 which maps to 96E3)
        0x6f533e => [0xc170, false], # Korean hangul
        0x213d66 => [0x5f92, false], # East Asian ideograph
        0x215f51 => [0x96e8, false], # East Asian ideograph
        0x225f3b => [0x7610, false], # East Asian ideograph
        0x284345 => [0x680c, false], # East Asian ideograph
        0x6f5848 => [0xca08, false], # Korean hangul
        0x6f245c => [0x3142, false], # Korean hangul
        0x235f53 => [0x9f2f, false], # East Asian ideograph
        0x225f54 => [0x762d, false], # East Asian ideograph
        0x234571 => [0x936b, false], # East Asian ideograph
        0x6f505b => [0xbbc0, false], # Korean hangul
        0x275f55 => [0x7535, false], # East Asian ideograph
        0x6f533f => [0xc18c, false], # Korean hangul
        0x213d67 => [0x5f91, false], # East Asian ideograph
        0x6f4d64 => [0xb4e0, false], # Korean hangul
        0x213924 => [0x5922, false], # East Asian ideograph
        0x6f245d => [0x3145, false], # Korean hangul
        0x275128 => [0x7ecb, false], # East Asian ideograph
        0x4b5f58 => [0xf9b2, false], # East Asian ideograph
        0x227049 => [0x7d0f, false], # East Asian ideograph
        0x224e30 => [0x6fa0, false], # East Asian ideograph
        0x293338 => [0x8bfd, false], # East Asian ideograph
        0x2e715a => [0x7e27, false], # East Asian ideograph
        0x215f5a => [0x9707, false], # East Asian ideograph
        0x6f5340 => [0xc18d, false], # Korean hangul
        0x223c21 => [0x67b9, false], # East Asian ideograph
        0x213c22 => [0x5d11, false], # East Asian ideograph
        0x215b3e => [0x8efe, false], # East Asian ideograph
        0x223c24 => [0x67e3, false], # East Asian ideograph
        0x213c25 => [0x5d14, false], # East Asian ideograph
        0x233c26 => [0x8f39, false], # East Asian ideograph
        0x233c27 => [0x8f34, false], # East Asian ideograph
        0x273c28 => [0x5c9a, false], # East Asian ideograph
        0x223c29 => [0x67e2, false], # East Asian ideograph
        0x273c2a => [0x5d2d, false], # East Asian ideograph
        0x273c2b => [0x5c96, false], # East Asian ideograph
        0x213c2c => [0x5d9d, false], # East Asian ideograph
        0x273c2d => [0x5c7f, false], # East Asian ideograph
        0x273c2e => [0x5cb3, false], # East Asian ideograph
        0x223c2f => [0x67e7, false], # East Asian ideograph
        0x223c30 => [0x6849, false], # East Asian ideograph
        0x223c31 => [0x683e, false], # East Asian ideograph
        0x273c32 => [0x5dc5, false], # East Asian ideograph
        0x214a32 => [0x71ec, false], # East Asian ideograph
        0x213c34 => [0x5ddd, false], # East Asian ideograph
        0x215f5e => [0x9711, false], # East Asian ideograph
        0x223c36 => [0x6814, false], # East Asian ideograph
        0x223c37 => [0x684b, false], # East Asian ideograph
        0x223c38 => [0x681e, false], # East Asian ideograph
        0x213c39 => [0x5de7, false], # East Asian ideograph
        0x213c3a => [0x5de6, false], # East Asian ideograph
        0x223c3b => [0x6833, false], # East Asian ideograph
        0x213c3c => [0x5dee, false], # East Asian ideograph
        0x233c3d => [0x8f52, false], # East Asian ideograph
        0x213c3e => [0x5df2, false], # East Asian ideograph
        0x213c3f => [0x5df3, false], # East Asian ideograph
        0x223c40 => [0x6831, false], # East Asian ideograph
        0x215f60 => [0x9716, false], # East Asian ideograph
        0x223c42 => [0x6835, false], # East Asian ideograph
        0x223c43 => [0x683b, false], # East Asian ideograph
        0x223c44 => [0x684e, false], # East Asian ideograph
        0x213c46 => [0x5e06, false], # East Asian ideograph
        0x234124 => [0x918d, false], # East Asian ideograph
        0x233c48 => [0x8f56, false], # East Asian ideograph
        0x213c49 => [0x5e1a, false], # East Asian ideograph
        0x223c4a => [0x684d, false], # East Asian ideograph
        0x233c4b => [0x8f55, false], # East Asian ideograph
        0x233c4c => [0x8f58, false], # East Asian ideograph
        0x215f62 => [0x970d, false], # East Asian ideograph
        0x233c4e => [0x8f5e, false], # East Asian ideograph
        0x273c4f => [0x5e05, false], # East Asian ideograph
        0x273c51 => [0x5e08, false], # East Asian ideograph
        0x273c52 => [0x5e10, false], # East Asian ideograph
        0x273c53 => [0x5e26, false], # East Asian ideograph
        0x213c54 => [0x5e38, false], # East Asian ideograph
        0x223c55 => [0x685d, false], # East Asian ideograph
        0x223c56 => [0x685e, false], # East Asian ideograph
        0x233c57 => [0x8f62, false], # East Asian ideograph
        0x273c58 => [0x5e27, false], # East Asian ideograph
        0x233c59 => [0x8f63, false], # East Asian ideograph
        0x233c5a => [0x8f64, false], # East Asian ideograph
        0x213c5b => [0x5e54, false], # East Asian ideograph
        0x273c5c => [0x5e3c, false], # East Asian ideograph
        0x213c5d => [0x5e55, false], # East Asian ideograph
        0x273c5e => [0x5e01, false], # East Asian ideograph
        0x213c5f => [0x5e62, false], # East Asian ideograph
        0x273c60 => [0x5e1c, false], # East Asian ideograph
        0x273c61 => [0x5e2e, false], # East Asian ideograph
        0x213c63 => [0x5e73, false], # East Asian ideograph
        0x6f5177 => [0xbe5a, false], # Korean hangul
        0x223c65 => [0x685a, false], # East Asian ideograph
        0x233c66 => [0x8fa5, false], # East Asian ideograph
        0x273c67 => [0x5e72, false], # East Asian ideograph (Version J extension)
        0x223c68 => [0x686b, false], # East Asian ideograph
        0x223c69 => [0x686c, false], # East Asian ideograph
        0x213c6a => [0x5e7d, false], # East Asian ideograph
        0x223c6b => [0x6879, false], # East Asian ideograph
        0x233c6c => [0x8fb5, false], # East Asian ideograph
        0x213c6d => [0x5e87, false], # East Asian ideograph
        0x233c6e => [0x8fbb, false], # East Asian ideograph
        0x213c6f => [0x5e9a, false], # East Asian ideograph
        0x233c70 => [0x8fbc, false], # East Asian ideograph
        0x215f68 => [0x9738, false], # East Asian ideograph
        0x223c72 => [0x687e, false], # East Asian ideograph
        0x213c73 => [0x5e95, false], # East Asian ideograph
        0x233c74 => [0x8fbf, false], # East Asian ideograph
        0x233c75 => [0x8fd2, false], # East Asian ideograph
        0x273c76 => [0x5e93, false], # East Asian ideograph
        0x225f69 => [0x7647, false], # East Asian ideograph
        0x213c78 => [0x5ead, false], # East Asian ideograph
        0x213c79 => [0x5eb7, false], # East Asian ideograph
        0x233c7a => [0x8fca, false], # East Asian ideograph
        0x233c7b => [0x8fd3, false], # East Asian ideograph
        0x213c7c => [0x5eb5, false], # East Asian ideograph
        0x215f6a => [0x9732, false], # East Asian ideograph
        0x273c7e => [0x5395, false], # East Asian ideograph
        0x275f6b => [0x9701, false], # East Asian ideograph
        0x6f5849 => [0xca09, false], # Korean hangul
        0x6f2461 => [0x314b, false], # Korean hangul
        0x275725 => [0x8682, false], # East Asian ideograph
        0x275122 => [0x7eb3, false], # East Asian ideograph
        0x6f5273 => [0xc0d8, false], # Korean hangul
        0x235f6d => [0x9f45, false], # East Asian ideograph
        0x4c476e => [0x6cad, false], # East Asian ideograph
        0x225a2c => [0x7432, false], # East Asian ideograph
        0x225f6e => [0x764d, false], # East Asian ideograph
        0x6f2528 => [0x3163, false], # Korean hangul
        0x2f312b => [0x89bb, false], # East Asian ideograph
        0x235f6f => [0x9f46, false], # East Asian ideograph
        0x4b5f70 => [0x9752, false], # East Asian ideograph
        0x6f2462 => [0x314c, false], # Korean hangul
        0x235f71 => [0x9f48, false], # East Asian ideograph
        0x275123 => [0x7ea7, false], # East Asian ideograph
        0x214a36 => [0x720d, false], # East Asian ideograph
        0x224e35 => [0x6fb4, false], # East Asian ideograph
        0x335234 => [0x99e1, false], # East Asian ideograph
        0x2e3d62 => [0x684a, false], # East Asian ideograph
        0x235f73 => [0x9f49, false], # East Asian ideograph
        0x69253b => [0x30bb, false], # Katakana letter SE
        0x276230 => [0x9e20, false], # East Asian ideograph
        0x23456b => [0x9374, false], # East Asian ideograph
        0x215f75 => [0x9760, false], # East Asian ideograph
        0x692431 => [0x3051, false], # Hiragana letter KE
        0x274a21 => [0x70bd, false], # East Asian ideograph
        0x23345f => [0x8b4d, false], # East Asian ideograph
        0x224d63 => [0x6f5f, false], # East Asian ideograph
        0x274a22 => [0x7096, false], # East Asian ideograph
        0x4d446b => [0x954e, false], # East Asian ideograph
        0x6f4a23 => [0xadc4, false], # Korean hangul
        0x6f5346 => [0xc19f, false], # Korean hangul
        0x276231 => [0x9e22, false], # East Asian ideograph
        0x215f79 => [0x9768, false], # East Asian ideograph
        0x274a24 => [0x706f, false], # East Asian ideograph
        0x345e3b => [0x80ac, false], # East Asian ideograph
        0x215f7a => [0x9769, false], # East Asian ideograph
        0x274a25 => [0x7116, false], # East Asian ideograph
        0x275568 => [0x8298, false], # East Asian ideograph
        0x215f7b => [0x9776, false], # East Asian ideograph
        0x274a26 => [0x70e7, false], # East Asian ideograph
        0x6f5d67 => [0xd73c, false], # Korean hangul
        0x215f7c => [0x9774, false], # East Asian ideograph
        0x6f4a27 => [0xadd3, false], # Korean hangul
        0x2e3172 => [0x5261, false], # East Asian ideograph
        0x276236 => [0x9e35, false], # East Asian ideograph
        0x2d4a28 => [0x8b8c, false], # East Asian ideograph
        0x6f5347 => [0xc1a1, false], # Korean hangul
        0x213d6f => [0x5fa9, false], # East Asian ideograph
        0x215f7e => [0x9785, false], # East Asian ideograph
        0x33456d => [0x826a, false], # East Asian ideograph
        0x6f5178 => [0xbe5b, false], # Korean hangul
        0x224a2a => [0x6ddf, false], # East Asian ideograph
        0x6f2465 => [0x3132, false], # Korean hangul
        0x275126 => [0x7eca, false], # East Asian ideograph
        0x23567a => [0x9b95, false], # East Asian ideograph
        0x6f4a2c => [0xadf8, false], # Korean hangul
        0x224a2d => [0x6dd3, false], # East Asian ideograph
        0x6f5348 => [0xc1a5, false], # Korean hangul
        0x276233 => [0x51e4, false], # East Asian ideograph
        0x274a2e => [0x8425, false], # East Asian ideograph
        0x2e3328 => [0x6528, false], # East Asian ideograph
        0x6f5d6b => [0xd751, false], # Korean hangul
        0x234a2f => [0x9642, false], # East Asian ideograph
        0x4b462a => [0x6b74, false], # East Asian ideograph
        0x233d21 => [0x8fda, false], # East Asian ideograph
        0x233d22 => [0x8fd5, false], # East Asian ideograph
        0x213d23 => [0x5ec9, false], # East Asian ideograph
        0x213d24 => [0x5ec8, false], # East Asian ideograph
        0x223d25 => [0x686d, false], # East Asian ideograph
        0x213d26 => [0x5ed6, false], # East Asian ideograph
        0x273d27 => [0x5e9f, false], # East Asian ideograph
        0x213d28 => [0x5eda, false], # East Asian ideograph
        0x213d29 => [0x5edd, false], # East Asian ideograph
        0x273d2a => [0x5e7f, false], # East Asian ideograph
        0x273d2b => [0x5e99, false], # East Asian ideograph
        0x273d2c => [0x5382, false], # East Asian ideograph
        0x273d2d => [0x5e9e, false], # East Asian ideograph
        0x273d2e => [0x5e90, false], # East Asian ideograph
        0x233d2f => [0x8fe4, false], # East Asian ideograph
        0x233d30 => [0x8fee, false], # East Asian ideograph
        0x223d32 => [0x688b, false], # East Asian ideograph
        0x274a33 => [0x70e9, false], # East Asian ideograph
        0x213d34 => [0x5eff, false], # East Asian ideograph
        0x233d35 => [0x8ff9, false], # East Asian ideograph
        0x213d36 => [0x5f04, false], # East Asian ideograph
        0x213d37 => [0x5f08, false], # East Asian ideograph
        0x213d38 => [0x5f0a, false], # East Asian ideograph
        0x223d39 => [0x68a3, false], # East Asian ideograph
        0x213d3a => [0x5f12, false], # East Asian ideograph
        0x213d3b => [0x5f13, false], # East Asian ideograph
        0x233d3c => [0x8ffb, false], # East Asian ideograph
        0x213d3d => [0x5f14, false], # East Asian ideograph
        0x213d3e => [0x5f18, false], # East Asian ideograph
        0x214a35 => [0x7206, false], # East Asian ideograph
        0x223d40 => [0x688f, false], # East Asian ideograph
        0x213d41 => [0x5f1f, false], # East Asian ideograph
        0x213d42 => [0x5f26, false], # East Asian ideograph
        0x223d43 => [0x687b, false], # East Asian ideograph
        0x233d44 => [0x9011, false], # East Asian ideograph
        0x224a36 => [0x6ddc, false], # East Asian ideograph
        0x213d46 => [0x5f31, false], # East Asian ideograph
        0x273d47 => [0x5f20, false], # East Asian ideograph
        0x213d48 => [0x5f37, false], # East Asian ideograph
        0x233d49 => [0x9021, false], # East Asian ideograph
        0x233d4a => [0x902d, false], # East Asian ideograph
        0x274a37 => [0x7089, false], # East Asian ideograph
        0x273d4c => [0x5f25, false], # East Asian ideograph
        0x273d4d => [0x5f2f, false], # East Asian ideograph
        0x233d4e => [0x902c, false], # East Asian ideograph
        0x273d4f => [0x6c47, false], # East Asian ideograph (duplicate simplified)
        0x223d50 => [0x692c, false], # East Asian ideograph
        0x274a38 => [0x70c2, false], # East Asian ideograph
        0x213d52 => [0x5f64, false], # East Asian ideograph
        0x223d53 => [0x690c, false], # East Asian ideograph
        0x213d54 => [0x5f6c, false], # East Asian ideograph
        0x213d55 => [0x5f69, false], # East Asian ideograph
        0x233d56 => [0x9037, false], # East Asian ideograph
        0x214a39 => [0x7228, false], # East Asian ideograph
        0x223d58 => [0x68d3, false], # East Asian ideograph
        0x213d59 => [0x5f71, false], # East Asian ideograph
        0x223d5b => [0x690a, false], # East Asian ideograph
        0x223d5c => [0x6909, false], # East Asian ideograph
        0x233d5d => [0x9052, false], # East Asian ideograph
        0x213d5e => [0x5f7f, false], # East Asian ideograph
        0x213d5f => [0x5f7c, false], # East Asian ideograph
        0x213d60 => [0x5f85, false], # East Asian ideograph
        0x213d61 => [0x5f88, false], # East Asian ideograph
        0x223d62 => [0x68ec, false], # East Asian ideograph
        0x223d63 => [0x692a, false], # East Asian ideograph
        0x223d64 => [0x68ea, false], # East Asian ideograph
        0x223d65 => [0x681f, false], # East Asian ideograph
        0x223d66 => [0x7439, false], # East Asian ideograph
        0x233d67 => [0x9049, false], # East Asian ideograph
        0x213d68 => [0x5f90, false], # East Asian ideograph
        0x234a3c => [0x9651, false], # East Asian ideograph
        0x233d6a => [0x9044, false], # East Asian ideograph
        0x213d6b => [0x5f99, false], # East Asian ideograph
        0x273d6c => [0x4ece, false], # East Asian ideograph
        0x223d6e => [0x68d6, false], # East Asian ideograph
        0x223d6f => [0x68eb, false], # East Asian ideograph
        0x225f48 => [0x761e, false], # East Asian ideograph
        0x213d71 => [0x5faa, false], # East Asian ideograph
        0x213d72 => [0x5fac, false], # East Asian ideograph
        0x223d73 => [0x68f1, false], # East Asian ideograph
        0x273d74 => [0x5f7b, false], # East Asian ideograph
        0x233d75 => [0x905d, false], # East Asian ideograph
        0x273d76 => [0x5f81, false], # East Asian ideograph
        0x213d77 => [0x5fbd, false], # East Asian ideograph
        0x233d78 => [0x905b, false], # East Asian ideograph
        0x223d79 => [0x68fc, false], # East Asian ideograph
        0x213d7a => [0x5fd9, false], # East Asian ideograph
        0x233d7b => [0x906b, false], # East Asian ideograph
        0x223d7c => [0x6913, false], # East Asian ideograph
        0x213d7d => [0x5fd6, false], # East Asian ideograph
        0x224e3c => [0x6fa8, false], # East Asian ideograph
        0x23523b => [0x99a3, false], # East Asian ideograph
        0x6f534c => [0xc1c4, false], # Korean hangul
        0x276237 => [0x9e2a, false], # East Asian ideograph
        0x224173 => [0x6a8d, false], # East Asian ideograph
        0x274a42 => [0x7237, false], # East Asian ideograph
        0x223d30 => [0x6898, false], # East Asian ideograph
        0x6f5925 => [0xcc30, false], # Korean hangul
        0x2d5c5b => [0x9089, false], # East Asian ideograph
        0x6f4a43 => [0xae4c, false], # Korean hangul
        0x234a44 => [0x965c, false], # East Asian ideograph
        0x232435 => [0x84da, false], # East Asian ideograph
        0x696e5c => [0x91df, false], # East Asian ideograph
        0x295929 => [0x9ca3, false], # East Asian ideograph
        0x213e51 => [0x608d, false], # East Asian ideograph
        0x274a45 => [0x5c14, false], # East Asian ideograph
        0x233023 => [0x8962, false], # East Asian ideograph
        0x214a46 => [0x7246, false], # East Asian ideograph
        0x6f534d => [0xc1c8, false], # Korean hangul
        0x276238 => [0x9e2d, false], # East Asian ideograph
        0x6f5740 => [0xc81d, false], # Korean hangul
        0x213932 => [0x5947, false], # East Asian ideograph
        0x295574 => [0x9604, false], # East Asian ideograph
        0x6f4a48 => [0xae5c, false], # Korean hangul
        0x277345 => [0x556e, false], # East Asian ideograph
        0x6f4a49 => [0xae5d, false], # Korean hangul
        0x29592a => [0x9cd3, false], # East Asian ideograph
        0x4b4352 => [0x66f5, false], # East Asian ideograph
        0x234a4a => [0x965f, false], # East Asian ideograph
        0x234a4b => [0x9656, false], # East Asian ideograph
        0x6f534e => [0xc1d7, false], # Korean hangul
        0x213d76 => [0x5fb5, false], # East Asian ideograph
        0x274a4c => [0x724d, false], # East Asian ideograph
        0x6f4a4d => [0xae65, false], # Korean hangul
        0x6f5771 => [0xc90c, false], # Korean hangul
        0x295928 => [0x9cd5, false], # East Asian ideograph
        0x6f4a4e => [0xae68, false], # Korean hangul
        0x334050 => [0x62d5, false], # East Asian ideograph
        0x23523e => [0x99a6, false], # East Asian ideograph
        0x4c2539 => [0x5d73, false], # East Asian ideograph
        0x4d3363 => [0x8c25, false], # East Asian ideograph
        0x224a50 => [0x6e27, false], # East Asian ideograph
        0x6f534f => [0xc1e0, false], # Korean hangul
        0x27623a => [0x9e33, false], # East Asian ideograph
        0x4b485f => [0x6ede, false], # East Asian ideograph (variant of 27485F which maps to 6EDE)
        0x234a51 => [0x966c, false], # East Asian ideograph
        0x23235c => [0x84b4, false], # East Asian ideograph
        0x285a47 => [0x73ae, false], # East Asian ideograph
        0x2d3165 => [0x349e, false], # East Asian ideograph (not found in unified han)
        0x6f4a52 => [0xae78, false], # Korean hangul
        0x275571 => [0x848b, false], # East Asian ideograph
        0x274a53 => [0x5b83, false], # East Asian ideograph
        0x227059 => [0x7d35, false], # East Asian ideograph
        0x33523f => [0x8b71, false], # East Asian ideograph
        0x473422 => [0x8c2a, false], # East Asian ideograph
        0x224a55 => [0x6e49, false], # East Asian ideograph
        0x213d78 => [0x5fc3, false], # East Asian ideograph
        0x213935 => [0x5951, false], # East Asian ideograph
        0x223d34 => [0x686f, false], # East Asian ideograph
        0x4b3248 => [0x50b2, false], # East Asian ideograph (not in Unicode)
        0x6f4a57 => [0xae84, false], # Korean hangul
        0x224a58 => [0x6e3c, false], # East Asian ideograph
        0x6f5d69 => [0xd749, false], # Korean hangul
        0x6f4a59 => [0xaebc, false], # Korean hangul
        0x274a5a => [0x7275, false], # East Asian ideograph
        0x6f5351 => [0xc1e8, false], # Korean hangul
        0x27623c => [0x9e3f, false], # East Asian ideograph
        0x223e21 => [0x6907, false], # East Asian ideograph
        0x213e22 => [0x5feb, false], # East Asian ideograph
        0x213e23 => [0x5fe0, false], # East Asian ideograph
        0x213e24 => [0x5ff1, false], # East Asian ideograph
        0x233e25 => [0x906f, false], # East Asian ideograph
        0x233e26 => [0x9079, false], # East Asian ideograph
        0x213e27 => [0x5ff5, false], # East Asian ideograph
        0x233e28 => [0x9076, false], # East Asian ideograph
        0x213e29 => [0x6014, false], # East Asian ideograph
        0x223e2a => [0x68de, false], # East Asian ideograph
        0x223e2b => [0x691b, false], # East Asian ideograph
        0x233e2c => [0x9085, false], # East Asian ideograph
        0x223e2d => [0x68fb, false], # East Asian ideograph
        0x213e2e => [0x601d, false], # East Asian ideograph
        0x234a5d => [0x967b, false], # East Asian ideograph
        0x213e30 => [0x6021, false], # East Asian ideograph
        0x213e31 => [0x6020, false], # East Asian ideograph
        0x213e32 => [0x6028, false], # East Asian ideograph
        0x223e33 => [0x68e1, false], # East Asian ideograph
        0x213e34 => [0x6027, false], # East Asian ideograph
        0x214a5e => [0x7296, false], # East Asian ideograph
        0x213e36 => [0x6015, false], # East Asian ideograph
        0x223e37 => [0x68d1, false], # East Asian ideograph
        0x223e38 => [0x68d0, false], # East Asian ideograph
        0x223e39 => [0x6908, false], # East Asian ideograph
        0x233e3a => [0x908b, false], # East Asian ideograph
        0x213e3b => [0x6043, false], # East Asian ideograph
        0x213e3c => [0x6065, false], # East Asian ideograph
        0x213e3d => [0x6050, false], # East Asian ideograph
        0x223e3e => [0x68e8, false], # East Asian ideograph
        0x223e3f => [0x68f0, false], # East Asian ideograph
        0x223e40 => [0x68c3, false], # East Asian ideograph
        0x214a60 => [0x729b, false], # East Asian ideograph
        0x235164 => [0x9940, false], # East Asian ideograph
        0x233e43 => [0x909b, false], # East Asian ideograph
        0x233e44 => [0x909c, false], # East Asian ideograph
        0x213e45 => [0x606f, false], # East Asian ideograph
        0x223e46 => [0x68d4, false], # East Asian ideograph
        0x274a61 => [0x728a, false], # East Asian ideograph
        0x233e48 => [0x90a1, false], # East Asian ideograph
        0x223e49 => [0x68c6, false], # East Asian ideograph
        0x213e4a => [0x608c, false], # East Asian ideograph
        0x223e4b => [0x68c7, false], # East Asian ideograph
        0x213e4c => [0x607f, false], # East Asian ideograph
        0x214a62 => [0x72a7, false], # East Asian ideograph
        0x213e4e => [0x609a, false], # East Asian ideograph
        0x213e4f => [0x6096, false], # East Asian ideograph
        0x213e50 => [0x6084, false], # East Asian ideograph
        0x233e51 => [0x90a8, false], # East Asian ideograph
        0x224828 => [0x6cce, false], # East Asian ideograph
        0x234a63 => [0x9684, false], # East Asian ideograph
        0x233e54 => [0x90a0, false], # East Asian ideograph
        0x223e55 => [0x6938, false], # East Asian ideograph
        0x213e56 => [0x60a8, false], # East Asian ideograph
        0x273e57 => [0x5ff0, false], # East Asian ideograph
        0x233e58 => [0x90af, false], # East Asian ideograph
        0x233e59 => [0x90b3, false], # East Asian ideograph
        0x6f4c5d => [0xb2a1, false], # Korean hangul
        0x213e5b => [0x60c5, false], # East Asian ideograph (variant of 4B3E5B which maps to 60C5)
        0x273e5c => [0x95f7, false], # East Asian ideograph
        0x223e5d => [0x6958, false], # East Asian ideograph
        0x273e5e => [0x6005, false], # East Asian ideograph
        0x213e5f => [0x60bb, false], # East Asian ideograph
        0x213e60 => [0x60e0, false], # East Asian ideograph
        0x233e61 => [0x90b2, false], # East Asian ideograph
        0x213e62 => [0x60dc, false], # East Asian ideograph
        0x213e63 => [0x60d8, false], # East Asian ideograph
        0x223e64 => [0x6945, false], # East Asian ideograph
        0x223e65 => [0x695d, false], # East Asian ideograph
        0x223e66 => [0x6932, false], # East Asian ideograph
        0x213e67 => [0x60c6, false], # East Asian ideograph
        0x233e68 => [0x90c9, false], # East Asian ideograph
        0x223e69 => [0x696e, false], # East Asian ideograph
        0x223e6a => [0x6963, false], # East Asian ideograph
        0x223e6b => [0x6948, false], # East Asian ideograph
        0x273e6c => [0x60ec, false], # East Asian ideograph
        0x213e6d => [0x60f3, false], # East Asian ideograph
        0x223e6e => [0x6939, false], # East Asian ideograph
        0x233e6f => [0x90d5, false], # East Asian ideograph
        0x273e70 => [0x607b, false], # East Asian ideograph
        0x214a68 => [0x72c0, false], # East Asian ideograph
        0x233e72 => [0x90be, false], # East Asian ideograph
        0x223e73 => [0x6937, false], # East Asian ideograph
        0x213e74 => [0x60f9, false], # East Asian ideograph
        0x213e75 => [0x6123, false], # East Asian ideograph
        0x213e76 => [0x60f4, false], # East Asian ideograph
        0x273e77 => [0x7231, false], # East Asian ideograph
        0x233e78 => [0x90c8, false], # East Asian ideograph
        0x233e79 => [0x90c3, false], # East Asian ideograph
        0x223e7a => [0x696c, false], # East Asian ideograph
        0x223e7b => [0x694e, false], # East Asian ideograph
        0x213e7c => [0x6109, false], # East Asian ideograph
        0x224a6a => [0x6e51, false], # East Asian ideograph
        0x273e7e => [0x607c, false], # East Asian ideograph
        0x234137 => [0x91a8, false], # East Asian ideograph
        0x224a6b => [0x6e44, false], # East Asian ideograph
        0x275576 => [0x8361, false], # East Asian ideograph
        0x27734c => [0x5456, false], # East Asian ideograph
        0x21385b => [0x5879, false], # East Asian ideograph
        0x293b5b => [0x8f81, false], # East Asian ideograph
        0x234a6d => [0x9682, false], # East Asian ideograph
        0x234a6e => [0x9683, false], # East Asian ideograph
        0x6f5355 => [0xc1fc, false], # Korean hangul
        0x335238 => [0x8989, false], # East Asian ideograph
        0x694c68 => [0x5301, false], # East Asian ideograph
        0x21393a => [0x5958, false], # East Asian ideograph
        0x2d5c5a => [0x8fe9, false], # East Asian ideograph
        0x2d3a41 => [0x5afa, false], # East Asian ideograph
        0x214a70 => [0x72f9, false], # East Asian ideograph
        0x6f4a71 => [0xaf48, false], # Korean hangul
        0x213f2d => [0x6177, false], # East Asian ideograph
        0x295932 => [0x9cd8, false], # East Asian ideograph
        0x274a72 => [0x72c8, false], # East Asian ideograph
        0x23302c => [0x895c, false], # East Asian ideograph
        0x276239 => [0x9e2f, false], # East Asian ideograph
        0x6f4a73 => [0xaf4c, false], # Korean hangul
        0x6f5356 => [0xc1fd, false], # Korean hangul
        0x276241 => [0x9e45, false], # East Asian ideograph
        0x21393b => [0x595a, false], # East Asian ideograph
        0x4b324e => [0x50e7, false], # East Asian ideograph (variant of 21324E which maps to 50E7)
        0x2e4e72 => [0x6f74, false], # East Asian ideograph
        0x6f5774 => [0xc911, false], # Korean hangul
        0x6f4a75 => [0xaf5c, false], # Korean hangul
        0x6f4a76 => [0xaf5d, false], # Korean hangul
        0x213521 => [0x53c9, false], # East Asian ideograph
        0x224a77 => [0x6e4e, false], # East Asian ideograph
        0x23302d => [0x895d, false], # East Asian ideograph
        0x4b4a78 => [0x72f0, false], # East Asian ideograph
        0x6f5357 => [0xc200, false], # Korean hangul
        0x213523 => [0x53ca, false], # East Asian ideograph
        0x276242 => [0x9e51, false], # East Asian ideograph
        0x275d49 => [0x94ae, false], # East Asian ideograph
        0x274a79 => [0x72b9, false], # East Asian ideograph
        0x223d3b => [0x6874, false], # East Asian ideograph
        0x234a7a => [0x9697, false], # East Asian ideograph
        0x224d68 => [0x6f5d, false], # East Asian ideograph
        0x6f4a7b => [0xaf84, false], # Korean hangul
        0x213526 => [0x53d4, false], # East Asian ideograph
        0x227061 => [0x7d3a, false], # East Asian ideograph
        0x6f4a7c => [0xaf88, false], # Korean hangul
        0x225a30 => [0x7415, false], # East Asian ideograph
        0x213527 => [0x53d7, false], # East Asian ideograph
        0x227c49 => [0x82bc, false], # East Asian ideograph
        0x6f4a7d => [0xaf90, false], # Korean hangul
        0x6f5358 => [0xc204, false], # Korean hangul
        0x6f7623 => [0x317f, false], # Korean hangul
        0x274a7e => [0x72f1, false], # East Asian ideograph
        0x223d3c => [0x6875, false], # East Asian ideograph
        0x227d2b => [0x8344, false], # East Asian ideograph
        0x21352a => [0x66fc, false], # East Asian ideograph
        0x213936 => [0x594e, false], # East Asian ideograph
        0x21352b => [0x53e2, false], # East Asian ideograph
        0x2d4b5b => [0x78af, false], # East Asian ideograph
        0x6f5359 => [0xc20d, false], # Korean hangul
        0x23352d => [0x8b95, false], # East Asian ideograph
        0x276244 => [0x9e4c, false], # East Asian ideograph
        0x23352e => [0x8b94, false], # East Asian ideograph
        0x3a7970 => [0x81d5, false], # East Asian ideograph
        0x21352f => [0x53ee, false], # East Asian ideograph
        0x6f5331 => [0xc138, false], # Korean hangul
        0x275138 => [0x7edc, false], # East Asian ideograph
        0x223f21 => [0x6952, false], # East Asian ideograph
        0x213f22 => [0x6168, false], # East Asian ideograph
        0x233f23 => [0x90df, false], # East Asian ideograph
        0x213f24 => [0x613c, false], # East Asian ideograph
        0x223f25 => [0x695b, false], # East Asian ideograph
        0x233f26 => [0x90e2, false], # East Asian ideograph
        0x223531 => [0x64eb, false], # East Asian ideograph
        0x233f28 => [0x90db, false], # East Asian ideograph
        0x273f29 => [0x5ffe, false], # East Asian ideograph
        0x233f2a => [0x90dc, false], # East Asian ideograph
        0x273f2b => [0x6006, false], # East Asian ideograph
        0x233f2c => [0x90d7, false], # East Asian ideograph
        0x233f2d => [0x90e4, false], # East Asian ideograph
        0x233f2e => [0x90ef, false], # East Asian ideograph
        0x233f2f => [0x90ea, false], # East Asian ideograph
        0x213f30 => [0x6170, false], # East Asian ideograph
        0x213f31 => [0x615a, false], # East Asian ideograph
        0x233f32 => [0x90f0, false], # East Asian ideograph
        0x233f33 => [0x90f4, false], # East Asian ideograph
        0x233f34 => [0x90f2, false], # East Asian ideograph
        0x223f35 => [0x6978, false], # East Asian ideograph
        0x273f36 => [0x8651, false], # East Asian ideograph
        0x223f37 => [0x697b, false], # East Asian ideograph
        0x273f38 => [0x60e8, false], # East Asian ideograph
        0x273f39 => [0x60ef, false], # East Asian ideograph
        0x273f3a => [0x6078, false], # East Asian ideograph
        0x273f3b => [0x6002, false], # East Asian ideograph
        0x273f3c => [0x6b32, false], # East Asian ideograph
        0x223f3d => [0x6944, false], # East Asian ideograph
        0x233f3e => [0x90eb, false], # East Asian ideograph
        0x233f3f => [0x90f3, false], # East Asian ideograph
        0x213f40 => [0x618e, false], # East Asian ideograph
        0x273f41 => [0x60af, false], # East Asian ideograph
        0x273f42 => [0x6124, false], # East Asian ideograph
        0x213f43 => [0x61ac, false], # East Asian ideograph
        0x273f44 => [0x60ee, false], # East Asian ideograph
        0x273f45 => [0x6187, false], # East Asian ideograph
        0x233f46 => [0x90fc, false], # East Asian ideograph
        0x273f47 => [0x60eb, false], # East Asian ideograph
        0x273f48 => [0x5fc6, false], # East Asian ideograph
        0x233f49 => [0x9104, false], # East Asian ideograph
        0x273f4a => [0x5e94, false], # East Asian ideograph
        0x213537 => [0x53eb, false], # East Asian ideograph
        0x233f4c => [0x9106, false], # East Asian ideograph
        0x213f4d => [0x61c2, false], # East Asian ideograph
        0x273f4e => [0x6073, false], # East Asian ideograph
        0x213f4f => [0x61c8, false], # East Asian ideograph
        0x213940 => [0x596a, false], # East Asian ideograph
        0x232368 => [0x84cd, false], # East Asian ideograph
        0x213f52 => [0x61e6, false], # East Asian ideograph
        0x213f53 => [0x61f2, false], # East Asian ideograph (variant of 4B3F53 which maps to 61F2)
        0x273f54 => [0x6000, false], # East Asian ideograph
        0x273f55 => [0x61d2, false], # East Asian ideograph
        0x273f56 => [0x60ac, false], # East Asian ideograph
        0x233f57 => [0x910f, false], # East Asian ideograph
        0x273f58 => [0x5fcf, false], # East Asian ideograph
        0x273f59 => [0x6151, false], # East Asian ideograph
        0x233f5a => [0x9116, false], # East Asian ideograph
        0x273f5b => [0x60e7, false], # East Asian ideograph
        0x233f5c => [0x9114, false], # East Asian ideograph
        0x23353a => [0x8b9f, false], # East Asian ideograph
        0x213f5e => [0x620a, false], # East Asian ideograph
        0x213f5f => [0x620e, false], # East Asian ideograph
        0x223f60 => [0x69bc, false], # East Asian ideograph
        0x223f61 => [0x69a7, false], # East Asian ideograph
        0x233f62 => [0x9123, false], # East Asian ideograph (Version J extension)
        0x233f63 => [0x9118, false], # East Asian ideograph
        0x233f64 => [0x911c, false], # East Asian ideograph
        0x213f65 => [0x6216, false], # East Asian ideograph
        0x233f66 => [0x9120, false], # East Asian ideograph
        0x233f67 => [0x9122, false], # East Asian ideograph
        0x223f68 => [0x69d9, false], # East Asian ideograph
        0x213f69 => [0x621f, false], # East Asian ideograph
        0x223f6a => [0x698e, false], # East Asian ideograph
        0x213f6b => [0x6222, false], # East Asian ideograph
        0x213f6c => [0x622a, false], # East Asian ideograph
        0x223f6d => [0x69d6, false], # East Asian ideograph
        0x273f6e => [0x6218, false], # East Asian ideograph
        0x273f6f => [0x620f, false], # East Asian ideograph
        0x213f70 => [0x6234, false], # East Asian ideograph
        0x233f71 => [0x9124, false], # East Asian ideograph
        0x233f72 => [0x911a, false], # East Asian ideograph
        0x213f73 => [0x623f, false], # East Asian ideograph
        0x233f74 => [0x9125, false], # East Asian ideograph
        0x223f75 => [0x69a5, false], # East Asian ideograph
        0x213f76 => [0x6241, false], # East Asian ideograph
        0x233f77 => [0x912f, false], # East Asian ideograph
        0x223f78 => [0x69d1, false], # East Asian ideograph
        0x27513b => [0x4e1d, false], # East Asian ideograph
        0x223f7a => [0x69f6, false], # East Asian ideograph
        0x21753f => [0x579e, false], # East Asian ideograph
        0x213f7d => [0x6253, false], # East Asian ideograph
        0x223f7e => [0x69d5, false], # East Asian ideograph
        0x217540 => [0x57b5, false], # East Asian ideograph
        0x6f535d => [0xc21c, false], # Korean hangul
        0x276248 => [0x9e5e, false], # East Asian ideograph
        0x223542 => [0x64f7, false], # East Asian ideograph
        0x213543 => [0x540c, false], # East Asian ideograph
        0x213544 => [0x540a, false], # East Asian ideograph
        0x29593a => [0x9c85, false], # East Asian ideograph
        0x23524d => [0x99bf, false], # East Asian ideograph
        0x213545 => [0x540d, false], # East Asian ideograph
        0x6f535e => [0xc21f, false], # Korean hangul
        0x223546 => [0x6504, false], # East Asian ideograph
        0x234e5e => [0x97e0, false], # East Asian ideograph
        0x2d3547 => [0x55ab, false], # East Asian ideograph
        0x6f5c66 => [0xd560, false], # Korean hangul
        0x234141 => [0x91af, false], # East Asian ideograph
        0x692436 => [0x3056, false], # Hiragana letter ZA
        0x696868 => [0x84d9, false], # East Asian ideograph
        0x233478 => [0x8b85, false], # East Asian ideograph
        0x4c354a => [0x64b8, false], # East Asian ideograph
        0x22587d => [0x73cc, false], # East Asian ideograph
        0x22354b => [0x64fd, false], # East Asian ideograph
        0x333021 => [0x58f9, false], # East Asian ideograph
        0x225f5c => [0x7633, false], # East Asian ideograph
        0x217933 => [0x5940, false], # East Asian ideograph
        0x234142 => [0x91b1, false], # East Asian ideograph
        0x23346b => [0x8b6d, false], # East Asian ideograph
        0x23354d => [0x8c4b, false], # East Asian ideograph
        0x2d7a44 => [0x598d, false], # East Asian ideograph
        0x27513e => [0x7ee2, false], # East Asian ideograph
        0x23472c => [0x93d3, false], # East Asian ideograph
        0x22354f => [0x6508, false], # East Asian ideograph
        0x395e42 => [0x9274, false], # East Asian ideograph
        0x233550 => [0x8c4f, false], # East Asian ideograph
        0x3f5f35 => [0x6b9e, false], # East Asian ideograph
        0x39303a => [0x5efc, false], # East Asian ideograph
        0x213552 => [0x543e, false], # East Asian ideograph
        0x27513f => [0x7ee5, false], # East Asian ideograph
        0x213553 => [0x5427, false], # East Asian ideograph
        0x213554 => [0x5440, false], # East Asian ideograph
        0x274476 => [0x6808, false], # East Asian ideograph
        0x233555 => [0x8c5c, false], # East Asian ideograph
        0x6f5a26 => [0xcee8, false], # Korean hangul
        0x213556 => [0x5446, false], # East Asian ideograph
        0x217557 => [0x57a1, false], # East Asian ideograph
        0x213266 => [0x512a, false], # East Asian ideograph
        0x293c30 => [0x8f98, false], # East Asian ideograph
        0x224b38 => [0x6e69, false], # East Asian ideograph
        0x6f5332 => [0xc139, false], # Korean hangul
        0x293725 => [0x8d3d, false], # East Asian ideograph
        0x287229 => [0x7f17, false], # East Asian ideograph
        0x223559 => [0x651a, false], # East Asian ideograph
        0x6f5362 => [0xc22b, false], # Korean hangul
        0x27624d => [0x9e6d, false], # East Asian ideograph
        0x214021 => [0x6252, false], # East Asian ideograph
        0x214022 => [0x625b, false], # East Asian ideograph
        0x214023 => [0x6263, false], # East Asian ideograph
        0x214024 => [0x6258, false], # East Asian ideograph
        0x214025 => [0x6296, false], # East Asian ideograph
        0x214026 => [0x6297, false], # East Asian ideograph
        0x214027 => [0x6292, false], # East Asian ideograph
        0x214028 => [0x6276, false], # East Asian ideograph
        0x214029 => [0x6289, false], # East Asian ideograph
        0x21402a => [0x627f, false], # East Asian ideograph
        0x21402b => [0x6279, false], # East Asian ideograph
        0x21402c => [0x6280, false], # East Asian ideograph
        0x21402d => [0x628a, false], # East Asian ideograph
        0x21402e => [0x626d, false], # East Asian ideograph
        0x21402f => [0x627c, false], # East Asian ideograph
        0x214030 => [0x627e, false], # East Asian ideograph
        0x214031 => [0x626f, false], # East Asian ideograph
        0x214032 => [0x6284, false], # East Asian ideograph
        0x214033 => [0x6295, false], # East Asian ideograph
        0x214034 => [0x6291, false], # East Asian ideograph
        0x214035 => [0x6298, false], # East Asian ideograph
        0x214036 => [0x626e, false], # East Asian ideograph
        0x214037 => [0x6273, false], # East Asian ideograph
        0x214038 => [0x6293, false], # East Asian ideograph
        0x214039 => [0x62c9, false], # East Asian ideograph
        0x21403a => [0x62c4, false], # East Asian ideograph
        0x21403b => [0x62cc, false], # East Asian ideograph
        0x21403c => [0x62a8, false], # East Asian ideograph
        0x21403d => [0x62dc, false], # East Asian ideograph
        0x21403e => [0x62bf, false], # East Asian ideograph
        0x21403f => [0x62c2, false], # East Asian ideograph
        0x214040 => [0x62b9, false], # East Asian ideograph
        0x214041 => [0x62d2, false], # East Asian ideograph
        0x214042 => [0x62d3, false], # East Asian ideograph
        0x214043 => [0x62db, false], # East Asian ideograph
        0x214044 => [0x62ab, false], # East Asian ideograph
        0x214045 => [0x62cb, false], # East Asian ideograph
        0x214046 => [0x62d4, false], # East Asian ideograph
        0x214047 => [0x62bd, false], # East Asian ideograph
        0x214048 => [0x62bc, false], # East Asian ideograph
        0x214049 => [0x62d0, false], # East Asian ideograph (variant of 4B4049 which maps to 62D0)
        0x21404a => [0x62c8, false], # East Asian ideograph
        0x21404b => [0x62d9, false], # East Asian ideograph
        0x21404c => [0x62da, false], # East Asian ideograph
        0x21404d => [0x62ac, false], # East Asian ideograph
        0x21404e => [0x62c7, false], # East Asian ideograph
        0x21404f => [0x62b1, false], # East Asian ideograph
        0x214050 => [0x62d6, false], # East Asian ideograph
        0x214051 => [0x62d8, false], # East Asian ideograph
        0x214052 => [0x62cd, false], # East Asian ideograph
        0x214053 => [0x62b5, false], # East Asian ideograph
        0x214054 => [0x62ce, false], # East Asian ideograph
        0x214055 => [0x62d7, false], # East Asian ideograph
        0x214056 => [0x62c6, false], # East Asian ideograph
        0x214057 => [0x6309, false], # East Asian ideograph
        0x214058 => [0x6316, false], # East Asian ideograph
        0x214059 => [0x62fc, false], # East Asian ideograph
        0x21405a => [0x62f3, false], # East Asian ideograph
        0x21405b => [0x6308, false], # East Asian ideograph
        0x21405c => [0x62ed, false], # East Asian ideograph
        0x21405d => [0x6301, false], # East Asian ideograph
        0x21405e => [0x62ee, false], # East Asian ideograph
        0x21405f => [0x62ef, false], # East Asian ideograph
        0x214060 => [0x62f7, false], # East Asian ideograph
        0x214061 => [0x6307, false], # East Asian ideograph
        0x214062 => [0x62f1, false], # East Asian ideograph
        0x214063 => [0x62fd, false], # East Asian ideograph
        0x214064 => [0x6311, false], # East Asian ideograph
        0x214065 => [0x62ec, false], # East Asian ideograph
        0x214066 => [0x62f4, false], # East Asian ideograph (variant of 4B4066 which maps to 62F4)
        0x214067 => [0x62ff, false], # East Asian ideograph
        0x224068 => [0x6a2d, false], # East Asian ideograph
        0x214069 => [0x6342, false], # East Asian ideograph
        0x21406a => [0x632a, false], # East Asian ideograph
        0x21406b => [0x6355, false], # East Asian ideograph
        0x21406c => [0x633e, false], # East Asian ideograph
        0x21406d => [0x632f, false], # East Asian ideograph
        0x21406e => [0x634e, false], # East Asian ideograph
        0x21406f => [0x634f, false], # East Asian ideograph
        0x214070 => [0x6350, false], # East Asian ideograph
        0x214071 => [0x6349, false], # East Asian ideograph
        0x224072 => [0x6a1d, false], # East Asian ideograph
        0x214073 => [0x632b, false], # East Asian ideograph
        0x214074 => [0x6328, false], # East Asian ideograph
        0x214075 => [0x633a, false], # East Asian ideograph
        0x214076 => [0x63a5, false], # East Asian ideograph
        0x214077 => [0x6369, false], # East Asian ideograph
        0x214078 => [0x63a0, false], # East Asian ideograph
        0x214079 => [0x6396, false], # East Asian ideograph
        0x21407a => [0x63a7, false], # East Asian ideograph
        0x21407b => [0x6372, false], # East Asian ideograph
        0x21407c => [0x6377, false], # East Asian ideograph
        0x21407d => [0x6383, false], # East Asian ideograph
        0x21407e => [0x636b, false], # East Asian ideograph
        0x2d5c74 => [0x96a3, false], # East Asian ideograph
        0x2d5831 => [0x89a7, false], # East Asian ideograph
        0x21756c => [0x57be, false], # East Asian ideograph
        0x693729 => [0x7c82, false], # East Asian ideograph
        0x23356d => [0x8c73, false], # East Asian ideograph
        0x6f5966 => [0xce5c, false], # Korean hangul
        0x29252d => [0x8311, false], # East Asian ideograph
        0x6f5366 => [0xc232, false], # Korean hangul
        0x276251 => [0x76d0, false], # East Asian ideograph
        0x6f5463 => [0xc46c, false], # Korean hangul
        0x6f4f23 => [0xb800, false], # Korean hangul
        0x21356f => [0x547b, false], # East Asian ideograph
        0x6f4c71 => [0xb2eb, false], # Korean hangul
        0x233571 => [0x8c75, false], # East Asian ideograph
        0x28722a => [0x7f02, false], # East Asian ideograph
        0x213572 => [0x5484, false], # East Asian ideograph
        0x6f4a54 => [0xae7b, false], # Korean hangul
        0x39593f => [0x8a3c, false], # East Asian ideograph
        0x233573 => [0x8c77, false], # East Asian ideograph
        0x276252 => [0x7877, false], # East Asian ideograph
        0x213d7b => [0x5fd8, false], # East Asian ideograph
        0x6f4f24 => [0xb801, false], # Korean hangul
        0x213574 => [0x5468, false], # East Asian ideograph
        0x223d4b => [0x68b4, false], # East Asian ideograph
        0x692568 => [0x30e8, false], # Katakana letter YO
        0x213575 => [0x5486, false], # East Asian ideograph
        0x213939 => [0x5957, false], # East Asian ideograph
        0x6f5a2f => [0xcf01, false], # Korean hangul
        0x393b6e => [0x5c97, false], # East Asian ideograph
        0x2d6021 => [0x978c, false], # East Asian ideograph
        0x335652 => [0x87c1, false], # East Asian ideograph
        0x223577 => [0x652e, false], # East Asian ideograph
        0x216022 => [0x978b, false], # East Asian ideograph
        0x216023 => [0x978f, false], # East Asian ideograph
        0x215b66 => [0x8fc6, false], # East Asian ideograph
        0x694838 => [0x567a, false], # East Asian ideograph
        0x216024 => [0x9798, false], # East Asian ideograph
        0x4b5036 => [0x7c14, false], # East Asian ideograph
        0x277360 => [0x5181, false], # East Asian ideograph
        0x21357b => [0x5471, false], # East Asian ideograph
        0x21357c => [0x549a, false], # East Asian ideograph
        0x454e43 => [0x788c, false], # East Asian ideograph (variant of 214E43 which maps to 788C)
        0x21357d => [0x548e, false], # East Asian ideograph
        0x216028 => [0x97ad, false], # East Asian ideograph
        0x6f4f26 => [0xb808, false], # Korean hangul
        0x215724 => [0x87a2, false], # East Asian ideograph
        0x275e7b => [0x9635, false], # East Asian ideograph
        0x6f5d6e => [0xd758, false], # Korean hangul
        0x21602b => [0x97c6, false], # East Asian ideograph
        0x335259 => [0x7e59, false], # East Asian ideograph
        0x27602c => [0x97e6, false], # East Asian ideograph
        0x27623d => [0x9e3d, false], # East Asian ideograph
        0x4b4347 => [0x66a8, false], # East Asian ideograph
        0x6f536a => [0xc26c, false], # Korean hangul
        0x27602d => [0x97e7, false], # East Asian ideograph
        0x6f4f27 => [0xb809, false], # Korean hangul
        0x6f592b => [0xcc3e, false], # Korean hangul
        0x213938 => [0x5950, false], # East Asian ideograph
        0x2d3a60 => [0x6588, false], # East Asian ideograph
        0x27602f => [0x97ec, false], # East Asian ideograph
        0x214121 => [0x6367, false], # East Asian ideograph
        0x214122 => [0x6398, false], # East Asian ideograph
        0x214123 => [0x639b, false], # East Asian ideograph
        0x214124 => [0x63aa, false], # East Asian ideograph
        0x214125 => [0x6371, false], # East Asian ideograph
        0x214126 => [0x63a9, false], # East Asian ideograph
        0x214127 => [0x638c, false], # East Asian ideograph
        0x214128 => [0x6389, false], # East Asian ideograph
        0x214129 => [0x63a2, false], # East Asian ideograph
        0x21412a => [0x6399, false], # East Asian ideograph
        0x21412b => [0x63a1, false], # East Asian ideograph
        0x21412c => [0x6388, false], # East Asian ideograph
        0x21412d => [0x63ac, false], # East Asian ideograph
        0x21412e => [0x633d, false], # East Asian ideograph
        0x21412f => [0x6392, false], # East Asian ideograph
        0x214130 => [0x63a3, false], # East Asian ideograph
        0x214131 => [0x6376, false], # East Asian ideograph
        0x214132 => [0x638f, false], # East Asian ideograph
        0x214133 => [0x63a8, false], # East Asian ideograph
        0x214134 => [0x637b, false], # East Asian ideograph
        0x214135 => [0x6368, false], # East Asian ideograph (variant of 4B4135 which maps to 6368)
        0x214136 => [0x6384, false], # East Asian ideograph
        0x214137 => [0x6380, false], # East Asian ideograph
        0x214138 => [0x63c6, false], # East Asian ideograph
        0x214139 => [0x63c9, false], # East Asian ideograph
        0x21413a => [0x63cd, false], # East Asian ideograph
        0x21413b => [0x63e1, false], # East Asian ideograph
        0x21413c => [0x63c0, false], # East Asian ideograph
        0x21413d => [0x63e9, false], # East Asian ideograph
        0x21413e => [0x63d0, false], # East Asian ideograph
        0x21413f => [0x63da, false], # East Asian ideograph
        0x214140 => [0x63d6, false], # East Asian ideograph
        0x214141 => [0x63ed, false], # East Asian ideograph
        0x214142 => [0x63ee, false], # East Asian ideograph
        0x214143 => [0x63cf, false], # East Asian ideograph
        0x214144 => [0x63e3, false], # East Asian ideograph
        0x214145 => [0x63f4, false], # East Asian ideograph
        0x214146 => [0x63db, false], # East Asian ideograph (variant of 454146 which maps to 63DB)
        0x214147 => [0x63d2, false], # East Asian ideograph
        0x234148 => [0x91ae, false], # East Asian ideograph
        0x214149 => [0x641e, false], # East Asian ideograph
        0x21414a => [0x642a, false], # East Asian ideograph
        0x23414b => [0x91b4, false], # East Asian ideograph
        0x23414c => [0x91b2, false], # East Asian ideograph
        0x21414d => [0x640f, false], # East Asian ideograph
        0x21414e => [0x6414, false], # East Asian ideograph
        0x21414f => [0x640d, false], # East Asian ideograph
        0x214150 => [0x642d, false], # East Asian ideograph
        0x214151 => [0x643d, false], # East Asian ideograph
        0x214152 => [0x6416, false], # East Asian ideograph
        0x214153 => [0x6417, false], # East Asian ideograph
        0x214154 => [0x641c, false], # East Asian ideograph
        0x214155 => [0x6436, false], # East Asian ideograph
        0x214156 => [0x642c, false], # East Asian ideograph
        0x214157 => [0x6458, false], # East Asian ideograph
        0x214158 => [0x6469, false], # East Asian ideograph
        0x214159 => [0x6454, false], # East Asian ideograph
        0x21415a => [0x6452, false], # East Asian ideograph
        0x21415b => [0x646f, false], # East Asian ideograph
        0x21415c => [0x6478, false], # East Asian ideograph
        0x21415d => [0x6479, false], # East Asian ideograph
        0x21415e => [0x647a, false], # East Asian ideograph
        0x21415f => [0x645f, false], # East Asian ideograph
        0x214160 => [0x6451, false], # East Asian ideograph
        0x214161 => [0x6467, false], # East Asian ideograph
        0x214162 => [0x649e, false], # East Asian ideograph
        0x214163 => [0x64a4, false], # East Asian ideograph
        0x214164 => [0x6487, false], # East Asian ideograph
        0x214165 => [0x6488, false], # East Asian ideograph
        0x214166 => [0x64a5, false], # East Asian ideograph
        0x214167 => [0x64b0, false], # East Asian ideograph
        0x214168 => [0x6493, false], # East Asian ideograph
        0x214169 => [0x6495, false], # East Asian ideograph
        0x21416a => [0x6492, false], # East Asian ideograph
        0x21416b => [0x64a9, false], # East Asian ideograph
        0x21416c => [0x6491, false], # East Asian ideograph
        0x21416d => [0x64ae, false], # East Asian ideograph
        0x21416e => [0x64b2, false], # East Asian ideograph
        0x21416f => [0x64ad, false], # East Asian ideograph
        0x214170 => [0x649a, false], # East Asian ideograph
        0x214171 => [0x64ab, false], # East Asian ideograph
        0x214172 => [0x64ac, false], # East Asian ideograph
        0x214173 => [0x64c5, false], # East Asian ideograph
        0x214174 => [0x64c1, false], # East Asian ideograph
        0x214175 => [0x64d8, false], # East Asian ideograph
        0x214176 => [0x64ca, false], # East Asian ideograph
        0x214177 => [0x64bb, false], # East Asian ideograph
        0x214178 => [0x64c2, false], # East Asian ideograph
        0x214179 => [0x64bc, false], # East Asian ideograph
        0x21417a => [0x64cb, false], # East Asian ideograph
        0x21417b => [0x64cd, false], # East Asian ideograph
        0x21417c => [0x64da, false], # East Asian ideograph
        0x21417d => [0x64c4, false], # East Asian ideograph
        0x21417e => [0x64c7, false], # East Asian ideograph
        0x705c50 => [0x82c4, false], # East Asian ideograph
        0x216040 => [0x9813, false], # East Asian ideograph
        0x393e61 => [0x60aa, false], # East Asian ideograph
        0x6f536e => [0xc27d, false], # Korean hangul
        0x216041 => [0x9812, false], # East Asian ideograph
        0x2d3571 => [0x546a, false], # East Asian ideograph
        0x6f4f2b => [0xb819, false], # Korean hangul
        0x29483e => [0x9554, false], # East Asian ideograph
        0x276042 => [0x9882, false], # East Asian ideograph
        0x276043 => [0x9887, false], # East Asian ideograph
        0x6f5d6f => [0xd759, false], # Korean hangul
        0x276044 => [0x9886, false], # East Asian ideograph
        0x69594b => [0x6327, false], # East Asian ideograph
        0x276045 => [0x9889, false], # East Asian ideograph
        0x27623e => [0x9e49, false], # East Asian ideograph
        0x6f536f => [0xc27f, false], # Korean hangul
        0x276046 => [0x5934, false], # East Asian ideograph
        0x6f4f2c => [0xb81b, false], # Korean hangul
        0x213954 => [0x5999, false], # East Asian ideograph
        0x29483f => [0x9572, false], # East Asian ideograph
        0x236047 => [0x9f76, false], # East Asian ideograph
        0x6f5775 => [0xc918, false], # Korean hangul
        0x216048 => [0x9838, false], # East Asian ideograph
        0x6f503a => [0xbaa8, false], # Korean hangul
        0x216049 => [0x983b, false], # East Asian ideograph
        0x213e58 => [0x60e6, false], # East Asian ideograph
        0x222c47 => [0x60d3, false], # East Asian ideograph
        0x21604a => [0x9839, false], # East Asian ideograph
        0x6f5370 => [0xc281, false], # Korean hangul
        0x27625b => [0x9ea6, false], # East Asian ideograph
        0x27604b => [0x9894, false], # East Asian ideograph
        0x6f4f2d => [0xb81d, false], # Korean hangul
        0x27604c => [0x9890, false], # East Asian ideograph
        0x225b2a => [0x748a, false], # East Asian ideograph
        0x27604d => [0x9897, false], # East Asian ideograph
        0x213269 => [0x5132, false], # East Asian ideograph
        0x4c725d => [0x7a39, false], # East Asian ideograph
        0x27604e => [0x989c, false], # East Asian ideograph
        0x27604f => [0x989d, false], # East Asian ideograph
        0x2d4730 => [0x51b5, false], # East Asian ideograph
        0x27625c => [0x9eb8, false], # East Asian ideograph
        0x276050 => [0x9898, false], # East Asian ideograph
        0x276051 => [0x989a, false], # East Asian ideograph
        0x216052 => [0x9853, false], # East Asian ideograph
        0x393b78 => [0x5cc4, false], # East Asian ideograph (duplicate simplified)
        0x276053 => [0x7c7b, false], # East Asian ideograph
        0x284f39 => [0x6cf8, false], # East Asian ideograph
        0x276054 => [0x98a0, false], # East Asian ideograph
        0x6f5372 => [0xc289, false], # Korean hangul
        0x216055 => [0x9858, false], # East Asian ideograph
        0x6f4f2f => [0xb825, false], # Korean hangul
        0x4b4866 => [0x6e89, false], # East Asian ideograph
        0x276056 => [0x987e, false], # East Asian ideograph
        0x69243a => [0x305a, false], # Hiragana letter ZU
        0x276057 => [0x98a4, false], # East Asian ideograph
        0x276058 => [0x663e, false], # East Asian ideograph
        0x213861 => [0x589c, false], # East Asian ideograph
        0x4b614d => [0x9a13, false], # East Asian ideograph
        0x216059 => [0x9871, false], # East Asian ideograph
        0x4c4333 => [0x6aaa, false], # East Asian ideograph
        0x27625e => [0x9762, false], # East Asian ideograph
        0x27605a => [0x98a6, false], # East Asian ideograph
        0x6f4f30 => [0xb828, false], # Korean hangul
        0x214221 => [0x64ce, false], # East Asian ideograph
        0x214222 => [0x64d4, false], # East Asian ideograph
        0x214223 => [0x64d2, false], # East Asian ideograph
        0x214224 => [0x64bf, false], # East Asian ideograph
        0x234225 => [0x9201, false], # East Asian ideograph
        0x214226 => [0x64f0, false], # East Asian ideograph
        0x214227 => [0x64e6, false], # East Asian ideograph
        0x214228 => [0x64ec, false], # East Asian ideograph
        0x214229 => [0x64f1, false], # East Asian ideograph
        0x21422a => [0x64f4, false], # East Asian ideograph
        0x21422b => [0x64f2, false], # East Asian ideograph
        0x21422c => [0x6506, false], # East Asian ideograph
        0x21422d => [0x6500, false], # East Asian ideograph
        0x27422e => [0x6270, false], # East Asian ideograph
        0x21422f => [0x64fb, false], # East Asian ideograph
        0x214230 => [0x64fa, false], # East Asian ideograph
        0x214231 => [0x650f, false], # East Asian ideograph
        0x214232 => [0x6518, false], # East Asian ideograph
        0x214233 => [0x6514, false], # East Asian ideograph
        0x214234 => [0x6519, false], # East Asian ideograph
        0x214235 => [0x651d, false], # East Asian ideograph
        0x214236 => [0x651c, false], # East Asian ideograph
        0x214237 => [0x6523, false], # East Asian ideograph
        0x214238 => [0x6524, false], # East Asian ideograph
        0x214239 => [0x652b, false], # East Asian ideograph
        0x21423a => [0x652a, false], # East Asian ideograph
        0x21423b => [0x652c, false], # East Asian ideograph
        0x21423c => [0x652f, false], # East Asian ideograph
        0x21423d => [0x6536, false], # East Asian ideograph
        0x21423e => [0x6539, false], # East Asian ideograph
        0x21423f => [0x653b, false], # East Asian ideograph
        0x214240 => [0x653e, false], # East Asian ideograph
        0x214241 => [0x653f, false], # East Asian ideograph
        0x214242 => [0x6545, false], # East Asian ideograph
        0x214243 => [0x6548, false], # East Asian ideograph
        0x214244 => [0x654e, false], # East Asian ideograph
        0x214245 => [0x6556, false], # East Asian ideograph
        0x214246 => [0x6551, false], # East Asian ideograph
        0x274247 => [0x8d25, false], # East Asian ideograph
        0x214248 => [0x655d, false], # East Asian ideograph
        0x214249 => [0x6558, false], # East Asian ideograph
        0x21424a => [0x654f, false], # East Asian ideograph
        0x21424b => [0x6566, false], # East Asian ideograph
        0x21424c => [0x6562, false], # East Asian ideograph
        0x21424d => [0x6563, false], # East Asian ideograph
        0x21424e => [0x655e, false], # East Asian ideograph
        0x21424f => [0x5553, false], # East Asian ideograph
        0x214250 => [0x656c, false], # East Asian ideograph
        0x214251 => [0x6572, false], # East Asian ideograph
        0x214252 => [0x6575, false], # East Asian ideograph
        0x214253 => [0x6577, false], # East Asian ideograph
        0x214254 => [0x6578, false], # East Asian ideograph
        0x214255 => [0x6574, false], # East Asian ideograph
        0x214256 => [0x6582, false], # East Asian ideograph
        0x214257 => [0x6583, false], # East Asian ideograph
        0x214258 => [0x6587, false], # East Asian ideograph
        0x214259 => [0x6591, false], # East Asian ideograph
        0x21425a => [0x6590, false], # East Asian ideograph
        0x6f4f32 => [0xb834, false], # Korean hangul
        0x21425c => [0x6599, false], # East Asian ideograph
        0x21425d => [0x659c, false], # East Asian ideograph
        0x21425e => [0x659f, false], # East Asian ideograph
        0x21425f => [0x65a1, false], # East Asian ideograph
        0x214260 => [0x65a4, false], # East Asian ideograph
        0x214261 => [0x65a5, false], # East Asian ideograph
        0x214262 => [0x65a7, false], # East Asian ideograph
        0x274263 => [0x65a9, false], # East Asian ideograph
        0x214264 => [0x65af, false], # East Asian ideograph
        0x214265 => [0x65b0, false], # East Asian ideograph
        0x274266 => [0x65ad, false], # East Asian ideograph
        0x214267 => [0x65b9, false], # East Asian ideograph
        0x224268 => [0x6ab4, false], # East Asian ideograph
        0x214269 => [0x65bd, false], # East Asian ideograph
        0x21426a => [0x65c1, false], # East Asian ideograph
        0x21426b => [0x65c5, false], # East Asian ideograph
        0x21426c => [0x65ce, false], # East Asian ideograph
        0x21426d => [0x65cb, false], # East Asian ideograph
        0x21426e => [0x65cc, false], # East Asian ideograph
        0x21426f => [0x65cf, false], # East Asian ideograph
        0x214270 => [0x65d7, false], # East Asian ideograph
        0x214271 => [0x65d6, false], # East Asian ideograph
        0x214272 => [0x65e2, false], # East Asian ideograph
        0x214273 => [0x65e5, false], # East Asian ideograph
        0x234274 => [0x923f, false], # East Asian ideograph
        0x214275 => [0x65e9, false], # East Asian ideograph
        0x214276 => [0x65ec, false], # East Asian ideograph
        0x214277 => [0x65ed, false], # East Asian ideograph
        0x214278 => [0x65e8, false], # East Asian ideograph
        0x214279 => [0x65f1, false], # East Asian ideograph
        0x21427a => [0x65fa, false], # East Asian ideograph
        0x21427b => [0x6606, false], # East Asian ideograph
        0x21427c => [0x6614, false], # East Asian ideograph
        0x21427d => [0x660c, false], # East Asian ideograph
        0x21427e => [0x6600, false], # East Asian ideograph
        0x6f5779 => [0xc954, false], # Korean hangul
        0x21606b => [0x98ef, false], # East Asian ideograph
        0x27606c => [0x9972, false], # East Asian ideograph
        0x453f6d => [0x52e0, false], # East Asian ideograph
        0x29373a => [0x8d46, false], # East Asian ideograph
        0x22606d => [0x76ad, false], # East Asian ideograph
        0x6f5377 => [0xc2a4, false], # Korean hangul
        0x27606e => [0x9971, false], # East Asian ideograph
        0x6f4f34 => [0xb837, false], # Korean hangul
        0x21395c => [0x59b9, false], # East Asian ideograph
        0x27606f => [0x9970, false], # East Asian ideograph
        0x69243b => [0x305b, false], # Hiragana letter SE
        0x276070 => [0x997a, false], # East Asian ideograph
        0x2d362a => [0x95a7, false], # East Asian ideograph
        0x275156 => [0x7f04, false], # East Asian ideograph
        0x277272 => [0x54d2, false], # East Asian ideograph
        0x236071 => [0x9fa5, false], # East Asian ideograph
        0x213862 => [0x58ae, false], # East Asian ideograph
        0x216072 => [0x990c, false], # East Asian ideograph
        0x6f5378 => [0xc2a5, false], # Korean hangul
        0x276073 => [0x9977, false], # East Asian ideograph
        0x215b76 => [0x8ff7, false], # East Asian ideograph
        0x21395d => [0x59c6, false], # East Asian ideograph
        0x216074 => [0x9910, false], # East Asian ideograph
        0x276075 => [0x9981, false], # East Asian ideograph
        0x6f5d71 => [0xd761, false], # Korean hangul
        0x276076 => [0x4f59, false], # East Asian ideograph
        0x295955 => [0x9c8e, false], # East Asian ideograph
        0x6f4b21 => [0xaf9c, false], # Korean hangul
        0x216077 => [0x9913, false], # East Asian ideograph
        0x214b22 => [0x733e, false], # East Asian ideograph
        0x2d4738 => [0x6ffc, false], # East Asian ideograph
        0x276078 => [0x997c, false], # East Asian ideograph
        0x214b23 => [0x7345, false], # East Asian ideograph
        0x276079 => [0x9986, false], # East Asian ideograph
        0x4b553f => [0x83bd, false], # East Asian ideograph (variant of 21553F which maps to 83BD)
        0x224b24 => [0x6e5c, false], # East Asian ideograph
        0x27607a => [0x996f, false], # East Asian ideograph
        0x27607b => [0x9984, false], # East Asian ideograph
        0x224e6a => [0x700d, false], # East Asian ideograph
        0x27607c => [0x9985, false], # East Asian ideograph
        0x214b27 => [0x7368, false], # East Asian ideograph
        0x6f537a => [0xc2ac, false], # Korean hangul
        0x217334 => [0x5686, false], # East Asian ideograph
        0x214b28 => [0x7370, false], # East Asian ideograph
        0x29484a => [0x956c, false], # East Asian ideograph
        0x27607e => [0x998f, false], # East Asian ideograph
        0x6f5d6d => [0xd757, false], # Korean hangul
        0x214b29 => [0x7372, false], # East Asian ideograph
        0x4c5447 => [0x71e0, false], # East Asian ideograph (variant of 225447 which maps to 71E0)
        0x214b2a => [0x7377, false], # East Asian ideograph
        0x2e7374 => [0x7e89, false], # East Asian ideograph
        0x214b2b => [0x7378, false], # East Asian ideograph
        0x2e6060 => [0x76a1, false], # East Asian ideograph
        0x214b2c => [0x7375, false], # East Asian ideograph
        0x6f537b => [0xc2ad, false], # Korean hangul
        0x214b2d => [0x737a, false], # East Asian ideograph
        0x213960 => [0x59af, false], # East Asian ideograph
        0x286222 => [0x7726, false], # East Asian ideograph
        0x214b2e => [0x737b, false], # East Asian ideograph
        0x335f34 => [0x90c4, false], # East Asian ideograph
        0x21393d => [0x5962, false], # East Asian ideograph
        0x274b2f => [0x7321, false], # East Asian ideograph
        0x295958 => [0x9c9a, false], # East Asian ideograph
        0x214321 => [0x660e, false], # East Asian ideograph
        0x214322 => [0x6613, false], # East Asian ideograph
        0x214323 => [0x6602, false], # East Asian ideograph
        0x214324 => [0x660f, false], # East Asian ideograph
        0x214325 => [0x6625, false], # East Asian ideograph
        0x214326 => [0x6627, false], # East Asian ideograph
        0x214327 => [0x662f, false], # East Asian ideograph
        0x214328 => [0x662d, false], # East Asian ideograph
        0x214329 => [0x6620, false], # East Asian ideograph
        0x21432a => [0x661f, false], # East Asian ideograph
        0x21432b => [0x6628, false], # East Asian ideograph
        0x21432c => [0x664f, false], # East Asian ideograph
        0x21432d => [0x6642, false], # East Asian ideograph
        0x21432e => [0x6652, false], # East Asian ideograph
        0x21432f => [0x6649, false], # East Asian ideograph
        0x214330 => [0x6643, false], # East Asian ideograph
        0x214331 => [0x664c, false], # East Asian ideograph
        0x214332 => [0x665d, false], # East Asian ideograph
        0x214333 => [0x6664, false], # East Asian ideograph
        0x214334 => [0x6668, false], # East Asian ideograph
        0x214335 => [0x6666, false], # East Asian ideograph
        0x214336 => [0x665a, false], # East Asian ideograph
        0x214337 => [0x666f, false], # East Asian ideograph
        0x214338 => [0x666e, false], # East Asian ideograph
        0x214339 => [0xfa12, false], # East Asian ideograph
        0x21433a => [0x6691, false], # East Asian ideograph
        0x21433b => [0x6670, false], # East Asian ideograph
        0x21433c => [0x6676, false], # East Asian ideograph
        0x21433d => [0x667a, false], # East Asian ideograph
        0x21433e => [0x6697, false], # East Asian ideograph
        0x21433f => [0x6687, false], # East Asian ideograph
        0x214340 => [0x6689, false], # East Asian ideograph
        0x214341 => [0x6688, false], # East Asian ideograph
        0x214342 => [0x6696, false], # East Asian ideograph
        0x214343 => [0x66a2, false], # East Asian ideograph
        0x214344 => [0x66ab, false], # East Asian ideograph
        0x214345 => [0x66b4, false], # East Asian ideograph
        0x214346 => [0x66ae, false], # East Asian ideograph
        0x214347 => [0x66c1, false], # East Asian ideograph
        0x214348 => [0x66c9, false], # East Asian ideograph
        0x214349 => [0x66c6, false], # East Asian ideograph
        0x21434a => [0x66b9, false], # East Asian ideograph
        0x21434b => [0x66d6, false], # East Asian ideograph
        0x21434c => [0x66d9, false], # East Asian ideograph
        0x21434d => [0x66e0, false], # East Asian ideograph
        0x21434e => [0x66dd, false], # East Asian ideograph
        0x21434f => [0x66e6, false], # East Asian ideograph
        0x214350 => [0x66f0, false], # East Asian ideograph
        0x214351 => [0x66f2, false], # East Asian ideograph
        0x214352 => [0x66f3, false], # East Asian ideograph
        0x214353 => [0x66f4, false], # East Asian ideograph
        0x214354 => [0x66f7, false], # East Asian ideograph
        0x214355 => [0x66f8, false], # East Asian ideograph
        0x214356 => [0x66f9, false], # East Asian ideograph
        0x214357 => [0x52d7, false], # East Asian ideograph
        0x214358 => [0x66fe, false], # East Asian ideograph
        0x214359 => [0x66ff, false], # East Asian ideograph
        0x21435a => [0x6703, false], # East Asian ideograph
        0x21435b => [0x6708, false], # East Asian ideograph
        0x21435c => [0x6709, false], # East Asian ideograph
        0x21435d => [0x670d, false], # East Asian ideograph
        0x21435e => [0x670b, false], # East Asian ideograph
        0x21435f => [0x6717, false], # East Asian ideograph
        0x214360 => [0x6715, false], # East Asian ideograph
        0x214361 => [0x6714, false], # East Asian ideograph
        0x214362 => [0x671b, false], # East Asian ideograph
        0x214363 => [0x671d, false], # East Asian ideograph
        0x214364 => [0x671f, false], # East Asian ideograph
        0x6f537e => [0xc2b5, false], # Korean hangul
        0x234366 => [0x92c8, false], # East Asian ideograph
        0x214367 => [0x6728, false], # East Asian ideograph
        0x214369 => [0x672c, false], # East Asian ideograph
        0x23436a => [0x92c3, false], # East Asian ideograph
        0x21436b => [0x672a, false], # East Asian ideograph
        0x29436c => [0x950d, false], # East Asian ideograph
        0x21436d => [0x673d, false], # East Asian ideograph
        0x22436e => [0x6b17, false], # East Asian ideograph
        0x21436f => [0x6731, false], # East Asian ideograph
        0x214370 => [0x6735, false], # East Asian ideograph
        0x214371 => [0x675e, false], # East Asian ideograph
        0x214372 => [0x6751, false], # East Asian ideograph
        0x214373 => [0x674e, false], # East Asian ideograph
        0x214374 => [0x675c, false], # East Asian ideograph
        0x234375 => [0x92e6, false], # East Asian ideograph
        0x214376 => [0x6756, false], # East Asian ideograph
        0x214377 => [0x675f, false], # East Asian ideograph
        0x214378 => [0x674f, false], # East Asian ideograph
        0x214379 => [0x6749, false], # East Asian ideograph
        0x23437a => [0x92d9, false], # East Asian ideograph
        0x21437b => [0x676d, false], # East Asian ideograph
        0x21437c => [0x678b, false], # East Asian ideograph
        0x21437d => [0x6795, false], # East Asian ideograph
        0x21437e => [0x6789, false], # East Asian ideograph
        0x21777b => [0x58a9, false], # East Asian ideograph
        0x4b3f40 => [0x618e, false], # East Asian ideograph (variant of 213F40)
        0x214b40 => [0x73ed, false], # East Asian ideograph
        0x27626a => [0x70b9, false], # East Asian ideograph
        0x214b41 => [0x73ee, false], # East Asian ideograph
        0x214b42 => [0x73e0, false], # East Asian ideograph
        0x214b43 => [0x7405, false], # East Asian ideograph
        0x6f4b44 => [0xb07c, false], # Korean hangul
        0x23457e => [0x938b, false], # East Asian ideograph
        0x214b45 => [0x7403, false], # East Asian ideograph
        0x336062 => [0x98c3, false], # East Asian ideograph
        0x214b46 => [0x740a, false], # East Asian ideograph
        0x517954 => [0x734e, false], # East Asian ideograph
        0x274b47 => [0x73b0, false], # East Asian ideograph
        0x6f577b => [0xc960, false], # Korean hangul
        0x214b48 => [0x7406, false], # East Asian ideograph
        0x214b49 => [0x740d, false], # East Asian ideograph
        0x282577 => [0x5ce4, false], # East Asian ideograph
        0x214b4a => [0x743a, false], # East Asian ideograph
        0x6f5338 => [0xc14d, false], # Korean hangul
        0x6f4b4b => [0xb090, false], # Korean hangul
        0x213966 => [0x59d4, false], # East Asian ideograph
        0x6f4b4c => [0xb091, false], # Korean hangul
        0x2d584d => [0x548f, false], # East Asian ideograph
        0x214b4d => [0x7434, false], # East Asian ideograph
        0x2e3a33 => [0x80ad, false], # East Asian ideograph
        0x213864 => [0x58c7, false], # East Asian ideograph (variant of 4B3864 which maps to 58C7)
        0x214b4f => [0x7433, false], # East Asian ideograph
        0x6f4b50 => [0xb099, false], # Korean hangul
        0x6f5021 => [0xba38, false], # Korean hangul
        0x214b51 => [0x7425, false], # East Asian ideograph
        0x6f4b52 => [0xb09c, false], # Korean hangul
        0x213f36 => [0x616e, false], # East Asian ideograph
        0x234b53 => [0x96ca, false], # East Asian ideograph
        0x6f4b54 => [0xb0a0, false], # Korean hangul
        0x6f4b55 => [0xb0a1, false], # Korean hangul
        0x6f4f40 => [0xb8b0, false], # Korean hangul
        0x6f5930 => [0xcc4c, false], # Korean hangul
        0x6f4b56 => [0xb0a8, false], # Korean hangul
        0x274b57 => [0x73f2, false], # East Asian ideograph
        0x6f4b58 => [0xb0ab, false], # Korean hangul
        0x214b59 => [0x745e, false], # East Asian ideograph
        0x214b5a => [0x745c, false], # East Asian ideograph
        0x6f4f41 => [0xb8cc, false], # Korean hangul
        0x214421 => [0x6787, false], # East Asian ideograph
        0x214422 => [0x6777, false], # East Asian ideograph
        0x214423 => [0x679d, false], # East Asian ideograph
        0x214424 => [0x6797, false], # East Asian ideograph
        0x214425 => [0x676f, false], # East Asian ideograph
        0x214426 => [0x6771, false], # East Asian ideograph
        0x214427 => [0x6773, false], # East Asian ideograph
        0x214428 => [0x679c, false], # East Asian ideograph
        0x214429 => [0x6775, false], # East Asian ideograph
        0x21442a => [0x679a, false], # East Asian ideograph
        0x21442b => [0x6790, false], # East Asian ideograph
        0x22442c => [0x6b37, false], # East Asian ideograph
        0x21442d => [0x677e, false], # East Asian ideograph
        0x21442e => [0x67d3, false], # East Asian ideograph
        0x21442f => [0x67f1, false], # East Asian ideograph
        0x214430 => [0x67ff, false], # East Asian ideograph
        0x214431 => [0x67d4, false], # East Asian ideograph
        0x214432 => [0x67c4, false], # East Asian ideograph
        0x214433 => [0x67af, false], # East Asian ideograph
        0x214434 => [0x67d0, false], # East Asian ideograph
        0x214435 => [0x67d1, false], # East Asian ideograph
        0x214436 => [0x67ef, false], # East Asian ideograph
        0x214437 => [0x67e9, false], # East Asian ideograph
        0x214438 => [0x67b6, false], # East Asian ideograph
        0x214439 => [0x67ec, false], # East Asian ideograph
        0x21443a => [0x67e5, false], # East Asian ideograph
        0x21443b => [0x67fa, false], # East Asian ideograph
        0x21443c => [0x67da, false], # East Asian ideograph
        0x21443d => [0x6805, false], # East Asian ideograph
        0x21443e => [0x67de, false], # East Asian ideograph
        0x21443f => [0x67b8, false], # East Asian ideograph
        0x214440 => [0x67cf, false], # East Asian ideograph
        0x214441 => [0x67f3, false], # East Asian ideograph
        0x214442 => [0x6848, false], # East Asian ideograph
        0x214443 => [0x6821, false], # East Asian ideograph
        0x214444 => [0x6838, false], # East Asian ideograph
        0x214445 => [0x6853, false], # East Asian ideograph
        0x214446 => [0x6846, false], # East Asian ideograph
        0x214447 => [0x6842, false], # East Asian ideograph
        0x214448 => [0x6854, false], # East Asian ideograph
        0x214449 => [0x6817, false], # East Asian ideograph
        0x21444a => [0x683d, false], # East Asian ideograph
        0x21444b => [0x6851, false], # East Asian ideograph
        0x21444c => [0x6829, false], # East Asian ideograph
        0x21444d => [0x6850, false], # East Asian ideograph
        0x21444e => [0x6839, false], # East Asian ideograph
        0x23444f => [0x9344, false], # East Asian ideograph
        0x214450 => [0x67f4, false], # East Asian ideograph
        0x214451 => [0x6843, false], # East Asian ideograph
        0x214452 => [0x6840, false], # East Asian ideograph
        0x214453 => [0x682a, false], # East Asian ideograph
        0x214454 => [0x6845, false], # East Asian ideograph
        0x214455 => [0x683c, false], # East Asian ideograph
        0x214456 => [0x6813, false], # East Asian ideograph (variant of 4B4456 which maps to 6813)
        0x214457 => [0x6881, false], # East Asian ideograph
        0x214458 => [0x6893, false], # East Asian ideograph
        0x214459 => [0x68af, false], # East Asian ideograph
        0x21445a => [0x6876, false], # East Asian ideograph
        0x21445b => [0x68b0, false], # East Asian ideograph
        0x21445c => [0x68a7, false], # East Asian ideograph
        0x21445d => [0x6897, false], # East Asian ideograph
        0x21445e => [0x68b5, false], # East Asian ideograph
        0x21445f => [0x68b3, false], # East Asian ideograph
        0x214460 => [0x68a2, false], # East Asian ideograph
        0x214461 => [0x687f, false], # East Asian ideograph
        0x214462 => [0x68b1, false], # East Asian ideograph
        0x214463 => [0x689d, false], # East Asian ideograph
        0x214464 => [0x68ad, false], # East Asian ideograph
        0x214465 => [0x6886, false], # East Asian ideograph
        0x234466 => [0x9312, false], # East Asian ideograph
        0x214467 => [0x68a8, false], # East Asian ideograph
        0x214468 => [0x689f, false], # East Asian ideograph
        0x214469 => [0x6894, false], # East Asian ideograph
        0x21446a => [0x6883, false], # East Asian ideograph
        0x21446b => [0x68d5, false], # East Asian ideograph
        0x21446c => [0x68fa, false], # East Asian ideograph
        0x21446d => [0x68c4, false], # East Asian ideograph
        0x21446e => [0x68f2, false], # East Asian ideograph
        0x21446f => [0x68d2, false], # East Asian ideograph
        0x214470 => [0x68e3, false], # East Asian ideograph
        0x214471 => [0x68df, false], # East Asian ideograph
        0x214472 => [0x68cb, false], # East Asian ideograph
        0x214473 => [0x68ee, false], # East Asian ideograph
        0x214474 => [0x690d, false], # East Asian ideograph
        0x214475 => [0x6905, false], # East Asian ideograph
        0x214476 => [0x68e7, false], # East Asian ideograph
        0x214477 => [0x68e0, false], # East Asian ideograph
        0x214478 => [0x68f5, false], # East Asian ideograph
        0x214479 => [0x68cd, false], # East Asian ideograph
        0x21447a => [0x68d7, false], # East Asian ideograph
        0x21447b => [0x68d8, false], # East Asian ideograph
        0x27447c => [0x832d, false], # East Asian ideograph
        0x21447d => [0x68f9, false], # East Asian ideograph
        0x21447e => [0x68da, false], # East Asian ideograph
        0x214b6b => [0x74cf, false], # East Asian ideograph
        0x275166 => [0x7ee9, false], # East Asian ideograph
        0x214b6c => [0x74dc, false], # East Asian ideograph
        0x6f2457 => [0x3131, false], # Korean hangul
        0x6f576c => [0xc8fc, false], # Korean hangul
        0x214b6d => [0x74e0, false], # East Asian ideograph
        0x6f4b6e => [0xb108, false], # Korean hangul
        0x70755d => [0x8e3a, false], # East Asian ideograph
        0x6f4b6f => [0xb109, false], # Korean hangul
        0x39525b => [0x66dc, false], # East Asian ideograph
        0x214b71 => [0x74f6, false], # East Asian ideograph
        0x4b3f4a => [0x5fdc, false], # East Asian ideograph
        0x234e35 => [0x97be, false], # East Asian ideograph
        0x29474d => [0x956b, false], # East Asian ideograph
        0x6f4b73 => [0xb10f, false], # Korean hangul
        0x6f4f46 => [0xb8f0, false], # Korean hangul
        0x214b74 => [0x750c, false], # East Asian ideograph
        0x224b75 => [0x6ea4, false], # East Asian ideograph
        0x214b76 => [0x7518, false], # East Asian ideograph
        0x4b3f4b => [0x601c, false], # East Asian ideograph (variant of 273F4B)
        0x234b77 => [0x96f4, false], # East Asian ideograph
        0x2d3622 => [0x8aee, false], # East Asian ideograph
        0x6f4d67 => [0xb4ec, false], # Korean hangul
        0x214b78 => [0x751c, false], # East Asian ideograph
        0x275e32 => [0x9556, false], # East Asian ideograph
        0x214b79 => [0x751f, false], # East Asian ideograph
        0x6f577d => [0xc96c, false], # Korean hangul
        0x335f43 => [0x9d08, false], # East Asian ideograph
        0x333d2a => [0x5e83, false], # East Asian ideograph
        0x2d5856 => [0x612c, false], # East Asian ideograph
        0x274b7a => [0x4ea7, false], # East Asian ideograph
        0x4b5437 => [0x820e, false], # East Asian ideograph
        0x694b7b => [0x9ebf, false], # East Asian ideograph
        0x213032 => [0x4e26, false], # East Asian ideograph
        0x214b7c => [0x7525, false], # East Asian ideograph
        0x6f533a => [0xc154, false], # Korean hangul
        0x276276 => [0x51ac, false], # East Asian ideograph
        0x214b7d => [0x7528, false], # East Asian ideograph
        0x6f4f48 => [0xb8f9, false], # Korean hangul
        0x275e33 => [0x9557, false], # East Asian ideograph
        0x21352d => [0x53f8, false], # East Asian ideograph
        0x217629 => [0x57e3, false], # East Asian ideograph
        0x23362a => [0x8c86, false], # East Asian ideograph
        0x213866 => [0x58c1, false], # East Asian ideograph
        0x23527b => [0x99e3, false], # East Asian ideograph
        0x21762c => [0x57f6, false], # East Asian ideograph
        0x6f4f49 => [0xb8fb, false], # Korean hangul
        0x23362d => [0x8c85, false], # East Asian ideograph
        0x29485c => [0x9565, false], # East Asian ideograph
        0x21352e => [0x53e4, false], # East Asian ideograph
        0x4d5858 => [0x9be3, false], # East Asian ideograph
        0x6f5d75 => [0xd76c, false], # Korean hangul
        0x214521 => [0x690e, false], # East Asian ideograph
        0x214522 => [0x68c9, false], # East Asian ideograph
        0x214523 => [0x6954, false], # East Asian ideograph
        0x214524 => [0x6930, false], # East Asian ideograph
        0x214525 => [0x6977, false], # East Asian ideograph
        0x214526 => [0x6975, false], # East Asian ideograph
        0x214527 => [0x695a, false], # East Asian ideograph
        0x214528 => [0x6960, false], # East Asian ideograph
        0x214529 => [0x696b, false], # East Asian ideograph
        0x21452a => [0x694a, false], # East Asian ideograph
        0x21452b => [0x6968, false], # East Asian ideograph
        0x21452c => [0x695e, false], # East Asian ideograph
        0x21452d => [0x696d, false], # East Asian ideograph
        0x21452e => [0x6979, false], # East Asian ideograph
        0x21452f => [0x6953, false], # East Asian ideograph
        0x214530 => [0x6986, false], # East Asian ideograph
        0x214531 => [0x69a8, false], # East Asian ideograph
        0x214532 => [0x6995, false], # East Asian ideograph
        0x214533 => [0x699c, false], # East Asian ideograph
        0x214534 => [0x6994, false], # East Asian ideograph
        0x214535 => [0x69c1, false], # East Asian ideograph
        0x214536 => [0x69b7, false], # East Asian ideograph
        0x214537 => [0x69ae, false], # East Asian ideograph
        0x214538 => [0x699b, false], # East Asian ideograph
        0x214539 => [0x69cb, false], # East Asian ideograph
        0x21453a => [0x69d3, false], # East Asian ideograph
        0x21453b => [0x69bb, false], # East Asian ideograph
        0x21453c => [0x69ab, false], # East Asian ideograph
        0x21453d => [0x69cc, false], # East Asian ideograph
        0x21453e => [0x69ad, false], # East Asian ideograph
        0x21453f => [0x69d0, false], # East Asian ideograph
        0x214540 => [0x69cd, false], # East Asian ideograph
        0x214541 => [0x69b4, false], # East Asian ideograph
        0x214542 => [0x6a1f, false], # East Asian ideograph
        0x214543 => [0x69e8, false], # East Asian ideograph
        0x274544 => [0x6837, false], # East Asian ideograph
        0x214545 => [0x69ea, false], # East Asian ideograph
        0x274546 => [0x6869, false], # East Asian ideograph
        0x214547 => [0x6a19, false], # East Asian ideograph
        0x214548 => [0x69fd, false], # East Asian ideograph
        0x214549 => [0x6a1e, false], # East Asian ideograph
        0x21454a => [0x6a13, false], # East Asian ideograph
        0x21454b => [0x6a21, false], # East Asian ideograph
        0x21454c => [0x69f3, false], # East Asian ideograph
        0x21454d => [0x6a0a, false], # East Asian ideograph
        0x21454e => [0x6a02, false], # East Asian ideograph
        0x21454f => [0x6a05, false], # East Asian ideograph
        0x214550 => [0x6a3d, false], # East Asian ideograph
        0x214551 => [0x6a58, false], # East Asian ideograph
        0x214552 => [0x6a59, false], # East Asian ideograph
        0x214553 => [0x6a62, false], # East Asian ideograph
        0x214554 => [0x6a44, false], # East Asian ideograph
        0x214555 => [0x6a39, false], # East Asian ideograph
        0x214556 => [0x6a6b, false], # East Asian ideograph
        0x214557 => [0x6a3a, false], # East Asian ideograph
        0x214558 => [0x6a38, false], # East Asian ideograph
        0x214559 => [0x6a47, false], # East Asian ideograph
        0x21455a => [0x6a61, false], # East Asian ideograph
        0x21455b => [0x6a4b, false], # East Asian ideograph
        0x21455c => [0x6a35, false], # East Asian ideograph
        0x21455d => [0x6a5f, false], # East Asian ideograph
        0x21455e => [0x6a80, false], # East Asian ideograph
        0x21455f => [0x6a94, false], # East Asian ideograph
        0x214560 => [0x6a84, false], # East Asian ideograph
        0x214561 => [0x6aa2, false], # East Asian ideograph
        0x214562 => [0x6a9c, false], # East Asian ideograph
        0x214563 => [0x6ab8, false], # East Asian ideograph
        0x214564 => [0x6ab3, false], # East Asian ideograph
        0x214565 => [0x6ac3, false], # East Asian ideograph
        0x214566 => [0x6abb, false], # East Asian ideograph
        0x234567 => [0x9354, false], # East Asian ideograph
        0x214568 => [0x6aac, false], # East Asian ideograph
        0x214569 => [0x6ae5, false], # East Asian ideograph
        0x21456a => [0x6ada, false], # East Asian ideograph
        0x21456b => [0x6add, false], # East Asian ideograph
        0x21456c => [0x6adb, false], # East Asian ideograph
        0x21456d => [0x6ad3, false], # East Asian ideograph
        0x21456e => [0x6b04, false], # East Asian ideograph
        0x21456f => [0x6afb, false], # East Asian ideograph
        0x214570 => [0x6b0a, false], # East Asian ideograph
        0x214571 => [0x6b16, false], # East Asian ideograph
        0x234572 => [0x936d, false], # East Asian ideograph
        0x214573 => [0x6b21, false], # East Asian ideograph
        0x214574 => [0x6b23, false], # East Asian ideograph
        0x27363e => [0x5458, false], # East Asian ideograph
        0x214576 => [0x6b3e, false], # East Asian ideograph
        0x214577 => [0x6b3a, false], # East Asian ideograph
        0x214578 => [0x6b3d, false], # East Asian ideograph
        0x214579 => [0x6b47, false], # East Asian ideograph
        0x21457a => [0x6b49, false], # East Asian ideograph
        0x21457b => [0x6b4c, false], # East Asian ideograph
        0x21457c => [0x6b50, false], # East Asian ideograph
        0x21457d => [0x6b59, false], # East Asian ideograph
        0x21457e => [0x6b5f, false], # East Asian ideograph
        0x6f7640 => [0xe8b2, false], # Korean hangul
        0x2d3b27 => [0x51a8, false], # East Asian ideograph
        0x453421 => [0x5271, false], # East Asian ideograph
        0x213641 => [0x5506, false], # East Asian ideograph
        0x4c4d3d => [0x6f62, false], # East Asian ideograph
        0x2d3642 => [0x6b38, false], # East Asian ideograph
        0x335f49 => [0x9d70, false], # East Asian ideograph
        0x4d5b7e => [0x9dc6, false], # East Asian ideograph
        0x27516f => [0x7f2b, false], # East Asian ideograph
        0x213867 => [0x58be, false], # East Asian ideograph
        0x213644 => [0x5556, false], # East Asian ideograph
        0x213645 => [0x5533, false], # East Asian ideograph
        0x6f4f4e => [0xb959, false], # Korean hangul
        0x275e39 => [0x94d9, false], # East Asian ideograph
        0x6f5449 => [0xc372, false], # Korean hangul
        0x234174 => [0x91f4, false], # East Asian ideograph
        0x213647 => [0x5537, false], # East Asian ideograph (Version J extension)
        0x2d3644 => [0x5557, false], # East Asian ideograph
        0x275170 => [0x7f2e, false], # East Asian ideograph
        0x6f553f => [0xc591, false], # Korean hangul
        0x213649 => [0x555e, false], # East Asian ideograph
        0x276245 => [0x9e4f, false], # East Asian ideograph
        0x275e3a => [0x9570, false], # East Asian ideograph
        0x6f764c => [0xe8be, false], # Korean hangul
        0x21764d => [0x57fd, false], # East Asian ideograph
        0x21764e => [0x57f8, false], # East Asian ideograph
        0x21364f => [0x5531, false], # East Asian ideograph
        0x2d5749 => [0x885e, false], # East Asian ideograph
        0x275e3b => [0x9508, false], # East Asian ideograph
        0x21574e => [0x521d, false], # East Asian ideograph
        0x6f5859 => [0xcac0, false], # Korean hangul
        0x233651 => [0x8cba, false], # East Asian ideograph
        0x233652 => [0x8cb5, false], # East Asian ideograph
        0x213653 => [0x553e, false], # East Asian ideograph
        0x213654 => [0x5563, false], # East Asian ideograph
        0x6f4f51 => [0xb968, false], # Korean hangul
        0x275e3c => [0x956d, false], # East Asian ideograph
        0x234177 => [0x91f1, false], # East Asian ideograph
        0x6f7656 => [0xe8c8, false], # Korean hangul
        0x213657 => [0x552e, false], # East Asian ideograph
        0x6f4a3a => [0xae38, false], # Korean hangul
        0x34682a => [0x7c7c, false], # East Asian ideograph
        0x275e3d => [0x94c1, false], # East Asian ideograph
        0x214621 => [0x6b61, false], # East Asian ideograph
        0x234622 => [0x938c, false], # East Asian ideograph
        0x214623 => [0x6b63, false], # East Asian ideograph
        0x214624 => [0x6b64, false], # East Asian ideograph
        0x214625 => [0x6b65, false], # East Asian ideograph
        0x214627 => [0x6b66, false], # East Asian ideograph
        0x214628 => [0x6b6a, false], # East Asian ideograph
        0x214629 => [0x6b72, false], # East Asian ideograph
        0x22462a => [0x6bf6, false], # East Asian ideograph
        0x21462b => [0x6b78, false], # East Asian ideograph
        0x21462c => [0x6b79, false], # East Asian ideograph
        0x21462d => [0x6b7b, false], # East Asian ideograph
        0x21462e => [0x6b7f, false], # East Asian ideograph
        0x21462f => [0x6b83, false], # East Asian ideograph
        0x214630 => [0x6b86, false], # East Asian ideograph
        0x214631 => [0x6b8a, false], # East Asian ideograph
        0x214632 => [0x6b89, false], # East Asian ideograph
        0x214633 => [0x6b98, false], # East Asian ideograph
        0x214634 => [0x6b96, false], # East Asian ideograph
        0x214635 => [0x6ba4, false], # East Asian ideograph
        0x214636 => [0x6bae, false], # East Asian ideograph
        0x214637 => [0x6baf, false], # East Asian ideograph
        0x214638 => [0x6bb2, false], # East Asian ideograph
        0x214639 => [0x6bb5, false], # East Asian ideograph
        0x21463a => [0x6bb7, false], # East Asian ideograph
        0x21463b => [0x6bba, false], # East Asian ideograph
        0x21463c => [0x6bbc, false], # East Asian ideograph
        0x21463d => [0x6bc0, false], # East Asian ideograph
        0x21463e => [0x6bbf, false], # East Asian ideograph
        0x21463f => [0x6bc5, false], # East Asian ideograph
        0x214640 => [0x6bc6, false], # East Asian ideograph
        0x214641 => [0x6bcb, false], # East Asian ideograph
        0x214642 => [0x6bcd, false], # East Asian ideograph
        0x214643 => [0x6bcf, false], # East Asian ideograph
        0x214644 => [0x6bd2, false], # East Asian ideograph
        0x214646 => [0x6bd4, false], # East Asian ideograph
        0x214647 => [0x6bd7, false], # East Asian ideograph
        0x214648 => [0x6bdb, false], # East Asian ideograph
        0x214649 => [0x6beb, false], # East Asian ideograph
        0x21464a => [0x6bef, false], # East Asian ideograph
        0x21464b => [0x6bfd, false], # East Asian ideograph
        0x21464c => [0x6c0f, false], # East Asian ideograph
        0x21464d => [0x6c11, false], # East Asian ideograph
        0x21464e => [0x6c10, false], # East Asian ideograph
        0x21464f => [0x6c13, false], # East Asian ideograph
        0x214650 => [0x6c16, false], # East Asian ideograph
        0x214651 => [0x6c1b, false], # East Asian ideograph
        0x214652 => [0x6c1f, false], # East Asian ideograph
        0x214653 => [0x6c27, false], # East Asian ideograph
        0x214654 => [0x6c26, false], # East Asian ideograph
        0x214655 => [0x6c23, false], # East Asian ideograph
        0x214656 => [0x6c28, false], # East Asian ideograph
        0x214657 => [0x6c24, false], # East Asian ideograph
        0x214658 => [0x6c2b, false], # East Asian ideograph
        0x214659 => [0x6c2e, false], # East Asian ideograph
        0x21465a => [0x6c33, false], # East Asian ideograph
        0x21465b => [0x6c2f, false], # East Asian ideograph (variant of 45465B which maps to 6C2F)
        0x21465c => [0x6c34, false], # East Asian ideograph
        0x21465d => [0x6c38, false], # East Asian ideograph
        0x21465e => [0x6c41, false], # East Asian ideograph
        0x23465f => [0x93e5, false], # East Asian ideograph
        0x214660 => [0x6c40, false], # East Asian ideograph
        0x214661 => [0x6c42, false], # East Asian ideograph
        0x214662 => [0x6c5e, false], # East Asian ideograph
        0x214663 => [0x6c57, false], # East Asian ideograph
        0x214664 => [0x6c5f, false], # East Asian ideograph
        0x214665 => [0x6c59, false], # East Asian ideograph
        0x214666 => [0x6c60, false], # East Asian ideograph
        0x214667 => [0x6c55, false], # East Asian ideograph
        0x214668 => [0x6c50, false], # East Asian ideograph
        0x214669 => [0x6c5d, false], # East Asian ideograph
        0x21466a => [0x6c9b, false], # East Asian ideograph
        0x21466b => [0x6c81, false], # East Asian ideograph
        0x21466d => [0x6c7a, false], # East Asian ideograph
        0x21466e => [0x6c6a, false], # East Asian ideograph
        0x21466f => [0x6c8c, false], # East Asian ideograph
        0x214670 => [0x6c90, false], # East Asian ideograph
        0x214671 => [0x6c72, false], # East Asian ideograph
        0x214672 => [0x6c70, false], # East Asian ideograph
        0x214673 => [0x6c68, false], # East Asian ideograph
        0x214674 => [0x6c96, false], # East Asian ideograph
        0x234675 => [0x93db, false], # East Asian ideograph
        0x214676 => [0x6c89, false], # East Asian ideograph (variant of 4B4676 which maps to 6C89)
        0x214677 => [0x6c99, false], # East Asian ideograph
        0x214678 => [0x6c7e, false], # East Asian ideograph
        0x214679 => [0x6c7d, false], # East Asian ideograph
        0x21467a => [0x6c92, false], # East Asian ideograph
        0x21467b => [0x6c83, false], # East Asian ideograph
        0x21467c => [0x6cb1, false], # East Asian ideograph
        0x23366a => [0x8ce1, false], # East Asian ideograph
        0x21467e => [0x6cf3, false], # East Asian ideograph
        0x21366b => [0x559d, false], # East Asian ideograph
        0x2d5421 => [0x9ad7, false], # East Asian ideograph
        0x6f4a56 => [0xae7d, false], # Korean hangul
        0x4c4359 => [0x6b05, false], # East Asian ideograph
        0x27366d => [0x5524, false], # East Asian ideograph
        0x21366e => [0x557e, false], # East Asian ideograph
        0x294869 => [0x9567, false], # East Asian ideograph
        0x284027 => [0x6864, false], # East Asian ideograph
        0x21366f => [0x55ac, false], # East Asian ideograph
        0x213670 => [0x5589, false], # East Asian ideograph
        0x223671 => [0x6595, false], # East Asian ideograph
        0x213672 => [0x55bb, false], # East Asian ideograph
        0x27406c => [0x631f, false], # East Asian ideograph
        0x4c3b60 => [0x6764, false], # East Asian ideograph
        0x294228 => [0x94ac, false], # East Asian ideograph
        0x213674 => [0x55df, false], # East Asian ideograph
        0x213675 => [0x55d1, false], # East Asian ideograph
        0x213869 => [0x58d3, false], # East Asian ideograph
        0x28734e => [0x7f32, false], # East Asian ideograph
        0x233676 => [0x8cee, false], # East Asian ideograph
        0x216121 => [0x993f, false], # East Asian ideograph
        0x213677 => [0x55e6, false], # East Asian ideograph
        0x4b6122 => [0x994b, false], # East Asian ideograph
        0x6f4f58 => [0xb985, false], # Korean hangul
        0x273678 => [0x556c, false], # East Asian ideograph
        0x275e43 => [0x94f8, false], # East Asian ideograph
        0x216123 => [0x9945, false], # East Asian ideograph
        0x6f5263 => [0xc0b0, false], # Korean hangul
        0x21353d => [0x53f2, false], # East Asian ideograph
        0x276124 => [0x9976, false], # East Asian ideograph
        0x27367a => [0x5417, false], # East Asian ideograph
        0x2d5424 => [0x5367, false], # East Asian ideograph
        0x23367b => [0x8cf1, false], # East Asian ideograph
        0x216126 => [0x995c, false], # East Asian ideograph
        0x6f5851 => [0xca54, false], # Korean hangul
        0x21367c => [0x55ef, false], # East Asian ideograph
        0x2d475b => [0x51c9, false], # East Asian ideograph
        0x276127 => [0x998b, false], # East Asian ideograph
        0x6f4f59 => [0xb987, false], # Korean hangul
        0x21767d => [0x5844, false], # East Asian ideograph
        0x275e44 => [0x9573, false], # East Asian ideograph
        0x21367e => [0x55c5, false], # East Asian ideograph
        0x396c6b => [0x60a4, false], # East Asian ideograph
        0x6f5b37 => [0xd168, false], # Korean hangul
        0x213e61 => [0x60e1, false], # East Asian ideograph
        0x224a4a => [0x6de6, false], # East Asian ideograph
        0x4b5d34 => [0x91b8, false], # East Asian ideograph
        0x27612c => [0x9a6c, false], # East Asian ideograph
        0x217971 => [0x59a0, false], # East Asian ideograph
        0x21353f => [0x540b, false], # East Asian ideograph
        0x27612e => [0x9a6d, false], # East Asian ideograph
        0x2d6260 => [0x5e85, false], # East Asian ideograph
        0x27612f => [0x9a70, false], # East Asian ideograph
        0x287351 => [0x7f33, false], # East Asian ideograph
        0x214721 => [0x6ce3, false], # East Asian ideograph
        0x214722 => [0x6cf0, false], # East Asian ideograph
        0x214723 => [0x6cb8, false], # East Asian ideograph
        0x214724 => [0x6cd3, false], # East Asian ideograph
        0x214725 => [0x6cab, false], # East Asian ideograph
        0x214726 => [0x6ce5, false], # East Asian ideograph
        0x214727 => [0x6cbd, false], # East Asian ideograph
        0x214728 => [0x6cb3, false], # East Asian ideograph
        0x214729 => [0x6cc4, false], # East Asian ideograph
        0x21472a => [0x6cd5, false], # East Asian ideograph
        0x21472b => [0x6ce2, false], # East Asian ideograph
        0x21472c => [0x6cbc, false], # East Asian ideograph
        0x21472d => [0x6cae, false], # East Asian ideograph
        0x21472e => [0x6cb9, false], # East Asian ideograph
        0x21472f => [0x6cf1, false], # East Asian ideograph
        0x214730 => [0x6cc1, false], # East Asian ideograph
        0x214731 => [0x6cbe, false], # East Asian ideograph
        0x214732 => [0x6cc5, false], # East Asian ideograph
        0x214733 => [0x6cd7, false], # East Asian ideograph
        0x234734 => [0x9413, false], # East Asian ideograph
        0x214735 => [0x6cdb, false], # East Asian ideograph
        0x214736 => [0x6ce1, false], # East Asian ideograph
        0x214737 => [0x6cbf, false], # East Asian ideograph
        0x214738 => [0x6cca, false], # East Asian ideograph
        0x214739 => [0x6ccc, false], # East Asian ideograph
        0x21473a => [0x6cc9, false], # East Asian ideograph
        0x21473b => [0x6d41, false], # East Asian ideograph
        0x21473c => [0x6d0b, false], # East Asian ideograph
        0x21473d => [0x6d32, false], # East Asian ideograph
        0x21473e => [0x6d25, false], # East Asian ideograph
        0x21473f => [0x6d31, false], # East Asian ideograph
        0x214740 => [0x6d2a, false], # East Asian ideograph
        0x214741 => [0x6d0c, false], # East Asian ideograph
        0x214742 => [0x6d1e, false], # East Asian ideograph
        0x214743 => [0x6d17, false], # East Asian ideograph
        0x214744 => [0x6d3b, false], # East Asian ideograph
        0x214745 => [0x6d1b, false], # East Asian ideograph
        0x214746 => [0x6d36, false], # East Asian ideograph
        0x214747 => [0x6d3d, false], # East Asian ideograph
        0x214748 => [0x6d3e, false], # East Asian ideograph
        0x214749 => [0x6d6a, false], # East Asian ideograph
        0x21474a => [0x6d95, false], # East Asian ideograph
        0x21474b => [0x6d78, false], # East Asian ideograph
        0x21474c => [0x6d66, false], # East Asian ideograph
        0x21474d => [0x6d59, false], # East Asian ideograph
        0x21474e => [0x6d87, false], # East Asian ideograph
        0x21474f => [0x6d88, false], # East Asian ideograph
        0x214750 => [0x6d6c, false], # East Asian ideograph
        0x214751 => [0x6d93, false], # East Asian ideograph
        0x214752 => [0x6d89, false], # East Asian ideograph
        0x214753 => [0x6d6e, false], # East Asian ideograph
        0x214754 => [0x6d74, false], # East Asian ideograph
        0x214755 => [0x6d5a, false], # East Asian ideograph
        0x214756 => [0x6d69, false], # East Asian ideograph
        0x214757 => [0x6d77, false], # East Asian ideograph
        0x214758 => [0x6dd9, false], # East Asian ideograph
        0x214759 => [0x6dda, false], # East Asian ideograph
        0x21475a => [0x6df3, false], # East Asian ideograph
        0x21475b => [0x6dbc, false], # East Asian ideograph
        0x21475c => [0x6de4, false], # East Asian ideograph
        0x21475d => [0x6db2, false], # East Asian ideograph
        0x21475e => [0x6de1, false], # East Asian ideograph
        0x21475f => [0x6dd2, false], # East Asian ideograph
        0x214760 => [0x6dae, false], # East Asian ideograph
        0x214761 => [0x6df8, false], # East Asian ideograph
        0x214762 => [0x6dc7, false], # East Asian ideograph
        0x214763 => [0x6dcb, false], # East Asian ideograph
        0x214764 => [0x6dc5, false], # East Asian ideograph
        0x214765 => [0x6dde, false], # East Asian ideograph
        0x214766 => [0x6daf, false], # East Asian ideograph
        0x214767 => [0x6db5, false], # East Asian ideograph
        0x214768 => [0x6dfa, false], # East Asian ideograph
        0x214769 => [0x6df9, false], # East Asian ideograph
        0x21476a => [0x6dcc, false], # East Asian ideograph
        0x21476b => [0x6df7, false], # East Asian ideograph
        0x21476c => [0x6db8, false], # East Asian ideograph
        0x21476d => [0x6dd1, false], # East Asian ideograph
        0x21476e => [0x6df1, false], # East Asian ideograph
        0x21476f => [0x6de8, false], # East Asian ideograph
        0x214770 => [0x6deb, false], # East Asian ideograph
        0x214771 => [0x6dd8, false], # East Asian ideograph
        0x214772 => [0x6dfb, false], # East Asian ideograph
        0x214773 => [0x6dee, false], # East Asian ideograph
        0x214774 => [0x6df5, false], # East Asian ideograph
        0x214775 => [0x6d8e, false], # East Asian ideograph
        0x214776 => [0x6dc6, false], # East Asian ideograph
        0x214777 => [0x6dea, false], # East Asian ideograph
        0x214778 => [0x6dc4, false], # East Asian ideograph
        0x214779 => [0x6e54, false], # East Asian ideograph
        0x21477a => [0x6e21, false], # East Asian ideograph
        0x21477b => [0x6e38, false], # East Asian ideograph
        0x21477c => [0x6e32, false], # East Asian ideograph
        0x21477d => [0x6e67, false], # East Asian ideograph
        0x21477e => [0x6e20, false], # East Asian ideograph
        0x4b5d38 => [0x91c8, false], # East Asian ideograph
        0x226140 => [0x76ec, false], # East Asian ideograph
        0x6f4f5e => [0xb9b0, false], # Korean hangul
        0x6f5936 => [0xcc64, false], # Korean hangul
        0x276141 => [0x9a97, false], # East Asian ideograph
        0x6f5777 => [0xc950, false], # Korean hangul
        0x29442e => [0x9502, false], # East Asian ideograph
        0x276142 => [0x9a9b, false], # East Asian ideograph
        0x276143 => [0x9a9e, false], # East Asian ideograph
        0x274d3d => [0x76d1, false], # East Asian ideograph
        0x6f5c28 => [0xd38d, false], # Korean hangul
        0x216144 => [0x9a30, false], # East Asian ideograph
        0x4b624f => [0x9d49, false], # East Asian ideograph
        0x276145 => [0x9a9a, false], # East Asian ideograph
        0x6f4f5f => [0xb9b4, false], # Korean hangul
        0x275e4a => [0x94bb, false], # East Asian ideograph
        0x273c31 => [0x5ce6, false], # East Asian ideograph
        0x21575d => [0x88c2, false], # East Asian ideograph
        0x6f585c => [0xcacd, false], # Korean hangul
        0x217533 => [0x5788, false], # East Asian ideograph
        0x276147 => [0x9a71, false], # East Asian ideograph
        0x213860 => [0x58b3, false], # East Asian ideograph
        0x216148 => [0x9a40, false], # East Asian ideograph
        0x6f772d => [0xae5f, false], # Korean hangul
        0x287236 => [0x7f07, false], # East Asian ideograph
        0x6f5c29 => [0xd38f, false], # Korean hangul
        0x276149 => [0x9aa1, false], # East Asian ideograph
        0x27614a => [0x9a84, false], # East Asian ideograph
        0x6f4c6d => [0xb2e4, false], # Korean hangul
        0x6f4f60 => [0xb9bc, false], # Korean hangul
        0x22614b => [0x7704, false], # East Asian ideograph
        0x225b5d => [0x74a1, false], # East Asian ideograph
        0x27614c => [0x9a7f, false], # East Asian ideograph
        0x27614d => [0x9a8c, false], # East Asian ideograph
        0x27614e => [0x9aa4, false], # East Asian ideograph
        0x21614f => [0x9a62, false], # East Asian ideograph
        0x6f4f61 => [0xb9bd, false], # Korean hangul
        0x275e4c => [0x957f, false], # East Asian ideograph
        0x216150 => [0x9a65, false], # East Asian ideograph
        0x21575f => [0x88df, false], # East Asian ideograph
        0x216151 => [0x9a6a, false], # East Asian ideograph
        0x226153 => [0x76f7, false], # East Asian ideograph
        0x293325 => [0x8bf9, false], # East Asian ideograph
        0x216154 => [0x9ab0, false], # East Asian ideograph
        0x6f4f62 => [0xb9bf, false], # Korean hangul
        0x235f5e => [0x9f39, false], # East Asian ideograph
        0x213f3d => [0x61a7, false], # East Asian ideograph
        0x216157 => [0x9abc, false], # East Asian ideograph
        0x287359 => [0x7f31, false], # East Asian ideograph
        0x276158 => [0x9ac5, false], # East Asian ideograph
        0x216159 => [0x9ad3, false], # East Asian ideograph
        0x6f4f63 => [0xb9c1, false], # Korean hangul
        0x275e4e => [0x95e9, false], # East Asian ideograph
        0x27615a => [0x4f53, false], # East Asian ideograph
        0x277c24 => [0x5a32, false], # East Asian ideograph
        0x456036 => [0x97ff, false], # East Asian ideograph
        0x214821 => [0x6e5b, false], # East Asian ideograph
        0x214822 => [0x6e1a, false], # East Asian ideograph
        0x214823 => [0x6e56, false], # East Asian ideograph
        0x214824 => [0x6e2f, false], # East Asian ideograph
        0x214825 => [0x6e6e, false], # East Asian ideograph
        0x214826 => [0x6e58, false], # East Asian ideograph
        0x214827 => [0x6e23, false], # East Asian ideograph
        0x214828 => [0x6e24, false], # East Asian ideograph
        0x214829 => [0x6e1b, false], # East Asian ideograph
        0x21482a => [0x6e25, false], # East Asian ideograph
        0x21482b => [0x6e4a, false], # East Asian ideograph
        0x21482c => [0x6e3a, false], # East Asian ideograph
        0x21482d => [0x6e6f, false], # East Asian ideograph
        0x21482e => [0x6e2d, false], # East Asian ideograph
        0x22482f => [0x6ce0, false], # East Asian ideograph
        0x214830 => [0x6e2c, false], # East Asian ideograph
        0x214831 => [0x6e26, false], # East Asian ideograph
        0x214832 => [0x6e4d, false], # East Asian ideograph
        0x214833 => [0x6e3e, false], # East Asian ideograph
        0x214834 => [0x6e43, false], # East Asian ideograph
        0x214835 => [0x6e19, false], # East Asian ideograph
        0x214836 => [0x6e1d, false], # East Asian ideograph
        0x214837 => [0x6ed3, false], # East Asian ideograph
        0x214838 => [0x6eb6, false], # East Asian ideograph
        0x214839 => [0x6ec2, false], # East Asian ideograph
        0x21483b => [0x6eaf, false], # East Asian ideograph
        0x21483c => [0x6ea2, false], # East Asian ideograph
        0x27483d => [0x6c9f, false], # East Asian ideograph
        0x23483e => [0x944c, false], # East Asian ideograph
        0x21483f => [0x6ea5, false], # East Asian ideograph
        0x214840 => [0x6e98, false], # East Asian ideograph
        0x214841 => [0x6e90, false], # East Asian ideograph
        0x214842 => [0x6ec5, false], # East Asian ideograph
        0x214843 => [0x6ec7, false], # East Asian ideograph
        0x214844 => [0x6ebc, false], # East Asian ideograph
        0x214845 => [0x6eab, false], # East Asian ideograph
        0x214846 => [0x6ed1, false], # East Asian ideograph
        0x214847 => [0x6ecb, false], # East Asian ideograph
        0x214848 => [0x6ec4, false], # East Asian ideograph
        0x214849 => [0x6ed4, false], # East Asian ideograph
        0x21484a => [0x6eaa, false], # East Asian ideograph
        0x21484b => [0x6e96, false], # East Asian ideograph
        0x21484c => [0x6e9c, false], # East Asian ideograph
        0x21484d => [0x6f33, false], # East Asian ideograph
        0x21484e => [0x6ef4, false], # East Asian ideograph
        0x21484f => [0x6eec, false], # East Asian ideograph
        0x214850 => [0x6efe, false], # East Asian ideograph
        0x214851 => [0x6f29, false], # East Asian ideograph
        0x214852 => [0x6f14, false], # East Asian ideograph
        0x214853 => [0x6f3e, false], # East Asian ideograph
        0x214854 => [0x6f2c, false], # East Asian ideograph
        0x214855 => [0x6f32, false], # East Asian ideograph
        0x214856 => [0x6f0f, false], # East Asian ideograph
        0x214857 => [0x6f22, false], # East Asian ideograph (variant of 4B4857 which maps to 6F22)
        0x214858 => [0x6eff, false], # East Asian ideograph
        0x214859 => [0x6f23, false], # East Asian ideograph
        0x21485a => [0x6f38, false], # East Asian ideograph
        0x21485b => [0x6f15, false], # East Asian ideograph
        0x21485c => [0x6f31, false], # East Asian ideograph
        0x21485d => [0x6f02, false], # East Asian ideograph
        0x21485e => [0x6f06, false], # East Asian ideograph
        0x21485f => [0x6eef, false], # East Asian ideograph
        0x214860 => [0x6f2b, false], # East Asian ideograph
        0x214861 => [0x6f2f, false], # East Asian ideograph
        0x214862 => [0x6f20, false], # East Asian ideograph
        0x214863 => [0x6f3f, false], # East Asian ideograph
        0x214864 => [0x6ef2, false], # East Asian ideograph
        0x214865 => [0x6f01, false], # East Asian ideograph
        0x214866 => [0x6f11, false], # East Asian ideograph
        0x214867 => [0x6ecc, false], # East Asian ideograph
        0x214868 => [0x6f2a, false], # East Asian ideograph
        0x214869 => [0x6f7c, false], # East Asian ideograph
        0x21486a => [0x6f88, false], # East Asian ideograph
        0x21486b => [0x6f84, false], # East Asian ideograph
        0x21486c => [0x6f51, false], # East Asian ideograph
        0x21486d => [0x6f64, false], # East Asian ideograph
        0x21486e => [0x6f97, false], # East Asian ideograph
        0x21486f => [0x6f54, false], # East Asian ideograph
        0x214870 => [0x6f7a, false], # East Asian ideograph
        0x214871 => [0x6f86, false], # East Asian ideograph
        0x214872 => [0x6f8e, false], # East Asian ideograph
        0x214873 => [0x6f6d, false], # East Asian ideograph
        0x214874 => [0x6f5b, false], # East Asian ideograph
        0x214875 => [0x6f6e, false], # East Asian ideograph
        0x214876 => [0x6f78, false], # East Asian ideograph
        0x214877 => [0x6f66, false], # East Asian ideograph
        0x214878 => [0x6f70, false], # East Asian ideograph
        0x214879 => [0x6f58, false], # East Asian ideograph
        0x21487a => [0x6fc2, false], # East Asian ideograph
        0x21487b => [0x6fb1, false], # East Asian ideograph
        0x21487c => [0x6fc3, false], # East Asian ideograph
        0x21487d => [0x6fa7, false], # East Asian ideograph
        0x21487e => [0x6fa1, false], # East Asian ideograph
        0x4d5875 => [0x9cd0, false], # East Asian ideograph
        0x2d5d65 => [0x8216, false], # East Asian ideograph
        0x28735d => [0x7ea9, false], # East Asian ideograph
        0x6f5c30 => [0xd3a8, false], # Korean hangul
        0x22616c => [0x7722, false], # East Asian ideograph
        0x22616d => [0x771a, false], # East Asian ideograph
        0x6f4f67 => [0xb9c9, false], # Korean hangul
        0x6f4b24 => [0xafbc, false], # Korean hangul
        0x21616f => [0x9b45, false], # East Asian ideograph
        0x28336f => [0x629f, false], # East Asian ideograph
        0x6f5c31 => [0xd3a9, false], # Korean hangul
        0x6f245e => [0x3147, false], # Korean hangul
        0x4b5d42 => [0x91e1, false], # East Asian ideograph
        0x27407d => [0x626b, false], # East Asian ideograph
        0x6f5029 => [0xba4e, false], # Korean hangul
        0x2d4327 => [0x6630, false], # East Asian ideograph
        0x275e53 => [0x5f00, false], # East Asian ideograph
        0x276173 => [0x9b47, false], # East Asian ideograph
        0x3f3573 => [0x8b3c, false], # East Asian ideograph
        0x226174 => [0x7740, false], # East Asian ideograph
        0x6f4e41 => [0xb610, false], # Korean hangul
        0x4b4c36 => [0x7575, false], # East Asian ideograph
        0x216175 => [0x9b77, false], # East Asian ideograph
        0x224b34 => [0x6e53, false], # East Asian ideograph
        0x216176 => [0x9b6f, false], # East Asian ideograph
        0x214c21 => [0x752c, false], # East Asian ideograph
        0x226177 => [0x7731, false], # East Asian ideograph
        0x214c22 => [0x752b, false], # East Asian ideograph
        0x6f4f69 => [0xb9ce, false], # Korean hangul
        0x276178 => [0x9c9b, false], # East Asian ideograph
        0x6f4b26 => [0xafc7, false], # Korean hangul
        0x276179 => [0x9c9c, false], # East Asian ideograph
        0x295269 => [0x9a80, false], # East Asian ideograph
        0x214c24 => [0x7530, false], # East Asian ideograph
        0x27617a => [0x9c94, false], # East Asian ideograph
        0x2f3143 => [0x89f5, false], # Unrelated variant of EACC 23315E which maps to 89F5
        0x213a6b => [0x5b8f, false], # East Asian ideograph
        0x27617b => [0x9ca8, false], # East Asian ideograph
        0x214c26 => [0x7531, false], # East Asian ideograph
        0x27617c => [0x9ca4, false], # East Asian ideograph
        0x214c27 => [0x7533, false], # East Asian ideograph
        0x6f4f6a => [0xb9cf, false], # Korean hangul
        0x275e55 => [0x95f4, false], # East Asian ideograph
        0x27617d => [0x9cb8, false], # East Asian ideograph
        0x6f4b27 => [0xafc8, false], # Korean hangul
        0x29593b => [0x9c9f, false], # East Asian ideograph
        0x27617e => [0x9cb3, false], # East Asian ideograph
        0x214c29 => [0x7538, false], # East Asian ideograph
        0x215b60 => [0x8fad, false], # East Asian ideograph
        0x214c2a => [0x753d, false], # East Asian ideograph
        0x294237 => [0x949b, false], # East Asian ideograph
        0x6f5c34 => [0xd3b4, false], # Korean hangul
        0x39553c => [0x5d0b, false], # East Asian ideograph
        0x6f5360 => [0xc228, false], # Korean hangul
        0x6f4c2b => [0xb153, false], # Korean hangul
        0x2d4c2c => [0x583a, false], # East Asian ideograph
        0x6f4f6b => [0xb9d0, false], # Korean hangul
        0x274c2d => [0x4ea9, false], # East Asian ideograph
        0x214c2e => [0x755c, false], # East Asian ideograph
        0x2d3661 => [0x6199, false], # East Asian ideograph
        0x6f4c2f => [0xb15c, false], # Korean hangul
        0x214921 => [0x6fa4, false], # East Asian ideograph
        0x214922 => [0x6fc1, false], # East Asian ideograph
        0x214924 => [0x6fc0, false], # East Asian ideograph
        0x214925 => [0x6fb3, false], # East Asian ideograph
        0x214926 => [0x6fdf, false], # East Asian ideograph
        0x214927 => [0x6fd8, false], # East Asian ideograph
        0x214928 => [0x6ff1, false], # East Asian ideograph
        0x214929 => [0x6fe0, false], # East Asian ideograph
        0x21492a => [0x6fef, false], # East Asian ideograph
        0x21492b => [0x6feb, false], # East Asian ideograph (variant of 4B492B which maps to 6FEB)
        0x21492c => [0x6fe1, false], # East Asian ideograph
        0x21492d => [0x6fe4, false], # East Asian ideograph
        0x21492e => [0x6f80, false], # East Asian ideograph
        0x22492f => [0x6d34, false], # East Asian ideograph (variant of 34492F which maps to 6D34)
        0x234930 => [0x9588, false], # East Asian ideograph
        0x214931 => [0x700b, false], # East Asian ideograph
        0x214932 => [0x7009, false], # East Asian ideograph
        0x214933 => [0x7006, false], # East Asian ideograph
        0x214934 => [0x6ffa, false], # East Asian ideograph
        0x214935 => [0x7011, false], # East Asian ideograph
        0x214936 => [0x6ffe, false], # East Asian ideograph
        0x214937 => [0x700f, false], # East Asian ideograph
        0x234938 => [0x959f, false], # East Asian ideograph
        0x214939 => [0x701a, false], # East Asian ideograph
        0x23493a => [0x95a0, false], # East Asian ideograph
        0x21493b => [0x701d, false], # East Asian ideograph
        0x22493c => [0x6d65, false], # East Asian ideograph
        0x21493d => [0x701f, false], # East Asian ideograph
        0x22493e => [0x6d5e, false], # East Asian ideograph
        0x21493f => [0x703e, false], # East Asian ideograph
        0x214940 => [0x704c, false], # East Asian ideograph
        0x214941 => [0x7051, false], # East Asian ideograph
        0x214942 => [0x7058, false], # East Asian ideograph
        0x274943 => [0x6e7e, false], # East Asian ideograph
        0x214944 => [0x7064, false], # East Asian ideograph
        0x214945 => [0x706b, false], # East Asian ideograph
        0x214946 => [0x7070, false], # East Asian ideograph
        0x214947 => [0x7076, false], # East Asian ideograph
        0x214948 => [0x707c, false], # East Asian ideograph
        0x214949 => [0x7078, false], # East Asian ideograph
        0x21494a => [0x707d, false], # East Asian ideograph
        0x21494b => [0x7095, false], # East Asian ideograph
        0x21494c => [0x708e, false], # East Asian ideograph
        0x23494d => [0x95b9, false], # East Asian ideograph
        0x21494e => [0x7099, false], # East Asian ideograph
        0x21494f => [0x708a, false], # East Asian ideograph
        0x214950 => [0x70ab, false], # East Asian ideograph
        0x214951 => [0x70ba, false], # East Asian ideograph
        0x214952 => [0x70ac, false], # East Asian ideograph
        0x214953 => [0x70b3, false], # East Asian ideograph
        0x214954 => [0x70af, false], # East Asian ideograph
        0x214955 => [0x70ad, false], # East Asian ideograph
        0x214956 => [0x70ae, false], # East Asian ideograph
        0x214957 => [0x70b8, false], # East Asian ideograph
        0x214958 => [0x70ca, false], # East Asian ideograph
        0x214959 => [0x70e4, false], # East Asian ideograph
        0x21495a => [0x70d8, false], # East Asian ideograph
        0x21495b => [0x70c8, false], # East Asian ideograph
        0x21495c => [0x70d9, false], # East Asian ideograph
        0x23495d => [0x95ce, false], # East Asian ideograph
        0x21495e => [0x70f9, false], # East Asian ideograph
        0x21495f => [0x7109, false], # East Asian ideograph
        0x214960 => [0x710a, false], # East Asian ideograph
        0x214961 => [0x70fd, false], # East Asian ideograph
        0x214962 => [0x7119, false], # East Asian ideograph
        0x214963 => [0x716e, false], # East Asian ideograph
        0x214964 => [0x711a, false], # East Asian ideograph
        0x214965 => [0x7136, false], # East Asian ideograph
        0x214966 => [0x7121, false], # East Asian ideograph
        0x214967 => [0x7130, false], # East Asian ideograph
        0x214968 => [0x7126, false], # East Asian ideograph
        0x214969 => [0x714e, false], # East Asian ideograph
        0x21496a => [0x7149, false], # East Asian ideograph
        0x21496b => [0x7159, false], # East Asian ideograph
        0x21496c => [0x7164, false], # East Asian ideograph
        0x21496d => [0x7169, false], # East Asian ideograph
        0x21496e => [0x715c, false], # East Asian ideograph
        0x21496f => [0x716c, false], # East Asian ideograph
        0x214970 => [0x7166, false], # East Asian ideograph
        0x214971 => [0x7167, false], # East Asian ideograph
        0x214972 => [0x715e, false], # East Asian ideograph
        0x214973 => [0x7165, false], # East Asian ideograph
        0x214974 => [0x714c, false], # East Asian ideograph
        0x214975 => [0x717d, false], # East Asian ideograph
        0x234976 => [0x95e7, false], # East Asian ideograph
        0x214977 => [0x7199, false], # East Asian ideograph
        0x214978 => [0x718a, false], # East Asian ideograph
        0x214979 => [0x7184, false], # East Asian ideograph
        0x21497a => [0x719f, false], # East Asian ideograph
        0x21497b => [0x71a8, false], # East Asian ideograph
        0x21497c => [0x71ac, false], # East Asian ideograph
        0x21497d => [0x71b1, false], # East Asian ideograph
        0x21497e => [0x71d9, false], # East Asian ideograph
        0x6f4c40 => [0xb1dc, false], # Korean hangul
        0x2d432e => [0x66ec, false], # East Asian ideograph
        0x214c41 => [0x7599, false], # East Asian ideograph
        0x395e71 => [0x5742, false], # East Asian ideograph
        0x214c42 => [0x759a, false], # East Asian ideograph
        0x6f586e => [0xcb64, false], # Korean hangul
        0x214c43 => [0x75a4, false], # East Asian ideograph
        0x6f5c39 => [0xd3c5, false], # Korean hangul
        0x224c44 => [0x6f00, false], # East Asian ideograph
        0x4b3b31 => [0x5b9f, false], # East Asian ideograph
        0x6f4c45 => [0xb208, false], # Korean hangul
        0x6f4f70 => [0xb9dd, false], # Korean hangul
        0x275e5b => [0x9601, false], # East Asian ideograph
        0x6f4b2d => [0xafd8, false], # Korean hangul
        0x294440 => [0x9506, false], # East Asian ideograph
        0x333475 => [0x9628, false], # East Asian ideograph
        0x234c47 => [0x9723, false], # East Asian ideograph
        0x27727e => [0x54d9, false], # East Asian ideograph
        0x21386e => [0x58de, false], # East Asian ideograph
        0x6f4c48 => [0xb213, false], # Korean hangul
        0x6f5c3a => [0xd3c8, false], # Korean hangul
        0x214c49 => [0x75b2, false], # East Asian ideograph
        0x234e60 => [0x97e1, false], # East Asian ideograph
        0x214c4a => [0x75bd, false], # East Asian ideograph
        0x6f4f71 => [0xb9de, false], # Korean hangul
        0x275e5c => [0x9600, false], # East Asian ideograph
        0x6f4877 => [0xac2d, false], # Korean hangul
        0x214c4b => [0x75be, false], # East Asian ideograph
        0x294441 => [0x9507, false], # East Asian ideograph
        0x333d54 => [0x4efd, false], # East Asian ideograph
        0x695c71 => [0x6a78, false], # East Asian ideograph
        0x276f69 => [0x5459, false], # East Asian ideograph
        0x6f5c3b => [0xd3c9, false], # Korean hangul
        0x70603a => [0x55ea, false], # East Asian ideograph
        0x69554e => [0x5b36, false], # East Asian ideograph
        0x214c4e => [0x75d5, false], # East Asian ideograph
        0x6f5852 => [0xca5c, false], # Korean hangul
        0x6f2460 => [0x314a, false], # Korean hangul
        0x6f4c4f => [0xb258, false], # Korean hangul
        0x6f4f72 => [0xb9e1, false], # Korean hangul
        0x275e5d => [0x5408, false], # East Asian ideograph
        0x214c50 => [0x75b5, false], # East Asian ideograph
        0x214c51 => [0x75ca, false], # East Asian ideograph (variant of 4B4C51 which maps to 75CA)
        0x2e3f2d => [0x69b2, false], # East Asian ideograph
        0x2f2a73 => [0x87ca, false], # East Asian ideograph
        0x214c52 => [0x75db, false], # East Asian ideograph
        0x6f5c3c => [0xd3d0, false], # Korean hangul
        0x285150 => [0x70c3, false], # East Asian ideograph
        0x213e66 => [0x60b2, false], # East Asian ideograph
        0x293336 => [0x8be4, false], # East Asian ideograph
        0x6f4c54 => [0xb274, false], # Korean hangul
        0x6f4f73 => [0xb9e3, false], # Korean hangul
        0x275e5e => [0x9605, false], # East Asian ideograph
        0x2e4731 => [0x6c73, false], # East Asian ideograph
        0x6f4b30 => [0xb000, false], # Korean hangul
        0x214c56 => [0x75d9, false], # East Asian ideograph
        0x214c57 => [0x75e2, false], # East Asian ideograph
        0x6f5c3d => [0xd3ec, false], # Korean hangul
        0x234c58 => [0x9730, false], # East Asian ideograph
        0x6f4c59 => [0xb294, false], # Korean hangul
        0x275e5f => [0x95fe, false], # East Asian ideograph
        0x214c5a => [0x75f0, false], # East Asian ideograph
        0x394444 => [0x8988, false], # East Asian ideograph
        0x214a21 => [0x71be, false], # East Asian ideograph
        0x214a22 => [0x71c9, false], # East Asian ideograph
        0x214a23 => [0x71d0, false], # East Asian ideograph
        0x214a24 => [0x71c8, false], # East Asian ideograph
        0x214a25 => [0x71dc, false], # East Asian ideograph
        0x214a26 => [0x71d2, false], # East Asian ideograph
        0x214a27 => [0x71b9, false], # East Asian ideograph
        0x214a28 => [0x71d5, false], # East Asian ideograph
        0x214a29 => [0x71ce, false], # East Asian ideograph
        0x214a2a => [0x71c3, false], # East Asian ideograph
        0x214a2b => [0x71c4, false], # East Asian ideograph
        0x214a2c => [0x71ee, false], # East Asian ideograph
        0x214a2d => [0x71e7, false], # East Asian ideograph
        0x214a2e => [0x71df, false], # East Asian ideograph
        0x214a2f => [0x71e5, false], # East Asian ideograph
        0x214a30 => [0x71ed, false], # East Asian ideograph
        0x214a31 => [0x71e6, false], # East Asian ideograph
        0x234a32 => [0x963c, false], # East Asian ideograph
        0x214a33 => [0x71f4, false], # East Asian ideograph
        0x214a34 => [0x71fb, false], # East Asian ideograph
        0x224a35 => [0x6ddd, false], # East Asian ideograph
        0x274a36 => [0x70c1, false], # East Asian ideograph
        0x214a37 => [0x7210, false], # East Asian ideograph
        0x214a38 => [0x721b, false], # East Asian ideograph
        0x224a39 => [0x6ddb, false], # East Asian ideograph
        0x214a3a => [0x722a, false], # East Asian ideograph
        0x214a3b => [0x722d, false], # East Asian ideograph
        0x214a3c => [0x722c, false], # East Asian ideograph
        0x214a3d => [0x7230, false], # East Asian ideograph
        0x214a3e => [0x7235, false], # East Asian ideograph (variant of 4B4A3E which maps to 7235)
        0x214a3f => [0x7236, false], # East Asian ideograph
        0x214a40 => [0x7238, false], # East Asian ideograph
        0x214a41 => [0x7239, false], # East Asian ideograph
        0x214a42 => [0x723a, false], # East Asian ideograph
        0x214a43 => [0x723b, false], # East Asian ideograph
        0x214a44 => [0x723d, false], # East Asian ideograph
        0x214a45 => [0x723e, false], # East Asian ideograph
        0x224a46 => [0x6df0, false], # East Asian ideograph
        0x214a47 => [0x7247, false], # East Asian ideograph
        0x214a48 => [0x7248, false], # East Asian ideograph
        0x214a49 => [0x724c, false], # East Asian ideograph
        0x214a4a => [0x7252, false], # East Asian ideograph
        0x214a4b => [0x7256, false], # East Asian ideograph
        0x214a4c => [0x7258, false], # East Asian ideograph
        0x214a4d => [0x7259, false], # East Asian ideograph
        0x214a4e => [0x725b, false], # East Asian ideograph
        0x214a4f => [0x725f, false], # East Asian ideograph
        0x214a50 => [0x725d, false], # East Asian ideograph
        0x214a51 => [0x7262, false], # East Asian ideograph
        0x214a52 => [0x7261, false], # East Asian ideograph
        0x214a53 => [0x7260, false], # East Asian ideograph
        0x214a54 => [0x7267, false], # East Asian ideograph
        0x214a55 => [0x7269, false], # East Asian ideograph
        0x214a56 => [0x726f, false], # East Asian ideograph
        0x214a57 => [0x7272, false], # East Asian ideograph
        0x214a58 => [0x7274, false], # East Asian ideograph
        0x214a59 => [0x7279, false], # East Asian ideograph
        0x214a5a => [0x727d, false], # East Asian ideograph
        0x214a5b => [0x7281, false], # East Asian ideograph
        0x214a5c => [0x7280, false], # East Asian ideograph
        0x214a5d => [0x7284, false], # East Asian ideograph
        0x274a5e => [0x8366, false], # East Asian ideograph
        0x214a5f => [0x7292, false], # East Asian ideograph
        0x224a60 => [0x6e8a, false], # East Asian ideograph
        0x214a61 => [0x72a2, false], # East Asian ideograph
        0x274a62 => [0x727a, false], # East Asian ideograph
        0x214a63 => [0x72ac, false], # East Asian ideograph
        0x214a64 => [0x72af, false], # East Asian ideograph
        0x214a65 => [0x72c4, false], # East Asian ideograph
        0x214a66 => [0x72c2, false], # East Asian ideograph
        0x214a67 => [0x72d9, false], # East Asian ideograph
        0x274a68 => [0x72b6, false], # East Asian ideograph
        0x214a69 => [0x72ce, false], # East Asian ideograph
        0x214a6a => [0x72d7, false], # East Asian ideograph
        0x214a6b => [0x72d0, false], # East Asian ideograph
        0x214a6c => [0x72e1, false], # East Asian ideograph
        0x214a6d => [0x72e9, false], # East Asian ideograph
        0x214a6e => [0x72e0, false], # East Asian ideograph
        0x214a6f => [0x72fc, false], # East Asian ideograph
        0x274a70 => [0x72ed, false], # East Asian ideograph
        0x224a71 => [0x6e73, false], # East Asian ideograph
        0x214a72 => [0x72fd, false], # East Asian ideograph
        0x214a73 => [0x72f7, false], # East Asian ideograph
        0x214a74 => [0x731c, false], # East Asian ideograph
        0x214a75 => [0x731b, false], # East Asian ideograph
        0x214a76 => [0x7313, false], # East Asian ideograph
        0x214a77 => [0x7316, false], # East Asian ideograph
        0x214a78 => [0x7319, false], # East Asian ideograph
        0x214a79 => [0x7336, false], # East Asian ideograph
        0x214a7a => [0x7337, false], # East Asian ideograph
        0x214a7b => [0x7329, false], # East Asian ideograph
        0x214a7c => [0x7325, false], # East Asian ideograph
        0x214a7d => [0x7334, false], # East Asian ideograph
        0x214a7e => [0x7344, false], # East Asian ideograph
        0x2d4c3c => [0x53e0, false], # East Asian ideograph
        0x214c6b => [0x7634, false], # East Asian ideograph
        0x6f5c41 => [0xd3fc, false], # Korean hangul
        0x213d4a => [0x5f46, false], # East Asian ideograph
        0x214c6c => [0x7638, false], # East Asian ideograph
        0x214c6d => [0x7646, false], # East Asian ideograph
        0x6f4f78 => [0xb9f4, false], # Korean hangul
        0x275e63 => [0x9611, false], # East Asian ideograph
        0x234c6e => [0x9749, false], # East Asian ideograph
        0x233d5b => [0x9046, false], # East Asian ideograph
        0x6f4c6f => [0xb2e6, false], # Korean hangul
        0x6f4c70 => [0xb2e8, false], # Korean hangul
        0x214c71 => [0x7658, false], # East Asian ideograph
        0x4d477b => [0x943e, false], # East Asian ideograph
        0x35344d => [0x8b5b, false], # East Asian ideograph
        0x6f4f79 => [0xb9f5, false], # Korean hangul
        0x275e64 => [0x95f1, false], # East Asian ideograph
        0x274c73 => [0x75d2, false], # East Asian ideograph
        0x21355e => [0x542e, false], # East Asian ideograph
        0x275a21 => [0x8d45, false], # East Asian ideograph
        0x6f4c74 => [0xb2ee, false], # Korean hangul
        0x234147 => [0x91ad, false], # East Asian ideograph
        0x217d7c => [0x5b56, false], # East Asian ideograph
        0x214c75 => [0x7669, false], # East Asian ideograph
        0x6f5c43 => [0xd3ff, false], # Korean hangul
        0x234c76 => [0x975a, false], # East Asian ideograph
        0x233721 => [0x8cf7, false], # East Asian ideograph
        0x287161 => [0x7efb, false], # East Asian ideograph
        0x3a787d => [0x80fc, false], # East Asian ideograph
        0x214c77 => [0x766c, false], # East Asian ideograph
        0x273722 => [0x545b, false], # East Asian ideograph
        0x275e65 => [0x677f, false], # East Asian ideograph
        0x214c78 => [0x7671, false], # East Asian ideograph
        0x213723 => [0x55e1, false], # East Asian ideograph
        0x21754e => [0x579d, false], # East Asian ideograph
        0x214c79 => [0x7672, false], # East Asian ideograph (variant of 4B4C79 which maps to 7672)
        0x694c7a => [0x9453, false], # East Asian ideograph
        0x213725 => [0x561b, false], # East Asian ideograph
        0x6f5c44 => [0xd401, false], # Korean hangul
        0x214c7b => [0x767c, false], # East Asian ideograph
        0x233726 => [0x8cfe, false], # East Asian ideograph
        0x6f4c7c => [0xb2ff, false], # Korean hangul
        0x223727 => [0x65ae, false], # East Asian ideograph
        0x2e4739 => [0x6c67, false], # East Asian ideograph (variant of 224739 which maps to 6C67)
        0x214c7d => [0x767d, false], # East Asian ideograph
        0x6f7728 => [0xae07, false], # Korean hangul
        0x215336 => [0x80b4, false], # East Asian ideograph
        0x2d4c7e => [0x4f70, false], # East Asian ideograph
        0x217729 => [0x582d, false], # East Asian ideograph
        0x2d5447 => [0x824a, false], # East Asian ideograph
        0x21372a => [0x561f, false], # East Asian ideograph
        0x6f5c45 => [0xd440, false], # Korean hangul
        0x2d3a47 => [0x5acb, false], # East Asian ideograph
        0x4b4358 => [0x66fd, false], # East Asian ideograph
        0x23372b => [0x8d07, false], # East Asian ideograph
        0x217850 => [0x58d6, false], # East Asian ideograph
        0x334a28 => [0x91bc, false], # East Asian ideograph
        0x27372c => [0x53f9, false], # East Asian ideograph
        0x6f593c => [0xcca0, false], # Korean hangul
        0x275e67 => [0x95ef, false], # East Asian ideograph
        0x6f4b39 => [0xb044, false], # Korean hangul
        0x275a24 => [0x8d3e, false], # East Asian ideograph
        0x27372e => [0x5455, false], # East Asian ideograph
        0x21372f => [0x560e, false], # East Asian ideograph
        0x6f5c46 => [0xd444, false], # Korean hangul
        0x224a6d => [0x6e63, false], # East Asian ideograph
        0x293340 => [0x8c02, false], # East Asian ideograph
        0x214b21 => [0x733f, false], # East Asian ideograph
        0x224b22 => [0x6e28, false], # East Asian ideograph
        0x274b23 => [0x72ee, false], # East Asian ideograph
        0x214b24 => [0x7350, false], # East Asian ideograph
        0x6f4b25 => [0xafc0, false], # Korean hangul
        0x214b26 => [0x7357, false], # East Asian ideograph
        0x274b27 => [0x72ec, false], # East Asian ideograph
        0x224b28 => [0x6e5e, false], # East Asian ideograph
        0x274b29 => [0x83b7, false], # East Asian ideograph
        0x274b2a => [0x72b7, false], # East Asian ideograph
        0x274b2b => [0x517d, false], # East Asian ideograph
        0x224b2c => [0x6e84, false], # East Asian ideograph
        0x223732 => [0x65c3, false], # East Asian ideograph
        0x224b2e => [0x6e2e, false], # East Asian ideograph
        0x234b2f => [0x96a4, false], # East Asian ideograph
        0x214b30 => [0x7384, false], # East Asian ideograph
        0x214b31 => [0x7387, false], # East Asian ideograph
        0x214b32 => [0x7389, false], # East Asian ideograph
        0x223733 => [0x65c4, false], # East Asian ideograph
        0x214b34 => [0x7396, false], # East Asian ideograph
        0x214b35 => [0x739f, false], # East Asian ideograph
        0x214b36 => [0x73a8, false], # East Asian ideograph
        0x214b37 => [0x73a9, false], # East Asian ideograph
        0x214b38 => [0x73ab, false], # East Asian ideograph
        0x214b39 => [0x73bb, false], # East Asian ideograph
        0x214b3a => [0x73ca, false], # East Asian ideograph
        0x214b3b => [0x73b7, false], # East Asian ideograph
        0x214b3c => [0x73c0, false], # East Asian ideograph
        0x6f4b3d => [0xb04c, false], # Korean hangul
        0x214b3e => [0x73b2, false], # East Asian ideograph
        0x214b3f => [0x73cd, false], # East Asian ideograph
        0x224b40 => [0x6e2a, false], # East Asian ideograph
        0x224b41 => [0x6e4c, false], # East Asian ideograph
        0x224b42 => [0x6e22, false], # East Asian ideograph
        0x224b43 => [0x6ece, false], # East Asian ideograph
        0x214b44 => [0x7409, false], # East Asian ideograph
        0x224b45 => [0x6e9b, false], # East Asian ideograph
        0x224b46 => [0x6e9f, false], # East Asian ideograph
        0x214b47 => [0x73fe, false], # East Asian ideograph
        0x224b48 => [0x6ec8, false], # East Asian ideograph
        0x224b49 => [0x6ed8, false], # East Asian ideograph
        0x224b4a => [0x6e8f, false], # East Asian ideograph
        0x214b4b => [0x7435, false], # East Asian ideograph
        0x214b4c => [0x7436, false], # East Asian ideograph
        0x224b4d => [0x6e93, false], # East Asian ideograph
        0x214b4e => [0x742a, false], # East Asian ideograph
        0x224b4f => [0x6ea0, false], # East Asian ideograph
        0x214b50 => [0x7422, false], # East Asian ideograph
        0x224b51 => [0x6eb1, false], # East Asian ideograph
        0x234b52 => [0x96ce, false], # East Asian ideograph
        0x214b53 => [0x7455, false], # East Asian ideograph
        0x214b54 => [0x745f, false], # East Asian ideograph
        0x214b55 => [0x745a, false], # East Asian ideograph
        0x214b56 => [0x7441, false], # East Asian ideograph
        0x214b57 => [0x743f, false], # East Asian ideograph
        0x214b58 => [0x745b, false], # East Asian ideograph
        0x224b59 => [0x6e92, false], # East Asian ideograph
        0x224b5a => [0x6ea7, false], # East Asian ideograph
        0x214b5b => [0x7459, false], # East Asian ideograph
        0x214b5c => [0x7483, false], # East Asian ideograph
        0x214b5d => [0x7469, false], # East Asian ideograph
        0x274b5e => [0x739b, false], # East Asian ideograph
        0x214b5f => [0x7463, false], # East Asian ideograph
        0x214b60 => [0x7464, false], # East Asian ideograph
        0x214b61 => [0x7470, false], # East Asian ideograph
        0x214b62 => [0x748b, false], # East Asian ideograph
        0x214b63 => [0x749c, false], # East Asian ideograph (variant of 4B4B63 which maps to 749C)
        0x214b64 => [0x74a3, false], # East Asian ideograph
        0x214b65 => [0x74a7, false], # East Asian ideograph
        0x214b66 => [0x74a9, false], # East Asian ideograph
        0x214b67 => [0x74b0, false], # East Asian ideograph
        0x214b68 => [0x74a6, false], # East Asian ideograph
        0x214b69 => [0x74bd, false], # East Asian ideograph
        0x224b6a => [0x6ec9, false], # East Asian ideograph
        0x274b6b => [0x73d1, false], # East Asian ideograph
        0x224b6c => [0x6eb3, false], # East Asian ideograph
        0x224b6d => [0x6eb7, false], # East Asian ideograph
        0x214b6e => [0x74e2, false], # East Asian ideograph
        0x214b6f => [0x74e3, false], # East Asian ideograph
        0x214b70 => [0x74e6, false], # East Asian ideograph
        0x234b71 => [0x96e9, false], # East Asian ideograph
        0x214b72 => [0x74f7, false], # East Asian ideograph
        0x214b73 => [0x7504, false], # East Asian ideograph
        0x234b74 => [0x96f1, false], # East Asian ideograph
        0x214b75 => [0x7515, false], # East Asian ideograph
        0x234b76 => [0x96f0, false], # East Asian ideograph
        0x214b77 => [0x751a, false], # East Asian ideograph
        0x234b78 => [0x96fa, false], # East Asian ideograph
        0x224b79 => [0x6ecf, false], # East Asian ideograph
        0x214b7a => [0x7522, false], # East Asian ideograph
        0x214b7b => [0x7526, false], # East Asian ideograph
        0x224b7c => [0x6eca, false], # East Asian ideograph
        0x224b7d => [0x6ed5, false], # East Asian ideograph
        0x214b7e => [0x7529, false], # East Asian ideograph
        0x273740 => [0x53fd, false], # East Asian ideograph
        0x6f526b => [0xc0c0, false], # Korean hangul
        0x294c76 => [0x9753, false], # East Asian ideograph
        0x213565 => [0x542b, false], # East Asian ideograph
        0x277742 => [0x57d8, false], # East Asian ideograph
        0x293344 => [0x8c19, false], # East Asian ideograph
        0x273744 => [0x5428, false], # East Asian ideograph
        0x27624f => [0x9e3e, false], # East Asian ideograph
        0x223745 => [0x65dc, false], # East Asian ideograph
        0x6f593d => [0xcca8, false], # Korean hangul
        0x6f4b3e => [0xb053, false], # Korean hangul
        0x213746 => [0x5679, false], # East Asian ideograph
        0x223747 => [0x65dd, false], # East Asian ideograph
        0x223748 => [0x65df, false], # East Asian ideograph
        0x293345 => [0x8be8, false], # East Asian ideograph
        0x217749 => [0x583d, false], # East Asian ideograph
        0x4b3b43 => [0x5bfe, false], # East Asian ideograph
        0x276175 => [0x9c7f, false], # East Asian ideograph
        0x21374a => [0x5671, false], # East Asian ideograph
        0x6f5d70 => [0xd760, false], # Korean hangul
        0x6f4b3f => [0xb054, false], # Korean hangul
        0x21374b => [0x566f, false], # East Asian ideograph
        0x6f5863 => [0xcb14, false], # Korean hangul
        0x21374c => [0x5662, false], # East Asian ideograph (variant of 4B374C which maps to 5662)
        0x282f47 => [0x620b, false], # East Asian ideograph
        0x22374e => [0x65e4, false], # East Asian ideograph
        0x6f543a => [0xc314, false], # Korean hangul
        0x6f4b40 => [0xb055, false], # Korean hangul
        0x692525 => [0x30a5, false], # Katakana letter small U
        0x4b4c51 => [0x75ca, false], # East Asian ideograph
        0x273751 => [0x5413, false], # East Asian ideograph
        0x213752 => [0x5690, false], # East Asian ideograph
        0x6f5c4d => [0xd488, false], # Korean hangul
        0x70604c => [0x55f5, false], # East Asian ideograph
        0x224a74 => [0x6e4f, false], # East Asian ideograph
        0x334e73 => [0x79a5, false], # East Asian ideograph
        0x234a30 => [0x963d, false], # East Asian ideograph
        0x273754 => [0x565c, false], # East Asian ideograph
        0x2d4343 => [0x6636, false], # East Asian ideograph
        0x2d4768 => [0x6d45, false], # East Asian ideograph (variant of 274768 which maps to 6D45)
        0x223755 => [0x65f0, false], # East Asian ideograph
        0x213756 => [0x56a8, false], # East Asian ideograph
        0x4b375a => [0x53b3, false], # East Asian ideograph
        0x213757 => [0x56b0, false], # East Asian ideograph
        0x2d3758 => [0x54bd, false], # East Asian ideograph
        0x294162 => [0x9486, false], # East Asian ideograph
        0x212a3b => [0xe8e8, false], # EACC component character
        0x284056 => [0x6920, false], # East Asian ideograph
        0x21375a => [0x56b4, false], # East Asian ideograph
        0x6f592d => [0xcc44, false], # Korean hangul
        0x224c21 => [0x6ec3, false], # East Asian ideograph
        0x234c22 => [0x96ff, false], # East Asian ideograph
        0x214c23 => [0x752d, false], # East Asian ideograph
        0x224c24 => [0x6eb4, false], # East Asian ideograph
        0x214c25 => [0x7532, false], # East Asian ideograph
        0x224c26 => [0x6eb2, false], # East Asian ideograph
        0x234c27 => [0x9702, false], # East Asian ideograph
        0x214c28 => [0x7537, false], # East Asian ideograph
        0x224c29 => [0x6eb5, false], # East Asian ideograph
        0x234c2a => [0x9705, false], # East Asian ideograph
        0x214c2b => [0x754f, false], # East Asian ideograph
        0x214c2c => [0x754c, false], # East Asian ideograph
        0x214c2d => [0x755d, false], # East Asian ideograph
        0x224c2e => [0x6ef8, false], # East Asian ideograph
        0x214c2f => [0x7554, false], # East Asian ideograph
        0x224c30 => [0x6f37, false], # East Asian ideograph
        0x214c31 => [0x7559, false], # East Asian ideograph
        0x214c32 => [0x7566, false], # East Asian ideograph
        0x214c33 => [0x7562, false], # East Asian ideograph
        0x224c34 => [0x6efd, false], # East Asian ideograph
        0x224c35 => [0x6f09, false], # East Asian ideograph
        0x214c36 => [0x756b, false], # East Asian ideograph
        0x214c37 => [0x756a, false], # East Asian ideograph
        0x214c38 => [0x7578, false], # East Asian ideograph
        0x214c39 => [0x7576, false], # East Asian ideograph
        0x214c3a => [0x7586, false], # East Asian ideograph
        0x214c3b => [0x7587, false], # East Asian ideograph
        0x214c3c => [0x758a, false], # East Asian ideograph
        0x224c3d => [0x6f63, false], # East Asian ideograph
        0x224c3e => [0x6f12, false], # East Asian ideograph
        0x214c3f => [0x7591, false], # East Asian ideograph
        0x214c40 => [0x759d, false], # East Asian ideograph
        0x224c41 => [0x6f1a, false], # East Asian ideograph
        0x224c42 => [0x6ef6, false], # East Asian ideograph
        0x224c43 => [0x6f19, false], # East Asian ideograph
        0x214c44 => [0x75ab, false], # East Asian ideograph
        0x214c45 => [0x75a5, false], # East Asian ideograph
        0x214c46 => [0x75c7, false], # East Asian ideograph
        0x214c47 => [0x75c5, false], # East Asian ideograph
        0x214c48 => [0x75b3, false], # East Asian ideograph
        0x234c49 => [0x9722, false], # East Asian ideograph
        0x234c4a => [0x9724, false], # East Asian ideograph
        0x224c4b => [0x6f24, false], # East Asian ideograph
        0x214c4c => [0x75bc, false], # East Asian ideograph
        0x214c4d => [0x75b9, false], # East Asian ideograph
        0x234c4e => [0x9728, false], # East Asian ideograph
        0x214c4f => [0x75d4, false], # East Asian ideograph
        0x234c50 => [0x9726, false], # East Asian ideograph
        0x224c51 => [0x6f18, false], # East Asian ideograph
        0x234c52 => [0x9731, false], # East Asian ideograph
        0x214c53 => [0x75e3, false], # East Asian ideograph
        0x214c54 => [0x75d8, false], # East Asian ideograph
        0x214c55 => [0x75de, false], # East Asian ideograph
        0x274c56 => [0x75c9, false], # East Asian ideograph
        0x224c57 => [0x6f1f, false], # East Asian ideograph
        0x214c58 => [0x7601, false], # East Asian ideograph
        0x214c59 => [0x7600, false], # East Asian ideograph
        0x224c5a => [0x6f0a, false], # East Asian ideograph
        0x214c5b => [0x75f2, false], # East Asian ideograph
        0x234c5c => [0x9736, false], # East Asian ideograph
        0x214c5d => [0x75f4, false], # East Asian ideograph
        0x214c5e => [0x75ff, false], # East Asian ideograph
        0x214c5f => [0x75fa, false], # East Asian ideograph
        0x224c60 => [0x6ef9, false], # East Asian ideograph
        0x224c61 => [0x6eee, false], # East Asian ideograph
        0x224c62 => [0x6f41, false], # East Asian ideograph
        0x214c63 => [0x760b, false], # East Asian ideograph
        0x224c64 => [0x6f95, false], # East Asian ideograph
        0x214c65 => [0x7620, false], # East Asian ideograph
        0x214c66 => [0x7629, false], # East Asian ideograph
        0x214c67 => [0x761f, false], # East Asian ideograph
        0x214c68 => [0x7624, false], # East Asian ideograph
        0x214c69 => [0x7626, false], # East Asian ideograph
        0x214c6a => [0x7621, false], # East Asian ideograph
        0x224c6b => [0x6f49, false], # East Asian ideograph
        0x234c6c => [0x9746, false], # East Asian ideograph
        0x224c6d => [0x6f30, false], # East Asian ideograph
        0x214c6e => [0x7642, false], # East Asian ideograph
        0x214c6f => [0x764c, false], # East Asian ideograph
        0x214c70 => [0x7656, false], # East Asian ideograph
        0x274c71 => [0x75a0, false], # East Asian ideograph
        0x6f4c72 => [0xb2ec, false], # Korean hangul
        0x214c73 => [0x7662, false], # East Asian ideograph
        0x214c74 => [0x7665, false], # East Asian ideograph
        0x234c75 => [0x9758, false], # East Asian ideograph
        0x214c76 => [0x766e, false], # East Asian ideograph
        0x224c77 => [0x6eeb, false], # East Asian ideograph
        0x224c78 => [0x6f08, false], # East Asian ideograph
        0x224c79 => [0x6f0e, false], # East Asian ideograph
        0x214c7a => [0x7678, false], # East Asian ideograph
        0x224c7b => [0x6f35, false], # East Asian ideograph
        0x214c7c => [0x767b, false], # East Asian ideograph
        0x234c7d => [0x9764, false], # East Asian ideograph
        0x214c7e => [0x767e, false], # East Asian ideograph
        0x21376b => [0x56f1, false], # East Asian ideograph
        0x6f5c52 => [0xd4e8, false], # Korean hangul
        0x21376d => [0x5703, false], # East Asian ideograph
        0x2d4348 => [0x6681, false], # East Asian ideograph
        0x2e4747 => [0x6d64, false], # East Asian ideograph
        0x3f424f => [0x542f, false], # East Asian ideograph (variant of 27424F which maps to 542F)
        0x6f4b46 => [0xb080, false], # Korean hangul
        0x21376e => [0x5708, false], # East Asian ideograph
        0x212a2f => [0xe8dd, false], # EACC component character
        0x27376f => [0x56ef, false], # East Asian ideograph
        0x273770 => [0x56f4, false], # East Asian ideograph
        0x6f5c53 => [0xd504, false], # Korean hangul
        0x27632b => [0x9f99, false], # East Asian ideograph
        0x213771 => [0x5712, false], # East Asian ideograph
        0x294163 => [0x948c, false], # East Asian ideograph
        0x224637 => [0x6bfa, false], # East Asian ideograph
        0x213772 => [0x5713, false], # East Asian ideograph
        0x2d4349 => [0x66a6, false], # East Asian ideograph
        0x6f526d => [0xc0c5, false], # Korean hangul
        0x213430 => [0x52c9, false], # East Asian ideograph
        0x6f4b47 => [0xb084, false], # Korean hangul
        0x275a32 => [0x8d4f, false], # East Asian ideograph
        0x216f21 => [0x544f, false], # East Asian ideograph
        0x213774 => [0x5716, false], # East Asian ideograph
        0x233775 => [0x8d8d, false], # East Asian ideograph
        0x6f4e2a => [0xb554, false], # Korean hangul
        0x29334e => [0x8c0c, false], # East Asian ideograph
        0x276221 => [0x9cc3, false], # East Asian ideograph
        0x213777 => [0x572d, false], # East Asian ideograph
        0x216222 => [0x9c0d, false], # East Asian ideograph
        0x6f4b48 => [0xb08c, false], # Korean hangul
        0x29445b => [0x9516, false], # East Asian ideograph
        0x276223 => [0x9cab, false], # East Asian ideograph
        0x276224 => [0x9ccd, false], # East Asian ideograph
        0x694c5d => [0x6762, false], # East Asian ideograph
        0x222225 => [0x5bef, false], # East Asian ideograph
        0x706054 => [0x5623, false], # East Asian ideograph
        0x395568 => [0x83dd, false], # East Asian ideograph
        0x234e7b => [0x980f, false], # East Asian ideograph
        0x216226 => [0x9c31, false], # East Asian ideograph
        0x276177 => [0x9c8d, false], # East Asian ideograph
        0x21377c => [0x5751, false], # East Asian ideograph
        0x276227 => [0x9cd4, false], # East Asian ideograph
        0x6f4b49 => [0xb08d, false], # Korean hangul
        0x21377d => [0x574a, false], # East Asian ideograph
        0x276228 => [0x9cd7, false], # East Asian ideograph
        0x276229 => [0x9cdd, false], # East Asian ideograph
        0x6f5c56 => [0xd50c, false], # Korean hangul
        0x27622a => [0x9cde, false], # East Asian ideograph
        0x284d27 => [0x6d9d, false], # East Asian ideograph
        0x27622b => [0x9cdc, false], # East Asian ideograph
        0x27622c => [0x9cd6, false], # East Asian ideograph
        0x28405e => [0x67fd, false], # East Asian ideograph
        0x21622d => [0x9c77, false], # East Asian ideograph
        0x4b4c5b => [0x75f3, false], # East Asian ideograph
        0x27622e => [0x9c88, false], # East Asian ideograph
        0x2d5238 => [0x898a, false], # East Asian ideograph
        0x2e363f => [0x52c5, false], # East Asian ideograph
        0x6f5c57 => [0xd514, false], # Korean hangul
        0x27622f => [0x9e1f, false], # East Asian ideograph
        0x6f5959 => [0xcda7, false], # Korean hangul
        0x214d21 => [0x7682, false], # East Asian ideograph
        0x214d22 => [0x7684, false], # East Asian ideograph
        0x214d23 => [0x7687, false], # East Asian ideograph
        0x214d24 => [0x7686, false], # East Asian ideograph
        0x234d25 => [0x9767, false], # East Asian ideograph
        0x214d26 => [0x768e, false], # East Asian ideograph
        0x214d27 => [0x7696, false], # East Asian ideograph
        0x214d28 => [0x7693, false], # East Asian ideograph
        0x214d29 => [0x769a, false], # East Asian ideograph
        0x214d2a => [0x76ae, false], # East Asian ideograph
        0x214d2b => [0x76b0, false], # East Asian ideograph
        0x214d2c => [0x76b4, false], # East Asian ideograph
        0x274d2d => [0x76b1, false], # East Asian ideograph
        0x214d2e => [0x76bf, false], # East Asian ideograph
        0x214d2f => [0x76c2, false], # East Asian ideograph
        0x224d30 => [0x6f60, false], # East Asian ideograph
        0x234d31 => [0x9777, false], # East Asian ideograph
        0x214d32 => [0x76c6, false], # East Asian ideograph
        0x214d33 => [0x76ca, false], # East Asian ideograph
        0x214d34 => [0x76cd, false], # East Asian ideograph
        0x214d35 => [0x76ce, false], # East Asian ideograph
        0x214d36 => [0x76d4, false], # East Asian ideograph
        0x214d37 => [0x76d2, false], # East Asian ideograph
        0x214d38 => [0x76dc, false], # East Asian ideograph
        0x214d39 => [0x76db, false], # East Asian ideograph
        0x234d3a => [0x9780, false], # East Asian ideograph
        0x214d3b => [0x76df, false], # East Asian ideograph
        0x234d3c => [0x9781, false], # East Asian ideograph
        0x214d3d => [0x76e3, false], # East Asian ideograph
        0x274d3e => [0x76d8, false], # East Asian ideograph
        0x274d3f => [0x5362, false], # East Asian ideograph
        0x214d40 => [0x76e5, false], # East Asian ideograph
        0x214d41 => [0x76ea, false], # East Asian ideograph
        0x214d42 => [0x76ee, false], # East Asian ideograph
        0x214d43 => [0x76ef, false], # East Asian ideograph
        0x214d44 => [0x76f2, false], # East Asian ideograph
        0x214d45 => [0x76f4, false], # East Asian ideograph
        0x214d46 => [0x7709, false], # East Asian ideograph
        0x214d47 => [0x76f9, false], # East Asian ideograph
        0x214d48 => [0x76f8, false], # East Asian ideograph
        0x214d49 => [0x7701, false], # East Asian ideograph
        0x214d4a => [0x770b, false], # East Asian ideograph
        0x214d4b => [0x76fc, false], # East Asian ideograph
        0x214d4c => [0x76fe, false], # East Asian ideograph
        0x214d4d => [0x7729, false], # East Asian ideograph
        0x214d4e => [0x7720, false], # East Asian ideograph
        0x214d4f => [0x771e, false], # East Asian ideograph
        0x214d50 => [0x7728, false], # East Asian ideograph
        0x214d51 => [0x7737, false], # East Asian ideograph
        0x214d52 => [0x773c, false], # East Asian ideograph
        0x214d53 => [0x7736, false], # East Asian ideograph
        0x214d54 => [0x7738, false], # East Asian ideograph
        0x214d55 => [0x773a, false], # East Asian ideograph
        0x274d56 => [0x4f17, false], # East Asian ideograph
        0x274d57 => [0x56f0, false], # East Asian ideograph
        0x214d58 => [0x776b, false], # East Asian ideograph
        0x214d59 => [0x775b, false], # East Asian ideograph
        0x214d5a => [0x776a, false], # East Asian ideograph
        0x214d5b => [0x7766, false], # East Asian ideograph
        0x214d5c => [0x7779, false], # East Asian ideograph
        0x274d5d => [0x7750, false], # East Asian ideograph
        0x214d5e => [0x7763, false], # East Asian ideograph
        0x214d5f => [0x775c, false], # East Asian ideograph
        0x214d60 => [0x776c, false], # East Asian ideograph
        0x214d61 => [0x7768, false], # East Asian ideograph
        0x214d62 => [0x7765, false], # East Asian ideograph
        0x214d63 => [0x777d, false], # East Asian ideograph
        0x214d64 => [0x7771, false], # East Asian ideograph
        0x214d65 => [0x777f, false], # East Asian ideograph
        0x214d66 => [0x7784, false], # East Asian ideograph
        0x214d67 => [0x7761, false], # East Asian ideograph
        0x214d68 => [0x7787, false], # East Asian ideograph
        0x214d69 => [0x778e, false], # East Asian ideograph
        0x214d6a => [0x778c, false], # East Asian ideograph
        0x214d6b => [0x7791, false], # East Asian ideograph
        0x214d6c => [0x779f, false], # East Asian ideograph
        0x214d6d => [0x779e, false], # East Asian ideograph
        0x214d6e => [0x77a0, false], # East Asian ideograph
        0x214d6f => [0x77a5, false], # East Asian ideograph
        0x214d70 => [0x77b3, false], # East Asian ideograph
        0x214d71 => [0x77aa, false], # East Asian ideograph
        0x214d72 => [0x77b0, false], # East Asian ideograph
        0x214d73 => [0x77ad, false], # East Asian ideograph
        0x214d74 => [0x77ac, false], # East Asian ideograph
        0x214d75 => [0x77a7, false], # East Asian ideograph
        0x214d76 => [0x77bd, false], # East Asian ideograph
        0x214d77 => [0x77bf, false], # East Asian ideograph
        0x214d78 => [0x77bb, false], # East Asian ideograph
        0x224d79 => [0x6fa6, false], # East Asian ideograph
        0x214d7a => [0x77d3, false], # East Asian ideograph
        0x214d7b => [0x77d7, false], # East Asian ideograph
        0x214d7c => [0x77da, false], # East Asian ideograph
        0x214d7d => [0x77db, false], # East Asian ideograph
        0x214d7e => [0x77dc, false], # East Asian ideograph
        0x276240 => [0x9e44, false], # East Asian ideograph
        0x216241 => [0x9d5d, false], # East Asian ideograph
        0x233d74 => [0x9062, false], # East Asian ideograph
        0x216242 => [0x9d89, false], # East Asian ideograph
        0x293b6d => [0x8f8a, false], # East Asian ideograph
        0x276243 => [0x9e4a, false], # East Asian ideograph
        0x6f4d6a => [0xb4f1, false], # Korean hangul
        0x216244 => [0x9d6a, false], # East Asian ideograph
        0x216245 => [0x9d6c, false], # East Asian ideograph
        0x6f4b4f => [0xb098, false], # Korean hangul
        0x276246 => [0x9e64, false], # East Asian ideograph
        0x333d75 => [0x5fb3, false], # East Asian ideograph
        0x276247 => [0x83ba, false], # East Asian ideograph
        0x6f5c5c => [0xd544, false], # Korean hangul
        0x232248 => [0x8453, false], # East Asian ideograph
        0x276249 => [0x9e67, false], # East Asian ideograph
        0x27624a => [0x9e25, false], # East Asian ideograph
        0x27624b => [0x9e36, false], # East Asian ideograph
        0x27624c => [0x9e70, false], # East Asian ideograph
        0x2e3645 => [0x69e3, false], # East Asian ideograph
        0x21624d => [0x9dfa, false], # East Asian ideograph
        0x293357 => [0x8c14, false], # East Asian ideograph
        0x27624e => [0x9e66, false], # East Asian ideograph
        0x21624f => [0x9e1e, false], # East Asian ideograph
        0x4b4f4c => [0x7a4f, false], # East Asian ideograph
        0x6f4b51 => [0xb09a, false], # Korean hangul
        0x276250 => [0x54b8, false], # East Asian ideograph
        0x294021 => [0x90f8, false], # East Asian ideograph
        0x235b4d => [0x9d7b, false], # East Asian ideograph
        0x233934 => [0x8dec, false], # East Asian ideograph
        0x6f5c5e => [0xd54d, false], # Korean hangul
        0x216252 => [0x9e7c, false], # East Asian ideograph
        0x344177 => [0x8264, false], # East Asian ideograph
        0x39526b => [0x7094, false], # East Asian ideograph
        0x216256 => [0x9e97, false], # East Asian ideograph
        0x2d5461 => [0x8306, false], # East Asian ideograph
        0x6f5c5f => [0xd54f, false], # Korean hangul
        0x274931 => [0x6c88, false], # East Asian ideograph
        0x293359 => [0x8c11, false], # East Asian ideograph
        0x4b5d70 => [0x92ad, false], # East Asian ideograph
        0x234a42 => [0x9660, false], # East Asian ideograph
        0x216259 => [0x9e9d, false], # East Asian ideograph
        0x6f4b53 => [0xb09f, false], # Korean hangul
        0x294466 => [0x9515, false], # East Asian ideograph
        0x214e21 => [0x77e2, false], # East Asian ideograph
        0x214e22 => [0x77e3, false], # East Asian ideograph
        0x214e23 => [0x77e5, false], # East Asian ideograph
        0x214e24 => [0x77e9, false], # East Asian ideograph
        0x214e25 => [0x77ed, false], # East Asian ideograph
        0x214e26 => [0x77ee, false], # East Asian ideograph
        0x214e27 => [0x77ef, false], # East Asian ideograph
        0x214e28 => [0x77f3, false], # East Asian ideograph
        0x214e29 => [0x77fd, false], # East Asian ideograph
        0x214e2a => [0x7802, false], # East Asian ideograph
        0x214e2b => [0x780d, false], # East Asian ideograph
        0x214e2c => [0x780c, false], # East Asian ideograph
        0x234e2d => [0x97b8, false], # East Asian ideograph
        0x214e2e => [0x7830, false], # East Asian ideograph
        0x214e2f => [0x781d, false], # East Asian ideograph
        0x214e30 => [0x7834, false], # East Asian ideograph
        0x214e31 => [0x7838, false], # East Asian ideograph
        0x214e32 => [0x7837, false], # East Asian ideograph
        0x214e33 => [0x7827, false], # East Asian ideograph
        0x214e34 => [0x782d, false], # East Asian ideograph
        0x214e35 => [0x7825, false], # East Asian ideograph
        0x214e36 => [0x786b, false], # East Asian ideograph
        0x214e37 => [0x784f, false], # East Asian ideograph
        0x234e38 => [0x97c0, false], # East Asian ideograph
        0x214e39 => [0x786c, false], # East Asian ideograph
        0x214e3a => [0x785d, false], # East Asian ideograph
        0x214e3b => [0x786f, false], # East Asian ideograph
        0x214e3c => [0x78b0, false], # East Asian ideograph
        0x214e3d => [0x7897, false], # East Asian ideograph
        0x214e3e => [0x788e, false], # East Asian ideograph
        0x214e3f => [0x7898, false], # East Asian ideograph
        0x214e40 => [0x7889, false], # East Asian ideograph
        0x214e41 => [0x7891, false], # East Asian ideograph
        0x214e42 => [0x787c, false], # East Asian ideograph
        0x214e43 => [0x788c, false], # East Asian ideograph
        0x214e44 => [0x78a7, false], # East Asian ideograph
        0x214e45 => [0x78a9, false], # East Asian ideograph
        0x214e46 => [0x789f, false], # East Asian ideograph
        0x214e47 => [0x78b3, false], # East Asian ideograph
        0x214e48 => [0x78cb, false], # East Asian ideograph
        0x214e49 => [0x78ba, false], # East Asian ideograph
        0x214e4a => [0x78c1, false], # East Asian ideograph
        0x214e4b => [0x78c5, false], # East Asian ideograph
        0x214e4c => [0x78bc, false], # East Asian ideograph
        0x214e4d => [0x78d5, false], # East Asian ideograph
        0x214e4e => [0x78be, false], # East Asian ideograph
        0x214e4f => [0x78ca, false], # East Asian ideograph
        0x214e50 => [0x78d0, false], # East Asian ideograph
        0x214e51 => [0x78e8, false], # East Asian ideograph
        0x214e52 => [0x78ec, false], # East Asian ideograph
        0x214e53 => [0x78da, false], # East Asian ideograph
        0x214e54 => [0x78f7, false], # East Asian ideograph
        0x214e55 => [0x78f4, false], # East Asian ideograph
        0x214e56 => [0x78fa, false], # East Asian ideograph (variant of 4B4E56 which maps to 78FA)
        0x214e57 => [0x7901, false], # East Asian ideograph
        0x214e58 => [0x78ef, false], # East Asian ideograph
        0x234e59 => [0x97dd, false], # East Asian ideograph
        0x214e5a => [0x7919, false], # East Asian ideograph
        0x214e5b => [0x7926, false], # East Asian ideograph
        0x214e5c => [0x792c, false], # East Asian ideograph
        0x224e5d => [0x6fde, false], # East Asian ideograph
        0x214e5e => [0x792b, false], # East Asian ideograph
        0x214e5f => [0x793a, false], # East Asian ideograph
        0x214e60 => [0x7940, false], # East Asian ideograph
        0x214e61 => [0x793e, false], # East Asian ideograph
        0x214e62 => [0x7941, false], # East Asian ideograph
        0x214e63 => [0x7945, false], # East Asian ideograph
        0x214e64 => [0x7949, false], # East Asian ideograph
        0x214e65 => [0x7948, false], # East Asian ideograph
        0x214e66 => [0x7947, false], # East Asian ideograph
        0x224e67 => [0x700c, false], # East Asian ideograph
        0x214e68 => [0x7960, false], # East Asian ideograph
        0x214e69 => [0x7950, false], # East Asian ideograph
        0x214e6a => [0x7956, false], # East Asian ideograph
        0x214e6b => [0x795e, false], # East Asian ideograph
        0x214e6c => [0x795d, false], # East Asian ideograph
        0x214e6d => [0x795f, false], # East Asian ideograph
        0x214e6e => [0x795a, false], # East Asian ideograph
        0x214e6f => [0x7957, false], # East Asian ideograph
        0x214e70 => [0x7965, false], # East Asian ideograph
        0x214e71 => [0x7968, false], # East Asian ideograph
        0x214e72 => [0x796d, false], # East Asian ideograph
        0x234e73 => [0x97fa, false], # East Asian ideograph
        0x214e74 => [0x7981, false], # East Asian ideograph
        0x214e75 => [0x797f, false], # East Asian ideograph
        0x214e76 => [0x798f, false], # East Asian ideograph
        0x214e77 => [0x798d, false], # East Asian ideograph
        0x214e78 => [0x798e, false], # East Asian ideograph
        0x214e79 => [0x79a6, false], # East Asian ideograph
        0x214e7a => [0x79a7, false], # East Asian ideograph
        0x214e7b => [0x79aa, false], # East Asian ideograph
        0x214e7c => [0x79ae, false], # East Asian ideograph
        0x214e7d => [0x79b1, false], # East Asian ideograph
        0x214e7e => [0x79b9, false], # East Asian ideograph
        0x6f5c63 => [0xd558, false], # Korean hangul
        0x6f4e2d => [0xb55f, false], # Korean hangul
        0x29335d => [0x8c16, false], # East Asian ideograph
        0x216461 => [0x4ec8, false], # East Asian ideograph
        0x234a46 => [0x9658, false], # East Asian ideograph
        0x69626d => [0x7874, false], # East Asian ideograph
        0x6f4b57 => [0xb0a9, false], # Korean hangul
        0x27626f => [0x515a, false], # East Asian ideograph
        0x6f5c64 => [0xd559, false], # Korean hangul
        0x274936 => [0x6ee4, false], # East Asian ideograph
        0x6f5821 => [0xc974, false], # Korean hangul
        0x6f4d53 => [0xb450, false], # Korean hangul
        0x216271 => [0x9ef4, false], # East Asian ideograph
        0x216272 => [0x9ef7, false], # East Asian ideograph
        0x4b6159 => [0x81b8, false], # East Asian ideograph
        0x216273 => [0x9f07, false], # East Asian ideograph
        0x216275 => [0x9f13, false], # East Asian ideograph
        0x274937 => [0x6d4f, false], # East Asian ideograph
        0x6f5822 => [0xc988, false], # Korean hangul
        0x216276 => [0x9f15, false], # East Asian ideograph
        0x274633 => [0x6b8b, false], # East Asian ideograph
        0x6f4d22 => [0xb308, false], # Korean hangul
        0x6f4b59 => [0xb0ac, false], # Korean hangul
        0x4b6278 => [0x9f21, false], # East Asian ideograph
        0x224d23 => [0x6f7e, false], # East Asian ideograph
        0x21712d => [0x5593, false], # East Asian ideograph
        0x224d24 => [0x6f9d, false], # East Asian ideograph
        0x21627a => [0x9f34, false], # East Asian ideograph
        0x6f4d25 => [0xb313, false], # Korean hangul
        0x6f5823 => [0xc989, false], # Korean hangul
        0x23227b => [0x8484, false], # East Asian ideograph
        0x22464a => [0x6c05, false], # East Asian ideograph
        0x6f4d26 => [0xb314, false], # Korean hangul
        0x6f534b => [0xc1b0, false], # Korean hangul
        0x23227c => [0x8478, false], # East Asian ideograph
        0x224d27 => [0x6f87, false], # East Asian ideograph
        0x6f4b5a => [0xb0ad, false], # Korean hangul
        0x21627d => [0x9f4a, false], # East Asian ideograph
        0x6f4d28 => [0xb354, false], # Korean hangul
        0x27627e => [0x658e, false], # East Asian ideograph
        0x274d29 => [0x7691, false], # East Asian ideograph
        0x274d7c => [0x77a9, false], # East Asian ideograph
        0x6f5c67 => [0xd565, false], # Korean hangul
        0x6f4d2a => [0xb358, false], # Korean hangul
        0x6f5824 => [0xc98c, false], # Korean hangul
        0x213c7a => [0x5eb8, false], # East Asian ideograph
        0x224d2b => [0x6f6f, false], # East Asian ideograph
        0x6f5271 => [0xc0cf, false], # Korean hangul
        0x234d2c => [0x976b, false], # East Asian ideograph
        0x6f4b5b => [0xb0ae, false], # Korean hangul
        0x214d2d => [0x76ba, false], # East Asian ideograph
        0x6f4c39 => [0xb192, false], # Korean hangul
        0x29402b => [0x90ba, false], # East Asian ideograph
        0x23393e => [0x8df2, false], # East Asian ideograph
        0x282d79 => [0x60ab, false], # East Asian ideograph
        0x6f4d2e => [0xb364, false], # Korean hangul
        0x2d3251 => [0x510c, false], # East Asian ideograph
        0x6f492c => [0xac84, false], # Korean hangul
        0x6f5c68 => [0xd568, false], # Korean hangul
        0x224d2f => [0x6f5a, false], # East Asian ideograph
        0x293362 => [0x8c1d, false], # East Asian ideograph
        0x214f21 => [0x79bd, false], # East Asian ideograph
        0x214f22 => [0x842c, false], # East Asian ideograph
        0x214f23 => [0x79be, false], # East Asian ideograph
        0x214f24 => [0x79c0, false], # East Asian ideograph
        0x214f25 => [0x79c1, false], # East Asian ideograph
        0x214f26 => [0x79bf, false], # East Asian ideograph
        0x214d31 => [0x76c8, false], # East Asian ideograph
        0x214f28 => [0x79d1, false], # East Asian ideograph
        0x214f29 => [0x79cb, false], # East Asian ideograph
        0x214f2a => [0x79d2, false], # East Asian ideograph
        0x214f2b => [0x79e4, false], # East Asian ideograph
        0x214f2c => [0x79e6, false], # East Asian ideograph
        0x214f2d => [0x79e3, false], # East Asian ideograph
        0x214f2e => [0x79df, false], # East Asian ideograph
        0x214f2f => [0x79e7, false], # East Asian ideograph
        0x214f30 => [0x79e9, false], # East Asian ideograph
        0x224f31 => [0x702d, false], # East Asian ideograph
        0x214f32 => [0x7a05, false], # East Asian ideograph
        0x214f33 => [0x7a0d, false], # East Asian ideograph
        0x214f34 => [0x7a08, false], # East Asian ideograph
        0x214f35 => [0x7a0b, false], # East Asian ideograph
        0x214f36 => [0x7a00, false], # East Asian ideograph
        0x214f37 => [0x7a1f, false], # East Asian ideograph
        0x234f38 => [0x981f, false], # East Asian ideograph
        0x214f39 => [0x7a20, false], # East Asian ideograph
        0x214f3a => [0x7a1a, false], # East Asian ideograph
        0x214f3b => [0x7a14, false], # East Asian ideograph
        0x214f3c => [0x7a31, false], # East Asian ideograph
        0x214f3d => [0x7a2e, false], # East Asian ideograph
        0x214f3e => [0x7a3f, false], # East Asian ideograph
        0x214f3f => [0x7a3c, false], # East Asian ideograph
        0x274f40 => [0x8c37, false], # East Asian ideograph
        0x214f41 => [0x7a3d, false], # East Asian ideograph
        0x214f42 => [0x7a37, false], # East Asian ideograph
        0x214f43 => [0x7a3b, false], # East Asian ideograph
        0x214f44 => [0x7a4d, false], # East Asian ideograph
        0x214f45 => [0x7a4e, false], # East Asian ideograph
        0x214f46 => [0x7a4c, false], # East Asian ideograph
        0x214f47 => [0x7a46, false], # East Asian ideograph
        0x214f48 => [0x7a57, false], # East Asian ideograph
        0x274f49 => [0x7a51, false], # East Asian ideograph
        0x214f4a => [0x7a62, false], # East Asian ideograph
        0x274f4b => [0x83b7, false], # East Asian ideograph (duplicate simplified)
        0x214f4c => [0x7a69, false], # East Asian ideograph
        0x214f4d => [0x7a74, false], # East Asian ideograph
        0x214f4e => [0x7a76, false], # East Asian ideograph
        0x214f4f => [0x7a79, false], # East Asian ideograph
        0x214f50 => [0x7a7a, false], # East Asian ideograph
        0x214f51 => [0x7a7f, false], # East Asian ideograph
        0x214f52 => [0x7a81, false], # East Asian ideograph
        0x214f53 => [0x7a84, false], # East Asian ideograph
        0x214f54 => [0x7a88, false], # East Asian ideograph
        0x214f55 => [0x7a92, false], # East Asian ideograph
        0x214f56 => [0x7a95, false], # East Asian ideograph
        0x214f57 => [0x7a98, false], # East Asian ideograph
        0x214f58 => [0x7a96, false], # East Asian ideograph
        0x214f59 => [0x7a97, false], # East Asian ideograph
        0x214f5a => [0x7a9f, false], # East Asian ideograph
        0x214f5b => [0x7aa0, false], # East Asian ideograph
        0x214f5c => [0x7aaa, false], # East Asian ideograph
        0x214d3a => [0x76de, false], # East Asian ideograph
        0x214f5e => [0x7aaf, false], # East Asian ideograph
        0x214f5f => [0x7aae, false], # East Asian ideograph
        0x274f60 => [0x7aa5, false], # East Asian ideograph
        0x274f61 => [0x7a8d, false], # East Asian ideograph
        0x274f62 => [0x7a9c, false], # East Asian ideograph
        0x274f63 => [0x7aa6, false], # East Asian ideograph
        0x214f64 => [0x7aca, false], # East Asian ideograph
        0x214f65 => [0x7acb, false], # East Asian ideograph
        0x214f66 => [0x7ad9, false], # East Asian ideograph
        0x214f67 => [0x7ae5, false], # East Asian ideograph
        0x214f68 => [0x7ae3, false], # East Asian ideograph
        0x214d3c => [0x76e1, false], # East Asian ideograph
        0x214f6a => [0x7aef, false], # East Asian ideograph
        0x274f6b => [0x7ade, false], # East Asian ideograph
        0x214f6c => [0x7af9, false], # East Asian ideograph
        0x214f6d => [0x7afa, false], # East Asian ideograph
        0x214f6e => [0x7aff, false], # East Asian ideograph
        0x214f6f => [0x7afd, false], # East Asian ideograph
        0x214f70 => [0x7b06, false], # East Asian ideograph
        0x214f71 => [0x7b11, false], # East Asian ideograph
        0x214f72 => [0x7b20, false], # East Asian ideograph
        0x214f73 => [0x7b2c, false], # East Asian ideograph
        0x214f74 => [0x7b28, false], # East Asian ideograph
        0x214d3e => [0x76e4, false], # East Asian ideograph
        0x214f76 => [0x7b1e, false], # East Asian ideograph
        0x214f77 => [0x7b19, false], # East Asian ideograph
        0x214f78 => [0x7b26, false], # East Asian ideograph
        0x214f79 => [0x7b46, false], # East Asian ideograph
        0x214f7a => [0x7b49, false], # East Asian ideograph
        0x214d3f => [0x76e7, false], # East Asian ideograph
        0x214f7c => [0x7b56, false], # East Asian ideograph
        0x214f7d => [0x7b52, false], # East Asian ideograph
        0x214f7e => [0x7b4b, false], # East Asian ideograph
        0x234d40 => [0x9784, false], # East Asian ideograph
        0x6f4b5f => [0xb0b4, false], # Korean hangul
        0x294472 => [0x951e, false], # East Asian ideograph
        0x4b4d41 => [0x862f, false], # East Asian ideograph
        0x6f4d42 => [0xb3cc, false], # Korean hangul
        0x2e3654 => [0x657f, false], # East Asian ideograph
        0x2d502b => [0x693e, false], # East Asian ideograph
        0x234d43 => [0x977f, false], # East Asian ideograph
        0x6f5829 => [0xc9c0, false], # Korean hangul
        0x224d44 => [0x6f0b, false], # East Asian ideograph
        0x2d4362 => [0x6722, false], # East Asian ideograph
        0x4b4d45 => [0x76f4, false], # East Asian ideograph (variant of 214D45 which maps to 76F4)
        0x6f4b60 => [0xb0b5, false], # Korean hangul
        0x217577 => [0x57d2, false], # East Asian ideograph
        0x6f4d46 => [0xb3d7, false], # Korean hangul
        0x213145 => [0x4f9d, false], # East Asian ideograph
        0x217134 => [0x5588, false], # East Asian ideograph
        0x6f4d47 => [0xb3d9, false], # Korean hangul
        0x6f5c6d => [0xd571, false], # Korean hangul
        0x27493f => [0x6f9c, false], # East Asian ideograph
        0x6f582a => [0xc9c1, false], # Korean hangul
        0x276256 => [0x4e3d, false], # East Asian ideograph
        0x224651 => [0x6c0c, false], # East Asian ideograph
        0x234d49 => [0x9789, false], # East Asian ideograph
        0x6f4d4a => [0xb400, false], # Korean hangul
        0x6f4b61 => [0xb0b8, false], # Korean hangul
        0x294474 => [0x951f, false], # East Asian ideograph
        0x224d4b => [0x6f6c, false], # East Asian ideograph
        0x294031 => [0x909d, false], # East Asian ideograph
        0x333944 => [0x5b2d, false], # East Asian ideograph
        0x6f4d4c => [0xb418, false], # Korean hangul
        0x283f30 => [0x6966, false], # East Asian ideograph
        0x224d4d => [0x6f8b, false], # East Asian ideograph
        0x6f582b => [0xc9c4, false], # Korean hangul
        0x6f4d4e => [0xb420, false], # Korean hangul
        0x2d4364 => [0x671e, false], # East Asian ideograph
        0x2d4d4f => [0x771f, false], # East Asian ideograph
        0x276327 => [0x9f89, false], # East Asian ideograph
        0x6f586a => [0xcb50, false], # Korean hangul
        0x6f4d50 => [0xb429, false], # Korean hangul
        0x213147 => [0x4f75, false], # East Asian ideograph
        0x6f4d51 => [0xb42b, false], # Korean hangul
        0x212329 => [0xff09, false], # Ideographic right parenthesis
        0x6f4d52 => [0xb42c, false], # Korean hangul
        0x6f582c => [0xc9c7, false], # Korean hangul
        0x4b3b67 => [0x6b67, false], # East Asian ideograph
        0x234d54 => [0x9794, false], # East Asian ideograph
        0x6f4b63 => [0xb0bc, false], # Korean hangul
        0x335773 => [0x88b4, false], # East Asian ideograph
        0x6f4d55 => [0xb454, false], # Korean hangul
        0x27514a => [0x7ef0, false], # East Asian ideograph
        0x214d56 => [0x773e, false], # East Asian ideograph
        0x6f5c70 => [0xd578, false], # Korean hangul
        0x276b5b => [0x5250, false], # East Asian ideograph
        0x214d57 => [0x774f, false], # East Asian ideograph
        0x3f5959 => [0x8276, false], # East Asian ideograph
        0x224d58 => [0x6e88, false], # East Asian ideograph
        0x35347b => [0x8b2d, false], # East Asian ideograph
        0x234d59 => [0x979b, false], # East Asian ideograph
        0x6f4b64 => [0xb0c4, false], # Korean hangul
        0x224d5a => [0x6f55, false], # East Asian ideograph
        0x213149 => [0x4f73, false], # East Asian ideograph
        0x21623e => [0x9d61, false], # East Asian ideograph
        0x235021 => [0x9865, false], # East Asian ideograph
        0x235022 => [0x9866, false], # East Asian ideograph
        0x215023 => [0x7b54, false], # East Asian ideograph
        0x215024 => [0x7b60, false], # East Asian ideograph
        0x215025 => [0x7b77, false], # East Asian ideograph
        0x215026 => [0x7b75, false], # East Asian ideograph
        0x215027 => [0x7ba1, false], # East Asian ideograph
        0x215028 => [0x7b94, false], # East Asian ideograph
        0x235029 => [0x986c, false], # East Asian ideograph
        0x21502a => [0x7b9d, false], # East Asian ideograph
        0x21502b => [0x7b8b, false], # East Asian ideograph
        0x21502c => [0x7b97, false], # East Asian ideograph
        0x21502d => [0x7b8f, false], # East Asian ideograph
        0x21502e => [0x7bc7, false], # East Asian ideograph
        0x214d5d => [0x775e, false], # East Asian ideograph
        0x235030 => [0x9873, false], # East Asian ideograph
        0x215031 => [0x7bb1, false], # East Asian ideograph
        0x215032 => [0x7bb4, false], # East Asian ideograph
        0x215033 => [0x7bc0, false], # East Asian ideograph
        0x215034 => [0x7bc6, false], # East Asian ideograph
        0x215035 => [0x7bc1, false], # East Asian ideograph
        0x215036 => [0x7c11, false], # East Asian ideograph
        0x215037 => [0x7bd9, false], # East Asian ideograph
        0x215038 => [0x7bdb, false], # East Asian ideograph
        0x235039 => [0x98ad, false], # East Asian ideograph
        0x21503a => [0x7bc9, false], # East Asian ideograph
        0x21503b => [0x7be1, false], # East Asian ideograph
        0x21503c => [0x7be9, false], # East Asian ideograph
        0x21503d => [0x7c07, false], # East Asian ideograph
        0x21503e => [0x7c0d, false], # East Asian ideograph
        0x21503f => [0x7bfe, false], # East Asian ideograph
        0x235040 => [0x98b4, false], # East Asian ideograph
        0x215041 => [0x7c21, false], # East Asian ideograph
        0x215042 => [0x7c2b, false], # East Asian ideograph
        0x215043 => [0x7c2a, false], # East Asian ideograph
        0x215044 => [0x7c27, false], # East Asian ideograph
        0x215045 => [0x7c1e, false], # East Asian ideograph
        0x215046 => [0x7c23, false], # East Asian ideograph
        0x215047 => [0x7c3f, false], # East Asian ideograph
        0x215048 => [0x7c3e, false], # East Asian ideograph
        0x215049 => [0x7c38, false], # East Asian ideograph
        0x21504a => [0x7c37, false], # East Asian ideograph
        0x27504b => [0x7b7e, false], # East Asian ideograph
        0x21504c => [0x7c43, false], # East Asian ideograph
        0x23504d => [0x98bb, false], # East Asian ideograph
        0x23504e => [0x98c0, false], # East Asian ideograph
        0x21504f => [0x7c50, false], # East Asian ideograph
        0x215050 => [0x7c60, false], # East Asian ideograph
        0x215051 => [0x7c5f, false], # East Asian ideograph
        0x215052 => [0x7c64, false], # East Asian ideograph
        0x215053 => [0x7c6c, false], # East Asian ideograph
        0x215054 => [0x7c6e, false], # East Asian ideograph
        0x215055 => [0x7c72, false], # East Asian ideograph
        0x215056 => [0x7c73, false], # East Asian ideograph
        0x215057 => [0x7c89, false], # East Asian ideograph
        0x215058 => [0x7c92, false], # East Asian ideograph
        0x215059 => [0x7c97, false], # East Asian ideograph
        0x21505a => [0x7c9f, false], # East Asian ideograph
        0x21505b => [0x7ca5, false], # East Asian ideograph
        0x21505c => [0x7ca4, false], # East Asian ideograph
        0x21505d => [0x7cb1, false], # East Asian ideograph
        0x21505e => [0x7cb3, false], # East Asian ideograph
        0x23505f => [0x98e1, false], # East Asian ideograph
        0x275060 => [0x7c8b, false], # East Asian ideograph
        0x215061 => [0xfa1d, false], # East Asian ideograph
        0x275062 => [0x80e1, false], # East Asian ideograph (duplicate simplified)
        0x215063 => [0x7cd6, false], # East Asian ideograph
        0x215064 => [0x7cd5, false], # East Asian ideograph
        0x215065 => [0x7ce0, false], # East Asian ideograph
        0x215066 => [0x7cdc, false], # East Asian ideograph
        0x215067 => [0x7cdf, false], # East Asian ideograph
        0x215068 => [0x7cde, false], # East Asian ideograph
        0x215069 => [0x7ce2, false], # East Asian ideograph
        0x21506a => [0x7cd9, false], # East Asian ideograph
        0x21506b => [0x7ce7, false], # East Asian ideograph
        0x21506c => [0x7cef, false], # East Asian ideograph
        0x2e506d => [0x70b1, false], # East Asian ideograph
        0x21506e => [0x7cfb, false], # East Asian ideograph
        0x21506f => [0x7cfe, false], # East Asian ideograph
        0x215070 => [0x7d00, false], # East Asian ideograph
        0x215071 => [0x7d02, false], # East Asian ideograph
        0x215072 => [0x7d05, false], # East Asian ideograph
        0x225073 => [0x70a9, false], # East Asian ideograph
        0x215074 => [0x7d04, false], # East Asian ideograph
        0x215075 => [0x7d07, false], # East Asian ideograph
        0x215076 => [0x7d21, false], # East Asian ideograph
        0x215077 => [0x7d0b, false], # East Asian ideograph
        0x225078 => [0x70ea, false], # East Asian ideograph
        0x215079 => [0x7d20, false], # East Asian ideograph
        0x21507a => [0x7d1c, false], # East Asian ideograph
        0x21507b => [0x7d22, false], # East Asian ideograph
        0x27507c => [0x7eb0, false], # East Asian ideograph
        0x234d6a => [0x97ac, false], # East Asian ideograph
        0x21507e => [0x7d10, false], # East Asian ideograph
        0x6f5c74 => [0xd587, false], # Korean hangul
        0x6f4d6b => [0xb514, false], # Korean hangul
        0x6f5831 => [0xc9d3, false], # Korean hangul
        0x6f4d6c => [0xb515, false], # Korean hangul
        0x6f7641 => [0xe8b3, false], # Korean hangul
        0x2d4d6d => [0x7792, false], # East Asian ideograph
        0x2d3f27 => [0x6120, false], # East Asian ideograph
        0x69252d => [0x30ad, false], # Katakana letter KI
        0x6f4d6e => [0xb51b, false], # Korean hangul
        0x4b4c79 => [0x7672, false], # East Asian ideograph
        0x6f4d6f => [0xb51c, false], # Korean hangul
        0x4c4c35 => [0x6e0c, false], # East Asian ideograph
        0x6f5c75 => [0xd588, false], # Korean hangul
        0x234d70 => [0x97ae, false], # East Asian ideograph
        0x6f5832 => [0xc9d5, false], # Korean hangul
        0x234d71 => [0x97a8, false], # East Asian ideograph
        0x334a58 => [0x89dd, false], # East Asian ideograph
        0x6f4d72 => [0xb527, false], # Korean hangul
        0x4b537d => [0x9acc, false], # East Asian ideograph
        0x6f592a => [0xcc3d, false], # Korean hangul
        0x6f5337 => [0xc14b, false], # Korean hangul
        0x274d73 => [0x4e86, false], # East Asian ideograph
        0x224d74 => [0x6f9f, false], # East Asian ideograph
        0x2d325f => [0x50bb, false], # East Asian ideograph (variant of 4B325F which maps to 50BB)
        0x6f4d75 => [0xb52a, false], # Korean hangul
        0x6f5833 => [0xc9d6, false], # Korean hangul
        0x29416a => [0x948d, false], # East Asian ideograph
        0x22465a => [0x6c18, false], # East Asian ideograph
        0x2d3821 => [0x962f, false], # East Asian ideograph
        0x6f5274 => [0xc0d9, false], # Korean hangul
        0x213822 => [0x5747, false], # East Asian ideograph
        0x39447d => [0x6ac2, false], # East Asian ideograph
        0x234d78 => [0x97a5, false], # East Asian ideograph
        0x2d4d21 => [0x7681, false], # East Asian ideograph
        0x6f4d79 => [0xb531, false], # Korean hangul
        0x287360 => [0x7f2c, false], # East Asian ideograph
        0x2f3a5e => [0x8e6e, false], # East Asian ideograph
        0x234d7a => [0x97b2, false], # East Asian ideograph
        0x2d5836 => [0x89e6, false], # East Asian ideograph
        0x6f5834 => [0xc9d9, false], # Korean hangul
        0x22465b => [0x6c19, false], # East Asian ideograph
        0x216032 => [0x7ae0, false], # East Asian ideograph
        0x4b372c => [0x5606, false], # East Asian ideograph
        0x234d7c => [0x97b4, false], # East Asian ideograph
        0x213827 => [0x5783, false], # East Asian ideograph
        0x224d7d => [0x6fbc, false], # East Asian ideograph
        0x213828 => [0x576a, false], # East Asian ideograph
        0x235b67 => [0x9daa, false], # East Asian ideograph
        0x4b3773 => [0x56e3, false], # East Asian ideograph
        0x213829 => [0x5769, false], # East Asian ideograph
        0x34782a => [0x90c5, false], # East Asian ideograph
        0x284d49 => [0x6da0, false], # East Asian ideograph
        0x213f35 => [0x6162, false], # East Asian ideograph
        0x21382b => [0x5761, false], # East Asian ideograph
        0x4b5946 => [0x8a33, false], # East Asian ideograph
        0x21382c => [0x5764, false], # East Asian ideograph
        0x27383e => [0x57a9, false], # East Asian ideograph
        0x6f586c => [0xcb59, false], # Korean hangul
        0x6f543d => [0xc31c, false], # Korean hangul
        0x4b382e => [0x57c0, false], # East Asian ideograph
        0x23382f => [0x8da1, false], # East Asian ideograph
        0x693c32 => [0x9d2b, false], # East Asian ideograph
        0x215121 => [0x7d17, false], # East Asian ideograph
        0x215122 => [0x7d0d, false], # East Asian ideograph (variant of 455122 which maps to 7D0D)
        0x215123 => [0x7d1a, false], # East Asian ideograph
        0x215124 => [0x7d19, false], # East Asian ideograph
        0x215125 => [0x7d1b, false], # East Asian ideograph
        0x215126 => [0x7d46, false], # East Asian ideograph
        0x213831 => [0x578b, false], # East Asian ideograph
        0x215128 => [0x7d3c, false], # East Asian ideograph
        0x215129 => [0x7d2e, false], # East Asian ideograph
        0x21512a => [0x7d39, false], # East Asian ideograph
        0x27512b => [0x7ec4, false], # East Asian ideograph
        0x21512c => [0x7d30, false], # East Asian ideograph
        0x21512d => [0x7d33, false], # East Asian ideograph
        0x21512e => [0x7d2f, false], # East Asian ideograph
        0x27512f => [0x7ecc, false], # East Asian ideograph
        0x275130 => [0x7ec8, false], # East Asian ideograph
        0x275131 => [0x7edf, false], # East Asian ideograph
        0x275132 => [0x7ede, false], # East Asian ideograph
        0x215133 => [0x7d68, false], # East Asian ideograph
        0x275134 => [0x7ed3, false], # East Asian ideograph
        0x215135 => [0x7d2b, false], # East Asian ideograph
        0x215136 => [0x7d62, false], # East Asian ideograph
        0x215137 => [0x7d76, false], # East Asian ideograph
        0x215138 => [0x7d61, false], # East Asian ideograph
        0x275139 => [0x7ed9, false], # East Asian ideograph
        0x21513a => [0x7d6e, false], # East Asian ideograph
        0x21513b => [0x7d72, false], # East Asian ideograph
        0x27513c => [0x7ecf, false], # East Asian ideograph
        0x21513d => [0x7d91, false], # East Asian ideograph
        0x21513e => [0x7d79, false], # East Asian ideograph
        0x21513f => [0x7d8f, false], # East Asian ideograph
        0x275140 => [0x7ed1, false], # East Asian ideograph
        0x275141 => [0x7efc, false], # East Asian ideograph
        0x225142 => [0x70f7, false], # East Asian ideograph
        0x215143 => [0x7db0, false], # East Asian ideograph
        0x275144 => [0x7d27, false], # East Asian ideograph
        0x275145 => [0x7eeb, false], # East Asian ideograph
        0x275146 => [0x7f00, false], # East Asian ideograph
        0x215147 => [0x7dba, false], # East Asian ideograph
        0x275148 => [0x7f51, false], # East Asian ideograph
        0x275149 => [0x7eb2, false], # East Asian ideograph
        0x22514a => [0x7110, false], # East Asian ideograph
        0x21514b => [0x7db5, false], # East Asian ideograph
        0x21514c => [0x7da0, false], # East Asian ideograph
        0x27514d => [0x7ef8, false], # East Asian ideograph
        0x27514e => [0x7ef4, false], # East Asian ideograph
        0x27514f => [0x7ef5, false], # East Asian ideograph
        0x275150 => [0x7eb6, false], # East Asian ideograph
        0x275151 => [0x7f01, false], # East Asian ideograph
        0x215152 => [0x7de0, false], # East Asian ideograph
        0x235153 => [0x9933, false], # East Asian ideograph
        0x235154 => [0x9942, false], # East Asian ideograph (variant of 4D5154 which maps to 9942)
        0x275155 => [0x7eea, false], # East Asian ideograph
        0x215156 => [0x7dd8, false], # East Asian ideograph
        0x275157 => [0x7f05, false], # East Asian ideograph
        0x275158 => [0x7f09, false], # East Asian ideograph
        0x275159 => [0x7f13, false], # East Asian ideograph
        0x27515a => [0x7f18, false], # East Asian ideograph
        0x21515b => [0x7de8, false], # East Asian ideograph
        0x21515c => [0x7dda, false], # East Asian ideograph
        0x27515d => [0x7f0d, false], # East Asian ideograph
        0x27515e => [0x7f0e, false], # East Asian ideograph
        0x27515f => [0x7f23, false], # East Asian ideograph
        0x275160 => [0x7f22, false], # East Asian ideograph
        0x275161 => [0x8426, false], # East Asian ideograph
        0x275162 => [0x7f1a, false], # East Asian ideograph
        0x215163 => [0x7dfb, false], # East Asian ideograph
        0x275164 => [0x53bf, false], # East Asian ideograph (variant of 455164 which maps to 53BF)
        0x215165 => [0x7e2e, false], # East Asian ideograph
        0x215166 => [0x7e3e, false], # East Asian ideograph
        0x275167 => [0x7f2a, false], # East Asian ideograph
        0x275168 => [0x7f15, false], # East Asian ideograph
        0x215169 => [0x7e32, false], # East Asian ideograph
        0x23516a => [0x9948, false], # East Asian ideograph
        0x21516b => [0x7e41, false], # East Asian ideograph
        0x23516c => [0x9947, false], # East Asian ideograph
        0x23516d => [0x9949, false], # East Asian ideograph
        0x21516e => [0x7e31, false], # East Asian ideograph
        0x22516f => [0x713f, false], # East Asian ideograph
        0x235170 => [0x9943, false], # East Asian ideograph
        0x275171 => [0x7ec7, false], # East Asian ideograph
        0x215172 => [0x7e61, false], # East Asian ideograph
        0x235173 => [0x994e, false], # East Asian ideograph
        0x235174 => [0x9950, false], # East Asian ideograph
        0x215175 => [0x7e6b, false], # East Asian ideograph
        0x215176 => [0x7e69, false], # East Asian ideograph
        0x215177 => [0x7e6d, false], # East Asian ideograph
        0x215178 => [0x7e79, false], # East Asian ideograph
        0x215179 => [0x7e6a, false], # East Asian ideograph
        0x21517a => [0x8fae, false], # East Asian ideograph
        0x27517b => [0x7f24, false], # East Asian ideograph
        0x21517c => [0x7e82, false], # East Asian ideograph
        0x21517d => [0x7e7c, false], # East Asian ideograph
        0x21517e => [0x7e8f, false], # East Asian ideograph
        0x6f5947 => [0xcce4, false], # Korean hangul
        0x227841 => [0x80ef, false], # East Asian ideograph
        0x217144 => [0x5581, false], # East Asian ideograph
        0x4b3774 => [0x56f3, false], # East Asian ideograph
        0x235729 => [0x9b8e, false], # East Asian ideograph
        0x287321 => [0x7f26, false], # East Asian ideograph
        0x6f5c7d => [0xd5d2, false], # Korean hangul
        0x6f583a => [0xc9e0, false], # Korean hangul
        0x216038 => [0x9802, false], # East Asian ideograph
        0x213844 => [0x5831, false], # East Asian ideograph
        0x4b594b => [0x5909, false], # East Asian ideograph
        0x6f5d72 => [0xd763, false], # Korean hangul
        0x4c3b31 => [0x6798, false], # East Asian ideograph
        0x213845 => [0x582f, false], # East Asian ideograph
        0x213a30 => [0x5abc, false], # East Asian ideograph
        0x6f5c7e => [0xd5d8, false], # Korean hangul
        0x213848 => [0x5830, false], # East Asian ideograph
        0x274638 => [0x6b7c, false], # East Asian ideograph
        0x213849 => [0x5824, false], # East Asian ideograph
        0x21384a => [0x5834, false], # East Asian ideograph
        0x21784b => [0x58c6, false], # East Asian ideograph
        0x394042 => [0x646d, false], # East Asian ideograph
        0x284b28 => [0x6d48, false], # East Asian ideograph
        0x22384c => [0x665e, false], # East Asian ideograph
        0x69525d => [0x53fa, false], # East Asian ideograph
        0x705d46 => [0x841c, false], # East Asian ideograph
        0x27384d => [0x6d82, false], # East Asian ideograph
        0x21603a => [0x9805, false], # East Asian ideograph
        0x234a62 => [0x967e, false], # East Asian ideograph
        0x213158 => [0x4fd1, false], # East Asian ideograph
        0x223850 => [0x667e, false], # East Asian ideograph
        0x21387c => [0x5919, false], # East Asian ideograph
        0x213851 => [0x584c, false], # East Asian ideograph
        0x213852 => [0x585a, false], # East Asian ideograph
        0x213853 => [0x586d, false], # East Asian ideograph
        0x6f5276 => [0xc0dc, false], # Korean hangul
        0x217854 => [0x58d2, false], # East Asian ideograph
        0x213855 => [0x5862, false], # East Asian ideograph
        0x335b70 => [0x5ef8, false], # East Asian ideograph
        0x213f4e => [0x61c7, false], # East Asian ideograph
        0x273856 => [0x5757, false], # East Asian ideograph
        0x6f4e33 => [0xb5a8, false], # Korean hangul
        0x6f583e => [0xc9e7, false], # Korean hangul
        0x22687e => [0x7a2d, false], # East Asian ideograph
        0x4b3b79 => [0x5d8c, false], # East Asian ideograph
        0x217428 => [0x5704, false], # East Asian ideograph
        0x334621 => [0x8b99, false], # East Asian ideograph
        0x233859 => [0x8daf, false], # East Asian ideograph
        0x21385a => [0x588a, false], # East Asian ideograph
        0x215221 => [0x7e8c, false], # East Asian ideograph
        0x275222 => [0x7f28, false], # East Asian ideograph
        0x215223 => [0x7e96, false], # East Asian ideograph
        0x215224 => [0x7e9c, false], # East Asian ideograph
        0x6f5225 => [0xbe75, false], # Korean hangul
        0x215226 => [0x7f38, false], # East Asian ideograph
        0x215227 => [0x7f3a, false], # East Asian ideograph
        0x225228 => [0x7135, false], # East Asian ideograph
        0x235229 => [0x995d, false], # East Asian ideograph
        0x6f522a => [0xbe84, false], # Korean hangul
        0x21522b => [0x7f50, false], # East Asian ideograph
        0x21522c => [0x7f55, false], # East Asian ideograph
        0x21522d => [0x7f54, false], # East Asian ideograph
        0x21522e => [0x7f5f, false], # East Asian ideograph
        0x21522f => [0x7f72, false], # East Asian ideograph
        0x215230 => [0x7f6e, false], # East Asian ideograph
        0x224223 => [0x6a89, false], # East Asian ideograph
        0x215232 => [0x7f6a, false], # East Asian ideograph
        0x215233 => [0x7f70, false], # East Asian ideograph
        0x215234 => [0x7f75, false], # East Asian ideograph
        0x215235 => [0x7f77, false], # East Asian ideograph
        0x215236 => [0x7f79, false], # East Asian ideograph
        0x215237 => [0x7f85, false], # East Asian ideograph
        0x275238 => [0x7f81, false], # East Asian ideograph
        0x215239 => [0x7f8a, false], # East Asian ideograph
        0x21523a => [0x7f8c, false], # East Asian ideograph
        0x21523b => [0x7f8e, false], # East Asian ideograph
        0x21523c => [0x7f94, false], # East Asian ideograph
        0x21523d => [0x7f9e, false], # East Asian ideograph
        0x21523e => [0x7f9a, false], # East Asian ideograph
        0x21523f => [0x5584, false], # East Asian ideograph
        0x215240 => [0x7fa8, false], # East Asian ideograph
        0x215241 => [0x7fa4, false], # East Asian ideograph
        0x235242 => [0x99aa, false], # East Asian ideograph
        0x215243 => [0x7faf, false], # East Asian ideograph
        0x215244 => [0x7fb2, false], # East Asian ideograph
        0x215245 => [0x7fb6, false], # East Asian ideograph
        0x215246 => [0x7fb8, false], # East Asian ideograph
        0x215247 => [0x7fb9, false], # East Asian ideograph
        0x215248 => [0x7fbd, false], # East Asian ideograph
        0x235249 => [0x99b5, false], # East Asian ideograph
        0x21524a => [0x7fc5, false], # East Asian ideograph
        0x21524b => [0x7fc1, false], # East Asian ideograph
        0x21524c => [0x7fcc, false], # East Asian ideograph
        0x21524d => [0x7fd2, false], # East Asian ideograph
        0x21524e => [0x7fce, false], # East Asian ideograph (variant of 4B524E which maps to 7FCE)
        0x21524f => [0x7fd4, false], # East Asian ideograph
        0x215250 => [0x7fd5, false], # East Asian ideograph
        0x215251 => [0x7fe0, false], # East Asian ideograph
        0x215252 => [0x7fe1, false], # East Asian ideograph
        0x215253 => [0x7fdf, false], # East Asian ideograph
        0x235254 => [0x99bd, false], # East Asian ideograph
        0x215255 => [0x7ff0, false], # East Asian ideograph
        0x215256 => [0x7ff3, false], # East Asian ideograph
        0x215257 => [0x7ffc, false], # East Asian ideograph
        0x215258 => [0x7ff9, false], # East Asian ideograph
        0x215259 => [0x7ffb, false], # East Asian ideograph
        0x21525a => [0x7ff1, false], # East Asian ideograph
        0x21525b => [0x8000, false], # East Asian ideograph
        0x22525c => [0x7143, false], # East Asian ideograph
        0x21525d => [0x8003, false], # East Asian ideograph
        0x21525e => [0x8006, false], # East Asian ideograph
        0x21525f => [0x8005, false], # East Asian ideograph
        0x215260 => [0x800c, false], # East Asian ideograph
        0x215261 => [0x8010, false], # East Asian ideograph
        0x215262 => [0x800d, false], # East Asian ideograph
        0x215263 => [0x8012, false], # East Asian ideograph
        0x215264 => [0x8015, false], # East Asian ideograph
        0x215265 => [0x8018, false], # East Asian ideograph
        0x215266 => [0x8019, false], # East Asian ideograph
        0x215267 => [0x8017, false], # East Asian ideograph
        0x215268 => [0x801c, false], # East Asian ideograph
        0x235269 => [0x99d8, false], # East Asian ideograph
        0x21526a => [0x8036, false], # East Asian ideograph
        0x21526b => [0x803f, false], # East Asian ideograph
        0x21526c => [0x803d, false], # East Asian ideograph
        0x21526d => [0x804a, false], # East Asian ideograph
        0x21526e => [0x8046, false], # East Asian ideograph
        0x21526f => [0x8056, false], # East Asian ideograph
        0x215270 => [0x8058, false], # East Asian ideograph
        0x215271 => [0x805e, false], # East Asian ideograph
        0x215272 => [0x805a, false], # East Asian ideograph
        0x215273 => [0x8071, false], # East Asian ideograph
        0x215274 => [0x8072, false], # East Asian ideograph
        0x215275 => [0x8073, false], # East Asian ideograph
        0x215276 => [0x8070, false], # East Asian ideograph
        0x215277 => [0x806f, false], # East Asian ideograph
        0x215278 => [0x8077, false], # East Asian ideograph
        0x275279 => [0x8042, false], # East Asian ideograph
        0x23527a => [0x99f0, false], # East Asian ideograph
        0x27527b => [0x542c, false], # East Asian ideograph
        0x21527c => [0x807f, false], # East Asian ideograph
        0x6f527c => [0xc0f4, false], # Korean hangul
        0x21527e => [0x8084, false], # East Asian ideograph
        0x21386b => [0x58d9, false], # East Asian ideograph
        0x6f5842 => [0xc9f0, false], # Korean hangul
        0x27386c => [0x5792, false], # East Asian ideograph
        0x6f5a23 => [0xcead, false], # Korean hangul
        0x21386d => [0x58df, false], # East Asian ideograph
        0x27386e => [0x574f, false], # East Asian ideograph
        0x4b3850 => [0x5861, false], # East Asian ideograph (variant of 213850)
        0x23395c => [0x8e1e, false], # East Asian ideograph
        0x235732 => [0x9b98, false], # East Asian ideograph
        0x6f4e34 => [0xb5ab, false], # Korean hangul
        0x213870 => [0x58e4, false], # East Asian ideograph
        0x276065 => [0x9965, false], # East Asian ideograph
        0x4b3b7e => [0x5d15, false], # East Asian ideograph
        0x273871 => [0x575d, false], # East Asian ideograph
        0x6f5d76 => [0xd770, false], # Korean hangul
        0x223872 => [0x6693, false], # East Asian ideograph
        0x233873 => [0x8dbf, false], # East Asian ideograph
        0x273874 => [0x58ee, false], # East Asian ideograph
        0x343875 => [0x5fde, false], # East Asian ideograph
        0x284d58 => [0x6ca9, false], # East Asian ideograph
        0x705c43 => [0x82ca, false], # East Asian ideograph
        0x223876 => [0x6690, false], # East Asian ideograph
        0x276321 => [0x9f7f, false], # East Asian ideograph
        0x213877 => [0x58fd, false], # East Asian ideograph
        0x276322 => [0x9f83, false], # East Asian ideograph
        0x4d5d49 => [0x9e81, false], # East Asian ideograph
        0x226323 => [0x77b6, false], # East Asian ideograph
        0x213879 => [0x5914, false], # East Asian ideograph
        0x276324 => [0x9f84, false], # East Asian ideograph
        0x2e765f => [0x8037, false], # East Asian ideograph
        0x21387a => [0x5915, false], # East Asian ideograph
        0x284d59 => [0x6ed7, false], # East Asian ideograph
        0x276325 => [0x9f88, false], # East Asian ideograph
        0x6f542c => [0xc2f1, false], # Korean hangul
        0x213e2a => [0x6035, false], # East Asian ideograph (variant of 4B3E2A which maps to 6035)
        0x6f5675 => [0xc78a, false], # Korean hangul
        0x276326 => [0x9f87, false], # East Asian ideograph
        0x22787c => [0x8153, false], # East Asian ideograph
        0x216327 => [0x9f6c, false], # East Asian ideograph
        0x21387d => [0x591a, false], # East Asian ideograph
        0x276328 => [0x9f8a, false], # East Asian ideograph
        0x2d517d => [0x7d99, false], # East Asian ideograph
        0x4b484a => [0x6e13, false], # East Asian ideograph
        0x2d3272 => [0x706e, false], # East Asian ideograph
        0x232329 => [0x845c, false], # East Asian ideograph
        0x6f5846 => [0xc9fc, false], # Korean hangul
        0x27632a => [0x9f8b, false], # East Asian ideograph
        0x234a6c => [0x9689, false], # East Asian ideograph
        0x22632b => [0x77b9, false], # East Asian ideograph
        0x453051 => [0x8d30, false], # East Asian ideograph
        0x6f4b7d => [0xb124, false], # Korean hangul
        0x21632c => [0x9f94, false], # East Asian ideograph
        0x27632d => [0x9f9f, false], # East Asian ideograph
        0x4b484b => [0x51d6, false], # East Asian ideograph
        0x235736 => [0x9b9f, false], # East Asian ideograph
        0x6f5847 => [0xca00, false], # Korean hangul
        0x6f5427 => [0xc2e4, false], # Korean hangul
        0x275321 => [0x8083, false], # East Asian ideograph
        0x215322 => [0x8087, false], # East Asian ideograph
        0x215323 => [0x8089, false], # East Asian ideograph
        0x235324 => [0x9a02, false], # East Asian ideograph
        0x215325 => [0x808c, false], # East Asian ideograph
        0x215326 => [0x8093, false], # East Asian ideograph
        0x215327 => [0x809d, false], # East Asian ideograph
        0x215328 => [0x8098, false], # East Asian ideograph
        0x215329 => [0x809b, false], # East Asian ideograph
        0x21532a => [0x809a, false], # East Asian ideograph
        0x22532b => [0x7180, false], # East Asian ideograph
        0x22532c => [0x7189, false], # East Asian ideograph
        0x21532d => [0x80aa, false], # East Asian ideograph
        0x21532e => [0x80ba, false], # East Asian ideograph
        0x21532f => [0x80a5, false], # East Asian ideograph
        0x235330 => [0x99fb, false], # East Asian ideograph
        0x235331 => [0x99fd, false], # East Asian ideograph
        0x215332 => [0x80b1, false], # East Asian ideograph
        0x225333 => [0x7196, false], # East Asian ideograph
        0x215334 => [0x80a1, false], # East Asian ideograph
        0x215335 => [0x80a9, false], # East Asian ideograph
        0x27495d => [0x4e4c, false], # East Asian ideograph
        0x215337 => [0x80d6, false], # East Asian ideograph
        0x215338 => [0x80cc, false], # East Asian ideograph
        0x215339 => [0x80e5, false], # East Asian ideograph
        0x21533a => [0x80da, false], # East Asian ideograph
        0x21533b => [0x80e1, false], # East Asian ideograph
        0x21533c => [0x80c3, false], # East Asian ideograph
        0x21533d => [0x80db, false], # East Asian ideograph
        0x21533e => [0x80c4, false], # East Asian ideograph
        0x21533f => [0x80ce, false], # East Asian ideograph
        0x215340 => [0x80de, false], # East Asian ideograph
        0x215341 => [0x80e4, false], # East Asian ideograph
        0x215342 => [0x80f0, false], # East Asian ideograph
        0x215343 => [0x8102, false], # East Asian ideograph
        0x215344 => [0x8105, false], # East Asian ideograph
        0x215345 => [0x80f1, false], # East Asian ideograph
        0x215346 => [0x80f4, false], # East Asian ideograph
        0x215347 => [0x80ed, false], # East Asian ideograph
        0x235348 => [0x9a10, false], # East Asian ideograph
        0x215349 => [0x8106, false], # East Asian ideograph
        0x21534a => [0x80f3, false], # East Asian ideograph
        0x21534b => [0x80f8, false], # East Asian ideograph
        0x23534c => [0x9a24, false], # East Asian ideograph
        0x21534d => [0x8108, false], # East Asian ideograph
        0x21534e => [0x812b, false], # East Asian ideograph
        0x21534f => [0x812f, false], # East Asian ideograph
        0x215350 => [0x8116, false], # East Asian ideograph
        0x225351 => [0x71a4, false], # East Asian ideograph
        0x215352 => [0x8129, false], # East Asian ideograph
        0x215353 => [0x8155, false], # East Asian ideograph
        0x215354 => [0x8154, false], # East Asian ideograph
        0x215355 => [0x814b, false], # East Asian ideograph
        0x215356 => [0x8151, false], # East Asian ideograph
        0x215357 => [0x8150, false], # East Asian ideograph
        0x215358 => [0x814e, false], # East Asian ideograph
        0x275359 => [0x80c0, false], # East Asian ideograph
        0x21535a => [0x8146, false], # East Asian ideograph
        0x21535b => [0x813e, false], # East Asian ideograph
        0x21535c => [0x8171, false], # East Asian ideograph
        0x21535d => [0x8170, false], # East Asian ideograph
        0x21535e => [0x8178, false], # East Asian ideograph
        0x21535f => [0x8165, false], # East Asian ideograph
        0x215360 => [0x816e, false], # East Asian ideograph
        0x215361 => [0x8173, false], # East Asian ideograph
        0x275362 => [0x80bf, false], # East Asian ideograph
        0x215363 => [0x8179, false], # East Asian ideograph
        0x215364 => [0x817a, false], # East Asian ideograph
        0x215365 => [0x8166, false], # East Asian ideograph
        0x215366 => [0x8180, false], # East Asian ideograph
        0x225367 => [0x71d1, false], # East Asian ideograph
        0x215368 => [0x817f, false], # East Asian ideograph
        0x215369 => [0x818a, false], # East Asian ideograph
        0x21536a => [0x8188, false], # East Asian ideograph
        0x21536b => [0x819d, false], # East Asian ideograph
        0x21536c => [0x81a0, false], # East Asian ideograph
        0x22536d => [0x71ca, false], # East Asian ideograph
        0x21536e => [0x819a, false], # East Asian ideograph
        0x21536f => [0x819c, false], # East Asian ideograph
        0x215370 => [0x81b3, false], # East Asian ideograph
        0x275371 => [0x817b, false], # East Asian ideograph
        0x215372 => [0x81a8, false], # East Asian ideograph
        0x215373 => [0x81c6, false], # East Asian ideograph
        0x215374 => [0x81ba, false], # East Asian ideograph
        0x215375 => [0x81c3, false], # East Asian ideograph
        0x215376 => [0x81c0, false], # East Asian ideograph
        0x215377 => [0x81c2, false], # East Asian ideograph
        0x275378 => [0x8113, false], # East Asian ideograph
        0x275379 => [0x80c6, false], # East Asian ideograph
        0x27537a => [0x8138, false], # East Asian ideograph
        0x27537b => [0x810d, false], # East Asian ideograph
        0x21537c => [0x81cd, false], # East Asian ideograph
        0x27537d => [0x8191, false], # East Asian ideograph
        0x27537e => [0x814a, false], # East Asian ideograph
        0x234156 => [0x91bf, false], # East Asian ideograph
        0x276b79 => [0x523f, false], # East Asian ideograph
        0x6f584b => [0xca0c, false], # Korean hangul
        0x2d3b3f => [0x5c02, false], # East Asian ideograph
        0x21313b => [0x4f43, false], # East Asian ideograph
        0x2d615a => [0x8ec6, false], # East Asian ideograph
        0x6f2526 => [0x3161, false], # Korean hangul
        0x276b7a => [0x523d, false], # East Asian ideograph
        0x6f584c => [0xca0d, false], # Korean hangul
        0x69543a => [0x57aa, false], # East Asian ideograph
        0x232349 => [0x8497, false], # East Asian ideograph
        0x6f5279 => [0xc0e5, false], # Korean hangul
        0x274c33 => [0x6bd5, false], # East Asian ideograph
        0x213168 => [0x4fc4, false], # East Asian ideograph
        0x22234b => [0x5c8d, false], # East Asian ideograph
        0x213f51 => [0x61e3, false], # East Asian ideograph
        0x2d3279 => [0x514e, false], # East Asian ideograph
        0x6f4e36 => [0xb5b1, false], # Korean hangul
        0x6f2471 => [0x3149, false], # Korean hangul
        0x6f584d => [0xca18, false], # Korean hangul
        0x224674 => [0x6c3f, false], # East Asian ideograph
        0x6f4929 => [0xac80, false], # Korean hangul
        0x6f5164 => [0xbdd4, false], # Korean hangul
        0x217e21 => [0x5b5b, false], # East Asian ideograph
        0x213169 => [0x4fc2, false], # East Asian ideograph
        0x217158 => [0x55cc, false], # East Asian ideograph
        0x233967 => [0x8e27, false], # East Asian ideograph
        0x4b594a => [0x8aad, false], # East Asian ideograph
        0x6f5158 => [0xbd80, false], # Korean hangul
        0x6f584e => [0xca4c, false], # Korean hangul
        0x6f7721 => [0xad35, false], # Korean hangul
        0x213e33 => [0x6025, false], # East Asian ideograph
        0x295a28 => [0x9e28, false], # East Asian ideograph
        0x2d4b72 => [0x7506, false], # East Asian ideograph
        0x6f584f => [0xca4d, false], # Korean hangul
        0x232358 => [0x84b9, false], # East Asian ideograph
        0x347431 => [0x58dc, false], # East Asian ideograph
        0x21715a => [0x55db, false], # East Asian ideograph
        0x233969 => [0x8e18, false], # East Asian ideograph
        0x215421 => [0x81da, false], # East Asian ideograph
        0x235422 => [0x9a4d, false], # East Asian ideograph
        0x215423 => [0x81e3, false], # East Asian ideograph
        0x235424 => [0x9a52, false], # East Asian ideograph
        0x275425 => [0x4e34, false], # East Asian ideograph
        0x215426 => [0x81ea, false], # East Asian ideograph
        0x215427 => [0x81ec, false], # East Asian ideograph
        0x215428 => [0x81ed, false], # East Asian ideograph
        0x215429 => [0x81f3, false], # East Asian ideograph
        0x22542a => [0x71de, false], # East Asian ideograph
        0x21542b => [0x81fa, false], # East Asian ideograph
        0x21542c => [0x81fb, false], # East Asian ideograph
        0x21542d => [0x81fc, false], # East Asian ideograph
        0x21542e => [0x81fe, false], # East Asian ideograph
        0x21542f => [0x8200, false], # East Asian ideograph
        0x215430 => [0x8202, false], # East Asian ideograph
        0x215431 => [0x8205, false], # East Asian ideograph
        0x215432 => [0x8207, false], # East Asian ideograph
        0x275433 => [0x5174, false], # East Asian ideograph
        0x275434 => [0x4e3e, false], # East Asian ideograph
        0x215435 => [0x820a, false], # East Asian ideograph
        0x215436 => [0x820c, false], # East Asian ideograph
        0x215437 => [0x820d, false], # East Asian ideograph
        0x215438 => [0x8210, false], # East Asian ideograph
        0x215439 => [0x8212, false], # East Asian ideograph
        0x23543a => [0x9a6b, false], # East Asian ideograph
        0x21543b => [0x821b, false], # East Asian ideograph
        0x21543c => [0x821c, false], # East Asian ideograph
        0x21543d => [0x821e, false], # East Asian ideograph
        0x21543e => [0x821f, false], # East Asian ideograph
        0x21543f => [0x8222, false], # East Asian ideograph
        0x215440 => [0x822a, false], # East Asian ideograph
        0x235441 => [0x9aab, false], # East Asian ideograph
        0x215442 => [0x822c, false], # East Asian ideograph
        0x215443 => [0x8228, false], # East Asian ideograph
        0x215444 => [0x8237, false], # East Asian ideograph
        0x215445 => [0x8235, false], # East Asian ideograph
        0x215446 => [0x8239, false], # East Asian ideograph
        0x215447 => [0x8236, false], # East Asian ideograph
        0x215448 => [0x8247, false], # East Asian ideograph
        0x215449 => [0x8258, false], # East Asian ideograph
        0x21544a => [0x8259, false], # East Asian ideograph
        0x21544b => [0x8266, false], # East Asian ideograph
        0x21544c => [0x826e, false], # East Asian ideograph
        0x21544d => [0x826f, false], # East Asian ideograph
        0x21544e => [0x8271, false], # East Asian ideograph
        0x21544f => [0x8272, false], # East Asian ideograph
        0x215450 => [0x827e, false], # East Asian ideograph
        0x215451 => [0x8292, false], # East Asian ideograph
        0x215452 => [0x828b, false], # East Asian ideograph
        0x215453 => [0x828d, false], # East Asian ideograph
        0x215454 => [0x82b3, false], # East Asian ideograph
        0x215455 => [0x829d, false], # East Asian ideograph
        0x215456 => [0x8299, false], # East Asian ideograph
        0x215457 => [0x82bd, false], # East Asian ideograph
        0x215458 => [0x82ad, false], # East Asian ideograph
        0x215459 => [0x82ac, false], # East Asian ideograph
        0x21545a => [0x82a5, false], # East Asian ideograph
        0x21545b => [0x829f, false], # East Asian ideograph
        0x27545c => [0x520d, false], # East Asian ideograph
        0x21545d => [0x82b1, false], # East Asian ideograph
        0x21545e => [0x82b9, false], # East Asian ideograph
        0x69545f => [0x58e5, false], # East Asian ideograph
        0x215460 => [0x82e7, false], # East Asian ideograph
        0x215461 => [0x8305, false], # East Asian ideograph
        0x215462 => [0x8309, false], # East Asian ideograph
        0x215463 => [0x82e3, false], # East Asian ideograph
        0x215464 => [0x82db, false], # East Asian ideograph
        0x215465 => [0x82e6, false], # East Asian ideograph
        0x215466 => [0x8304, false], # East Asian ideograph
        0x215467 => [0x82e5, false], # East Asian ideograph
        0x215468 => [0x8302, false], # East Asian ideograph
        0x215469 => [0x82dc, false], # East Asian ideograph
        0x21546a => [0x82d7, false], # East Asian ideograph
        0x21546b => [0x82f1, false], # East Asian ideograph
        0x21546c => [0x8301, false], # East Asian ideograph
        0x23546d => [0x9ad6, false], # East Asian ideograph
        0x21546e => [0x82d4, false], # East Asian ideograph
        0x21546f => [0x82d1, false], # East Asian ideograph
        0x215470 => [0x82de, false], # East Asian ideograph
        0x215471 => [0x82df, false], # East Asian ideograph
        0x215472 => [0x832b, false], # East Asian ideograph
        0x215473 => [0x8352, false], # East Asian ideograph
        0x235474 => [0x9adf, false], # East Asian ideograph
        0x215475 => [0x8338, false], # East Asian ideograph
        0x215476 => [0x8354, false], # East Asian ideograph
        0x235477 => [0x9ae2, false], # East Asian ideograph
        0x215478 => [0x8349, false], # East Asian ideograph
        0x215479 => [0x8335, false], # East Asian ideograph
        0x21547a => [0x8334, false], # East Asian ideograph
        0x21547b => [0x8336, false], # East Asian ideograph
        0x21547c => [0x8331, false], # East Asian ideograph
        0x21547d => [0x8340, false], # East Asian ideograph
        0x21547e => [0x8317, false], # East Asian ideograph
        0x6f5853 => [0xca5d, false], # Korean hangul
        0x295166 => [0x9969, false], # East Asian ideograph
        0x234a79 => [0x9696, false], # East Asian ideograph
        0x226450 => [0x7826, false], # East Asian ideograph
        0x6f5d73 => [0xd765, false], # Korean hangul
        0x6f4b35 => [0xb014, false], # Korean hangul
        0x2d6162 => [0x9a0c, false], # East Asian ideograph
        0x6f5872 => [0xcbe7, false], # Korean hangul
        0x21316f => [0x4fef, false], # East Asian ideograph
        0x4b4858 => [0x6e80, false], # East Asian ideograph
        0x6f5854 => [0xca61, false], # Korean hangul
        0x222370 => [0x5cd5, false], # East Asian ideograph
        0x22467b => [0x6c62, false], # East Asian ideograph
        0x213e39 => [0x6063, false], # East Asian ideograph
        0x6f7648 => [0xe8ba, false], # Korean hangul
        0x4b374c => [0x5662, false], # East Asian ideograph
        0x29594f => [0x9ce2, false], # East Asian ideograph
        0x696373 => [0x7b02, false], # East Asian ideograph
        0x295a70 => [0x9e48, false], # East Asian ideograph
        0x6f5364 => [0xc22f, false], # Korean hangul
        0x27496a => [0x70bc, false], # East Asian ideograph
        0x6f5855 => [0xca84, false], # Korean hangul
        0x292375 => [0x83b3, false], # East Asian ideograph
        0x22467c => [0x6c4a, false], # East Asian ideograph
        0x224e21 => [0x6faa, false], # East Asian ideograph
        0x6f4e22 => [0xb541, false], # Korean hangul
        0x6f4e23 => [0xb543, false], # Korean hangul
        0x225346 => [0x719e, false], # East Asian ideograph
        0x222379 => [0x5c8d, false], # East Asian ideograph (not in Unicode)
        0x234e24 => [0x97b3, false], # East Asian ideograph
        0x6f5856 => [0xca98, false], # Korean hangul
        0x224e25 => [0x6fbf, false], # East Asian ideograph
        0x6f527b => [0xc0ec, false], # Korean hangul
        0x224e26 => [0x6fc7, false], # East Asian ideograph
        0x275a78 => [0x8e52, false], # East Asian ideograph
        0x274e27 => [0x77eb, false], # East Asian ideograph
        0x213172 => [0x5025, false], # East Asian ideograph
        0x6f4e28 => [0xb54d, false], # Korean hangul
        0x27574a => [0x51b2, false], # East Asian ideograph (duplicate simplified)
        0x227e23 => [0x83a6, false], # East Asian ideograph
        0x234e29 => [0x97b9, false], # East Asian ideograph
        0x6f5857 => [0xcabc, false], # Korean hangul
        0x29516a => [0x9990, false], # East Asian ideograph
        0x51356a => [0x8bc3, false], # East Asian ideograph
        0x6f4e2b => [0xb55c, false], # Korean hangul
        0x6f594d => [0xcd19, false], # Korean hangul
        0x6f5d5c => [0xd700, false], # Korean hangul
        0x6f4e2c => [0xb55d, false], # Korean hangul
        0x213173 => [0x5011, false], # East Asian ideograph
        0x4b613f => [0x9a08, false], # East Asian ideograph
        0x214e2d => [0x65ab, false], # East Asian ideograph
        0x224e2e => [0x6f5e, false], # East Asian ideograph
        0x6f5858 => [0xcabd, false], # Korean hangul
        0x6f546a => [0xc4f0, false], # Korean hangul
        0x224e2f => [0x6fc8, false], # East Asian ideograph
        0x2d5763 => [0x88e1, false], # East Asian ideograph
        0x235521 => [0x9ae7, false], # East Asian ideograph
        0x215522 => [0x834f, false], # East Asian ideograph
        0x215523 => [0x8339, false], # East Asian ideograph
        0x215524 => [0x838e, false], # East Asian ideograph
        0x215525 => [0x8398, false], # East Asian ideograph
        0x215526 => [0x839e, false], # East Asian ideograph
        0x215527 => [0x8378, false], # East Asian ideograph
        0x215528 => [0x83a2, false], # East Asian ideograph
        0x225529 => [0x7225, false], # East Asian ideograph
        0x22552a => [0x7226, false], # East Asian ideograph
        0x21552b => [0x83ab, false], # East Asian ideograph
        0x21552c => [0x8392, false], # East Asian ideograph (variant of 4B552C which maps to 8392)
        0x21552d => [0x838a, false], # East Asian ideograph
        0x21552e => [0x8393, false], # East Asian ideograph
        0x21552f => [0x83a0, false], # East Asian ideograph
        0x215530 => [0x8389, false], # East Asian ideograph
        0x215531 => [0x8377, false], # East Asian ideograph
        0x215532 => [0x837c, false], # East Asian ideograph
        0x215533 => [0x837b, false], # East Asian ideograph
        0x215534 => [0x840d, false], # East Asian ideograph
        0x215535 => [0x83e0, false], # East Asian ideograph
        0x215536 => [0x83e9, false], # East Asian ideograph
        0x6f5537 => [0xc57d, false], # Korean hangul
        0x215538 => [0x8403, false], # East Asian ideograph
        0x215539 => [0x83c5, false], # East Asian ideograph
        0x21553a => [0x83c1, false], # East Asian ideograph
        0x21553b => [0x840b, false], # East Asian ideograph
        0x21553c => [0x83ef, false], # East Asian ideograph
        0x6f553d => [0xc58f, false], # Korean hangul
        0x21553e => [0x83f1, false], # East Asian ideograph
        0x21553f => [0x83bd, false], # East Asian ideograph
        0x6f5540 => [0xc595, false], # Korean hangul
        0x235541 => [0x9b05, false], # East Asian ideograph
        0x215542 => [0x840c, false], # East Asian ideograph
        0x225543 => [0x7241, false], # East Asian ideograph
        0x215544 => [0x83dc, false], # East Asian ideograph
        0x215545 => [0x83ca, false], # East Asian ideograph
        0x215546 => [0x83f2, false], # East Asian ideograph
        0x215547 => [0x840e, false], # East Asian ideograph
        0x215548 => [0x8404, false], # East Asian ideograph
        0x215549 => [0x843d, false], # East Asian ideograph
        0x21554a => [0x8482, false], # East Asian ideograph
        0x21554b => [0x8431, false], # East Asian ideograph
        0x21554c => [0x8475, false], # East Asian ideograph
        0x21554d => [0x8466, false], # East Asian ideograph
        0x21554e => [0x8457, false], # East Asian ideograph
        0x22554f => [0x7250, false], # East Asian ideograph
        0x215550 => [0x846c, false], # East Asian ideograph
        0x214e38 => [0x7843, false], # East Asian ideograph
        0x215552 => [0x845b, false], # East Asian ideograph
        0x215553 => [0x8477, false], # East Asian ideograph
        0x215554 => [0x843c, false], # East Asian ideograph
        0x215555 => [0x8435, false], # East Asian ideograph
        0x225556 => [0x725a, false], # East Asian ideograph
        0x215557 => [0x8463, false], # East Asian ideograph
        0x215558 => [0x8469, false], # East Asian ideograph
        0x225559 => [0x7263, false], # East Asian ideograph
        0x21555a => [0x84b2, false], # East Asian ideograph
        0x21555b => [0x849e, false], # East Asian ideograph
        0x21555c => [0x84bf, false], # East Asian ideograph
        0x21555d => [0x84c6, false], # East Asian ideograph
        0x21555e => [0x84c4, false], # East Asian ideograph
        0x21555f => [0x84c9, false], # East Asian ideograph
        0x215560 => [0x849c, false], # East Asian ideograph
        0x215561 => [0x84cb, false], # East Asian ideograph
        0x215562 => [0x84b8, false], # East Asian ideograph
        0x275563 => [0x836a, false], # East Asian ideograph
        0x275564 => [0x82ce, false], # East Asian ideograph
        0x215565 => [0x84d3, false], # East Asian ideograph
        0x225566 => [0x7276, false], # East Asian ideograph
        0x215567 => [0x84bc, false], # East Asian ideograph
        0x225568 => [0x7277, false], # East Asian ideograph
        0x215569 => [0x84ff, false], # East Asian ideograph
        0x21556a => [0x8517, false], # East Asian ideograph
        0x22556b => [0x727e, false], # East Asian ideograph
        0x21556c => [0x84ee, false], # East Asian ideograph
        0x21556d => [0x852c, false], # East Asian ideograph
        0x27556e => [0x836b, false], # East Asian ideograph
        0x21556f => [0x8513, false], # East Asian ideograph
        0x6f5570 => [0xc61b, false], # Korean hangul
        0x215571 => [0x8523, false], # East Asian ideograph
        0x215572 => [0x8521, false], # East Asian ideograph
        0x275573 => [0x535c, false], # East Asian ideograph
        0x225574 => [0x7289, false], # East Asian ideograph
        0x215575 => [0x8525, false], # East Asian ideograph
        0x235576 => [0x9b2f, false], # East Asian ideograph
        0x215577 => [0x854a, false], # East Asian ideograph
        0x215578 => [0x8559, false], # East Asian ideograph
        0x215579 => [0x8548, false], # East Asian ideograph
        0x21557a => [0x8568, false], # East Asian ideograph
        0x21557b => [0x8543, false], # East Asian ideograph
        0x21557c => [0x856a, false], # East Asian ideograph
        0x21557d => [0x8549, false], # East Asian ideograph
        0x21557e => [0x8584, false], # East Asian ideograph
        0x224e40 => [0x6fa5, false], # East Asian ideograph
        0x224e41 => [0x6fb0, false], # East Asian ideograph
        0x4b6048 => [0x981a, false], # East Asian ideograph
        0x224e42 => [0x6fae, false], # East Asian ideograph
        0x2f585c => [0x9c51, false], # Unrelated variant of EACC 235945 which maps to 9C51
        0x224e43 => [0x6fd9, false], # East Asian ideograph
        0x276260 => [0x4e48, false], # East Asian ideograph
        0x21393f => [0x5969, false], # East Asian ideograph
        0x224e44 => [0x6fda, false], # East Asian ideograph
        0x274e45 => [0x7855, false], # East Asian ideograph
        0x6f5b3c => [0xd1b0, false], # Korean hangul
        0x273422 => [0x5218, false], # East Asian ideograph
        0x6f4e46 => [0xb664, false], # Korean hangul
        0x286b7c => [0x7b15, false], # East Asian ideograph
        0x22316c => [0x636c, false], # East Asian ideograph
        0x6f4e47 => [0xb69c, false], # Korean hangul
        0x6f585d => [0xcad1, false], # Korean hangul
        0x295170 => [0x998d, false], # East Asian ideograph
        0x274e49 => [0x786e, false], # East Asian ideograph
        0x6f4c4c => [0xb220, false], # Korean hangul
        0x6f4e4a => [0xb6ab, false], # Korean hangul
        0x213179 => [0x5006, false], # East Asian ideograph
        0x234e4b => [0x97ce, false], # East Asian ideograph
        0x2d753a => [0x9654, false], # East Asian ideograph
        0x2d5321 => [0x7c9b, false], # East Asian ideograph
        0x274e4c => [0x7801, false], # East Asian ideograph
        0x3f3078 => [0x5023, false], # East Asian ideograph
        0x6f585e => [0xcad2, false], # Korean hangul
        0x6f4e4d => [0xb6f0, false], # Korean hangul
        0x234e4e => [0x97d0, false], # East Asian ideograph
        0x4b552c => [0x8392, false], # East Asian ideograph
        0x29546d => [0x9acb, false], # East Asian ideograph
        0x333564 => [0x5415, false], # East Asian ideograph
        0x275154 => [0x7ec3, false], # East Asian ideograph
        0x224e50 => [0x6fd4, false], # East Asian ideograph
        0x213930 => [0x5949, false], # East Asian ideograph
        0x234e51 => [0x97d4, false], # East Asian ideograph
        0x6f585f => [0xcad3, false], # Korean hangul
        0x295172 => [0x9994, false], # East Asian ideograph
        0x21605d => [0x98b1, false], # East Asian ideograph
        0x213e44 => [0x606c, false], # East Asian ideograph
        0x274e53 => [0x7816, false], # East Asian ideograph
        0x234642 => [0x93a7, false], # East Asian ideograph
        0x234e54 => [0x97d9, false], # East Asian ideograph
        0x69245c => [0x307c, false], # Hiragana letter BO
        0x6f4e55 => [0xb72f, false], # Korean hangul
        0x224e56 => [0x6fe9, false], # East Asian ideograph
        0x6f5860 => [0xcad8, false], # Korean hangul
        0x224e57 => [0x6ff8, false], # East Asian ideograph
        0x4b3758 => [0x56a5, false], # East Asian ideograph (variant of 213758 which maps to 56A5)
        0x274e58 => [0x77f6, false], # East Asian ideograph
        0x214e59 => [0x790e, false], # East Asian ideograph
        0x274e5a => [0x788d, false], # East Asian ideograph
        0x2d5c40 => [0x5fa8, false], # East Asian ideograph
        0x215621 => [0x85aa, false], # East Asian ideograph
        0x215622 => [0x856d, false], # East Asian ideograph
        0x235623 => [0x9b37, false], # East Asian ideograph
        0x275624 => [0x59dc, false], # East Asian ideograph
        0x215625 => [0x857e, false], # East Asian ideograph
        0x215626 => [0x8594, false], # East Asian ideograph
        0x215627 => [0x859c, false], # East Asian ideograph
        0x225628 => [0x728f, false], # East Asian ideograph
        0x215629 => [0x85cd, false], # East Asian ideograph (variant of 4B5629 which maps to 85CD)
        0x27562a => [0x8428, false], # East Asian ideograph
        0x21562b => [0x85cf, false], # East Asian ideograph
        0x21562c => [0x85af, false], # East Asian ideograph
        0x21562d => [0x85d0, false], # East Asian ideograph
        0x27562e => [0x501f, false], # East Asian ideograph
        0x214e5d => [0x792a, false], # East Asian ideograph
        0x215630 => [0x85e9, false], # East Asian ideograph
        0x215631 => [0x85dd, false], # East Asian ideograph
        0x275632 => [0x85ae, false], # East Asian ideograph
        0x215633 => [0x85e4, false], # East Asian ideograph
        0x215634 => [0x85d5, false], # East Asian ideograph
        0x224e5e => [0x6fee, false], # East Asian ideograph
        0x215636 => [0x85fb, false], # East Asian ideograph
        0x215637 => [0x85f9, false], # East Asian ideograph
        0x215638 => [0x8611, false], # East Asian ideograph
        0x215639 => [0x85fa, false], # East Asian ideograph
        0x27563a => [0x82a6, false], # East Asian ideograph
        0x27563b => [0x82f9, false], # East Asian ideograph
        0x27563c => [0x82cf, false], # East Asian ideograph
        0x27563d => [0x8574, false], # East Asian ideograph
        0x27563e => [0x5170, false], # East Asian ideograph
        0x21563f => [0x8617, false], # East Asian ideograph
        0x215640 => [0x861a, false], # East Asian ideograph
        0x215641 => [0x8638, false], # East Asian ideograph
        0x275642 => [0x841d, false], # East Asian ideograph
        0x215643 => [0x864e, false], # East Asian ideograph
        0x215644 => [0x8650, false], # East Asian ideograph
        0x215645 => [0x8654, false], # East Asian ideograph
        0x215646 => [0x5f6a, false], # East Asian ideograph
        0x215647 => [0x8655, false], # East Asian ideograph
        0x275648 => [0x864f, false], # East Asian ideograph
        0x215649 => [0x865b, false], # East Asian ideograph
        0x27564a => [0x53f7, false], # East Asian ideograph
        0x21564b => [0x865e, false], # East Asian ideograph
        0x22564c => [0x72ab, false], # East Asian ideograph
        0x224e62 => [0x6ff0, false], # East Asian ideograph
        0x22564e => [0x72b0, false], # East Asian ideograph
        0x21564f => [0x8679, false], # East Asian ideograph
        0x215650 => [0x86a9, false], # East Asian ideograph
        0x215651 => [0x86aa, false], # East Asian ideograph
        0x215652 => [0x868a, false], # East Asian ideograph
        0x215653 => [0x8693, false], # East Asian ideograph
        0x215654 => [0x86a4, false], # East Asian ideograph
        0x215655 => [0x868c, false], # East Asian ideograph
        0x215656 => [0x86a3, false], # East Asian ideograph
        0x215657 => [0x86c0, false], # East Asian ideograph
        0x215658 => [0x86c7, false], # East Asian ideograph
        0x215659 => [0x86b5, false], # East Asian ideograph
        0x27565a => [0x65e6, false], # East Asian ideograph
        0x21565b => [0x86b6, false], # East Asian ideograph
        0x21565c => [0x86c4, false], # East Asian ideograph
        0x21565d => [0x86c6, false], # East Asian ideograph
        0x21565e => [0x86b1, false], # East Asian ideograph
        0x21565f => [0x86af, false], # East Asian ideograph
        0x225660 => [0x72d6, false], # East Asian ideograph
        0x215661 => [0x86d9, false], # East Asian ideograph
        0x215662 => [0x86ed, false], # East Asian ideograph
        0x215663 => [0x86d4, false], # East Asian ideograph
        0x225664 => [0x72d2, false], # East Asian ideograph
        0x224e66 => [0x7005, false], # East Asian ideograph
        0x215666 => [0x86fb, false], # East Asian ideograph
        0x225667 => [0x72c9, false], # East Asian ideograph
        0x215668 => [0x8707, false], # East Asian ideograph
        0x215669 => [0x8703, false], # East Asian ideograph
        0x21566a => [0x8708, false], # East Asian ideograph
        0x214e67 => [0x7955, false], # East Asian ideograph
        0x21566c => [0x86fe, false], # East Asian ideograph
        0x21566d => [0x8713, false], # East Asian ideograph
        0x21566e => [0x8702, false], # East Asian ideograph
        0x21566f => [0x871c, false], # East Asian ideograph
        0x215670 => [0x873f, false], # East Asian ideograph
        0x215671 => [0x873b, false], # East Asian ideograph
        0x215672 => [0x8722, false], # East Asian ideograph
        0x225673 => [0x72e8, false], # East Asian ideograph
        0x215674 => [0x8734, false], # East Asian ideograph
        0x215675 => [0x8718, false], # East Asian ideograph
        0x215676 => [0x8755, false], # East Asian ideograph
        0x215677 => [0x8760, false], # East Asian ideograph
        0x215678 => [0x8776, false], # East Asian ideograph
        0x225679 => [0x72e5, false], # East Asian ideograph
        0x27567a => [0x867e, false], # East Asian ideograph
        0x21567b => [0x8778, false], # East Asian ideograph
        0x21567c => [0x8768, false], # East Asian ideograph
        0x21567d => [0x874c, false], # East Asian ideograph
        0x22567e => [0x72fa, false], # East Asian ideograph
        0x6f4e6b => [0xb790, false], # Korean hangul
        0x6f5421 => [0xc2b7, false], # Korean hangul
        0x234e6c => [0x97f5, false], # East Asian ideograph
        0x6f5339 => [0xc151, false], # Korean hangul
        0x6f4e6d => [0xb797, false], # Korean hangul
        0x69245d => [0x307d, false], # Hiragana letter PO
        0x6f4e6e => [0xb798, false], # Korean hangul
        0x274e6f => [0x53ea, false], # East Asian ideograph (duplicate simplified)
        0x6f5865 => [0xcb20, false], # Korean hangul
        0x6f4e70 => [0xb79c, false], # Korean hangul
        0x6f5422 => [0xc2b9, false], # Korean hangul
        0x6f5a2a => [0xcef7, false], # Korean hangul
        0x6f4e71 => [0xb7a0, false], # Korean hangul
        0x234648 => [0x939a, false], # East Asian ideograph
        0x213441 => [0x5305, false], # East Asian ideograph
        0x224e72 => [0x7026, false], # East Asian ideograph
        0x214e73 => [0x797a, false], # East Asian ideograph
        0x286c58 => [0x7ba7, false], # East Asian ideograph
        0x6f5633 => [0xc68d, false], # Korean hangul
        0x6f4e3b => [0xb5bc, false], # Korean hangul
        0x6f5866 => [0xcb21, false], # Korean hangul
        0x395179 => [0x7d75, false], # East Asian ideograph
        0x6f4e76 => [0xb7ad, false], # Korean hangul
        0x213921 => [0x5920, false], # East Asian ideograph
        0x274e77 => [0x7978, false], # East Asian ideograph
        0x27785e => [0x5786, false], # East Asian ideograph
        0x213922 => [0x5924, false], # East Asian ideograph
        0x274e78 => [0x796f, false], # East Asian ideograph
        0x213923 => [0x5925, false], # East Asian ideograph
        0x234e79 => [0x9807, false], # East Asian ideograph
        0x273924 => [0x68a6, false], # East Asian ideograph
        0x6f5867 => [0xcb41, false], # Korean hangul
        0x213f37 => [0x6155, false], # East Asian ideograph
        0x6f4e7a => [0xb7ec, false], # Korean hangul
        0x6f4d61 => [0xb4d0, false], # Korean hangul
        0x226464 => [0x7876, false], # East Asian ideograph
        0x274e7b => [0x7985, false], # East Asian ideograph
        0x69482b => [0x7560, false], # East Asian ideograph
        0x274e7c => [0x793c, false], # East Asian ideograph
        0x4b4f3c => [0x79f0, false], # East Asian ideograph (variant of 274F3C which maps to 79F0)
        0x213927 => [0x592b, false], # East Asian ideograph
        0x274e7d => [0x7977, false], # East Asian ideograph
        0x2d5323 => [0x5b8d, false], # East Asian ideograph
        0x234e7e => [0x980d, false], # East Asian ideograph
        0x2d3929 => [0x6b80, false], # East Asian ideograph
        0x213376 => [0x5274, false], # East Asian ideograph
        0x6f5868 => [0xcb48, false], # Korean hangul
        0x6f5433 => [0xc300, false], # Korean hangul
        0x216066 => [0x98e7, false], # East Asian ideograph
        0x21392b => [0x5931, false], # East Asian ideograph
        0x3f304c => [0x5e79, false], # East Asian ideograph
        0x2e3a26 => [0x661d, false], # East Asian ideograph
        0x225359 => [0x71b4, false], # East Asian ideograph
        0x21392e => [0x593e, false], # East Asian ideograph
        0x6f5869 => [0xcb49, false], # Korean hangul
        0x396b33 => [0x5259, false], # East Asian ideograph (not in Unicode)
        0x216067 => [0x98e9, false], # East Asian ideograph
        0x235721 => [0x9b83, false], # East Asian ideograph
        0x215722 => [0x8783, false], # East Asian ideograph
        0x215723 => [0x8782, false], # East Asian ideograph
        0x275724 => [0x8424, false], # East Asian ideograph
        0x225725 => [0x72fe, false], # East Asian ideograph
        0x215726 => [0x878d, false], # East Asian ideograph
        0x215727 => [0x879f, false], # East Asian ideograph
        0x215728 => [0x87d1, false], # East Asian ideograph
        0x215729 => [0x87c0, false], # East Asian ideograph
        0x21572a => [0x87ab, false], # East Asian ideograph
        0x23572b => [0x9b90, false], # East Asian ideograph
        0x27572c => [0x877c, false], # East Asian ideograph
        0x22572d => [0x7301, false], # East Asian ideograph
        0x22572e => [0x72f3, false], # East Asian ideograph
        0x23572f => [0x9b97, false], # East Asian ideograph
        0x215730 => [0x87c6, false], # East Asian ideograph
        0x215731 => [0x87cb, false], # East Asian ideograph
        0x215732 => [0x87ef, false], # East Asian ideograph
        0x215733 => [0x87f2, false], # East Asian ideograph
        0x215734 => [0x87ec, false], # East Asian ideograph
        0x225735 => [0x730b, false], # East Asian ideograph
        0x225736 => [0x7317, false], # East Asian ideograph
        0x215737 => [0x880d, false], # East Asian ideograph
        0x215738 => [0x87f9, false], # East Asian ideograph
        0x215739 => [0x8814, false], # East Asian ideograph
        0x21573a => [0x8815, false], # East Asian ideograph
        0x22573b => [0x7307, false], # East Asian ideograph
        0x23573c => [0x9bad, false], # East Asian ideograph
        0x23573d => [0x9b9a, false], # East Asian ideograph
        0x22573e => [0x7318, false], # East Asian ideograph
        0x27573f => [0x86ca, false], # East Asian ideograph
        0x215740 => [0x8839, false], # East Asian ideograph
        0x275741 => [0x8695, false], # East Asian ideograph
        0x215742 => [0x883b, false], # East Asian ideograph
        0x235743 => [0x9b99, false], # East Asian ideograph
        0x215744 => [0x884c, false], # East Asian ideograph
        0x215745 => [0x884d, false], # East Asian ideograph
        0x225746 => [0x7331, false], # East Asian ideograph
        0x215747 => [0x8857, false], # East Asian ideograph
        0x215748 => [0x8859, false], # East Asian ideograph
        0x225749 => [0x7338, false], # East Asian ideograph
        0x22574a => [0x7322, false], # East Asian ideograph
        0x21574b => [0x8861, false], # East Asian ideograph
        0x22574c => [0x7332, false], # East Asian ideograph
        0x22574d => [0x732c, false], # East Asian ideograph
        0x22574e => [0x7327, false], # East Asian ideograph
        0x22574f => [0x732b, false], # East Asian ideograph
        0x215750 => [0x886b, false], # East Asian ideograph
        0x215751 => [0x8882, false], # East Asian ideograph
        0x225752 => [0x732f, false], # East Asian ideograph
        0x215753 => [0x8870, false], # East Asian ideograph
        0x215754 => [0x8877, false], # East Asian ideograph
        0x225755 => [0x7328, false], # East Asian ideograph
        0x235756 => [0x9bc7, false], # East Asian ideograph
        0x215757 => [0x8892, false], # East Asian ideograph
        0x215758 => [0x8896, false], # East Asian ideograph
        0x235759 => [0x9bd2, false], # East Asian ideograph
        0x22575a => [0x7347, false], # East Asian ideograph
        0x22575b => [0x7348, false], # East Asian ideograph
        0x22575c => [0x7349, false], # East Asian ideograph
        0x23393a => [0x8de6, false], # East Asian ideograph
        0x21575e => [0x88b1, false], # East Asian ideograph
        0x23575f => [0x9bc1, false], # East Asian ideograph
        0x215760 => [0x88d9, false], # East Asian ideograph
        0x215761 => [0x88d8, false], # East Asian ideograph
        0x215762 => [0x88dc, false], # East Asian ideograph
        0x215763 => [0x88cf, false], # East Asian ideograph
        0x215764 => [0x88d4, false], # East Asian ideograph
        0x225765 => [0x7340, false], # East Asian ideograph
        0x215766 => [0x88d5, false], # East Asian ideograph
        0x215767 => [0x8902, false], # East Asian ideograph
        0x225768 => [0x734d, false], # East Asian ideograph
        0x215769 => [0x88f8, false], # East Asian ideograph
        0x21576a => [0x88f9, false], # East Asian ideograph
        0x21576b => [0x88f4, false], # East Asian ideograph
        0x23576c => [0x9bd3, false], # East Asian ideograph
        0x21576d => [0x88e8, false], # East Asian ideograph
        0x21576e => [0x891a, false], # East Asian ideograph
        0x21576f => [0x8910, false], # East Asian ideograph
        0x6f5770 => [0xc906, false], # Korean hangul
        0x215771 => [0x8913, false], # East Asian ideograph
        0x235772 => [0x9bc8, false], # East Asian ideograph
        0x215773 => [0x8932, false], # East Asian ideograph
        0x225774 => [0x735d, false], # East Asian ideograph
        0x215775 => [0x8925, false], # East Asian ideograph
        0x215776 => [0x892b, false], # East Asian ideograph
        0x235777 => [0x9bd7, false], # East Asian ideograph
        0x215778 => [0x8936, false], # East Asian ideograph
        0x225779 => [0x7360, false], # East Asian ideograph
        0x23577a => [0x9bd6, false], # East Asian ideograph
        0x21577b => [0x895f, false], # East Asian ideograph
        0x23577c => [0x9beb, false], # East Asian ideograph
        0x21577d => [0x8956, false], # East Asian ideograph
        0x22577e => [0x7362, false], # East Asian ideograph
        0x273940 => [0x593a, false], # East Asian ideograph
        0x223941 => [0x66aa, false], # East Asian ideograph
        0x232b33 => [0x874d, false], # East Asian ideograph
        0x2d506f => [0x7cfa, false], # East Asian ideograph
        0x6f586d => [0xcb5d, false], # Korean hangul
        0x6f7643 => [0xe8b5, false], # Korean hangul
        0x213943 => [0x5974, false], # East Asian ideograph
        0x213944 => [0x5976, false], # East Asian ideograph
        0x4b3322 => [0x5168, false], # East Asian ideograph (variant of 213322 which maps to 5168)
        0x27564c => [0x4e8f, false], # East Asian ideograph
        0x4b5e3d => [0x9421, false], # East Asian ideograph
        0x213946 => [0x5983, false], # East Asian ideograph
        0x6f5365 => [0xc231, false], # Korean hangul
        0x6f516d => [0xbe44, false], # Korean hangul
        0x213947 => [0x5978, false], # East Asian ideograph
        0x2d4c2d => [0x756e, false], # East Asian ideograph
        0x6f542b => [0xc2ef, false], # Korean hangul
        0x213e53 => [0x6089, false], # East Asian ideograph
        0x213949 => [0x5979, false], # East Asian ideograph
        0x21794b => [0x595c, false], # East Asian ideograph
        0x454e75 => [0x7984, false], # East Asian ideograph
        0x213c7d => [0x5ec2, false], # East Asian ideograph
        0x6f586f => [0xcbb8, false], # Korean hangul
        0x2d394d => [0x59ac, false], # East Asian ideograph
        0x217e43 => [0x5b93, false], # East Asian ideograph
        0x22394e => [0x66c8, false], # East Asian ideograph
        0x4b3324 => [0x634c, false], # East Asian ideograph (variant of 2D3324 which maps to 634C)
        0x21394f => [0x59a4, false], # East Asian ideograph
        0x213f58 => [0x61fa, false], # East Asian ideograph
        0x213950 => [0x59a3, false], # East Asian ideograph
        0x6f4e3d => [0xb5c4, false], # Korean hangul
        0x213951 => [0x5993, false], # East Asian ideograph
        0x2f5870 => [0x9c1b, false], # East Asian ideograph
        0x6f7646 => [0xe8b8, false], # Korean hangul
        0x213952 => [0x599e, false], # East Asian ideograph
        0x213e55 => [0x60a0, false], # East Asian ideograph
        0x4b3768 => [0x56d8, false], # East Asian ideograph
        0x213953 => [0x599d, false], # East Asian ideograph
        0x233954 => [0x8e23, false], # East Asian ideograph
        0x213955 => [0x59a5, false], # East Asian ideograph
        0x335760 => [0x88e0, false], # East Asian ideograph
        0x2d3956 => [0x59d9, false], # East Asian ideograph
        0x6f4f22 => [0xb7ff, false], # Korean hangul
        0x6f5871 => [0xcbe4, false], # Korean hangul
        0x6f7647 => [0xe8b9, false], # Korean hangul
        0x213957 => [0x5996, false], # East Asian ideograph
        0x213958 => [0x59be, false], # East Asian ideograph
        0x333642 => [0x8a92, false], # East Asian ideograph
        0x2d3f67 => [0x621e, false], # East Asian ideograph
        0x6f5878 => [0xcc1d, false], # Korean hangul
        0x4c7959 => [0x817d, false], # East Asian ideograph
        0x273437 => [0x80dc, false], # East Asian ideograph
        0x214f63 => [0x7ac7, false], # East Asian ideograph
        0x4b524e => [0x7fce, false], # East Asian ideograph
        0x21395a => [0x59ae, false], # East Asian ideograph
        0x3a4034 => [0x6855, false], # East Asian ideograph
        0x275821 => [0x889c, false], # East Asian ideograph
        0x275822 => [0x886c, false], # East Asian ideograph
        0x215823 => [0x8972, false], # East Asian ideograph
        0x215824 => [0x897f, false], # East Asian ideograph
        0x225825 => [0x7367, false], # East Asian ideograph
        0x215826 => [0x8983, false], # East Asian ideograph
        0x235827 => [0x9be4, false], # East Asian ideograph
        0x275828 => [0x89c1, false], # East Asian ideograph
        0x275829 => [0x89c4, false], # East Asian ideograph
        0x27582a => [0x89c5, false], # East Asian ideograph
        0x27582b => [0x89c6, false], # East Asian ideograph
        0x27582c => [0x4eb2, false], # East Asian ideograph
        0x27582d => [0x89ce, false], # East Asian ideograph
        0x21582e => [0x89ac, false], # East Asian ideograph
        0x27582f => [0x89d0, false], # East Asian ideograph
        0x215830 => [0x89ba, false], # East Asian ideograph
        0x275831 => [0x89c8, false], # East Asian ideograph
        0x215832 => [0x89c0, false], # East Asian ideograph
        0x215833 => [0x89d2, false], # East Asian ideograph
        0x235834 => [0x9bd4, false], # East Asian ideograph
        0x215835 => [0x89f4, false], # East Asian ideograph
        0x225836 => [0x737c, false], # East Asian ideograph
        0x215837 => [0x8a00, false], # East Asian ideograph
        0x215838 => [0x8a08, false], # East Asian ideograph
        0x215839 => [0x8a02, false], # East Asian ideograph
        0x21583a => [0x8a03, false], # East Asian ideograph
        0x21583b => [0x8a10, false], # East Asian ideograph
        0x21583c => [0x8a18, false], # East Asian ideograph
        0x21583d => [0x8a0e, false], # East Asian ideograph
        0x23583e => [0x9bff, false], # East Asian ideograph
        0x21583f => [0x8a15, false], # East Asian ideograph
        0x215840 => [0x8a0a, false], # East Asian ideograph
        0x275841 => [0x8bab, false], # East Asian ideograph
        0x225842 => [0x738e, false], # East Asian ideograph
        0x235843 => [0x9c06, false], # East Asian ideograph
        0x235844 => [0x9c15, false], # East Asian ideograph
        0x215845 => [0x8a23, false], # East Asian ideograph
        0x275846 => [0x8bb6, false], # East Asian ideograph
        0x225847 => [0x7392, false], # East Asian ideograph
        0x215848 => [0x8a31, false], # East Asian ideograph
        0x275849 => [0x8bbe, false], # East Asian ideograph
        0x27584a => [0x8bb9, false], # East Asian ideograph
        0x27584b => [0x8bbc, false], # East Asian ideograph
        0x27584c => [0x6ce8, false], # East Asian ideograph
        0x21584d => [0x8a60, false], # East Asian ideograph
        0x27584e => [0x8bc4, false], # East Asian ideograph
        0x27584f => [0x8bcd, false], # East Asian ideograph
        0x6f5850 => [0xca50, false], # Korean hangul
        0x215851 => [0x8a41, false], # East Asian ideograph
        0x235852 => [0x9c02, false], # East Asian ideograph
        0x215853 => [0x8a5b, false], # East Asian ideograph
        0x235854 => [0x9c10, false], # East Asian ideograph
        0x215855 => [0x8a46, false], # East Asian ideograph
        0x215856 => [0x8a34, false], # East Asian ideograph
        0x275857 => [0x8bca, false], # East Asian ideograph
        0x275858 => [0x8be7, false], # East Asian ideograph
        0x215859 => [0x8a72, false], # East Asian ideograph
        0x21585a => [0x8a73, false], # East Asian ideograph
        0x21585b => [0x8a66, false], # East Asian ideograph
        0x27585c => [0x8bd7, false], # East Asian ideograph
        0x27585d => [0x8bd8, false], # East Asian ideograph
        0x27585e => [0x8be3, false], # East Asian ideograph
        0x27585f => [0x8bd9, false], # East Asian ideograph
        0x275860 => [0x8bda, false], # East Asian ideograph
        0x215861 => [0x8a87, false], # East Asian ideograph
        0x275862 => [0x8bdb, false], # East Asian ideograph
        0x215863 => [0x8a6d, false], # East Asian ideograph
        0x215864 => [0x8a79, false], # East Asian ideograph
        0x275865 => [0x8be2, false], # East Asian ideograph
        0x275866 => [0x8bdd, false], # East Asian ideograph
        0x275867 => [0x8be0, false], # East Asian ideograph
        0x215868 => [0x8a6c, false], # East Asian ideograph
        0x235869 => [0x9c2f, false], # East Asian ideograph
        0x22586a => [0x73c2, false], # East Asian ideograph
        0x22586b => [0x73d0, false], # East Asian ideograph
        0x21586c => [0x8a9e, false], # East Asian ideograph
        0x21586d => [0x8a8c, false], # East Asian ideograph
        0x21586e => [0x8a93, false], # East Asian ideograph
        0x22586f => [0x73bf, false], # East Asian ideograph
        0x275870 => [0x8ba4, false], # East Asian ideograph
        0x275871 => [0x8bef, false], # East Asian ideograph
        0x275872 => [0x8bf2, false], # East Asian ideograph
        0x275873 => [0x8bf0, false], # East Asian ideograph
        0x275874 => [0x8bf1, false], # East Asian ideograph
        0x275875 => [0x8bf3, false], # East Asian ideograph
        0x215876 => [0x8abc, false], # East Asian ideograph
        0x275877 => [0x8c06, false], # East Asian ideograph
        0x275878 => [0x8c05, false], # East Asian ideograph
        0x215879 => [0x8ac7, false], # East Asian ideograph
        0x21587a => [0x8acb, false], # East Asian ideograph (variant of 4B587A which maps to 8ACB)
        0x27587b => [0x8bf8, false], # East Asian ideograph
        0x27587c => [0x8bfe, false], # East Asian ideograph
        0x27587d => [0x8c03, false], # East Asian ideograph
        0x23587e => [0x9c46, false], # East Asian ideograph
        0x6f5875 => [0xcc10, false], # Korean hangul
        0x6f764b => [0xe8bd, false], # Korean hangul
        0x21796b => [0x5998, false], # East Asian ideograph
        0x6f4b62 => [0xb0bb, false], # Korean hangul
        0x293c5a => [0x8f73, false], # East Asian ideograph
        0x2d396e => [0x4f84, false], # East Asian ideograph
        0x28732d => [0x7f2f, false], # East Asian ideograph
        0x4b6145 => [0x9a12, false], # East Asian ideograph
        0x21396f => [0x5a01, false], # East Asian ideograph
        0x2d4c35 => [0x7567, false], # East Asian ideograph
        0x2d3970 => [0x5a63, false], # East Asian ideograph
        0x213971 => [0x59e6, false], # East Asian ideograph
        0x6f5879 => [0xcc21, false], # Korean hangul
        0x213972 => [0x59da, false], # East Asian ideograph
        0x213973 => [0x5a11, false], # East Asian ideograph
        0x2d3974 => [0x5b43, false], # East Asian ideograph
        0x6f5877 => [0xcc1c, false], # Korean hangul
        0x6f764d => [0xe8bf, false], # Korean hangul
        0x6f5434 => [0xc308, false], # Korean hangul
        0x213e5c => [0x60b6, false], # East Asian ideograph
        0x4b376f => [0x56fd, false], # East Asian ideograph
        0x213976 => [0x5a1c, false], # East Asian ideograph
        0x692421 => [0x3041, false], # Hiragana letter small A
        0x213977 => [0x5a13, false], # East Asian ideograph
        0x2d5773 => [0x7d5d, false], # East Asian ideograph
        0x213978 => [0x59ec, false], # East Asian ideograph
        0x33354e => [0x608b, false], # East Asian ideograph
        0x213979 => [0x5a20, false], # East Asian ideograph
        0x216424 => [0x4e0f, false], # East Asian ideograph
        0x6f764e => [0xe8c0, false], # Korean hangul
        0x6f535c => [0xc219, false], # Korean hangul
        0x213e5d => [0x60d1, false], # East Asian ideograph
        0x2d397b => [0x5a31, false], # East Asian ideograph
        0x2d3f6e => [0x6226, false], # East Asian ideograph
        0x21397c => [0x5a0c, false], # East Asian ideograph
        0x692427 => [0x3047, false], # Hiragana letter small E
        0x6f5841 => [0xc9ef, false], # Korean hangul
        0x22797d => [0x81a6, false], # East Asian ideograph
        0x692428 => [0x3048, false], # Hiragana letter E
        0x21397e => [0x5a25, false], # East Asian ideograph
        0x222429 => [0x5cdd, false], # East Asian ideograph
        0x6f764f => [0xe8c1, false], # Korean hangul
        0x6f5436 => [0xc30b, false], # Korean hangul
        0x213e5e => [0x60b5, false], # East Asian ideograph
        0x33304c => [0x4e79, false], # East Asian ideograph
        0x215c34 => [0x904b, false], # East Asian ideograph
        0x2d3f6f => [0x622f, false], # East Asian ideograph
        0x27742e => [0x56f5, false], # East Asian ideograph
        0x216d41 => [0x534c, false], # East Asian ideograph
        0x6f587a => [0xcc22, false], # Korean hangul
        0x6f7650 => [0xe8c2, false], # Korean hangul
        0x6f5437 => [0xc30c, false], # Korean hangul
        0x69242f => [0x304f, false], # Hiragana letter KU
        0x4b3772 => [0x5186, false], # East Asian ideograph
        0x4b5631 => [0x82b8, false], # East Asian ideograph
        0x225921 => [0x73d3, false], # East Asian ideograph
        0x215922 => [0x8ab0, false], # East Asian ideograph
        0x215923 => [0x8a95, false], # East Asian ideograph
        0x215924 => [0x8ad6, false], # East Asian ideograph
        0x275925 => [0x8c1b, false], # East Asian ideograph
        0x235926 => [0x9c44, false], # East Asian ideograph
        0x215927 => [0x8aeb, false], # East Asian ideograph
        0x225928 => [0x73e5, false], # East Asian ideograph
        0x235929 => [0x9c39, false], # East Asian ideograph
        0x22592a => [0x73d9, false], # East Asian ideograph
        0x22592b => [0x73ef, false], # East Asian ideograph
        0x21592c => [0x8b01, false], # East Asian ideograph (variant of 2D592C which maps to 8B01)
        0x21592d => [0x8b02, false], # East Asian ideograph
        0x21592e => [0x8afe, false], # East Asian ideograph
        0x27592f => [0x8bbd, false], # East Asian ideograph
        0x235930 => [0x9c47, false], # East Asian ideograph
        0x215931 => [0x8b17, false], # East Asian ideograph
        0x225932 => [0x73d6, false], # East Asian ideograph
        0x215933 => [0x8b0e, false], # East Asian ideograph
        0x235934 => [0x9c37, false], # East Asian ideograph
        0x225935 => [0x73bc, false], # East Asian ideograph
        0x215936 => [0x8b21, false], # East Asian ideograph
        0x215937 => [0x8b04, false], # East Asian ideograph
        0x235938 => [0x9c52, false], # East Asian ideograph
        0x275939 => [0x8c28, false], # East Asian ideograph
        0x22593a => [0x73de, false], # East Asian ideograph
        0x23593b => [0x9c58, false], # East Asian ideograph
        0x22593c => [0x73e6, false], # East Asian ideograph
        0x21593d => [0x8b5c, false], # East Asian ideograph
        0x21593e => [0x8b4e, false], # East Asian ideograph
        0x21593f => [0x8b49, false], # East Asian ideograph
        0x275940 => [0x8c2d, false], # East Asian ideograph
        0x215941 => [0x8b41, false], # East Asian ideograph
        0x275942 => [0x8ba5, false], # East Asian ideograph
        0x215943 => [0x8b70, false], # East Asian ideograph
        0x215944 => [0x8b6c, false], # East Asian ideograph
        0x225945 => [0x73f6, false], # East Asian ideograph
        0x215946 => [0x8b6f, false], # East Asian ideograph
        0x225947 => [0x73fa, false], # East Asian ideograph
        0x275948 => [0x62a4, false], # East Asian ideograph
        0x215949 => [0x8b7d, false], # East Asian ideograph
        0x27594a => [0x8bfb, false], # East Asian ideograph
        0x21594b => [0x8b8a, false], # East Asian ideograph
        0x27594c => [0x8ba9, false], # East Asian ideograph
        0x21594d => [0x8b96, false], # East Asian ideograph
        0x21594e => [0x8b92, false], # East Asian ideograph
        0x23594f => [0x9c67, false], # East Asian ideograph
        0x6f5950 => [0xcd2c, false], # Korean hangul
        0x215951 => [0x8c41, false], # East Asian ideograph
        0x215952 => [0x8c3f, false], # East Asian ideograph
        0x215953 => [0x8c46, false], # East Asian ideograph
        0x225954 => [0x73f5, false], # East Asian ideograph
        0x235955 => [0x9c5f, false], # East Asian ideograph
        0x235956 => [0x9c60, false], # East Asian ideograph
        0x215957 => [0x8c4e, false], # East Asian ideograph
        0x235958 => [0x9c6d, false], # East Asian ideograph
        0x215959 => [0x8c54, false], # East Asian ideograph
        0x21595a => [0x8c5a, false], # East Asian ideograph
        0x23595b => [0x9c68, false], # East Asian ideograph
        0x22595c => [0x7407, false], # East Asian ideograph
        0x21595d => [0x8c6a, false], # East Asian ideograph
        0x22595e => [0x7412, false], # East Asian ideograph
        0x21595f => [0x8c6c, false], # East Asian ideograph
        0x215960 => [0x8c7a, false], # East Asian ideograph
        0x215961 => [0x8c79, false], # East Asian ideograph
        0x215962 => [0x8c82, false], # East Asian ideograph
        0x225963 => [0x743c, false], # East Asian ideograph
        0x215964 => [0x8c89, false], # East Asian ideograph
        0x215965 => [0x8c8d, false], # East Asian ideograph
        0x225966 => [0x742e, false], # East Asian ideograph
        0x225967 => [0x742f, false], # East Asian ideograph
        0x275968 => [0x8d1d, false], # East Asian ideograph
        0x225969 => [0x7414, false], # East Asian ideograph
        0x22596a => [0x742c, false], # East Asian ideograph
        0x27596b => [0x8d21, false], # East Asian ideograph
        0x27596c => [0x8d22, false], # East Asian ideograph
        0x27596d => [0x8d23, false], # East Asian ideograph
        0x22596e => [0x742b, false], # East Asian ideograph
        0x21596f => [0x8ca8, false], # East Asian ideograph
        0x225970 => [0x73f7, false], # East Asian ideograph
        0x225971 => [0x741a, false], # East Asian ideograph
        0x275972 => [0x8d29, false], # East Asian ideograph
        0x235973 => [0x9ce7, false], # East Asian ideograph
        0x235974 => [0x9cf0, false], # East Asian ideograph
        0x215975 => [0x8cbb, false], # East Asian ideograph
        0x215976 => [0x8cc1, false], # East Asian ideograph
        0x235977 => [0x9cf2, false], # East Asian ideograph
        0x225978 => [0x7416, false], # East Asian ideograph
        0x215979 => [0x8cbc, false], # East Asian ideograph
        0x22597a => [0x7426, false], # East Asian ideograph
        0x21597b => [0x8cb6, false], # East Asian ideograph
        0x21597c => [0x8cbd, false], # East Asian ideograph
        0x27597d => [0x8d37, false], # East Asian ideograph
        0x21597e => [0x8cbf, false], # East Asian ideograph
        0x222441 => [0x5cf4, false], # East Asian ideograph
        0x224f2b => [0x701e, false], # East Asian ideograph (variant of 4C4F2B which maps to 701E)
        0x6f587e => [0xcc28, false], # Korean hangul
        0x275e58 => [0x9602, false], # East Asian ideograph
        0x6f543b => [0xc315, false], # Korean hangul
        0x217e52 => [0x5ba7, false], # East Asian ideograph
        0x333573 => [0x8656, false], # East Asian ideograph
        0x222446 => [0x5cf1, false], # East Asian ideograph
        0x69243f => [0x305f, false], # Hiragana letter TA
        0x27613e => [0x9a8f, false], # East Asian ideograph
        0x6f5b69 => [0xd305, false], # Korean hangul
        0x2d5c2f => [0x8fe8, false], # East Asian ideograph
        0x2d4c3e => [0x758e, false], # East Asian ideograph
        0x6f4e74 => [0xb7ab, false], # Korean hangul
        0x6f7655 => [0xe8c7, false], # Korean hangul
        0x225f7b => [0x7657, false], # East Asian ideograph
        0x213e64 => [0x60d5, false], # East Asian ideograph
        0x234662 => [0x93f9, false], # East Asian ideograph
        0x696449 => [0x7c13, false], # East Asian ideograph
        0x276137 => [0x9a76, false], # East Asian ideograph
        0x69244a => [0x306a, false], # Hiragana letter NA
        0x4b6147 => [0x99c6, false], # East Asian ideograph
        0x333556 => [0x9a03, false], # East Asian ideograph
        0x69644c => [0x7c17, false], # East Asian ideograph
        0x6f4d66 => [0xb4e4, false], # Korean hangul
        0x213e65 => [0x60bc, false], # East Asian ideograph
        0x69644e => [0x7bf6, false], # East Asian ideograph
        0x6f587b => [0xcc27, false], # Korean hangul
        0x2d3b33 => [0x8a67, false], # East Asian ideograph
        0x214b2f => [0x7380, false], # East Asian ideograph
        0x6f5438 => [0xc30d, false], # Korean hangul
        0x6f543e => [0xc324, false], # Korean hangul
        0x6f7651 => [0xe8c3, false], # Korean hangul
        0x216452 => [0x4eb3, false], # East Asian ideograph
        0x294e43 => [0x97af, false], # East Asian ideograph
        0x234664 => [0x93c4, false], # East Asian ideograph
        0x342453 => [0x5cbd, false], # East Asian ideograph
        0x692454 => [0x3074, false], # Hiragana letter PI
        0x225372 => [0x71ba, false], # East Asian ideograph
        0x4b6455 => [0x4eb6, false], # East Asian ideograph
        0x6f4f7a => [0xb9f7, false], # Korean hangul
        0x6f543f => [0xc327, false], # Korean hangul
        0x4b503b => [0x7c12, false], # East Asian ideograph
        0x692457 => [0x3077, false], # Hiragana letter PU
        0x223e23 => [0x691a, false], # East Asian ideograph
        0x234222 => [0x91e4, false], # East Asian ideograph
        0x692459 => [0x3079, false], # Hiragana letter BE
        0x21645a => [0x4ebc, false], # East Asian ideograph
        0x4b4444 => [0x8988, false], # East Asian ideograph (Version J extension)
        0x215a21 => [0x8cc5, false], # East Asian ideograph
        0x275a22 => [0x8d44, false], # East Asian ideograph
        0x275a23 => [0x8d3c, false], # East Asian ideograph
        0x215a24 => [0x8cc8, false], # East Asian ideograph
        0x275a25 => [0x8d3f, false], # East Asian ideograph
        0x215a26 => [0x8cb2, false], # East Asian ideograph
        0x275a27 => [0x8d41, false], # East Asian ideograph
        0x225a28 => [0x7420, false], # East Asian ideograph
        0x275a29 => [0x5bbe, false], # East Asian ideograph
        0x275a2a => [0x8d48, false], # East Asian ideograph
        0x275a2b => [0x8d4a, false], # East Asian ideograph
        0x215a2c => [0x8ce0, false], # East Asian ideograph
        0x275a2d => [0x8d4b, false], # East Asian ideograph
        0x6f5a2e => [0xcf00, false], # Korean hangul
        0x275a2f => [0x5356, false], # East Asian ideograph
        0x235a30 => [0x9d25, false], # East Asian ideograph
        0x215a31 => [0x8ce4, false], # East Asian ideograph
        0x215a32 => [0x8cde, false], # East Asian ideograph
        0x275a33 => [0x8d50, false], # East Asian ideograph
        0x215a34 => [0x8cea, false], # East Asian ideograph
        0x275a35 => [0x8d4c, false], # East Asian ideograph
        0x215a36 => [0x8cf4, false], # East Asian ideograph
        0x215a37 => [0x8cfd, false], # East Asian ideograph
        0x275a38 => [0x8d5a, false], # East Asian ideograph
        0x275a39 => [0x8d58, false], # East Asian ideograph
        0x275a3a => [0x8d2d, false], # East Asian ideograph
        0x275a3b => [0x8d60, false], # East Asian ideograph
        0x275a3c => [0x8d5d, false], # East Asian ideograph
        0x275a3d => [0x8d5e, false], # East Asian ideograph
        0x275a3e => [0x8d62, false], # East Asian ideograph
        0x275a3f => [0x8d61, false], # East Asian ideograph
        0x275a40 => [0x8d43, false], # East Asian ideograph
        0x275a41 => [0x8d4e, false], # East Asian ideograph
        0x275a42 => [0x8d63, false], # East Asian ideograph
        0x215a43 => [0x8d64, false], # East Asian ideograph
        0x215a44 => [0x8d67, false], # East Asian ideograph
        0x215a45 => [0x8d66, false], # East Asian ideograph
        0x215a46 => [0x8d6b, false], # East Asian ideograph
        0x215a47 => [0x8d6d, false], # East Asian ideograph
        0x235a48 => [0x9d1f, false], # East Asian ideograph
        0x215a49 => [0x8d74, false], # East Asian ideograph
        0x215a4a => [0x8d73, false], # East Asian ideograph
        0x215a4b => [0x8d77, false], # East Asian ideograph
        0x215a4c => [0x8d85, false], # East Asian ideograph
        0x215a4d => [0x8d8a, false], # East Asian ideograph
        0x215a4e => [0x8d81, false], # East Asian ideograph
        0x275a4f => [0x8d75, false], # East Asian ideograph
        0x215a50 => [0x8d95, false], # East Asian ideograph
        0x215a51 => [0x8da3, false], # East Asian ideograph
        0x215a52 => [0x8d9f, false], # East Asian ideograph
        0x275a53 => [0x8d8b, false], # East Asian ideograph
        0x215a54 => [0x8db3, false], # East Asian ideograph
        0x215a55 => [0x8db4, false], # East Asian ideograph
        0x215a56 => [0x8dbe, false], # East Asian ideograph
        0x215a57 => [0x8dce, false], # East Asian ideograph
        0x215a58 => [0x8ddd, false], # East Asian ideograph
        0x214b33 => [0x738b, false], # East Asian ideograph
        0x215a5a => [0x8dcb, false], # East Asian ideograph
        0x215a5b => [0x8dda, false], # East Asian ideograph
        0x215a5c => [0x8dc6, false], # East Asian ideograph
        0x215a5d => [0x8dd1, false], # East Asian ideograph
        0x215a5e => [0x8dcc, false], # East Asian ideograph
        0x215a5f => [0x8de1, false], # East Asian ideograph
        0x215a60 => [0x8ddf, false], # East Asian ideograph
        0x215a61 => [0x8de8, false], # East Asian ideograph
        0x225a62 => [0x7473, false], # East Asian ideograph
        0x235a63 => [0x9d3e, false], # East Asian ideograph
        0x215a64 => [0x8dea, false], # East Asian ideograph
        0x215a65 => [0x8def, false], # East Asian ideograph
        0x215a66 => [0x8dfc, false], # East Asian ideograph
        0x215a67 => [0x8e2b, false], # East Asian ideograph
        0x235a68 => [0x9d42, false], # East Asian ideograph
        0x235a69 => [0x9d40, false], # East Asian ideograph
        0x215a6a => [0x8e1d, false], # East Asian ideograph
        0x215a6b => [0x8e0f, false], # East Asian ideograph
        0x215a6c => [0x8e29, false], # East Asian ideograph
        0x215a6d => [0x8e1f, false], # East Asian ideograph
        0x215a6e => [0x8e44, false], # East Asian ideograph
        0x215a6f => [0x8e31, false], # East Asian ideograph
        0x215a70 => [0x8e42, false], # East Asian ideograph
        0x215a71 => [0x8e34, false], # East Asian ideograph
        0x215a72 => [0x8e39, false], # East Asian ideograph
        0x215a73 => [0x8e35, false], # East Asian ideograph
        0x215a74 => [0x8e49, false], # East Asian ideograph
        0x235a75 => [0x9d53, false], # East Asian ideograph
        0x215a76 => [0x8e48, false], # East Asian ideograph
        0x215a77 => [0x8e4a, false], # East Asian ideograph
        0x215a78 => [0x8e63, false], # East Asian ideograph
        0x215a79 => [0x8e59, false], # East Asian ideograph
        0x215a7a => [0x8e66, false], # East Asian ideograph
        0x215a7b => [0x8e64, false], # East Asian ideograph
        0x215a7c => [0x8e72, false], # East Asian ideograph
        0x215a7d => [0x8e6c, false], # East Asian ideograph
        0x275a7e => [0x8df7, false], # East Asian ideograph
        0x6f5439 => [0xc313, false], # Korean hangul
        0x6f5443 => [0xc343, false], # Korean hangul
        0x22646b => [0x789a, false], # East Asian ideograph
        0x213a28 => [0x5a41, false], # East Asian ideograph
        0x6f2458 => [0x3134, false], # Korean hangul
        0x234226 => [0x922b, false], # East Asian ideograph
        0x27515c => [0x7ebf, false], # East Asian ideograph
        0x475222 => [0x9957, false], # East Asian ideograph
        0x6f246e => [0x3143, false], # Korean hangul
        0x335333 => [0x80bb, false], # East Asian ideograph
        0x6f2470 => [0x3146, false], # Korean hangul
        0x692471 => [0x3091, false], # Hiragana letter WE
        0x232472 => [0x852f, false], # East Asian ideograph
        0x6f2473 => [0x3150, false], # Korean hangul
        0x224f35 => [0x7021, false], # East Asian ideograph
        0x6f2474 => [0x3151, false], # Korean hangul
        0x6f5445 => [0xc368, false], # Korean hangul
        0x6f5a31 => [0xcf08, false], # Korean hangul
        0x6f2476 => [0x3153, false], # Korean hangul
        0x295729 => [0x9c87, false], # East Asian ideograph
        0x6f4f21 => [0xb7fd, false], # Korean hangul
        0x232477 => [0x84f7, false], # East Asian ideograph
        0x274f22 => [0x4e07, false], # East Asian ideograph
        0x234f23 => [0x980e, false], # East Asian ideograph
        0x6f4e42 => [0xb611, false], # Korean hangul
        0x224f24 => [0x7020, false], # East Asian ideograph
        0x6f5446 => [0xc369, false], # Korean hangul
        0x6f247a => [0x3157, false], # Korean hangul
        0x274f25 => [0x53b6, false], # East Asian ideograph
        0x4b333e => [0xf92e, false], # East Asian ideograph
        0x224f26 => [0x7027, false], # East Asian ideograph
        0x214f27 => [0x79c9, false], # East Asian ideograph
        0x22537a => [0x71bf, false], # East Asian ideograph
        0x22537c => [0x71b8, false], # East Asian ideograph
        0x29247d => [0x835c, false], # East Asian ideograph
        0x6f4f28 => [0xb80c, false], # Korean hangul
        0x212a38 => [0xe8e5, false], # EACC component character
        0x276775 => [0x4f65, false], # East Asian ideograph
        0x6f5447 => [0xc36c, false], # Korean hangul
        0x6f4f2a => [0xb818, false], # Korean hangul
        0x23422a => [0x9292, false], # East Asian ideograph
        0x2d5d56 => [0x920e, false], # East Asian ideograph
        0x234f2c => [0x9826, false], # East Asian ideograph
        0x6f5d5a => [0xd6fc, false], # Korean hangul
        0x234f2d => [0x981e, false], # East Asian ideograph
        0x6f7652 => [0xe8c4, false], # Korean hangul
        0x6f4f2e => [0xb824, false], # Korean hangul
        0x2d3c26 => [0x5d18, false], # East Asian ideograph
        0x6f5448 => [0xc370, false], # Korean hangul
        0x213e70 => [0x60fb, false], # East Asian ideograph
        0x224f2f => [0x702e, false], # East Asian ideograph
        0x225b21 => [0x7489, false], # East Asian ideograph
        0x225b22 => [0x747c, false], # East Asian ideograph
        0x215b23 => [0x8e82, false], # East Asian ideograph
        0x215b24 => [0x8e81, false], # East Asian ideograph
        0x215b25 => [0x8e87, false], # East Asian ideograph
        0x215b26 => [0x8e89, false], # East Asian ideograph
        0x214f31 => [0x79fb, false], # East Asian ideograph
        0x225b28 => [0x747e, false], # East Asian ideograph
        0x275b29 => [0x8dc3, false], # East Asian ideograph
        0x235b2a => [0x9d52, false], # East Asian ideograph
        0x215b2b => [0x8ea1, false], # East Asian ideograph
        0x235b2c => [0x9d77, false], # East Asian ideograph
        0x224f39 => [0x7018, false], # East Asian ideograph
        0x215b2e => [0x8eac, false], # East Asian ideograph
        0x215b2f => [0x8eb2, false], # East Asian ideograph
        0x225b30 => [0x747a, false], # East Asian ideograph
        0x215b31 => [0x8ec0, false], # East Asian ideograph
        0x215b32 => [0x8eca, false], # East Asian ideograph
        0x275b33 => [0x8f67, false], # East Asian ideograph
        0x215b34 => [0x8ecd, false], # East Asian ideograph
        0x215b35 => [0x8ecc, false], # East Asian ideograph
        0x215b36 => [0x8ed2, false], # East Asian ideograph
        0x215b37 => [0x8ed4, false], # East Asian ideograph
        0x215b38 => [0x8edf, false], # East Asian ideograph
        0x215b39 => [0x8edb, false], # East Asian ideograph
        0x215b3a => [0x8efb, false], # East Asian ideograph
        0x275b3b => [0x8f74, false], # East Asian ideograph
        0x215b3c => [0x8efc, false], # East Asian ideograph
        0x215b3d => [0x8f03, false], # East Asian ideograph
        0x225b3e => [0x747d, false], # East Asian ideograph
        0x235b3f => [0x9d78, false], # East Asian ideograph
        0x215b40 => [0x8f0a, false], # East Asian ideograph
        0x215b41 => [0x8f14, false], # East Asian ideograph
        0x235b42 => [0x9d7e, false], # East Asian ideograph
        0x215b43 => [0x8f15, false], # East Asian ideograph
        0x215b44 => [0x8f13, false], # East Asian ideograph
        0x215b45 => [0x8f26, false], # East Asian ideograph
        0x215b46 => [0x8f1b, false], # East Asian ideograph
        0x235b47 => [0x9d69, false], # East Asian ideograph
        0x215b48 => [0x8f1d, false], # East Asian ideograph
        0x215b49 => [0x8f29, false], # East Asian ideograph
        0x215b4a => [0x8f2a, false], # East Asian ideograph
        0x275b4b => [0x8f8e, false], # East Asian ideograph
        0x215b4c => [0x8f3b, false], # East Asian ideograph
        0x215b4d => [0x8f2f, false], # East Asian ideograph
        0x215b4e => [0x8f38, false], # East Asian ideograph
        0x235b4f => [0x9d83, false], # East Asian ideograph
        0x215b50 => [0x8f3e, false], # East Asian ideograph
        0x215b51 => [0x8f45, false], # East Asian ideograph
        0x215b52 => [0x8f42, false], # East Asian ideograph (variant of 4B5B52 which maps to 8F42)
        0x215b53 => [0x8f3f, false], # East Asian ideograph
        0x225b54 => [0x749f, false], # East Asian ideograph
        0x225b55 => [0x749d, false], # East Asian ideograph
        0x215b56 => [0x8f54, false], # East Asian ideograph
        0x225b57 => [0x749e, false], # East Asian ideograph
        0x215b58 => [0x8f5f, false], # East Asian ideograph
        0x215b59 => [0x8f61, false], # East Asian ideograph
        0x215b5a => [0x8f9b, false], # East Asian ideograph
        0x215b5b => [0x8f9c, false], # East Asian ideograph
        0x215b5c => [0x8f9f, false], # East Asian ideograph
        0x224f3a => [0x7023, false], # East Asian ideograph
        0x235b5e => [0x9d92, false], # East Asian ideograph
        0x215b5f => [0x8fa6, false], # East Asian ideograph
        0x225b60 => [0x74b2, false], # East Asian ideograph
        0x215b61 => [0x8faf, false], # East Asian ideograph
        0x215b62 => [0x8fb0, false], # East Asian ideograph
        0x215b63 => [0x8fb1, false], # East Asian ideograph
        0x215b64 => [0x8fb2, false], # East Asian ideograph
        0x295e6a => [0x9eea, false], # East Asian ideograph
        0x225b66 => [0x74b4, false], # East Asian ideograph
        0x225b67 => [0x74ab, false], # East Asian ideograph
        0x215b68 => [0x8fc4, false], # East Asian ideograph
        0x215b69 => [0x5de1, false], # East Asian ideograph
        0x215b6a => [0x8fce, false], # East Asian ideograph
        0x215b6b => [0x8fd1, false], # East Asian ideograph
        0x215b6c => [0x8fd4, false], # East Asian ideograph
        0x275b6d => [0x8ff0, false], # East Asian ideograph
        0x215b6e => [0x8fe6, false], # East Asian ideograph
        0x215b6f => [0x8fe2, false], # East Asian ideograph
        0x235b70 => [0x9d96, false], # East Asian ideograph
        0x215b71 => [0x8fe5, false], # East Asian ideograph
        0x6f544b => [0xc379, false], # Korean hangul
        0x215b73 => [0x8feb, false], # East Asian ideograph
        0x215b74 => [0x9001, false], # East Asian ideograph
        0x215b75 => [0x9006, false], # East Asian ideograph
        0x225b76 => [0x74b8, false], # East Asian ideograph
        0x215b77 => [0x9000, false], # East Asian ideograph
        0x6f5b78 => [0xd329, false], # Korean hangul
        0x235b79 => [0x9dc0, false], # East Asian ideograph
        0x225b7a => [0x74c0, false], # East Asian ideograph
        0x23422e => [0x9207, false], # East Asian ideograph
        0x215b7c => [0x9005, false], # East Asian ideograph
        0x215b7d => [0x9019, false], # East Asian ideograph
        0x215b7e => [0x9023, false], # East Asian ideograph
        0x214f40 => [0x7a40, false], # East Asian ideograph
        0x393c52 => [0x8d26, false], # East Asian ideograph
        0x224f41 => [0x703c, false], # East Asian ideograph
        0x213941 => [0x596e, false], # East Asian ideograph
        0x6f4f42 => [0xb8e1, false], # Korean hangul
        0x6f544c => [0xc37c, false], # Korean hangul
        0x4b4f43 => [0x7a32, false], # East Asian ideograph
        0x213a31 => [0x5a9b, false], # East Asian ideograph
        0x224f44 => [0x7035, false], # East Asian ideograph
        0x213a41 => [0x5afb, false], # East Asian ideograph
        0x234f45 => [0x9832, false], # East Asian ideograph
        0x294331 => [0x94f1, false], # East Asian ideograph
        0x274f46 => [0x7a23, false], # East Asian ideograph
        0x6f4f47 => [0xb8f8, false], # Korean hangul
        0x2d4f48 => [0x7a42, false], # East Asian ideograph
        0x213a32 => [0x5acc, false], # East Asian ideograph
        0x294231 => [0x94af, false], # East Asian ideograph
        0x214f49 => [0x7a61, false], # East Asian ideograph
        0x4c3744 => [0x65d9, false], # East Asian ideograph
        0x274f4a => [0x79fd, false], # East Asian ideograph
        0x214f4b => [0x7a6b, false], # East Asian ideograph
        0x227631 => [0x8008, false], # East Asian ideograph
        0x274f4c => [0x7a33, false], # East Asian ideograph
        0x6f544e => [0xc384, false], # Korean hangul
        0x6f4f4d => [0xb958, false], # Korean hangul
        0x6f5361 => [0xc229, false], # Korean hangul
        0x213a33 => [0x5ac1, false], # East Asian ideograph
        0x234f4e => [0x9844, false], # East Asian ideograph
        0x4b393a => [0x5f09, false], # East Asian ideograph
        0x6f4f4f => [0xb95c, false], # Korean hangul
        0x334755 => [0x6fec, false], # East Asian ideograph
        0x6f4f50 => [0xb960, false], # Korean hangul
        0x6f4c69 => [0xb2dd, false], # Korean hangul
        0x23533e => [0x9a0b, false], # East Asian ideograph
        0x224f51 => [0x7034, false], # East Asian ideograph
        0x274564 => [0x69df, false], # East Asian ideograph
        0x6f544f => [0xc388, false], # Korean hangul
        0x213e77 => [0x611b, false], # East Asian ideograph
        0x6f4f52 => [0xb96d, false], # Korean hangul
        0x213a34 => [0x5ac9, false], # East Asian ideograph
        0x217e79 => [0x5be0, false], # East Asian ideograph
        0x224f53 => [0x7039, false], # East Asian ideograph
        0x224f54 => [0x703a, false], # East Asian ideograph
        0x395e6f => [0x7a7d, false], # East Asian ideograph
        0x6f4f55 => [0xb978, false], # Korean hangul
        0x23533f => [0x9a09, false], # East Asian ideograph
        0x6f4e44 => [0xb618, false], # Korean hangul
        0x6f4f56 => [0xb97c, false], # Korean hangul
        0x6f5450 => [0xc399, false], # Korean hangul
        0x6f4f57 => [0xb984, false], # Korean hangul
        0x213a35 => [0x5abe, false], # East Asian ideograph
        0x696b27 => [0x8977, false], # East Asian ideograph
        0x234233 => [0x91fe, false], # East Asian ideograph
        0x3a2f7c => [0x64c0, false], # East Asian ideograph
        0x395a2f => [0x58f2, false], # East Asian ideograph
        0x334f59 => [0x7a93, false], # East Asian ideograph
        0x293c57 => [0x8f79, false], # East Asian ideograph
        0x6f4f5a => [0xb989, false], # Korean hangul
        0x215c21 => [0x901f, false], # East Asian ideograph
        0x215c22 => [0x9017, false], # East Asian ideograph
        0x215c23 => [0x901d, false], # East Asian ideograph
        0x215c24 => [0x9010, false], # East Asian ideograph
        0x225c25 => [0x74bf, false], # East Asian ideograph
        0x215c26 => [0x900d, false], # East Asian ideograph
        0x215c27 => [0x901e, false], # East Asian ideograph
        0x235c28 => [0x9dbb, false], # East Asian ideograph
        0x274123 => [0x6302, false], # East Asian ideograph
        0x215c2a => [0x900f, false], # East Asian ideograph
        0x215c2b => [0x9022, false], # East Asian ideograph
        0x215c2c => [0x9016, false], # East Asian ideograph
        0x215c2d => [0x901b, false], # East Asian ideograph
        0x215c2e => [0x9014, false], # East Asian ideograph
        0x214f5d => [0x7aa9, false], # East Asian ideograph
        0x215c30 => [0x9035, false], # East Asian ideograph
        0x215c31 => [0x9031, false], # East Asian ideograph
        0x235c32 => [0x9db9, false], # East Asian ideograph
        0x275c33 => [0x8fdb, false], # East Asian ideograph
        0x275c34 => [0x8fd0, false], # East Asian ideograph
        0x2d4f5e => [0x7ab0, false], # East Asian ideograph
        0x215c36 => [0x9053, false], # East Asian ideograph
        0x215c37 => [0x9042, false], # East Asian ideograph
        0x215c38 => [0x9050, false], # East Asian ideograph
        0x275c39 => [0x8fbe, false], # East Asian ideograph
        0x275c3a => [0x8fdd, false], # East Asian ideograph
        0x274f5f => [0x7a77, false], # East Asian ideograph
        0x275c3c => [0x8fc2, false], # East Asian ideograph
        0x215c3d => [0x904f, false], # East Asian ideograph
        0x235c3e => [0x9dd9, false], # East Asian ideograph
        0x215c3f => [0x904d, false], # East Asian ideograph
        0x215c40 => [0x9051, false], # East Asian ideograph
        0x214f60 => [0x7aba, false], # East Asian ideograph
        0x215c42 => [0x903e, false], # East Asian ideograph
        0x215c43 => [0x9058, false], # East Asian ideograph
        0x275c44 => [0x8fdc, false], # East Asian ideograph
        0x275c45 => [0x900a, false], # East Asian ideograph
        0x215c46 => [0x9063, false], # East Asian ideograph
        0x214f61 => [0x7ac5, false], # East Asian ideograph
        0x275c48 => [0x9012, false], # East Asian ideograph
        0x215c49 => [0x9069, false], # East Asian ideograph
        0x215c4a => [0x906e, false], # East Asian ideograph
        0x215c4b => [0x9068, false], # East Asian ideograph
        0x215c4c => [0x906d, false], # East Asian ideograph
        0x214f62 => [0x7ac4, false], # East Asian ideograph
        0x215c4e => [0x9074, false], # East Asian ideograph
        0x275c4f => [0x9009, false], # East Asian ideograph
        0x275c50 => [0x8fdf, false], # East Asian ideograph
        0x215c51 => [0x9077, false], # East Asian ideograph
        0x215c52 => [0x907c, false], # East Asian ideograph
        0x275c53 => [0x9057, false], # East Asian ideograph
        0x215c54 => [0x907f, false], # East Asian ideograph
        0x215c55 => [0x907d, false], # East Asian ideograph
        0x275c56 => [0x8fc8, false], # East Asian ideograph
        0x235c57 => [0x9df2, false], # East Asian ideograph
        0x215c58 => [0x9082, false], # East Asian ideograph
        0x215c59 => [0x9080, false], # East Asian ideograph
        0x275c5a => [0x8fe9, false], # East Asian ideograph (variant of 2D5C5A which maps to 8FE9)
        0x275c5b => [0x8fb9, false], # East Asian ideograph
        0x275c5c => [0x9026, false], # East Asian ideograph
        0x275c5d => [0x903b, false], # East Asian ideograph
        0x215c5e => [0x9091, false], # East Asian ideograph
        0x215c5f => [0x9095, false], # East Asian ideograph
        0x215c60 => [0x90a3, false], # East Asian ideograph
        0x215c61 => [0x90a2, false], # East Asian ideograph
        0x215c62 => [0x90aa, false], # East Asian ideograph
        0x215c63 => [0x90a6, false], # East Asian ideograph
        0x215c64 => [0x90b5, false], # East Asian ideograph
        0x215c65 => [0x90b1, false], # East Asian ideograph
        0x215c66 => [0x90b8, false], # East Asian ideograph
        0x215c67 => [0x90ce, false], # East Asian ideograph
        0x215c68 => [0x90ca, false], # East Asian ideograph
        0x4b5564 => [0x77c7, false], # East Asian ideograph
        0x235c6a => [0x9ded, false], # East Asian ideograph
        0x215c6b => [0x90e8, false], # East Asian ideograph
        0x215c6c => [0x90ed, false], # East Asian ideograph
        0x275c6d => [0x90ae, false], # East Asian ideograph
        0x215c6e => [0x90fd, false], # East Asian ideograph
        0x215c6f => [0x9102, false], # East Asian ideograph
        0x275c70 => [0x4e61, false], # East Asian ideograph
        0x275c71 => [0x90b9, false], # East Asian ideograph
        0x215c72 => [0x9119, false], # East Asian ideograph
        0x275c73 => [0x90d1, false], # East Asian ideograph
        0x275c74 => [0x90bb, false], # East Asian ideograph
        0x275c75 => [0x9093, false], # East Asian ideograph
        0x215c76 => [0x9131, false], # East Asian ideograph
        0x214f69 => [0x7aed, false], # East Asian ideograph
        0x215c78 => [0x9149, false], # East Asian ideograph
        0x215c79 => [0x914b, false], # East Asian ideograph
        0x215c7a => [0x914a, false], # East Asian ideograph
        0x215c7b => [0x9152, false], # East Asian ideograph
        0x215c7c => [0x914d, false], # East Asian ideograph
        0x215c7d => [0x914c, false], # East Asian ideograph
        0x215c7e => [0x9157, false], # East Asian ideograph
        0x6f5454 => [0xc3df, false], # Korean hangul
        0x6f5a34 => [0xcf13, false], # Korean hangul
        0x214f6b => [0x7af6, false], # East Asian ideograph
        0x213a39 => [0x5ab3, false], # East Asian ideograph
        0x234237 => [0x9226, false], # East Asian ideograph
        0x6f4f6d => [0xb9d8, false], # Korean hangul
        0x695a31 => [0x64f6, false], # East Asian ideograph
        0x6f4e45 => [0xb625, false], # Korean hangul
        0x234f6f => [0x9857, false], # East Asian ideograph
        0x27456a => [0x6988, false], # East Asian ideograph
        0x213e7d => [0x6108, false], # East Asian ideograph
        0x395821 => [0x97e4, false], # East Asian ideograph
        0x274f70 => [0x5df4, false], # East Asian ideograph (duplicate simplified)
        0x213a3a => [0x5ae1, false], # East Asian ideograph
        0x224f71 => [0x7052, false], # East Asian ideograph
        0x234f72 => [0x9856, false], # East Asian ideograph
        0x295e7a => [0x9efe, false], # East Asian ideograph
        0x224f73 => [0x705c, false], # East Asian ideograph
        0x213f39 => [0x6163, false], # East Asian ideograph
        0x6f4f74 => [0xb9e4, false], # Korean hangul
        0x6f5456 => [0xc3e8, false], # Korean hangul
        0x213e7e => [0x60f1, false], # East Asian ideograph
        0x214f75 => [0x7b1b, false], # East Asian ideograph
        0x213a3b => [0x5ad7, false], # East Asian ideograph
        0x284e66 => [0x6ee2, false], # East Asian ideograph
        0x213a21 => [0x5a46, false], # East Asian ideograph
        0x234f77 => [0x9862, false], # East Asian ideograph
        0x275235 => [0x7f62, false], # East Asian ideograph
        0x224f78 => [0x7059, false], # East Asian ideograph
        0x213a23 => [0x5a6a, false], # East Asian ideograph
        0x274f79 => [0x7b14, false], # East Asian ideograph
        0x213a24 => [0x5a36, false], # East Asian ideograph
        0x6f5457 => [0xc3ed, false], # Korean hangul
        0x22427e => [0x6aed, false], # East Asian ideograph
        0x214f7b => [0x7b50, false], # East Asian ideograph
        0x213a26 => [0x5a40, false], # East Asian ideograph
        0x695e63 => [0x6e82, false], # East Asian ideograph
        0x275679 => [0x80e1, false], # East Asian ideograph (duplicate simplified)
        0x224f7c => [0x7061, false], # East Asian ideograph
        0x213a27 => [0x5a66, false], # East Asian ideograph
        0x224f7d => [0x705d, false], # East Asian ideograph
        0x223a28 => [0x6705, false], # East Asian ideograph
        0x335347 => [0x81d9, false], # East Asian ideograph
        0x293b4f => [0x8f78, false], # East Asian ideograph
        0x234f7e => [0x9868, false], # East Asian ideograph
        0x6f4f7b => [0xb9f8, false], # Korean hangul
        0x6f5458 => [0xc3f4, false], # Korean hangul
        0x6f5363 => [0xc22d, false], # Korean hangul
        0x2d5d68 => [0x8021, false], # East Asian ideograph
        0x394928 => [0x6d5c, false], # East Asian ideograph
        0x29366a => [0x8d53, false], # East Asian ideograph
        0x227a2c => [0x81b5, false], # East Asian ideograph
        0x223173 => [0x637f, false], # East Asian ideograph
        0x2d5179 => [0x7e62, false], # East Asian ideograph
        0x213a2e => [0x5a92, false], # East Asian ideograph
        0x6f5459 => [0xc3f5, false], # Korean hangul
        0x2d3a2f => [0x58fb, false], # East Asian ideograph
        0x4b3351 => [0x5204, false], # East Asian ideograph
        0x215d21 => [0x9163, false], # East Asian ideograph
        0x215d22 => [0x9165, false], # East Asian ideograph
        0x215d23 => [0x916c, false], # East Asian ideograph
        0x215d24 => [0x9169, false], # East Asian ideograph
        0x215d25 => [0x916a, false], # East Asian ideograph
        0x215d26 => [0x9175, false], # East Asian ideograph
        0x215d27 => [0x9178, false], # East Asian ideograph
        0x215d28 => [0x9177, false], # East Asian ideograph
        0x215d29 => [0x9187, false], # East Asian ideograph
        0x215d2a => [0x9189, false], # East Asian ideograph
        0x215d2b => [0x918b, false], # East Asian ideograph
        0x215d2c => [0x9183, false], # East Asian ideograph
        0x215d2d => [0x9192, false], # East Asian ideograph
        0x215d2e => [0x91a3, false], # East Asian ideograph
        0x275d2f => [0x915d, false], # East Asian ideograph
        0x215d30 => [0x919c, false], # East Asian ideograph
        0x275d31 => [0x533b, false], # East Asian ideograph
        0x225d32 => [0x7512, false], # East Asian ideograph
        0x215d33 => [0x91ba, false], # East Asian ideograph
        0x275d34 => [0x917f, false], # East Asian ideograph
        0x275d35 => [0x8845, false], # East Asian ideograph
        0x215d36 => [0x91c7, false], # East Asian ideograph
        0x215d37 => [0x91c9, false], # East Asian ideograph
        0x215d38 => [0x91cb, false], # East Asian ideograph
        0x235d39 => [0x9e1c, false], # East Asian ideograph
        0x235d3a => [0x9e1b, false], # East Asian ideograph
        0x215d3b => [0x91ce, false], # East Asian ideograph
        0x235d3c => [0x9e75, false], # East Asian ideograph
        0x275d3d => [0x5398, false], # East Asian ideograph
        0x215d3e => [0x91d1, false], # East Asian ideograph
        0x215d3f => [0x91dd, false], # East Asian ideograph
        0x215d40 => [0x91d8, false], # East Asian ideograph
        0x215d41 => [0x91d7, false], # East Asian ideograph
        0x235d42 => [0x9e7a, false], # East Asian ideograph
        0x215d43 => [0x91f5, false], # East Asian ideograph
        0x225d44 => [0x7524, false], # East Asian ideograph
        0x215d45 => [0x91e3, false], # East Asian ideograph
        0x215d46 => [0x91e7, false], # East Asian ideograph
        0x235d47 => [0x9e80, false], # East Asian ideograph
        0x215d48 => [0x920d, false], # East Asian ideograph
        0x215d49 => [0x9215, false], # East Asian ideograph
        0x215d4a => [0x9209, false], # East Asian ideograph
        0x275d4b => [0x949e, false], # East Asian ideograph
        0x215d4c => [0x921e, false], # East Asian ideograph
        0x215d4d => [0x9210, false], # East Asian ideograph
        0x2d4c5d => [0x7661, false], # East Asian ideograph
        0x225d4f => [0x753f, false], # East Asian ideograph
        0x215d50 => [0x9238, false], # East Asian ideograph
        0x225d51 => [0x7540, false], # East Asian ideograph
        0x225d52 => [0x753e, false], # East Asian ideograph
        0x215d53 => [0x9240, false], # East Asian ideograph
        0x215d54 => [0x924b, false], # East Asian ideograph
        0x235d55 => [0x9e90, false], # East Asian ideograph
        0x215d56 => [0x9264, false], # East Asian ideograph
        0x215d57 => [0x9251, false], # East Asian ideograph
        0x235d58 => [0x9e8c, false], # East Asian ideograph
        0x215d59 => [0x9278, false], # East Asian ideograph
        0x215d5a => [0x9280, false], # East Asian ideograph
        0x275d5b => [0x94d0, false], # East Asian ideograph
        0x215d5c => [0x9285, false], # East Asian ideograph
        0x235d5d => [0x9e9b, false], # East Asian ideograph
        0x215d5e => [0x9296, false], # East Asian ideograph
        0x225d5f => [0x755f, false], # East Asian ideograph
        0x215d60 => [0x9293, false], # East Asian ideograph
        0x275d61 => [0x8854, false], # East Asian ideograph
        0x215d62 => [0x92c5, false], # East Asian ideograph
        0x215d63 => [0x92bb, false], # East Asian ideograph
        0x275d64 => [0x9510, false], # East Asian ideograph
        0x275d65 => [0x94fa, false], # East Asian ideograph
        0x275d66 => [0x9500, false], # East Asian ideograph
        0x215d67 => [0x92c1, false], # East Asian ideograph
        0x215d68 => [0x92e4, false], # East Asian ideograph
        0x215d69 => [0x92bc, false], # East Asian ideograph
        0x215d6a => [0x92d2, false], # East Asian ideograph
        0x225d6b => [0x756c, false], # East Asian ideograph
        0x215d6c => [0x9336, false], # East Asian ideograph
        0x275d6d => [0x952f, false], # East Asian ideograph
        0x215d6e => [0x9333, false], # East Asian ideograph
        0x215d6f => [0x932f, false], # East Asian ideograph
        0x215d70 => [0x9322, false], # East Asian ideograph
        0x215d71 => [0x92fc, false], # East Asian ideograph
        0x215d72 => [0x932b, false], # East Asian ideograph
        0x215d73 => [0x931a, false], # East Asian ideograph
        0x215d74 => [0x9304, false], # East Asian ideograph
        0x213a3e => [0x5ae9, false], # East Asian ideograph
        0x275d76 => [0x9526, false], # East Asian ideograph
        0x275d77 => [0x9540, false], # East Asian ideograph
        0x275d78 => [0x9541, false], # East Asian ideograph
        0x235d79 => [0x9eaf, false], # East Asian ideograph
        0x215d7a => [0x9365, false], # East Asian ideograph
        0x213a3f => [0x5ad8, false], # East Asian ideograph
        0x215d7c => [0x934b, false], # East Asian ideograph
        0x215d7d => [0x9328, false], # East Asian ideograph
        0x215d7e => [0x9370, false], # East Asian ideograph
        0x4b5e3f => [0x922c, false], # East Asian ideograph
        0x213a40 => [0x5ae6, false], # East Asian ideograph
        0x6f5367 => [0xc234, false], # Korean hangul
        0x233a41 => [0x8e61, false], # East Asian ideograph
        0x6f5825 => [0xc990, false], # Korean hangul
        0x6f545d => [0xc434, false], # Korean hangul
        0x284971 => [0x6d9e, false], # East Asian ideograph
        0x224a32 => [0x6dfc, false], # East Asian ideograph
        0x227a43 => [0x81d0, false], # East Asian ideograph
        0x213a44 => [0x5b0c, false], # East Asian ideograph
        0x29366b => [0x8d55, false], # East Asian ideograph
        0x233a45 => [0x8e74, false], # East Asian ideograph
        0x213a46 => [0x5b34, false], # East Asian ideograph
        0x213a47 => [0x5b1d, false], # East Asian ideograph
        0x6f545e => [0xc43c, false], # Korean hangul
        0x6f5a36 => [0xcf1c, false], # Korean hangul
        0x273a48 => [0x5ad4, false], # East Asian ideograph
        0x213a43 => [0x5b0b, false], # East Asian ideograph
        0x395829 => [0x69fb, false], # East Asian ideograph
        0x6f4c3e => [0xb1d0, false], # Korean hangul
        0x4b3a49 => [0x5b37, false], # East Asian ideograph
        0x2d3b54 => [0x5c4a, false], # East Asian ideograph
        0x6f5923 => [0xcc2e, false], # Korean hangul
        0x213a4a => [0x5b30, false], # East Asian ideograph
        0x287855 => [0x80eb, false], # East Asian ideograph
        0x233a4b => [0x8e69, false], # East Asian ideograph
        0x213a4c => [0x5b40, false], # East Asian ideograph
        0x6f545f => [0xc43f, false], # Korean hangul
        0x213a4d => [0x5b50, false], # East Asian ideograph
        0x213a4e => [0x5b51, false], # East Asian ideograph
        0x217a4f => [0x59ee, false], # East Asian ideograph
        0x225b3f => [0x7485, false], # East Asian ideograph
        0x295e7c => [0x9f0b, false], # East Asian ideograph
        0x29456f => [0x9538, false], # East Asian ideograph
        0x6f5460 => [0xc464, false], # Korean hangul
        0x233a52 => [0x8e83, false], # East Asian ideograph
        0x213a45 => [0x5af5, false], # East Asian ideograph
        0x334243 => [0x52b9, false], # East Asian ideograph
        0x233a53 => [0x8e84, false], # East Asian ideograph
        0x2d592c => [0x8b01, false], # East Asian ideograph
        0x294335 => [0x94f5, false], # East Asian ideograph
        0x4c3a55 => [0x6741, false], # East Asian ideograph
        0x4b623b => [0x9d12, false], # East Asian ideograph
        0x217a56 => [0x59fd, false], # East Asian ideograph
        0x6f5461 => [0xc465, false], # Korean hangul
        0x213a57 => [0x5b5f, false], # East Asian ideograph
        0x6f5a39 => [0xcf2c, false], # Korean hangul
        0x213a58 => [0x5b63, false], # East Asian ideograph
        0x692544 => [0x30c4, false], # Katakana letter TU
        0x225622 => [0x728d, false], # East Asian ideograph
        0x215e21 => [0x937e, false], # East Asian ideograph
        0x275e22 => [0x9524, false], # East Asian ideograph
        0x275e23 => [0x9539, false], # East Asian ideograph
        0x215e24 => [0x935b, false], # East Asian ideograph
        0x275e25 => [0x9551, false], # East Asian ideograph
        0x215e26 => [0x9394, false], # East Asian ideograph
        0x275e27 => [0x9547, false], # East Asian ideograph
        0x275e28 => [0x9501, false], # East Asian ideograph
        0x215e29 => [0x93a2, false], # East Asian ideograph
        0x275e2a => [0x954d, false], # East Asian ideograph
        0x275e2b => [0x955c, false], # East Asian ideograph
        0x275e2c => [0x955d, false], # East Asian ideograph
        0x215e2d => [0x93d6, false], # East Asian ideograph
        0x275e2e => [0x955e, false], # East Asian ideograph
        0x215e2f => [0x93df, false], # East Asian ideograph
        0x275e30 => [0x94ff, false], # East Asian ideograph
        0x275e31 => [0x94fe, false], # East Asian ideograph
        0x215e32 => [0x93e2, false], # East Asian ideograph
        0x215e33 => [0x93dc, false], # East Asian ideograph
        0x215e34 => [0x93e4, false], # East Asian ideograph
        0x225e35 => [0x7598, false], # East Asian ideograph
        0x215e36 => [0x93cd, false], # East Asian ideograph
        0x235e37 => [0x9ec8, false], # East Asian ideograph
        0x215e39 => [0x9403, false], # East Asian ideograph
        0x215e3a => [0x942e, false], # East Asian ideograph
        0x225e3b => [0x75a3, false], # East Asian ideograph
        0x215e3c => [0x9433, false], # East Asian ideograph
        0x215e3d => [0x9435, false], # East Asian ideograph
        0x215e3e => [0x943a, false], # East Asian ideograph
        0x215e3f => [0x9438, false], # East Asian ideograph
        0x215e40 => [0x9432, false], # East Asian ideograph
        0x223a60 => [0x675d, false], # East Asian ideograph
        0x215e42 => [0x9451, false], # East Asian ideograph
        0x215e43 => [0x9444, false], # East Asian ideograph
        0x215e44 => [0x9463, false], # East Asian ideograph
        0x215e45 => [0x9460, false], # East Asian ideograph
        0x215e46 => [0x9472, false], # East Asian ideograph
        0x215e47 => [0x9470, false], # East Asian ideograph
        0x215e48 => [0x947e, false], # East Asian ideograph
        0x215e49 => [0x947c, false], # East Asian ideograph
        0x235e4a => [0x9ed0, false], # East Asian ideograph
        0x215e4b => [0x947f, false], # East Asian ideograph
        0x215e4c => [0x9577, false], # East Asian ideograph
        0x215e4d => [0x9580, false], # East Asian ideograph
        0x215e4e => [0x9582, false], # East Asian ideograph
        0x215e4f => [0x9583, false], # East Asian ideograph
        0x215e50 => [0x9589, false], # East Asian ideograph
        0x215e51 => [0x9594, false], # East Asian ideograph
        0x215e52 => [0x958f, false], # East Asian ideograph
        0x235e53 => [0x9eda, false], # East Asian ideograph
        0x215e54 => [0x9591, false], # East Asian ideograph
        0x235e55 => [0x9edf, false], # East Asian ideograph
        0x215e56 => [0x9592, false], # East Asian ideograph
        0x215e57 => [0x9598, false], # East Asian ideograph
        0x215e58 => [0x95a1, false], # East Asian ideograph
        0x235e59 => [0x9ee5, false], # East Asian ideograph
        0x215e5a => [0x95a9, false], # East Asian ideograph
        0x215e5b => [0x95a3, false], # East Asian ideograph
        0x215e5c => [0x95a5, false], # East Asian ideograph
        0x215e5d => [0x95a4, false], # East Asian ideograph
        0x215e5e => [0x95b1, false], # East Asian ideograph
        0x215e5f => [0x95ad, false], # East Asian ideograph
        0x235e60 => [0x9eee, false], # East Asian ideograph
        0x215e61 => [0x95ca, false], # East Asian ideograph
        0x215e62 => [0x95cb, false], # East Asian ideograph
        0x215e63 => [0x95cc, false], # East Asian ideograph
        0x215e64 => [0x95c8, false], # East Asian ideograph
        0x215e65 => [0x95c6, false], # East Asian ideograph
        0x235e66 => [0x9ef0, false], # East Asian ideograph
        0x215e67 => [0x95d6, false], # East Asian ideograph
        0x215e68 => [0x95d0, false], # East Asian ideograph
        0x215e69 => [0x95dc, false], # East Asian ideograph
        0x215e6a => [0x95e1, false], # East Asian ideograph
        0x215e6b => [0x95e2, false], # East Asian ideograph
        0x215e6c => [0x961c, false], # East Asian ideograph
        0x215e6d => [0x9621, false], # East Asian ideograph
        0x215e6e => [0x9632, false], # East Asian ideograph
        0x215e6f => [0x9631, false], # East Asian ideograph
        0x215e70 => [0x962e, false], # East Asian ideograph
        0x215e71 => [0x962a, false], # East Asian ideograph
        0x215e72 => [0x9640, false], # East Asian ideograph
        0x215e73 => [0x963f, false], # East Asian ideograph
        0x215e74 => [0x963b, false], # East Asian ideograph
        0x215e75 => [0x9644, false], # East Asian ideograph
        0x215e76 => [0x9650, false], # East Asian ideograph
        0x235e77 => [0x9efc, false], # East Asian ideograph
        0x215e78 => [0x964b, false], # East Asian ideograph
        0x215e79 => [0x964d, false], # East Asian ideograph
        0x235e7a => [0x9efd, false], # East Asian ideograph
        0x215e7b => [0x9663, false], # East Asian ideograph
        0x235e7c => [0x9eff, false], # East Asian ideograph
        0x215e7d => [0x9661, false], # East Asian ideograph
        0x225e7e => [0x7603, false], # East Asian ideograph
        0x6f5465 => [0xc479, false], # Korean hangul
        0x223a6b => [0x6763, false], # East Asian ideograph
        0x223a6e => [0x6753, false], # East Asian ideograph
        0x2d355c => [0x5434, false], # East Asian ideograph
        0x213a6f => [0x5b98, false], # East Asian ideograph
        0x6f5466 => [0xc480, false], # Korean hangul
        0x6f5440 => [0xc328, false], # Korean hangul
        0x293a70 => [0x8e9c, false], # East Asian ideograph
        0x213a4b => [0x5b38, false], # East Asian ideograph
        0x294936 => [0x95f3, false], # East Asian ideograph
        0x215821 => [0x896a, false], # East Asian ideograph
        0x233a71 => [0x8ea9, false], # East Asian ideograph
        0x692524 => [0x30a4, false], # Katakana letter I
        0x213a72 => [0x5ba5, false], # East Asian ideograph
        0x6f595f => [0xce04, false], # Korean hangul
        0x2d3b52 => [0x6eba, false], # East Asian ideograph
        0x223a75 => [0x6793, false], # East Asian ideograph
        0x23424a => [0x9216, false], # East Asian ideograph
        0x6f2521 => [0x315c, false], # Korean hangul
        0x4b4767 => [0x6db5, false], # East Asian ideograph (variant of 214767 which maps to 6DB5)
        0x28342c => [0x63ba, false], # East Asian ideograph
        0x295a44 => [0x9e32, false], # East Asian ideograph
        0x223a78 => [0x677c, false], # East Asian ideograph
        0x692523 => [0x30a3, false], # Katakana letter small I
        0x292524 => [0x848c, false], # East Asian ideograph
        0x29322a => [0x8bb5, false], # East Asian ideograph
        0x223a7a => [0x679f, false], # East Asian ideograph
        0x226065 => [0x76a4, false], # East Asian ideograph
        0x23424b => [0x9211, false], # East Asian ideograph
        0x4d472c => [0x952a, false], # East Asian ideograph
        0x4d5934 => [0x9ca6, false], # East Asian ideograph
        0x213a7c => [0x5bae, false], # East Asian ideograph
        0x692527 => [0x30a7, false], # Katakana letter small E
        0x213d2e => [0x5eec, false], # East Asian ideograph
        0x233a7d => [0x8eb6, false], # East Asian ideograph
        0x692528 => [0x30a8, false], # Katakana letter E
        0x6f5469 => [0xc4d5, false], # Korean hangul
        0x69252a => [0x30aa, false], # Katakana letter O
        0x283b22 => [0x4e2b, false], # East Asian ideograph
        0x22652c => [0x7892, false], # East Asian ideograph
        0x28342e => [0x63bc, false], # East Asian ideograph
        0x235359 => [0x9a2f, false], # East Asian ideograph
        0x22252d => [0x5d47, false], # East Asian ideograph
        0x2d4829 => [0x51cf, false], # East Asian ideograph
        0x6f5027 => [0xba4b, false], # Korean hangul
        0x23252f => [0x84e7, false], # East Asian ideograph
        0x215f21 => [0x9664, false], # East Asian ideograph
        0x215f22 => [0x966a, false], # East Asian ideograph
        0x215f23 => [0x9673, false], # East Asian ideograph
        0x215f24 => [0x9678, false], # East Asian ideograph
        0x215f25 => [0x9675, false], # East Asian ideograph
        0x215f26 => [0x9672, false], # East Asian ideograph
        0x215f27 => [0x9676, false], # East Asian ideograph
        0x215f28 => [0x9677, false], # East Asian ideograph
        0x215f29 => [0x9674, false], # East Asian ideograph
        0x215f2a => [0x9670, false], # East Asian ideograph
        0x215f2b => [0x968a, false], # East Asian ideograph
        0x215f2c => [0x968e, false], # East Asian ideograph
        0x215f2d => [0x968b, false], # East Asian ideograph
        0x215f2e => [0x967d, false], # East Asian ideograph
        0x235f2f => [0x9f0f, false], # East Asian ideograph
        0x215f30 => [0x9686, false], # East Asian ideograph
        0x235f31 => [0x9f10, false], # East Asian ideograph
        0x235f32 => [0x9f12, false], # East Asian ideograph
        0x235f33 => [0x9f16, false], # East Asian ideograph
        0x235f34 => [0x9f17, false], # East Asian ideograph
        0x215f35 => [0x9695, false], # East Asian ideograph
        0x215f36 => [0x969c, false], # East Asian ideograph
        0x235f37 => [0x9f1a, false], # East Asian ideograph
        0x215f38 => [0x96a7, false], # East Asian ideograph
        0x215f39 => [0x96a8, false], # East Asian ideograph
        0x215f3a => [0x96aa, false], # East Asian ideograph
        0x215f3b => [0x96b1, false], # East Asian ideograph
        0x215f3c => [0x96b4, false], # East Asian ideograph
        0x215f3d => [0x96b8, false], # East Asian ideograph
        0x225f3e => [0x7625, false], # East Asian ideograph
        0x225f3f => [0x761a, false], # East Asian ideograph
        0x215f40 => [0x96c7, false], # East Asian ideograph
        0x215f41 => [0x96c6, false], # East Asian ideograph
        0x215f42 => [0x96c4, false], # East Asian ideograph
        0x215f43 => [0x96c1, false], # East Asian ideograph
        0x215f44 => [0x96c5, false], # East Asian ideograph
        0x215f45 => [0x96cd, false], # East Asian ideograph
        0x215f46 => [0x96cb, false], # East Asian ideograph
        0x215f47 => [0x96c9, false], # East Asian ideograph
        0x215f48 => [0x96cc, false], # East Asian ideograph
        0x215f49 => [0x96d5, false], # East Asian ideograph
        0x215f4a => [0x96d6, false], # East Asian ideograph
        0x215f4b => [0x96dc, false], # East Asian ideograph
        0x215f4c => [0x96de, false], # East Asian ideograph
        0x215f4d => [0x96db, false], # East Asian ideograph
        0x215f4e => [0x96d9, false], # East Asian ideograph
        0x215f4f => [0x96e2, false], # East Asian ideograph
        0x225f50 => [0x7622, false], # East Asian ideograph
        0x225f51 => [0x762f, false], # East Asian ideograph
        0x215f52 => [0x96ea, false], # East Asian ideograph
        0x215f53 => [0x96ef, false], # East Asian ideograph
        0x215f54 => [0x96f2, false], # East Asian ideograph
        0x215f55 => [0x96fb, false], # East Asian ideograph
        0x215f56 => [0x96f7, false], # East Asian ideograph
        0x215f57 => [0x96f9, false], # East Asian ideograph
        0x215f58 => [0x96f6, false], # East Asian ideograph
        0x215f59 => [0x9700, false], # East Asian ideograph
        0x23424f => [0x92a2, false], # East Asian ideograph
        0x215f5b => [0x9704, false], # East Asian ideograph
        0x215f5c => [0x9709, false], # East Asian ideograph
        0x215f5d => [0x9706, false], # East Asian ideograph
        0x225f5e => [0x763b, false], # East Asian ideograph
        0x215f5f => [0x970e, false], # East Asian ideograph
        0x225f60 => [0x763c, false], # East Asian ideograph
        0x215f61 => [0x970f, false], # East Asian ideograph
        0x225f62 => [0x7635, false], # East Asian ideograph
        0x215f63 => [0x9713, false], # East Asian ideograph
        0x235f64 => [0x9f3d, false], # East Asian ideograph
        0x215f65 => [0x971e, false], # East Asian ideograph
        0x215f66 => [0x972a, false], # East Asian ideograph
        0x225f67 => [0x7648, false], # East Asian ideograph
        0x225f68 => [0x764e, false], # East Asian ideograph
        0x235f69 => [0x9f41, false], # East Asian ideograph
        0x225f6a => [0x7643, false], # East Asian ideograph
        0x215f6b => [0x973d, false], # East Asian ideograph
        0x215f6c => [0x973e, false], # East Asian ideograph
        0x215f6d => [0x9744, false], # East Asian ideograph
        0x215f6e => [0x9742, false], # East Asian ideograph
        0x225f6f => [0x7649, false], # East Asian ideograph
        0x215f70 => [0x9751, false], # East Asian ideograph
        0x215f71 => [0xfa1c, false], # East Asian ideograph
        0x215f72 => [0x975b, false], # East Asian ideograph (variant of 4B5F72 which maps to 975B)
        0x215f73 => [0x975c, false], # East Asian ideograph
        0x215f74 => [0x975e, false], # East Asian ideograph
        0x225f75 => [0x7654, false], # East Asian ideograph
        0x215f76 => [0x9761, false], # East Asian ideograph
        0x215f78 => [0x9766, false], # East Asian ideograph
        0x235f79 => [0x9f4e, false], # East Asian ideograph
        0x225f7a => [0x765c, false], # East Asian ideograph
        0x235f7b => [0x9f4f, false], # East Asian ideograph
        0x235f7c => [0x9f54, false], # East Asian ideograph
        0x215f7d => [0x977c, false], # East Asian ideograph
        0x235f7e => [0x9f55, false], # East Asian ideograph
        0x216540 => [0x4f66, false], # East Asian ideograph
        0x692541 => [0x30c1, false], # Katakana letter TI
        0x6f5622 => [0xc644, false], # Korean hangul
        0x6f546e => [0xc500, false], # Korean hangul
        0x6f502b => [0xba54, false], # Korean hangul
        0x215829 => [0x898f, false], # East Asian ideograph
        0x234251 => [0x9230, false], # East Asian ideograph
        0x225c28 => [0x74b5, false], # East Asian ideograph
        0x216544 => [0x4f67, false], # East Asian ideograph
        0x695429 => [0x5726, false], # East Asian ideograph
        0x6f515c => [0xbd88, false], # Korean hangul
        0x292546 => [0x8368, false], # East Asian ideograph
        0x233145 => [0x89dc, false], # East Asian ideograph
        0x692547 => [0x30c7, false], # Katakana letter DE
        0x225c29 => [0x74ba, false], # East Asian ideograph
        0x213a48 => [0x5b2a, false], # East Asian ideograph
        0x6f492d => [0xac85, false], # Korean hangul
        0x69254a => [0x30ca, false], # Katakana letter NA
        0x29254b => [0x835b, false], # East Asian ideograph
        0x2d482f => [0x6e07, false], # East Asian ideograph
        0x6f5442 => [0xc330, false], # Korean hangul
        0x70586f => [0x4eeb, false], # East Asian ideograph
        0x274142 => [0x6325, false], # East Asian ideograph
        0x6f502d => [0xba58, false], # Korean hangul
        0x23254d => [0x8553, false], # East Asian ideograph
        0x21654e => [0x4f5a, false], # East Asian ideograph
        0x335065 => [0x7a45, false], # East Asian ideograph
        0x2e3b22 => [0x690f, false], # East Asian ideograph
        0x224f61 => [0x7044, false], # East Asian ideograph
        0x692550 => [0x30d0, false], # Katakana letter BA
        0x6f5c71 => [0xd57c, false], # Korean hangul
        0x222551 => [0x5d8e, false], # East Asian ideograph
        0x6f586b => [0xcb58, false], # Korean hangul
        0x335834 => [0x89e7, false], # East Asian ideograph
        0x2d593d => [0x8ae9, false], # East Asian ideograph
        0x4b4476 => [0x685f, false], # East Asian ideograph
        0x692555 => [0x30d5, false], # Katakana letter HU
        0x216556 => [0x4f82, false], # East Asian ideograph
        0x6f5a3a => [0xcf2d, false], # Korean hangul
        0x6f502f => [0xba64, false], # Korean hangul
        0x692557 => [0x30d7, false], # Katakana letter PU
        0x294942 => [0x9606, false], # East Asian ideograph
        0x234255 => [0x9248, false], # East Asian ideograph
        0x692558 => [0x30d8, false], # Katakana letter HE
        0x47347b => [0x8c2b, false], # East Asian ideograph
        0x6f5262 => [0xc0af, false], # Korean hangul
        0x23255a => [0x8546, false], # East Asian ideograph
        0x216021 => [0x978d, false], # East Asian ideograph
        0x226022 => [0x7664, false], # East Asian ideograph
        0x236023 => [0x9f57, false], # East Asian ideograph
        0x226024 => [0x7659, false], # East Asian ideograph
        0x216025 => [0x97a0, false], # East Asian ideograph
        0x216026 => [0x97a3, false], # East Asian ideograph
        0x216027 => [0x97a6, false], # East Asian ideograph
        0x236028 => [0x9f60, false], # East Asian ideograph
        0x216029 => [0x97c3, false], # East Asian ideograph
        0x21602a => [0x97c1, false], # East Asian ideograph
        0x22602b => [0x765f, false], # East Asian ideograph
        0x21602c => [0x97cb, false], # East Asian ideograph
        0x21602d => [0x97cc, false], # East Asian ideograph
        0x21602e => [0x97d3, false], # East Asian ideograph
        0x21602f => [0x97dc, false], # East Asian ideograph
        0x216030 => [0x97ed, false], # East Asian ideograph
        0x216031 => [0x97f3, false], # East Asian ideograph
        0x226032 => [0x7667, false], # East Asian ideograph
        0x216033 => [0x7adf, false], # East Asian ideograph
        0x216034 => [0x97f6, false], # East Asian ideograph
        0x226035 => [0x766a, false], # East Asian ideograph
        0x216036 => [0x97ff, false], # East Asian ideograph (variant of 456036 which maps to 97FF)
        0x226037 => [0x766d, false], # East Asian ideograph
        0x226038 => [0x766f, false], # East Asian ideograph
        0x216039 => [0x9803, false], # East Asian ideograph
        0x22603a => [0x7670, false], # East Asian ideograph
        0x21603b => [0x9806, false], # East Asian ideograph
        0x21603c => [0x9808, false], # East Asian ideograph
        0x21603d => [0x9810, false], # East Asian ideograph
        0x21603e => [0x980a, false], # East Asian ideograph
        0x21603f => [0x9811, false], # East Asian ideograph
        0x226040 => [0x7676, false], # East Asian ideograph
        0x226041 => [0x7677, false], # East Asian ideograph
        0x216042 => [0x980c, false], # East Asian ideograph
        0x216043 => [0x9817, false], # East Asian ideograph
        0x216044 => [0x9818, false], # East Asian ideograph (variant of 4B6044 which maps to 9818)
        0x216045 => [0x9821, false], # East Asian ideograph
        0x216046 => [0x982d, false], # East Asian ideograph
        0x216047 => [0x9830, false], # East Asian ideograph
        0x226048 => [0x7680, false], # East Asian ideograph
        0x21582f => [0x89b2, false], # East Asian ideograph
        0x22604a => [0x768b, false], # East Asian ideograph
        0x21604b => [0x9837, false], # East Asian ideograph
        0x21604c => [0x9824, false], # East Asian ideograph
        0x21604d => [0x9846, false], # East Asian ideograph
        0x21604e => [0x9854, false], # East Asian ideograph
        0x21604f => [0x984d, false], # East Asian ideograph
        0x216050 => [0x984c, false], # East Asian ideograph
        0x216051 => [0x984e, false], # East Asian ideograph
        0x226052 => [0x7695, false], # East Asian ideograph
        0x216053 => [0x985e, false], # East Asian ideograph (variant of 4B6053 which maps to 985E)
        0x216054 => [0x985a, false], # East Asian ideograph
        0x226055 => [0x656b, false], # East Asian ideograph
        0x216056 => [0x9867, false], # East Asian ideograph
        0x216057 => [0x986b, false], # East Asian ideograph
        0x216058 => [0x986f, false], # East Asian ideograph
        0x226059 => [0x7699, false], # East Asian ideograph
        0x21605a => [0x9870, false], # East Asian ideograph
        0x21605b => [0x98a8, false], # East Asian ideograph
        0x21605c => [0x98af, false], # East Asian ideograph
        0x22605d => [0x769c, false], # East Asian ideograph
        0x21605e => [0x98b3, false], # East Asian ideograph
        0x22605f => [0x769d, false], # East Asian ideograph
        0x216060 => [0x98ba, false], # East Asian ideograph
        0x236061 => [0x9f93, false], # East Asian ideograph
        0x216062 => [0x98c4, false], # East Asian ideograph
        0x216063 => [0x98db, false], # East Asian ideograph
        0x216064 => [0x98df, false], # East Asian ideograph
        0x216065 => [0x98e2, false], # East Asian ideograph
        0x226066 => [0x76a5, false], # East Asian ideograph
        0x226067 => [0x76a6, false], # East Asian ideograph
        0x216068 => [0x98ed, false], # East Asian ideograph
        0x216069 => [0x98ea, false], # East Asian ideograph
        0x21606a => [0x98ee, false], # East Asian ideograph
        0x23606b => [0x9fa0, false], # East Asian ideograph
        0x21606c => [0x98fc, false], # East Asian ideograph
        0x21606d => [0x98f4, false], # East Asian ideograph
        0x21606e => [0x98fd, false], # East Asian ideograph
        0x21606f => [0x98fe, false], # East Asian ideograph
        0x216070 => [0x9903, false], # East Asian ideograph
        0x216071 => [0x990a, false], # East Asian ideograph
        0x236072 => [0x9fa4, false], # East Asian ideograph
        0x216073 => [0x9909, false], # East Asian ideograph
        0x226074 => [0x76b8, false], # East Asian ideograph
        0x216075 => [0x9912, false], # East Asian ideograph
        0x216076 => [0x9918, false], # East Asian ideograph
        0x226077 => [0x76bd, false], # East Asian ideograph
        0x216078 => [0x9905, false], # East Asian ideograph
        0x216079 => [0x9928, false], # East Asian ideograph
        0x21607a => [0x991e, false], # East Asian ideograph
        0x21607b => [0x991b, false], # East Asian ideograph
        0x21607c => [0x9921, false], # East Asian ideograph
        0x21607d => [0x9935, false], # East Asian ideograph
        0x21607e => [0x993e, false], # East Asian ideograph
        0x6f5369 => [0xc258, false], # Korean hangul
        0x6f5033 => [0xba71, false], # Korean hangul
        0x213a5b => [0x5b6b, false], # East Asian ideograph
        0x69256b => [0x30eb, false], # Katakana letter RU
        0x69256c => [0x30ec, false], # Katakana letter RE
        0x293670 => [0x8d49, false], # East Asian ideograph
        0x69656d => [0x7e83, false], # East Asian ideograph
        0x224f67 => [0x7047, false], # East Asian ideograph
        0x235172 => [0x994c, false], # East Asian ideograph
        0x69256f => [0x30ef, false], # Katakana letter WA
        0x2e4c35 => [0x6de5, false], # East Asian ideograph
        0x6f5034 => [0xba74, false], # Korean hangul
        0x213a5c => [0x5b70, false], # East Asian ideograph
        0x6f5934 => [0xcc59, false], # Korean hangul
        0x4d4f39 => [0x988c, false], # East Asian ideograph
        0x292571 => [0x835e, false], # East Asian ideograph
        0x226573 => [0x78e0, false], # East Asian ideograph
        0x6f4e4c => [0xb6b1, false], # Korean hangul
        0x292574 => [0x83b8, false], # East Asian ideograph
        0x4b3a47 => [0x88ca, false], # East Asian ideograph
        0x6f5035 => [0xba78, false], # Korean hangul
        0x692575 => [0x30f5, false], # Katakana letter small KA
        0x294948 => [0x960f, false], # East Asian ideograph
        0x213378 => [0x5275, false], # East Asian ideograph
        0x225c32 => [0x74cc, false], # East Asian ideograph
        0x216576 => [0x4f9c, false], # East Asian ideograph
        0x215021 => [0x7b4d, false], # East Asian ideograph
        0x6f5c6c => [0xd56d, false], # Korean hangul
        0x232577 => [0x858c, false], # East Asian ideograph
        0x214b6a => [0x74ca, false], # East Asian ideograph
        0x224f69 => [0x7049, false], # East Asian ideograph
        0x216940 => [0x5133, false], # East Asian ideograph
        0x692578 => [0x309c, false], # Katakana-hiragana semi-voiced sound mark
        0x275023 => [0x8345, false], # East Asian ideograph
        0x2e742e => [0x7516, false], # East Asian ideograph
        0x6f5479 => [0xc53d, false], # Korean hangul
        0x6f5024 => [0xba40, false], # Korean hangul
        0x6f5036 => [0xba83, false], # Korean hangul
        0x213a5e => [0x5b71, false], # East Asian ideograph
        0x294949 => [0x9608, false], # East Asian ideograph
        0x225025 => [0x7066, false], # East Asian ideograph
        0x2e257b => [0x5d1f, false], # East Asian ideograph
        0x6f5026 => [0xba49, false], # Korean hangul
        0x6f4a5f => [0xaecd, false], # Korean hangul
        0x225027 => [0x7065, false], # East Asian ideograph
        0x235369 => [0x9a36, false], # East Asian ideograph
        0x225028 => [0x7068, false], # East Asian ideograph
        0x234f26 => [0x9816, false], # East Asian ideograph
        0x215029 => [0x7b95, false], # East Asian ideograph
        0x276272 => [0x9ee9, false], # East Asian ideograph
        0x213a5f => [0x5b75, false], # East Asian ideograph
        0x27502a => [0x94b3, false], # East Asian ideograph
        0x27502b => [0x7b3a, false], # East Asian ideograph
        0x6f502c => [0xba55, false], # Korean hangul
        0x224f6b => [0x7055, false], # East Asian ideograph
        0x23536a => [0x9a2e, false], # East Asian ideograph
        0x2d502d => [0x7b5d, false], # East Asian ideograph
        0x4c4339 => [0x69de, false], # East Asian ideograph
        0x6f502e => [0xba5c, false], # Korean hangul
        0x6f5038 => [0xba85, false], # Korean hangul
        0x213a60 => [0x5b78, false], # East Asian ideograph
        0x21502f => [0x7bad, false], # East Asian ideograph
        0x215030 => [0x7bc4, false], # East Asian ideograph
        0x216122 => [0x993d, false], # East Asian ideograph
        0x226123 => [0x76cb, false], # East Asian ideograph
        0x216124 => [0x9952, false], # East Asian ideograph
        0x216125 => [0x9951, false], # East Asian ideograph
        0x226126 => [0x76cc, false], # East Asian ideograph
        0x216127 => [0x995e, false], # East Asian ideograph
        0x216128 => [0x9996, false], # East Asian ideograph
        0x216129 => [0x9999, false], # East Asian ideograph
        0x21612a => [0x99a5, false], # East Asian ideograph
        0x21612b => [0x99a8, false], # East Asian ideograph
        0x21612c => [0x99ac, false], # East Asian ideograph
        0x21612d => [0x99ae, false], # East Asian ideograph
        0x21612e => [0x99ad, false], # East Asian ideograph
        0x21612f => [0x99b3, false], # East Asian ideograph
        0x216130 => [0x99b1, false], # East Asian ideograph
        0x216131 => [0x99b4, false], # East Asian ideograph
        0x216132 => [0x99c1, false], # East Asian ideograph
        0x275033 => [0x8282, false], # East Asian ideograph
        0x216134 => [0x99dd, false], # East Asian ideograph
        0x216135 => [0x99d5, false], # East Asian ideograph
        0x216136 => [0x99df, false], # East Asian ideograph
        0x216137 => [0x99db, false], # East Asian ideograph
        0x216138 => [0x99d2, false], # East Asian ideograph
        0x216139 => [0x99d9, false], # East Asian ideograph
        0x21613a => [0x99d1, false], # East Asian ideograph
        0x21613b => [0x99ed, false], # East Asian ideograph
        0x21613c => [0x99f1, false], # East Asian ideograph
        0x21613d => [0x9a01, false], # East Asian ideograph
        0x21613e => [0x99ff, false], # East Asian ideograph
        0x21613f => [0x99e2, false], # East Asian ideograph
        0x216140 => [0x9a0e, false], # East Asian ideograph
        0x216141 => [0x9a19, false], # East Asian ideograph
        0x216142 => [0x9a16, false], # East Asian ideograph
        0x216143 => [0x9a2b, false], # East Asian ideograph
        0x226144 => [0x76ed, false], # East Asian ideograph
        0x216145 => [0x9a37, false], # East Asian ideograph
        0x216146 => [0x9a43, false], # East Asian ideograph
        0x216147 => [0x9a45, false], # East Asian ideograph
        0x226148 => [0x76f1, false], # East Asian ideograph
        0x216149 => [0x9a3e, false], # East Asian ideograph
        0x21614a => [0x9a55, false], # East Asian ideograph
        0x21614b => [0x9a5a, false], # East Asian ideograph
        0x21614c => [0x9a5b, false], # East Asian ideograph
        0x21614d => [0x9a57, false], # East Asian ideograph
        0x21614e => [0x9a5f, false], # East Asian ideograph
        0x22614f => [0x7708, false], # East Asian ideograph
        0x226150 => [0x7707, false], # East Asian ideograph
        0x275038 => [0x7bac, false], # East Asian ideograph
        0x216152 => [0x9aa8, false], # East Asian ideograph
        0x216153 => [0x9aaf, false], # East Asian ideograph
        0x226154 => [0x770a, false], # East Asian ideograph
        0x216155 => [0x9ab7, false], # East Asian ideograph
        0x216156 => [0x9ab8, false], # East Asian ideograph
        0x215039 => [0x7be4, false], # East Asian ideograph
        0x216158 => [0x9acf, false], # East Asian ideograph
        0x226159 => [0x76fb, false], # East Asian ideograph
        0x21615a => [0x9ad4, false], # East Asian ideograph
        0x21615b => [0x9ad2, false], # East Asian ideograph
        0x21615c => [0x9ad8, false], # East Asian ideograph
        0x21615d => [0x9ae5, false], # East Asian ideograph
        0x22615e => [0x772b, false], # East Asian ideograph
        0x21615f => [0x9aee, false], # East Asian ideograph
        0x216160 => [0x9afb, false], # East Asian ideograph
        0x216161 => [0x9aed, false], # East Asian ideograph
        0x216162 => [0x9b03, false], # East Asian ideograph
        0x216163 => [0x9b06, false], # East Asian ideograph
        0x216164 => [0x9b0d, false], # East Asian ideograph
        0x216165 => [0x9b1a, false], # East Asian ideograph
        0x216166 => [0x9b22, false], # East Asian ideograph
        0x216167 => [0x9b25, false], # East Asian ideograph
        0x216168 => [0x9b27, false], # East Asian ideograph
        0x27503c => [0x7b5b, false], # East Asian ideograph
        0x21616a => [0x9b31, false], # East Asian ideograph
        0x21616b => [0x9b32, false], # East Asian ideograph
        0x21616c => [0x9b3c, false], # East Asian ideograph
        0x21616d => [0x9b41, false], # East Asian ideograph
        0x21616e => [0x9b42, false], # East Asian ideograph
        0x22616f => [0x7721, false], # East Asian ideograph
        0x216170 => [0x9b44, false], # East Asian ideograph
        0x216171 => [0x9b4f, false], # East Asian ideograph
        0x216172 => [0x9b54, false], # East Asian ideograph
        0x216173 => [0x9b58, false], # East Asian ideograph
        0x216174 => [0x9b5a, false], # East Asian ideograph
        0x226175 => [0x7739, false], # East Asian ideograph
        0x226176 => [0x772f, false], # East Asian ideograph
        0x216177 => [0x9b91, false], # East Asian ideograph
        0x216178 => [0x9bab, false], # East Asian ideograph
        0x216179 => [0x9bae, false], # East Asian ideograph
        0x21617a => [0x9baa, false], # East Asian ideograph
        0x21617b => [0x9bca, false], # East Asian ideograph
        0x21617c => [0x9bc9, false], # East Asian ideograph
        0x21617d => [0x9be8, false], # East Asian ideograph
        0x21617e => [0x9be7, false], # East Asian ideograph
        0x215040 => [0x7bf7, false], # East Asian ideograph
        0x275041 => [0x7b80, false], # East Asian ideograph
        0x225042 => [0x7086, false], # East Asian ideograph
        0x6f503c => [0xbaab, false], # Korean hangul
        0x29596b => [0x9ca1, false], # East Asian ideograph
        0x335f3d => [0x96b7, false], # East Asian ideograph
        0x29494f => [0x960a, false], # East Asian ideograph
        0x6f5043 => [0xbac3, false], # Korean hangul
        0x4b5044 => [0x7c27, false], # East Asian ideograph (variant of 215044 which maps to 7C27)
        0x275045 => [0x7baa, false], # East Asian ideograph
        0x2e3d73 => [0x7a1c, false], # East Asian ideograph
        0x275046 => [0x7bd1, false], # East Asian ideograph
        0x6f5047 => [0xbb34, false], # Korean hangul
        0x6f503d => [0xbaac, false], # Korean hangul
        0x213a65 => [0x5b87, false], # East Asian ideograph
        0x294950 => [0x960c, false], # East Asian ideograph
        0x235048 => [0x98b8, false], # East Asian ideograph
        0x225c3a => [0x74d4, false], # East Asian ideograph
        0x27583a => [0x8ba3, false], # East Asian ideograph
        0x225049 => [0x7084, false], # East Asian ideograph
        0x2d594c => [0x8b72, false], # East Asian ideograph
        0x6f5960 => [0xce20, false], # Korean hangul
        0x22504a => [0x7081, false], # East Asian ideograph
        0x235370 => [0x9a41, false], # East Asian ideograph
        0x21504b => [0x7c3d, false], # East Asian ideograph
        0x4d3032 => [0x88ae, false], # East Asian ideograph
        0x6f5a3d => [0xcf54, false], # Korean hangul
        0x27504c => [0x7bee, false], # East Asian ideograph
        0x274153 => [0x6363, false], # East Asian ideograph
        0x4d5c6b => [0x9d50, false], # East Asian ideograph
        0x213a66 => [0x5b88, false], # East Asian ideograph
        0x21504d => [0x7c4c, false], # East Asian ideograph
        0x234264 => [0x925e, false], # East Asian ideograph
        0x2d3b77 => [0x5ce9, false], # East Asian ideograph
        0x21504e => [0x7c4d, false], # East Asian ideograph
        0x6f5025 => [0xba48, false], # Korean hangul
        0x2d504f => [0x7c58, false], # East Asian ideograph
        0x69542a => [0x5737, false], # East Asian ideograph
        0x293b59 => [0x8f82, false], # East Asian ideograph
        0x4b4b2b => [0x7363, false], # East Asian ideograph
        0x275050 => [0x7b3c, false], # East Asian ideograph
        0x275051 => [0x7c41, false], # East Asian ideograph
        0x6f503f => [0xbab8, false], # Korean hangul
        0x213a67 => [0x5b89, false], # East Asian ideograph
        0x294952 => [0x960d, false], # East Asian ideograph
        0x275052 => [0x7b7e, false], # East Asian ideograph (duplicate simplified)
        0x6f5963 => [0xce35, false], # Korean hangul
        0x2d3b78 => [0x5cef, false], # East Asian ideograph
        0x275053 => [0x7bf1, false], # East Asian ideograph
        0x4d594e => [0x9bf5, false], # East Asian ideograph
        0x3f614c => [0x99c5, false], # East Asian ideograph
        0x275054 => [0x7ba9, false], # East Asian ideograph
        0x4b6258 => [0x68ba, false], # East Asian ideograph
        0x275055 => [0x5401, false], # East Asian ideograph
        0x6f245a => [0x3139, false], # Korean hangul
        0x225056 => [0x7088, false], # East Asian ideograph
        0x6f5040 => [0xbab9, false], # Korean hangul
        0x213a68 => [0x5b85, false], # East Asian ideograph
        0x21583e => [0x8a0c, false], # East Asian ideograph
        0x2d3b79 => [0x5d8b, false], # East Asian ideograph
        0x6f5058 => [0xbb88, false], # Korean hangul
        0x2d594f => [0x8b83, false], # East Asian ideograph
        0x225059 => [0x708c, false], # East Asian ideograph
        0x224b31 => [0x6e5d, false], # East Asian ideograph
        0x216221 => [0x9c13, false], # East Asian ideograph
        0x226222 => [0x7725, false], # East Asian ideograph
        0x216223 => [0x9bfd, false], # East Asian ideograph
        0x216224 => [0x9c2d, false], # East Asian ideograph
        0x216225 => [0x9c25, false], # East Asian ideograph
        0x226226 => [0x7734, false], # East Asian ideograph
        0x216227 => [0x9c3e, false], # East Asian ideograph
        0x216228 => [0x9c3b, false], # East Asian ideograph
        0x216229 => [0x9c54, false], # East Asian ideograph
        0x21622a => [0x9c57, false], # East Asian ideograph
        0x21622b => [0x9c56, false], # East Asian ideograph
        0x21622c => [0x9c49, false], # East Asian ideograph
        0x22622d => [0x7747, false], # East Asian ideograph
        0x21622e => [0x9c78, false], # East Asian ideograph
        0x21622f => [0x9ce5, false], # East Asian ideograph
        0x216230 => [0x9ce9, false], # East Asian ideograph
        0x226231 => [0x7745, false], # East Asian ideograph
        0x226232 => [0x774d, false], # East Asian ideograph
        0x216233 => [0x9cf3, false], # East Asian ideograph
        0x216234 => [0x9d06, false], # East Asian ideograph
        0x216235 => [0x9d09, false], # East Asian ideograph
        0x216236 => [0x9d15, false], # East Asian ideograph
        0x226237 => [0x774e, false], # East Asian ideograph
        0x216238 => [0x9d28, false], # East Asian ideograph
        0x216239 => [0x9d26, false], # East Asian ideograph
        0x22623a => [0x775f, false], # East Asian ideograph
        0x21505f => [0x7cbd, false], # East Asian ideograph
        0x21623c => [0x9d3b, false], # East Asian ideograph
        0x21623d => [0x9d3f, false], # East Asian ideograph
        0x22623e => [0x7752, false], # East Asian ideograph
        0x21623f => [0x9d51, false], # East Asian ideograph
        0x216240 => [0x9d60, false], # East Asian ideograph
        0x215060 => [0x7cb9, false], # East Asian ideograph
        0x226242 => [0x7758, false], # East Asian ideograph
        0x216243 => [0x9d72, false], # East Asian ideograph
        0x226244 => [0x7756, false], # East Asian ideograph
        0x226245 => [0x775a, false], # East Asian ideograph
        0x216246 => [0x9db4, false], # East Asian ideograph
        0x216247 => [0x9daf, false], # East Asian ideograph
        0x216248 => [0x9dc2, false], # East Asian ideograph
        0x216249 => [0x9dd3, false], # East Asian ideograph
        0x21624a => [0x9dd7, false], # East Asian ideograph
        0x21624b => [0x9de5, false], # East Asian ideograph
        0x21624c => [0x9df9, false], # East Asian ideograph
        0x215062 => [0x7cca, false], # East Asian ideograph
        0x21624e => [0x9e1a, false], # East Asian ideograph
        0x22624f => [0x7762, false], # East Asian ideograph
        0x216250 => [0x9e79, false], # East Asian ideograph
        0x216251 => [0x9e7d, false], # East Asian ideograph
        0x226252 => [0x7780, false], # East Asian ideograph
        0x216253 => [0x9e7f, false], # East Asian ideograph
        0x216254 => [0x9e82, false], # East Asian ideograph
        0x216255 => [0x9e8b, false], # East Asian ideograph
        0x226256 => [0x776f, false], # East Asian ideograph
        0x216257 => [0x9e92, false], # East Asian ideograph
        0x216258 => [0x9e93, false], # East Asian ideograph
        0x224b33 => [0x6e30, false], # East Asian ideograph
        0x21625a => [0x9e9f, false], # East Asian ideograph
        0x21625b => [0x9ea5, false], # East Asian ideograph
        0x21625c => [0x9ea9, false], # East Asian ideograph
        0x21625d => [0x9eb4, false], # East Asian ideograph
        0x21625e => [0x9eb5, false], # East Asian ideograph
        0x22625f => [0x7785, false], # East Asian ideograph
        0x216260 => [0x9ebc, false], # East Asian ideograph
        0x216261 => [0x9ebe, false], # East Asian ideograph
        0x216262 => [0x9ec3, false], # East Asian ideograph
        0x216263 => [0x9ecd, false], # East Asian ideograph
        0x216264 => [0x9ece, false], # East Asian ideograph
        0x216265 => [0x9ecf, false], # East Asian ideograph
        0x226266 => [0x778b, false], # East Asian ideograph (variant of 4C6266 which maps to 778B)
        0x216267 => [0x58a8, false], # East Asian ideograph
        0x216268 => [0x9ed8, false], # East Asian ideograph
        0x216269 => [0x9ed4, false], # East Asian ideograph
        0x22626a => [0x778d, false], # East Asian ideograph
        0x21626b => [0x9edc, false], # East Asian ideograph
        0x21626c => [0x9edb, false], # East Asian ideograph
        0x21626d => [0x9edd, false], # East Asian ideograph
        0x21626e => [0x9ee0, false], # East Asian ideograph
        0x21626f => [0x9ee8, false], # East Asian ideograph
        0x216270 => [0x9eef, false], # East Asian ideograph
        0x235068 => [0x98f1, false], # East Asian ideograph
        0x226272 => [0x7798, false], # East Asian ideograph
        0x226273 => [0x7796, false], # East Asian ideograph
        0x216274 => [0x9f0e, false], # East Asian ideograph
        0x226275 => [0x77a2, false], # East Asian ideograph
        0x226276 => [0x7799, false], # East Asian ideograph
        0x216277 => [0x9f19, false], # East Asian ideograph
        0x216278 => [0x9f20, false], # East Asian ideograph
        0x216279 => [0x9f2c, false], # East Asian ideograph
        0x22627a => [0x77b5, false], # East Asian ideograph
        0x21627b => [0x9f3b, false], # East Asian ideograph
        0x21627c => [0x9f3e, false], # East Asian ideograph
        0x22627d => [0x77b7, false], # East Asian ideograph
        0x21627e => [0x9f4b, false], # East Asian ideograph
        0x6f5044 => [0xbafc, false], # Korean hangul
        0x27506b => [0x7cae, false], # East Asian ideograph
        0x6f5964 => [0xce58, false], # Korean hangul
        0x225c41 => [0x74db, false], # East Asian ideograph
        0x236040 => [0x9f6f, false], # East Asian ideograph
        0x23506c => [0x98eb, false], # East Asian ideograph
        0x6f506d => [0xbc14, false], # Korean hangul
        0x23315e => [0x89f5, false], # East Asian ideograph
        0x6f506e => [0xbc15, false], # Korean hangul
        0x4b4049 => [0x62d0, false], # East Asian ideograph
        0x234f34 => [0x982b, false], # East Asian ideograph
        0x22506f => [0x70a7, false], # East Asian ideograph
        0x27415a => [0x5c4f, false], # East Asian ideograph
        0x696273 => [0x78b5, false], # East Asian ideograph
        0x275070 => [0x7eaa, false], # East Asian ideograph
        0x215843 => [0x8a13, false], # East Asian ideograph
        0x225071 => [0x70b5, false], # East Asian ideograph
        0x275072 => [0x7ea2, false], # East Asian ideograph
        0x295a65 => [0x9e39, false], # East Asian ideograph
        0x215073 => [0x7d09, false], # East Asian ideograph
        0x396179 => [0x5c20, false], # East Asian ideograph
        0x275074 => [0x7ea6, false], # East Asian ideograph
        0x27415b => [0x631a, false], # East Asian ideograph
        0x6f5046 => [0xbb18, false], # Korean hangul
        0x275075 => [0x7ea5, false], # East Asian ideograph
        0x215844 => [0x8a2a, false], # East Asian ideograph
        0x275076 => [0x7eba, false], # East Asian ideograph
        0x213b21 => [0x5bc6, false], # East Asian ideograph
        0x275077 => [0x7eb9, false], # East Asian ideograph
        0x213b22 => [0x5bc7, false], # East Asian ideograph
        0x235379 => [0x9a42, false], # East Asian ideograph
        0x215078 => [0x7d0a, false], # East Asian ideograph
        0x6f5729 => [0xc7b0, false], # Korean hangul
        0x213b23 => [0x5bc5, false], # East Asian ideograph
        0x6f5c72 => [0xd584, false], # Korean hangul
        0x6f5079 => [0xbc2d, false], # Korean hangul
        0x6f536d => [0xc27c, false], # Korean hangul
        0x29495a => [0x9612, false], # East Asian ideograph
        0x27507a => [0x7ead, false], # East Asian ideograph
        0x213b25 => [0x5bc2, false], # East Asian ideograph
        0x22507b => [0x70e5, false], # East Asian ideograph
        0x213b26 => [0x5bbf, false], # East Asian ideograph
        0x21507c => [0x7d15, false], # East Asian ideograph
        0x224f7b => [0x705e, false], # East Asian ideograph
        0x23537a => [0x9a44, false], # East Asian ideograph
        0x22507d => [0x70d3, false], # East Asian ideograph
        0x234f37 => [0x9820, false], # East Asian ideograph
        0x27507e => [0x7ebd, false], # East Asian ideograph
        0x335276 => [0x8061, false], # East Asian ideograph
        0x6f5048 => [0xbb35, false], # Korean hangul
        0x215846 => [0x8a1d, false], # East Asian ideograph
        0x2d3b2a => [0x5ebd, false], # East Asian ideograph
        0x2d5957 => [0x7aea, false], # East Asian ideograph
        0x4b386c => [0x5841, false], # East Asian ideograph
        0x295a68 => [0x9e3a, false], # East Asian ideograph
        0x453336 => [0x5b82, false], # East Asian ideograph
        0x224b39 => [0x6e6b, false], # East Asian ideograph
        0x213b2d => [0x5be8, false], # East Asian ideograph
        0x273b2e => [0x5bdd, false], # East Asian ideograph
        0x6f5049 => [0xbb36, false], # Korean hangul
        0x213a71 => [0x5b9b, false], # East Asian ideograph
        0x6f5965 => [0xce59, false], # Korean hangul
        0x213b2f => [0x5be4, false], # East Asian ideograph
        0x4b515a => [0x7e01, false], # East Asian ideograph
        0x216321 => [0x9f52, false], # East Asian ideograph
        0x216322 => [0x9f5f, false], # East Asian ideograph
        0x216323 => [0x9f63, false], # East Asian ideograph
        0x216324 => [0x9f61, false], # East Asian ideograph (variant of 456324 which maps to 9F61)
        0x216325 => [0x9f66, false], # East Asian ideograph
        0x216326 => [0x9f5c, false], # East Asian ideograph
        0x233b31 => [0x8ece, false], # East Asian ideograph
        0x216328 => [0x9f6a, false], # East Asian ideograph
        0x216329 => [0x9f77, false], # East Asian ideograph
        0x21632a => [0x9f72, false], # East Asian ideograph
        0x21632b => [0x9f8d, false], # East Asian ideograph
        0x22632c => [0x77bc, false], # East Asian ideograph
        0x21632d => [0x9f9c, false], # East Asian ideograph
        0x216330 => [0x8288, false], # East Asian ideograph
        0x213b33 => [0x5bdf, false], # East Asian ideograph
        0x6f504a => [0xbb38, false], # Korean hangul
        0x226335 => [0x77cd, false], # East Asian ideograph
        0x29307d => [0x89cf, false], # East Asian ideograph
        0x696733 => [0x81a4, false], # East Asian ideograph
        0x225c47 => [0x74de, false], # East Asian ideograph
        0x6f4c3f => [0xb1d4, false], # Korean hangul
        0x213b35 => [0x5bec, false], # East Asian ideograph
        0x706340 => [0x61b7, false], # East Asian ideograph
        0x45462b => [0x7688, false], # East Asian ideograph
        0x226345 => [0x77de, false], # East Asian ideograph
        0x226346 => [0x77df, false], # East Asian ideograph
        0x23537d => [0x9a48, false], # East Asian ideograph
        0x224b3b => [0x6e8b, false], # East Asian ideograph
        0x213b37 => [0x5beb, false], # East Asian ideograph
        0x335738 => [0x880f, false], # East Asian ideograph
        0x69634e => [0x7a43, false], # East Asian ideograph
        0x22634f => [0x77e7, false], # East Asian ideograph
        0x274160 => [0x63b4, false], # East Asian ideograph
        0x226352 => [0x77e6, false], # East Asian ideograph
        0x226355 => [0x77ec, false], # East Asian ideograph
        0x273b39 => [0x5b9d, false], # East Asian ideograph
        0x69254d => [0x30cd, false], # Katakana letter NE
        0x226359 => [0x77f0, false], # East Asian ideograph
        0x22635a => [0x77f1, false], # East Asian ideograph
        0x22635c => [0x77f4, false], # East Asian ideograph
        0x217b3a => [0x5a38, false], # East Asian ideograph
        0x226360 => [0x77fc, false], # East Asian ideograph
        0x213b3b => [0x5bfa, false], # East Asian ideograph
        0x23537e => [0x9a4c, false], # East Asian ideograph
        0x226367 => [0x77f8, false], # East Asian ideograph
        0x226368 => [0x77fb, false], # East Asian ideograph
        0x277b3c => [0x5a05, false], # East Asian ideograph
        0x355739 => [0x9c76, false], # East Asian ideograph
        0x234944 => [0x95ab, false], # East Asian ideograph
        0x226370 => [0x7809, false], # East Asian ideograph
        0x226371 => [0x7806, false], # East Asian ideograph
        0x226373 => [0x7819, false], # East Asian ideograph
        0x226374 => [0x7811, false], # East Asian ideograph
        0x293b3e => [0x8f71, false], # East Asian ideograph
        0x4c6376 => [0x7839, false], # East Asian ideograph
        0x226378 => [0x7812, false], # East Asian ideograph
        0x223b3f => [0x67a1, false], # East Asian ideograph
        0x213b40 => [0x5c07, false], # East Asian ideograph
        0x4b5e27 => [0x93ad, false], # East Asian ideograph
        0x273b42 => [0x5bfb, false], # East Asian ideograph
        0x6f504d => [0xbb3d, false], # Korean hangul
        0x6f5935 => [0xcc60, false], # Korean hangul
        0x294960 => [0x9619, false], # East Asian ideograph
        0x213b43 => [0x5c0d, false], # East Asian ideograph
        0x223a31 => [0x6710, false], # East Asian ideograph
        0x273b44 => [0x5bfc, false], # East Asian ideograph
        0x224b3e => [0x6e76, false], # East Asian ideograph
        0x234f3d => [0x9833, false], # East Asian ideograph
        0x2d4850 => [0x6eda, false], # East Asian ideograph
        0x293b47 => [0x8f77, false], # East Asian ideograph
        0x6f504e => [0xbb44, false], # Korean hangul
        0x275f39 => [0x968f, false], # East Asian ideograph
        0x23573f => [0x9ba8, false], # East Asian ideograph
        0x274b74 => [0x74ef, false], # East Asian ideograph
        0x217b48 => [0x5a50, false], # East Asian ideograph
        0x213b49 => [0x5c24, false], # East Asian ideograph
        0x234e3b => [0x97c5, false], # East Asian ideograph
        0x4b4053 => [0x627a, false], # East Asian ideograph
        0x213b4b => [0x5c31, false], # East Asian ideograph
        0x273b4c => [0x5c34, false], # East Asian ideograph
        0x6f504f => [0xbb47, false], # Korean hangul
        0x275f3a => [0x9669, false], # East Asian ideograph
        0x2d625f => [0x83fb, false], # East Asian ideograph
        0x213634 => [0x5501, false], # East Asian ideograph
        0x213b4e => [0x5c3a, false], # East Asian ideograph
        0x394956 => [0x792e, false], # East Asian ideograph
        0x2d7552 => [0x579b, false], # East Asian ideograph
        0x213b4f => [0x5c3c, false], # East Asian ideograph
        0x69562c => [0x599b, false], # East Asian ideograph
        0x294e54 => [0x97ea, false], # East Asian ideograph
        0x692574 => [0x30f4, false], # Katakana letter VU
        0x233b51 => [0x8eff, false], # East Asian ideograph
        0x6f5050 => [0xbb49, false], # Korean hangul
        0x275f3b => [0x9690, false], # East Asian ideograph
        0x213635 => [0x54fc, false], # East Asian ideograph
        0x27516c => [0x7f1d, false], # East Asian ideograph
        0x4b4537 => [0x6804, false], # East Asian ideograph
        0x213b54 => [0x5c46, false], # East Asian ideograph
        0x45304c => [0x69a6, false], # East Asian ideograph
        0x234f40 => [0x982e, false], # East Asian ideograph
        0x2d4853 => [0x7001, false], # East Asian ideograph
        0x224a3d => [0x6da4, false], # East Asian ideograph
        0x213b56 => [0x5c48, false], # East Asian ideograph
        0x6f5051 => [0xbb4d, false], # Korean hangul
        0x275f3c => [0x9647, false], # East Asian ideograph
        0x334277 => [0x65ef, false], # East Asian ideograph
        0x213b58 => [0x5c4b, false], # East Asian ideograph
        0x213b59 => [0x5c4d, false], # East Asian ideograph
        0x23316b => [0x89ff, false], # East Asian ideograph
        0x213b5a => [0x5c55, false], # East Asian ideograph
        0x213b5b => [0x5c51, false], # East Asian ideograph
        0x226424 => [0x781b, false], # East Asian ideograph
        0x216425 => [0x5187, false], # East Asian ideograph
        0x226426 => [0x782c, false], # East Asian ideograph
        0x226427 => [0x7823, false], # East Asian ideograph
        0x226428 => [0x782b, false], # East Asian ideograph
        0x216429 => [0x4e28, false], # East Asian ideograph
        0x22642a => [0x7829, false], # East Asian ideograph
        0x22642d => [0x7822, false], # East Asian ideograph
        0x21642e => [0x4e31, false], # East Asian ideograph
        0x2d3748 => [0x8b5f, false], # East Asian ideograph
        0x226431 => [0x7835, false], # East Asian ideograph
        0x226432 => [0x7833, false], # East Asian ideograph
        0x226433 => [0x782e, false], # East Asian ideograph
        0x216434 => [0x4e42, false], # East Asian ideograph
        0x226435 => [0x7820, false], # East Asian ideograph
        0x216437 => [0x738d, false], # East Asian ideograph
        0x226438 => [0x783d, false], # East Asian ideograph
        0x22643b => [0x781f, false], # East Asian ideograph
        0x21643c => [0x4e5c, false], # East Asian ideograph
        0x22643d => [0x7831, false], # East Asian ideograph
        0x21643f => [0x6c39, false], # East Asian ideograph
        0x274168 => [0x6320, false], # East Asian ideograph
        0x6f5053 => [0xbb50, false], # Korean hangul
        0x275f3e => [0x53ea, false], # East Asian ideograph (duplicate simplified)
        0x226444 => [0x784d, false], # East Asian ideograph
        0x216446 => [0x4e85, false], # East Asian ideograph
        0x273b61 => [0x5c42, false], # East Asian ideograph
        0x226448 => [0x7848, false], # East Asian ideograph
        0x226449 => [0x7853, false], # East Asian ideograph
        0x22644a => [0x7854, false], # East Asian ideograph
        0x22644b => [0x7845, false], # East Asian ideograph
        0x22644c => [0x7852, false], # East Asian ideograph
        0x295938 => [0x9cdf, false], # East Asian ideograph
        0x22644e => [0x7850, false], # East Asian ideograph
        0x22644f => [0x7858, false], # East Asian ideograph
        0x216450 => [0x4ea0, false], # East Asian ideograph
        0x216451 => [0x4ea2, false], # East Asian ideograph
        0x226452 => [0x7847, false], # East Asian ideograph
        0x233b63 => [0x8f27, false], # East Asian ideograph
        0x216455 => [0x4eb6, false], # East Asian ideograph (variant of 4B6455 which maps to 4EB6)
        0x226456 => [0x784c, false], # East Asian ideograph
        0x695630 => [0x5cbc, false], # East Asian ideograph
        0x216458 => [0x4eb9, false], # East Asian ideograph
        0x223b64 => [0x67b7, false], # East Asian ideograph
        0x22645a => [0x7868, false], # East Asian ideograph
        0x22645b => [0x786d, false], # East Asian ideograph
        0x21645e => [0x4ec9, false], # East Asian ideograph
        0x226460 => [0x7864, false], # East Asian ideograph
        0x226461 => [0x785c, false], # East Asian ideograph
        0x216462 => [0x4ece, false], # East Asian ideograph (not in Unicode)
        0x216463 => [0x4ee8, false], # East Asian ideograph
        0x215852 => [0x8a54, false], # East Asian ideograph
        0x213639 => [0x54fa, false], # East Asian ideograph
        0x226466 => [0x786a, false], # East Asian ideograph
        0x226469 => [0x7886, false], # East Asian ideograph
        0x21646b => [0x4ee1, false], # East Asian ideograph
        0x22646c => [0x787f, false], # East Asian ideograph
        0x22646d => [0x7887, false], # East Asian ideograph
        0x213c2a => [0x5d84, false], # East Asian ideograph
        0x226470 => [0x7894, false], # East Asian ideograph
        0x696471 => [0x7cc0, false], # East Asian ideograph
        0x216472 => [0x4f08, false], # East Asian ideograph
        0x216473 => [0x4f0e, false], # East Asian ideograph
        0x696474 => [0x7cd8, false], # East Asian ideograph
        0x216475 => [0x4f03, false], # East Asian ideograph
        0x226476 => [0x788f, false], # East Asian ideograph
        0x234f44 => [0x982f, false], # East Asian ideograph
        0x6f544a => [0xc378, false], # Korean hangul
        0x21647c => [0x4f22, false], # East Asian ideograph
        0x22647e => [0x7899, false], # East Asian ideograph
        0x213b6b => [0x5cb7, false], # East Asian ideograph
        0x225c52 => [0x74e7, false], # East Asian ideograph
        0x27516d => [0x603b, false], # East Asian ideograph
        0x223b6d => [0x6802, false], # East Asian ideograph
        0x695632 => [0x5cc5, false], # East Asian ideograph
        0x213b6e => [0x5ca1, false], # East Asian ideograph
        0x4c3a5b => [0x6859, false], # East Asian ideograph
        0x213b6f => [0x5cab, false], # East Asian ideograph
        0x6f5056 => [0xbb61, false], # Korean hangul
        0x294969 => [0x961a, false], # East Asian ideograph
        0x215854 => [0x8a50, false], # East Asian ideograph
        0x21363b => [0x54ee, false], # East Asian ideograph
        0x21762a => [0x57fb, false], # East Asian ideograph
        0x27583f => [0x8baa, false], # East Asian ideograph
        0x213b71 => [0x5cb1, false], # East Asian ideograph
        0x6f5961 => [0xce21, false], # Korean hangul
        0x213b72 => [0x5cd9, false], # East Asian ideograph
        0x275f2c => [0x9636, false], # East Asian ideograph
        0x223b74 => [0x67df, false], # East Asian ideograph
        0x6f5057 => [0xbb63, false], # Korean hangul
        0x233b75 => [0x8f17, false], # East Asian ideograph
        0x23576b => [0x9bbb, false], # East Asian ideograph
        0x213b77 => [0x5ce8, false], # East Asian ideograph
        0x216622 => [0x4fe4, false], # East Asian ideograph
        0x6f4e53 => [0xb729, false], # Korean hangul
        0x223b78 => [0x6806, false], # East Asian ideograph
        0x223b79 => [0x67ae, false], # East Asian ideograph
        0x213b7a => [0x5cea, false], # East Asian ideograph
        0x336054 => [0x985b, false], # East Asian ideograph
        0x213b7b => [0x5d07, false], # East Asian ideograph
        0x27527a => [0x804b, false], # East Asian ideograph
        0x213b7c => [0x5d06, false], # East Asian ideograph
        0x216627 => [0x4fc5, false], # East Asian ideograph
        0x6f773e => [0xcb4c, false], # Korean hangul
        0x692435 => [0x3055, false], # Hiragana letter SA
        0x233b7d => [0x8f2d, false], # East Asian ideograph
        0x455746 => [0x672f, false], # East Asian ideograph
        0x213b7e => [0x5d16, false], # East Asian ideograph
        0x6f5059 => [0xbb8c, false], # Korean hangul
        0x216629 => [0x4fc9, false], # East Asian ideograph
        0x4b516a => [0x7ef7, false], # East Asian ideograph
        0x6f7737 => [0xc5ab, false], # Korean hangul
        0x6f5a67 => [0xd0a5, false], # Korean hangul
        0x6f5d23 => [0xd5dd, false], # Korean hangul
        0x2d485c => [0x6f44, false], # East Asian ideograph
        0x6f505a => [0xbba4, false], # Korean hangul
        0x21363f => [0x54e9, false], # East Asian ideograph
        0x22262f => [0x5dae, false], # East Asian ideograph
        0x27516e => [0x7eb5, false], # East Asian ideograph
        0x283462 => [0x6322, false], # East Asian ideograph
        0x334c7b => [0x767a, false], # East Asian ideograph
        0x216527 => [0x4ef5, false], # East Asian ideograph
        0x216528 => [0x4f07, false], # East Asian ideograph
        0x226529 => [0x7893, false], # East Asian ideograph
        0x21652a => [0x4f00, false], # East Asian ideograph
        0x21652c => [0x4f0b, false], # East Asian ideograph
        0x22652d => [0x7896, false], # East Asian ideograph
        0x22652f => [0x78b2, false], # East Asian ideograph
        0x6f5371 => [0xc288, false], # Korean hangul
        0x226531 => [0x78a1, false], # East Asian ideograph
        0x216532 => [0x4f3b, false], # East Asian ideograph
        0x292633 => [0x84e3, false], # East Asian ideograph
        0x216536 => [0x4f58, false], # East Asian ideograph
        0x216537 => [0x4f62, false], # East Asian ideograph
        0x216539 => [0x4f64, false], # East Asian ideograph
        0x21653a => [0x4f49, false], # East Asian ideograph
        0x22653b => [0x78a4, false], # East Asian ideograph
        0x22653e => [0x78b4, false], # East Asian ideograph
        0x21653f => [0x4f3e, false], # East Asian ideograph
        0x226540 => [0x78ad, false], # East Asian ideograph
        0x226541 => [0x78a3, false], # East Asian ideograph
        0x226543 => [0x789e, false], # East Asian ideograph
        0x226544 => [0x78a8, false], # East Asian ideograph
        0x232636 => [0x857b, false], # East Asian ideograph
        0x214b5e => [0x746a, false], # East Asian ideograph
        0x226548 => [0x78ab, false], # East Asian ideograph
        0x234f4b => [0x9847, false], # East Asian ideograph
        0x4b6637 => [0x4fe3, false], # East Asian ideograph
        0x21654d => [0x4f68, false], # East Asian ideograph
        0x22654e => [0x78bb, false], # East Asian ideograph
        0x21654f => [0x4f5f, false], # East Asian ideograph
        0x6f505c => [0xbbc4, false], # Korean hangul
        0x29496f => [0x95fc, false], # East Asian ideograph
        0x226555 => [0x78cc, false], # East Asian ideograph
        0x226556 => [0x78c9, false], # East Asian ideograph
        0x216557 => [0x4f7c, false], # East Asian ideograph
        0x226558 => [0x78d1, false], # East Asian ideograph
        0x21655a => [0x4f98, false], # East Asian ideograph
        0x21655b => [0x4f92, false], # East Asian ideograph
        0x21655c => [0x4f7d, false], # East Asian ideograph
        0x22655e => [0x78c8, false], # East Asian ideograph
        0x226560 => [0x78d4, false], # East Asian ideograph
        0x274e3b => [0x781a, false], # East Asian ideograph
        0x216562 => [0x4f76, false], # East Asian ideograph
        0x216564 => [0x4fa2, false], # East Asian ideograph
        0x4c6565 => [0x78b9, false], # East Asian ideograph
        0x216566 => [0x4f91, false], # East Asian ideograph
        0x216567 => [0x4f95, false], # East Asian ideograph
        0x226568 => [0x78df, false], # East Asian ideograph
        0x22656a => [0x78e7, false], # East Asian ideograph
        0x21656c => [0x4f4c, false], # East Asian ideograph
        0x21656d => [0x4f97, false], # East Asian ideograph
        0x22656e => [0x78db, false], # East Asian ideograph
        0x22656f => [0x78e1, false], # East Asian ideograph
        0x216570 => [0x4f79, false], # East Asian ideograph
        0x216571 => [0x4f9a, false], # East Asian ideograph
        0x216572 => [0x4f81, false], # East Asian ideograph
        0x216573 => [0x4f78, false], # East Asian ideograph
        0x225c5a => [0x74f0, false], # East Asian ideograph
        0x4b516e => [0x7e26, false], # East Asian ideograph
        0x226576 => [0x78ee, false], # East Asian ideograph
        0x226577 => [0x78e3, false], # East Asian ideograph
        0x226579 => [0x78f2, false], # East Asian ideograph
        0x21657b => [0x4f7a, false], # East Asian ideograph
        0x21657c => [0x4fcd, false], # East Asian ideograph
        0x2d5529 => [0x830e, false], # East Asian ideograph (variant of 275529)
        0x22657e => [0x7905, false], # East Asian ideograph
        0x6f5d27 => [0xd5ec, false], # Korean hangul
        0x6f505e => [0xbbd0, false], # Korean hangul
        0x217a75 => [0x5a16, false], # East Asian ideograph
        0x224f5d => [0x7043, false], # East Asian ideograph
        0x216643 => [0x4fb9, false], # East Asian ideograph
        0x232644 => [0x8597, false], # East Asian ideograph
        0x283466 => [0x63ff, false], # East Asian ideograph
        0x6f5d28 => [0xd5f4, false], # Korean hangul
        0x287269 => [0x7ec9, false], # East Asian ideograph
        0x2d3c38 => [0x9245, false], # East Asian ideograph
        0x216646 => [0x501e, false], # East Asian ideograph
        0x217e25 => [0x5b67, false], # East Asian ideograph
        0x6f4b42 => [0xb059, false], # Korean hangul
        0x6f505f => [0xbbf8, false], # Korean hangul
        0x282647 => [0x5cbf, false], # East Asian ideograph
        0x275f4a => [0x867d, false], # East Asian ideograph
        0x225c5c => [0x74ee, false], # East Asian ideograph
        0x217633 => [0x5800, false], # East Asian ideograph
        0x23605b => [0x9f8e, false], # East Asian ideograph
        0x276649 => [0x4f1c, false], # East Asian ideograph
        0x274e3e => [0x7815, false], # East Asian ideograph
        0x293866 => [0x8db1, false], # East Asian ideograph
        0x29563c => [0x9b49, false], # East Asian ideograph
        0x6f5c73 => [0xd585, false], # Korean hangul
        0x4b3c21 => [0x5d5c, false], # East Asian ideograph
        0x21664c => [0x5007, false], # East Asian ideograph
        0x21664d => [0x5013, false], # East Asian ideograph
        0x23264e => [0x8586, false], # East Asian ideograph
        0x6f5d2a => [0xd5f7, false], # Korean hangul
        0x222650 => [0x5ddb, false], # East Asian ideograph
        0x22606a => [0x76aa, false], # East Asian ideograph
        0x292651 => [0x84df, false], # East Asian ideograph
        0x275f4c => [0x9e21, false], # East Asian ideograph
        0x696466 => [0x7cad, false], # East Asian ideograph
        0x217635 => [0x57ec, false], # East Asian ideograph
        0x6f5264 => [0xc0b3, false], # Korean hangul
        0x393770 => [0x56f2, false], # East Asian ideograph
        0x2d552d => [0x8358, false], # East Asian ideograph
        0x6f5d2b => [0xd5f9, false], # Korean hangul
        0x4b4066 => [0x62f4, false], # East Asian ideograph
        0x286655 => [0x783b, false], # East Asian ideograph
        0x4b3c23 => [0x5ce5, false], # East Asian ideograph
        0x274177 => [0x631e, false], # East Asian ideograph
        0x222656 => [0x5de4, false], # East Asian ideograph
        0x217636 => [0x5807, false], # East Asian ideograph
        0x292658 => [0x83b6, false], # East Asian ideograph
        0x282659 => [0x5def, false], # East Asian ideograph
        0x692432 => [0x3052, false], # Hiragana letter GE
        0x276068 => [0x996c, false], # East Asian ideograph
        0x706d3b => [0x7818, false], # East Asian ideograph
        0x226621 => [0x78f9, false], # East Asian ideograph
        0x226622 => [0x78fd, false], # East Asian ideograph
        0x6f5063 => [0xbc00, false], # Korean hangul
        0x216626 => [0x4fb7, false], # East Asian ideograph
        0x226627 => [0x78fe, false], # East Asian ideograph
        0x226629 => [0x78fb, false], # East Asian ideograph
        0x21662a => [0x4fe5, false], # East Asian ideograph
        0x22662b => [0x7904, false], # East Asian ideograph
        0x21662c => [0x4fe7, false], # East Asian ideograph
        0x22662e => [0x7912, false], # East Asian ideograph
        0x226632 => [0x790c, false], # East Asian ideograph
        0x216633 => [0x4fdc, false], # East Asian ideograph
        0x226634 => [0x7913, false], # East Asian ideograph
        0x216635 => [0x4fd4, false], # East Asian ideograph
        0x216637 => [0x4fc1, false], # East Asian ideograph
        0x21663b => [0x4fdb, false], # East Asian ideograph
        0x21663e => [0x4fc6, false], # East Asian ideograph
        0x706640 => [0x80ec, false], # East Asian ideograph
        0x6f5064 => [0xbc08, false], # Korean hangul
        0x226643 => [0x791e, false], # East Asian ideograph
        0x6f4c21 => [0xb128, false], # Korean hangul
        0x226646 => [0x7922, false], # East Asian ideograph
        0x292661 => [0x8360, false], # East Asian ideograph
        0x216648 => [0x503f, false], # East Asian ideograph
        0x216649 => [0x5005, false], # East Asian ideograph
        0x4d5973 => [0x51eb, false], # East Asian ideograph
        0x22664c => [0x7924, false], # East Asian ideograph
        0x22664d => [0x7927, false], # East Asian ideograph
        0x21664e => [0x5022, false], # East Asian ideograph
        0x226650 => [0x7929, false], # East Asian ideograph
        0x216652 => [0x4ff5, false], # East Asian ideograph
        0x6f2463 => [0x314d, false], # Korean hangul
        0x226655 => [0x7931, false], # East Asian ideograph
        0x393428 => [0x5227, false], # East Asian ideograph
        0x276235 => [0x9e26, false], # East Asian ideograph
        0x216659 => [0x4ff4, false], # East Asian ideograph
        0x21665b => [0x5037, false], # East Asian ideograph
        0x22665d => [0x7934, false], # East Asian ideograph
        0x21665e => [0x502e, false], # East Asian ideograph
        0x6f5065 => [0xbc09, false], # Korean hangul
        0x226660 => [0x7936, false], # East Asian ideograph
        0x216661 => [0x4ff6, false], # East Asian ideograph
        0x216662 => [0x501c, false], # East Asian ideograph
        0x6f4c22 => [0xb12c, false], # Korean hangul
        0x226665 => [0x793d, false], # East Asian ideograph
        0x216666 => [0x502c, false], # East Asian ideograph
        0x226667 => [0x7942, false], # East Asian ideograph
        0x226668 => [0x793f, false], # East Asian ideograph
        0x216669 => [0x5010, false], # East Asian ideograph
        0x22666a => [0x794a, false], # East Asian ideograph
        0x22666b => [0x794d, false], # East Asian ideograph
        0x292668 => [0x8369, false], # East Asian ideograph
        0x226675 => [0x7946, false], # East Asian ideograph
        0x226677 => [0x7958, false], # East Asian ideograph
        0x216679 => [0x503d, false], # East Asian ideograph
        0x22667a => [0x795c, false], # East Asian ideograph
        0x22667b => [0x794f, false], # East Asian ideograph
        0x22667c => [0x7953, false], # East Asian ideograph
        0x22667d => [0x7953, false], # Unrelated variant of EACC 22667C which maps to 7953
        0x6f4c23 => [0xb134, false], # Korean hangul
        0x225c63 => [0x74f8, false], # East Asian ideograph
        0x236062 => [0x9f95, false], # East Asian ideograph
        0x2d336b => [0x5c05, false], # East Asian ideograph
        0x213f71 => [0x6233, false], # East Asian ideograph
        0x6f5d30 => [0xd610, false], # Korean hangul
        0x224b57 => [0x6ea8, false], # East Asian ideograph
        0x27627d => [0x9f50, false], # East Asian ideograph
        0x6f5d77 => [0xd774, false], # Korean hangul
        0x213b2e => [0x5be2, false], # East Asian ideograph
        0x6f4c24 => [0xb135, false], # Korean hangul
        0x21763b => [0x580f, false], # East Asian ideograph
        0x227a3a => [0x81ca, false], # East Asian ideograph
        0x232672 => [0x85bf, false], # East Asian ideograph
        0x6f5528 => [0xc557, false], # Korean hangul
        0x6f5068 => [0xbc0d, false], # Korean hangul
        0x6f4c25 => [0xb137, false], # Korean hangul
        0x295a48 => [0x9e31, false], # East Asian ideograph
        0x395a36 => [0x983c, false], # East Asian ideograph
        0x6f4939 => [0xaca1, false], # Korean hangul
        0x275121 => [0x7eb1, false], # East Asian ideograph
        0x222677 => [0x5e12, false], # East Asian ideograph
        0x225122 => [0x70dd, false], # East Asian ideograph
        0x225123 => [0x70e1, false], # East Asian ideograph
        0x27417e => [0x62e9, false], # East Asian ideograph
        0x226679 => [0x795b, false], # East Asian ideograph
        0x275f54 => [0x4e91, false], # East Asian ideograph
        0x235124 => [0x9907, false], # East Asian ideograph
        0x6f4c26 => [0xb140, false], # Korean hangul
        0x225c66 => [0x74fb, false], # East Asian ideograph
        0x275125 => [0x7eb7, false], # East Asian ideograph
        0x4b3869 => [0x5727, false], # East Asian ideograph
        0x235c22 => [0x9dc7, false], # East Asian ideograph
        0x225126 => [0x70e3, false], # East Asian ideograph
        0x6f5d33 => [0xd614, false], # Korean hangul
        0x6f5127 => [0xbc85, false], # Korean hangul
        0x235128 => [0x9902, false], # East Asian ideograph
        0x6f5374 => [0xc298, false], # Korean hangul
        0x6f506a => [0xbc11, false], # Korean hangul
        0x275129 => [0x624e, false], # East Asian ideograph
        0x277d2b => [0x5a06, false], # East Asian ideograph
        0x225c67 => [0x74ff, false], # East Asian ideograph
        0x27512a => [0x7ecd, false], # East Asian ideograph
        0x21512b => [0x7d44, false], # East Asian ideograph
        0x6f4f31 => [0xb82c, false], # Korean hangul
        0x27413f => [0x626c, false], # East Asian ideograph
        0x6f5d34 => [0xd615, false], # Korean hangul
        0x27512c => [0x7ec6, false], # East Asian ideograph
        0x27512d => [0x7ec5, false], # East Asian ideograph
        0x4c7328 => [0x5fad, false], # East Asian ideograph (variant of 2E7328 which maps to 5FAD)
        0x22512e => [0x70d1, false], # East Asian ideograph
        0x215869 => [0x8aaa, false], # East Asian ideograph
        0x21512f => [0x7d40, false], # East Asian ideograph
        0x215130 => [0x7d42, false], # East Asian ideograph
        0x216722 => [0x506f, false], # East Asian ideograph
        0x216723 => [0x5050, false], # East Asian ideograph
        0x216725 => [0x5070, false], # East Asian ideograph
        0x215131 => [0x7d71, false], # East Asian ideograph
        0x216729 => [0x5053, false], # East Asian ideograph
        0x21672a => [0x506a, false], # East Asian ideograph
        0x21672c => [0x5056, false], # East Asian ideograph
        0x215132 => [0x7d5e, false], # East Asian ideograph
        0x226730 => [0x7972, false], # East Asian ideograph
        0x216731 => [0x506d, false], # East Asian ideograph
        0x275133 => [0x7ed2, false], # East Asian ideograph
        0x6f4c29 => [0xb150, false], # Korean hangul
        0x216738 => [0x505d, false], # East Asian ideograph
        0x215134 => [0x7d50, false], # East Asian ideograph
        0x21673b => [0x5058, false], # East Asian ideograph
        0x21673c => [0x5072, false], # East Asian ideograph
        0x22673e => [0x797c, false], # East Asian ideograph
        0x3f6179 => [0x5c1f, false], # East Asian ideograph
        0x216741 => [0x5041, false], # East Asian ideograph
        0x6f5d36 => [0xd638, false], # Korean hangul
        0x275136 => [0x7eda, false], # East Asian ideograph
        0x216746 => [0x5015, false], # East Asian ideograph
        0x293430 => [0x8bb4, false], # East Asian ideograph
        0x216748 => [0x507a, false], # East Asian ideograph
        0x21674a => [0x506c, false], # East Asian ideograph
        0x275137 => [0x7edd, false], # East Asian ideograph
        0x21674d => [0x506b, false], # East Asian ideograph
        0x21674e => [0x5094, false], # East Asian ideograph
        0x22674f => [0x798b, false], # East Asian ideograph
        0x216750 => [0x509e, false], # East Asian ideograph
        0x235138 => [0x9915, false], # East Asian ideograph
        0x216752 => [0x509b, false], # East Asian ideograph
        0x216753 => [0x509a, false], # East Asian ideograph
        0x226754 => [0x7994, false], # East Asian ideograph
        0x226755 => [0x7993, false], # East Asian ideograph
        0x215139 => [0x7d66, false], # East Asian ideograph
        0x21675a => [0x508c, false], # East Asian ideograph
        0x21675c => [0x5088, false], # East Asian ideograph
        0x23513a => [0x9924, false], # East Asian ideograph
        0x22675f => [0x79a1, false], # East Asian ideograph
        0x226760 => [0x799b, false], # East Asian ideograph
        0x226761 => [0x79a3, false], # East Asian ideograph
        0x216762 => [0x508e, false], # East Asian ideograph
        0x23513b => [0x991f, false], # East Asian ideograph
        0x224b5e => [0x6e8e, false], # East Asian ideograph
        0x216767 => [0x50a6, false], # East Asian ideograph
        0x274c76 => [0x763e, false], # East Asian ideograph
        0x21513c => [0x7d93, false], # East Asian ideograph
        0x21676a => [0x5092, false], # East Asian ideograph
        0x21676c => [0x509c, false], # East Asian ideograph
        0x2d442d => [0x6780, false], # East Asian ideograph
        0x22676e => [0x79a9, false], # East Asian ideograph
        0x27513d => [0x6346, false], # East Asian ideograph
        0x226770 => [0x79ab, false], # East Asian ideograph
        0x216771 => [0x50c7, false], # East Asian ideograph
        0x216775 => [0x50c9, false], # East Asian ideograph
        0x22677a => [0x79b3, false], # East Asian ideograph
        0x22513f => [0x70fa, false], # East Asian ideograph
        0x21677c => [0x50b4, false], # East Asian ideograph
        0x6f5d38 => [0xd63c, false], # Korean hangul
        0x212a23 => [0xe8d2, false], # EACC component character
        0x215140 => [0x7d81, false], # East Asian ideograph
        0x334f5e => [0x7a91, false], # East Asian ideograph
        0x215141 => [0x7d9c, false], # East Asian ideograph
        0x6f5375 => [0xc29b, false], # Korean hangul
        0x213538 => [0x53f0, false], # East Asian ideograph (duplicate simplified)
        0x6f506f => [0xbc16, false], # Korean hangul
        0x215142 => [0x7dbb, false], # East Asian ideograph
        0x6f4c2c => [0xb154, false], # Korean hangul
        0x284140 => [0x6861, false], # East Asian ideograph
        0x235143 => [0x9929, false], # East Asian ideograph
        0x2d3765 => [0x8086, false], # East Asian ideograph
        0x215144 => [0x7dca, false], # East Asian ideograph
        0x6f5d39 => [0xd640, false], # Korean hangul
        0x215145 => [0x7dbe, false], # East Asian ideograph
        0x224b60 => [0x6ed9, false], # East Asian ideograph
        0x215146 => [0x7db4, false], # East Asian ideograph
        0x2e2d79 => [0x6128, false], # East Asian ideograph
        0x4b5724 => [0x86cd, false], # East Asian ideograph
        0x235147 => [0x991a, false], # East Asian ideograph
        0x275242 => [0x4e49, false], # East Asian ideograph
        0x6f4c2d => [0xb155, false], # Korean hangul
        0x6f5c48 => [0xd46f, false], # Korean hangul
        0x215148 => [0x7db2, false], # East Asian ideograph
        0x215149 => [0x7db1, false], # East Asian ideograph
        0x6f5d3a => [0xd648, false], # Korean hangul
        0x21514a => [0x7dbd, false], # East Asian ideograph
        0x224b61 => [0x6ebd, false], # East Asian ideograph
        0x234f60 => [0x9852, false], # East Asian ideograph
        0x4b3c32 => [0x5dd3, false], # East Asian ideograph
        0x6f5071 => [0xbc1b, false], # Korean hangul
        0x22514c => [0x7103, false], # East Asian ideograph
        0x21586f => [0x8aa3, false], # East Asian ideograph
        0x21514d => [0x7da2, false], # East Asian ideograph
        0x22582b => [0x736b, false], # East Asian ideograph
        0x4b615f => [0x9aea, false], # East Asian ideograph
        0x21514e => [0x7dad, false], # East Asian ideograph
        0x2d3324 => [0x634c, false], # East Asian ideograph
        0x6f5d3b => [0xd649, false], # Korean hangul
        0x21514f => [0x7dbf, false], # East Asian ideograph
        0x2d356a => [0x8a36, false], # East Asian ideograph
        0x215150 => [0x7db8, false], # East Asian ideograph
        0x6f5072 => [0xbc1c, false], # Korean hangul
        0x215151 => [0x7dc7, false], # East Asian ideograph
        0x22482d => [0x6cf2, false], # East Asian ideograph
        0x275152 => [0x7f14, false], # East Asian ideograph
        0x6f493b => [0xaca9, false], # Korean hangul
        0x2d3768 => [0x56ec, false], # East Asian ideograph
        0x215153 => [0x7def, false], # East Asian ideograph
        0x294346 => [0x94d1, false], # East Asian ideograph
        0x2f5d3c => [0x6ef7, false], # East Asian ideograph
        0x215154 => [0x7df4, false], # East Asian ideograph (variant of 4B5154 which maps to 7DF4)
        0x224b63 => [0x6ec1, false], # East Asian ideograph
        0x234f62 => [0x984b, false], # East Asian ideograph
        0x235155 => [0x9932, false], # East Asian ideograph
        0x6f5073 => [0xbc1d, false], # Korean hangul
        0x225156 => [0x7112, false], # East Asian ideograph
        0x6f4c30 => [0xb178, false], # Korean hangul
        0x69675c => [0x825d, false], # East Asian ideograph
        0x215157 => [0x7dec, false], # East Asian ideograph
        0x234179 => [0x91e9, false], # East Asian ideograph
        0x215158 => [0x7ddd, false], # East Asian ideograph
        0x213e37 => [0x6012, false], # East Asian ideograph
        0x6f5d3d => [0xd64d, false], # Korean hangul
        0x215159 => [0x7de9, false], # East Asian ideograph
        0x21515a => [0x7de3, false], # East Asian ideograph
        0x213539 => [0x53e5, false], # East Asian ideograph
        0x6f5074 => [0xbc1f, false], # Korean hangul
        0x216822 => [0x50c2, false], # East Asian ideograph
        0x27515b => [0x7f16, false], # East Asian ideograph
        0x226825 => [0x79bc, false], # East Asian ideograph
        0x225c71 => [0x7505, false], # East Asian ideograph
        0x226828 => [0x79c6, false], # East Asian ideograph
        0x22515c => [0x710c, false], # East Asian ideograph
        0x22682a => [0x79c8, false], # East Asian ideograph
        0x21682c => [0x50ba, false], # East Asian ideograph
        0x22682d => [0x79d4, false], # East Asian ideograph
        0x21682e => [0x50cd, false], # East Asian ideograph
        0x21515d => [0x7d9e, false], # East Asian ideograph
        0x6f4f33 => [0xb835, false], # Korean hangul
        0x226832 => [0x79d6, false], # East Asian ideograph
        0x216834 => [0x50ef, false], # East Asian ideograph
        0x21515e => [0x7dde, false], # East Asian ideograph
        0x293438 => [0x8c29, false], # East Asian ideograph
        0x21683a => [0x50f4, false], # East Asian ideograph
        0x21515f => [0x7e11, false], # East Asian ideograph
        0x21683c => [0x50dd, false], # East Asian ideograph
        0x22683d => [0x79ec, false], # East Asian ideograph
        0x22683e => [0x79eb, false], # East Asian ideograph (variant of 4C683E which maps to 79EB)
        0x6f5075 => [0xbc24, false], # Korean hangul
        0x215160 => [0x7e0a, false], # East Asian ideograph
        0x226842 => [0x79e1, false], # East Asian ideograph
        0x6f4c32 => [0xb17a, false], # Korean hangul
        0x226844 => [0x79dd, false], # East Asian ideograph
        0x226845 => [0x79ed, false], # East Asian ideograph
        0x216846 => [0x50d9, false], # East Asian ideograph
        0x215161 => [0x7e08, false], # East Asian ideograph
        0x226848 => [0x79f8, false], # East Asian ideograph
        0x215162 => [0x7e1b, false], # East Asian ideograph
        0x2e684e => [0x8020, false], # East Asian ideograph
        0x22684f => [0x7a02, false], # East Asian ideograph
        0x226850 => [0x7a0a, false], # East Asian ideograph
        0x6f5d3f => [0xd654, false], # Korean hangul
        0x6f5625 => [0xc651, false], # Korean hangul
        0x275163 => [0x81f4, false], # East Asian ideograph
        0x226854 => [0x7a09, false], # East Asian ideograph
        0x216855 => [0x50ec, false], # East Asian ideograph
        0x4b442d => [0x67a9, false], # East Asian ideograph
        0x215164 => [0x7e23, false], # East Asian ideograph
        0x21685b => [0x510e, false], # East Asian ideograph
        0x22685c => [0x7a03, false], # East Asian ideograph
        0x6f5076 => [0xbc25, false], # Korean hangul
        0x275165 => [0x7f29, false], # East Asian ideograph
        0x226861 => [0x7a0c, false], # East Asian ideograph
        0x293160 => [0x89ef, false], # East Asian ideograph
        0x225166 => [0x7113, false], # East Asian ideograph
        0x216866 => [0x5107, false], # East Asian ideograph
        0x216867 => [0x510f, false], # East Asian ideograph
        0x216868 => [0x50fe, false], # East Asian ideograph
        0x216869 => [0x510b, false], # East Asian ideograph
        0x21686a => [0x50fd, false], # East Asian ideograph
        0x22686b => [0x7a11, false], # East Asian ideograph
        0x22686c => [0x7a18, false], # East Asian ideograph
        0x21686d => [0x5101, false], # East Asian ideograph
        0x234e43 => [0x97c9, false], # East Asian ideograph
        0x22686f => [0x7a19, false], # East Asian ideograph (variant of 2E686F which maps to 7A19)
        0x6f5d40 => [0xd655, false], # Korean hangul
        0x226871 => [0x7a1e, false], # East Asian ideograph
        0x216872 => [0x5113, false], # East Asian ideograph
        0x234f66 => [0x983f, false], # East Asian ideograph
        0x226876 => [0x7a17, false], # East Asian ideograph
        0x275169 => [0x7f27, false], # East Asian ideograph
        0x216878 => [0x511a, false], # East Asian ideograph
        0x216879 => [0x9797, false], # East Asian ideograph
        0x6f5077 => [0xbc27, false], # Korean hangul
        0x21516a => [0x7e43, false], # East Asian ideograph
        0x21687e => [0x5126, false], # East Asian ideograph
        0x6f4c34 => [0xb180, false], # Korean hangul
        0x223a5b => [0x6745, false], # East Asian ideograph
        0x33516b => [0x7dd0, false], # East Asian ideograph
        0x4c735d => [0x7d4b, false], # East Asian ideograph
        0x22516c => [0x711e, false], # East Asian ideograph
        0x2e3729 => [0x65b5, false], # East Asian ideograph
        0x6f5d41 => [0xd658, false], # Korean hangul
        0x21516d => [0x7e3d, false], # East Asian ideograph
        0x6f5451 => [0xc3d8, false], # Korean hangul
        0x22516e => [0x7120, false], # East Asian ideograph
        0x2d4437 => [0x67fe, false], # East Asian ideograph
        0x21516f => [0x7e45, false], # East Asian ideograph
        0x6f4c35 => [0xb188, false], # Korean hangul
        0x215170 => [0x7e55, false], # East Asian ideograph
        0x275174 => [0x7f2d, false], # East Asian ideograph
        0x235171 => [0x994d, false], # East Asian ideograph
        0x473539 => [0x8b9e, false], # East Asian ideograph
        0x29426d => [0x94cd, false], # East Asian ideograph
        0x275172 => [0x7ee3, false], # East Asian ideograph
        0x224b69 => [0x6ebb, false], # East Asian ideograph
        0x275173 => [0x7ed5, false], # East Asian ideograph
        0x6f5b23 => [0xd0f0, false], # Korean hangul
        0x215174 => [0x7e5a, false], # East Asian ideograph
        0x6f4c36 => [0xb189, false], # Korean hangul
        0x225175 => [0x712d, false], # East Asian ideograph
        0x2d376f => [0x5700, false], # East Asian ideograph
        0x275176 => [0x7ef3, false], # East Asian ideograph
        0x213c21 => [0x5d0e, false], # East Asian ideograph
        0x275177 => [0x8327, false], # East Asian ideograph
        0x2d3c22 => [0x5d10, false], # East Asian ideograph
        0x275178 => [0x7ece, false], # East Asian ideograph
        0x223c23 => [0x67c2, false], # East Asian ideograph
        0x6f507a => [0xbc30, false], # Korean hangul
        0x275179 => [0x7ed8, false], # East Asian ideograph
        0x215878 => [0x8ad2, false], # East Asian ideograph
        0x225c77 => [0x7503, false], # East Asian ideograph
        0x27517a => [0x8fab, false], # East Asian ideograph
        0x217c25 => [0x5a9e, false], # East Asian ideograph
        0x45465b => [0x6c2f, false], # East Asian ideograph
        0x21517b => [0x7e7d, false], # East Asian ideograph
        0x223c26 => [0x67ca, false], # East Asian ideograph
        0x274e59 => [0x7840, false], # East Asian ideograph
        0x6f5d44 => [0xd667, false], # Korean hangul
        0x6f517c => [0xbe61, false], # Korean hangul
        0x213c27 => [0x5d4c, false], # East Asian ideograph
        0x234f6a => [0x985c, false], # East Asian ideograph
        0x27517d => [0x7ee7, false], # East Asian ideograph
        0x223c28 => [0x67ce, false], # East Asian ideograph
        0x2d443a => [0x6942, false], # East Asian ideograph
        0x23517e => [0x9955, false], # East Asian ideograph
        0x213b32 => [0x5be7, false], # East Asian ideograph
        0x213c29 => [0x5d69, false], # East Asian ideograph
        0x223c2a => [0x67f2, false], # East Asian ideograph
        0x275e3e => [0x94db, false], # East Asian ideograph
        0x2d5547 => [0x837d, false], # East Asian ideograph
        0x223c2b => [0x67c3, false], # East Asian ideograph
        0x6f5d45 => [0xd669, false], # Korean hangul
        0x234f6b => [0x9859, false], # East Asian ideograph
        0x223c2d => [0x67dd, false], # East Asian ideograph
        0x6f507c => [0xbc34, false], # Korean hangul
        0x275f67 => [0x96fe, false], # East Asian ideograph
        0x213c2e => [0x5dbd, false], # East Asian ideograph
        0x6f4d43 => [0xb3d0, false], # Korean hangul
        0x233e5f => [0x90ad, false], # East Asian ideograph
        0x213c2f => [0x5dba, false], # East Asian ideograph (variant of 4B3C2F which maps to 5DBA)
        0x6f493d => [0xacac, false], # Korean hangul
        0x233c30 => [0x8f46, false], # East Asian ideograph
        0x226922 => [0x7a2c, false], # East Asian ideograph
        0x6f5d46 => [0xd670, false], # Korean hangul
        0x233c31 => [0x8f4a, false], # East Asian ideograph
        0x216929 => [0x5124, false], # East Asian ideograph
        0x6f5452 => [0xc3d9, false], # Korean hangul
        0x21692b => [0x5129, false], # East Asian ideograph
        0x213c32 => [0x5dd4, false], # East Asian ideograph
        0x6f507d => [0xbc37, false], # Korean hangul
        0x216930 => [0x5131, false], # East Asian ideograph
        0x273c33 => [0x5ca9, false], # East Asian ideograph
        0x29454d => [0x9534, false], # East Asian ideograph
        0x275175 => [0x7cfb, false], # East Asian ideograph (duplicate simplified)
        0x226939 => [0x7a48, false], # East Asian ideograph
        0x22693d => [0x7a4b, false], # East Asian ideograph
        0x22693e => [0x7a47, false], # East Asian ideograph
        0x22693f => [0x7a44, false], # East Asian ideograph
        0x274e5c => [0x77fe, false], # East Asian ideograph
        0x6f5d47 => [0xd671, false], # Korean hangul
        0x216944 => [0x513a, false], # East Asian ideograph
        0x213c36 => [0x5de2, false], # East Asian ideograph
        0x696946 => [0x8630, false], # East Asian ideograph
        0x216947 => [0x5139, false], # East Asian ideograph
        0x216948 => [0x513b, false], # East Asian ideograph
        0x213c37 => [0x5de5, false], # East Asian ideograph
        0x22694d => [0x7a5f, false], # East Asian ideograph
        0x22694f => [0x7a60, false], # East Asian ideograph
        0x216951 => [0x5159, false], # East Asian ideograph
        0x216952 => [0x515b, false], # East Asian ideograph
        0x213663 => [0x55aa, false], # East Asian ideograph
        0x29454e => [0x9545, false], # East Asian ideograph
        0x216955 => [0x515d, false], # East Asian ideograph
        0x216956 => [0x515e, false], # East Asian ideograph
        0x225838 => [0x737e, false], # East Asian ideograph
        0x216958 => [0x515f, false], # East Asian ideograph
        0x216959 => [0x5161, false], # East Asian ideograph
        0x69695b => [0x86ab, false], # East Asian ideograph
        0x21695c => [0x5163, false], # East Asian ideograph
        0x6f4f35 => [0xb838, false], # Korean hangul
        0x274e5d => [0x783a, false], # East Asian ideograph
        0x22695f => [0x7a70, false], # East Asian ideograph
        0x6f5d48 => [0xd683, false], # Korean hangul
        0x696962 => [0x86ef, false], # East Asian ideograph
        0x213c3b => [0x5deb, false], # East Asian ideograph
        0x226966 => [0x7a75, false], # East Asian ideograph
        0x216967 => [0x5182, false], # East Asian ideograph
        0x216969 => [0x5184, false], # East Asian ideograph
        0x22696b => [0x7a80, false], # East Asian ideograph
        0x21696e => [0x518f, false], # East Asian ideograph
        0x213c3d => [0x5df1, false], # East Asian ideograph
        0x216970 => [0x5194, false], # East Asian ideograph
        0x216971 => [0x5193, false], # East Asian ideograph
        0x2e403d => [0x6ac1, false], # East Asian ideograph
        0x216975 => [0x5196, false], # East Asian ideograph
        0x226978 => [0x7a8a, false], # East Asian ideograph
        0x22697a => [0x7a94, false], # East Asian ideograph
        0x21697b => [0x51a1, false], # East Asian ideograph
        0x21697c => [0x51a3, false], # East Asian ideograph
        0x22697e => [0x68a5, false], # East Asian ideograph
        0x213c40 => [0x5df4, false], # East Asian ideograph
        0x223c41 => [0x6832, false], # East Asian ideograph
        0x213c42 => [0x5dfd, false], # East Asian ideograph
        0x275b28 => [0x8e0c, false], # East Asian ideograph
        0x213c43 => [0x5dfe, false], # East Asian ideograph
        0x275e3f => [0x94ce, false], # East Asian ideograph
        0x213c44 => [0x5e02, false], # East Asian ideograph
        0x6f5471 => [0xc510, false], # Korean hangul
        0x29565d => [0x9c82, false], # East Asian ideograph
        0x217c45 => [0x5ab7, false], # East Asian ideograph
        0x2d4440 => [0x6822, false], # East Asian ideograph
        0x223c47 => [0x682b, false], # East Asian ideograph
        0x294551 => [0x9517, false], # East Asian ideograph
        0x223c48 => [0x682d, false], # East Asian ideograph
        0x6f493e => [0xacaf, false], # Korean hangul
        0x235c3a => [0x9ddf, false], # East Asian ideograph
        0x233c49 => [0x8f57, false], # East Asian ideograph
        0x6f5d4b => [0xd68d, false], # Korean hangul
        0x213c4a => [0x5e16, false], # East Asian ideograph
        0x334f71 => [0x54b2, false], # East Asian ideograph
        0x213c4b => [0x5e15, false], # East Asian ideograph
        0x275f6d => [0x972d, false], # East Asian ideograph
        0x213c4c => [0x5e1b, false], # East Asian ideograph
        0x4b3321 => [0x5185, false], # East Asian ideograph
        0x233c4d => [0x8f5c, false], # East Asian ideograph
        0x213c4e => [0x5e1d, false], # East Asian ideograph
        0x284f7d => [0x704f, false], # East Asian ideograph
        0x29426f => [0x94bd, false], # East Asian ideograph
        0x223c4f => [0x6844, false], # East Asian ideograph
        0x4b5e5d => [0x95d4, false], # East Asian ideograph
        0x224730 => [0x6c78, false], # East Asian ideograph
        0x6f5379 => [0xc2a8, false], # Korean hangul
        0x217c50 => [0x5aba, false], # East Asian ideograph
        0x275f6e => [0x96f3, false], # East Asian ideograph
        0x213c51 => [0x5e2b, false], # East Asian ideograph
        0x6f5b26 => [0xd131, false], # Korean hangul
        0x213668 => [0x55ae, false], # East Asian ideograph
        0x232635 => [0x8598, false], # East Asian ideograph
        0x213c52 => [0x5e33, false], # East Asian ideograph
        0x233c53 => [0x8f5d, false], # East Asian ideograph
        0x6f5d4d => [0xd6a1, false], # Korean hangul
        0x224731 => [0x6c74, false], # East Asian ideograph
        0x213c55 => [0x5e37, false], # East Asian ideograph
        0x275f6f => [0x7075, false], # East Asian ideograph
        0x213c56 => [0x5e45, false], # East Asian ideograph
        0x6f4c41 => [0xb1e8, false], # Korean hangul
        0x275b2c => [0x8e8f, false], # East Asian ideograph
        0x213226 => [0x5000, false], # East Asian ideograph
        0x223c58 => [0x6834, false], # East Asian ideograph
        0x334f37 => [0x5ee9, false], # East Asian ideograph
        0x695d36 => [0x6b1f, false], # East Asian ideograph
        0x223c59 => [0x6812, false], # East Asian ideograph
        0x224732 => [0x6c86, false], # East Asian ideograph
        0x213c5a => [0x5e5b, false], # East Asian ideograph
        0x216a22 => [0x51aa, false], # East Asian ideograph
        0x216a23 => [0x51ab, false], # East Asian ideograph
        0x6f4c42 => [0xb1fd, false], # Korean hangul
        0x216a26 => [0x51b1, false], # East Asian ideograph
        0x29233c => [0x836d, false], # East Asian ideograph
        0x226a28 => [0x7aa3, false], # East Asian ideograph
        0x213227 => [0x4fee, false], # East Asian ideograph
        0x226a2b => [0x7a9e, false], # East Asian ideograph
        0x226a2c => [0x7aa7, false], # East Asian ideograph
        0x226a2e => [0x7aa8, false], # East Asian ideograph
        0x46284c => [0x5ed0, false], # East Asian ideograph
        0x226a31 => [0x7aac, false], # East Asian ideograph
        0x6f5d4f => [0xd6c4, false], # Korean hangul
        0x216a35 => [0x51bc, false], # East Asian ideograph
        0x226a36 => [0x7ab3, false], # East Asian ideograph
        0x226a3a => [0x7abd, false], # East Asian ideograph
        0x2d3c5f => [0x6a66, false], # East Asian ideograph
        0x226a3c => [0x7ab6, false], # East Asian ideograph
        0x226a3d => [0x7ab8, false], # East Asian ideograph
        0x226a3e => [0x7ab5, false], # East Asian ideograph
        0x226a3f => [0x7abb, false], # East Asian ideograph
        0x213c60 => [0x5e5f, false], # East Asian ideograph
        0x6f4c43 => [0xb204, false], # Korean hangul
        0x216a43 => [0x51ca, false], # East Asian ideograph
        0x216a46 => [0x51c7, false], # East Asian ideograph
        0x213c61 => [0x5e6b, false], # East Asian ideograph
        0x226a49 => [0x7acd, false], # East Asian ideograph
        0x226a4b => [0x7acf, false], # East Asian ideograph
        0x216a4e => [0x51d1, false], # East Asian ideograph
        0x216a4f => [0x51d0, false], # East Asian ideograph
        0x287271 => [0x7ea4, false], # East Asian ideograph (duplicate simplified)
        0x226a51 => [0x7ad3, false], # East Asian ideograph
        0x226a52 => [0x7ad4, false], # East Asian ideograph
        0x216a54 => [0x51d3, false], # East Asian ideograph
        0x226a55 => [0x7ada, false], # East Asian ideograph
        0x226a5a => [0x7ae1, false], # East Asian ideograph
        0x226a5e => [0x7ae6, false], # East Asian ideograph
        0x233c65 => [0x8fa4, false], # East Asian ideograph
        0x277d48 => [0x5ad2, false], # East Asian ideograph
        0x696a61 => [0x88c3, false], # East Asian ideograph
        0x21765b => [0x57dd, false], # East Asian ideograph
        0x216a63 => [0x51d9, false], # East Asian ideograph
        0x226a66 => [0x7aeb, false], # East Asian ideograph
        0x216a68 => [0x51e2, false], # East Asian ideograph
        0x226a6b => [0x7af0, false], # East Asian ideograph
        0x696a6d => [0x8904, false], # East Asian ideograph
        0x6f5d51 => [0xd6c8, false], # Korean hangul
        0x213c68 => [0x5e7b, false], # East Asian ideograph
        0x216a73 => [0x5160, false], # East Asian ideograph
        0x226a76 => [0x7af5, false], # East Asian ideograph
        0x213c69 => [0x5e7c, false], # East Asian ideograph
        0x216a78 => [0x51f5, false], # East Asian ideograph
        0x216a79 => [0x51f7, false], # East Asian ideograph
        0x226a7c => [0x7afe, false], # East Asian ideograph
        0x2d3c6a => [0x51fc, false], # East Asian ideograph
        0x273c6b => [0x51e0, false], # East Asian ideograph
        0x4b4d56 => [0x8846, false], # East Asian ideograph
        0x2d5554 => [0x855a, false], # East Asian ideograph
        0x213c6c => [0x5e8f, false], # East Asian ideograph
        0x6f5d52 => [0xd6cc, false], # Korean hangul
        0x233c6d => [0x8fb7, false], # East Asian ideograph
        0x295222 => [0x98e8, false], # East Asian ideograph
        0x234b35 => [0x96a9, false], # East Asian ideograph
        0x227333 => [0x7e50, false], # East Asian ideograph
        0x6f4c46 => [0xb20b, false], # Korean hangul
        0x275b31 => [0x8eaf, false], # East Asian ideograph
        0x213c70 => [0x5e97, false], # East Asian ideograph
        0x22326a => [0x63f9, false], # East Asian ideograph
        0x223c71 => [0x689b, false], # East Asian ideograph
        0x6f5d53 => [0xd6d1, false], # Korean hangul
        0x213c72 => [0x5e9c, false], # East Asian ideograph
        0x29344d => [0x8c2e, false], # East Asian ideograph
        0x6f5972 => [0xce7c, false], # Korean hangul
        0x223c74 => [0x68b6, false], # East Asian ideograph
        0x6f4c47 => [0xb20c, false], # Korean hangul
        0x275b32 => [0x8f66, false], # East Asian ideograph
        0x213c75 => [0x5ea6, false], # East Asian ideograph
        0x223c76 => [0x6882, false], # East Asian ideograph
        0x226721 => [0x7951, false], # East Asian ideograph
        0x6f5d54 => [0xd6d4, false], # Korean hangul
        0x273c77 => [0x5750, false], # East Asian ideograph
        0x23595c => [0x9c6f, false], # East Asian ideograph
        0x234b37 => [0x96ae, false], # East Asian ideograph
        0x226723 => [0x7954, false], # East Asian ideograph
        0x28232b => [0x5c66, false], # East Asian ideograph
        0x232724 => [0x8624, false], # East Asian ideograph
        0x223c7a => [0x6890, false], # East Asian ideograph
        0x4b4d59 => [0x775b, false], # East Asian ideograph (variant of 214D59 which maps to 775B)
        0x2f317d => [0x8a7e, false], # East Asian ideograph
        0x213c7b => [0x5eb6, false], # East Asian ideograph
        0x6f5d55 => [0xd6d7, false], # Korean hangul
        0x275d67 => [0x94dd, false], # East Asian ideograph
        0x217c7c => [0x5aeb, false], # East Asian ideograph
        0x2d462c => [0x6b7a, false], # East Asian ideograph
        0x224739 => [0x6c67, false], # East Asian ideograph
        0x233c7d => [0x8fcd, false], # East Asian ideograph
        0x4b5a23 => [0x621d, false], # East Asian ideograph
        0x213c7e => [0x5ec1, false], # East Asian ideograph
        0x2d4756 => [0x6f94, false], # East Asian ideograph
        0x275b34 => [0x519b, false], # East Asian ideograph
        0x4b5c47 => [0x9059, false], # East Asian ideograph
        0x22672a => [0x7967, false], # East Asian ideograph
        0x235c45 => [0x9dd6, false], # East Asian ideograph
        0x6f4866 => [0xac10, false], # Korean hangul
        0x6f5b27 => [0xd134, false], # Korean hangul
        0x4c6266 => [0x778b, false], # East Asian ideograph
        0x22672d => [0x796b, false], # East Asian ideograph
        0x2d6222 => [0x9c0c, false], # East Asian ideograph
        0x6f4c4a => [0xb215, false], # Korean hangul
        0x275b35 => [0x8f68, false], # East Asian ideograph
        0x6f4f38 => [0xb85c, false], # Korean hangul
        0x33476f => [0x6d44, false], # East Asian ideograph
        0x6f5d57 => [0xd6e4, false], # Korean hangul
        0x216b24 => [0x5213, false], # East Asian ideograph
        0x216b26 => [0x5216, false], # East Asian ideograph
        0x226b27 => [0x7b39, false], # East Asian ideograph
        0x22473b => [0x6c84, false], # East Asian ideograph
        0x216b2a => [0x521c, false], # East Asian ideograph
        0x226b2d => [0x7b0f, false], # East Asian ideograph
        0x226b2e => [0x7b08, false], # East Asian ideograph
        0x275f79 => [0x9765, false], # East Asian ideograph
        0x6f4c4b => [0xb217, false], # Korean hangul
        0x226b33 => [0x7b0a, false], # East Asian ideograph
        0x29455e => [0x94e1, false], # East Asian ideograph
        0x226b35 => [0x7b35, false], # East Asian ideograph
        0x226b36 => [0x7b25, false], # East Asian ideograph
        0x216b37 => [0x5232, false], # East Asian ideograph
        0x226b39 => [0x7b38, false], # East Asian ideograph
        0x226b3b => [0x7b3b, false], # East Asian ideograph
        0x216b3e => [0x5244, false], # East Asian ideograph
        0x226b3f => [0x7b24, false], # East Asian ideograph
        0x226b40 => [0x7b33, false], # East Asian ideograph
        0x226b42 => [0x7b2a, false], # East Asian ideograph
        0x216b43 => [0x5249, false], # East Asian ideograph
        0x226b44 => [0x7b18, false], # East Asian ideograph
        0x282736 => [0x5e0f, false], # East Asian ideograph
        0x226b47 => [0x7b31, false], # East Asian ideograph
        0x234b3b => [0x96b0, false], # East Asian ideograph
        0x226b4a => [0x7b2b, false], # East Asian ideograph
        0x216b4b => [0x525a, false], # East Asian ideograph
        0x216b4c => [0x5252, false], # East Asian ideograph
        0x226b4d => [0x7b1f, false], # East Asian ideograph
        0x213b36 => [0x5be9, false], # East Asian ideograph
        0x216b50 => [0x525f, false], # East Asian ideograph
        0x226b52 => [0x7b4a, false], # East Asian ideograph
        0x226b53 => [0x7b59, false], # East Asian ideograph (not in Unicode)
        0x226b54 => [0x7b04, false], # East Asian ideograph (variant of 2E6B54 which maps to 7B04)
        0x226b55 => [0x7b47, false], # East Asian ideograph
        0x216739 => [0x5048, false], # East Asian ideograph
        0x226b59 => [0x7b58, false], # East Asian ideograph
        0x226b5b => [0x7b6c, false], # East Asian ideograph
        0x696b5c => [0x8ada, false], # East Asian ideograph
        0x216b5e => [0x5268, false], # East Asian ideograph
        0x216b5f => [0x7b9a, false], # East Asian ideograph
        0x226b60 => [0x7b48, false], # East Asian ideograph
        0x226b61 => [0x7b45, false], # East Asian ideograph
        0x226b62 => [0x7b4c, false], # East Asian ideograph
        0x226b63 => [0x7b4e, false], # East Asian ideograph
        0x234b3c => [0x96b2, false], # East Asian ideograph
        0x226b68 => [0x7b66, false], # East Asian ideograph
        0x706b6a => [0x8159, false], # East Asian ideograph
        0x216b6b => [0x5278, false], # East Asian ideograph
        0x226b6c => [0x7b64, false], # East Asian ideograph
        0x226b6e => [0x7b69, false], # East Asian ideograph
        0x275b38 => [0x8f6f, false], # East Asian ideograph
        0x226b70 => [0x7b6d, false], # East Asian ideograph
        0x226b74 => [0x7b62, false], # East Asian ideograph
        0x226b75 => [0x7b6e, false], # East Asian ideograph
        0x226b76 => [0x7b74, false], # East Asian ideograph
        0x233a30 => [0x8e50, false], # East Asian ideograph
        0x216b79 => [0x528c, false], # East Asian ideograph
        0x216b7a => [0x528a, false], # East Asian ideograph
        0x226b7b => [0x7b6f, false], # East Asian ideograph
        0x216b7c => [0x5290, false], # East Asian ideograph
        0x226b7e => [0x7b65, false], # East Asian ideograph
        0x4b3a2f => [0x805f, false], # East Asian ideograph
        0x275b39 => [0x8f6d, false], # East Asian ideograph
        0x6f4867 => [0xac11, false], # Korean hangul
        0x27457a => [0x6b20, false], # East Asian ideograph (duplicate simplified)
        0x222969 => [0x5f54, false], # East Asian ideograph
        0x4b3c53 => [0x5e2f, false], # East Asian ideograph
        0x234b3e => [0x96b3, false], # East Asian ideograph
        0x4c6564 => [0x78d9, false], # East Asian ideograph
        0x282747 => [0x5e3b, false], # East Asian ideograph
        0x22584c => [0x7393, false], # East Asian ideograph
        0x6f4f39 => [0xb85d, false], # Korean hangul
        0x2f5d5c => [0x730a, false], # East Asian ideograph
        0x294944 => [0x9603, false], # East Asian ideograph
        0x22674a => [0x7998, false], # East Asian ideograph
        0x33306c => [0x8b90, false], # East Asian ideograph
        0x21674b => [0x505f, false], # East Asian ideograph
        0x273d65 => [0x540e, false], # East Asian ideograph
        0x6f4c50 => [0xb25c, false], # Korean hangul
        0x27583b => [0x8ba6, false], # East Asian ideograph
        0x213235 => [0x5074, false], # East Asian ideograph
        0x22674d => [0x7999, false], # East Asian ideograph
        0x22674e => [0x7995, false], # East Asian ideograph
        0x6f5d5d => [0xd711, false], # Korean hangul
        0x226750 => [0x7996, false], # East Asian ideograph
        0x333051 => [0x8cb3, false], # East Asian ideograph
        0x2d6229 => [0x9c53, false], # East Asian ideograph
        0x6f4c51 => [0xb260, false], # Korean hangul
        0x275b3c => [0x8f76, false], # East Asian ideograph
        0x294564 => [0x9536, false], # East Asian ideograph
        0x292752 => [0x830f, false], # East Asian ideograph
        0x233a34 => [0x8e5c, false], # East Asian ideograph
        0x6f5a29 => [0xcef5, false], # Korean hangul
        0x6f5d5e => [0xd718, false], # Korean hangul
        0x274a30 => [0x70db, false], # East Asian ideograph
        0x292577 => [0x8297, false], # East Asian ideograph
        0x4b4a62 => [0x72a0, false], # East Asian ideograph
        0x273d67 => [0x5f84, false], # East Asian ideograph
        0x6f4c52 => [0xb268, false], # Korean hangul
        0x275b3d => [0x8f83, false], # East Asian ideograph
        0x217669 => [0x5819, false], # East Asian ideograph
        0x223636 => [0x6549, false], # East Asian ideograph
        0x2d5561 => [0x76d6, false], # East Asian ideograph
        0x6f5d5f => [0xd719, false], # Korean hangul
        0x274a31 => [0x707f, false], # East Asian ideograph
        0x293459 => [0x8c2f, false], # East Asian ideograph
        0x284e30 => [0x6e11, false], # East Asian ideograph
        0x29584b => [0x9cbd, false], # East Asian ideograph
        0x6f584a => [0xca0b, false], # Korean hangul
        0x34715a => [0x7e1a, false], # East Asian ideograph
        0x216c21 => [0x5293, false], # East Asian ideograph
        0x6f4c53 => [0xb269, false], # Korean hangul
        0x275b3e => [0x8f7c, false], # East Asian ideograph
        0x226c26 => [0x7b71, false], # East Asian ideograph
        0x226c27 => [0x7b70, false], # East Asian ideograph
        0x216c29 => [0x5298, false], # East Asian ideograph
        0x235c4f => [0x9de9, false], # East Asian ideograph
        0x216c2b => [0x529a, false], # East Asian ideograph
        0x216c2c => [0x5299, false], # East Asian ideograph
        0x226c2d => [0x7b9c, false], # East Asian ideograph
        0x216c2e => [0x52a6, false], # East Asian ideograph
        0x22275d => [0x5e68, false], # East Asian ideograph
        0x212a2b => [0xe8d9, false], # EACC component character
        0x216c31 => [0x52ad, false], # East Asian ideograph
        0x226c33 => [0x7b92, false], # East Asian ideograph
        0x226c34 => [0x7b91, false], # East Asian ideograph
        0x226c35 => [0x7b90, false], # East Asian ideograph
        0x216c37 => [0x52bb, false], # East Asian ideograph
        0x226c38 => [0x7ba3, false], # East Asian ideograph
        0x226c3a => [0x7b8d, false], # East Asian ideograph
        0x28275f => [0x5e31, false], # East Asian ideograph
        0x216c3c => [0x52ca, false], # East Asian ideograph
        0x216c3d => [0x52cd, false], # East Asian ideograph
        0x2e6c3e => [0x7b59, false], # East Asian ideograph
        0x2d622c => [0x9f08, false], # East Asian ideograph
        0x216c40 => [0x52d0, false], # East Asian ideograph
        0x226c41 => [0x7b85, false], # East Asian ideograph
        0x706c42 => [0x70bb, false], # East Asian ideograph
        0x226c43 => [0x7b8e, false], # East Asian ideograph
        0x226c44 => [0x7b98, false], # East Asian ideograph
        0x213239 => [0x504c, false], # East Asian ideograph
        0x226c46 => [0x7b86, false], # East Asian ideograph
        0x226c48 => [0x7b99, false], # East Asian ideograph
        0x6f4f3a => [0xb860, false], # Korean hangul
        0x216c4c => [0x52e3, false], # East Asian ideograph
        0x216c4e => [0x52e1, false], # East Asian ideograph
        0x6f5d61 => [0xd720, false], # Korean hangul
        0x216c50 => [0x55e7, false], # East Asian ideograph
        0x226c52 => [0x7bb2, false], # East Asian ideograph
        0x216c53 => [0x52e9, false], # East Asian ideograph
        0x6f5623 => [0xc648, false], # Korean hangul
        0x226c58 => [0x7bcb, false], # East Asian ideograph
        0x226c59 => [0x7bb8, false], # East Asian ideograph
        0x226c5a => [0x7bcf, false], # East Asian ideograph
        0x226c5c => [0x7bd0, false], # East Asian ideograph
        0x216c5e => [0x52f7, false], # East Asian ideograph
        0x292765 => [0x82c8, false], # East Asian ideograph
        0x226c60 => [0x7bbe, false], # East Asian ideograph
        0x216c61 => [0x52f9, false], # East Asian ideograph
        0x216c62 => [0x52fa, false], # East Asian ideograph
        0x216c64 => [0x52fc, false], # East Asian ideograph
        0x216c69 => [0x5307, false], # East Asian ideograph
        0x216c6a => [0x5303, false], # East Asian ideograph
        0x216c6b => [0x5306, false], # East Asian ideograph (not in Unicode)
        0x6f5d62 => [0xd728, false], # Korean hangul
        0x216c6e => [0x530a, false], # East Asian ideograph
        0x226c6f => [0x7bcc, false], # East Asian ideograph
        0x216560 => [0x4f80, false], # East Asian ideograph
        0x216c77 => [0x5311, false], # East Asian ideograph
        0x213f6a => [0x6221, false], # East Asian ideograph
        0x6f5975 => [0xce87, false], # Korean hangul
        0x216c7b => [0x6706, false], # East Asian ideograph
        0x234767 => [0x93f5, false], # East Asian ideograph
        0x21323b => [0x500f, false], # East Asian ideograph
        0x343e38 => [0x7bda, false], # East Asian ideograph
        0x4b6167 => [0x95d8, false], # East Asian ideograph
        0x6f5d63 => [0xd729, false], # Korean hangul
        0x6f5532 => [0xc571, false], # Korean hangul
        0x216561 => [0x4f74, false], # East Asian ideograph
        0x4b5a31 => [0x8cce, false], # East Asian ideograph
        0x6f4c57 => [0xb290, false], # Korean hangul
        0x275b42 => [0x8f84, false], # East Asian ideograph
        0x333e7d => [0x7652, false], # East Asian ideograph
        0x4b4925 => [0x6fb3, false], # East Asian ideograph (variant of 214925 which maps to 6FB3)
        0x226771 => [0x79a8, false], # East Asian ideograph
        0x225a7e => [0x7488, false], # East Asian ideograph
        0x6f5921 => [0xcc29, false], # Korean hangul
        0x692577 => [0x309b, false], # Katakana-hiragana voiced sound mark
        0x224b26 => [0x6e31, false], # East Asian ideograph
        0x6f5a3e => [0xcf55, false], # Korean hangul
        0x6f4c58 => [0xb291, false], # Korean hangul
        0x275b43 => [0x8f7b, false], # East Asian ideograph
        0x226775 => [0x79b0, false], # East Asian ideograph
        0x233a3b => [0x8e67, false], # East Asian ideograph
        0x275221 => [0x7eed, false], # East Asian ideograph
        0x6f5922 => [0xcc2c, false], # Korean hangul
        0x215222 => [0x7e93, false], # East Asian ideograph
        0x234b48 => [0x96b9, false], # East Asian ideograph
        0x4d445b => [0x9306, false], # East Asian ideograph (variant of 23445B which maps to 9306)
        0x275223 => [0x7ea4, false], # East Asian ideograph
        0x275224 => [0x7f06, false], # East Asian ideograph
        0x21323e => [0x50a2, false], # East Asian ideograph
        0x6f4f3b => [0xb864, false], # Korean hangul
        0x2d334f => [0x5202, false], # East Asian ideograph
        0x21677b => [0x50ca, false], # East Asian ideograph
        0x4b4b2c => [0x731f, false], # East Asian ideograph
        0x213933 => [0x5944, false], # East Asian ideograph
        0x27677c => [0x4f1b, false], # East Asian ideograph
        0x4c6022 => [0x7596, false], # East Asian ideograph
        0x225227 => [0x7139, false], # East Asian ideograph
        0x4b3c5e => [0x5e64, false], # East Asian ideograph
        0x234b49 => [0x96bc, false], # East Asian ideograph
        0x215228 => [0x7f3d, false], # East Asian ideograph
        0x6f4c5a => [0xb298, false], # Korean hangul
        0x275b45 => [0x8f87, false], # East Asian ideograph
        0x215229 => [0x7f44, false], # East Asian ideograph
        0x22363e => [0x6554, false], # East Asian ideograph
        0x23522b => [0x995f, false], # East Asian ideograph
        0x6f5924 => [0xcc2f, false], # Korean hangul
        0x22522c => [0x713b, false], # East Asian ideograph
        0x516122 => [0x9988, false], # East Asian ideograph
        0x6f522d => [0xbe90, false], # Korean hangul
        0x6f4c5b => [0xb299, false], # Korean hangul
        0x22522e => [0x711c, false], # East Asian ideograph
        0x213240 => [0x5099, false], # East Asian ideograph
        0x23522f => [0x9997, false], # East Asian ideograph
        0x225b59 => [0x74a0, false], # East Asian ideograph
        0x4b6168 => [0x9599, false], # East Asian ideograph
        0x235230 => [0x9998, false], # East Asian ideograph
        0x226d22 => [0x7bdd, false], # East Asian ideograph
        0x216d23 => [0x531a, false], # East Asian ideograph
        0x226d24 => [0x7be5, false], # East Asian ideograph
        0x216d25 => [0x531f, false], # East Asian ideograph
        0x215231 => [0x7f69, false], # East Asian ideograph
        0x226d29 => [0x7be8, false], # East Asian ideograph
        0x277267 => [0x5452, false], # East Asian ideograph
        0x225232 => [0x713d, false], # East Asian ideograph
        0x226d2e => [0x7bf9, false], # East Asian ideograph
        0x226d2f => [0x7bd4, false], # East Asian ideograph
        0x6f4c5c => [0xb2a0, false], # Korean hangul
        0x226d32 => [0x7bdf, false], # East Asian ideograph
        0x275233 => [0x7f5a, false], # East Asian ideograph
        0x226d35 => [0x7bd8, false], # East Asian ideograph
        0x216d36 => [0x5335, false], # East Asian ideograph
        0x226d37 => [0x7bea, false], # Unrelated variant of EACC 3A6A7C which maps to 7BEA
        0x213c2d => [0x5dbc, false], # East Asian ideograph
        0x275234 => [0x9a82, false], # East Asian ideograph
        0x216d3a => [0x5338, false], # East Asian ideograph
        0x226d3b => [0x7c06, false], # East Asian ideograph
        0x226d3e => [0x7bf0, false], # East Asian ideograph
        0x275d6b => [0x952d, false], # East Asian ideograph
        0x696d40 => [0x8ec5, false], # East Asian ideograph
        0x226d41 => [0x7c0f, false], # East Asian ideograph
        0x216d42 => [0x534d, false], # East Asian ideograph
        0x6f5926 => [0xcc38, false], # Korean hangul
        0x706d45 => [0x783c, false], # East Asian ideograph
        0x226d46 => [0x7c0b, false], # East Asian ideograph
        0x222534 => [0x5d74, false], # East Asian ideograph
        0x275237 => [0x7f57, false], # East Asian ideograph
        0x216d4c => [0x5363, false], # East Asian ideograph
        0x2d6235 => [0x9d76, false], # East Asian ideograph
        0x216d4e => [0x5365, false], # East Asian ideograph (not in Unicode)
        0x226d4f => [0x7bf4, false], # East Asian ideograph
        0x215238 => [0x7f88, false], # East Asian ideograph
        0x216d53 => [0x536c, false], # East Asian ideograph
        0x226d54 => [0x7bf3, false], # East Asian ideograph
        0x216d57 => [0x5372, false], # East Asian ideograph
        0x216d58 => [0x537a, false], # East Asian ideograph
        0x4b492b => [0x6feb, false], # East Asian ideograph
        0x226d5a => [0x7c09, false], # East Asian ideograph
        0x226d5b => [0x7c03, false], # East Asian ideograph
        0x226d5c => [0x7bfc, false], # East Asian ideograph
        0x216d5d => [0x5380, false], # East Asian ideograph
        0x226d5f => [0x7c1c, false], # East Asian ideograph
        0x226d61 => [0x7c26, false], # East Asian ideograph
        0x226d62 => [0x7c28, false], # East Asian ideograph
        0x22523b => [0x7129, false], # East Asian ideograph
        0x216d64 => [0x538e, false], # East Asian ideograph
        0x233d3f => [0x9004, false], # East Asian ideograph
        0x226d66 => [0x7c1f, false], # East Asian ideograph
        0x216d67 => [0x5394, false], # East Asian ideograph
        0x226d68 => [0x7c2f, false], # East Asian ideograph
        0x23523c => [0x99a1, false], # East Asian ideograph
        0x6f4c5e => [0xb2a5, false], # Korean hangul
        0x216d6d => [0x5399, false], # East Asian ideograph
        0x6f523d => [0xbf18, false], # Korean hangul
        0x285f48 => [0x7617, false], # East Asian ideograph
        0x213243 => [0x5096, false], # East Asian ideograph
        0x216d74 => [0x8652, false], # East Asian ideograph
        0x226d75 => [0x7c30, false], # East Asian ideograph
        0x6f4f3c => [0xb86c, false], # Korean hangul
        0x216d7a => [0x53a4, false], # East Asian ideograph
        0x216d7b => [0x53ab, false], # East Asian ideograph
        0x2d5941 => [0x5629, false], # East Asian ideograph
        0x6f5928 => [0xcc3b, false], # Korean hangul
        0x2d5240 => [0x7fa1, false], # East Asian ideograph
        0x235241 => [0x99a9, false], # East Asian ideograph
        0x6f4c5f => [0xb2a6, false], # Korean hangul
        0x215242 => [0x7fa9, false], # East Asian ideograph
        0x283d30 => [0x67a7, false], # East Asian ideograph
        0x235c5b => [0x9df8, false], # East Asian ideograph
        0x225243 => [0x712e, false], # East Asian ideograph
        0x6f5244 => [0xbf51, false], # Korean hangul
        0x6f5929 => [0xcc3c, false], # Korean hangul
        0x226969 => [0x7a78, false], # East Asian ideograph
        0x6f523b => [0xbf08, false], # Korean hangul
        0x28337b => [0x62a0, false], # East Asian ideograph
        0x6f4c60 => [0xb2aa, false], # Korean hangul
        0x4b5247 => [0x7fae, false], # East Asian ideograph
        0x334256 => [0x6b5b, false], # East Asian ideograph
        0x22585d => [0x73a5, false], # East Asian ideograph
        0x235c5c => [0x9dfc, false], # East Asian ideograph
        0x225248 => [0x7177, false], # East Asian ideograph
        0x233a43 => [0x8e5d, false], # East Asian ideograph
        0x4b492e => [0x6e0b, false], # East Asian ideograph
        0x234e4c => [0x97cd, false], # East Asian ideograph
        0x2d7345 => [0x56d3, false], # East Asian ideograph
        0x215249 => [0x7fbf, false], # East Asian ideograph
        0x284e3e => [0x6cf6, false], # East Asian ideograph
        0x212a3a => [0xe8e7, false], # EACC component character
        0x2d524a => [0x7fc4, false], # East Asian ideograph
        0x39483b => [0x9061, false], # East Asian ideograph
        0x276029 => [0x9791, false], # East Asian ideograph
        0x6f524b => [0xbfd0, false], # Korean hangul
        0x2e337b => [0x630e, false], # East Asian ideograph
        0x6f4c61 => [0xb2ac, false], # Korean hangul
        0x6f524c => [0xbfd4, false], # Korean hangul
        0x27524d => [0x4e60, false], # East Asian ideograph
        0x233a44 => [0x8e75, false], # East Asian ideograph
        0x23524e => [0x99bc, false], # East Asian ideograph
        0x293468 => [0x8c35, false], # East Asian ideograph
        0x6f545a => [0xc410, false], # Korean hangul
        0x23524f => [0x99c3, false], # East Asian ideograph
        0x6f5250 => [0xc058, false], # Korean hangul
        0x6f4c62 => [0xb2c8, false], # Korean hangul
        0x275b4d => [0x8f91, false], # East Asian ideograph
        0x275251 => [0x7fc6, false], # East Asian ideograph
        0x213247 => [0x50b5, false], # East Asian ideograph
        0x4b4d73 => [0x66b8, false], # East Asian ideograph
        0x225252 => [0x7152, false], # East Asian ideograph
        0x235253 => [0x99b9, false], # East Asian ideograph
        0x6f592c => [0xcc3f, false], # Korean hangul
        0x215254 => [0x7fe9, false], # East Asian ideograph
        0x274d3a => [0x76cf, false], # East Asian ideograph
        0x225255 => [0x715d, false], # East Asian ideograph
        0x6f4c63 => [0xb2c9, false], # Korean hangul
        0x225256 => [0x7141, false], # East Asian ideograph
        0x227636 => [0x800f, false], # East Asian ideograph
        0x4b4931 => [0x6e16, false], # East Asian ideograph
        0x4d3359 => [0x56af, false], # East Asian ideograph
        0x275258 => [0x7fd8, false], # East Asian ideograph
        0x21656e => [0x4f94, false], # East Asian ideograph
        0x284e41 => [0x6f4b, false], # East Asian ideograph
        0x225259 => [0x7175, false], # East Asian ideograph
        0x692546 => [0x30c6, false], # Katakana letter TE
        0x22525a => [0x7173, false], # East Asian ideograph
        0x6f4c64 => [0xb2cc, false], # Korean hangul
        0x275b4f => [0x8f96, false], # East Asian ideograph
        0x33525b => [0x71ff, false], # East Asian ideograph
        0x226e27 => [0x7c35, false], # East Asian ideograph
        0x23347b => [0x8b7e, false], # East Asian ideograph
        0x21525c => [0x8001, false], # East Asian ideograph
        0x226e2a => [0x7c40, false], # East Asian ideograph
        0x2d5573 => [0x83d4, false], # East Asian ideograph
        0x216e2c => [0x53b5, false], # East Asian ideograph
        0x216e2e => [0x53b9, false], # East Asian ideograph
        0x22525d => [0x715a, false], # East Asian ideograph
        0x226e30 => [0x7c39, false], # East Asian ideograph
        0x6f592e => [0xcc45, false], # Korean hangul
        0x226e34 => [0x7c3b, false], # East Asian ideograph
        0x226e35 => [0x7c34, false], # East Asian ideograph
        0x6f5426 => [0xc2e3, false], # Korean hangul
        0x226e3b => [0x7c42, false], # East Asian ideograph
        0x216e3e => [0x53d0, false], # East Asian ideograph
        0x70727d => [0x87a8, false], # East Asian ideograph
        0x275b50 => [0x8f97, false], # East Asian ideograph
        0x225260 => [0x714b, false], # East Asian ideograph
        0x4c6e42 => [0x7c31, false], # East Asian ideograph
        0x21324a => [0x50be, false], # East Asian ideograph
        0x225862 => [0x73a2, false], # East Asian ideograph
        0x226e46 => [0x7c4e, false], # East Asian ideograph
        0x235261 => [0x99d3, false], # East Asian ideograph
        0x216e48 => [0x53da, false], # East Asian ideograph
        0x4d5574 => [0x9b2e, false], # East Asian ideograph
        0x275e47 => [0x94a5, false], # East Asian ideograph
        0x225262 => [0x7147, false], # East Asian ideograph
        0x6f592f => [0xcc48, false], # Korean hangul
        0x235263 => [0x99d4, false], # East Asian ideograph
        0x226e54 => [0x7c5d, false], # East Asian ideograph
        0x226e56 => [0x7c5c, false], # East Asian ideograph
        0x226e57 => [0x7c5a, false], # East Asian ideograph
        0x226e58 => [0x7c5b, false], # East Asian ideograph
        0x226e59 => [0x7c59, false], # East Asian ideograph
        0x226e5b => [0x7c5e, false], # East Asian ideograph
        0x226e5c => [0x7c67, false], # East Asian ideograph
        0x6f4c66 => [0xb2d8, false], # Korean hangul
        0x226e5e => [0x7c63, false], # East Asian ideograph
        0x235265 => [0x99c9, false], # East Asian ideograph
        0x226e61 => [0x7c68, false], # East Asian ideograph
        0x226e62 => [0x7c65, false], # East Asian ideograph
        0x2d3132 => [0x4ecf, false], # East Asian ideograph
        0x225266 => [0x7171, false], # East Asian ideograph
        0x216e68 => [0x5406, false], # East Asian ideograph
        0x286e69 => [0x7c16, false], # East Asian ideograph
        0x225267 => [0x715f, false], # East Asian ideograph
        0x216e6c => [0x544c, false], # East Asian ideograph
        0x216e6d => [0x5445, false], # East Asian ideograph
        0x226e6f => [0x7c6f, false], # East Asian ideograph
        0x216e70 => [0x5432, false], # East Asian ideograph
        0x226970 => [0x7a85, false], # East Asian ideograph
        0x226e75 => [0x7c75, false], # East Asian ideograph
        0x216e76 => [0x5421, false], # East Asian ideograph
        0x215269 => [0x8033, false], # East Asian ideograph
        0x216e78 => [0x5430, false], # East Asian ideograph
        0x226e79 => [0x7c7e, false], # East Asian ideograph
        0x226e7a => [0x7c78, false], # East Asian ideograph
        0x6f4c67 => [0xb2d9, false], # Korean hangul
        0x275b52 => [0x6bc2, false], # East Asian ideograph
        0x226e7d => [0x7c7d, false], # East Asian ideograph
        0x27517e => [0x7f20, false], # East Asian ideograph
        0x692560 => [0x30e0, false], # Katakana letter MU
        0x215022 => [0x7b4f, false], # East Asian ideograph
        0x6f5323 => [0xc11e, false], # Korean hangul
        0x225421 => [0x71dd, false], # East Asian ideograph
        0x6f486c => [0xac16, false], # Korean hangul
        0x2d526c => [0x8ead, false], # East Asian ideograph
        0x274a46 => [0x5899, false], # East Asian ideograph
        0x6f5931 => [0xcc54, false], # Korean hangul
        0x225f5f => [0x7630, false], # East Asian ideograph
        0x6f5b2d => [0xd145, false], # Korean hangul
        0x22253f => [0x5d75, false], # East Asian ideograph
        0x234b57 => [0x96d2, false], # East Asian ideograph
        0x69654f => [0x7e05, false], # East Asian ideograph
        0x4b526e => [0x8046, false], # East Asian ideograph (variant of 21526E which maps to 8046)
        0x6f4c68 => [0xb2db, false], # Korean hangul
        0x27526f => [0x5723, false], # East Asian ideograph
        0x22763b => [0x801f, false], # East Asian ideograph
        0x225422 => [0x71c0, false], # East Asian ideograph
        0x335821 => [0x97c8, false], # East Asian ideograph
        0x275271 => [0x95fb, false], # East Asian ideograph
        0x6f5932 => [0xcc55, false], # Korean hangul
        0x6f5272 => [0xc0d0, false], # Korean hangul
        0x2d446b => [0x6936, false], # East Asian ideograph
        0x2d6241 => [0x9d5e, false], # East Asian ideograph
        0x21346a => [0x536f, false], # East Asian ideograph
        0x275b54 => [0x8f99, false], # East Asian ideograph
        0x235274 => [0x99ec, false], # East Asian ideograph
        0x2e625f => [0x77c1, false], # East Asian ideograph
        0x275275 => [0x8038, false], # East Asian ideograph
        0x4b4937 => [0x56a0, false], # East Asian ideograph
        0x225276 => [0x7172, false], # East Asian ideograph
        0x223d21 => [0x6872, false], # East Asian ideograph
        0x6f5933 => [0xcc58, false], # Korean hangul
        0x29486f => [0x9569, false], # East Asian ideograph
        0x275277 => [0x8054, false], # East Asian ideograph
        0x223d22 => [0x689c, false], # East Asian ideograph
        0x275278 => [0x804c, false], # East Asian ideograph
        0x6f5979 => [0xce94, false], # Korean hangul
        0x2f3c2d => [0x8f3c, false], # East Asian ideograph
        0x6f4c6a => [0xb2e2, false], # Korean hangul
        0x275b55 => [0x8f6c, false], # East Asian ideograph
        0x215279 => [0x8076, false], # East Asian ideograph
        0x2e7d24 => [0x83f0, false], # East Asian ideograph
        0x225867 => [0x73b6, false], # East Asian ideograph
        0x235d66 => [0x9e9e, false], # East Asian ideograph
        0x21527a => [0x807e, false], # East Asian ideograph
        0x213d25 => [0x5ed3, false], # East Asian ideograph
        0x275e48 => [0x92ae, false], # East Asian ideograph
        0x235823 => [0x9bd5, false], # East Asian ideograph
        0x21527b => [0x807d, false], # East Asian ideograph
        0x217d26 => [0x5aff, false], # East Asian ideograph
        0x287061 => [0x7ec0, false], # East Asian ideograph
        0x23527c => [0x99ea, false], # East Asian ideograph
        0x213d27 => [0x5ee2, false], # East Asian ideograph
        0x284668 => [0x6c29, false], # East Asian ideograph
        0x6f527d => [0xc0f7, false], # Korean hangul
        0x333d28 => [0x53a8, false], # East Asian ideograph
        0x275b56 => [0x8f9a, false], # East Asian ideograph
        0x6f527e => [0xc0f9, false], # Korean hangul
        0x2d3d29 => [0x53ae, false], # East Asian ideograph
        0x33417e => [0x629e, false], # East Asian ideograph
        0x213d2a => [0x5ee3, false], # East Asian ideograph (variant of 4B3D2A which maps to 5EE3)
        0x287279 => [0x7f25, false], # East Asian ideograph
        0x294568 => [0x9518, false], # East Asian ideograph
        0x213d2b => [0x5edf, false], # East Asian ideograph
        0x287062 => [0x7ec1, false], # East Asian ideograph
        0x6f545c => [0xc430, false], # Korean hangul
        0x226975 => [0x7a86, false], # East Asian ideograph
        0x22475c => [0x6ca0, false], # East Asian ideograph
        0x216133 => [0x99d0, false], # East Asian ideograph
        0x226532 => [0x78b6, false], # East Asian ideograph
        0x213d2d => [0x9f90, false], # East Asian ideograph
        0x275b57 => [0x8f7f, false], # East Asian ideograph
        0x223d2e => [0x68a9, false], # East Asian ideograph
        0x213d2f => [0x5ef3, false], # East Asian ideograph
        0x212a30 => [0xe8de, false], # EACC component character
        0x6f5d79 => [0xd789, false], # Korean hangul
        0x226f21 => [0x7c81, false], # East Asian ideograph
        0x216577 => [0x4f90, false], # East Asian ideograph
        0x216f24 => [0x542a, false], # East Asian ideograph
        0x33314c => [0x5fa0, false], # East Asian ideograph
        0x216f26 => [0x5422, false], # East Asian ideograph
        0x274d3c => [0x5c3d, false], # East Asian ideograph
        0x226f28 => [0x7c8e, false], # East Asian ideograph
        0x226f29 => [0x7c91, false], # East Asian ideograph
        0x226f2a => [0x7c83, false], # East Asian ideograph
        0x226f2c => [0x7c8d, false], # East Asian ideograph
        0x213d32 => [0x5ef6, false], # East Asian ideograph
        0x216f2e => [0x545f, false], # East Asian ideograph
        0x216f2f => [0x549c, false], # East Asian ideograph
        0x27393f => [0x5941, false], # East Asian ideograph
        0x223d33 => [0x68a0, false], # East Asian ideograph
        0x213252 => [0x50d6, false], # East Asian ideograph
        0x216f35 => [0x5488, false], # East Asian ideograph
        0x216f37 => [0x547f, false], # East Asian ideograph
        0x216f39 => [0x5482, false], # East Asian ideograph
        0x226f3a => [0x7c99, false], # East Asian ideograph
        0x226f3b => [0x7c98, false], # East Asian ideograph
        0x6f5d7a => [0xd78c, false], # Korean hangul
        0x226f3e => [0x7c9c, false], # East Asian ideograph
        0x226f40 => [0x7c95, false], # East Asian ideograph
        0x6f5937 => [0xcc70, false], # Korean hangul
        0x226f42 => [0x7ca7, false], # East Asian ideograph
        0x226f43 => [0x7ca2, false], # East Asian ideograph
        0x226f45 => [0x7c9e, false], # East Asian ideograph
        0x226f46 => [0x7ca9, false], # East Asian ideograph
        0x6f5939 => [0xcc98, false], # Korean hangul
        0x226f48 => [0x7ca8, false], # East Asian ideograph
        0x226f49 => [0x7ca1, false], # East Asian ideograph
        0x226f4a => [0x7cac, false], # East Asian ideograph
        0x216f4b => [0x5474, false], # East Asian ideograph
        0x226f4c => [0x7ca6, false], # East Asian ideograph
        0x6f4c6e => [0xb2e5, false], # Korean hangul
        0x216f52 => [0x5466, false], # East Asian ideograph
        0x216f53 => [0x5464, false], # East Asian ideograph
        0x226f54 => [0x7cb2, false], # East Asian ideograph
        0x216f55 => [0x54a4, false], # East Asian ideograph
        0x213d39 => [0x5f0f, false], # East Asian ideograph
        0x226f58 => [0x7cbb, false], # East Asian ideograph
        0x226f59 => [0x7cbf, false], # East Asian ideograph
        0x216f5a => [0x54ad, false], # East Asian ideograph
        0x216f5b => [0x54ba, false], # East Asian ideograph
        0x216f5c => [0x54cf, false], # East Asian ideograph
        0x696f5d => [0x9596, false], # East Asian ideograph
        0x226f5e => [0x7cba, false], # East Asian ideograph
        0x226f5f => [0x7cbc, false], # East Asian ideograph
        0x216f60 => [0x54a5, false], # East Asian ideograph
        0x216f63 => [0x54a7, false], # East Asian ideograph
        0x226f64 => [0x7cc2, false], # East Asian ideograph
        0x216f66 => [0x54a2, false], # East Asian ideograph
        0x216f67 => [0x5472, false], # East Asian ideograph
        0x216f68 => [0x5470, false], # East Asian ideograph
        0x216f69 => [0x54bc, false], # East Asian ideograph
        0x216f6a => [0x54b7, false], # East Asian ideograph
        0x216f6b => [0x54de, false], # East Asian ideograph
        0x216f6c => [0x54d6, false], # East Asian ideograph
        0x226f6d => [0x7ccc, false], # East Asian ideograph
        0x226f6f => [0x7cc9, false], # East Asian ideograph
        0x226f71 => [0x7cd2, false], # East Asian ideograph
        0x6f4a22 => [0xadc0, false], # Korean hangul
        0x29442d => [0x94a1, false], # East Asian ideograph
        0x216f74 => [0x54c6, false], # East Asian ideograph
        0x225429 => [0x71cb, false], # East Asian ideograph
        0x226f77 => [0x7ce1, false], # East Asian ideograph
        0x6f5d7c => [0xd798, false], # Korean hangul
        0x33467a => [0x6ca1, false], # East Asian ideograph
        0x223d3f => [0x6877, false], # East Asian ideograph
        0x216f7c => [0x54e2, false], # East Asian ideograph
        0x216f7d => [0x5507, false], # East Asian ideograph
        0x233d40 => [0x9008, false], # East Asian ideograph
        0x233d59 => [0x9036, false], # East Asian ideograph
        0x277d74 => [0x5a08, false], # East Asian ideograph
        0x333d42 => [0x7d43, false], # East Asian ideograph
        0x213a63 => [0x5b7f, false], # East Asian ideograph
        0x213d43 => [0x5f27, false], # East Asian ideograph
        0x2d3366 => [0x5234, false], # East Asian ideograph
        0x6f5d7d => [0xd799, false], # Korean hangul
        0x223d44 => [0x688e, false], # East Asian ideograph
        0x6f593a => [0xcc99, false], # Korean hangul
        0x233d45 => [0x900b, false], # East Asian ideograph
        0x277030 => [0x5457, false], # East Asian ideograph
        0x697023 => [0x9666, false], # East Asian ideograph
        0x6f4a64 => [0xaedc, false], # Korean hangul
        0x213d47 => [0x5f35, false], # East Asian ideograph
        0x235c6d => [0x9dee, false], # East Asian ideograph
        0x233d48 => [0x900c, false], # East Asian ideograph
        0x6f5d7e => [0xd79b, false], # Korean hangul
        0x213d49 => [0x5f3c, false], # East Asian ideograph
        0x6f593b => [0xcc9c, false], # Korean hangul
        0x394944 => [0x6b12, false], # East Asian ideograph
        0x6f5b2f => [0xd14d, false], # Korean hangul
        0x224762 => [0x6ceb, false], # East Asian ideograph
        0x4b3b52 => [0x8132, false], # East Asian ideograph
        0x2d4474 => [0x690d, false], # East Asian ideograph (variant of 214474 which maps to 690D)
        0x273d4b => [0x5f39, false], # East Asian ideograph
        0x2d4031 => [0x64a6, false], # East Asian ideograph
        0x6f5a32 => [0xcf10, false], # Korean hangul
        0x213d4c => [0x5f4c, false], # East Asian ideograph
        0x213d4d => [0x5f4e, false], # East Asian ideograph
        0x4b4940 => [0x6f45, false], # East Asian ideograph
        0x23582b => [0x9bf1, false], # East Asian ideograph
        0x6f5d25 => [0xd5e5, false], # Korean hangul
        0x213d4e => [0x5f57, false], # East Asian ideograph
        0x6f5030 => [0xba65, false], # Korean hangul
        0x224763 => [0x6cee, false], # East Asian ideograph
        0x226539 => [0x78b7, false], # East Asian ideograph
        0x213d50 => [0x5f5d, false], # East Asian ideograph
        0x6f4c73 => [0xb2ed, false], # Korean hangul
        0x223d51 => [0x6917, false], # East Asian ideograph
        0x225870 => [0x73c8, false], # East Asian ideograph
        0x217247 => [0x5642, false], # East Asian ideograph
        0x4b6247 => [0x9d2c, false], # East Asian ideograph
        0x217d52 => [0x5b2c, false], # East Asian ideograph
        0x23582c => [0x9be1, false], # East Asian ideograph
        0x213d53 => [0x5f65, false], # East Asian ideograph
        0x28706a => [0x7ed0, false], # East Asian ideograph
        0x294871 => [0x954a, false], # East Asian ideograph
        0x6f5d78 => [0xd788, false], # Korean hangul
        0x224764 => [0x6cc0, false], # East Asian ideograph
        0x6f597b => [0xcea0, false], # Korean hangul
        0x275b5f => [0x529e, false], # East Asian ideograph
        0x285f5e => [0x7618, false], # East Asian ideograph
        0x223d56 => [0x690b, false], # East Asian ideograph
        0x213259 => [0x5100, false], # East Asian ideograph
        0x233d57 => [0x9034, false], # East Asian ideograph
        0x455122 => [0x7d0d, false], # East Asian ideograph
        0x23582d => [0x9bdb, false], # East Asian ideograph
        0x233d58 => [0x902f, false], # East Asian ideograph
        0x6f593e => [0xcca9, false], # Korean hangul
        0x223d59 => [0x6904, false], # East Asian ideograph
        0x6f5b45 => [0xd230, false], # Korean hangul
        0x234b64 => [0x96df, false], # East Asian ideograph
        0x6f4c75 => [0xb2f3, false], # Korean hangul
        0x275b60 => [0x8f9e, false], # East Asian ideograph
        0x227022 => [0x7cdd, false], # East Asian ideograph
        0x217023 => [0x5517, false], # East Asian ideograph
        0x217024 => [0x54fd, false], # East Asian ideograph
        0x217025 => [0x54e7, false], # East Asian ideograph
        0x217027 => [0x54f3, false], # East Asian ideograph
        0x227028 => [0x7ced, false], # East Asian ideograph
        0x213d5c => [0x5f80, false], # East Asian ideograph
        0x21702a => [0x54e4, false], # East Asian ideograph
        0x21702b => [0x550a, false], # East Asian ideograph
        0x21702d => [0x54ff, false], # East Asian ideograph
        0x21702e => [0x5518, false], # East Asian ideograph
        0x223d5d => [0x6929, false], # East Asian ideograph
        0x227030 => [0x7cf2, false], # East Asian ideograph
        0x6f593f => [0xccab, false], # Korean hangul
        0x217032 => [0x54ef, false], # East Asian ideograph
        0x217034 => [0x5508, false], # East Asian ideograph
        0x227035 => [0x7cf4, false], # East Asian ideograph
        0x217038 => [0x54f6, false], # East Asian ideograph
        0x227039 => [0x7cf6, false], # East Asian ideograph
        0x6f4c76 => [0xb2f4, false], # Korean hangul
        0x21703e => [0x550e, false], # East Asian ideograph
        0x275b61 => [0x8fa9, false], # East Asian ideograph
        0x4b5c50 => [0x9045, false], # East Asian ideograph
        0x692563 => [0x30e3, false], # Katakana letter small YA
        0x21325b => [0x50fb, false], # East Asian ideograph
        0x217044 => [0x5523, false], # East Asian ideograph
        0x227045 => [0x7d08, false], # East Asian ideograph
        0x217046 => [0x550f, false], # East Asian ideograph
        0x217047 => [0x5511, false], # East Asian ideograph
        0x23582f => [0x9be2, false], # East Asian ideograph
        0x22704a => [0x7d13, false], # East Asian ideograph
        0x21704b => [0x5575, false], # East Asian ideograph
        0x21704d => [0x5573, false], # East Asian ideograph
        0x21704e => [0x554c, false], # East Asian ideograph
        0x21704f => [0x5576, false], # East Asian ideograph
        0x217050 => [0x554d, false], # East Asian ideograph
        0x217051 => [0x555a, false], # East Asian ideograph
        0x227052 => [0x7d1d, false], # East Asian ideograph
        0x217053 => [0x553c, false], # East Asian ideograph
        0x217055 => [0x5550, false], # East Asian ideograph
        0x217057 => [0x5539, false], # East Asian ideograph
        0x217058 => [0x5548, false], # East Asian ideograph
        0x217059 => [0x552d, false], # East Asian ideograph
        0x21705a => [0x5551, false], # East Asian ideograph
        0x6f4c77 => [0xb2f5, false], # Korean hangul
        0x21705d => [0x552a, false], # East Asian ideograph
        0x213d65 => [0x5f8c, false], # East Asian ideograph
        0x217060 => [0x5562, false], # East Asian ideograph
        0x217061 => [0x5536, false], # East Asian ideograph
        0x227062 => [0x7d32, false], # East Asian ideograph
        0x217064 => [0x5549, false], # East Asian ideograph
        0x227065 => [0x7d31, false], # East Asian ideograph
        0x335830 => [0x658d, false], # East Asian ideograph
        0x227068 => [0x7d45, false], # East Asian ideograph
        0x21706a => [0x5540, false], # East Asian ideograph
        0x21706b => [0x5535, false], # East Asian ideograph
        0x22706c => [0x7d29, false], # East Asian ideograph
        0x6f5941 => [0xccb4, false], # Korean hangul
        0x22706f => [0x7d41, false], # East Asian ideograph
        0x217070 => [0x5545, false], # East Asian ideograph
        0x227071 => [0x7d3e, false], # East Asian ideograph
        0x234b67 => [0x96dd, false], # East Asian ideograph
        0x213d69 => [0x5f98, false], # East Asian ideograph
        0x217079 => [0x553f, false], # East Asian ideograph
        0x22707a => [0x7d5c, false], # East Asian ideograph
        0x21707b => [0x5541, false], # East Asian ideograph
        0x22707c => [0x7d53, false], # East Asian ideograph
        0x21707d => [0x5565, false], # East Asian ideograph
        0x22707e => [0x7d5a, false], # East Asian ideograph
        0x275779 => [0x891b, false], # East Asian ideograph
        0x225432 => [0x71eb, false], # East Asian ideograph
        0x235831 => [0x9bf0, false], # East Asian ideograph
        0x213d6c => [0x5f9e, false], # East Asian ideograph
        0x6f5942 => [0xccb5, false], # Korean hangul
        0x213f27 => [0x614d, false], # East Asian ideograph
        0x213b3f => [0x5c08, false], # East Asian ideograph
        0x283d6e => [0x67a8, false], # East Asian ideograph
        0x2d6251 => [0x5869, false], # East Asian ideograph
        0x6f4c79 => [0xb2f9, false], # Korean hangul
        0x275b64 => [0x519c, false], # East Asian ideograph
        0x273d6f => [0x590d, false], # East Asian ideograph
        0x21325e => [0x5102, false], # East Asian ideograph
        0x6f4a24 => [0xadc8, false], # Korean hangul
        0x4b4947 => [0x7ac3, false], # East Asian ideograph
        0x6f5943 => [0xccb8, false], # Korean hangul
        0x213f28 => [0x614b, false], # East Asian ideograph
        0x213d73 => [0x5fae, false], # East Asian ideograph
        0x2d6252 => [0x78b1, false], # East Asian ideograph
        0x295a59 => [0x9e38, false], # East Asian ideograph
        0x277328 => [0x54dc, false], # East Asian ideograph
        0x213d74 => [0x5fb9, false], # East Asian ideograph
        0x21325f => [0x510d, false], # East Asian ideograph
        0x285b21 => [0x740f, false], # East Asian ideograph
        0x233a5d => [0x8e94, false], # East Asian ideograph
        0x213d75 => [0x5fb7, false], # East Asian ideograph
        0x275d71 => [0x94a2, false], # East Asian ideograph
        0x217d76 => [0x5b4b, false], # East Asian ideograph
        0x6f5023 => [0xba3c, false], # Korean hangul
        0x226822 => [0x79b8, false], # East Asian ideograph
        0x223d78 => [0x68fd, false], # East Asian ideograph
        0x226823 => [0x79ba, false], # East Asian ideograph
        0x213d79 => [0x5fc5, false], # East Asian ideograph
        0x4b7e6a => [0x5bc3, false], # East Asian ideograph (variant of 217E6A which maps to 5BC3)
        0x223f3e => [0x696f, false], # East Asian ideograph
        0x212a33 => [0xe8e0, false], # EACC component character
        0x223d7b => [0x68f3, false], # East Asian ideograph
        0x6f5945 => [0xccc7, false], # Korean hangul
        0x6f5b31 => [0xd154, false], # Korean hangul
        0x213d7c => [0x5fcc, false], # East Asian ideograph
        0x27493c => [0x6fd2, false], # East Asian ideograph
        0x213261 => [0x5109, false], # East Asian ideograph
        0x233a5f => [0x8e92, false], # East Asian ideograph
        0x29282a => [0x8539, false], # East Asian ideograph
        0x29494d => [0x9609, false], # East Asian ideograph
        0x6f5342 => [0xc190, false], # Korean hangul
        0x696d3f => [0x8ebe, false], # East Asian ideograph
        0x213f2b => [0x6134, false], # East Asian ideograph
        0x334729 => [0x6e2b, false], # East Asian ideograph
        0x6f4c7d => [0xb300, false], # Korean hangul
        0x2f2a64 => [0x87b5, false], # East Asian ideograph
        0x22682e => [0x79d5, false], # East Asian ideograph
        0x287431 => [0x575b, false], # East Asian ideograph (duplicate simplified)
        0x275e60 => [0x960e, false], # East Asian ideograph
        0x233a60 => [0x8e93, false], # East Asian ideograph
        0x22282f => [0x5ea4, false], # East Asian ideograph
        0x227122 => [0x7d70, false], # East Asian ideograph
        0x217123 => [0x5591, false], # East Asian ideograph
        0x697124 => [0x98aa, false], # East Asian ideograph
        0x217125 => [0x5577, false], # East Asian ideograph
        0x217126 => [0x55a8, false], # East Asian ideograph
        0x217127 => [0x55ad, false], # East Asian ideograph
        0x227129 => [0x7d67, false], # East Asian ideograph
        0x21712a => [0x5605, false], # East Asian ideograph
        0x22712b => [0x7d6a, false], # East Asian ideograph
        0x22712c => [0x7d6b, false], # East Asian ideograph
        0x216832 => [0x50d4, false], # East Asian ideograph
        0x21712f => [0x5586, false], # East Asian ideograph
        0x227130 => [0x7d73, false], # East Asian ideograph
        0x227134 => [0x7d4e, false], # East Asian ideograph
        0x217136 => [0x55b4, false], # East Asian ideograph
        0x227137 => [0x7d8b, false], # East Asian ideograph
        0x227139 => [0x7d88, false], # East Asian ideograph
        0x22713b => [0x7d85, false], # East Asian ideograph
        0x2d514a => [0x6dd6, false], # East Asian ideograph
        0x22713d => [0x7d8e, false], # East Asian ideograph
        0x216835 => [0x50e6, false], # East Asian ideograph
        0x6f5948 => [0xcd08, false], # Korean hangul
        0x227142 => [0x7d7f, false], # East Asian ideograph
        0x217143 => [0x55e2, false], # East Asian ideograph (variant of 2D7143 which maps to 55E2)
        0x227144 => [0x7d86, false], # East Asian ideograph
        0x217145 => [0x558e, false], # East Asian ideograph
        0x217147 => [0x55b5, false], # East Asian ideograph
        0x227148 => [0x7d8d, false], # East Asian ideograph
        0x217149 => [0x558f, false], # East Asian ideograph
        0x21714b => [0x5559, false], # East Asian ideograph
        0x22714d => [0x7d83, false], # East Asian ideograph
        0x22714f => [0x7d7d, false], # East Asian ideograph
        0x217150 => [0x55a4, false], # East Asian ideograph
        0x217151 => [0x5592, false], # East Asian ideograph
        0x217152 => [0x5599, false], # East Asian ideograph
        0x227154 => [0x7d7b, false], # East Asian ideograph
        0x233a62 => [0x8e90, false], # East Asian ideograph
        0x217156 => [0x55f4, false], # East Asian ideograph
        0x227158 => [0x7d7a, false], # East Asian ideograph
        0x227159 => [0x7d96, false], # East Asian ideograph
        0x22715a => [0x7d5b, false], # East Asian ideograph
        0x22715b => [0x7d8c, false], # East Asian ideograph
        0x21715c => [0x55de, false], # East Asian ideograph
        0x21715d => [0x55d9, false], # East Asian ideograph
        0x21715e => [0x55c3, false], # East Asian ideograph
        0x21715f => [0x55c9, false], # East Asian ideograph
        0x217161 => [0x55ca, false], # East Asian ideograph
        0x227162 => [0x7dae, false], # East Asian ideograph
        0x21683b => [0x50ce, false], # East Asian ideograph
        0x217164 => [0x55d4, false], # East Asian ideograph
        0x217165 => [0x55c4, false], # East Asian ideograph
        0x227167 => [0x7dcb, false], # East Asian ideograph
        0x227169 => [0x7daa, false], # East Asian ideograph
        0x22716a => [0x7dce, false], # East Asian ideograph
        0x22716b => [0x7dc9, false], # East Asian ideograph
        0x692565 => [0x30e5, false], # Katakana letter small YU
        0x22716e => [0x7dc5, false], # East Asian ideograph
        0x22716f => [0x7da6, false], # East Asian ideograph
        0x217170 => [0x55d2, false], # East Asian ideograph
        0x223664 => [0x6585, false], # East Asian ideograph
        0x277c36 => [0x59ab, false], # East Asian ideograph
        0x22543a => [0x71f5, false], # East Asian ideograph
        0x227174 => [0x7dc4, false], # East Asian ideograph
        0x217175 => [0x55e5, false], # East Asian ideograph
        0x6f4871 => [0xac1c, false], # Korean hangul
        0x217177 => [0x55d6, false], # East Asian ideograph
        0x227178 => [0x7dac, false], # East Asian ideograph
        0x217179 => [0x55f2, false], # East Asian ideograph
        0x6f5c77 => [0xd590, false], # Korean hangul
        0x2e717c => [0x7d63, false], # East Asian ideograph
        0x22717d => [0x7db9, false], # East Asian ideograph
        0x21717e => [0x5627, false], # East Asian ideograph
        0x292840 => [0x84e0, false], # East Asian ideograph
        0x235f5f => [0x9f37, false], # East Asian ideograph
        0x216841 => [0x50f3, false], # East Asian ideograph
        0x216842 => [0x50e8, false], # East Asian ideograph
        0x217255 => [0x5635, false], # East Asian ideograph
        0x2d514d => [0x7d2c, false], # East Asian ideograph
        0x216844 => [0x50f0, false], # East Asian ideograph
        0x6f594b => [0xcd10, false], # Korean hangul
        0x224772 => [0x6cf5, false], # East Asian ideograph
        0x6f4e69 => [0xb78d, false], # Korean hangul
        0x23307d => [0x89af, false], # East Asian ideograph
        0x295b35 => [0x9e2b, false], # East Asian ideograph
        0x4b7874 => [0x590a, false], # East Asian ideograph
        0x23284c => [0x8645, false], # East Asian ideograph
        0x6f4a26 => [0xadd1, false], # Korean hangul
        0x22543d => [0x71f3, false], # East Asian ideograph
        0x234e53 => [0x97d8, false], # East Asian ideograph
        0x69684d => [0x8422, false], # East Asian ideograph
        0x333623 => [0x9f69, false], # East Asian ideograph
        0x225b61 => [0x74b1, false], # East Asian ideograph
        0x6f553b => [0xc58c, false], # Korean hangul
        0x706d3f => [0x781c, false], # East Asian ideograph
        0x4b587a => [0x8acb, false], # East Asian ideograph
        0x6f7723 => [0xe8ca, false], # Korean hangul
        0x213f32 => [0x615d, false], # East Asian ideograph
        0x234730 => [0x93e6, false], # East Asian ideograph
        0x222851 => [0x5ecc, false], # East Asian ideograph
        0x6f594e => [0xcd1b, false], # Korean hangul
        0x284e62 => [0x6f4d, false], # East Asian ideograph
        0x6f5732 => [0xc7cc, false], # Korean hangul
        0x6f5328 => [0xc126, false], # Korean hangul
        0x213f33 => [0x6182, false], # East Asian ideograph
        0x285f6f => [0x7605, false], # East Asian ideograph
        0x6f4a3d => [0xae43, false], # Korean hangul
        0x6f4e25 => [0xb545, false], # Korean hangul
        0x295f2b => [0x9f0d, false], # East Asian ideograph
        0x212a35 => [0xe8e2, false], # EACC component character
        0x6f594f => [0xcd1d, false], # Korean hangul
        0x6f5b33 => [0xd15d, false], # Korean hangul
        0x274621 => [0x6b22, false], # East Asian ideograph
        0x232859 => [0x864d, false], # East Asian ideograph
        0x224333 => [0x6adf, false], # East Asian ideograph
        0x234732 => [0x940b, false], # East Asian ideograph
        0x21797c => [0x5997, false], # East Asian ideograph
        0x23285a => [0x8653, false], # East Asian ideograph
        0x6f4e43 => [0xb614, false], # Korean hangul
        0x227222 => [0x7d9f, false], # East Asian ideograph
        0x217224 => [0x55fb, false], # East Asian ideograph
        0x217225 => [0x5612, false], # East Asian ideograph
        0x217227 => [0x55f8, false], # East Asian ideograph
        0x217228 => [0x560f, false], # East Asian ideograph
        0x227229 => [0x7de1, false], # East Asian ideograph
        0x22722a => [0x7dd9, false], # East Asian ideograph
        0x22722b => [0x7de4, false], # East Asian ideograph
        0x6f4f44 => [0xb8e9, false], # Korean hangul
        0x21722e => [0x561e, false], # East Asian ideograph
        0x217539 => [0x5790, false], # East Asian ideograph
        0x227231 => [0x7dd7, false], # East Asian ideograph
        0x295263 => [0x9a75, false], # East Asian ideograph
        0x217234 => [0x561c, false], # East Asian ideograph
        0x217235 => [0x5610, false], # East Asian ideograph
        0x217236 => [0x5601, false], # East Asian ideograph
        0x217238 => [0x5613, false], # East Asian ideograph
        0x217239 => [0x55f6, false], # East Asian ideograph
        0x22723a => [0x7e06, false], # East Asian ideograph
        0x21685f => [0x5105, false], # East Asian ideograph
        0x21723c => [0x5602, false], # East Asian ideograph
        0x22723e => [0x7de6, false], # East Asian ideograph
        0x697240 => [0x9bb4, false], # East Asian ideograph
        0x217242 => [0x561d, false], # East Asian ideograph
        0x27577c => [0x88c6, false], # East Asian ideograph
        0x217244 => [0x55ff, false], # East Asian ideograph
        0x697245 => [0x9bcf, false], # East Asian ideograph
        0x227246 => [0x7ddc, false], # East Asian ideograph
        0x216861 => [0x50fc, false], # East Asian ideograph
        0x217248 => [0x564c, false], # East Asian ideograph
        0x227249 => [0x7de5, false], # East Asian ideograph
        0x22724b => [0x7df5, false], # East Asian ideograph
        0x6f4e6a => [0xb78f, false], # Korean hangul
        0x295021 => [0x98a2, false], # East Asian ideograph
        0x2e7328 => [0x5fad, false], # East Asian ideograph
        0x227250 => [0x7e17, false], # East Asian ideograph
        0x227251 => [0x7e1e, false], # East Asian ideograph
        0x227252 => [0x7e21, false], # East Asian ideograph
        0x227253 => [0x7e0b, false], # East Asian ideograph
        0x224335 => [0x6ade, false], # East Asian ideograph
        0x227256 => [0x7e22, false], # East Asian ideograph
        0x217257 => [0x5649, false], # East Asian ideograph
        0x217258 => [0x5641, false], # East Asian ideograph
        0x2d5124 => [0x5e0b, false], # East Asian ideograph
        0x22725b => [0x7e20, false], # East Asian ideograph
        0x21725c => [0x5658, false], # East Asian ideograph
        0x22725d => [0x7e1d, false], # East Asian ideograph
        0x21725e => [0x5654, false], # East Asian ideograph
        0x216865 => [0x5106, false], # East Asian ideograph
        0x217260 => [0x562a, false], # East Asian ideograph
        0x217261 => [0x563d, false], # East Asian ideograph
        0x217264 => [0x562c, false], # East Asian ideograph
        0x227265 => [0x7e15, false], # East Asian ideograph
        0x6f5474 => [0xc52c, false], # Korean hangul
        0x217267 => [0x5638, false], # East Asian ideograph
        0x4d5154 => [0x9942, false], # East Asian ideograph
        0x217269 => [0x564d, false], # East Asian ideograph
        0x22726a => [0x7e0f, false], # East Asian ideograph
        0x21726b => [0x562b, false], # East Asian ideograph
        0x21726c => [0x564f, false], # East Asian ideograph
        0x22726d => [0x7e3b, false], # East Asian ideograph
        0x21726e => [0x5670, false], # East Asian ideograph
        0x21726f => [0x565f, false], # East Asian ideograph
        0x217270 => [0x567c, false], # East Asian ideograph
        0x227271 => [0x7e34, false], # East Asian ideograph
        0x227272 => [0x7e2d, false], # East Asian ideograph
        0x227273 => [0x7e2f, false], # East Asian ideograph
        0x227275 => [0x7e36, false], # East Asian ideograph
        0x227277 => [0x7e3a, false], # East Asian ideograph
        0x217278 => [0x5676, false], # East Asian ideograph
        0x227279 => [0x7e39, false], # East Asian ideograph
        0x21727a => [0x5666, false], # East Asian ideograph
        0x21727b => [0x5673, false], # East Asian ideograph
        0x21727c => [0x566d, false], # East Asian ideograph
        0x22727d => [0x7e44, false], # East Asian ideograph
        0x21727e => [0x5672, false], # East Asian ideograph
        0x33537d => [0x9ad5, false], # East Asian ideograph
        0x4b3e7e => [0x60a9, false], # East Asian ideograph
        0x6f5953 => [0xcd94, false], # Korean hangul
        0x6f7729 => [0xae0e, false], # Korean hangul
        0x224b30 => [0x6e36, false], # East Asian ideograph
        0x2d6262 => [0x9ec4, false], # East Asian ideograph
        0x2d4049 => [0x67b4, false], # East Asian ideograph
        0x4b5c54 => [0x8f9f, false], # East Asian ideograph (duplicate simplified)
        0x2e686f => [0x7a19, false], # East Asian ideograph
        0x6f4873 => [0xac20, false], # Korean hangul
        0x33362a => [0x9b28, false], # East Asian ideograph
        0x216871 => [0x5115, false], # East Asian ideograph
        0x6f5954 => [0xcd95, false], # Korean hangul
        0x6f5b34 => [0xd15f, false], # Korean hangul
        0x234b7a => [0x96f5, false], # East Asian ideograph
        0x6f4f45 => [0xb8ec, false], # Korean hangul
        0x6f5031 => [0xba67, false], # Korean hangul
        0x6f5955 => [0xcd98, false], # Korean hangul
        0x215321 => [0x8085, false], # East Asian ideograph
        0x6f772b => [0xae11, false], # Korean hangul
        0x213f3a => [0x615f, false], # East Asian ideograph
        0x6f5322 => [0xc11d, false], # Korean hangul
        0x225323 => [0x7192, false], # East Asian ideograph
        0x2d5e21 => [0x9418, false], # East Asian ideograph
        0x29415c => [0x917e, false], # East Asian ideograph
        0x215324 => [0x808b, false], # East Asian ideograph
        0x6f5325 => [0xc123, false], # Korean hangul
        0x224539 => [0x6bab, false], # East Asian ideograph
        0x6f5956 => [0xcd9c, false], # Korean hangul
        0x695326 => [0x54d8, false], # East Asian ideograph
        0x22477d => [0x6cc2, false], # East Asian ideograph
        0x23287c => [0x866f, false], # East Asian ideograph
        0x6f5327 => [0xc125, false], # Korean hangul
        0x275e35 => [0x9558, false], # East Asian ideograph
        0x2d404c => [0x6283, false], # East Asian ideograph
        0x6f4b6b => [0xb0e0, false], # Korean hangul
        0x275735 => [0x8681, false], # East Asian ideograph
        0x6f4a28 => [0xaddc, false], # Korean hangul
        0x235329 => [0x99f8, false], # East Asian ideograph
        0x225447 => [0x71e0, false], # East Asian ideograph
        0x23532a => [0x99f4, false], # East Asian ideograph
        0x6f5957 => [0xcda4, false], # Korean hangul
        0x21532b => [0x8096, false], # East Asian ideograph
        0x276842 => [0x507e, false], # East Asian ideograph
        0x274629 => [0x5c81, false], # East Asian ideograph
        0x213f3c => [0x617e, false], # East Asian ideograph
        0x21532c => [0x80b2, false], # East Asian ideograph
        0x6f5235 => [0xbed8, false], # Korean hangul
        0x6f532d => [0xc12f, false], # Korean hangul
        0x213273 => [0x5147, false], # East Asian ideograph
        0x6f532e => [0xc130, false], # Korean hangul
        0x275d75 => [0x9525, false], # East Asian ideograph
        0x216f43 => [0x546b, false], # East Asian ideograph
        0x6f5958 => [0xcda5, false], # Korean hangul
        0x215330 => [0x80a2, false], # East Asian ideograph
        0x27462a => [0x5386, false], # East Asian ideograph
        0x217325 => [0x5693, false], # East Asian ideograph
        0x227326 => [0x7e3f, false], # East Asian ideograph
        0x215331 => [0x80ab, false], # East Asian ideograph
        0x227328 => [0x7e47, false], # East Asian ideograph
        0x225332 => [0x7185, false], # East Asian ideograph
        0x22732f => [0x7e51, false], # East Asian ideograph
        0x217332 => [0x56ba, false], # East Asian ideograph
        0x215333 => [0x80af, false], # East Asian ideograph
        0x227334 => [0x7e67, false], # East Asian ideograph
        0x217335 => [0x5684, false], # East Asian ideograph
        0x217336 => [0x5691, false], # East Asian ideograph
        0x227337 => [0x7e56, false], # East Asian ideograph
        0x6f5334 => [0xc140, false], # Korean hangul
        0x21733e => [0x569e, false], # East Asian ideograph
        0x6f5335 => [0xc148, false], # Korean hangul
        0x27462b => [0x5f52, false], # East Asian ideograph
        0x217342 => [0x569a, false], # East Asian ideograph
        0x213f3e => [0x61b2, false], # East Asian ideograph
        0x225336 => [0x717c, false], # East Asian ideograph
        0x227348 => [0x7e68, false], # East Asian ideograph
        0x227349 => [0x7e6e, false], # East Asian ideograph
        0x21734b => [0x56ad, false], # East Asian ideograph
        0x21734c => [0x56a6, false], # East Asian ideograph
        0x22734e => [0x7e70, false], # East Asian ideograph
        0x6f4e66 => [0xb780, false], # Korean hangul
        0x227351 => [0x7e6f, false], # East Asian ideograph
        0x227352 => [0x7e73, false], # East Asian ideograph
        0x217353 => [0x56b2, false], # East Asian ideograph
        0x235849 => [0x9c0a, false], # East Asian ideograph
        0x225339 => [0x7198, false], # East Asian ideograph
        0x227358 => [0x7e7b, false], # East Asian ideograph
        0x227359 => [0x7e7e, false], # East Asian ideograph
        0x21735a => [0x56b3, false], # East Asian ideograph
        0x22735b => [0x7e81, false], # East Asian ideograph
        0x6f595a => [0xcda9, false], # Korean hangul
        0x22735d => [0x7e8a, false], # East Asian ideograph
        0x22735e => [0x7e87, false], # East Asian ideograph
        0x6f7730 => [0xaf50, false], # Korean hangul
        0x227360 => [0x7e88, false], # East Asian ideograph
        0x217362 => [0x56cf, false], # East Asian ideograph
        0x4b533b => [0x695c, false], # East Asian ideograph
        0x227364 => [0x7e86, false], # East Asian ideograph
        0x23473d => [0x93fb, false], # East Asian ideograph
        0x217367 => [0x56cd, false], # East Asian ideograph
        0x22533c => [0x7197, false], # East Asian ideograph
        0x22736a => [0x7e91, false], # East Asian ideograph
        0x21736b => [0x56d7, false], # East Asian ideograph
        0x22736d => [0x7e94, false], # East Asian ideograph
        0x70736e => [0x7ba2, false], # East Asian ideograph
        0x23533d => [0x9a0f, false], # East Asian ideograph
        0x227370 => [0x7e9b, false], # East Asian ideograph
        0x227371 => [0x7e9a, false], # East Asian ideograph
        0x22544b => [0x720c, false], # East Asian ideograph
        0x227373 => [0x7e99, false], # East Asian ideograph
        0x227374 => [0x7e98, false], # East Asian ideograph
        0x22533e => [0x71b5, false], # East Asian ideograph
        0x217376 => [0x56ee, false], # East Asian ideograph
        0x217377 => [0x56e7, false], # East Asian ideograph
        0x217379 => [0x56fb, false], # East Asian ideograph
        0x22533f => [0x71a9, false], # East Asian ideograph
        0x21737e => [0x56f7, false], # East Asian ideograph
        0x222569 => [0x5d97, false], # East Asian ideograph
        0x295340 => [0x9a92, false], # East Asian ideograph
        0x4b3853 => [0x586d, false], # East Asian ideograph (not in Unicode)
        0x4b5629 => [0x85cd, false], # East Asian ideograph
        0x6f5341 => [0xc18e, false], # Korean hangul
        0x6f4a29 => [0xade0, false], # Korean hangul
        0x225342 => [0x71a5, false], # East Asian ideograph
        0x334260 => [0x89d4, false], # East Asian ideograph
        0x275e50 => [0x95ed, false], # East Asian ideograph
        0x23584b => [0x9c08, false], # East Asian ideograph
        0x2e3936 => [0x66cd, false], # East Asian ideograph
        0x6f595c => [0xcdc4, false], # Korean hangul
        0x235344 => [0x9a04, false], # East Asian ideograph
        0x6f7732 => [0xb060, false], # Korean hangul
        0x235345 => [0x9a11, false], # East Asian ideograph
        0x275b7e => [0x8fde, false], # East Asian ideograph
        0x2d5e28 => [0x93c1, false], # East Asian ideograph
        0x213a6a => [0x5b8b, false], # East Asian ideograph
        0x235347 => [0x9a05, false], # East Asian ideograph
        0x4b4874 => [0x6ff3, false], # East Asian ideograph
        0x23584c => [0x9c14, false], # East Asian ideograph
        0x215348 => [0x80fd, false], # East Asian ideograph
        0x6f5349 => [0xc1a8, false], # Korean hangul
        0x705d5c => [0x8488, false], # East Asian ideograph
        0x6f7733 => [0xb9c4, false], # Korean hangul
        0x224b32 => [0x6e72, false], # East Asian ideograph
        0x6f5128 => [0xbc88, false], # Korean hangul
        0x22655a => [0x78d8, false], # East Asian ideograph
        0x27534a => [0x8090, false], # East Asian ideograph
        0x226d4b => [0x7c0c, false], # East Asian ideograph
        0x334740 => [0x6d1a, false], # East Asian ideograph
        0x4b562b => [0x8535, false], # East Asian ideograph
        0x2d534b => [0x80f7, false], # East Asian ideograph
        0x27573c => [0x86ce, false], # East Asian ideograph
        0x21534c => [0x810a, false], # East Asian ideograph
        0x23584d => [0x9c04, false], # East Asian ideograph
        0x23534d => [0x9a22, false], # East Asian ideograph
        0x6f595e => [0xcde8, false], # Korean hangul
        0x22534e => [0x71af, false], # East Asian ideograph
        0x6f5b36 => [0xd161, false], # Korean hangul
        0x6f7734 => [0xc54d, false], # Korean hangul
        0x23534f => [0x9a20, false], # East Asian ideograph
        0x6f5350 => [0xc1e4, false], # Korean hangul
        0x21327a => [0x5152, false], # East Asian ideograph
        0x2d6134 => [0x99de, false], # East Asian ideograph
        0x233a78 => [0x8eb3, false], # East Asian ideograph
        0x4c6f7b => [0x7ce8, false], # East Asian ideograph
        0x235352 => [0x9a27, false], # East Asian ideograph
        0x6f5343 => [0xc194, false], # Korean hangul
        0x6f5353 => [0xc1f1, false], # Korean hangul
        0x6f7735 => [0xc54f, false], # Korean hangul
        0x213f44 => [0x619a, false], # East Asian ideograph
        0x6f5354 => [0xc1f3, false], # Korean hangul
        0x4c7c45 => [0x82ae, false], # East Asian ideograph
        0x225355 => [0x719a, false], # East Asian ideograph
        0x4d4832 => [0x953f, false], # East Asian ideograph
        0x27573e => [0x8721, false], # East Asian ideograph
        0x22367a => [0x65a0, false], # East Asian ideograph
        0x223237 => [0x63b1, false], # East Asian ideograph
        0x6f5629 => [0xc65c, false], # Korean hangul
        0x225357 => [0x71b3, false], # East Asian ideograph
        0x27407b => [0x5377, false], # East Asian ideograph
        0x275358 => [0x80be, false], # East Asian ideograph
        0x213f45 => [0x61a9, false], # East Asian ideograph
        0x215359 => [0x8139, false], # East Asian ideograph
        0x2d416e => [0x6534, false], # East Asian ideograph
        0x23535a => [0x9a38, false], # East Asian ideograph
        0x217421 => [0x56f9, false], # East Asian ideograph
        0x6f4a2a => [0xade4, false], # Korean hangul
        0x294435 => [0x950a, false], # East Asian ideograph
        0x217424 => [0x56ff, false], # East Asian ideograph
        0x227425 => [0x7f43, false], # East Asian ideograph
        0x275e51 => [0x95f5, false], # East Asian ideograph
        0x217427 => [0x5705, false], # East Asian ideograph
        0x227428 => [0x7f45, false], # East Asian ideograph
        0x217429 => [0x5702, false], # East Asian ideograph
        0x22742b => [0x7f4b, false], # East Asian ideograph
        0x21742c => [0x570a, false], # East Asian ideograph
        0x21742d => [0x5709, false], # East Asian ideograph
        0x22742e => [0x7f4c, false], # East Asian ideograph
        0x22742f => [0x7f4d, false], # East Asian ideograph
        0x217430 => [0x570c, false], # East Asian ideograph
        0x217431 => [0x5715, false], # East Asian ideograph
        0x227432 => [0x7f4f, false], # East Asian ideograph
        0x213f46 => [0x6194, false], # East Asian ideograph
        0x27535e => [0x80a0, false], # East Asian ideograph
        0x224345 => [0x6ae8, false], # East Asian ideograph
        0x217437 => [0x571c, false], # East Asian ideograph
        0x4b423a => [0x64b9, false], # East Asian ideograph
        0x217439 => [0x571d, false], # East Asian ideograph
        0x21743a => [0x571e, false], # East Asian ideograph
        0x6f535f => [0xc220, false], # Korean hangul
        0x22743e => [0x7f60, false], # East Asian ideograph
        0x22743f => [0x7f61, false], # East Asian ideograph
        0x235360 => [0x9a2d, false], # East Asian ideograph
        0x217442 => [0x572e, false], # East Asian ideograph
        0x227443 => [0x7f5d, false], # East Asian ideograph
        0x227445 => [0x7f5b, false], # East Asian ideograph
        0x235361 => [0x9a35, false], # East Asian ideograph
        0x217448 => [0x5738, false], # East Asian ideograph
        0x21744c => [0x572a, false], # East Asian ideograph
        0x215362 => [0x816b, false], # East Asian ideograph
        0x6f4b43 => [0xb05d, false], # Korean hangul
        0x227450 => [0x7f65, false], # East Asian ideograph
        0x227451 => [0x7f66, false], # East Asian ideograph
        0x213f47 => [0x618a, false], # East Asian ideograph
        0x227453 => [0x7f6d, false], # East Asian ideograph
        0x227454 => [0x7f6b, false], # East Asian ideograph
        0x227455 => [0x7f67, false], # East Asian ideograph
        0x227457 => [0x7f68, false], # East Asian ideograph
        0x235364 => [0x9a32, false], # East Asian ideograph
        0x21327e => [0x5165, false], # East Asian ideograph
        0x22745e => [0x7f71, false], # East Asian ideograph
        0x275365 => [0x8111, false], # East Asian ideograph
        0x227460 => [0x7f73, false], # East Asian ideograph
        0x227463 => [0x7f76, false], # East Asian ideograph
        0x6f5022 => [0xba39, false], # Korean hangul
        0x217465 => [0x5745, false], # East Asian ideograph
        0x6f572f => [0xc7c1, false], # Korean hangul
        0x217468 => [0x574b, false], # East Asian ideograph
        0x217469 => [0x574c, false], # East Asian ideograph
        0x21746a => [0x573f, false], # East Asian ideograph
        0x215367 => [0x818f, false], # East Asian ideograph
        0x22746c => [0x7f7d, false], # East Asian ideograph
        0x6f7739 => [0xc61c, false], # Korean hangul
        0x217470 => [0x5768, false], # East Asian ideograph
        0x6f5368 => [0xc250, false], # Korean hangul
        0x227472 => [0x7f86, false], # East Asian ideograph
        0x6f4f25 => [0xb807, false], # Korean hangul
        0x217475 => [0x578a, false], # East Asian ideograph
        0x225369 => [0x71c7, false], # East Asian ideograph
        0x345175 => [0x7162, false], # East Asian ideograph
        0x217479 => [0x5774, false], # East Asian ideograph
        0x21747a => [0x5767, false], # East Asian ideograph
        0x22536a => [0x71b7, false], # East Asian ideograph
        0x22747e => [0x7f96, false], # East Asian ideograph
        0x6f536b => [0xc270, false], # Korean hangul
        0x27536c => [0x80f6, false], # East Asian ideograph
        0x274636 => [0x6b93, false], # East Asian ideograph
        0x6f5521 => [0xc549, false], # Korean hangul
        0x21536d => [0x819b, false], # East Asian ideograph
        0x224348 => [0x6af5, false], # East Asian ideograph
        0x4b5632 => [0x7c54, false], # East Asian ideograph
        0x27536e => [0x80a4, false], # East Asian ideograph
        0x454f45 => [0x9896, false], # East Asian ideograph
        0x22536f => [0x71cf, false], # East Asian ideograph
        0x4c5541 => [0x4e2c, false], # East Asian ideograph
        0x225370 => [0x71d6, false], # East Asian ideograph
        0x213031 => [0x4e1e, false], # East Asian ideograph
        0x215371 => [0x81a9, false], # East Asian ideograph
        0x274637 => [0x6ba1, false], # East Asian ideograph
        0x6f5522 => [0xc54a, false], # Korean hangul
        0x213f4a => [0x61c9, false], # East Asian ideograph
        0x217463 => [0x5749, false], # East Asian ideograph
        0x6f5373 => [0xc290, false], # Korean hangul
        0x335172 => [0x7d89, false], # East Asian ideograph
        0x212a29 => [0xe8d7, false], # EACC component character
        0x6f4a2b => [0xadec, false], # Korean hangul
        0x235374 => [0x9a3b, false], # East Asian ideograph
        0x4b496a => [0x932c, false], # East Asian ideograph
        0x225375 => [0x71c2, false], # East Asian ideograph
        0x6f5376 => [0xc29d, false], # Korean hangul
        0x233e21 => [0x9070, false], # East Asian ideograph
        0x213f4b => [0x6190, false], # East Asian ideograph
        0x225377 => [0x71c5, false], # East Asian ideograph
        0x4b385e => [0x5897, false], # East Asian ideograph
        0x234749 => [0x93fa, false], # East Asian ideograph
        0x2d6275 => [0x76b7, false], # East Asian ideograph
        0x215378 => [0x81bf, false], # East Asian ideograph
        0x216431 => [0x4e36, false], # East Asian ideograph
        0x215379 => [0x81bd, false], # East Asian ideograph
        0x223e24 => [0x6919, false], # East Asian ideograph
        0x4b496b => [0x83f8, false], # East Asian ideograph
        0x21537a => [0x81c9, false], # East Asian ideograph
        0x213e25 => [0x5ffd, false], # East Asian ideograph
        0x21537b => [0x81be, false], # East Asian ideograph
        0x39304c => [0x4e81, false], # East Asian ideograph
        0x213e26 => [0x5fdd, false], # East Asian ideograph
        0x23603f => [0x9f6e, false], # East Asian ideograph
        0x27537c => [0x8110, false], # East Asian ideograph
        0x33474a => [0x6d1f, false], # East Asian ideograph
        0x21537d => [0x81cf, false], # East Asian ideograph
        0x213e28 => [0x5fff, false], # East Asian ideograph
        0x275746 => [0x672e, false], # East Asian ideograph
        0x21537e => [0x81d8, false], # East Asian ideograph
        0x6f4e26 => [0xb54b, false], # Korean hangul
        0x227042 => [0x7d06, false], # East Asian ideograph
        0x294471 => [0x951d, false], # East Asian ideograph
        0x233e2a => [0x907b, false], # East Asian ideograph
        0x6f5968 => [0xce61, false], # Korean hangul
        0x6f5b38 => [0xd1a0, false], # Korean hangul
        0x213e2b => [0x602a, false], # East Asian ideograph
        0x213e2c => [0x602f, false], # East Asian ideograph
        0x6f585b => [0xcacc, false], # Korean hangul
        0x213e2d => [0x6016, false], # East Asian ideograph
        0x213e2f => [0x600f, false], # East Asian ideograph
        0x6f5969 => [0xce68, false], # Korean hangul
        0x227523 => [0x7f97, false], # East Asian ideograph
        0x227524 => [0x7f95, false], # East Asian ideograph
        0x51456d => [0x822e, false], # East Asian ideograph
        0x217526 => [0x5770, false], # East Asian ideograph
        0x217528 => [0x5771, false], # East Asian ideograph
        0x21752a => [0x576e, false], # East Asian ideograph
        0x22752c => [0x7fa2, false], # East Asian ideograph
        0x21752d => [0x5776, false], # East Asian ideograph
        0x21752e => [0x5789, false], # East Asian ideograph
        0x217530 => [0x577f, false], # East Asian ideograph
        0x217531 => [0x5775, false], # East Asian ideograph
        0x217532 => [0x577b, false], # East Asian ideograph
        0x227533 => [0x7fa7, false], # East Asian ideograph
        0x217535 => [0x5773, false], # East Asian ideograph
        0x223241 => [0x636d, false], # East Asian ideograph
        0x217538 => [0x579f, false], # East Asian ideograph
        0x233e34 => [0x9083, false], # East Asian ideograph
        0x21753a => [0x5793, false], # East Asian ideograph
        0x22753b => [0x7fb0, false], # East Asian ideograph
        0x22753c => [0x7fad, false], # East Asian ideograph
        0x6f596a => [0xce69, false], # Korean hangul
        0x22753f => [0x7fb1, false], # East Asian ideograph
        0x227540 => [0x7fb4, false], # East Asian ideograph
        0x6f5527 => [0xc555, false], # Korean hangul
        0x227542 => [0x7fb5, false], # East Asian ideograph
        0x217543 => [0x579a, false], # East Asian ideograph
        0x217545 => [0x5794, false], # East Asian ideograph
        0x217547 => [0x57a4, false], # East Asian ideograph
        0x217548 => [0x5799, false], # East Asian ideograph
        0x217549 => [0x578c, false], # East Asian ideograph
        0x22754a => [0x7fbc, false], # East Asian ideograph
        0x21754b => [0x5797, false], # East Asian ideograph
        0x22754c => [0x7fbe, false], # East Asian ideograph
        0x275749 => [0x536b, false], # East Asian ideograph
        0x227551 => [0x7fc3, false], # East Asian ideograph
        0x217552 => [0x579c, false], # East Asian ideograph
        0x217554 => [0x57a7, false], # East Asian ideograph
        0x227557 => [0x7fca, false], # East Asian ideograph
        0x217559 => [0x3013, false], # East Asian ideograph (not found in unified han)
        0x21755b => [0x5795, false], # East Asian ideograph
        0x213e3a => [0x6068, false], # East Asian ideograph
        0x21755f => [0x57b8, false], # East Asian ideograph
        0x217560 => [0x57c7, false], # East Asian ideograph
        0x227567 => [0x7fdb, false], # East Asian ideograph
        0x227568 => [0x7fe3, false], # East Asian ideograph
        0x2d3e3c => [0x803b, false], # East Asian ideograph
        0x21756a => [0x5809, false], # East Asian ideograph
        0x22756c => [0x7fe6, false], # East Asian ideograph
        0x22756f => [0x7fe5, false], # East Asian ideograph
        0x22545c => [0x5911, false], # East Asian ideograph
        0x217571 => [0x57db, false], # East Asian ideograph
        0x227572 => [0x7fec, false], # East Asian ideograph
        0x227573 => [0x7feb, false], # East Asian ideograph
        0x273b60 => [0x5c61, false], # East Asian ideograph
        0x213e3e => [0x606d, false], # East Asian ideograph
        0x227577 => [0x7fef, false], # East Asian ideograph
        0x6f596c => [0xce6d, false], # Korean hangul
        0x22757a => [0x7fee, false], # East Asian ideograph
        0x233e3f => [0x9099, false], # East Asian ideograph
        0x28632c => [0x7751, false], # East Asian ideograph
        0x293066 => [0x89c7, false], # East Asian ideograph
        0x21757e => [0x57c6, false], # East Asian ideograph
        0x233e40 => [0x9097, false], # East Asian ideograph
        0x4b563a => [0x82a6, false], # East Asian ideograph (variant of 27563A which maps to 82A6)
        0x4b3421 => [0x5263, false], # East Asian ideograph
        0x275936 => [0x8c23, false], # East Asian ideograph
        0x213e41 => [0x604d, false], # East Asian ideograph
        0x6f4860 => [0xac01, false], # Korean hangul
        0x4c695f => [0x7a63, false], # East Asian ideograph
        0x213e42 => [0x606b, false], # East Asian ideograph
        0x395f49 => [0x5f6b, false], # East Asian ideograph
        0x23585c => [0x9c09, false], # East Asian ideograph
        0x233643 => [0x8c9f, false], # East Asian ideograph
        0x213e43 => [0x6069, false], # East Asian ideograph
        0x6f5b39 => [0xd1a1, false], # Korean hangul
        0x223e44 => [0x6911, false], # East Asian ideograph
        0x6f552a => [0xc559, false], # Korean hangul
        0x4b5a7e => [0x5c69, false], # East Asian ideograph
        0x213e46 => [0x606a, false], # East Asian ideograph
        0x6f4861 => [0xac02, false], # Korean hangul
        0x223e47 => [0x68ef, false], # East Asian ideograph
        0x22545e => [0x720a, false], # East Asian ideograph
        0x6f4f4a => [0xb8fd, false], # Korean hangul
        0x213e48 => [0x6070, false], # East Asian ideograph
        0x213e49 => [0x6055, false], # East Asian ideograph
        0x274640 => [0x6bb4, false], # East Asian ideograph
        0x234751 => [0x9427, false], # East Asian ideograph
        0x33433e => [0x95c7, false], # East Asian ideograph
        0x213e4b => [0x60a6, false], # East Asian ideograph
        0x4d4835 => [0x954c, false], # East Asian ideograph
        0x2d3c21 => [0x57fc, false], # East Asian ideograph
        0x275f2e => [0x9633, false], # East Asian ideograph
        0x393e4c => [0x6142, false], # East Asian ideograph
        0x4b4973 => [0x7115, false], # East Asian ideograph
        0x213e4d => [0x609f, false], # East Asian ideograph
        0x27407e => [0x626a, false], # East Asian ideograph
        0x6f596f => [0xce74, false], # Korean hangul
        0x6f552c => [0xc55f, false], # Korean hangul
        0x697058 => [0x9779, false], # East Asian ideograph
        0x275e36 => [0x9559, false], # East Asian ideograph
        0x2d627e => [0x658b, false], # East Asian ideograph
        0x213b48 => [0x5c1a, false], # East Asian ideograph
        0x2d5e3b => [0x92b9, false], # East Asian ideograph
        0x6f4863 => [0xac07, false], # Korean hangul
        0x6f4a2d => [0xadf9, false], # Korean hangul
        0x393078 => [0x9ae3, false], # East Asian ideograph
        0x227e51 => [0x8423, false], # East Asian ideograph
        0x225460 => [0x7217, false], # East Asian ideograph
        0x223247 => [0x63d3, false], # East Asian ideograph
        0x213e52 => [0x60a3, false], # East Asian ideograph
        0x6f5970 => [0xce75, false], # Korean hangul
        0x6f5542 => [0xc598, false], # Korean hangul
        0x223e53 => [0x6974, false], # East Asian ideograph
        0x6f552d => [0xc560, false], # Korean hangul
        0x213e54 => [0x6094, false], # East Asian ideograph
        0x2d4066 => [0x63ce, false], # East Asian ideograph
        0x216433 => [0x4e3f, false], # East Asian ideograph
        0x6f4864 => [0xac08, false], # Korean hangul
        0x4b4975 => [0x6427, false], # East Asian ideograph
        0x275d7a => [0x9532, false], # East Asian ideograph
        0x213e57 => [0x60b4, false], # East Asian ideograph
        0x395d23 => [0x91bb, false], # East Asian ideograph
        0x4b517e => [0x7e92, false], # East Asian ideograph
        0x6f5971 => [0xce78, false], # Korean hangul
        0x223e58 => [0x6962, false], # East Asian ideograph
        0x224b36 => [0x6e39, false], # East Asian ideograph
        0x6f5468 => [0xc4d4, false], # Korean hangul
        0x213e59 => [0x60cb, false], # East Asian ideograph
        0x4b563f => [0x6a98, false], # East Asian ideograph
        0x2d4067 => [0x62cf, false], # East Asian ideograph
        0x6f4865 => [0xac09, false], # Korean hangul
        0x6f7621 => [0x3181, false], # Korean hangul
        0x217622 => [0x57c4, false], # East Asian ideograph
        0x233e5b => [0x90b6, false], # East Asian ideograph
        0x6f7624 => [0xe8b0, false], # Korean hangul
        0x6f7625 => [0xe8b1, false], # Korean hangul
        0x4b4556 => [0x6a2a, false], # East Asian ideograph
        0x217627 => [0x70fe, false], # East Asian ideograph
        0x227629 => [0x7ffd, false], # East Asian ideograph
        0x22762a => [0x7ffe, false], # East Asian ideograph
        0x21762b => [0x5803, false], # East Asian ideograph
        0x22762c => [0x7fff, false], # East Asian ideograph
        0x21762d => [0x57e6, false], # East Asian ideograph
        0x22762e => [0x8004, false], # East Asian ideograph
        0x233e5d => [0x90b0, false], # East Asian ideograph
        0x29332c => [0x8bfc, false], # East Asian ideograph
        0x217631 => [0x57ed, false], # East Asian ideograph
        0x227633 => [0x800b, false], # East Asian ideograph
        0x227634 => [0x800e, false], # East Asian ideograph
        0x227635 => [0x8011, false], # East Asian ideograph
        0x234755 => [0x9409, false], # East Asian ideograph
        0x227637 => [0x8014, false], # East Asian ideograph
        0x277638 => [0x57ad, false], # East Asian ideograph
        0x227639 => [0x8016, false], # East Asian ideograph
        0x223e5f => [0x6957, false], # East Asian ideograph
        0x21763d => [0x57f4, false], # East Asian ideograph
        0x22763e => [0x801d, false], # East Asian ideograph
        0x294179 => [0x9492, false], # East Asian ideograph
        0x217640 => [0x580d, false], # East Asian ideograph
        0x223e60 => [0x693f, false], # East Asian ideograph
        0x6f7642 => [0xe8b4, false], # Korean hangul
        0x217643 => [0x57ef, false], # East Asian ideograph
        0x6f7644 => [0xe8b6, false], # Korean hangul
        0x6f7645 => [0xe8b7, false], # Korean hangul
        0x6f4f4b => [0xb904, false], # Korean hangul
        0x273e61 => [0x6076, false], # East Asian ideograph
        0x217648 => [0x5801, false], # East Asian ideograph
        0x217649 => [0x5812, false], # East Asian ideograph
        0x6f764a => [0xe8bc, false], # Korean hangul
        0x22764b => [0x8025, false], # East Asian ideograph
        0x22764c => [0x8026, false], # East Asian ideograph
        0x22764d => [0x802a, false], # East Asian ideograph
        0x22764e => [0x8029, false], # East Asian ideograph
        0x22764f => [0x8028, false], # East Asian ideograph
        0x217650 => [0x580c, false], # East Asian ideograph
        0x217651 => [0x5813, false], # East Asian ideograph
        0x217652 => [0x57f0, false], # East Asian ideograph
        0x6f7653 => [0xe8c5, false], # Korean hangul
        0x6f7654 => [0xe8c6, false], # Korean hangul
        0x287655 => [0x8027, false], # East Asian ideograph
        0x217656 => [0x580b, false], # East Asian ideograph
        0x6f7657 => [0xe8c9, false], # Korean hangul
        0x217658 => [0x57f3, false], # East Asian ideograph
        0x217659 => [0x5804, false], # East Asian ideograph
        0x21765a => [0x57cf, false], # East Asian ideograph
        0x22765b => [0x8030, false], # East Asian ideograph
        0x22765d => [0x8031, false], # East Asian ideograph
        0x21765f => [0x5847, false], # East Asian ideograph
        0x227660 => [0x8035, false], # East Asian ideograph
        0x225021 => [0x9e02, false], # East Asian ideograph
        0x4d2f5d => [0x8941, false], # East Asian ideograph (variant of 232F5D which maps to 8941)
        0x217667 => [0x581b, false], # East Asian ideograph
        0x227669 => [0x8039, false], # East Asian ideograph
        0x21766a => [0x5833, false], # East Asian ideograph
        0x22766b => [0x8041, false], # East Asian ideograph
        0x21766c => [0x581e, false], # East Asian ideograph
        0x21766d => [0x583f, false], # East Asian ideograph
        0x213f59 => [0x61fe, false], # East Asian ideograph
        0x227670 => [0x8043, false], # East Asian ideograph
        0x213e68 => [0x60b8, false], # East Asian ideograph
        0x217676 => [0x5828, false], # East Asian ideograph
        0x213e69 => [0x60da, false], # East Asian ideograph
        0x217678 => [0x582e, false], # East Asian ideograph
        0x6f4868 => [0xac12, false], # Korean hangul
        0x21767a => [0x581d, false], # East Asian ideograph
        0x22767b => [0x8052, false], # East Asian ideograph
        0x233e6a => [0x90bd, false], # East Asian ideograph
        0x22767e => [0x8062, false], # East Asian ideograph
        0x225b69 => [0x74aa, false], # East Asian ideograph
        0x213e6b => [0x610f, false], # East Asian ideograph
        0x2d4d34 => [0x76c7, false], # East Asian ideograph
        0x4b4046 => [0x629c, false], # East Asian ideograph
        0x213e6c => [0x611c, false], # East Asian ideograph
        0x29306f => [0x89cb, false], # East Asian ideograph
        0x213f5a => [0x61ff, false], # East Asian ideograph
        0x224832 => [0x6cd2, false], # East Asian ideograph
        0x234d62 => [0x979c, false], # East Asian ideograph
        0x6f773d => [0xc733, false], # Korean hangul
        0x2d4844 => [0x6fd5, false], # East Asian ideograph
        0x2d4461 => [0x6746, false], # East Asian ideograph
        0x213e6e => [0x611f, false], # East Asian ideograph
        0x6f4869 => [0xac13, false], # Korean hangul
        0x39417c => [0x62e0, false], # East Asian ideograph
        0x213e6f => [0x60f0, false], # East Asian ideograph
        0x225466 => [0x7215, false], # East Asian ideograph
        0x22723c => [0x7df2, false], # East Asian ideograph
        0x223e70 => [0x696a, false], # East Asian ideograph
        0x6f5976 => [0xce89, false], # Korean hangul
        0x213e71 => [0x60fa, false], # East Asian ideograph
        0x224b37 => [0x6e71, false], # East Asian ideograph
        0x213e72 => [0x611a, false], # East Asian ideograph
        0x234759 => [0x9404, false], # East Asian ideograph
        0x333d48 => [0x5f3a, false], # East Asian ideograph
        0x213e73 => [0x6115, false], # East Asian ideograph
        0x6f486a => [0xac14, false], # Korean hangul
        0x212a3d => [0xe8ea, false], # EACC component character
        0x235866 => [0x9c1c, false], # East Asian ideograph
        0x233e75 => [0x90c7, false], # East Asian ideograph
        0x456064 => [0x9963, false], # East Asian ideograph
        0x6f5977 => [0xce90, false], # Korean hangul
        0x6f5b3b => [0xd1a8, false], # Korean hangul
        0x2d3b7b => [0x5d08, false], # East Asian ideograph
        0x222921 => [0x5ef1, false], # East Asian ideograph
        0x213f5c => [0x6200, false], # East Asian ideograph
        0x697060 => [0x9790, false], # East Asian ideograph
        0x4b506c => [0x7cab, false], # East Asian ideograph
        0x215d32 => [0x91ac, false], # East Asian ideograph
        0x225347 => [0x71b2, false], # East Asian ideograph
        0x213e78 => [0x610e, false], # East Asian ideograph
        0x2d5e43 => [0x92f3, false], # East Asian ideograph
        0x6f486b => [0xac15, false], # Korean hangul
        0x213e79 => [0x6100, false], # East Asian ideograph
        0x23364e => [0x8cb0, false], # East Asian ideograph
        0x213e7a => [0x6101, false], # East Asian ideograph
        0x4d2925 => [0x8770, false], # East Asian ideograph
        0x6f5344 => [0xc19c, false], # Korean hangul
        0x6f5978 => [0xce91, false], # Korean hangul
        0x213e7b => [0x60f6, false], # East Asian ideograph
        0x6f5535 => [0xc575, false], # Korean hangul
        0x4b3870 => [0x58cc, false], # East Asian ideograph
        0x232927 => [0x867c, false], # East Asian ideograph
        0x223e7d => [0x6980, false], # East Asian ideograph
        0x28736d => [0x624d, false], # East Asian ideograph
        0x275e62 => [0x9615, false], # East Asian ideograph
        0x223e7e => [0x6933, false], # East Asian ideograph
        0x225469 => [0x7213, false], # East Asian ideograph
        0x6f4935 => [0xac94, false], # Korean hangul
        0x6f4e72 => [0xb7a8, false], # Korean hangul
        0x2d4d38 => [0x76d7, false], # East Asian ideograph
        0x6f5536 => [0xc57c, false], # Korean hangul
        0x4b3871 => [0x57bb, false], # East Asian ideograph
        0x4b5647 => [0x51e6, false], # East Asian ideograph
        0x6f486d => [0xac17, false], # Korean hangul
        0x2d5434 => [0x64e7, false], # East Asian ideograph
        0x234e5c => [0x97de, false], # East Asian ideograph
        0x225b6a => [0x7490, false], # East Asian ideograph
        0x23292f => [0x86a8, false], # East Asian ideograph
        0x6f597a => [0xce98, false], # Korean hangul
        0x4b5221 => [0x7d9a, false], # East Asian ideograph
        0x217721 => [0x5848, false], # East Asian ideograph
        0x6f7722 => [0xad7b, false], # Korean hangul
        0x217723 => [0x5818, false], # East Asian ideograph
        0x6f7724 => [0xad89, false], # Korean hangul
        0x6f7725 => [0xad9d, false], # Korean hangul
        0x217726 => [0x57f5, false], # East Asian ideograph
        0x4b5d36 => [0x91c6, false], # East Asian ideograph
        0x227728 => [0x8063, false], # East Asian ideograph
        0x21315b => [0x4fbf, false], # East Asian ideograph
        0x6f772a => [0xae0f, false], # Korean hangul
        0x21772b => [0x5820, false], # East Asian ideograph
        0x6f772c => [0xae14, false], # Korean hangul
        0x6f486e => [0xac19, false], # Korean hangul
        0x6f772e => [0xaeed, false], # Korean hangul
        0x6f772f => [0xaf09, false], # Korean hangul
        0x217730 => [0x584e, false], # East Asian ideograph
        0x6f7731 => [0xafbf, false], # Korean hangul
        0x227732 => [0x806c, false], # East Asian ideograph
        0x217733 => [0x585d, false], # East Asian ideograph
        0x6f4926 => [0xac78, false], # Korean hangul
        0x217735 => [0x5859, false], # East Asian ideograph
        0x6f7736 => [0xc552, false], # Korean hangul
        0x217737 => [0x584b, false], # East Asian ideograph
        0x6f7738 => [0xc5b1, false], # Korean hangul
        0x227739 => [0x8075, false], # East Asian ideograph
        0x6f773a => [0xc61d, false], # Korean hangul
        0x2d3876 => [0x58f7, false], # East Asian ideograph
        0x6f773c => [0xe8cb, false], # Korean hangul
        0x21773d => [0x5865, false], # East Asian ideograph
        0x22773e => [0x807b, false], # East Asian ideograph
        0x22773f => [0x8079, false], # East Asian ideograph
        0x217740 => [0x586c, false], # East Asian ideograph
        0x213f60 => [0x620d, false], # East Asian ideograph
        0x217742 => [0x5852, false], # East Asian ideograph
        0x33475e => [0x6fb9, false], # East Asian ideograph
        0x217745 => [0x5864, false], # East Asian ideograph
        0x227747 => [0x808a, false], # East Asian ideograph
        0x217748 => [0x584f, false], # East Asian ideograph
        0x227749 => [0x808e, false], # East Asian ideograph
        0x213533 => [0x53fc, false], # East Asian ideograph
        0x6f486f => [0xac1a, false], # Korean hangul
        0x21774d => [0x584d, false], # East Asian ideograph
        0x22774e => [0x809f, false], # East Asian ideograph
        0x6f5730 => [0xc7c8, false], # Korean hangul
        0x225029 => [0x7054, false], # East Asian ideograph
        0x232939 => [0x8698, false], # East Asian ideograph
        0x217758 => [0x5892, false], # East Asian ideograph
        0x6f597c => [0xcea1, false], # Korean hangul
        0x21775a => [0x588e, false], # East Asian ideograph
        0x22775c => [0x670a, false], # East Asian ideograph
        0x70775d => [0x9b0f, false], # East Asian ideograph
        0x282632 => [0x5d58, false], # East Asian ideograph
        0x21775f => [0x5840, false], # East Asian ideograph
        0x227760 => [0x80a7, false], # East Asian ideograph
        0x227761 => [0x80b0, false], # East Asian ideograph
        0x33475f => [0x60bd, false], # East Asian ideograph
        0x21576c => [0x88fd, false], # East Asian ideograph
        0x217765 => [0x5890, false], # East Asian ideograph
        0x217768 => [0x5898, false], # East Asian ideograph
        0x227769 => [0x80b5, false], # East Asian ideograph
        0x22776a => [0x80a6, false], # East Asian ideograph
        0x21776b => [0x587d, false], # East Asian ideograph
        0x6f5c36 => [0xd3b8, false], # Korean hangul
        0x21776f => [0x587f, false], # East Asian ideograph
        0x217770 => [0x5881, false], # East Asian ideograph
        0x707771 => [0x9ee2, false], # East Asian ideograph (Version J extension)
        0x227773 => [0x80e0, false], # East Asian ideograph
        0x21693e => [0x5135, false], # East Asian ideograph
        0x235b26 => [0x9d5a, false], # East Asian ideograph
        0x6f597d => [0xcea3, false], # Korean hangul
        0x213d62 => [0x5f8b, false], # East Asian ideograph
        0x22777b => [0x80df, false], # East Asian ideograph
        0x22777d => [0x80c2, false], # East Asian ideograph
        0x21777e => [0x58a1, false], # East Asian ideograph
        0x226940 => [0x7a5c, false], # East Asian ideograph
        0x4b7421 => [0xf9a9, false], # East Asian ideograph
        0x4c433f => [0x6a65, false], # East Asian ideograph
        0x6f4d21 => [0xb304, false], # Korean hangul
        0x45606b => [0x98f0, false], # East Asian ideograph
        0x27482d => [0x6c64, false], # East Asian ideograph
        0x293b6b => [0x8f8b, false], # East Asian ideograph
        0x276944 => [0x50a9, false], # East Asian ideograph
        0x213f63 => [0x6212, false], # East Asian ideograph
        0x2d5e4a => [0x945a, false], # East Asian ideograph
        0x6f4a30 => [0xae00, false], # Korean hangul
        0x275e57 => [0x95f8, false], # East Asian ideograph
        0x6f5321 => [0xc11c, false], # Korean hangul
        0x276948 => [0x50a5, false], # East Asian ideograph
        0x6f553c => [0xc58d, false], # Korean hangul
        0x213f64 => [0x6211, false], # East Asian ideograph
        0x215d3a => [0x91cd, false], # East Asian ideograph
        0x234762 => [0x9423, false], # East Asian ideograph
        0x23294b => [0x86bf, false], # East Asian ideograph
        0x6f4956 => [0xacf6, false], # Korean hangul
        0x6f4b41 => [0xb057, false], # Korean hangul
        0x292c5d => [0x867f, false], # East Asian ideograph
        0x4b3435 => [0x52b4, false], # East Asian ideograph
        0x6f4e27 => [0xb54c, false], # Korean hangul
        0x222951 => [0x5f33, false], # East Asian ideograph
        0x223258 => [0x63e6, false], # East Asian ideograph
        0x235870 => [0x9c2e, false], # East Asian ideograph
        0x227247 => [0x7df1, false], # East Asian ideograph
        0x6f5b3d => [0xd1b1, false], # Korean hangul
        0x6f553e => [0xc590, false], # Korean hangul
        0x213f66 => [0x6215, false], # East Asian ideograph
        0x2d3539 => [0x52fe, false], # East Asian ideograph
        0x22613b => [0x76e6, false], # East Asian ideograph
        0x4b3436 => [0x52f2, false], # East Asian ideograph
        0x6f4e6c => [0xb791, false], # Korean hangul
        0x23517a => [0x9958, false], # East Asian ideograph
        0x214c30 => [0x755a, false], # East Asian ideograph
        0x226957 => [0x7a6e, false], # East Asian ideograph
        0x222958 => [0x5f38, false], # East Asian ideograph
        0x2d424f => [0x555f, false], # East Asian ideograph
        0x215d3d => [0x91d0, false], # East Asian ideograph
        0x22613c => [0x76e9, false], # East Asian ideograph
        0x334342 => [0x7156, false], # East Asian ideograph
        0x513d67 => [0x8ff3, false], # East Asian ideograph
        0x217824 => [0x58b1, false], # East Asian ideograph
        0x227827 => [0x80d9, false], # East Asian ideograph
        0x4b4544 => [0x69d8, false], # East Asian ideograph
        0x4c695c => [0x7a06, false], # East Asian ideograph
        0x22782a => [0x80dd, false], # East Asian ideograph
        0x21782b => [0x58ad, false], # East Asian ideograph
        0x22782d => [0x80cf, false], # East Asian ideograph
        0x21782e => [0x58a0, false], # East Asian ideograph
        0x22782f => [0x80cd, false], # East Asian ideograph
        0x227830 => [0x80d7, false], # East Asian ideograph
        0x213f68 => [0x621a, false], # East Asian ideograph
        0x217832 => [0x58a6, false], # East Asian ideograph
        0x227833 => [0x80f2, false], # East Asian ideograph
        0x227834 => [0x80fa, false], # East Asian ideograph
        0x227838 => [0x80fe, false], # East Asian ideograph
        0x21783a => [0x58c8, false], # East Asian ideograph
        0x2d3c36 => [0x5de3, false], # East Asian ideograph
        0x22783c => [0x8103, false], # East Asian ideograph
        0x275762 => [0x8865, false], # East Asian ideograph
        0x227840 => [0x80f9, false], # East Asian ideograph
        0x217841 => [0x58bc, false], # East Asian ideograph
        0x227842 => [0x80d4, false], # East Asian ideograph
        0x33365a => [0x5405, false], # East Asian ideograph
        0x4b4545 => [0x6982, false], # East Asian ideograph
        0x217849 => [0x58bf, false], # East Asian ideograph
        0x22784b => [0x8118, false], # East Asian ideograph
        0x21784c => [0x58ba, false], # East Asian ideograph
        0x222962 => [0x5f4d, false], # East Asian ideograph
        0x6f5541 => [0xc597, false], # Korean hangul
        0x227850 => [0x8130, false], # East Asian ideograph
        0x232963 => [0x86b4, false], # East Asian ideograph
        0x227854 => [0x8124, false], # East Asian ideograph
        0x227855 => [0x811b, false], # East Asian ideograph
        0x217856 => [0x58ce, false], # East Asian ideograph
        0x2d5e50 => [0x9587, false], # East Asian ideograph
        0x6f4878 => [0xac2f, false], # Korean hangul
        0x21785a => [0x58e0, false], # East Asian ideograph
        0x21785e => [0x58da, false], # East Asian ideograph
        0x227860 => [0x812a, false], # East Asian ideograph
        0x227861 => [0x811e, false], # East Asian ideograph
        0x294362 => [0x94ea, false], # East Asian ideograph
        0x227864 => [0x8121, false], # East Asian ideograph
        0x212321 => [0x3000, false], # Ideographic space per ANSI Z39.64
        0x227866 => [0x8117, false], # East Asian ideograph
        0x227869 => [0x813a, false], # East Asian ideograph
        0x22786a => [0x815a, false], # East Asian ideograph
        0x21786c => [0x58fc, false], # East Asian ideograph
        0x22786d => [0x8148, false], # East Asian ideograph
        0x28786e => [0x80e8, false], # East Asian ideograph
        0x4b387d => [0x591b, false], # East Asian ideograph
        0x217870 => [0x5902, false], # East Asian ideograph
        0x213b27 => [0x5bcc, false], # East Asian ideograph
        0x217873 => [0x5906, false], # East Asian ideograph
        0x217874 => [0x6535, false], # East Asian ideograph
        0x227877 => [0x814c, false], # East Asian ideograph
        0x21787a => [0x5910, false], # East Asian ideograph
        0x21787c => [0x8641, false], # East Asian ideograph
        0x22787d => [0x8141, false], # East Asian ideograph
        0x22325d => [0x63f6, false], # East Asian ideograph
        0x214c34 => [0x7570, false], # East Asian ideograph
        0x343a5b => [0x572c, false], # East Asian ideograph
        0x2e735d => [0x7d56, false], # East Asian ideograph
        0x276871 => [0x4faa, false], # East Asian ideograph
        0x6f5543 => [0xc59c, false], # Korean hangul
        0x696464 => [0x7c90, false], # East Asian ideograph
        0x213b28 => [0x5bd2, false], # East Asian ideograph
        0x234326 => [0x924c, false], # East Asian ideograph
        0x275765 => [0x88c5, false], # East Asian ideograph
        0x23296f => [0x86e9, false], # East Asian ideograph
        0x22325e => [0x63f2, false], # East Asian ideograph
        0x214c35 => [0x7565, false], # East Asian ideograph
        0x6f557c => [0xc63a, false], # Korean hangul
        0x235433 => [0x9a64, false], # East Asian ideograph
        0x222971 => [0x5f61, false], # East Asian ideograph
        0x6f5544 => [0xc5b4, false], # Korean hangul
        0x2d5e24 => [0x7145, false], # East Asian ideograph
        0x23476a => [0x9407, false], # East Asian ideograph
        0x4b403d => [0x62dd, false], # East Asian ideograph
        0x6f487b => [0xac38, false], # Korean hangul
        0x2d4d65 => [0x53e1, false], # East Asian ideograph
        0x232974 => [0x86d5, false], # East Asian ideograph
        0x22325f => [0x63f8, false], # East Asian ideograph
        0x23365e => [0x8cd8, false], # East Asian ideograph
        0x235434 => [0x9a66, false], # East Asian ideograph
        0x275421 => [0x80ea, false], # East Asian ideograph
        0x275e37 => [0x9535, false], # East Asian ideograph
        0x215422 => [0x81df, false], # East Asian ideograph
        0x6f5862 => [0xcb10, false], # Korean hangul
        0x6f487c => [0xac39, false], # Korean hangul
        0x6f4a32 => [0xae08, false], # Korean hangul
        0x6f5423 => [0xc2dc, false], # Korean hangul
        0x27623f => [0x9e43, false], # East Asian ideograph
        0x275e59 => [0x95fa, false], # East Asian ideograph
        0x215424 => [0x81e5, false], # East Asian ideograph
        0x23365f => [0x8cd5, false], # East Asian ideograph
        0x456076 => [0x9980, false], # East Asian ideograph
        0x215425 => [0x81e8, false], # East Asian ideograph
        0x6f5b48 => [0xd23d, false], # Korean hangul
        0x275142 => [0x7efd, false], # East Asian ideograph
        0x225426 => [0x71d4, false], # East Asian ideograph
        0x235427 => [0x9a4a, false], # East Asian ideograph
        0x6f487d => [0xac40, false], # Korean hangul
        0x4b5428 => [0x81ed, false], # East Asian ideograph (variant of 215428 which maps to 81ED)
        0x235879 => [0x9c24, false], # East Asian ideograph
        0x6f5c6a => [0xd56b, false], # Korean hangul
        0x23542a => [0x9a58, false], # East Asian ideograph
        0x6f546d => [0xc4f8, false], # Korean hangul
        0x6f5547 => [0xc5b8, false], # Korean hangul
        0x27542b => [0x53f0, false], # East Asian ideograph
        0x333d4c => [0x7030, false], # East Asian ideograph
        0x23542c => [0x9a56, false], # East Asian ideograph
        0x6f542d => [0xc2f6, false], # Korean hangul
        0x293d4e => [0x8ff8, false], # East Asian ideograph
        0x28352a => [0x6448, false], # East Asian ideograph
        0x47577a => [0x9bd6, false], # East Asian ideograph (variant of 23577A which maps to 9BD6)
        0x6f5b3f => [0xd1b5, false], # Korean hangul
        0x6f5430 => [0xc2f8, false], # Korean hangul
        0x227925 => [0x814d, false], # East Asian ideograph
        0x6f5431 => [0xc2f9, false], # Korean hangul
        0x217928 => [0x592c, false], # East Asian ideograph
        0x21792b => [0x592f, false], # East Asian ideograph
        0x275432 => [0x4e0e, false], # East Asian ideograph
        0x22792e => [0x6720, false], # East Asian ideograph
        0x21507d => [0x7d14, false], # East Asian ideograph
        0x217930 => [0x593c, false], # East Asian ideograph
        0x395f68 => [0x8987, false], # East Asian ideograph
        0x227932 => [0x8160, false], # East Asian ideograph
        0x215433 => [0x8208, false], # East Asian ideograph
        0x225039 => [0x7074, false], # East Asian ideograph
        0x217938 => [0x594d, false], # East Asian ideograph
        0x215434 => [0x8209, false], # East Asian ideograph
        0x22793b => [0x8169, false], # East Asian ideograph
        0x22793c => [0x817c, false], # East Asian ideograph
        0x275435 => [0x65e7, false], # East Asian ideograph
        0x294e5c => [0x97eb, false], # East Asian ideograph
        0x227941 => [0x8161, false], # East Asian ideograph
        0x6f5a65 => [0xd081, false], # Korean hangul
        0x217943 => [0x5953, false], # East Asian ideograph
        0x225436 => [0x71e8, false], # East Asian ideograph
        0x227946 => [0x8176, false], # East Asian ideograph
        0x227947 => [0x8174, false], # East Asian ideograph
        0x227948 => [0x8167, false], # East Asian ideograph
        0x334633 => [0x6b8b, false], # East Asian ideograph (variant of 274633 which maps to 6B8B)
        0x22794b => [0x816f, false], # East Asian ideograph
        0x22794d => [0x8182, false], # East Asian ideograph
        0x4c794e => [0x80b7, false], # East Asian ideograph
        0x21794f => [0x5961, false], # East Asian ideograph
        0x227951 => [0x818b, false], # East Asian ideograph
        0x227952 => [0x8186, false], # East Asian ideograph
        0x217954 => [0x596c, false], # East Asian ideograph
        0x217955 => [0x596d, false], # East Asian ideograph
        0x274830 => [0x6d4b, false], # East Asian ideograph
        0x4b6324 => [0x9f62, false], # East Asian ideograph
        0x227959 => [0x8183, false], # East Asian ideograph
        0x21543a => [0x8214, false], # East Asian ideograph
        0x334770 => [0x5a6c, false], # East Asian ideograph
        0x69543b => [0x57b0, false], # East Asian ideograph
        0x217965 => [0x597c, false], # East Asian ideograph
        0x6f4a33 => [0xae09, false], # Korean hangul
        0x217969 => [0x59a7, false], # East Asian ideograph
        0x22796a => [0x819f, false], # East Asian ideograph
        0x22796b => [0x81a3, false], # East Asian ideograph
        0x287941 => [0x8136, false], # East Asian ideograph
        0x21796f => [0x599a, false], # East Asian ideograph
        0x227970 => [0x8198, false], # East Asian ideograph
        0x22503b => [0x707a, false], # East Asian ideograph
        0x335e3d => [0x9244, false], # East Asian ideograph
        0x227975 => [0x8195, false], # East Asian ideograph
        0x227977 => [0x8197, false], # East Asian ideograph
        0x22543f => [0x71e1, false], # East Asian ideograph
        0x22797c => [0x81aa, false], # East Asian ideograph
        0x224372 => [0x6b1e, false], # East Asian ideograph
        0x22797e => [0x6725, false], # East Asian ideograph
        0x213b30 => [0x5bde, false], # East Asian ideograph
        0x2d5440 => [0x6841, false], # East Asian ideograph
        0x6f4862 => [0xac04, false], # Korean hangul
        0x215441 => [0x822b, false], # East Asian ideograph
        0x275068 => [0x7caa, false], # East Asian ideograph
        0x695442 => [0x57d6, false], # East Asian ideograph
        0x227255 => [0x7e12, false], # East Asian ideograph
        0x235443 => [0x9ab1, false], # East Asian ideograph
        0x294e79 => [0x9878, false], # East Asian ideograph
        0x6f5444 => [0xc345, false], # Korean hangul
        0x213b31 => [0x5be6, false], # East Asian ideograph
        0x235445 => [0x9ab3, false], # East Asian ideograph
        0x33432f => [0x664b, false], # East Asian ideograph
        0x225651 => [0x72c6, false], # East Asian ideograph
        0x2d5446 => [0x8229, false], # East Asian ideograph
        0x216e57 => [0x53fb, false], # East Asian ideograph
        0x214c3e => [0x758f, false], # East Asian ideograph
        0x276329 => [0x9f8c, false], # East Asian ideograph
        0x6f554d => [0xc5c4, false], # Korean hangul
        0x235449 => [0x9ab6, false], # East Asian ideograph
        0x227427 => [0x7f46, false], # East Asian ideograph
        0x224a62 => [0x6e4b, false], # East Asian ideograph
        0x27544a => [0x8231, false], # East Asian ideograph
        0x294161 => [0x9487, false], # East Asian ideograph
        0x27544b => [0x8230, false], # East Asian ideograph
        0x283955 => [0x6619, false], # East Asian ideograph
        0x23544c => [0x9abb, false], # East Asian ideograph
        0x233667 => [0x8ce8, false], # East Asian ideograph
        0x4d2f7a => [0x891d, false], # East Asian ideograph
        0x6f5345 => [0xc19d, false], # Korean hangul
        0x6f544d => [0xc37d, false], # Korean hangul
        0x235871 => [0x9c28, false], # East Asian ideograph
        0x6f554e => [0xc5c5, false], # Korean hangul
        0x27544e => [0x8270, false], # East Asian ideograph
        0x234774 => [0x943f, false], # East Asian ideograph
        0x22544f => [0x71fc, false], # East Asian ideograph
        0x235450 => [0x9aba, false], # East Asian ideograph
        0x695451 => [0x58b9, false], # East Asian ideograph
        0x233668 => [0x8ce9, false], # East Asian ideograph
        0x4b4553 => [0x6955, false], # East Asian ideograph
        0x6f4e77 => [0xb7b4, false], # Korean hangul
        0x274831 => [0x6da1, false], # East Asian ideograph
        0x233225 => [0x8a22, false], # East Asian ideograph
        0x6f554f => [0xc5c6, false], # Korean hangul
        0x6f5453 => [0xc3dc, false], # Korean hangul
        0x235454 => [0x9abd, false], # East Asian ideograph
        0x2e6c26 => [0x7be0, false], # East Asian ideograph
        0x6f4a34 => [0xae0b, false], # Korean hangul
        0x6f5455 => [0xc3e0, false], # Korean hangul
        0x225456 => [0x71f9, false], # East Asian ideograph
        0x225040 => [0x7093, false], # East Asian ideograph
        0x23543f => [0x9aad, false], # East Asian ideograph
        0x235457 => [0x9ac1, false], # East Asian ideograph
        0x6f5550 => [0xc5c7, false], # Korean hangul
        0x275458 => [0x5df4, false], # East Asian ideograph (duplicate simplified)
        0x274222 => [0x62c5, false], # East Asian ideograph
        0x4b3b22 => [0x51a6, false], # East Asian ideograph
        0x235459 => [0x9ac0, false], # East Asian ideograph
        0x22572c => [0x72fb, false], # East Asian ideograph
        0x23545a => [0x9ac2, false], # East Asian ideograph
        0x217a21 => [0x5990, false], # East Asian ideograph
        0x22545b => [0x720e, false], # East Asian ideograph
        0x217a24 => [0x59c5, false], # East Asian ideograph
        0x217a25 => [0x59b5, false], # East Asian ideograph
        0x217a28 => [0x59cf, false], # East Asian ideograph
        0x21545c => [0x82bb, false], # East Asian ideograph
        0x217a2a => [0x59ba, false], # East Asian ideograph
        0x3f377b => [0x784e, false], # East Asian ideograph (Version J extension)
        0x217a2c => [0x59b8, false], # East Asian ideograph
        0x6f546f => [0xc501, false], # Korean hangul
        0x227a2e => [0x81b0, false], # East Asian ideograph
        0x227a2f => [0x81b4, false], # East Asian ideograph
        0x215d4f => [0x9237, false], # East Asian ideograph
        0x227a33 => [0x81b7, false], # East Asian ideograph
        0x217a35 => [0x59b2, false], # East Asian ideograph
        0x227a37 => [0x81bb, false], # East Asian ideograph
        0x227a38 => [0x81c1, false], # East Asian ideograph
        0x227a39 => [0x81cc, false], # East Asian ideograph
        0x217a3a => [0x59b7, false], # East Asian ideograph
        0x227a3b => [0x81c4, false], # East Asian ideograph
        0x217a3e => [0x59c1, false], # East Asian ideograph
        0x227a40 => [0x81d1, false], # East Asian ideograph
        0x227a41 => [0x81ce, false], # East Asian ideograph
        0x217a43 => [0x59f9, false], # East Asian ideograph
        0x217a44 => [0x59f8, false], # East Asian ideograph
        0x212a43 => [0xe8f0, false], # EACC component character
        0x225461 => [0x7207, false], # East Asian ideograph
        0x227a4b => [0x81db, false], # East Asian ideograph
        0x6f5552 => [0xc5c9, false], # Korean hangul
        0x6f5462 => [0xc468, false], # Korean hangul
        0x227a4f => [0x81dd, false], # East Asian ideograph
        0x217a50 => [0x59f1, false], # East Asian ideograph
        0x217a51 => [0x5a00, false], # East Asian ideograph
        0x217a52 => [0x59de, false], # East Asian ideograph
        0x227a53 => [0x81de, false], # East Asian ideograph
        0x227a56 => [0x81e0, false], # East Asian ideograph
        0x227a57 => [0x81e2, false], # East Asian ideograph
        0x6f5464 => [0xc474, false], # Korean hangul
        0x227a5b => [0x81e7, false], # East Asian ideograph
        0x217a5d => [0x59f6, false], # East Asian ideograph
        0x217a5e => [0x59dd, false], # East Asian ideograph
        0x217a5f => [0x59fa, false], # East Asian ideograph
        0x227a60 => [0x81ef, false], # East Asian ideograph
        0x217a61 => [0x59e4, false], # East Asian ideograph
        0x227a65 => [0x81f2, false], # East Asian ideograph
        0x227a68 => [0x81f6, false], # East Asian ideograph
        0x6f5553 => [0xc5ca, false], # Korean hangul
        0x6f5467 => [0xc494, false], # Korean hangul
        0x274225 => [0x6324, false], # East Asian ideograph
        0x217a6e => [0x5a2a, false], # East Asian ideograph
        0x213b38 => [0x5bf5, false], # East Asian ideograph
        0x227a70 => [0x8201, false], # East Asian ideograph
        0x2d5468 => [0x6959, false], # East Asian ideograph
        0x227a72 => [0x8201, false], # East Asian ideograph (not in Unicode)
        0x227a74 => [0x8203, false], # East Asian ideograph
        0x227a75 => [0x8204, false], # East Asian ideograph
        0x2d3c49 => [0x83f7, false], # East Asian ideograph
        0x227a77 => [0x820b, false], # East Asian ideograph
        0x217a78 => [0x5a09, false], # East Asian ideograph
        0x23546a => [0x9ad1, false], # East Asian ideograph
        0x217a7e => [0x5a12, false], # East Asian ideograph
        0x6f4e78 => [0xb7b5, false], # Korean hangul
        0x6f546b => [0xc4f1, false], # Korean hangul
        0x6f5554 => [0xc5cc, false], # Korean hangul
        0x6f546c => [0xc4f4, false], # Korean hangul
        0x274226 => [0x62e7, false], # East Asian ideograph
        0x213b39 => [0x5bf6, false], # East Asian ideograph
        0x21546d => [0x82d3, false], # East Asian ideograph
        0x234337 => [0x927c, false], # East Asian ideograph
        0x22546e => [0x7218, false], # East Asian ideograph
        0x28395c => [0x6654, false], # East Asian ideograph
        0x2d546f => [0x83c0, false], # East Asian ideograph
        0x22725e => [0x7e09, false], # East Asian ideograph
        0x4b4559 => [0x9792, false], # East Asian ideograph
        0x6f5470 => [0xc50c, false], # Korean hangul
        0x4b5227 => [0x6b20, false], # East Asian ideograph
        0x6f5555 => [0xc5ce, false], # Korean hangul
        0x225471 => [0x720b, false], # East Asian ideograph
        0x33477b => [0x904a, false], # East Asian ideograph
        0x235472 => [0x9adc, false], # East Asian ideograph
        0x4b5223 => [0x7e4a, false], # East Asian ideograph
        0x275777 => [0x4eb5, false], # East Asian ideograph
        0x215474 => [0x834a, false], # East Asian ideograph
        0x22725f => [0x7e1f, false], # East Asian ideograph
        0x23366f => [0x8ceb, false], # East Asian ideograph
        0x335445 => [0x67c1, false], # East Asian ideograph
        0x6f5475 => [0xc530, false], # Korean hangul
        0x6f5556 => [0xc5d0, false], # Korean hangul
        0x235476 => [0x9ae0, false], # East Asian ideograph
        0x274228 => [0x62df, false], # East Asian ideograph
        0x23477c => [0x943d, false], # East Asian ideograph
        0x215477 => [0x8350, false], # East Asian ideograph
        0x27593f => [0x8bc1, false], # East Asian ideograph
        0x233f22 => [0x90dd, false], # East Asian ideograph
        0x6f5478 => [0xc53b, false], # Korean hangul
        0x293f23 => [0x90cf, false], # East Asian ideograph
        0x6f5827 => [0xc999, false], # Korean hangul
        0x225479 => [0x721a, false], # East Asian ideograph
        0x233670 => [0x8cda, false], # East Asian ideograph
        0x212a44 => [0xe8f1, false], # EACC component character
        0x335446 => [0x8221, false], # East Asian ideograph
        0x22437e => [0x6b2c, false], # East Asian ideograph
        0x4b5154 => [0x7df4, false], # East Asian ideograph
        0x6f547c => [0xc544, false], # Korean hangul
        0x233f27 => [0x90d8, false], # East Asian ideograph
        0x6f5b6d => [0xd30d, false], # Korean hangul
        0x22547d => [0x721f, false], # East Asian ideograph
        0x273f28 => [0x6001, false], # East Asian ideograph
        0x223272 => [0x63eb, false], # East Asian ideograph
        0x6f4f53 => [0xb974, false], # Korean hangul
        0x213f29 => [0x613e, false], # East Asian ideograph
        0x225048 => [0x7096, false], # East Asian ideograph (not in Unicode)
        0x6f5624 => [0xc650, false], # Korean hangul
        0x213f2a => [0x6127, false], # East Asian ideograph
        0x27422a => [0x6269, false], # East Asian ideograph
        0x213c2b => [0x5d87, false], # East Asian ideograph
        0x213f2c => [0x6147, false], # East Asian ideograph
        0x275f37 => [0x9645, false], # East Asian ideograph
        0x223f2d => [0x6985, false], # East Asian ideograph
        0x273f2e => [0x5e86, false], # East Asian ideograph
        0x6f4e79 => [0xb7c9, false], # Korean hangul
        0x274833 => [0x6d51, false], # East Asian ideograph
        0x213f2f => [0x6167, false], # East Asian ideograph
        0x6f5559 => [0xc5d8, false], # Korean hangul
        0x27422b => [0x63b7, false], # East Asian ideograph
        0x2e624f => [0x772d, false], # East Asian ideograph
        0x227b27 => [0x821d, false], # East Asian ideograph
        0x227b29 => [0x8220, false], # East Asian ideograph
        0x6f4a36 => [0xae30, false], # Korean hangul
        0x217b2c => [0x5a60, false], # East Asian ideograph
        0x223f32 => [0x693d, false], # East Asian ideograph
        0x227b2e => [0x822d, false], # East Asian ideograph
        0x227b2f => [0x822f, false], # East Asian ideograph
        0x6f5477 => [0xc539, false], # Korean hangul
        0x217b31 => [0x5a67, false], # East Asian ideograph
        0x227b32 => [0x8238, false], # East Asian ideograph
        0x273f33 => [0x5fe7, false], # East Asian ideograph
        0x227b34 => [0x823a, false], # East Asian ideograph
        0x227b35 => [0x8233, false], # East Asian ideograph
        0x227b36 => [0x8234, false], # East Asian ideograph
        0x213f34 => [0x617c, false], # East Asian ideograph
        0x227b3a => [0x8232, false], # East Asian ideograph
        0x217b3b => [0x5a5e, false], # East Asian ideograph
        0x217b3c => [0x5a6d, false], # East Asian ideograph
        0x217b3d => [0x5a35, false], # East Asian ideograph
        0x217b3e => [0x5a55, false], # East Asian ideograph
        0x27422c => [0x64b5, false], # East Asian ideograph
        0x215d58 => [0x9234, false], # East Asian ideograph (variant of 4B5D58 which maps to 9234)
        0x217b41 => [0x5a2c, false], # East Asian ideograph
        0x227b42 => [0x8248, false], # East Asian ideograph
        0x227b43 => [0x8249, false], # East Asian ideograph
        0x227b45 => [0x8244, false], # East Asian ideograph
        0x227b47 => [0x8240, false], # East Asian ideograph
        0x227b48 => [0x8241, false], # East Asian ideograph
        0x217b49 => [0x5a65, false], # East Asian ideograph
        0x227b4a => [0x8245, false], # East Asian ideograph
        0x227b4b => [0x824b, false], # East Asian ideograph
        0x334e37 => [0x784e, false], # East Asian ideograph
        0x227b50 => [0x824f, false], # East Asian ideograph
        0x213f38 => [0x6158, false], # East Asian ideograph
        0x217b52 => [0x5a64, false], # East Asian ideograph
        0x227b53 => [0x824e, false], # East Asian ideograph
        0x227b56 => [0x8256, false], # East Asian ideograph
        0x227b57 => [0x8257, false], # East Asian ideograph
        0x2d6b33 => [0x5231, false], # East Asian ideograph (not in Unicode)
        0x6f555b => [0xc5e1, false], # Korean hangul
        0x223f3a => [0x6934, false], # East Asian ideograph
        0x227b5e => [0x825a, false], # East Asian ideograph
        0x227b62 => [0x825f, false], # East Asian ideograph
        0x223f3b => [0x6969, false], # East Asian ideograph
        0x217b65 => [0x5a8a, false], # East Asian ideograph
        0x227b67 => [0x8262, false], # East Asian ideograph
        0x217b69 => [0x5acf, false], # East Asian ideograph
        0x217b6a => [0x5a7a, false], # East Asian ideograph
        0x227b6b => [0x8268, false], # East Asian ideograph
        0x227b6f => [0x826d, false], # East Asian ideograph
        0x217b71 => [0x5a9f, false], # East Asian ideograph
        0x273f3e => [0x5baa, false], # East Asian ideograph
        0x227b77 => [0x8278, false], # East Asian ideograph
        0x213f3f => [0x6191, false], # East Asian ideograph
        0x227b7d => [0x827f, false], # East Asian ideograph
        0x23433f => [0x928d, false], # East Asian ideograph
        0x213f41 => [0x61ab, false], # East Asian ideograph
        0x23486c => [0x946f, false], # East Asian ideograph
        0x295f7c => [0x9f80, false], # East Asian ideograph
        0x6f4f54 => [0xb975, false], # Korean hangul
        0x213f42 => [0x61a4, false], # East Asian ideograph
        0x453d53 => [0x5f66, false], # East Asian ideograph
        0x4b4561 => [0x691c, false], # East Asian ideograph
        0x4b5061 => [0x7cbe, false], # East Asian ideograph
        0x2d4d5f => [0x7741, false], # East Asian ideograph
        0x6f555d => [0xc5e5, false], # Korean hangul
        0x27422f => [0x64de, false], # East Asian ideograph
        0x6f5a69 => [0xd0ac, false], # Korean hangul
        0x213b42 => [0x5c0b, false], # East Asian ideograph
        0x294666 => [0x933e, false], # East Asian ideograph
        0x223f45 => [0x69a0, false], # East Asian ideograph
        0x234340 => [0x92ee, false], # East Asian ideograph
        0x4b522b => [0x7f36, false], # East Asian ideograph
        0x6f5c50 => [0xd48d, false], # Korean hangul
        0x223f46 => [0x69b1, false], # East Asian ideograph
        0x23553c => [0x9b08, false], # East Asian ideograph
        0x233f47 => [0x90fe, false], # East Asian ideograph
        0x295031 => [0x98a7, false], # East Asian ideograph
        0x213f48 => [0x61b6, false], # East Asian ideograph
        0x6f555e => [0xc5ec, false], # Korean hangul
        0x213f49 => [0x61cd, false], # East Asian ideograph
        0x213b52 => [0x5c3f, false], # East Asian ideograph
        0x233f4a => [0x90ff, false], # East Asian ideograph
        0x6f4a37 => [0xae31, false], # Korean hangul
        0x273f4b => [0x601c, false], # East Asian ideograph
        0x213f4c => [0x61be, false], # East Asian ideograph
        0x516a26 => [0x51b4, false], # East Asian ideograph
        0x4d4d61 => [0x7ef1, false], # East Asian ideograph
        0x6f5b49 => [0xd23f, false], # Korean hangul
        0x275143 => [0x7efe, false], # East Asian ideograph
        0x274231 => [0x62e2, false], # East Asian ideograph
        0x295940 => [0x9cbc, false], # East Asian ideograph
        0x213b44 => [0x5c0e, false], # East Asian ideograph
        0x234a21 => [0x9627, false], # East Asian ideograph
        0x223f50 => [0x69ce, false], # East Asian ideograph
        0x22327a => [0x63dc, false], # East Asian ideograph
        0x227269 => [0x7e10, false], # East Asian ideograph
        0x6f5472 => [0xc528, false], # Korean hangul
        0x4b3f53 => [0x61f2, false], # East Asian ideograph
        0x4b5671 => [0x873b, false], # East Asian ideograph (variant of 215671 which maps to 873B)
        0x223f44 => [0x698a, false], # East Asian ideograph
        0x213f54 => [0x61f7, false], # East Asian ideograph
        0x234343 => [0x927a, false], # East Asian ideograph
        0x213f55 => [0x61f6, false], # East Asian ideograph
        0x27414f => [0x635f, false], # East Asian ideograph
        0x22327b => [0x63d7, false], # East Asian ideograph
        0x213f56 => [0x61f8, false], # East Asian ideograph
        0x223f51 => [0x69ca, false], # East Asian ideograph
        0x233237 => [0x8a57, false], # East Asian ideograph
        0x213f57 => [0x61f5, false], # East Asian ideograph
        0x6f5561 => [0xc5f0, false], # Korean hangul
        0x21355b => [0x5436, false], # East Asian ideograph
        0x233f58 => [0x9111, false], # East Asian ideograph
        0x215d5f => [0x927b, false], # East Asian ideograph
        0x224a66 => [0x6e62, false], # East Asian ideograph
        0x223f59 => [0x698d, false], # East Asian ideograph
        0x223f5a => [0x6991, false], # East Asian ideograph
        0x217c21 => [0x5aa6, false], # East Asian ideograph
        0x217c22 => [0x5a8c, false], # East Asian ideograph
        0x213f5b => [0x61fc, false], # East Asian ideograph
        0x227c24 => [0x828e, false], # East Asian ideograph
        0x227c25 => [0x8291, false], # East Asian ideograph
        0x217c26 => [0x5aa2, false], # East Asian ideograph
        0x227c27 => [0x828f, false], # East Asian ideograph
        0x227c28 => [0x8284, false], # East Asian ideograph
        0x273f5c => [0x604b, false], # East Asian ideograph
        0x227c2d => [0x8283, false], # East Asian ideograph
        0x227c2e => [0x828a, false], # East Asian ideograph
        0x213f5d => [0x6208, false], # East Asian ideograph
        0x225138 => [0x70cb, false], # East Asian ideograph
        0x274c78 => [0x762b, false], # East Asian ideograph
        0x227c34 => [0x82a7, false], # East Asian ideograph
        0x217c35 => [0x5a95, false], # East Asian ideograph
        0x217c36 => [0x5aaf, false], # East Asian ideograph
        0x227c38 => [0x82ab, false], # East Asian ideograph
        0x217c39 => [0x5ac8, false], # East Asian ideograph
        0x227c3a => [0x82b0, false], # East Asian ideograph
        0x227c3c => [0x82a4, false], # East Asian ideograph
        0x217c3e => [0x5ab5, false], # East Asian ideograph
        0x227c3f => [0x829a, false], # East Asian ideograph
        0x233f60 => [0x910b, false], # East Asian ideograph
        0x227c42 => [0x82a3, false], # East Asian ideograph
        0x6f4e7b => [0xb7ed, false], # Korean hangul
        0x227c44 => [0x82b7, false], # East Asian ideograph
        0x227c45 => [0x82ae, false], # East Asian ideograph (variant of 4C7C45 which maps to 82AE)
        0x227c46 => [0x82a9, false], # East Asian ideograph
        0x213f61 => [0x620c, false], # East Asian ideograph
        0x217c49 => [0x5ad1, false], # East Asian ideograph
        0x217c4a => [0x5a90, false], # East Asian ideograph
        0x6f5563 => [0xc5f6, false], # Korean hangul
        0x227c4c => [0x82a8, false], # East Asian ideograph
        0x213f62 => [0x6210, false], # East Asian ideograph
        0x227c4e => [0x82b4, false], # East Asian ideograph
        0x217c4f => [0x5ab8, false], # East Asian ideograph
        0x227c50 => [0x82a1, false], # East Asian ideograph
        0x226160 => [0x770e, false], # East Asian ideograph
        0x217c52 => [0x5aaa, false], # East Asian ideograph
        0x227c53 => [0x82aa, false], # East Asian ideograph
        0x227c55 => [0x82d9, false], # East Asian ideograph
        0x227c57 => [0x82fe, false], # East Asian ideograph
        0x2d3224 => [0x7b87, false], # East Asian ideograph
        0x217c59 => [0x5ad3, false], # East Asian ideograph
        0x227c5a => [0x82e0, false], # East Asian ideograph
        0x217c5b => [0x5ab1, false], # East Asian ideograph
        0x227c5c => [0x8300, false], # East Asian ideograph
        0x227c5f => [0x82ea, false], # East Asian ideograph
        0x227c60 => [0x82f7, false], # East Asian ideograph
        0x227c62 => [0x82ef, false], # East Asian ideograph
        0x227c63 => [0x833a, false], # East Asian ideograph
        0x227c64 => [0x82e4, false], # East Asian ideograph
        0x227c65 => [0x82d5, false], # East Asian ideograph
        0x227c67 => [0x8307, false], # East Asian ideograph
        0x227c68 => [0x82fa, false], # East Asian ideograph
        0x227c69 => [0x82f4, false], # East Asian ideograph
        0x227c6a => [0x82e2, false], # East Asian ideograph
        0x213f67 => [0x621b, false], # East Asian ideograph
        0x227c6d => [0x82d2, false], # East Asian ideograph
        0x217c6e => [0x5ae0, false], # East Asian ideograph
        0x227c71 => [0x82eb, false], # East Asian ideograph
        0x227c72 => [0x82d8, false], # East Asian ideograph
        0x227c73 => [0x82e1, false], # East Asian ideograph
        0x227c75 => [0x82f6, false], # East Asian ideograph
        0x28602b => [0x762a, false], # East Asian ideograph
        0x227c7b => [0x8310, false], # East Asian ideograph
        0x227c7c => [0x82f3, false], # East Asian ideograph
        0x233f6a => [0x911e, false], # East Asian ideograph
        0x4b4569 => [0x6a71, false], # East Asian ideograph
        0x23323b => [0x8a58, false], # East Asian ideograph
        0x6f5473 => [0xc529, false], # Korean hangul
        0x274237 => [0x631b, false], # East Asian ideograph
        0x6f5122 => [0xbc41, false], # Korean hangul
        0x226162 => [0x771b, false], # East Asian ideograph
        0x213f6d => [0x622e, false], # East Asian ideograph
        0x213f6e => [0x6230, false], # East Asian ideograph
        0x4b5973 => [0x8d2e, false], # East Asian ideograph
        0x275344 => [0x80c1, false], # East Asian ideograph
        0x213f6f => [0x6232, false], # East Asian ideograph
        0x23323c => [0x8a52, false], # East Asian ideograph
        0x6f5566 => [0xc5fd, false], # Korean hangul
        0x21355c => [0x5433, false], # East Asian ideograph
        0x274238 => [0x644a, false], # East Asian ideograph
        0x6f5123 => [0xbc43, false], # Korean hangul
        0x226163 => [0x7724, false], # East Asian ideograph
        0x213f72 => [0x6236, false], # East Asian ideograph
        0x234349 => [0x92aa, false], # East Asian ideograph
        0x6f4c38 => [0xb18d, false], # Korean hangul
        0x22557c => [0x728b, false], # East Asian ideograph
        0x213f74 => [0x623e, false], # East Asian ideograph
        0x225057 => [0x7098, false], # East Asian ideograph
        0x235b2f => [0x9d7a, false], # East Asian ideograph
        0x213f75 => [0x6240, false], # East Asian ideograph
        0x29325d => [0x8bd4, false], # East Asian ideograph
        0x2d3f76 => [0x78a5, false], # East Asian ideograph
        0x6f5a6b => [0xd0b5, false], # Korean hangul
        0x6f5124 => [0xbc44, false], # Korean hangul
        0x213b4c => [0x5c37, false], # East Asian ideograph
        0x223f77 => [0x69be, false], # East Asian ideograph
        0x4b6a22 => [0x7f83, false], # East Asian ideograph
        0x213f78 => [0x6248, false], # East Asian ideograph
        0x222a23 => [0x5f82, false], # East Asian ideograph
        0x233f79 => [0x912b, false], # East Asian ideograph
        0x4b456c => [0x6adb, false], # East Asian ideograph (variant of 21456C)
        0x2d602d => [0x976d, false], # East Asian ideograph
        0x213f7a => [0x624b, false], # East Asian ideograph
        0x212a25 => [0xe8d4, false], # EACC component character
        0x6f5568 => [0xc5ff, false], # Korean hangul
        0x294e7b => [0x9883, false], # East Asian ideograph
        0x4b6a26 => [0x6c8d, false], # East Asian ideograph
        0x6f4a39 => [0xae37, false], # Korean hangul
        0x2d5a34 => [0x8cad, false], # East Asian ideograph
        0x393d6f => [0x8907, false], # East Asian ideograph
        0x213f7e => [0x6254, false], # East Asian ideograph
        0x6f5a70 => [0xd0c0, false], # Korean hangul
        0x4b456d => [0x823b, false], # East Asian ideograph
        0x2d403f => [0x6255, false], # East Asian ideograph
        0x695f70 => [0x7195, false], # East Asian ideograph
        0x27423b => [0x63fd, false], # East Asian ideograph
        0x6f5126 => [0xbc84, false], # Korean hangul
        0x225731 => [0x731d, false], # East Asian ideograph
        0x213a7a => [0x5bb5, false], # East Asian ideograph
        0x696a2c => [0x87d0, false], # East Asian ideograph
        0x2e5a78 => [0x74a2, false], # East Asian ideograph
        0x277954 => [0x5956, false], # East Asian ideograph
        0x212a2e => [0xe8dc, false], # EACC component character
        0x333240 => [0x4ffb, false], # East Asian ideograph
        0x232a2f => [0x86fa, false], # East Asian ideograph
        0x227d21 => [0x830c, false], # East Asian ideograph
        0x227d22 => [0x82fb, false], # East Asian ideograph
        0x227d24 => [0x82fd, false], # East Asian ideograph
        0x215925 => [0x8ae6, false], # East Asian ideograph
        0x227d26 => [0x8333, false], # East Asian ideograph
        0x4b5238 => [0x7f87, false], # East Asian ideograph
        0x227d29 => [0x8328, false], # East Asian ideograph
        0x217d2a => [0x5afd, false], # East Asian ideograph
        0x217d2b => [0x5b08, false], # East Asian ideograph
        0x212a32 => [0xe8df, false], # EACC component character
        0x227d2e => [0x8351, false], # East Asian ideograph
        0x214c5c => [0x75f1, false], # East Asian ideograph
        0x6f5c7b => [0xd5cc, false], # Korean hangul
        0x4b456f => [0x685c, false], # East Asian ideograph
        0x227d35 => [0x831b, false], # East Asian ideograph
        0x217d38 => [0x5b03, false], # East Asian ideograph
        0x222a34 => [0x3013, false], # East Asian ideograph (not found in unified han)
        0x227d3b => [0x8356, false], # East Asian ideograph
        0x217d3d => [0x5b17, false], # East Asian ideograph
        0x217d3e => [0x5b16, false], # East Asian ideograph
        0x227d3f => [0x8322, false], # East Asian ideograph
        0x227d40 => [0x832c, false], # East Asian ideograph
        0x212a36 => [0xe8e3, false], # EACC component character
        0x217d47 => [0x5b1b, false], # East Asian ideograph
        0x227d48 => [0x833c, false], # East Asian ideograph
        0x227d4a => [0x834d, false], # East Asian ideograph
        0x334f3a => [0x7a49, false], # East Asian ideograph
        0x227d4d => [0x8343, false], # East Asian ideograph (variant of 4C7D4D which maps to 8343)
        0x22505c => [0x70b7, false], # East Asian ideograph
        0x4b4570 => [0x6a29, false], # East Asian ideograph
        0x227d52 => [0x832f, false], # East Asian ideograph
        0x227d53 => [0x8348, false], # East Asian ideograph
        0x227d54 => [0x8312, false], # East Asian ideograph
        0x227d56 => [0x8316, false], # East Asian ideograph
        0x212a39 => [0xe8e6, false], # EACC component character
        0x227d58 => [0x831a, false], # East Asian ideograph
        0x217d59 => [0x5b32, false], # East Asian ideograph
        0x2e6f43 => [0x9908, false], # East Asian ideograph
        0x6f5a6c => [0xd0b7, false], # Korean hangul
        0x6f5129 => [0xbc8b, false], # Korean hangul
        0x213b51 => [0x5c41, false], # East Asian ideograph
        0x227d5f => [0x8347, false], # East Asian ideograph
        0x227d62 => [0x83a8, false], # East Asian ideograph
        0x217d63 => [0x5b3f, false], # East Asian ideograph
        0x4b3021 => [0x58f1, false], # East Asian ideograph
        0x227d67 => [0x83ad, false], # East Asian ideograph
        0x6f5121 => [0xbc40, false], # Korean hangul
        0x286a3c => [0x7aad, false], # East Asian ideograph
        0x4c7d6a => [0x8323, false], # East Asian ideograph
        0x227d6d => [0x8373, false], # East Asian ideograph
        0x217d6e => [0x5b45, false], # East Asian ideograph
        0x6f4e7d => [0xb7f4, false], # Korean hangul
        0x227d72 => [0x83b0, false], # East Asian ideograph
        0x217d74 => [0x5b4c, false], # East Asian ideograph
        0x212a3e => [0xe8eb, false], # EACC component character
        0x227d76 => [0x831d, false], # East Asian ideograph
        0x6f556d => [0xc608, false], # Korean hangul
        0x282868 => [0x5e91, false], # East Asian ideograph
        0x227d7a => [0x838f, false], # East Asian ideograph
        0x6f512a => [0xbc8c, false], # Korean hangul
        0x227d7c => [0x8395, false], # East Asian ideograph
        0x227d7e => [0x8375, false], # East Asian ideograph
        0x215928 => [0x8af1, false], # East Asian ideograph
        0x234350 => [0x92a6, false], # East Asian ideograph
        0x51384d => [0x51c3, false], # East Asian ideograph
        0x69545c => [0x58d7, false], # East Asian ideograph
        0x212a41 => [0xe8ee, false], # EACC component character
        0x275e61 => [0x9614, false], # East Asian ideograph
        0x212a46 => [0x3013, false], # Ideographic geta symbol
        0x212a42 => [0xe8ef, false], # EACC component character
        0x23545d => [0x9ac8, false], # East Asian ideograph
        0x226a43 => [0x7abf, false], # East Asian ideograph
        0x6f556e => [0xc60c, false], # Korean hangul
        0x6f512b => [0xbc94, false], # Korean hangul
        0x22315c => [0x634b, false], # East Asian ideograph
        0x212a45 => [0xe8f2, false], # EACC component character
        0x6f5574 => [0xc62c, false], # Korean hangul
        0x2d314c => [0x5008, false], # East Asian ideograph
        0x27534d => [0x8109, false], # East Asian ideograph
        0x273b6e => [0x5188, false], # East Asian ideograph
        0x214c60 => [0x760d, false], # East Asian ideograph
        0x2d4d71 => [0x7719, false], # East Asian ideograph
        0x213e6a => [0x60df, false], # East Asian ideograph
        0x6f5c21 => [0xd33d, false], # Korean hangul
        0x2d3c61 => [0x5e47, false], # East Asian ideograph
        0x234667 => [0x93e7, false], # East Asian ideograph
        0x6f512c => [0xbc95, false], # Korean hangul
        0x21592a => [0x8adc, false], # East Asian ideograph
        0x2d3c65 => [0x79ca, false], # East Asian ideograph
        0x6f4e29 => [0xb550, false], # Korean hangul
        0x393e47 => [0x8cc9, false], # East Asian ideograph
        0x6f5032 => [0xba70, false], # Korean hangul
        0x2f386f => [0x8dd7, false], # East Asian ideograph
        0x214c61 => [0x7627, false], # East Asian ideograph
        0x6f2464 => [0x314e, false], # Korean hangul
        0x6f5b47 => [0xd23c, false], # Korean hangul
        0x6f512d => [0xbc97, false], # Korean hangul
        0x6f516e => [0xbe45, false], # Korean hangul
        0x226a4f => [0x7ad1, false], # East Asian ideograph
        0x4b3974 => [0x5b22, false], # East Asian ideograph
        0x3f5564 => [0x61de, false], # East Asian ideograph
        0x214c62 => [0x7613, false], # East Asian ideograph
        0x4b6130 => [0x99c4, false], # East Asian ideograph
        0x6f5571 => [0xc624, false], # Korean hangul
        0x6f512e => [0xbc98, false], # Korean hangul
        0x284642 => [0x6bf5, false], # East Asian ideograph
        0x226a54 => [0x7ad5, false], # East Asian ideograph
        0x2d5a3d => [0x8cdb, false], # East Asian ideograph
        0x225c50 => [0x74e4, false], # East Asian ideograph
        0x225062 => [0x70a1, false], # East Asian ideograph
        0x335461 => [0x6cd6, false], # East Asian ideograph
        0x213041 => [0x4e4d, false], # East Asian ideograph
        0x6f5572 => [0xc625, false], # Korean hangul
        0x6f512f => [0xbc99, false], # Korean hangul
        0x234355 => [0x929a, false], # East Asian ideograph
        0x6f4a3b => [0xae40, false], # Korean hangul
        0x217e59 => [0x5bc1, false], # East Asian ideograph
        0x2f2a5a => [0x868b, false], # Unrelated variant of EACC 23293D which maps to 868B
        0x22714b => [0x7d9b, false], # East Asian ideograph
        0x227e21 => [0x837f, false], # East Asian ideograph
        0x227e22 => [0x8399, false], # East Asian ideograph
        0x225063 => [0x70a3, false], # East Asian ideograph
        0x217e24 => [0x5b65, false], # East Asian ideograph
        0x227e25 => [0x8387, false], # East Asian ideograph
        0x227e26 => [0x83b9, false], # East Asian ideograph
        0x217e27 => [0x5c58, false], # East Asian ideograph (not in Unicode)
        0x217e28 => [0x5b6c, false], # East Asian ideograph
        0x217e2a => [0x5b6e, false], # East Asian ideograph
        0x227e2b => [0x83a9, false], # East Asian ideograph
        0x227e2f => [0x839b, false], # East Asian ideograph
        0x217e30 => [0x5b7b, false], # East Asian ideograph
        0x217e31 => [0x5b7c, false], # East Asian ideograph
        0x217e32 => [0x5b80, false], # East Asian ideograph
        0x227e33 => [0x83aa, false], # East Asian ideograph
        0x217e34 => [0x5b84, false], # East Asian ideograph
        0x217e35 => [0x5b82, false], # East Asian ideograph (not in Unicode)
        0x227e37 => [0x839c, false], # East Asian ideograph
        0x227e38 => [0x839f, false], # East Asian ideograph
        0x222a5f => [0x5fd1, false], # East Asian ideograph
        0x217e40 => [0x5b95, false], # East Asian ideograph
        0x227e41 => [0x83cf, false], # East Asian ideograph
        0x227e43 => [0x83f9, false], # East Asian ideograph
        0x2f445f => [0x941a, false], # East Asian ideograph
        0x227e45 => [0x8421, false], # East Asian ideograph
        0x6f5476 => [0xc538, false], # Korean hangul
        0x217e49 => [0x5bac, false], # East Asian ideograph
        0x6f5131 => [0xbca0, false], # Korean hangul
        0x294a44 => [0x9655, false], # East Asian ideograph
        0x21592f => [0x8af7, false], # East Asian ideograph
        0x227e52 => [0x83ea, false], # East Asian ideograph
        0x227e53 => [0x8413, false], # East Asian ideograph
        0x217e55 => [0x5bb7, false], # East Asian ideograph
        0x227e56 => [0x83fc, false], # East Asian ideograph
        0x227e57 => [0x83f6, false], # East Asian ideograph
        0x227e59 => [0x8410, false], # East Asian ideograph
        0x227e5a => [0x83e1, false], # East Asian ideograph
        0x217e5b => [0x3761, false], # East Asian ideograph (not found in unified han)
        0x227e60 => [0x83c6, false], # East Asian ideograph
        0x227e61 => [0x8407, false], # East Asian ideograph
        0x227e63 => [0x83eb, false], # East Asian ideograph
        0x216a66 => [0x51df, false], # East Asian ideograph
        0x6f5575 => [0xc62d, false], # Korean hangul
        0x217e68 => [0x5bd4, false], # East Asian ideograph
        0x217e6a => [0x5bc3, false], # East Asian ideograph
        0x227e6b => [0x83e2, false], # East Asian ideograph
        0x227e6d => [0x8401, false], # East Asian ideograph
        0x217e6e => [0x5bd6, false], # East Asian ideograph
        0x234358 => [0x92ab, false], # East Asian ideograph
        0x227e71 => [0x83d8, false], # East Asian ideograph
        0x227e72 => [0x83e5, false], # East Asian ideograph
        0x227e74 => [0x8418, false], # East Asian ideograph
        0x217e75 => [0x5bd7, false], # East Asian ideograph
        0x227e79 => [0x83ce, false], # East Asian ideograph
        0x227e7b => [0x83d3, false], # East Asian ideograph
        0x295b52 => [0x9e4e, false], # East Asian ideograph
        0x227e7d => [0x83d6, false], # East Asian ideograph
        0x217e7e => [0x5bea, false], # East Asian ideograph
        0x6f5576 => [0xc62e, false], # Korean hangul
        0x6f5133 => [0xbca4, false], # Korean hangul
        0x294a46 => [0x9649, false], # East Asian ideograph
        0x275f3d => [0x96b6, false], # East Asian ideograph
        0x4b6260 => [0x9ebd, false], # East Asian ideograph
        0x227e6a => [0x83bf, false], # East Asian ideograph
        0x2d6030 => [0x97ee, false], # East Asian ideograph
        0x235466 => [0x9ad0, false], # East Asian ideograph
        0x22454d => [0x6996, false], # East Asian ideograph
        0x6f5577 => [0xc633, false], # Korean hangul
        0x6f5134 => [0xbca7, false], # Korean hangul
        0x213b5c => [0x5c50, false], # East Asian ideograph
        0x215932 => [0x8b19, false], # East Asian ideograph
        0x6f4a3c => [0xae41, false], # Korean hangul
        0x2d3c6d => [0x8298, false], # East Asian ideograph (duplicate simplified)
        0x222a73 => [0x5ff8, false], # East Asian ideograph
        0x225068 => [0x7551, false], # East Asian ideograph
        0x6f5551 => [0xc5c8, false], # Korean hangul
        0x6f5135 => [0xbca8, false], # Korean hangul
        0x215521 => [0x5179, false], # East Asian ideograph
        0x223f5c => [0x69aa, false], # East Asian ideograph
        0x275274 => [0x58f0, false], # East Asian ideograph
        0x2d3c6e => [0x7240, false], # East Asian ideograph
        0x6f5523 => [0xc54c, false], # Korean hangul
        0x6f5524 => [0xc54e, false], # Korean hangul
        0x23324f => [0x8a7f, false], # East Asian ideograph
        0x233e37 => [0x9088, false], # East Asian ideograph
        0x4c4446 => [0x6b4e, false], # East Asian ideograph
        0x6f5525 => [0xc553, false], # Korean hangul
        0x232a7b => [0x877b, false], # East Asian ideograph
        0x6f5526 => [0xc554, false], # Korean hangul
        0x3a6a7c => [0x7bea, false], # East Asian ideograph
        0x235527 => [0x9aeb, false], # East Asian ideograph
        0x22763d => [0x801e, false], # East Asian ideograph
        0x235528 => [0x9af2, false], # East Asian ideograph
        0x215529 => [0x8396, false], # East Asian ideograph
        0x233250 => [0x8a86, false], # East Asian ideograph
        0x225424 => [0x71c1, false], # East Asian ideograph
        0x21552a => [0x83a7, false], # East Asian ideograph
        0x6f5137 => [0xbcb1, false], # Korean hangul
        0x213b5f => [0x5c5c, false], # East Asian ideograph
        0x6f552b => [0xc55e, false], # Korean hangul
        0x295f7b => [0x9f51, false], # East Asian ideograph
        0x2d3c70 => [0x576b, false], # East Asian ideograph
        0x27552d => [0x5e84, false], # East Asian ideograph
        0x2d552e => [0x82fa, false], # East Asian ideograph (variant of 227C68)
        0x23316e => [0x8a04, false], # East Asian ideograph
        0x2d493a => [0x702c, false], # East Asian ideograph
        0x215d79 => [0x9375, false], # East Asian ideograph
        0x28464c => [0x6be1, false], # East Asian ideograph
        0x213b60 => [0x5c62, false], # East Asian ideograph
        0x6f5531 => [0xc570, false], # Korean hangul
        0x293726 => [0x8d5c, false], # East Asian ideograph
        0x235532 => [0x9af9, false], # East Asian ideograph
        0x6f5533 => [0xc573, false], # Korean hangul
        0x6f5534 => [0xc574, false], # Korean hangul
        0x2f3363 => [0x8b1a, false], # East Asian ideograph
        0x6f5139 => [0xbcb5, false], # Korean hangul
        0x235535 => [0x9afd, false], # East Asian ideograph
        0x4b3474 => [0x537f, false], # East Asian ideograph (variant of 213474 which maps to 537F)
        0x6f582f => [0xc9d0, false], # Korean hangul
        0x22385a => [0x6678, false], # East Asian ideograph
        0x235536 => [0x9b01, false], # East Asian ideograph
        0x2d5a48 => [0x8d71, false], # East Asian ideograph
        0x295b59 => [0x9e5c, false], # East Asian ideograph
        0x235538 => [0x9b02, false], # East Asian ideograph
        0x6f5539 => [0xc584, false], # Korean hangul
        0x6f513a => [0xbcbc, false], # Korean hangul
        0x4b553a => [0x83c1, false], # East Asian ideograph (variant of 21553A which maps to 83C1)
        0x23553b => [0x9b00, false], # East Asian ideograph
        0x2d3830 => [0x573b, false], # East Asian ideograph
        0x27553c => [0x534e, false], # East Asian ideograph
        0x283542 => [0x64b7, false], # East Asian ideograph
        0x287531 => [0x7f9f, false], # East Asian ideograph
        0x33502a => [0x9257, false], # East Asian ideograph
        0x23553e => [0x9b04, false], # East Asian ideograph
        0x6f513b => [0xbcbd, false], # Korean hangul
        0x213b63 => [0x5c6c, false], # East Asian ideograph
        0x22565b => [0x72c1, false], # East Asian ideograph
        0x275947 => [0x8c34, false], # East Asian ideograph
        0x223442 => [0x648f, false], # East Asian ideograph
        0x213e3f => [0x6062, false], # East Asian ideograph
        0x3f4472 => [0x7881, false], # East Asian ideograph
        0x215541 => [0x840a, false], # East Asian ideograph
        0x4b5542 => [0x8420, false], # East Asian ideograph
        0x33502b => [0x724b, false], # East Asian ideograph
        0x235543 => [0x9b0b, false], # East Asian ideograph
        0x6f4b2b => [0xafce, false], # Korean hangul
        0x2e4d3d => [0x6d38, false], # East Asian ideograph
        0x696a5e => [0x88b0, false], # East Asian ideograph
        0x227431 => [0x7f4e, false], # East Asian ideograph
        0x213561 => [0x543b, false], # East Asian ideograph
        0x225d39 => [0x7517, false], # East Asian ideograph
        0x6f5545 => [0xc5b5, false], # Korean hangul
        0x6f5546 => [0xc5b6, false], # Korean hangul
        0x6f4f5b => [0xb98e, false], # Korean hangul
        0x295b5c => [0x9e5b, false], # East Asian ideograph
        0x225070 => [0x79cc, false], # East Asian ideograph
        0x235547 => [0x9b0e, false], # East Asian ideograph
        0x233256 => [0x8a61, false], # East Asian ideograph
        0x6f5548 => [0xc5b9, false], # Korean hangul
        0x274252 => [0x654c, false], # East Asian ideograph
        0x6f513d => [0xbcc4, false], # Korean hangul
        0x284651 => [0x6c07, false], # East Asian ideograph
        0x6f5549 => [0xc5ba, false], # Korean hangul
        0x213722 => [0x55c6, false], # East Asian ideograph
        0x233c2d => [0x8f40, false], # East Asian ideograph
        0x6f554a => [0xc5bb, false], # Korean hangul
        0x6f554b => [0xc5bc, false], # Korean hangul
        0x28575e => [0x72b8, false], # East Asian ideograph
        0x227849 => [0x811d, false], # East Asian ideograph
        0x393944 => [0x59b3, false], # East Asian ideograph
        0x6f554c => [0xc5bd, false], # Korean hangul
        0x355e76 => [0x82be, false], # East Asian ideograph
        0x27554d => [0x82c7, false], # East Asian ideograph
        0x6f513e => [0xbccc, false], # Korean hangul
        0x23554e => [0x9b11, false], # East Asian ideograph
        0x223f65 => [0x699e, false], # East Asian ideograph
        0x4b5959 => [0x8273, false], # East Asian ideograph
        0x21554f => [0x8449, false], # East Asian ideograph
        0x2d5550 => [0x585f, false], # East Asian ideograph
        0x28575f => [0x72f2, false], # East Asian ideograph
        0x283546 => [0x6445, false], # East Asian ideograph
        0x225072 => [0x70bf, false], # East Asian ideograph
        0x215551 => [0x846b, false], # East Asian ideograph
        0x233258 => [0x8a3e, false], # East Asian ideograph
        0x225552 => [0x7253, false], # East Asian ideograph
        0x274254 => [0x6570, false], # East Asian ideograph
        0x6f513f => [0xbccd, false], # Korean hangul
        0x225553 => [0x7255, false], # East Asian ideograph
        0x275276 => [0x806a, false], # East Asian ideograph
        0x6f7726 => [0xadb9, false], # Korean hangul
        0x235554 => [0x9b18, false], # East Asian ideograph
        0x6f585a => [0xcac4, false], # Korean hangul
        0x333f22 => [0x6168, false], # East Asian ideograph (variant of 213F22 which maps to 6168)
        0x275555 => [0x83b4, false] # East Asian ideograph
      }
      CHARSET_70 = { # Superscripts
        0x28 => [0x207d, false], # SUPERSCRIPT OPENING PARENTHESIS / SUPERSCRIPT LEFT PARENTHESIS
        0x29 => [0x207e, false], # SUPERSCRIPT CLOSING PARENTHESIS / SUPERSCRIPT RIGHT PARENTHESIS
        0x2b => [0x207a, false], # SUPERSCRIPT PLUS SIGN
        0x2d => [0x207b, false], # SUPERSCRIPT HYPHEN-MINUS / SUPERSCRIPT MINUS
        0x30 => [0x2070, false], # SUPERSCRIPT DIGIT ZERO
        0x31 => [0xb9, false], # SUPERSCRIPT DIGIT ONE
        0x32 => [0xb2, false], # SUPERSCRIPT DIGIT TWO
        0x33 => [0xb3, false], # SUPERSCRIPT DIGIT THREE
        0x34 => [0x2074, false], # SUPERSCRIPT DIGIT FOUR
        0x35 => [0x2075, false], # SUPERSCRIPT DIGIT FIVE
        0x36 => [0x2076, false], # SUPERSCRIPT DIGIT SIX
        0x37 => [0x2077, false], # SUPERSCRIPT DIGIT SEVEN
        0x38 => [0x2078, false], # SUPERSCRIPT DIGIT EIGHT
        0x39 => [0x2079, false] # SUPERSCRIPT DIGIT NINE
      }
      CHARSET_51 = { # Extended Cyrillic
        0xc0 => [0x491, false], # LOWERCASE GE WITH UPTURN / CYRILLIC SMALL LETTER GHE WITH UPTURN
        0xc1 => [0x452, false], # LOWERCASE DJE / CYRILLIC SMALL LETTER DJE (Serbian)
        0xc2 => [0x453, false], # CYRILLIC SMALL LETTER GJE
        0xc3 => [0x454, false], # LOWERCASE E / CYRILLIC SMALL LETTER UKRAINIAN IE
        0xc4 => [0x451, false], # CYRILLIC SMALL LETTER IO
        0xc5 => [0x455, false], # CYRILLIC SMALL LETTER DZE
        0xc6 => [0x456, false], # LOWERCASE I / CYRILLIC SMALL LETTER BYELORUSSIAN-UKRANIAN I
        0xc7 => [0x457, false], # LOWERCASE YI / CYRILLIC SMALL LETTER YI (Ukrainian)
        0xc8 => [0x458, false], # CYRILLIC SMALL LETTER JE
        0xc9 => [0x459, false], # CYRILLIC SMALL LETTER LJE
        0xca => [0x45a, false], # CYRILLIC SMALL LETTER NJE
        0xcb => [0x45b, false], # LOWERCASE TSHE / CYRILLIC SMALL LETTER TSHE (Serbian)
        0xcc => [0x45c, false], # CYRILLIC SMALL LETTER KJE
        0xcd => [0x45e, false], # LOWERCASE SHORT U / CYRILLIC SMALL LETTER SHORT U (Byelorussian)
        0xce => [0x45f, false], # CYRILLIC SMALL LETTER DZHE
        0xd0 => [0x463, false], # CYRILLIC SMALL LETTER YAT
        0xd1 => [0x473, false], # CYRILLIC SMALL LETTER FITA
        0xd2 => [0x475, false], # CYRILLIC SMALL LETTER IZHITSA
        0xd3 => [0x46b, false], # CYRILLIC SMALL LETTER BIG YUS
        0xdb => [0x5b, false], # OPENING SQUARE BRACKET / LEFT SQUARE BRACKET
        0xdd => [0x5d, false], # CLOSING SQUARE BRACKET / RIGHT SQUARE BRACKET
        0xdf => [0x5f, false], # SPACING UNDERSCORE / LOW LINE
        0xe0 => [0x490, false], # UPPERCASE GE WITH UPTURN / CYRILLIC CAPITAL LETTER GHE WITH UPTURN
        0xe1 => [0x402, false], # UPPERCASE DJE / CYRILLIC CAPITAL LETTER DJE (Serbian)
        0xe2 => [0x403, false], # CYRILLIC CAPITAL LETTER GJE
        0xe3 => [0x404, false], # UPPERCASE E / CYRILLIC CAPITAL LETTER UKRAINIAN IE
        0xe4 => [0x401, false], # CYRILLIC CAPITAL LETTER IO
        0xe5 => [0x405, false], # CYRILLIC CAPITAL LETTER DZE
        0xe6 => [0x406, false], # UPPERCASE I / CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRANIAN I
        0xe7 => [0x407, false], # UPPERCASE YI / CYRILLIC CAPITAL LETTER YI (Ukrainian)
        0xe8 => [0x408, false], # CYRILLIC CAPITAL LETTER JE
        0xe9 => [0x409, false], # CYRILLIC CAPITAL LETTER LJE
        0xea => [0x40a, false], # CYRILLIC CAPITAL LETTER NJE
        0xeb => [0x40b, false], # UPPERCASE TSHE / CYRILLIC CAPITAL LETTER TSHE (Serbian)
        0xec => [0x40c, false], # CYRILLIC CAPITAL LETTER KJE
        0xed => [0x40e, false], # UPPERCASE SHORT U / CYRILLIC CAPITAL LETTER SHORT U (Byelorussian)
        0xee => [0x40f, false], # CYRILLIC CAPITAL LETTER DZHE
        0xef => [0x42a, false], # CYRILLIC CAPITAL LETTER HARD SIGN
        0xf0 => [0x462, false], # CYRILLIC CAPITAL LETTER YAT
        0xf1 => [0x472, false], # CYRILLIC CAPITAL LETTER FITA
        0xf2 => [0x474, false], # CYRILLIC CAPITAL LETTER IZHITSA
        0xf3 => [0x46a, false] # CYRILLIC CAPITAL LETTER BIG YUS
      }
      CHARSET_53 = { # Basic Greek
        0x21 => [0x300, true], # COMBINING GRAVE ACCENT
        0x22 => [0x301, true], # COMBINING ACUTE ACCENT
        0x23 => [0x308, true], # COMBINING DIAERESIS
        0x24 => [0x342, true], # COMBINING GREEK PERISPOMENI / CIRCUMFLEX
        0x25 => [0x313, true], # COMBINING COMMA ABOVE / SMOOTH BREATHING
        0x26 => [0x314, true], # COMBINING REVERSED COMMA ABOVE / ROUGH BREATHING
        0x27 => [0x345, true], # COMBINING GREEK YPOGEGRAMMENI / IOTA SUBSCRIPT
        0x30 => [0xab, false], # LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
        0x31 => [0xbb, false], # RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
        0x32 => [0x201c, false], # LEFT DOUBLE QUOTATION MARK
        0x33 => [0x201d, false], # RIGHT DOUBLE QUOTATION MARK
        0x34 => [0x374, false], # GREEK NUMERAL SIGN / UPPER PRIME
        0x35 => [0x375, false], # GREEK LOWER NUMERAL SIGN / LOWER PRIME
        0x3b => [0x387, false], # GREEK ANO TELEIA / RAISED DOT, GREEK SEMICOLON
        0x3f => [0x37e, false], # GREEK QUESTION MARK
        0x41 => [0x391, false], # GREEK CAPITAL LETTER ALPHA
        0x42 => [0x392, false], # GREEK CAPITAL LETTER BETA
        0x44 => [0x393, false], # GREEK CAPITAL LETTER GAMMA
        0x45 => [0x394, false], # GREEK CAPITAL LETTER DELTA
        0x46 => [0x395, false], # GREEK CAPITAL LETTER EPSILON
        0x47 => [0x3da, false], # GREEK LETTER STIGMA
        0x48 => [0x3dc, false], # GREEK LETTER DIGAMMA
        0x49 => [0x396, false], # GREEK CAPITAL LETTER ZETA
        0x4a => [0x397, false], # GREEK CAPITAL LETTER ETA
        0x4b => [0x398, false], # GREEK CAPITAL LETTER THETA
        0x4c => [0x399, false], # GREEK CAPITAL LETTER IOTA
        0x4d => [0x39a, false], # GREEK CAPITAL LETTER KAPPA
        0x4e => [0x39b, false], # GREEK CAPITAL LETTER LAMDA
        0x4f => [0x39c, false], # GREEK CAPITAL LETTER MU
        0x50 => [0x39d, false], # GREEK CAPITAL LETTER NU
        0x51 => [0x39e, false], # GREEK CAPITAL LETTER XI
        0x52 => [0x39f, false], # GREEK CAPITAL LETTER OMICRON
        0x53 => [0x3a0, false], # GREEK CAPITAL LETTER PI
        0x54 => [0x3de, false], # GREEK LETTER KOPPA
        0x55 => [0x3a1, false], # GREEK CAPITAL LETTER RHO
        0x56 => [0x3a3, false], # GREEK CAPITAL LETTER SIGMA
        0x58 => [0x3a4, false], # GREEK CAPITAL LETTER TAU
        0x59 => [0x3a5, false], # GREEK CAPITAL LETTER UPSILON
        0x5a => [0x3a6, false], # GREEK CAPITAL LETTER PHI
        0x5b => [0x3a7, false], # GREEK CAPITAL LETTER CHI
        0x5c => [0x3a8, false], # GREEK CAPITAL LETTER PSI
        0x5d => [0x3a9, false], # GREEK CAPITAL LETTER OMEGA
        0x5e => [0x3e0, false], # GREEK LETTER SAMPI
        0x61 => [0x3b1, false], # GREEK SMALL LETTER ALPHA
        0x62 => [0x3b2, false], # GREEK SMALL LETTER BETA / SMALL LETTER BETA BEGINNING OF WORD
        0x63 => [0x3d0, false], # GREEK BETA SYMBOL / SMALL LETTER BETA MIDDLE OF WORD
        0x64 => [0x3b3, false], # GREEK SMALL LETTER GAMMA
        0x65 => [0x3b4, false], # GREEK SMALL LETTER DELTA
        0x66 => [0x3b5, false], # GREEK SMALL LETTER EPSILON
        0x67 => [0x3db, false], # GREEK SMALL LETTER STIGMA
        0x68 => [0x3dd, false], # GREEK SMALL LETTER DIGAMMA
        0x69 => [0x3b6, false], # GREEK SMALL LETTER ZETA
        0x6a => [0x3b7, false], # GREEK SMALL LETTER ETA
        0x6b => [0x3b8, false], # GREEK SMALL LETTER THETA
        0x6c => [0x3b9, false], # GREEK SMALL LETTER IOTA
        0x6d => [0x3ba, false], # GREEK SMALL LETTER KAPPA
        0x6e => [0x3bb, false], # GREEK SMALL LETTER LAMDA
        0x6f => [0x3bc, false], # GREEK SMALL LETTER MU
        0x70 => [0x3bd, false], # GREEK SMALL LETTER NU
        0x71 => [0x3be, false], # GREEK SMALL LETTER XI
        0x72 => [0x3bf, false], # GREEK SMALL LETTER OMICRON
        0x73 => [0x3c0, false], # GREEK SMALL LETTER PI
        0x74 => [0x3df, false], # GREEK SMALL LETTER KOPPA
        0x75 => [0x3c1, false], # GREEK SMALL LETTER RHO
        0x76 => [0x3c3, false], # GREEK SMALL LETTER SIGMA
        0x77 => [0x3c2, false], # GREEK SMALL LETTER FINAL SIGMA / SMALL LETTER SIGMA END OF WORD
        0x78 => [0x3c4, false], # GREEK SMALL LETTER TAU
        0x79 => [0x3c5, false], # GREEK SMALL LETTER UPSILON
        0x7a => [0x3c6, false], # GREEK SMALL LETTER PHI
        0x7b => [0x3c7, false], # GREEK SMALL LETTER CHI
        0x7c => [0x3c8, false], # GREEK SMALL LETTER PSI
        0x7d => [0x3c9, false], # GREEK SMALL LETTER OMEGA
        0x7e => [0x3e1, false] # GREEK SMALL LETTER SAMPI
      }
      CHARSET_42 = { # Basic Latin (ASCII)
        0x1b => [0x1b, false], # ESCAPE (Unlikely to occur in UCS/Unicode)
        0x1d => [0x1d, false], # RECORD TERMINATOR / GROUP SEPARATOR
        0x1e => [0x1e, false], # FIELD TERMINATOR / RECORD SEPARATOR
        0x1f => [0x1f, false], # SUBFIELD DELIMITER / UNIT SEPARATOR
        0x20 => [0x20, false], # SPACE, BLANK / SPACE
        0x21 => [0x21, false], # EXCLAMATION MARK
        0x22 => [0x22, false], # QUOTATION MARK
        0x23 => [0x23, false], # NUMBER SIGN
        0x24 => [0x24, false], # DOLLAR SIGN
        0x25 => [0x25, false], # PERCENT SIGN
        0x26 => [0x26, false], # AMPERSAND
        0x27 => [0x27, false], # APOSTROPHE
        0x28 => [0x28, false], # OPENING PARENTHESIS / LEFT PARENTHESIS
        0x29 => [0x29, false], # CLOSING PARENTHESIS / CLOSING PARENTHESIS
        0x2a => [0x2a, false], # ASTERISK
        0x2b => [0x2b, false], # PLUS SIGN
        0x2c => [0x2c, false], # COMMA
        0x2d => [0x2d, false], # HYPHEN-MINUS
        0x2e => [0x2e, false], # PERIOD, DECIMAL POINT / FULL STOP
        0x2f => [0x2f, false], # SLASH / SOLIDUS
        0x30 => [0x30, false], # DIGIT ZERO
        0x31 => [0x31, false], # DIGIT ONE
        0x32 => [0x32, false], # DIGIT TWO
        0x33 => [0x33, false], # DIGIT THREE
        0x34 => [0x34, false], # DIGIT FOUR
        0x35 => [0x35, false], # DIGIT FIVE
        0x36 => [0x36, false], # DIGIT SIX
        0x37 => [0x37, false], # DIGIT SEVEN
        0x38 => [0x38, false], # DIGIT EIGHT
        0x39 => [0x39, false], # DIGIT NINE
        0x3a => [0x3a, false], # COLON
        0x3b => [0x3b, false], # SEMICOLON
        0x3c => [0x3c, false], # LESS-THAN SIGN
        0x3d => [0x3d, false], # EQUALS SIGN
        0x3e => [0x3e, false], # GREATER-THAN SIGN
        0x3f => [0x3f, false], # QUESTION MARK
        0x40 => [0x40, false], # COMMERCIAL AT
        0x41 => [0x41, false], # LATIN CAPITAL LETTER A
        0x42 => [0x42, false], # LATIN CAPITAL LETTER B
        0x43 => [0x43, false], # LATIN CAPITAL LETTER C
        0x44 => [0x44, false], # LATIN CAPITAL LETTER D
        0x45 => [0x45, false], # LATIN CAPITAL LETTER E
        0x46 => [0x46, false], # LATIN CAPITAL LETTER F
        0x47 => [0x47, false], # LATIN CAPITAL LETTER G
        0x48 => [0x48, false], # LATIN CAPITAL LETTER H
        0x49 => [0x49, false], # LATIN CAPITAL LETTER I
        0x4a => [0x4a, false], # LATIN CAPITAL LETTER J
        0x4b => [0x4b, false], # LATIN CAPITAL LETTER K
        0x4c => [0x4c, false], # LATIN CAPITAL LETTER L
        0x4d => [0x4d, false], # LATIN CAPITAL LETTER M
        0x4e => [0x4e, false], # LATIN CAPITAL LETTER N
        0x4f => [0x4f, false], # LATIN CAPITAL LETTER O
        0x50 => [0x50, false], # LATIN CAPITAL LETTER P
        0x51 => [0x51, false], # LATIN CAPITAL LETTER Q
        0x52 => [0x52, false], # LATIN CAPITAL LETTER R
        0x53 => [0x53, false], # LATIN CAPITAL LETTER S
        0x54 => [0x54, false], # LATIN CAPITAL LETTER T
        0x55 => [0x55, false], # LATIN CAPITAL LETTER U
        0x56 => [0x56, false], # LATIN CAPITAL LETTER V
        0x57 => [0x57, false], # LATIN CAPITAL LETTER W
        0x58 => [0x58, false], # LATIN CAPITAL LETTER X
        0x59 => [0x59, false], # LATIN CAPITAL LETTER Y
        0x5a => [0x5a, false], # LATIN CAPITAL LETTER Z
        0x5b => [0x5b, false], # OPENING SQUARE BRACKET / LEFT SQUARE BRACKET
        0x5c => [0x5c, false], # REVERSE SLASH / REVERSE SOLIDUS
        0x5d => [0x5d, false], # CLOSING SQUARE BRACKET / RIGHT SQUARE BRACKET
        0x5e => [0x5e, false], # SPACING CIRCUMFLEX / CIRCUMFLEX ACCENT
        0x5f => [0x5f, false], # SPACING UNDERSCORE / LOW LINE
        0x60 => [0x60, false], # SPACING GRAVE / GRAVE ACCENT
        0x61 => [0x61, false], # LATIN SMALL LETTER A
        0x62 => [0x62, false], # LATIN SMALL LETTER B
        0x63 => [0x63, false], # LATIN SMALL LETTER C
        0x64 => [0x64, false], # LATIN SMALL LETTER D
        0x65 => [0x65, false], # LATIN SMALL LETTER E
        0x66 => [0x66, false], # LATIN SMALL LETTER F
        0x67 => [0x67, false], # LATIN SMALL LETTER G
        0x68 => [0x68, false], # LATIN SMALL LETTER H
        0x69 => [0x69, false], # LATIN SMALL LETTER I
        0x6a => [0x6a, false], # LATIN SMALL LETTER J
        0x6b => [0x6b, false], # LATIN SMALL LETTER K
        0x6c => [0x6c, false], # LATIN SMALL LETTER L
        0x6d => [0x6d, false], # LATIN SMALL LETTER M
        0x6e => [0x6e, false], # LATIN SMALL LETTER N
        0x6f => [0x6f, false], # LATIN SMALL LETTER O
        0x70 => [0x70, false], # LATIN SMALL LETTER P
        0x71 => [0x71, false], # LATIN SMALL LETTER Q
        0x72 => [0x72, false], # LATIN SMALL LETTER R
        0x73 => [0x73, false], # LATIN SMALL LETTER S
        0x74 => [0x74, false], # LATIN SMALL LETTER T
        0x75 => [0x75, false], # LATIN SMALL LETTER U
        0x76 => [0x76, false], # LATIN SMALL LETTER V
        0x77 => [0x77, false], # LATIN SMALL LETTER W
        0x78 => [0x78, false], # LATIN SMALL LETTER X
        0x79 => [0x79, false], # LATIN SMALL LETTER Y
        0x7a => [0x7a, false], # LATIN SMALL LETTER Z
        0x7b => [0x7b, false], # OPENING CURLY BRACKET / LEFT CURLY BRACKET
        0x7c => [0x7c, false], # VERTICAL BAR (FILL) / VERTICAL LINE
        0x7d => [0x7d, false], # CLOSING CURLY BRACKET / RIGHT CURLY BRACKET
        0x7e => [0x7e, false] # SPACING TILDE / TILDE
      }
      CHARSET_62 = { # Subscripts
        0x28 => [0x208d, false], # SUBSCRIPT OPENING PARENTHESIS / SUBSCRIPT LEFT PARENTHESIS
        0x29 => [0x208e, false], # SUBSCRIPT CLOSING PARENTHESIS / SUBSCRIPT RIGHT PARENTHESIS
        0x2b => [0x208a, false], # SUBSCRIPT PLUS SIGN
        0x2d => [0x208b, false], # SUBSCRIPT HYPHEN-MINUS / SUBSCRIPT MINUS
        0x30 => [0x2080, false], # SUBSCRIPT DIGIT ZERO
        0x31 => [0x2081, false], # SUBSCRIPT DIGIT ONE
        0x32 => [0x2082, false], # SUBSCRIPT DIGIT TWO
        0x33 => [0x2083, false], # SUBSCRIPT DIGIT THREE
        0x34 => [0x2084, false], # SUBSCRIPT DIGIT FOUR
        0x35 => [0x2085, false], # SUBSCRIPT DIGIT FIVE
        0x36 => [0x2086, false], # SUBSCRIPT DIGIT SIX
        0x37 => [0x2087, false], # SUBSCRIPT DIGIT SEVEN
        0x38 => [0x2088, false], # SUBSCRIPT DIGIT EIGHT
        0x39 => [0x2089, false] # SUBSCRIPT DIGIT NINE
      }
      CHARSET_67 = { # Greek Symbols
        0x61 => [0x3b1, false], # GREEK SMALL LETTER ALPHA
        0x62 => [0x3b2, false], # GREEK SMALL LETTER BETA
        0x63 => [0x3b3, false] # GREEK SMALL LETTER GAMMA
      }
      CHARSET_4E = { # Basic Cyrillic
        0x21 => [0x21, false], # EXCLAMATION MARK
        0x22 => [0x22, false], # QUOTATION MARK
        0x23 => [0x23, false], # NUMBER SIGN
        0x24 => [0x24, false], # DOLLAR SIGN
        0x25 => [0x25, false], # PERCENT SIGN
        0x26 => [0x26, false], # AMPERSAND
        0x27 => [0x27, false], # APOSTROPHE
        0x28 => [0x28, false], # OPENING PARENTHESIS / LEFT PARENTHESIS
        0x29 => [0x29, false], # CLOSING PARENTHESIS / RIGHT PARENTHESIS
        0x2a => [0x2a, false], # ASTERISK
        0x2b => [0x2b, false], # PLUS SIGN
        0x2c => [0x2c, false], # COMMA
        0x2d => [0x2d, false], # HYPHEN-MINUS
        0x2e => [0x2e, false], # PERIOD, DECIMAL POINT / FULL STOP
        0x2f => [0x2f, false], # SLASH / SOLIDUS
        0x30 => [0x30, false], # DIGIT ZERO
        0x31 => [0x31, false], # DIGIT ONE
        0x32 => [0x32, false], # DIGIT TWO
        0x33 => [0x33, false], # DIGIT THREE
        0x34 => [0x34, false], # DIGIT FOUR
        0x35 => [0x35, false], # DIGIT FIVE
        0x36 => [0x36, false], # DIGIT SIX
        0x37 => [0x37, false], # DIGIT SEVEN
        0x38 => [0x38, false], # DIGIT EIGHT
        0x39 => [0x39, false], # DIGIT NINE
        0x3a => [0x3a, false], # COLON
        0x3b => [0x3b, false], # SEMICOLON
        0x3c => [0x3c, false], # LESS-THAN SIGN
        0x3d => [0x3d, false], # EQUALS SIGN
        0x3e => [0x3e, false], # GREATER-THAN SIGN
        0x3f => [0x3f, false], # QUESTION MARK
        0x40 => [0x44e, false], # LOWERCASE IU / CYRILLIC SMALL LETTER YU
        0x41 => [0x430, false], # CYRILLIC SMALL LETTER A
        0x42 => [0x431, false], # CYRILLIC SMALL LETTER BE
        0x43 => [0x446, false], # CYRILLIC SMALL LETTER TSE
        0x44 => [0x434, false], # CYRILLIC SMALL LETTER DE
        0x45 => [0x435, false], # CYRILLIC SMALL LETTER IE
        0x46 => [0x444, false], # CYRILLIC SMALL LETTER EF
        0x47 => [0x433, false], # LOWERCASE GE / CYRILLIC SMALL LETTER GHE
        0x48 => [0x445, false], # LOWERCASE KHA / CYRILLIC SMALL LETTER HA
        0x49 => [0x438, false], # LOWERCASE II / CYRILLIC SMALL LETTER I
        0x4a => [0x439, false], # LOWERCASE SHORT II / CYRILLIC SMALL LETTER SHORT I
        0x4b => [0x43a, false], # CYRILLIC SMALL LETTER KA
        0x4c => [0x43b, false], # CYRILLIC SMALL LETTER EL
        0x4d => [0x43c, false], # CYRILLIC SMALL LETTER EM
        0x4e => [0x43d, false], # CYRILLIC SMALL LETTER EN
        0x4f => [0x43e, false], # CYRILLIC SMALL LETTER O
        0x50 => [0x43f, false], # CYRILLIC SMALL LETTER PE
        0x51 => [0x44f, false], # LOWERCASE IA / CYRILLIC SMALL LETTER YA
        0x52 => [0x440, false], # CYRILLIC SMALL LETTER ER
        0x53 => [0x441, false], # CYRILLIC SMALL LETTER ES
        0x54 => [0x442, false], # CYRILLIC SMALL LETTER TE
        0x55 => [0x443, false], # CYRILLIC SMALL LETTER U
        0x56 => [0x436, false], # CYRILLIC SMALL LETTER ZHE
        0x57 => [0x432, false], # CYRILLIC SMALL LETTER VE
        0x58 => [0x44c, false], # CYRILLIC SMALL LETTER SOFT SIGN
        0x59 => [0x44b, false], # LOWERCASE YERI / CYRILLIC SMALL LETTER YERI
        0x5a => [0x437, false], # CYRILLIC SMALL LETTER ZE
        0x5b => [0x448, false], # CYRILLIC SMALL LETTER SHA
        0x5c => [0x44d, false], # LOWERCASE REVERSED E / CYRILLIC SMALL LETTER E
        0x5d => [0x449, false], # CYRILLIC SMALL LETTER SHCHA
        0x5e => [0x447, false], # CYRILLIC SMALL LETTER CHE
        0x5f => [0x44a, false], # CYRILLIC SMALL LETTER HARD SIGN
        0x60 => [0x42e, false], # UPPERCASE IU / CYRILLIC CAPITAL LETTER YU
        0x61 => [0x410, false], # CYRILLIC CAPITAL LETTER A
        0x62 => [0x411, false], # CYRILLIC CAPITAL LETTER BE
        0x63 => [0x426, false], # CYRILLIC CAPITAL LETTER TSE
        0x64 => [0x414, false], # CYRILLIC CAPITAL LETTER DE
        0x65 => [0x415, false], # CYRILLIC CAPITAL LETTER IE
        0x66 => [0x424, false], # CYRILLIC CAPITAL LETTER EF
        0x67 => [0x413, false], # UPPERCASE GE / CYRILLIC CAPITAL LETTER GHE
        0x68 => [0x425, false], # UPPERCASE KHA / CYRILLIC CAPITAL LETTER HA
        0x69 => [0x418, false], # UPPERCASE II / CYRILLIC CAPITAL LETTER I
        0x6a => [0x419, false], # UPPERCASE SHORT II / CYRILLIC CAPITAL LETTER SHORT I
        0x6b => [0x41a, false], # CYRILLIC CAPITAL LETTER KA
        0x6c => [0x41b, false], # CYRILLIC CAPITAL LETTER EL
        0x6d => [0x41c, false], # CYRILLIC CAPITAL LETTER EM
        0x6e => [0x41d, false], # CYRILLIC CAPITAL LETTER EN
        0x6f => [0x41e, false], # CYRILLIC CAPITAL LETTER O
        0x70 => [0x41f, false], # CYRILLIC CAPITAL LETTER PE
        0x71 => [0x42f, false], # UPPERCASE IA / CYRILLIC CAPITAL LETTER YA
        0x72 => [0x420, false], # CYRILLIC CAPITAL LETTER ER
        0x73 => [0x421, false], # CYRILLIC CAPITAL LETTER ES
        0x74 => [0x422, false], # CYRILLIC CAPITAL LETTER TE
        0x75 => [0x423, false], # CYRILLIC CAPITAL LETTER U
        0x76 => [0x416, false], # CYRILLIC CAPITAL LETTER ZHE
        0x77 => [0x412, false], # CYRILLIC CAPITAL LETTER VE
        0x78 => [0x42c, false], # CYRILLIC CAPITAL LETTER SOFT SIGN
        0x79 => [0x42b, false], # UPPERCASE YERI / CYRILLIC CAPITAL LETTER YERI
        0x7a => [0x417, false], # CYRILLIC CAPITAL LETTER ZE
        0x7b => [0x428, false], # CYRILLIC CAPITAL LETTER SHA
        0x7c => [0x42d, false], # CYRILLIC CAPITAL LETTER E
        0x7d => [0x429, false], # CYRILLIC CAPITAL LETTER SHCHA
        0x7e => [0x427, false] # CYRILLIC CAPITAL LETTER CHE
      }
      CODESETS = {
        0x34 => CHARSET_34,
        0x45 => CHARSET_45,
        0x33 => CHARSET_33,
        0x32 => CHARSET_32,
        0x31 => CHARSET_31,
        0x70 => CHARSET_70,
        0x51 => CHARSET_51,
        0x53 => CHARSET_53,
        0x42 => CHARSET_42,
        0x62 => CHARSET_62,
        0x67 => CHARSET_67,
        0x4E => CHARSET_4E
      }

      # ODD_MAP for odd characters (all from III for now)
      ODD_MAP = {
        0x21203d => 0x2026, # HORIZONTAL ELLIPSIS
        0x212040 => 0x201c, # LEFT DOUBLE QUOTATION MARK
        0x7f2014 => 0x2014, # EM DASH
        0x7f2019 => 0x2019, # RIGHT SINGLE QUOTATION MARK
        0x7f2020 => 0x201d, # RIGHT DOUBLE QUOTATION MARK
        0x7f2122 => 0x2122 # TRADE MARK SIGN
      }
    end
  end
end
